//=============================================================================
// Frag.Ops  2004 Matt 'SquirrelZero' Farber
//=============================================================================
class Armor_Helmet extends FragOps_ArmorPiece;

var Sound ImpactSound;
var float LastImpactTime;

function BreakOff(rotator BreakDir)
{
	local vector RandomSpin;
	local PlayerController PC;
	local name MyBone;

	// get PC
	PC = Level.GetLocalPlayerController();
	if (PC == None || PC.BeyondViewDistance(Location, 6000))
	{
		Destroy();
		return;
	}


	// owner was destroyed
	if (Owner == None)
	{
		SetOwner(Instigator);
		if (Owner == None)
		{
			Destroy();
			return;
		}
	}

	// detail adjusetment
	if (Level.DetailMode < DM_High || Level.bDropDetail)
		LifeSpan = 10.0;
	else
		LifeSpan = 18.0;

	// cut in half for TDM games
	if (FOPawn(Owner).bTeamDeathMatch)
		LifeSpan *= 0.5;

	// initialize
	bActorShadows = false;
	bHardAttach = false;
	bAcceptsProjectors = false;
	MyBone = AttachmentBone;
	Owner.DetachFromBone(self);
	SetLocation(Owner.GetBoneCoords(MyBone).Origin+vect(0,0,4));

	// collide with world
	bCollideWorld = true;

	// karma at high detail
	if (Level.PhysicsDetailLevel == PDL_High && !PC.BeyondViewDistance(Location, 3000))
	{
		// give it a heavy karma parameters set
		KParams = new(none) class'Karma_Steel';

		// create a random spin
		RandomSpin.X = (FRand()*60) - 30;
		RandomSpin.Y = (FRand()*60) - 30;
		RandomSpin.Z = (FRand()*60) - 30;

		// set up linear and angular velocity
		KarmaParams(KParams).KVelDropBelowThreshold = 10;
		KarmaParams(KParams).KStartAngVel = (vector(BreakDir)+RandomSpin)*(6000+Rand(2001));
		KarmaParams(KParams).KStartLinVel = vector(BreakDir) * (300+Rand(61));
		KarmaParams(KParams).KStartLinVel.Z += 60+Rand(31);

		// turn karma on
		bUseCylinderCollision = false;
		KSetBlockKarma(true);
		SetPhysics(PHYS_Karma);
		KWake();
		GotoState('InitCollision','Begin');
		return;
	}

	// standard physics at low detail
	SetPhysics(PHYS_Falling);

	// give it some velocity
	Velocity = vector(BreakDir) * (180+Rand(61));
	Velocity.Z += 40 + Rand(21);

	// slow in water
	if (PhysicsVolume.bWaterVolume)
		Velocity *= 0.35;

	// setup rotation
	bFixedRotationDir = true;
	DesiredRotation = BreakDir;
	RotationRate.Yaw = 200000 * 2 *FRand() - 200000;
	RotationRate.Pitch = 200000 * 2 *FRand() - 200000;
	RotationRate.Roll = 200000 * 2 *FRand() - 200000;

	// setup collision
	bBounce = true;
	GotoState('InitCollision','Begin');
}

state InitCollision
{
	Begin:
		Sleep(0.25);
		SetCollision(true,true,false);
}

event KImpact(Actor other, Vector pos, vector ImpactVel, vector impactNorm)
{
	local float ImpactVolume;

	if (Level.TimeSeconds - LastImpactTime > 0.15 && (Other == None || Other.Physics != PHYS_Karma))
	{
		ImpactVolume = FClamp(VSize(ImpactVel)/(KarmaParams(KParams).KImpactThreshold*5.1), 0.1, 0.68);
		LastImpactTime = Level.TimeSeconds;
		PlaySound(ImpactSound,, ImpactVolume,,40,1.0);
	}
}

function HitWall( vector HitNormal, actor Wall )
{
	SetPhysics(PHYS_Falling);
	RandSpin(100000);
	Velocity = 0.35 * (Velocity - 2.0*HitNormal*(Velocity dot HitNormal));
	PlaySound(ImpactSound, SLOT_Misc, 0.68,,30,1.0);

	if (VSize(Velocity) < 40)
		bBounce = false;
}

function Landed(Vector HitNormal)
{
	DesiredRotation = rotator(-HitNormal);
	RotationRate.Yaw = 200000;
	RotationRate.Pitch = 200000;
	RotationRate.Roll = 200000;
	bRotateToDesired = true;
	SetPhysics(PHYS_None);
}

function RandSpin(float spinRate)
{
	DesiredRotation = RotRand();
	RotationRate.Yaw = spinRate * 2 *FRand() - spinRate;
	RotationRate.Pitch = spinRate * 2 *FRand() - spinRate;
	RotationRate.Roll = spinRate * 2 *FRand() - spinRate;	
}

event KVelDropBelow()
{
	SetPhysics(PHYS_None);
}

defaultproperties
{
	bHelmet=true
	bOnlyDrawIfAttached=false
	DrawScale=0.43
	bUseCylinderCollision=true
	bCanBeDamaged=true
	CollisionRadius=8.0
	CollisionHeight=6.0
	ImpactSound=sound'FOGameSounds.AmbientHit.HelmetImpact'
	AttachmentBone=Dummy13
}