//=============================================================================
// Frag.Ops  2004 Matt 'SquirrelZero' Farber
//=============================================================================
class Attachment_Base extends xWeaponAttachment;

// weapon class for 1st person spectating
var class<Weapon_Base> WeaponClass;

// again a nice emitter-based muzzle flash
var class<Emitter>     NewMuzFlash3rdClass;
var Emitter             NewMuzFlash3rd;
var name FlashBone;

// 3rd person shell ejection meshemitter
var class<Effect_ShellEjectorFirst> ShellEjector3rdClass;
var Effect_ShellEjectorFirst ShellEjector3rd;
var name ShellBone;

// xemitter mesh flashes
var class<xEmitter>     MeshFlash3rdClass;
var xEmitter            MeshFlash3rd;

// xemitter muzzle smoke
var class<xEmitter>     MuzSmoke3rdClass;
var xEmitter            MuzSmoke3rd;

// taclight
var name LightBone;
var vector LightOffset;

// for pawn anims
var bool bSidearm, bGrenade, bSMG, bMelee, bAkimbo, bFists, bIsAkimbo;

// for fire sound, weapon properties
var class<FireMode_Base> FireClass[2];
var float FireVolume[2];
var float FireRadius[2];

// arm rotation offsets for tighter weapon grip
var rotator ArmRotation;
var int LastFlashCount;

// master attachment
var Attachment_Base MasterAttachment;

// animations for special cases
var name SpecialIdle, SpecialFire;

replication
{
	reliable if ((bNetDirty) && (Role == ROLE_Authority))
		 MasterAttachment;

	reliable if ((bNetDirty) && (Role == ROLE_Authority))
		bIsAkimbo;
}

simulated function PostNetBeginPlay() 
{
	if (Instigator != None)
		Instigator.AttachToBone(self,Default.AttachmentBone);
	
	Super.PostNetBeginPlay();

	if (Level.NetMode == NM_DedicatedServer)
		return;

	if (ShellEjector3rd == None && ShellEjector3rdClass != None)
	{
		ShellEjector3rd = Spawn(ShellEjector3rdClass,self);
		ShellEjector3rd.bThird = true;
		ShellEjector3rd.InitEjector();
		AttachToBone(ShellEjector3rd, ShellBone);
		SetShellBone();
	}
}

simulated function Destroyed() 
{
	Super.Destroyed(); 
	if (NewMuzFlash3rd != None) 
		NewMuzFlash3rd.Destroy();
	if (MeshFlash3rd != None)
		MeshFlash3rd.Destroy();
	if (MuzSmoke3rd != None)
		MuzSmoke3rd.Destroy();
	if (ShellEjector3rd != None)
		ShellEjector3rd.Destroy();
}

simulated event ThirdPersonEffects() 
{ 
	local rotator r;
	
	if (FlashCount == LastFlashCount)
		return;

	if (Level.NetMode != NM_DedicatedServer)
	{
		if (FiringMode == 0 || FiringMode == 1)
		{
			if (FOPawn(Instigator) == None || FOPawn(Instigator).SpectatorWeapon != None)
				return;
			
			if (FlashCount > 0)
				FOPawn(Instigator).StartFiring(bHeavy, bRapidFire);

			if (FlashCount <= 0 || MeshFlash3rdClass == None || bGrenade)
				return;

			PlayFireSound(FiringMode);

			WeaponLight();
			
			if (MeshFlash3rd == None)
			{
				MeshFlash3rd = Spawn(MeshFlash3rdClass,self);
				AttachToBone(MeshFlash3rd, FlashBone);
			}

			if (MuzSmoke3rd == None)
			{
				MuzSmoke3rd = Spawn(MuzSmoke3rdClass,self);
				AttachToBone(MuzSmoke3rd, FlashBone);
			}

			if (ShellEjector3rd == None)
			{
				ShellEjector3rd = Spawn(ShellEjector3rdClass,self);
				ShellEjector3rd.bThird = true;
				ShellEjector3rd.InitEjector();
				AttachToBone(ShellEjector3rd, ShellBone);
			}

			r.Roll = 16384*Rand(5);
			SetBoneRotation(FlashBone, r, 0, 1.f);
			MeshFlash3rd.Trigger(self, Instigator);
			MuzSmoke3rd.Trigger(self, Instigator);
			SetShellBone();
			ShellEjector3rd.Trigger(self, Instigator);
		}
	}
	LastFlashCount = FlashCount;
}

simulated function SetShellBone();
simulated function SetAttachProps(FOPawn AttachPawn);

// weapon flashes go through walls, this should help stop it
simulated function WeaponLight()
{
	local vector EyeLoc;
	local PlayerController PC;

	PC = Level.GetLocalPlayerController();
	if (PC == None)
		return;

	EyeLoc = PC.ViewTarget.Location;
	if (Pawn(PC.ViewTarget) != None)
		EyeLoc.Z += Pawn(PC.ViewTarget).EyeHeight;
	
	if (FlashCount > 0 && !Level.bDropDetail && Instigator != None
	&& (PlayerController(Instigator.Controller) != None || (Level.TimeSeconds - LastRenderTime < 0.2 && FlashBone != '' && FastTrace(GetBoneCoords(FlashBone).Origin, EyeLoc) && !PC.BeyondViewDistance(GetBoneCoords(FlashBone).Origin, 3500))))
	{
		if ((Instigator != None) && (Instigator.Weapon != None))
		{
			LitWeapon = Instigator.Weapon;
			LitWeapon.bDynamicLight = true;
		}
		else
			bDynamicLight = true;
		SetTimer(0.075, false);
    }
    else 
		Timer();
}

simulated function PlayFireSound(int ModeNum)
{
	if (FireClass[ModeNum] != None && Instigator.Weapon == None)
		PlaySound(FireClass[ModeNum].Default.FireSound, SLOT_Misc,TransientSoundVolume*FireVolume[ModeNum],,FireRadius[ModeNum],(0.97)+(FRand()*0.06),false);
}

simulated function Timer()
{
	if (LitWeapon != None)
	{
		LitWeapon.bDynamicLight = false;
		LitWeapon = None;
	}
	bDynamicLight = false;
}

simulated function Hide(bool NewbHidden)
{
	Super.Hide(NewbHidden);
	
	if (MasterAttachment != None && bAkimbo)
		MasterAttachment.Hide(NewbHidden);
}

defaultproperties
{
	DrawScale=1.5
	MeshFlash3rdClass=class'Effect_MeshFlash3rd'
	MuzSmoke3rdClass=class'Effect_MuzSmoke3rd'
	ShellEjector3rdClass=class'Effect_ShellEjectorFirst'
	NewMuzFlash3rdClass=None
	bHeavy=false
	bRapidFire=false
	bAltRapidFire=false
	bReplicateMovement=true
	ArmRotation=(Yaw=-2000)
	NetUpdateFrequency=40
	AttachmentBone=righthand

	FlashBone=Bone_Flash
	ShellBone=Bone_Shell

	FireRadius[0]=550.0
	FireRadius[1]=550.0

	bDynamicLight=false
	LightType=LT_Steady
	LightEffect=LE_NonIncidence
	LightPeriod=2
	LightBrightness=170
	LightHue=30
	LightSaturation=140
	LightRadius=4.3
}