class DynObj_TriggeredDestroyable extends FragOps_DynamicObjects
	placeable;

// Karma Object type
enum EKarmaType
{
	KLead,
	KSteel,
	KRubber,
	KWood,
	KFeather
};

// ChunkKarma, can use karma for chunks instead
struct ChunkKarmaProps
{
	var() bool bUseKarmaInstead;
	var() EKarmaType KarmaType;
};

// Chunkrecord
struct ChunkRecord
{
	var() StaticMesh ChunkMesh;
	var() editinline float ChunkScale;
	var() Sound ChunkBounceSound;
	var() float ChunkVelocity;
	var() float ChunkLifeSpan;
	var() editinline ChunkKarmaProps ChunkKarma;
};

// these chunks will fly out everywhere when object is triggered if bBlowToPieces
var() editinline array<ChunkRecord> ExplodedChunks;

// instead of just flying in all directions, blow to pieces instead!
var() bool bBlowToPieces;

// trigger when damaged
var() bool bTriggerWhenDamaged;

// if bTriggerWhenDamaged, maximum damage this object can withstand before destruction
var() int MaxDamage;

// if not blown to pieces, sound this object makes when bouncing
var() Sound BounceSound;

// speed this object will fly off at
var() float ObjectSpeed;

// how long to make it live, 0.0 is default of 6 seconds
var() float ObjectLifeSpan;

// sound to play when object breaks
var() sound DestroyedSound;

// event called when object is destroyed
var() name DestroyedEvent;

// use karma or not
var() ChunkKarmaProps ObjectKarma;

// spawn offset
var() vector SpawnOffset;

var StaticMesh OrigMesh;
var int DamageTaken;
var bool bDestroyed;

function PostBeginPlay()
{
	Super.PostBeginPlay();
	OrigMesh = StaticMesh;
}

function TakeDamage(int Damage, Pawn instigatedBy, Vector HitLocation, Vector Momentum, class<DamageType> damageType)
{
	if (!bTriggerWhenDamaged)
		return;
	
	if (DamageType.Name == 'DamType_Machete' || DamageType.Name == 'DamType_AKBayonet' || DamageType.Name == 'DamType_MachetePoison')
		DamageTaken += Damage/3;
	else
		DamageTaken += Damage;

	if (DamageTaken >= MaxDamage)
		Trigger(None, InstigatedBy);
}

event Trigger(actor Other, pawn EventInstigator)
{
	if (bDestroyed)
		return;
	
	SetStaticMesh(None);
	SetCollision(false,false,false);

	if (bBlowToPieces)
		SpawnChunks();
	else
		GoFlying();

	TriggerEvent(DestroyedEvent, self, EventInstigator);

	PlaySound(DestroyedSound,,TransientSoundVolume*1.3);

	bDestroyed = true;
}

function GoFlying()
{
	local Effect_LAWChunk MeshCopy;
	
	// spawn a copy
	MeshCopy = Spawn(class'Effect_LAWChunk',,, Location + SpawnOffset, Rotation);
	if (MeshCopy != None)
	{
		MeshCopy.SetStaticMesh(OrigMesh);
		MeshCopy.SetDrawScale(DrawScale);
		MeshCopy.ImpactSound = BounceSound;
		MeshCopy.NewLifeSpan = ObjectLifeSpan;
		if (ObjectSpeed > 0)
			MeshCopy.Speed = ObjectSpeed;
		MeshCopy.SetKarma(ObjectKarma.bUseKarmaInstead,int(ObjectKarma.KarmaType));
	}
}

function SpawnChunks()
{
	local Effect_LAWChunk NewChunk;
	local vector start, newstart;
	local rotator rot;
	local int i;

	// spawn some chunks
	start = Location;
	for (i=0;i<ExplodedChunks.Length;i++)
	{
		rot = rotator(vect(0,0,1));
		rot.yaw += FRand()*32000-16000;
		rot.pitch += FRand()*32000-16000;
		rot.roll += FRand()*32000-16000;
		NewStart.X = (Start.X - 2) + (FRand()*4);
		NewStart.Y = (Start.Y - 2) + (FRand()*4);
		NewChunk = Spawn(class'Effect_LAWChunk',,, Start + SpawnOffset, rot);
		if (ExplodedChunks[i].ChunkMesh != None && NewChunk != None)
		{
			NewChunk.SetStaticMesh(ExplodedChunks[i].ChunkMesh);
			NewChunk.SetDrawScale(ExplodedChunks[i].ChunkScale);
			NewChunk.ImpactSound = ExplodedChunks[i].ChunkBounceSound;
			NewChunk.NewLifeSpan = ExplodedChunks[i].ChunkLifeSpan;
			if (ExplodedChunks[i].ChunkVelocity >= 1.0)
				NewChunk.Speed = ExplodedChunks[i].ChunkVelocity;
			NewChunk.SetKarma(ExplodedChunks[i].ChunkKarma.bUseKarmaInstead,int(ExplodedChunks[i].ChunkKarma.KarmaType));
		}
	}
}

function Reset()
{
	Super.Reset();
	SetStaticMesh(OrigMesh);
	SetCollision(true,true,true);
	bDestroyed = false;
	DamageTaken = 0;
}

defaultproperties
{
	Physics=PHYS_None
	bCanBeDamaged=true
	bProjTarget=true
	bHidden=false
	bWorldGeometry=true
	bCollideWorld=true
	bCollideActors=true
	bBlockPlayers=true
	bBlockActors=true
	bCollideWhenPlacing=false
	bAcceptsProjectors=true
	RemoteRole=ROLE_DumbProxy
	DrawType=DT_StaticMesh
	StaticMesh=StaticMesh'OpCanNiSM.decorative.weaponcrate'
	SpawnOffset=(Z=50)
}