class DynObj_TriggeredExplosion extends FragOps_DynamicObjects
	placeable;

// optional effects to spawn when objective is destroyed, keep these
// clean and fast, since we are already spawning a LAW explosion
// from the missile.
var() editinline class<Emitter> ExplodedEmitter;
var() editinline class<xEmitter> ExplodedXEmitter;

// only damage visible players
var() bool bOnlyDmgVisiblePlayers;

// shakes the screen if true 
var() bool bShakeScreen;

// explosion radius, how large the explosion area is
var() float ExplosionRadius;

// how much momentum (blowback) this explosion applies
var() float ExplosionMomentum;

// how much damage this explosion will deal, depends on how far from center player is
var() int ExplosionDamage;

// delay before triggering explosion
var() float ExplosionDelay;

// sound to play when object breaks
var() sound ExplosionSound;

// event to trigger when this one is triggered
var() name ExplosionEvent;

// kill all players within radius regardless of damage or team
var() bool bKillAllWithinRadius;

var bool bDestroyed;

function Trigger(actor Other, pawn EventInstigator)
{
	if (bDestroyed)
		return;

	Instigator = EventInstigator;
	SetTimer(ExplosionDelay, false);
	bDestroyed = true;
}

function Timer()
{
	local Effect_LAWEmitterSpawner LAWEmitterSpawner;

	TriggerEvent(ExplosionEvent, self, Instigator);
	
	// spawn the explosion emitter
	if (ExplodedEmitter != None || ExplodedXEmitter != None)
	{
		LAWEmitterSpawner = Spawn(class'Effect_LAWEmitterSpawner',,, Location, rotator(vect(0,0,1)));
		LAWEmitterSpawner.EmitterClass = ExplodedEmitter;
		LAWEmitterSpawner.XEmitterClass = ExplodedXEmitter;
		LAWEmitterSpawner.DoSpawn();
	}
	PlaySound(ExplosionSound,,TransientSoundVolume*1.3);
	DelayedHurtRadius(float(ExplosionDamage), ExplosionRadius, ExplosionMomentum, Instigator);
	if (bShakeScreen)
		ShakeView();
}

function DelayedHurtRadius(float DamageAmount, float DamageRadius, float Momentum, Pawn ExplosionInstigator)
{
	local actor Victims;
	local float damageScale, dist;
	local vector dir;

	SetCollision(true,false,false);
	foreach CollidingActors(class'Actor', Victims, DamageRadius, Location)
	{
		if ((Victims != self) && (Victims.Role == ROLE_Authority) && (!Victims.IsA('FluidSurfaceInfo')) && ((FastTrace(Victims.Location, Location) && bOnlyDmgVisiblePlayers) || !bOnlyDmgVisiblePlayers))
		{

			dir = Victims.Location - Location;
			if (bKillAllWithinRadius && FOPawn(Victims) != None)
			{
				FOPawn(Victims).LastBone = 'spine';
				if ((ExplosionInstigator.PlayerReplicationInfo != None && FOPawn(Victims).PlayerReplicationInfo != None && ExplosionInstigator.PlayerReplicationInfo.Team.TeamIndex == FOPawn(Victims).PlayerReplicationInfo.Team.TeamIndex) || ExplosionInstigator.Controller == None)
					FOPawn(Victims).Died(None, class'DamType_TriggeredExplosion', Victims.Location - 0.5 * (Victims.CollisionHeight + Victims.CollisionRadius) * dir);
				else
					FOPawn(Victims).Died(ExplosionInstigator.Controller, class'DamType_TriggeredExplosion', Victims.Location - 0.5 * (Victims.CollisionHeight + Victims.CollisionRadius) * dir);
				continue;
			}
			dist = FMax(1,VSize(dir));
			dir = dir/dist; 
			damageScale = 1 - FMax(0,(dist - Victims.CollisionRadius)/DamageRadius);
			if (FOPawn(Victims) != None)
				FOPawn(Victims).LastBone = 'spine';
			Victims.TakeDamage
			(
				damageScale * DamageAmount,
				ExplosionInstigator, 
				Victims.Location - 0.5 * (Victims.CollisionHeight + Victims.CollisionRadius) * dir,
				(damageScale * Momentum * dir),
				class'DamType_TriggeredExplosion'
			);
		} 
	}
	SetCollision(false,false,false);
	Instigator = None;
}

function ShakeView()
{
	local Controller C;
	local PlayerController PC;
	local float Dist, Scale;

	for (C=Level.ControllerList;C!=None;C=C.NextController)
	{
		PC = PlayerController(C);
		if (PC != None && PC.ViewTarget != None)
		{
			Dist = VSize(Location - PC.ViewTarget.Location);
			if (Dist < ExplosionRadius * 2.0)
			{
				if (Dist < ExplosionRadius)
					Scale = 1.0;
				else
					Scale = (ExplosionRadius*2.0 - Dist) / (ExplosionRadius);
				FOPlayer(PC).ClientShake(vect(350,350,350)*Scale,vect(9000,9000,9000),3.5, vect(10,10,10)*Scale,vect(2000,2000,2000),3.5);
			}
		}
	}
}

function Reset()
{
	// stop timer
	bDestroyed = false;
	SetTimer(0.0,false);
	Instigator = None;
}

defaultproperties
{
	ExplosionDelay=0.05
	DrawType=DT_Sprite
	bHidden=true
	Physics=PHYS_None
	bCollideWorld=false
	bCollideActors=false
	bBlockActors=false
	bBlockPlayers=false
	bWorldGeometry=false
	RemoteRole=ROLE_DumbProxy
}