//=============================================================================
// Frag.Ops  2004 Matt 'SquirrelZero' Farber
//=============================================================================
class Effect_BloodPool extends Projector;

var bool bGrow;
var() bool bRandomOrient;
var Texture AltProjTexture;
var float MasterDrawScale;
var vector SpawnLocation;

event PreBeginPlay()
{
	if ((Level.NetMode == NM_DedicatedServer) || (Level.DecalStayScale == 0.f))
	{
		Destroy();
		return;
	}
	Super.PreBeginPlay();
}

function PostBeginPlay()
{
	local Vector RX, RY, RZ;
	local Rotator R;

	if (Instigator == None)
		Instigator = Pawn(Owner);

	if (PhysicsVolume.bNoDecals || Instigator == None)
	{
		Destroy();
		return;
	}

	if (FRand() < 0.5)
		ProjTexture = Default.ProjTexture;
	else
		ProjTexture = AltProjTexture;

	if (bRandomOrient)
	{
		R.Yaw = 0;
		R.Pitch = 0;
		R.Roll = Rand(65535);
		GetAxes(R,RX,RY,RZ);
		RX = RX >> Rotation;
		RY = RY >> Rotation;
		RZ = RZ >> Rotation;
		R = OrthoRotation(RX,RY,RZ);
		SetRotation(R);
	}

	SetLocation(Location - Vector(Rotation)*18);
	Super.PostBeginPlay();
	MasterDrawScale = Default.DrawScale*(0.7+(FRand()*0.4));
	SetDrawScale(Default.DrawScale*0.05);
	SpawnLocation = Instigator.Location;
	bGrow = true;
}

function Tick(float dt)
{
	local float OldScale;

	Super.Tick(dt);
	
	if (Instigator == None)
		return;
	
	if (bGrow)
	{
		OldScale = DrawScale;
		DetachProjector();
		SetDrawScale(fClamp(OldScale+(dt*0.03), Default.DrawScale*0.05, MasterDrawScale));
		AttachProjector();
	}

	if (bGrow && (DrawScale == Default.DrawScale || VSize(SpawnLocation-Instigator.Location) > 20))
		bGrow = false;
}

defaultproperties
{
	ProjTexture=Texture'FOMisc.Projected.BloodPool'
	AltProjTexture=Texture'FOMisc.Projected.BloodPool'
	FadeInTime=0.125
	MaxTraceDistance=45
	DrawScale=0.65
	FOV=4

	MaterialBlendingOp=PB_None
	FrameBufferBlendingOp=PB_Modulate
	bGameRelevant=true
	bGradient=false
	bProjectBSP=true
	bProjectTerrain=true
	bProjectStaticMesh=true
	bClipStaticMesh=True
	bProjectActor=false
	bClipBSP=true
	bRandomOrient=true
	bNoDelete=false
	bStatic=false
	GradientTexture=GRADIENT_Clip
	RemoteRole=ROLE_None
	CullDistance=+5000.0
	LifeSpan=400
}