//=============================================================================
// Frag.Ops  2004 Matt 'SquirrelZero' Farber
//=============================================================================
class Effect_BulletHit extends Emitter
	config;

var Vector HitNormal, SpawnLoc;
var Material HitMaterial;
var class<actor> SplashEffect;
var class<DamageType> MyDamageType;

var config bool bShowTracers;
var bool bSpawned;
var bool bHitBody;

replication
{
	reliable if (Role == ROLE_Authority)
		SpawnLoc, HitNormal, HitMaterial, MyDamageType;
}

function ImpactAt(Vector hl, Vector hn, material hm, Pawn OwnerPawn, class<DamageType> NewDamageType)
{
	HitNormal = hn;
	HitMaterial = hm;
	SpawnLoc = hl;
	Instigator = OwnerPawn;
	MyDamageType = NewDamageType;
	if (Level.NetMode != NM_DedicatedServer)
		SpawnEffects();
}

simulated function PostNetBeginPlay()
{
	if (Level.NetMode == NM_Client && !bSpawned)
		SpawnEffects();
}

simulated function SpawnImpactEffects(rotator HitRot, vector EffectLoc)
{
	if (!bHitBody)
	{
		// spawn hit decal
		if (HitMaterial != None && HitMaterial.SurfaceType == EST_Metal)
			Spawn(class'Effect_BulletPock',,,EffectLoc, rotator(-HitNormal));
		else if (HitMaterial != None && HitMaterial.SurfaceType == EST_Wood)
			Spawn(class'Effect_BulletHoleWood',,,EffectLoc, rotator(-HitNormal));
		else if (HitMaterial != None && HitMaterial.SurfaceType == EST_Glass)
			Spawn(class'Effect_BulletHoleGlass',,,EffectLoc, rotator(-HitNormal));
		else
			Spawn(class'Effect_BulletHole',,,EffectLoc, rotator(-HitNormal));

		// spawn hit effect
		if (PhysicsVolume.bWaterVolume)
		{
			Spawn(class'pclImpactSmoke',,, EffectLoc, HitRot);
			Destroy();
			return;
		}
		Spawn(class'Effect_GetHitEffect'.static.GetHitEffect(HitMaterial, EffectLoc, vector(HitRot)),,, EffectLoc, HitRot);
	}
	Destroy();
}

simulated function SpawnEffects()
{
	bSpawned = true;
	// tracers
	SpawnTracer();
	SpawnImpactEffects(Rotator(HitNormal),SpawnLoc + HitNormal*2);
}

simulated function SpawnTracer()
{
	local Effect_BulletTracer BulletTracer;
	local class<Effect_BulletTracer> BulletTracerClass;
	local int TrailOffset;
	local rotator TrailRotation;

	if (Level.NetMode == NM_DedicatedServer || FOPawn(Instigator) == None)
		return;

	// get a random offset
	TrailOffset = Rand(50);

	if (Instigator.Controller != None && Level.GetLocalPlayerController() == Instigator.Controller && Instigator.Weapon != None) // if locally controlled
	{
		// for zoomed weapons, dont do tracers at all.. cant get it looking right with the FOV amplification
		if (Weapon_Base(Instigator.Weapon).bScopeZoomed || Weapon_Base(Instigator.Weapon).Hand >= 2 || Weapon_Base(Instigator.Weapon).Hand <= -2)
			bShowTracers = false;

		BulletTracerClass = class'Effect_BulletTracer';

		// place it at the weapon's tracer bone location
		if (Weapon_Base(Instigator.Weapon).bIsAkimbo)
			SetLocation(Weapon_Base(Instigator.Weapon).SlaveWeapon.GetBoneCoords(Weapon_Base(Instigator.Weapon).TracerBone).Origin);
		else
			SetLocation(Instigator.Weapon.GetBoneCoords(Weapon_Base(Instigator.Weapon).TracerBone).Origin);
	}
	else if (FOPawn(Instigator).SpectatorWeapon != None && !FOPawn(Instigator).SpectatorWeapon.bNoRender)
	{
		// first person spectator
		BulletTracerClass = class'Effect_BulletTracer';
		SetLocation(FOPawn(Instigator).SpectatorWeapon.GetBoneCoords(FOPawn(Instigator).SpectatorWeapon.Master.Default.TracerBone).Origin);
	}
	else if (FOPawn(Instigator).WeaponAttachment != None) // else for everyone else, attach it to the 3rd person weapon's flash bone
	{
		BulletTracerClass = class'Effect_BulletTracer3rd';
		SetLocation(FOPawn(Instigator).WeaponAttachment.GetBoneCoords(Attachment_Base(FOPawn(Instigator).WeaponAttachment).FlashBone).Origin);
	}
	else
		return; // else do nothing

	// so dead bodies aren't so... boring
	CheckBodyHits();
	
	// get the rotation
	TrailRotation = rotator(SpawnLoc - Location);
	
	// do splashes
	CheckSplash(SpawnLoc);

	// only spawn tracers randomly
	if (FRand() < 0.5 || (Level.bDropDetail && FRand() < 0.8) || !bShowTracers)
		return;

	// don't spawn if through wall
	if (!FastTrace(Location,Location + vector(TrailRotation)*(205+TrailOffset)))
		return;

	// spawn the tracer
	BulletTracer = spawn(BulletTracerClass,,, Location + vector(TrailRotation)*(180+TrailOffset), TrailRotation);
	
	// project it
	if (BulletTracer != None)
		BulletTracer.ProjectBeam(Location,SpawnLoc);
}

// this makes dead bodies more interesting
simulated function CheckBodyHits()
{
	local vector HitLocation, NewNormal, StartLoc;
	local Actor Other;

	// don't do in listen or standalone games
	if (Level.NetMode == NM_StandAlone || Level.NetMode == NM_ListenServer)
		return;

	// find a hit on a body first
	StartLoc = SpawnLoc;
	Other = Trace(HitLocation,NewNormal,SpawnLoc,Location+Normal(SpawnLoc-Location)*8,true);
	if (Other != None && ((FOPawn(Other) != None && Other.IsInState('Dying')) || Armor_Helmet(Other) != None))
	{
		// take damage
		bHitBody = true;
		Other.TakeDamage(30, Instigator, HitLocation, (SpawnLoc-Location), MyDamageType);	

		if (FOPawn(Other) != None)
			FOPawn(Other).PlayBodyHitSound();
	}
}

simulated function CheckSplash(vector BulletLoc)
{
	local Actor HitActor;
	local vector NewHitNormal, HitLocation;
	
	if ((Level.DetailMode != DM_Low) && (SplashEffect != None) && (Instigator == None || !Instigator.PhysicsVolume.bWaterVolume))
	{
		// check for splash
		bTraceWater = true;
		HitActor = Trace(HitLocation,NewHitNormal,BulletLoc,Location+Normal(SpawnLoc-Location)*8,true);
		bTraceWater = false;
		if ((FluidSurfaceInfo(HitActor) != None) || ((PhysicsVolume(HitActor) != None) && PhysicsVolume(HitActor).bWaterVolume))
			Spawn(SplashEffect,,,HitLocation,rot(16384,0,0));
	}
}

defaultproperties
{
	SplashEffect=class'Effect_BulletSplash'
	bShowTracers=true

	RemoteRole=ROLE_SimulatedProxy
	bHitBody=false
	bNoDelete=false
	bStatic=false
	bBlockPlayers=false
	bBounce=true
	bHighDetail=false
	AutoDestroy=false
	bNetTemporary=true
	bGameRelevant=true
	bReplicateInstigator=true
	LifeSpan=0.6
}