//=============================================================================
// Frag.Ops  2004 Matt 'SquirrelZero' Farber
//=============================================================================
class Effect_BulletShell extends Actor;

var array<sound> ImpactSounds;
var int ImpactSound;
var float Speed;
var float Lift;

function InitShell()
{
	local rotator NewRotation;

	// relevancy issue?
	if (Instigator == None || VSize(Location-Instigator.Location) > 400)
	{
		Destroy();
		return;
	}

	if (!FastTrace(Location,Level.GetLocalPlayerController().ViewTarget.Location))
	{
		if (VSize(Location-Level.GetLocalPlayerController().ViewTarget.Location) > 2500)
		{
			Destroy();
			return;
		}
		CullDistance = 2000;
	}

	// randomize rotation a bit
	NewRotation = Rotation;
	NewRotation.Yaw += 50-Rand(100);
	NewRotation.Pitch += 70-Rand(100);
	SetRotation(NewRotation);

	SetPhysics(PHYS_Falling);
	bBounce = true;

	Velocity = Vector(Rotation) * ((0.75*Speed) + ((FRand()*0.5)*Speed));
	Velocity.Z += Rand(Lift*0.25)+Lift*0.75;

	RandSpin(50000);

	if (PhysicsVolume != None && PhysicsVolume.bWaterVolume)
		Velocity *= 0.7;
}

function Landed(vector HitNormal)
{
	local rotator r, hr;

	hr = rotator(-HitNormal);
	r = Rotation;
	r.Pitch = hr.Pitch;
	SetRotation(r);
	LifeSpan = 5.0+FRand();
	if (Level.bDropDetail)
		LifeSpan *= 0.33;
	Disable('Tick');

	// cull earlier
	if (!FastTrace(Location,Level.GetLocalPlayerController().ViewTarget.Location) && Level.GetLocalPlayerController().ViewTarget != Instigator)
	{
		LifeSpan *= 0.5;
		CullDistance = 1200;
	}
	else
		CullDistance = 1900;
}

function HitWall(vector HitNormal, actor Wall)
{
	SetPhysics(PHYS_Falling);
	Velocity = (0.55+(FRand()*0.1))*(Velocity - 2.0*HitNormal*(Velocity dot HitNormal));
	RandSpin(100000);
	Speed = VSize(Velocity);

	if (Speed < 25)
		bBounce = false;

	if (ImpactSound < ImpactSounds.Length)
	{
		if (PhysicsVolume == None || !PhysicsVolume.bWaterVolume)
			PlaySound(ImpactSounds[ImpactSound],,0.26+(FRand()*0.1),,350,0.87+(FRand()*0.08));
		else
			PlaySound(ImpactSounds[ImpactSound],,0.08+(FRand()*0.08),,350,0.87+(FRand()*0.08));			
		ImpactSound++;
	}
}

// raided from the projectile class
function RandSpin(float spinRate)
{
	DesiredRotation = RotRand();
	RotationRate.Yaw = spinRate * 2 *FRand() - spinRate;
	RotationRate.Pitch = spinRate * 2 *FRand() - spinRate;
	RotationRate.Roll = spinRate * 2 *FRand() - spinRate;	
}

function Tick(float deltatime)
{
	if (Velocity.Z > 5 || Velocity.Z < -5)
	{
		if (Velocity.Z < 1200 && (PhysicsVolume == None || !PhysicsVolume.bWaterVolume))
			Velocity.Z -= 8;
	}
	Super.Tick(deltatime);
}

function Reset()
{
	Destroy();
}

event PhysicsVolumeChange(PhysicsVolume NewVolume)
{
	if (NewVolume != None && NewVolume.bWaterVolume)
		Velocity *= 0.7;
}

defaultproperties
{
	// shell impact sounds
	ImpactSounds(0)=sound'FOWeaponSounds.Misc.ShellHit1'
	ImpactSounds(1)=sound'FOWeaponSounds.Misc.ShellHit2'
	ImpactSounds(2)=sound'FOWeaponSounds.Misc.ShellHit3'
	ImpactSounds(3)=sound'FOWeaponSounds.Misc.ShellHit4'

	DrawType=DT_StaticMesh
	StaticMesh=StaticMesh'FOShellCasings.9mm'
	Skins(0)=Material'FOshellcasetex.shellshader9mm'

	LifeSpan=6
	AmbientGlow=62
	MaxLights=2
	bStatic=false
	bNoDelete=false
	RemoteRole=ROLE_None
	bCollideWorld=true
	bCollideActors=false
	bFixedRotationDir=true
	bAlwaysRelevant=true
	bUseDynamicLights=false
	bAcceptsProjectors=false
	DesiredRotation=(Pitch=1024,Yaw=1024,Roll=65535)

	// culled
	CullDistance=3500.0

	// collision
	bUseCylinderCollision=true
	CollisionHeight=+0.1
	CollisionRadius=+0.8

	// fluidsurface disturbance?
	bDisturbFluidSurface=true
	FluidSurfaceShootStrengthMod=0.4
}