//=============================================================================
// Frag.Ops  2004 Matt 'SquirrelZero' Farber
//=============================================================================
// i simply can't accept the performance loss from having to tick every tracer,
// since all our guns fire them.  Much more efficient than minigun tracers.
//=============================================================================
class Effect_BulletTracer extends Emitter;

var sound BulletWhiz[5];

simulated function ProjectBeam(vector BeamStartPoint, vector BeamEndPoint)
{
	local float LifeRange;

	// set up the liferanges, so particles fade out
	LifeRange = VSize(BeamEndPoint - Location) * 0.001;
	Emitters[0].LifeTimeRange.Min = LifeRange;
	Emitters[0].LifeTimeRange.Max = LifeRange;
	Emitters[0].FadeInEndTime = 0.03 * LifeRange;
	Emitters[0].FadeOutStartTime = 0.68 * LifeRange;
	
	// give it a lot of velocity
	Velocity = 7400 * vector(Rotation);

	// spawn the particle
	Emitters[0].SpawnParticle(1);
	Emitters[0].AllParticlesDead = false;

	// set up the lifespan, based on distance
	LifeSpan = VSize(BeamEndPoint - Location) * 0.0015;

	// play a whiz-by
	//if (FRand() < 0.5)
		//PlaySound(BulletWhiz[Rand(5)],,0.92,,25,1.1);
}

simulated function HitWall(vector HitNormal, actor Wall)
{
	Destroy();
}

simulated function Touch(Actor Other)
{
	if (Other == None)
		return;

	if (Other.bProjTarget || Other.bBlockActors || Other.bBlockPlayers || Other.DrawType == DT_StaticMesh )
	{
		if ((Velocity == vect(0,0,0) || Other.IsA('Mover')) && Other != Instigator && !Other.IsA('Effect_BulletTracer')) 
		{
			Destroy();
		}
	}
}

simulated function Landed(vector HitNormal)
{
	Destroy();
}

defaultproperties
{	
	Begin Object Class=BeamEmitter Name=BeamEmitter1
		StartSizeRange=(X=(Min=0.59,Max=0.59))
		Texture=Texture'FOMisc.Sprites.TracerBeamPointed'
		MaxParticles=2
		LifeTimeRange=(Min=0.04,Max=0.08)
		StartVelocityRange=(Z=(Min=0.0,Max=0.0),X=(Min=0.0,Max=50.0),Y=(Min=0.0,Max=0.0))
		Acceleration=(Z=0.000000,Y=0.0,X=0.0)
		StartLocationRange=(Z=(Min=-0.0,Max=0.0),Y=(Min=-0.0,Max=0.0),X=(Min=0.0,Max=1.0))
		UseRegularSizeScale=false
	        UseSizeScale=true
	        SizeScale(0)=(RelativeTime=1.00000,RelativeSize=1.0)
		UseColorScale=true
		ColorScale(0)=(Color=(B=22,G=22,R=22,A=255))
		InitialDelayRange=(Min=0.0,Max=0.0)
		FadeInEndTime=0.01
		FadeIn=true
		FadeOutStartTime=0.03
		ParticlesPerSecond=0
		InitialParticlesPerSecond=0
		FadeOut=true
		RespawnDeadParticles=false
		SecondsBeforeInactive=0.000000
		DrawStyle=PTDS_Translucent
		AutomaticInitialSpawning=false
		RelativeWarmupTime=1.0
		WarmupTicksPerSecond=1.0
		DetermineEndPointBy=PTEP_Distance
		CoordinateSystem=PTCS_Relative
		BeamDistanceRange=(Min=114.0,Max=114.0)
		RotatingSheets=5
	Name="BeamEmitter1"
	End Object
	Emitters(0)=BeamEmitter'BeamEmitter1'

	BulletWhiz[0]=sound'FOWeaponSounds.Misc.BulletWhiz1'
	BulletWhiz[1]=sound'FOWeaponSounds.Misc.BulletWhiz2'
	BulletWhiz[2]=sound'FOWeaponSounds.Misc.BulletWhiz3'
	BulletWhiz[3]=sound'FOWeaponSounds.Misc.BulletWhiz4'
	BulletWhiz[4]=sound'FOWeaponSounds.Misc.BulletWhiz1'

	Physics=PHYS_Projectile

	bBounce=true
	bBlockPlayers=false
	bCollideActors=true
	bCollideWorld=true
	bHighDetail=false
	AutoDestroy=false
	bNoDelete=false
	bDirectional=true
	bStatic=false
	bUnlit=true
	RemoteRole=ROLE_None

	LifeSpan=0.3
}