//=============================================================================
// Frag.Ops  2003 Matt 'SquirrelZero' Farber
//=============================================================================
// Quick, clean actor designed specifically for spawning effects, spawned from
// the server.
//=============================================================================
class Effect_ExplosionSpawner extends XEmitter;

// hitlocation, hitnormal
var vector HL, HN;

var class<Projector> ScorchMark;
var class<Emitter> ExplosionEmitterClass;
var class<Actor> ChunkEmitterClass;
var class<Effects> ExplosionEffectClass;
var class<Weapon_GrenadeProjectile> GrenadeClass;

replication
{
	reliable if (Role == ROLE_Authority)
		HL, HN;
}

function DoExplosion(vector HitLocation, vector HitNormal)
{
	HL = HitLocation;
	HN = HitNormal;

	if (Level.NetMode != NM_DedicatedServer)
		SpawnExplosion(hl, hn);
}

simulated function PostNetBeginPlay()
{
	if (Role < ROLE_Authority)
		SpawnExplosion(hl, hn);
}

simulated function PlayExplosionSound()
{
	PlaySound(sound'WeaponSounds.BExplosion3',,2.4*TransientSoundVolume,,1000);
}

simulated function SpawnExplosion(vector EffectLocation, vector EffectNormal)
{
	local Actor ActorNearby;
	local bool bFoundActor;
	
	PlayExplosionSound();
	if (EffectIsRelevant(Location,false))
	{
		if (ExplosionEmitterClass != None)
			Spawn(ExplosionEmitterClass,,, EffectLocation, rotator(vect(0,0,1)));
		if (ChunkEmitterClass != None)
		{
			foreach CollidingActors(class'Actor',ActorNearby,200.0,Location)
			{
				if (ActorNearby != None && (ActorNearby.StaticMesh != None || ActorNearby.bWorldGeometry || ActorNearby.bBlockActors || ActorNearby.bBlockPlayers))
				{
					bFoundActor = true;
					break;
				}
			}
		
			if (bFoundActor)
				Spawn(ChunkEmitterClass,,, EffectLocation, rotator(vect(0,0,1)));
		}
		if (ScorchMark != None)
			Spawn(ScorchMark, self,, EffectLocation, rotator(-EffectNormal));
		if (ExplosionEffectClass != None)
			Spawn(ExplosionEffectClass,,, EffectLocation, rotator(vect(0,0,1)));
	}

	// agitate dead bodies
	if (ExplosionEmitterClass != None && Level.NetMode != NM_ListenServer && Level.NetMode != NM_StandAlone)
	{
		SetLocation(EffectLocation);
		AgitateBodies();
	}
	Destroy();
}

// client damage simulation
simulated function AgitateBodies()
{
	local FOPawn Victims;
	local vector dir;
	local float damageScale, dist;

	SetCollision(true,false,false);
	foreach VisibleCollidingActors(class'FOPawn', Victims, GrenadeClass.Default.DamageRadius)
	{
		if (Victims.IsInState('Dying'))
		{
			dir = Victims.Location - Location;
			dist = FMax(1,VSize(dir));
			dir = dir/dist;
			damageScale = 1 - FMax(0,(dist - Victims.CollisionRadius)/GrenadeClass.Default.DamageRadius);

			Victims.TakeDamage
			(
				damageScale * GrenadeClass.Default.Damage,
				Instigator,
				Victims.Location - 0.5 * (Victims.CollisionHeight + Victims.CollisionRadius) * dir,
				(damageScale * GrenadeClass.Default.MomentumTransfer * dir),
				GrenadeClass.Default.MyDamageType
			);
		}
	}
	SetCollision(false,false,false);
}

defaultproperties
{
	bStatic=false
	bWorldGeometry=false
	bHidden=true
	bHighDetail=false
	bNoDelete=false
	bDirectional=true
	bGameRelevant=true
	bAlwaysRelevant=true
	bNetTemporary=true
	bNetInitialRotation=false
	RemoteRole=ROLE_SimulatedProxy
	LifeSpan=0.75
	NetPriority=1.5

	// settings
	ScorchMark=class'RocketMark'
	ExplosionEmitterClass=class'Effect_GrenadeHEExplosion'
	ChunkEmitterClass=class'ExplosionCrap'
	GrenadeClass=class'Weapon_GrenadeHEProjectile'
	mRegen=false
	Style=STY_None
	bUnlit=true
	Physics=PHYS_None

	mStartParticles=0
	mLifeRange(0)=0.0
	mMaxParticles=0
}