//=============================================================================
// Frag.Ops  2003 Matt 'SquirrelZero' Farber
//=============================================================================
// I needed a flickering light that actually worked.. so here we go.
//=============================================================================
class Effect_FlickeringLight extends Light;

// humming sound for light
var() sound LightOnSound;

// mesh to scale glow on (for example a light post, lightbulb, etc)
var() Actor BrightenMesh;

var bool bOn, bWasTriggered;

var() bool bIsTriggered;

replication
{
	reliable if (Role == ROLE_Authority)
		bOn;
}

simulated function PostBeginPlay()
{
	Super.PostBeginPlay();
	if (Level.NetMode == NM_DedicatedServer)
	{
		bDynamicLight = false;
		return;
	}

	LightType = LT_Steady;
	SetTimer(0.1,false);

	if (!bIsTriggered)
		Disable('Tick');
}

simulated function Timer()
{
	if (Level.NetMode == NM_DedicatedServer || (bIsTriggered && !bOn))
	{
		bDynamicLight = false;
		LightType = LT_None;
		return;
	}

	bDynamicLight = !bDynamicLight;
	LightType = LT_Steady;

	if (bDynamicLight)
	{
		AmbientSound = LightOnSound;
		if (BrightenMesh != None)
		{
			BrightenMesh.bUnlit = true;
			BrightenMesh.AmbientGlow = 5;
			BrightenMesh.ScaleGlow = 3.0;
		}
		SetTimer(FClamp(FRand()-0.5, 0.05, 0.2),false);
	}
	else
	{
		AmbientSound = None;
		if (BrightenMesh != None)
		{
			BrightenMesh.bUnlit = false;
			BrightenMesh.AmbientGlow = 0;
			BrightenMesh.ScaleGlow = 0.0;
		}
		SetTimer(FClamp(FRand(), 0.05, 1.0),false);
	}
}

function Trigger(actor Other, pawn EventInstigator)
{
	bOn = true;
}

simulated function Tick(float dt)
{
	Super.Tick(dt);
	if (Level.NetMode == NM_DedicatedServer)
		return;
	
	if (bOn && !bWasTriggered)
	{
		bWasTriggered = true;
		SetTimer(0.05,false);
	}
	else if (!bOn && bWasTriggered)
		bWasTriggered = false;
}

function Reset()
{
	bOn = false;
}

defaultproperties
{
	bAlwaysRelevant=true
	bIsTriggered=false
	bOn=false
	LightType=LT_Steady
	bDynamicLight=true
	bStatic=false
	LightOnSound=sound'IndoorAmbience.buzz1'
	RemoteRole=ROLE_SimulatedProxy
}