//=============================================================================
// Frag.Ops  2003 Matt 'SquirrelZero' Farber
//=============================================================================
// slimmed down version of flakchunk to save resources
//=============================================================================
class Effect_LAWChunk extends Actor;

var byte Bounces;
var sound ImpactSound; 
var float Speed;
var byte KarmaType, NewLifeSpan;
var bool bUseKarma;
var float LastImpactTime;

replication
{
	reliable if (bNetInitial && Role == ROLE_Authority)
		ImpactSound, bUseKarma, KarmaType, Speed, NewLifeSpan;

	reliable if (bNetInitial && !bUseKarma && Role == ROLE_Authority)
		Bounces;
}

simulated function Reset()
{
	SetPhysics(PHYS_None);
	KSetBlockKarma(false);
	Destroy();
}

function PreBeginPlay()
{
	local float r;

	Super.PreBeginPlay();
	
	if (Role < ROLE_Authority)
		return;

	r = FRand();
	if (r > 0.75)
		Bounces = 6;
	else if (r > 0.25)
		Bounces = 5;
	else
		Bounces = 4;
}

function SetKarma(bool bSetKarma, int SetKarmaType)
{
	if (Level.PhysicsDetailLevel != PDL_High)
	{
		bSetKarma = false;
		KarmaType = 0;
	}
	
	bUseKarma = bSetKarma;
	KarmaType = SetKarmaType;

	if (Level.NetMode != NM_DedicatedServer)
	{
		if (NewLifeSpan > 0)
			LifeSpan = NewLifeSpan;
		
		Velocity = Vector(Rotation) * (Speed);

		if (PhysicsVolume.bWaterVolume)
			Velocity *= 0.65;

		if (!bUseKarma)
			RandSpin(120000);

		if (bUseKarma)
			SwitchToKarma();
	}
}

simulated function PostNetBeginPlay()
{
	Super.PostNetBeginPlay();

	if (Role < ROLE_Authority)
	{
		if (NewLifeSpan > 0)
			LifeSpan = NewLifeSpan;
		
		Velocity = Vector(Rotation) * (Speed);

		if (PhysicsVolume.bWaterVolume)
			Velocity *= 0.65;

		if (!bUseKarma)
			RandSpin(120000);

		if (bUseKarma)
			SwitchToKarma();
	}
}		

simulated function SwitchToKarma()
{
	local vector OldVelocity;
	local vector RandomSpin;

	OldVelocity = Velocity;	

	switch(KarmaType)
	{
		// lead
		case 0:
			KParams = new(none) class'Karma_Lead';
			break;

		// steel
		case 1:
			KParams = new(none) class'Karma_Steel';
			break;
		
		// rubber
		case 2:
			KParams = new(none) class'Karma_Rubber';
			break;

		// feather
		case 3:
			KParams = new(none) class'Karma_Feather';
			break;

		// wood
		case 4:
			KParams = new(none) class'Karma_Wood';
			break;
	}

	// reset a couple things
	RotationRate = rot(0,0,0);
	DesiredRotation = rot(0,0,0); 
	bFixedRotationDir = false;
	bRotateToDesired = false;

	// create a random spin
	RandomSpin.X = (FRand()*60) - 30;
	RandomSpin.Y = (FRand()*60) - 30;
	RandomSpin.Z = (FRand()*60) - 30;

	// set up velocity, rotation
	KarmaParams(KParams).KStartAngVel = OldVelocity*vector(Rotation) + RandomSpin;
	KarmaParams(KParams).KStartLinVel = OldVelocity;

	// turn karma on
	SetCollision(true,false,false);
	KSetBlockKarma(true);
	SetPhysics(PHYS_Karma);
	KWake();
}

simulated event KImpact(Actor other, Vector pos, vector ImpactVel, vector impactNorm)
{
	local float ImpactVolume;

	if (Level.TimeSeconds - LastImpactTime > 0.1 && (Other == None || Other.Physics != PHYS_Karma))
	{
		ImpactVolume = FClamp(VSize(ImpactVel)/(KarmaParams(KParams).KImpactThreshold*6), 0.1, 0.68);
		LastImpactTime = Level.TimeSeconds;
		PlaySound(ImpactSound,, ImpactVolume,,300,1.0);
	}
}

simulated function HitWall( vector HitNormal, actor Wall )
{
	SetPhysics(PHYS_Falling);
	if (Bounces > 0)
	{
		RandSpin(120000);
		Velocity = 0.65 * (Velocity - 2.0*HitNormal*(Velocity dot HitNormal));
		Bounces = Bounces - 1;
		PlaySound(ImpactSound, SLOT_Misc, 0.68,,300,1.0);
		return;
	}
	bBounce = false;
}

simulated function Landed(Vector HitNormal)
{
	LifeSpan = 0.2;
	DesiredRotation = rotator(-HitNormal);
}

simulated function RandSpin(float spinRate)
{
	DesiredRotation = RotRand();
	RotationRate.Yaw = spinRate * 2 *FRand() - spinRate;
	RotationRate.Pitch = spinRate * 2 *FRand() - spinRate;
	RotationRate.Roll = spinRate * 2 *FRand() - spinRate;	
}

defaultproperties
{
	Speed=800.0
	
	DrawScale=1.0
	LifeSpan=6.0
	Style=STY_Alpha
	ScaleGlow=1.0
	DrawType=DT_StaticMesh
	StaticMesh=None
	bBounce=true
	Bounces=1
	AmbientGlow=20
	Physics=PHYS_Falling
	DesiredRotation=(Pitch=24000,Yaw=8000,Roll=6000)
	bFixedRotationDir=true
	bAcceptsProjectors=false
	bCollideWorld=true
	RemoteRole=ROLE_SimulatedProxy
	bUnlit=false
	bDisturbFluidSurface=true
	bGameRelevant=true
	bNetTemporary=true
	bOrientOnSlope=true
	bSkipActorPropertyReplication=true
}