//=============================================================================
// Frag.Ops  2003 Matt 'SquirrelZero' Farber
//=============================================================================
// Again based (loosely) off the taclight projector.
//=============================================================================
class Effect_LaserProjector extends DynamicProjector;

var rotator ControllerRotation;

// setup the pawn and controller variables, spawn the dynamic light
simulated function PostBeginPlay()
{
	if (bProjectActor)
		SetCollision(True, False, False);
	Disable('Tick');
}

// updates the laser projector position
simulated function UpdateLight(float DeltaTime)
{
	local vector StartTrace,EndTrace,X,Y,Z,HitLocation,HitNormal;
	local actor SurfaceActor;
	local float BeamLength;
	
	// we're changing its location and rotation, so detach it
	DetachProjector();

	// get the rotation
	GetAxes(ControllerRotation,X,Y,Z);

	// ok, first we need to get a location directly in front of the player
	StartTrace = Instigator.Location + Instigator.EyePosition() + X*(Instigator.CollisionRadius*0.55);

	// get a LONG distance
	EndTrace = StartTrace + 6000*vector(ControllerRotation);
	SurfaceActor = Trace(HitLocation,HitNormal,EndTrace,StartTrace,true);

	if (FOPawn(Instigator).IsControlledByMe())
	{
		// so it's brighter!
		bProjectOnBackFaces = true;
 
		BeamLength = VSize(HitLocation-Instigator.Location);
		
		// this will reduce the number of surfaces the projector hits, performance tweak
		if (HitLocation != vect(0,0,0))
			MaxTraceDistance = fMax((BeamLength*2.0),3000);
		else
			MaxTraceDistance = 6000;
	
		// put it in front of us
		SetLocation(StartTrace);
		SetRotation(ControllerRotation);
		
		// scale it up at distance a bit
		SetDrawScale(fClamp(Default.DrawScale+(BeamLength*0.000006),Default.DrawScale,Default.DrawScale*1.3));

		// reattach it, unless this taclight is not owned by this player
		AttachProjector();	
		return;
	}
	
	// if we hit nothing, do nothing
	if (HitLocation == vect(0,0,0))
		return;

	SetDrawScale(0.07);
	MaxTraceDistance = 20;

	// clear the base, we're going to be doing some adjustments
	if (Base != None);
		SetBase(none);

	// set location/rotation
	SetBase(Instigator);
	SetRotation(rotator(HitLocation - (Instigator.Location+Instigator.EyePosition())));
	SetLocation(HitLocation-(10*vector(Rotation)));

	// reattach it, unless this taclight is not owned by this player
	AttachProjector();
}

// override completely
simulated function Tick(float delta)
{}

defaultproperties
{
	ProjTexture=Texture'FOMisc.Projected.LaserPointerNew'
	MaterialBlendingOp=None
	FrameBufferBlendingOp=PB_Add
	MaxTraceDistance=6000
	bClipBSP=true
	bProjectOnUnlit=true
	bGradient=false
	bProjectOnAlpha=true
	bProjectOnParallelBSP=true
	bUnlit=true
	bLightChanged=true
	RemoteRole=ROLE_None
	DrawScale=0.05000
	FOV=1
	bGameRelevant=true
	bMovable=true
	bDirectional=true
	bNoProjectOnOwner=true
	bLevelStatic=false
}