//=============================================================================
// Frag.Ops  2003 Matt 'SquirrelZero' Farber
//=============================================================================
// Laser spawner base, works just like the Laser spawner, except even the
// owner uses replicated rotation updates for true WYSIWYG action.
//=============================================================================
class Effect_LaserSpawner extends Emitter;

// Laser projector
var Effect_LaserProjector Laser;

// Laser corona
//var Effect_LaserCorona3rd LaserCorona3rd;

var rotator LaserRotation;
var bool bLightOn;

// pawn controlling this Laser
var Pawn LightPawn;

replication
{
	reliable if (bNetDirty && Role == ROLE_Authority)
		bLightOn, LightPawn;

	reliable if (!bNetOwner && bNetDirty && Role == ROLE_Authority)
		LaserRotation;
}

function SetLight()
{
	//local Attachment_Base Attach3rd;
	if (Level.NetMode != NM_DedicatedServer && Role == ROLE_Authority)
	{
		// set our instigator
		Instigator = LightPawn;

		Laser = spawn(class'Effect_LaserProjector', None,, Location, Rotation);
		Laser.SetOwner(Instigator.Weapon);
		Laser.Instigator = LightPawn;

		/*
		
		if (!FOPlayer(Instigator.Controller).bBehindView)
			return;

		// attach a dynamic corona
		Attach3rd = Attachment_Base(FOPawn(LightPawn).WeaponAttachment);
		if (Attach3rd != None)
		{
			LaserCorona3rd = spawn(class'Effect_LaserCorona3rd', None,, Location, Rotation);
			if (LaserCorona3rd != None)
				Attach3rd.AttachToBone(LaserCorona3rd, Attach3rd.LightBone);
		}*/
	}
}

simulated function PostNetBeginPlay()
{
	//local Attachment_Base Attach3rd;

	if (Level.NetMode == NM_DedicatedServer || Level.NetMode == NM_StandAlone || Laser != None)
		return;

	if (LightPawn == None)
	{
		Disable('Tick');
		Destroy();
		return;
	}
	
	// set our instigator
	Instigator = LightPawn;

	// spawn the laser
	Laser = spawn(class'Effect_LaserProjector', None,, Location, Rotation);
	if (Laser != None)
	{
		if (FOPawn(LightPawn).IsControlledByMe())
			Laser.SetOwner(Instigator.Weapon);
		else
			Laser.SetOwner(Instigator);
		Laser.Instigator = LightPawn;
	}

	/*
	// attach a dynamic corona
	Attach3rd = Attachment_Base(FOPawn(LightPawn).WeaponAttachment);
	if (Attach3rd != None && !FOPawn(LightPawn).IsControlledByMe())
	{
		LaserCorona3rd = spawn(class'Effect_LaserCorona3rd', None,, Location, Rotation);
		if (LaserCorona3rd != None)
			Attach3rd.AttachToBone(LaserCorona3rd, Attach3rd.LightBone);
	}*/
}

simulated function Tick(float delta)
{
	if (LightPawn == None)
		return;
	
	if (LightPawn.Health <= 0)
	{
		Disable('Tick');
		Destroy();
		return;
	}

	if (Role == ROLE_Authority || FOPawn(LightPawn).IsControlledByMe())
		LaserRotation = LightPawn.GetViewRotation();
	if (Laser != None)
	{
		if (!bLightOn)
		{
			Laser.DetachProjector();
			//if (LaserCorona3rd != None)
				//LaserCorona3rd.HideCorona();
			return;
		}
		//if (LaserCorona3rd != None)
			//LaserCorona3rd.ShowCorona();
		Laser.Instigator = LightPawn;
		Laser.ControllerRotation = LaserRotation;
		Laser.UpdateLight(delta);
	}
}

simulated function Destroyed()
{
	Super.Destroyed();
	if (Laser != None)
	{
		Laser.DetachProjector();
		Laser.Destroy();
	}
}

function Reset()
{
}

defaultproperties
{
	bCollideActors=false
	bCollideWorld=false
	bReplicateInstigator=false
	bStatic=false
	bWorldGeometry=false
	bHighDetail=false
	bNoDelete=false
	bDirectional=true
	bGameRelevant=true
	bAlwaysRelevant=true
	bNetTemporary=false
	bNetInitialRotation=true
	RemoteRole=ROLE_SimulatedProxy
	bSkipActorPropertyReplication=true
	bHardAttach=true
	Physics=PHYS_None
	bOnlyDrawIfAttached=false
	LifeSpan=0
	bDisturbFluidSurface=false
	bCanBeDamaged=false
	StaticMesh=None

	NetPriority=1.5
	NetUpdateFrequency=160.0

	CollisionRadius=+0.00000
	CollisionHeight=+0.00000
}