//=============================================================================
// Frag.Ops  2004 Matt 'SquirrelZero' Farber
//=============================================================================
class Effect_ShellEjectorFirst extends Actor;

var StaticMesh BulletMesh, LowDetailMesh;
var float BulletScale, BulletSpeed, BulletLift;
var int MaxShells;
var array<Effect_BulletShell> Shells;
var array<sound> ImpactSounds;
var bool bThird;

function InitEjector()
{
	if (bThird)
		MaxShells = Default.MaxShells/2;
	Shells.Insert(0,MaxShells);	
	Disable('Tick');
}

function Trigger(Actor Other, Pawn EventInstigator)
{
	SpawnShell(Other, EventInstigator);
}

function SpawnShell(Actor Weapon, Pawn ShellInstigator)
{
	local int i;
	local Effect_BulletShell BulletShell;
	local rotator BulletRot;
	local vector BulletLoc,X,Y,Z;
	local bool bAdded;
	local name ShellBone;

	if (Level.GetLocalPlayerController().bBehindView && Attachment_Base(Weapon) == None && War_FOVehicleWeapon(Weapon) == None)
		return;

	if (Instigator == None)
		Instigator = ShellInstigator;

	// dont eject
	if (Instigator == None || (((Level.GetLocalPlayerController().ViewTarget == Instigator && !Level.GetLocalPlayerController().bBehindView) || Level.bDropDetail) && Attachment_Base(Weapon) != None))
		return;
	
	if (War_FOVehicleWeapon(Weapon) != None)
	{
		ShellBone = War_FOVehicleWeapon(Weapon).Default.ShellBone;
		// no shader in 3rd person
		if (Shader(Skins[0]) != None)
			Skins[0] = Shader(Default.Skins[0]).FallbackMaterial;
		if (LowDetailMesh != None)
			BulletMesh = LowDetailMesh;
	}
	else if (FOSpectatorWeapon(Weapon) != None)
		ShellBone = FOSpectatorWeapon(Weapon).Master.Default.ShellBone;
	else if (Weapon_Base(Weapon) != None)
		ShellBone = Weapon_Base(Weapon).Default.ShellBone;
	else if (Attachment_Base(Weapon) != None)
	{
		ShellBone = Attachment_Base(Weapon).Default.ShellBone;
		// no shader in 3rd person
		if (Shader(Skins[0]) != None)
			Skins[0] = Shader(Default.Skins[0]).FallbackMaterial;
		if (LowDetailMesh != None)
			BulletMesh = LowDetailMesh;
	}
	else
		ShellBone = Weapon_SlaveBase(Weapon).Master.Default.ShellBone;

	if (ShellBone != '' && (War_FOVehicleWeapon(Weapon) != None || (Attachment_Base(Weapon) != None && !FOPawn(Instigator).IsControlledByMe()) || (Weapon_SlaveBase(Weapon) != None && Weapon_SlaveBase(Weapon).Hand >= -1.0 && Weapon_SlaveBase(Weapon).Hand <= 1.0) || (Weapon_Base(Weapon) != None && Weapon_Base(Weapon).Hand >= -1.0 && Weapon_Base(Weapon).Hand <= 1.0 && !Weapon_Base(Weapon).bScopeZoomed) || (FOSpectatorWeapon(Weapon) != None && FOSpectatorWeapon(Weapon).Hand >= -1.0 && FOSpectatorWeapon(Weapon).Hand <= 1.0 && !FOSpectatorWeapon(Weapon).Controller.bTargetScopeZoomed)))
	{
		if (Weapon_Base(Weapon) != None && Weapon_Base(Weapon).bIsAkimbo)
		{
			Weapon.GetAxes(Weapon.Rotation,X,Y,Z);
			BulletLoc = (Weapon_Base(Weapon).SlaveWeapon.GetBoneCoords(ShellBone).Origin-13*(vector(Weapon.Rotation))) + Y*Weapon_Base(Weapon).ShellHackOffset;
			BulletRot = Weapon.GetBoneRotation(ShellBone);
			BulletRot.Yaw = (BulletRot.Yaw & 65535) - 32767;
			BulletShell = spawn(class'Effect_BulletShell',self,, BulletLoc, BulletRot);
		}
		else if (Inventory(Weapon) != None)
			BulletShell = spawn(class'Effect_BulletShell',self,, Weapon.GetBoneCoords(ShellBone).Origin-13*(vector(Weapon.Rotation)), Weapon.GetBoneRotation(ShellBone));
		else
			BulletShell = spawn(class'Effect_BulletShell',self,, Weapon.GetBoneCoords(ShellBone).Origin, Weapon.GetBoneRotation(ShellBone));
	}
	else if (Attachment_Base(Weapon) == None)
	{
		// scoped shells
		BulletRot = Weapon.Instigator.GetViewRotation();
		BulletRot.Yaw += 16384;
		BulletShell = spawn(class'Effect_BulletShell',self,, Instigator.Location + ((Instigator.CollisionRadius*0.55)*vector(Instigator.GetViewRotation())) + (8*vector(BulletRot)), BulletRot);
	}
	else 
		return;

	// fallback
	if (BulletShell == None)
		return;

	BulletShell.SetStaticMesh(BulletMesh);

	// perf tweak
	BulletShell.Skins[0] = Skins[0];
	BulletShell.SetDrawScale(BulletScale);
	BulletShell.Speed = BulletSpeed;
	BulletShell.ImpactSounds = ImpactSounds;
	BulletShell.Lift = BulletLift;
	BulletShell.Instigator = Instigator;

	// this gets around an issue with relevancy
	if ((Attachment_Base(Weapon) != None || War_FOVehicleWeapon(Weapon) != None) && ShellBone != '')
	{
		Weapon.BoneRefresh();
		BulletShell.SetLocation(Weapon.GetBoneCoords(ShellBone).Origin);
	}
	BulletShell.InitShell();

	// fallback
	if (BulletShell == None)
		return;

	// works like maxparticles
	for (i=0;i<Shells.Length;i++)
	{
		if (Shells[i] == None && !bAdded)
		{
			bAdded = true;
			Shells[i] = BulletShell;
		}
	}
	if (!bAdded)
	{
		Shells[0].Destroy();
		for (i=1;i<Shells.Length;i++)
			Shells[i-1] = Shells[i];
		Shells[Shells.Length-1] = BulletShell;
	}
}

defaultproperties
{
	// shell impact sounds
	ImpactSounds(0)=sound'FOWeaponSounds.Misc.ShellHit1'
	ImpactSounds(1)=sound'FOWeaponSounds.Misc.ShellHit2'
	ImpactSounds(2)=sound'FOWeaponSounds.Misc.ShellHit3'
	ImpactSounds(3)=sound'FOWeaponSounds.Misc.ShellHit4'
	
	BulletMesh=StaticMesh'FOShellCasings.9mm'
	LowDetailMesh=StaticMesh'FOShellCasings.9mm_low'
	Skins(0)=Material'FOshellcasetex.shellshader9mm'
	BulletScale=1.0
	BulletSpeed=160.0
	BulletLift=110.0
	MaxShells=15
	
	bHidden=true
	bStatic=false
	bNoDelete=false
	bHardAttach=true
	bGameRelevant=true
	bAlwaysRelevant=true
	RemoteRole=ROLE_None
}