//=============================================================================
// Frag.Ops  2003 Matt 'SquirrelZero' Farber
//=============================================================================
// Very much like the strobe light we see in UED, but of course doesn't work in
// the game, except more powerful.
//=============================================================================
class Effect_StrobeLight extends Light;

// humming sound for light
var() sound LightOnSound;

// mesh to scale glow on (for example a light post, lightbulb, etc)
var() StaticMeshActor BrightenMesh;

// strobe frequency
var() float OffTime;
var() float OnTime;
var() bool bTriggerInstead;
var() float TriggerTime; 
var() float DelayTime;

simulated function PostBeginPlay()
{
	Super.PostBeginPlay();
	if (Level.NetMode == NM_DedicatedServer)
		return;

	if (bTriggerInstead)
	{
		bDynamicLight = false;
		return;
	}

	if (bDynamicLight)
		SetTimer(OnTime,false);
	else
		SetTimer(OffTime,false);
}

event Trigger(actor Other, pawn EventInstigator)
{
	GotoState('Triggered','Begin');
	bDynamicLight = false;
}

simulated state Triggered
{
	simulated function BeginState();
	simulated function EndState();

Begin:
	Sleep(DelayTime);
	SetTimer(OffTime,false);
	Sleep(TriggerTime);
	GotoState('Off');
}

simulated state Off
{
	ignores Timer;

	simulated function Timer()
	{}

	simulated function BeginState()
	{
		bDynamicLight = false;
	}

	simulated function EndState();
}

simulated function Timer()
{
	if (Level.NetMode == NM_DedicatedServer)
		return;

	bDynamicLight = !bDynamicLight;

	if (bDynamicLight)
	{
		AmbientSound = LightOnSound;
		if (BrightenMesh != None)
		{
			BrightenMesh.bUnlit = true;
			BrightenMesh.AmbientGlow = 5;
			BrightenMesh.ScaleGlow = 3.0;
		}
		SetTimer(OnTime,false);
	}
	else
	{
		AmbientSound = None;
		if (BrightenMesh != None)
		{
			BrightenMesh.bUnlit = false;
			BrightenMesh.AmbientGlow = 0;
			BrightenMesh.ScaleGlow = 1.0;
		}
		SetTimer(OffTime,false);
	}
}

defaultproperties
{
	LightType=LT_Steady
	bDynamicLight=true
	bStatic=false
	RemoteRole=ROLE_None
	OffTime=0.05
	OnTime=0.05
	TriggerTime=3.8
	DelayTime=0.9
}