//=============================================================================
// Frag.Ops  2003 Matt 'SquirrelZero' Farber
//=============================================================================
// "UT2k3 Ultimate Flashlight"
// I wrote this using _both_ a dynamic projector and a dynamic light, with the 
// dynamic projector as the focal point and lightsource providing general 
// ambient illumination.  Either one by itself is pretty dull, but combine the 
// two and you've got yourself a really nice effect if done correctly.
//=============================================================================
class Effect_TacLightProjector extends DynamicProjector;

// add a light too, there seems to be a problem with the projector darkening terrain sometimes
var Effect_TacLightGlow TacLightGlow;

var bool bShowProjector;
var rotator ControllerRotation;

// setup the pawn and controller variables, spawn the dynamic light
simulated function PostBeginPlay()
{
	if (bProjectActor)
		SetCollision(True, False, False);

	TacLightGlow = spawn(class'Effect_TacLightGlow');
	//Disable('Tick');
}

// updates the taclight projector and dynamic light positions
simulated function UpdateLight(float DeltaTime)
{
	local vector StartTrace,EndTrace,X,Y,Z,HitLocation,HitNormal;
	local actor SurfaceActor;
	local float BeamLength;

	// we're changing its location and rotation, so detach it
	DetachProjector();

	// fallback
	if (TacLightGlow == None)
	{
		if (TacLightGlow != None)
			TacLightGlow.bDynamicLight = false;
		return;
	}

	GetAxes(ControllerRotation,X,Y,Z);

	// ok, first we need to get a location directly in front of the player
	StartTrace = Instigator.Location + Instigator.EyePosition() + X*(Instigator.CollisionRadius*0.55);

	// not too far out
	EndTrace = StartTrace + 1800*vector(ControllerRotation);
	SurfaceActor = Trace(HitLocation,HitNormal,EndTrace,StartTrace,true);

	// find out how far the first hit was
	BeamLength = VSize(StartTrace-HitLocation);

	// clear the base, we're going to be doing some adjustments
	if (Base != None);
		SetBase(none);

	// this makes a neat focus effect when you get close to a wall
	if (BeamLength <= 100)
	{
		SetLocation(StartTrace+vector(ControllerRotation)*Max((0.2*BeamLength), 1));
		SetBase(Instigator);
		SetRotation(ControllerRotation);
		SetDrawScale(FMax(0.02,(BeamLength/90))*Default.DrawScale);
	}
	// else we project it normally, positioning slightly in front of the player
	else 
	{
		SetLocation(StartTrace+vector(ControllerRotation)*100);
		SetBase(Instigator);
		SetRotation(ControllerRotation);
		SetDrawScale(Default.DrawScale);
	}

	// reattach it, unless this taclight is not owned by this player
	if (bShowProjector)
		AttachProjector();

	// turns the dynamic light on if it's off
	if (!TacLightGlow.bDynamicLight)
		TacLightGlow.bDynamicLight = true;

	// better than tracing for the actor, makes it more projector-like
	if (SurfaceActor != None)
	{
		// again, neat focus effect up close, starts earlier than the dynamic projector
		if (BeamLength <= 100)
		{
			TacLightGlow.LightBrightness = TacLightGlow.Default.LightBrightness * (1.0 + (1.0 - (BeamLength/100)));
			TaclightGlow.LightRadius = TacLightGlow.Default.LightRadius * FMax(0.3,(BeamLength/100));
		}
		// else we scale its radius and brightness depending on distance from the material
		else
		{
			// fades the lightsource out as it moves farther away
			if (BeamLength >= 1300)
				TacLightGlow.LightBrightness = TacLightGlow.Default.LightBrightness * ((1800-BeamLength)/500);
			// else normal brightness
			else
				TacLightGlow.LightBrightness = TacLightGlow.Default.LightBrightness;

			// this makes the light act more like a spotlight, resizing depending on distance
			TacLightGlow.LightRadius = TacLightGlow.Default.LightRadius + (4.2 * (BeamLength/1900));
		}
		TacLightGlow.SetLocation(HitLocation-vector(ControllerRotation)*100);
		TacLightGlow.SetRotation(rotator(HitLocation-StartTrace));

	}
	// else if the trace returned nothing, then we put the light right in front of the player, scale its brightness down,
	// and give it normal radius.  This makes it seem as if the light is casting a dull glow  onto the ground in front 
	// of the player, and is dim enough for players to not notice the transition.
	else
	{

		TacLightGlow.LightRadius = TacLightGlow.Default.LightRadius;
		TacLightGlow.LightBrightness = 50;
		TacLightGlow.SetLocation(StartTrace+vector(ControllerRotation)*84);
	}
}

// override completely
simulated function Tick(float delta)
{}

simulated function Destroyed()
{
	if (TacLightGlow != None)
		TacLightGlow.Destroy();
}

defaultproperties
{
	// for my projtexture, i'm using a 256x256 DXT1 texture, created in the same way as the bulldog headlights --
	// just a bright white circle dead center on a black background.  Make sure the texture is set to TC_Clamp
	// or it'll repeat itself over the whole surface.
	ProjTexture=Texture'FOMisc.Projected.TacLight'
	MaterialBlendingOp=PB_Modulate
	FrameBufferBlendingOp=PB_Add
	MaxTraceDistance=1600
	bClipBSP=true
	bProjectOnUnlit=true
	bGradient=true
	bProjectOnAlpha=true
	bProjectOnParallelBSP=true
	bUnlit=true
	bLightChanged=false
	RemoteRole=ROLE_None
	DrawScale=0.34000
	FOV=5
	bGameRelevant=true
	bMovable=true
	bDirectional=true
	bNoProjectOnOwner=true
	bLevelStatic=false
}