//=============================================================================
// Frag.Ops  2003 Matt 'SquirrelZero' Farber
//=============================================================================
// Master spawning class for the taclight.  Spawns then replicates the needed
// information to the projector.
//=============================================================================
class Effect_TacLightSpawner extends Emitter;

// taclight projector
var Effect_TacLightProjector TacLight;

// taclight corona
var Effect_TacLightCorona3rd TacLightCorona3rd;

var rotator TacLightRotation;
var bool bLightOn;

// pawn controlling this taclight
var Pawn LightPawn;

// pawn weapon attachment pointer
var Attachment_Base Attach3rd;

replication
{
	reliable if (bNetDirty && Role == ROLE_Authority)
		bLightOn, LightPawn;

	reliable if (!bNetOwner && bNetDirty && Role == ROLE_Authority)
		TacLightRotation;
}

function SetLight()
{
	if (Level.NetMode != NM_DedicatedServer && Role == ROLE_Authority)
	{
		// set our instigator
		Instigator = LightPawn;

		TacLight = spawn(class'Effect_TacLightProjector', None,, Location, Rotation);
		TacLight.SetOwner(Instigator.Weapon);
		TacLight.Instigator = LightPawn;
		TacLight.bShowProjector = true;

		Attach3rd = Attachment_Base(FOPawn(LightPawn).WeaponAttachment);

		if (!FOPlayer(Instigator.Controller).bBehindView)
			return;

		// attach a dynamic corona
		if (Attach3rd != None)
		{
			TacLightCorona3rd = spawn(class'Effect_TacLightCorona3rd', None,, Location, Rotation);
			if (TacLightCorona3rd != None)
				Attach3rd.AttachToBone(TacLightCorona3rd, Attach3rd.LightBone);
		}
	}
}

simulated function PostNetBeginPlay()
{
	if (Level.NetMode == NM_DedicatedServer || Level.NetMode == NM_StandAlone || TacLight != None)
		return;

	if (LightPawn == None)
	{
		Disable('Tick');
		Destroy();
		return;
	}

	// set our instigator
	Instigator = LightPawn;

	// spawn the light
	TacLight = spawn(class'Effect_TacLightProjector', None,, Location, Rotation);
	TacLight.Instigator = LightPawn;
	if (TacLight != None)
	{
		if (FOPawn(LightPawn).IsControlledByMe())
		{
			TacLight.SetOwner(Instigator.Weapon);
			TacLight.bShowProjector = true;
		}
		else
			TacLight.SetOwner(Instigator);
	}

	// attach a dynamic corona
	Attach3rd = Attachment_Base(FOPawn(LightPawn).WeaponAttachment);
	if (Attach3rd != None && !FOPawn(LightPawn).IsControlledByMe())
	{
		TacLightCorona3rd = spawn(class'Effect_TacLightCorona3rd', None,, Location, Rotation);
		if (TacLightCorona3rd != None)
			Attach3rd.AttachToBone(TacLightCorona3rd, Attach3rd.LightBone);
	}
}

simulated function Tick(float delta)
{
	if (LightPawn == None)
		return;
	
	if (LightPawn.Health <= 0)
	{
		Disable('Tick');
		Destroy();
		return;
	}

	if (Role == ROLE_Authority || FOPawn(LightPawn).IsControlledByMe())
		TacLightRotation = LightPawn.GetViewRotation();
	if (TacLight != None)
	{
		if (!bLightOn)
		{
			TacLight.DetachProjector();
			TacLight.TacLightGlow.bDynamicLight = false;
			if (TacLightCorona3rd != None)
				TacLightCorona3rd.HideCorona();
			return;
		}
		if (TacLightCorona3rd != None)
			TacLightCorona3rd.ShowCorona();
		TacLight.Instigator = LightPawn;
		TacLight.ControllerRotation = TacLightRotation;
		TacLight.UpdateLight(delta);
	}
}

simulated function Destroyed()
{
	Super.Destroyed();
	if (TacLight != None)
		TacLight.Destroy();
	if (TacLightCorona3rd != None)
		TacLightCorona3rd.Destroy();
}

function Reset()
{
}

defaultproperties
{
	bCollideActors=false
	bCollideWorld=false
	bReplicateInstigator=false
	bStatic=false
	bWorldGeometry=false
	bHighDetail=false
	bNoDelete=false
	bDirectional=true
	bGameRelevant=true
	bAlwaysRelevant=true
	bNetTemporary=false
	bNetInitialRotation=true
	RemoteRole=ROLE_SimulatedProxy
	bSkipActorPropertyReplication=true
	bHardAttach=true
	Physics=PHYS_None
	bOnlyDrawIfAttached=false
	LifeSpan=0
	bDisturbFluidSurface=false
	bCanBeDamaged=false
	StaticMesh=None

	NetPriority=1.5
	NetUpdateFrequency=150.0

	CollisionRadius=+0.00000
	CollisionHeight=+0.00000
}