//=============================================================================
// Frag.Ops  2004 Matt 'SquirrelZero' Farber
//=============================================================================

#exec OBJ LOAD file=FOCharacterModels.ukx
#exec OBJ LOAD file=FOCharacterFemales.ukx
#exec OBJ LOAD file=FOCharacters.utx
#exec OBJ LOAD file=glasses.utx

class FOPawn extends xPawn
	abstract
	config(User);

// makes bodies disappear after a while
var globalconfig bool bBodiesStay;

// equipment arrays
var class<Weapon_Base> EquipList[6], AttachList[6];

// trait modifiers
var float PoisonMod, SidearmMod, GrenadeMod, MeleeMod, AutoMod, ShotgunMod, SniperMod, NapalmMod;
var float SidearmDamageMod, GrenadeDamageMod, MeleeDamageMod, AutoDamageMod, ShotgunDamageMod, SniperDamageMod, NapalmDamageMod;
var float SidearmRecoilMod, AutoRecoilMod, ShotgunRecoilMod, SniperRecoilMod, AkimboRecoilMod;
var float ThrowMod, GrenadeConeMod, GrenadeRadiusMod, FootStepMod, SpeedMod, JumpMod, ArmorMod, APMod, HackMod, RestMod;
var bool bBloodyMess;

// key skins
var byte HeadSkin;
var byte BodySkin;

// hitzones
var int HeadDamage;
var int TorsoDamage;
var int LeftArmDamage;
var int RightArmDamage;
var int LeftLegDamage;
var int RightLegDamage;

// sounds
var() sound HeadShotSound, HeadCrackSound;
var() sound BodyHitSound, BodyGruntSound;
var float LastGruntTime;

// -- anims
// different idles
var name IdleSidearmAnim;
var name IdleGrenadeAnim;
var name IdleSMGAnim;
var name IdleMeleeAnim;
var name IdleAkimboAnim;
var name IdleDriveAnim;

// different fires
var name FireAnim;
var name FireSidearmAnim, FireSidearmHeavyAnim;
var name FireGrenadeAnim;
var name FireSMGAnim;
var name FireMeleeAnim;
var name FireMeleeAltAnim;
var name FireAkimboAnim;
var name FireAkimboAltAnim;
var name SprintAnim;

// grenade prime
var name GrenadePrimeAnim;

// crouch fires
var name FireSidearmCrouchAnim;
var name FireGrenadeCrouchAnim;
var name FireSMGCrouchAnim;
var name FireMeleeCrouchAnim;
var name FireRifleCrouchAnim;

// laders
var name LadderAnim;

// poison damage
var bool bPoisoned;
var Controller PoisonController;
var float InitialPoisonTime;
var float PoisonTime;
var int PoisonDamage;
var sound PoisonChokeSound;

// jump delays
var float LastLandedTime;

// vehicles
var vector SeatOffset;

// action system
var bool bSprinting, bInjuredLeg;
var int APArmorMod;
var float SprintMultiple, SprintTime, SprintStart, RestStart, RestTime;
var sound SprintBreathSound;
var sound RestingSound;
var float NextAirSpeed;

// breathing
var float BreathHoldTime, BreathHoldStart;
var bool bHoldingBreath;

// new shadows
var Effect_ShadowController RealtimeShadow;
var globalconfig bool bRealtimeShadows;

// blood pools
var array<Actor> BloodPool;
var int NumPools, MaxPools;

// spectators, so we can be more event-driven
var array<FOSpectatorController> SpectatorControllers;
var FOSpectatorWeapon SpectatorWeapon;

// hand skins
var Material SOGloveSkin;
var array<Material> SOSleaveSkins;
var Material MercGloveSkin;
var array<Material> MercSleaveSkins;

// sticky knife
var Weapon_KnifeStuck StuckKnife;

// armor pieces
struct sArmorPiece
{
	var class<FragOps_ArmorPiece> ArmorActor;
	var StaticMesh ArmorMesh;
	var name ArmorBone;
	var vector ArmorOffset;
	var Rotator ArmorRotation;
};

var sArmorPiece ShoulderArmor[2];
var sArmorPiece LegArmor[2];
var sArmorPiece Helmets[2];

var array<FragOps_ArmorPiece> ArmorPieces;
var Material ArmorSkins[2];

// war -- build zones
var War_BuildZone CurrentBuildZone;

// war -- objectives
var War_Objective_Retrieval War_CarriedObjective;

// war -- throw reduction
var float WarThrowMod;

// war -- misc
var bool bWarGame, bTeamDeathMatch;
var float TimeOfDeath;

// war -- medstation
var bool bCanHeal, bIsHealing;
var int HealthRestored, HealthRestoreMax;
var float LastHealTime;

// everything else
var Controller EscapedController;
var Texture SkeletonTexture;
var FragOps_CRI CriticalReplicationInfo;
var Obj_RetrievalObjective CarriedObjective;
var byte TeamIndex;
var bool bLastCrouched, bHasLight, bJustSpawned, bAnimChanged, bRestrictDamage, bAllowDeRes, bCanClash, bSwungLast, bPlayingAction, bJustStepped, bNewStep, bEscaped, bCanBuy, bInStasis, bIsReloading, bHadVehicle, bDriving;
var string MiscItem, LastMiscItem, RagdollType;
var name LastBone;
var vector CenterMove, OldRagLocation;
var float LastImpactTime, LastHeadShotTime, FallSpeedThreshold;
var int NumSevers, LastLadderClimbTime, TimeTillDeRes;

replication
{
	// skins
	reliable if ((bNetInitial) && (Role == ROLE_Authority))
		HeadSkin, BodySkin;

	// equiplist, buying-related variables
	reliable if ((bNetOwner) && (Role == ROLE_Authority))
		EquipList, AttachList, bCanBuy, CurrentBuildZone;

	// pawn status-related variables
	reliable if ((bNetOwner) && (bNetDirty) && (Role == ROLE_Authority))
		 bRestrictDamage, bPoisoned, APArmorMod, APMod, HackMod;

	// misc stuff the server should send
	reliable if ((bNetDirty) && (Role == ROLE_Authority))
		MiscItem, bIsReloading, CriticalReplicationInfo, LastBone, FootStepMod, RestMod;

	// hitzone damage
	reliable if ((bNetOwner) && (bNetDirty) && (Role == ROLE_Authority))
		HeadDamage, TorsoDamage, LeftArmDamage, RightArmDamage, LeftLegDamage, RightLegDamage, War_CarriedObjective, SniperRecoilmod, AkimboRecoilMod, ShotgunRecoilMod, SidearmRecoilMod, AutoRecoilMod;

	// buymenu functions
	reliable if (Role < ROLE_Authority && !bWarGame)
		ServerAddNewGear;

	reliable if (Role == ROLE_Authority)
		ClientGotoState;
}

simulated function BeginPlay()
{
	local War_LevelInfo WL;

	Super.BeginPlay();
	foreach AllActors(class'War_LevelInfo',WL)
	{
		if (WL != None)
		{
			if (class'FOPlayer'.default.bTeamBeacons)
				bNoTeamBeacon = false;
			bWarGame = true;
			if (WL.bTeamDeathMatch)
				bTeamDeathMatch = true;
			break;
		}
	}
}

function AddSpectator(FOSpectatorController NewSpecController)
{
	local int i;
	
	for (i=0;i<SpectatorControllers.Length;i++)
	{
		if (SpectatorControllers[i] == None)
			break;
	}
	SpectatorControllers[i] = NewSpecController;
}

function DelSpectator(FOSpectatorController NewSpecController)
{
	local int i;
	
	for (i=0;i<SpectatorControllers.Length;i++)
		if (SpectatorControllers[i] == NewSpecController)
		{
			SpectatorControllers[i] = None;
			break;
		}
}

// why is this not an actor function?
simulated function ClientGotoState(name StateName, optional name StateLabel)
{
	GotoState(StateName,StateLabel);
}

simulated function PostBeginPlay()
{
	Super(UnrealPawn).PostBeginPlay();
	AssignInitialPose();

	if (bActorShadows && bPlayerShadows && (Level.NetMode != NM_DedicatedServer))
	{
		// decide which type of shadow to spawn
		if (!bRealtimeShadows)
		{
			PlayerShadow = Spawn(class'ShadowProjector',Self,'',Location);
			PlayerShadow.ShadowActor = self;
			PlayerShadow.bBlobShadow = bBlobShadow;
			PlayerShadow.LightDirection = Normal(vect(1,1,3));
			PlayerShadow.LightDistance = 320;
			PlayerShadow.MaxTraceDistance = 350;
			PlayerShadow.InitShadow();
		}
		else
		{
			RealtimeShadow = Spawn(class'Effect_ShadowController',self,'',Location);
			RealtimeShadow.Instigator = self;
			RealtimeShadow.SetOwner(self);
			RealtimeShadow.Initialize();
		}
	}
	bCanDoubleJump = false;
	LastLadderClimbTime = Level.TimeSeconds;

	// only on botmatch/listen
	if (Level.NetMode == NM_StandAlone || Level.NetMode == NM_ListenServer)
		bForceSkelUpdate = true;
}

// drop a carried objective
function DropObjective()
{
	local vector X,Y,Z;
	local vector DropVel;

	// if completed, don't drop it
	if (CarriedObjective.bCompleted || CarriedObjective.bLinkComplete)
		return;
	
	// decide how objective should be dropped
	GetAxes(GetViewRotation(),X,Y,Z);
	if (Health <= 0)
		DropVel = Velocity;
	else
	{
		DropVel = Vector(GetViewRotation());
		DropVel = DropVel * ((Velocity dot DropVel) + 350) + Vect(0,0,175);
	}

	// reset objective to world collision settings
	CarriedObjective.SetBase(None);
	CarriedObjective.SetLocation(Location + 0.8 * CollisionRadius * X - 0.5 * CollisionRadius * Y);
	if (!CarriedObjective.bCompleted && !CarriedObjective.bLinkComplete)
		CarriedObjective.bHidden = false;
	CarriedObjective.SetCollision(true,false,false);
	CarriedObjective.bCollideWorld = true;
	CarriedObjective.SetPhysics(PHYS_Falling);
	CarriedObjective.Velocity = DropVel;
	CarriedObjective.Carrier = None;

	// broadcast dropped message
	FragOpsMission(Level.Game).BroadcastLocalizedMessage(class'Interface_ObjectiveMessage',7,None,None,CarriedObjective);

	// set to none
	CarriedObjective = None;
	if (PlayerReplicationInfo != None)
		FragOps_PRI(PlayerReplicationInfo).bHasObjective = false;
}

function DropWarObjective()
{
	local vector X,Y,Z;
	local vector DropVel;
	local name ObjBone;

	// if completed, don't drop it
	if (War_CarriedObjective.bCompleted)
		return;
	
	// decide how objective should be dropped
	GetAxes(GetViewRotation(),X,Y,Z);
	if (Health <= 0)
		DropVel = Velocity;
	else
	{
		DropVel = Vector(GetViewRotation());
		DropVel = DropVel * ((Velocity dot DropVel) + 350) + Vect(0,0,175);
	}

	// reset objective to world collision settings
	ObjBone = War_CarriedObjective.AttachmentBone;
	DetachFromBone(War_CarriedObjective);
	War_CarriedObjective.SetLocation(GetBoneCoords(ObjBone).Origin);
	War_CarriedObjective.SetRelativeLocation(vect(0,0,0));
	War_CarriedObjective.SetRelativeRotation(rot(0,0,0));
	War_CarriedObjective.SetLocation(Location + 0.8 * CollisionRadius * X - 0.5 * CollisionRadius * Y);
	War_CarriedObjective.SetCollision(true,false,false);
	War_CarriedObjective.bCollideWorld = true;
	War_CarriedObjective.SetPhysics(PHYS_Falling);
	War_CarriedObjective.Velocity = DropVel;
	War_CarriedObjective.SetRotation(War_CarriedObjective.SpawnRotation);
	War_CarriedObjective.Carrier = None;
	War_CarriedObjective.SetTimer(War_CarriedObjective.ResetSeconds,false);
	War_CarriedObjective.bOnlyDrawIfAttached = false;
	War_CarriedObjective.NetUpdateTime = Level.TimeSeconds - 1;
	War_CarriedObjective.SetOwner(None);

	// drop message
	War_CarriedObjective.PrintObjectiveSplash(War_CarriedObjective.Attack_DroppedMessage, War_CarriedObjective.Defend_DroppedMessage);

	// set to none
	War_CarriedObjective = None;
	if (PlayerReplicationInfo != None)
		FragOps_PRI(PlayerReplicationInfo).bHasObjective = false;
}

// action system
simulated state PlayerAction
{
	simulated function BeginState();
	simulated function EndState();

Begin:
}

// sprints
simulated state PlayerSprinting extends PlayerAction
{
	simulated function BeginState()
	{
		if (Role == ROLE_Authority && Level.NetMode != NM_StandAlone)
			ClientGotoState('PlayerSprinting','Begin');
		else
			PlaySound(SprintBreathSound, SLOT_Pain, 3.5,,10);

		// change running animation
		//MovementAnims[0] = Default.SprintAnim;
		//ChangeAnimation();

		if (Role < ROLE_Authority)
			return;

		FOPlayer(Controller).OldWaterSpeed = WaterSpeed;
		FOPlayer(Controller).OldAirSpeed = AirSpeed;
		FOPlayer(Controller).OldGroundSpeed = GroundSpeed;

		// initialize sprint
		GroundSpeed = GroundSpeed * SprintMultiple;

		if (Physics != PHYS_Falling)
			AirSpeed = AirSpeed * SprintMultiple;
		else
			NextAirSpeed = AirSpeed * SprintMultiple;
	}

	simulated function EndState()
	{
		local FOPlayer FP;

		bSprinting = false;

		// reset running animation
		//MovementAnims[0] = Default.MovementAnims[0];
		//if (Health > 0)
			//ChangeAnimation();

		if (Level.NetMode != NM_DedicatedServer)
			PlaySound(sound'FOWeaponSounds.Misc.ShellHit8', SLOT_Pain, 0.1,,5);

		if (Role == ROLE_Authority)
		{
			if (DrivenVehicle != None && DrivenVehicle.Controller != None)
				FP = FOPlayer(DrivenVehicle.Controller);
			else if (Controller != None)
				FP = FOPlayer(Controller);
			else
				return;

			WaterSpeed = FP.OldWaterSpeed;
			if (Physics != PHYS_Falling)
				AirSpeed = FP.OldAirSpeed;
			else
				NextAirSpeed = FP.OldAirSpeed;
			GroundSpeed = FP.OldGroundSpeed;
		}
	}

Begin:
	SprintStart = Level.TimeSeconds;
	bSprinting = true;
	Sleep(SprintTime);
	GotoState('');
}

function bool DoJump(bool bUpdating)
{
	if (!bIsCrouched && !Controller.IsInState('StickyObjective') && !Controller.IsInstate('PreRound') && !bWantsToCrouch && ((Physics == PHYS_Walking) || (Physics == PHYS_Flying) || (Physics == PHYS_Flying)) )
	{
		if (Role == ROLE_Authority)
		{
			if ( bCountJumps && (Inventory != None) )
				Inventory.OwnerEvent('Jumped');
		}
		if (Physics == PHYS_Flying)
			Velocity = JumpZ * Floor;
		else if (Physics == PHYS_Flying)
			Velocity.Z = 0;
		else if (bIsWalking)
			Velocity.Z = ModifyJump(Default.JumpZ);
		else
			Velocity.Z = ModifyJump(JumpZ);
		if ((Base != None) && !Base.bWorldGeometry)
			Velocity.Z += Base.Velocity.Z;
		SetPhysics(PHYS_Falling);
	        return true;
	}
	return false;
}

simulated state PlayerResting extends PlayerAction
{
	simulated function BeginState()
	{
		if (Role == ROLE_Authority && Level.NetMode != NM_StandAlone)
			ClientGotoState('PlayerResting','Begin');
	}

	simulated function Tick(float dt)
	{
		Global.Tick(dt);
		if (Role == ROLE_Authority && ((Physics == PHYS_Falling || (!bIsWalking && !bIsCrouched)) && VSize(Velocity) >= 50))
			FOPlayer(Controller).ServerUnAction();
	}

	simulated function EndState()
	{
		if (Level.NetMode != NM_DedicatedServer)
			PlaySound(sound'FOWeaponSounds.Misc.ShellHit8', SLOT_Pain, 0.1,,5);
		else
			ClientGotoState('','');
	}

Begin:
	if (FOPlayer(Controller).ActionPoints >= FOPlayer(Controller).GetMaxAP() && Role == ROLE_Authority)
		GotoState('');
	else
	{
		if (Level.NetMode != NM_DedicatedServer)
			PlaySound(RestingSound, SLOT_Pain, 0.3,,10,1.0);
		RestStart = Level.TimeSeconds;
		Sleep(RestTime*RestMod);
		if (Role == ROLE_Authority)
		{
			FOPlayer(Controller).AddAP(1);
			FOPlayer(Controller).ReceiveLocalizedMessage(class'Interface_APMessage', 3, None, None, None);
		}
		GotoState('PlayerResting','Begin');
	}
}

simulated state PlayerHoldingBreath extends PlayerAction
{
	simulated function BeginState()
	{
		bHoldingBreath = true;
		if (Role == ROLE_Authority && Level.NetMode != NM_StandAlone)
			ClientGotoState('PlayerHoldingBreath','Begin');
	}

	simulated function EndState()
	{
		bHoldingBreath = false;
	}

Begin:
	BreathHoldStart = Level.TimeSeconds;
	Sleep(BreathHoldTime);
	GotoState('');
}

simulated event HeadVolumeChange(PhysicsVolume newHeadVolume)
{
	if (((HeadVolume != None && HeadVolume.bWaterVolume) || (newHeadVolume != None && newHeadVolume.bWaterVolume)))
	{
		if (Role == ROLE_Authority && IsInState('PlayerAction'))
			StopAction();
		if (Level.NetMode != NM_DedicatedServer && Weapon != None && Weapon_Base(Weapon).bScopeZoomed)
		{
			Weapon_Base(Weapon).ServerSetZoom(0);
			Weapon_Base(Weapon).bScopeZoomed = false;
			Weapon_Base(Weapon).SetScope();
		}
	}
	if (Role == ROLE_Authority)
		Super.HeadVolumeChange(newHeadVolume);
}

function StopAction()
{
	if (!IsInState('PlayerAction'))
		return;

	GotoState('');
	if (Level.NetMode == NM_DedicatedServer)
		ClientGotoState('');
}

function PossessedBy(Controller C)
{
	Super.PossessedBy(C);	

	if (FOBot(C) != None)
	{
		MiscItem = class'Interface_BuyMenuPlus'.Default.Misc[Rand(3)];
		CriticalReplicationInfo = FOBot(C).CriticalreplicationInfo;
	}
	else if (FOPlayer(C) != None)
	{
		if (FOPlayer(C).SelectedTrait < class'Interface_TabPlayerTeam'.Default.Traits.Length)
			class'Interface_TabPlayerTeam'.Default.Traits[FOPlayer(C).SelectedTrait].static.AddTrait(self);
		CriticalReplicationInfo = FOPlayer(C).CriticalreplicationInfo;
	}
	else
		log("FragOps: FOPawn possessed by invalid Controller "$C);

	ResetSpeeds();

	if (War_FOPlayer(C) != None && Inventory == None && Weapon == None)
	{
		UpdateWeapons(C);
		SetNewInventory(0,0);
	}
}

function UpdateWeapons(Controller C)
{
	local int i;
	
	for (i=0;i<ArrayCount(EquipList);i++)
	{
		EquipList[i] = War_FOPlayer(C).DesiredEquip[i];
		if (i < ArrayCount(AttachList))
			AttachList[i] = War_FOPlayer(C).DesiredAttach[i];
	}
	MiscItem = War_FOPlayer(C).DesiredMiscItem;
	ResetSpeeds();
}

function UnPossessed()
{
	if (Controller != None && FOPlayer(Controller) != None && FOPlayer(Controller).SelectedTrait < class'Interface_TabPlayerTeam'.Default.Traits.Length)
		class'Interface_TabPlayerTeam'.Default.Traits[FOPlayer(Controller).SelectedTrait].static.RemoveTrait(self);
	StopAction();
	Super.UnPossessed();
}

// vehicle stuff
simulated function StartDriving(Vehicle V)
{
	Super(UnrealPawn).StartDriving(V);

	bDriving = true;

	SetTwistLook(0,0);

	SetBoneRotation('Bip01 L Forearm',rot(0,0,0));
	SetBoneRotation('Bip01 L Hand',rot(0,0,0));
	bReplicateMovement = false;

	// shadows
	if (PlayerShadow != None && War_Vehicle_M240MG(V) == None)
		PlayerShadow.bShadowActive = false;

	if (RealtimeShadow != None && War_Vehicle_M240MG(V) == None)
		RealtimeShadow.bShadowActive = false;
	
	// weapon attachmenst
	if (War_FOVehiclePawnFree(V) == None)
	{
		if (WeaponAttachment != None)
			WeaponAttachment.Hide(true);
	}
	else if (V.Controller != None)
		SetOwner(V.Controller);
	
	// anims
	if (!V.bHideRemoteDriver && V.bDrawDriverinTP)
	{
		if (War_FOVehiclePawnFree(V) == None)
			LoopAnim(V.DriveAnim,,0.02,1);
		LoopAnim(V.DriveAnim,,0.02,0);
	}
	
	// reenable collision
	if (V.bDriverCollideActors)
	{
		bBlockNonZeroExtentTraces = false;
		bBlockZeroExtentTraces = true;
		SetCollision(true,true);
	}
	else
		SetCollision(false,false);
}

simulated function StopDriving(Vehicle V)
{
	Super.StopDriving(V);

	bDoTorsoTwist = true;
	bRotateToDesired = true;
	bDriving = false;
	bReplicateMovement = true;

	// reenable shadow
	if (RealtimeShadow != None)
	{
		RealtimeShadow.bShadowActive = true;
		RealtimeShadow.FillLights();
	}

	// play weapon anim
	if (GetAnimSeq() != FireAnim)
	{
		AnimBlendParams(1, 0.9, 0.0, 0.2, FireRootBone);
		LoopAnim(IdleWeaponAnim,, 0.0, 1);
	}

	// return to normal bone rotations
	if (WeaponAttachment != None)
	{
		if (Attachment_Base(WeaponAttachment).bAkimbo)
			Attachment_Base(WeaponAttachment).SetAttachProps(self);
		else
			SetWeaponAttachment(WeaponAttachment);
	}
}
// - end vehicle stuff

function StartObjective(FragOps_ChainableObjective MyObjective, optional float ObjTime)
{
	if (ObjTime > 0.0)
	{
		if (Level.NetMode == NM_DedicatedServer || Level.NetMode == NM_ListenServer)
			FOPlayer(Controller).ClientSetObjective(MyObjective);
		FOPlayer(Controller).CurrentObjective = Obj_InteractObjective(MyObjective);
		Controller.GotoState('StickyObjective','Begin');
	}
}

function StartWarObjective(War_Objective MyObjective, optional float ObjTime)
{
	if (ObjTime > 0.0)
	{
		if (Level.NetMode == NM_DedicatedServer || Level.NetMode == NM_ListenServer)
			FOPlayer(Controller).ClientSetObjective(MyObjective);
		War_FOPlayer(Controller).CurWarObjective = War_Objective_Interactive(MyObjective);
		Controller.GotoState('StickyObjective','Begin');
	}
}

function DoEscape(Objective_EscapeVolume EscapeVolume)
{
	bEscaped = true;
	bRestrictDamage = true;
	FragOps_PRI(PlayerReplicationInfo).bEscaped = true;
	CriticalReplicationInfo.AddMoney(FragOpsMission(Level.Game).Default.EscapeMoney);
	StopAction();
	SetCollision(false,false,false);
	EscapedController = Controller;
	bHidden = true;
	Velocity = vect(0,0,0);
	Acceleration = vect(0,0,0);
	SetPhysics(PHYS_None);
	if (FOBot(Controller) != None)
		FOBot(Controller).DetachPawn();
	else if (EscapedController != None)
	{
		EscapedController.GotoState('Escaped','Begin');
		//if (Level.NetMode == NM_DedicatedServer && FOPlayer(Controller) != None)
			//FOPlayer(EscapedController).ClientGotoState('Escaped','Begin');
		FOPlayer(EscapedController).ReceiveLocalizedMessage(class'Interface_GameMessage',1,None,None,EscapeVolume);
	}
	else
		Destroy();
}

function ServerAddNewGear(byte AddNewMelee, byte AddNewSidearm, byte AddNewRifle, byte AddNewGrenade, byte AddNewGrenadeSec, byte AddNewMisc, class<Weapon_Base> AddSidearmAttach, class<Weapon_Base> AddRifleAttach, class<Weapon_Base> AddMeleeAttach, int Skrilla, byte AddSidearmClips, byte AddRifleClips)
{
	if (!bCanBuy)
		return;

	// Add weapons
	if (AddNewMelee != 255)
		EquipList[0] = class'FragOps.Interface_BuyMenuPlus'.Default.Melee[AddNewMelee];
	else
		EquipList[0] = class'Weapon_FistsBase';

	if (AddNewSidearm != 255)
		EquipList[1] = class'FragOps.Interface_BuyMenuPlus'.Default.Sidearms[AddNewSidearm];
	else
		EquipList[1] = None;

	if (AddNewRifle != 255)
		EquipList[2] = class'FragOps.Interface_BuyMenuPlus'.Default.Rifles[AddNewRifle];
	else
		EquipList[2] = None;

	if (AddNewGrenade != 255)
		EquipList[3] = class'FragOps.Interface_BuyMenuPlus'.Default.Grenades[AddNewGrenade];
	else
		EquipList[3] = None;

	if (AddNewGrenadeSec != 255)
		EquipList[4] = class'FragOps.Interface_BuyMenuPlus'.Default.Grenades[AddNewGrenadeSec];
	else
		EquipList[4] = None;

	if (AddNewMisc != 255)
		MiscItem = class'FragOps.Interface_BuyMenuPlus'.Default.Misc[AddNewMisc];
	else
	{
		HeadDamage = 0;
		TorsoDamage = 0;
		LeftArmDamage = 0;
		RightArmDamage = 0;
		LeftLegDamage = 0;
		RightLegDamage = 0;
	}

	// Add attachments
	AttachList[0] = AddMeleeAttach;
	AttachList[1] = AddSidearmAttach;
	AttachList[2] = AddRifleAttach;

	// Set money, cheat check -- will eventually compare all weapon/ammo purchases vs.
	// desired money to check for mismatches
	if (Skrilla > 9999)
	{
		FragOpsGame(Level.Game).CheatKick(Controller,1);
		return;
	}
	else
		CriticalReplicationInfo.Money = Skrilla;

	// Pass it to the inventory function
	SetNewInventory(int(AddSidearmClips), int(AddRifleClips));
}

// healing
function SetHeal(bool bShouldHeal, optional War_Build_MedStation HealStation)
{
	// no heals till next wave
	bCanHeal = false;

	// now we're healing
	bIsHealing = true;
}

// objective fix
event Touch(Actor Other)
{
	if (War_Objective_RetrievalVolume(Other) != None && War_CarriedObjective != None)
		Other.Touch(War_CarriedObjective);
	Super.Touch(Other);
}

// fix -- actors that are spawned inside the collision hull of other actors
// don't trigger the touch event. Called after players are respawned only.
function CheckBuyZones()
{
	local FragOps_BuyZone aTempBuyZone;

	if (bCanBuy)
		return;
	foreach TouchingActors(class'FragOps_BuyZone',aTempBuyZone)
	{
		aTempBuyZone.Touch(self);
	}
	if (bCanBuy)
		return;
	foreach CollidingActors(class'FragOps_BuyZone',aTempBuyZone,Default.CollisionRadius)
	{
		aTempBuyZone.Touch(self);
	}
	if (bCanBuy)
		return;
	foreach DynamicActors(class'FragOps_BuyZone',aTempBuyZone)
	{
		if (VSize(Location - aTempBuyZone.Location) < (aTempBuyZone.CollisionRadius*0.5))
			aTempBuyZone.Touch(self);
	}
}

// HUD check for supply messages
simulated function bool HasClientBuildZone()
{
	local War_BuildZone aTempBuildZone;

	foreach TouchingActors(class'War_BuildZone',aTempBuildZone)
	{
		return true;
		break;
	}
	return false;
}

function CheckBuildZones()
{
	local War_BuildZone aTempBuildZone;
	local bool bGotBuildZone;

	foreach TouchingActors(class'War_BuildZone',aTempBuildZone)
	{
		bGotBuildZone = true;
		aTempBuildZone.Touch(self);
	}
	foreach CollidingActors(class'War_BuildZone',aTempBuildZone,Default.CollisionRadius)
	{
		bGotBuildZone = true;
		aTempBuildZone.Touch(self);
	}

	// reset if no buildzone
	if (!bGotBuildZone && CurrentBuildZone != None)
		CurrentBuildZone.UnTouch(self);
}

function SetNewInventory(int NewSidearmClips, int NewRifleClips)
{
	local Weapon FOTempWeapon, TempRealWeapon, TempPendingWeapon;
	local class<Weapon_Base> FOTempWeaponClass;
	local Ammo_Base FOAmmoClass;
	local bool bShouldSetAmmo, bShouldSpawnWeapon, bShouldAutoSwitch, bBringSlaveUp;
	local int AddOldAmmo, i, j, Count;
	local FOPlayer FOWeaponPlayer;

	FOWeaponPlayer = FOPlayer(Controller);

	ResetZoneDamage();

	FOWeaponPlayer.BuySound();

	// spawn the weapons now
	for (i=0;i<5;i++)
	{
		bShouldAutoSwitch = false;
		Count = 0;
		if (EquipList[i] != None && EquipList[i].Default.bCanBeBought)
		{
			bShouldSetAmmo = false;
			bShouldSpawnWeapon = true;
			AddOldAmmo = 0;

			if (AttachList[i] != None)
				FOTempWeaponClass = AttachList[i];
			else
				FOTempWeaponClass = EquipList[i];

			if (FOTempWeaponClass != None && FindWeapon(i+1) != None)
				if (FOTempWeaponClass.Default.ItemName ~= FindWeapon(i+1).Default.ItemName)
				{
					if (FOTempWeaponClass.Default.bAkimbo)
					{
						if (AttachList[i] != None && !AttachList[i].Default.bIsSpecial)
						{
							for (j=0;j<5;j++)
							{
								if (EquipList[j] == None && FindWeapon(j+1) != None)
								{
									if (FindWeapon(j+1) == Weapon && !FindGreaterEquipItem(i+1))
										bBringSlaveUp = true;
								}
							}
							Weapon_Base(FindWeapon(i+1)).SpawnSlave(0,0,true,bBringSlaveUp);
						}
						else
							Weapon_Base(FindWeapon(i+1)).KillSlave();
					}
					bShouldSpawnWeapon = false;
				}

			if (FOTempWeaponClass != None && bShouldSpawnWeapon)
			{
				// for fast access
				TempRealWeapon = FindWeapon(i+1);
				while (TempRealWeapon != None && Count < 1000)
				{
					Count++;
					if (Weapon_Base(TempRealWeapon).Default.StaticName ~= FOTempWeaponClass.Default.StaticName)
					{
						bShouldSetAmmo = true;
						AddOldAmmo = TempRealWeapon.AmmoAmount(0);
					}
					Weapon_Base(TempRealWeapon).DestroyWeapon();
				}
				FOTempWeapon = AddNewInventory(FOTempWeaponClass,false,i);
				if (bShouldSetAmmo && FOTempWeapon != None)
				{
					Weapon_Base(FOTempWeapon).GetAmmo(0).AmmoAmount = AddOldAmmo;
				}
			}
		}
		else if (EquipList[i] == None && FindWeapon(i+1) != None)
		{
			if (FindWeapon(i+1) == Weapon && TempPendingWeapon == None && !FindGreaterEquipItem(i+1))
				bShouldAutoSwitch = true;
			Weapon_Base(FindWeapon(i+1)).DestroyWeapon();
			if (bShouldAutoSwitch)
				TempPendingWeapon = GetBestWeapon();
		}
	}
	if ((NewRifleClips > 0 || bWarGame) && FindWeapon(3) != None)
	{
		FOAmmoClass = Weapon_Base(FindWeapon(3)).GetAmmo(0);
		// cheat check?
		if (NewRifleClips > FOAmmoClass.MaxClips || bWarGame)
			NewRifleClips = FOAmmoClass.MaxClips;
		FOAmmoClass.NumClips = NewRifleClips;
	}
	if ((NewSidearmClips > 0 || bWarGame) && FindWeapon(2) != None)
	{
		FOAmmoClass = Weapon_Base(FindWeapon(2)).GetAmmo(0);
		// cheat check?
		if (NewSidearmClips > FOAmmoClass.MaxClips || bWarGame)
			NewSidearmClips = FOAmmoClass.MaxClips;
		FOAmmoClass.NumClips = NewSidearmClips;
	}
	if (TempPendingWeapon != None)
		FOPlayer(Controller).ClientPendingWeapon(TempPendingWeapon);
	if (bWarGame)
		UpdatePlayerWeapons();

	bJustSpawned = false;
}

// save player weapon selections
function UpdatePlayerWeapons()
{
	local int i, j;
	local array<class<Weapon_Base> > TempList, TempAttach;

	if (War_FOPlayer(Controller) == None)
		return;

	for (i=0;i<ArrayCount(EquipList);i++)
	{
		TempList[i] = EquipList[i];
		if (i < ArrayCount(AttachList))
			TempAttach[i] = AttachList[i];
		if (i == 1)
		{
			for (j=0;j<ArrayCount(class'Interface_BuyMenuPlus'.Default.Sidearms);j++)
			{
				if (EquipList[i] == class'Interface_BuyMenuPlus'.Default.Sidearms[j] && (j % 2) == 0 && PlayerReplicationInfo.Team.TeamIndex == 1)
				{
					if (class'Interface_BuyMenuPlus'.Default.Sidearms[j+1] == class'Interface_BuyMenuPlus'.Default.Sidearms[j])
						break;
					
					if (i < ArrayCount(AttachList))
						TempAttach[i] = None;
					TempList[i] = class'Interface_BuyMenuPlus'.Default.Sidearms[1];
				}
				else if (EquipList[i] == class'Interface_BuyMenuPlus'.Default.Sidearms[j] && (j % 2) != 0 && PlayerReplicationInfo.Team.TeamIndex == 0)
				{
					if (class'Interface_BuyMenuPlus'.Default.Sidearms[j-1] == class'Interface_BuyMenuPlus'.Default.Sidearms[j])
						break;
					
					if (i < ArrayCount(AttachList))
						TempAttach[i] = None;
					TempList[i] = class'Interface_BuyMenuPlus'.Default.Sidearms[0];
				}
			}
		}
		else if (i == 2)
		{
			for (j=0;j<ArrayCount(class'Interface_BuyMenuPlus'.Default.Rifles);j++)
			{
				if (EquipList[i] == class'Interface_BuyMenuPlus'.Default.Rifles[j] && (j % 2) == 0 && PlayerReplicationInfo.Team.TeamIndex == 1)
				{
					if (class'Interface_BuyMenuPlus'.Default.Rifles[j+1] == class'Interface_BuyMenuPlus'.Default.Rifles[j])
						break;

					if (i < ArrayCount(AttachList))
						TempAttach[i] = None;
					TempList[i] = class'Interface_BuyMenuPlus'.Default.Rifles[1];
				}
				else if (EquipList[i] == class'Interface_BuyMenuPlus'.Default.Rifles[j] && (j % 2) != 0 && PlayerReplicationInfo.Team.TeamIndex == 0)
				{
					if (class'Interface_BuyMenuPlus'.Default.Rifles[j-1] == class'Interface_BuyMenuPlus'.Default.Rifles[j])
						break;
					
					if (i < ArrayCount(AttachList))
						TempAttach[i] = None;
					TempList[i] = class'Interface_BuyMenuPlus'.Default.Rifles[0];
				}
			}
		}
		if (TempList[i] == None || War_PRI(PlayerReplicationInfo).CXP >= TempList[i].Default.CXPReq)
			War_FOPlayer(Controller).DesiredEquip[i] = TempList[i];
		if (i < ArrayCount(AttachList) && (TempAttach[i] == None || War_PRI(PlayerReplicationInfo).CXP >= TempAttach[i].Default.CXPReq))
			War_FOPlayer(Controller).DesiredAttach[i] = TempAttach[i];
	}
}

function Weapon GetBestWeapon()
{
	local float Rating;

	if (Inventory == None)
		return None;

	return (Inventory.RecommendWeapon(Rating));
}


function RepairArmor()
{
	HeadDamage = 0;
	TorsoDamage = 0;
	LeftArmDamage = 0;
	RightArmDamage = 0;
	LeftLegDamage = 0;
	RightLegDamage = 0;
}

function ResetSpeeds()
{
	// speed reduction for heavier armor
	switch (MiscItem)
	{
		case "MED. Armor":
			if (IsInState('PlayerSprinting'))
			{
				FOPlayer(Controller).OldWaterSpeed = ModifySpeed(Default.WaterSpeed * 0.45);
				FOPlayer(Controller).OldAirSpeed = ModifySpeed(Default.AirSpeed * 0.44);
				FOPlayer(Controller).OldGroundSpeed = ModifySpeed(Default.GroundSpeed * 0.45);
			}
			else
			{
				WaterSpeed = ModifySpeed(Default.WaterSpeed * 0.45);
				AirSpeed = ModifySpeed(Default.AirSpeed * 0.44);
				GroundSpeed = ModifySpeed(Default.GroundSpeed * 0.45);
			}
			UpdateArmorAP(0);
			break;

		case "HVY. Armor":
			if (IsInState('PlayerSprinting'))
			{
				FOPlayer(Controller).OldWaterSpeed = ModifySpeed(Default.WaterSpeed * 0.39);
				FOPlayer(Controller).OldAirSpeed = ModifySpeed(Default.AirSpeed * 0.38);
				FOPlayer(Controller).OldGroundSpeed = ModifySpeed(Default.GroundSpeed * 0.39);
			}
			else
			{
				WaterSpeed = ModifySpeed(Default.WaterSpeed * 0.39);
				AirSpeed = ModifySpeed(Default.AirSpeed * 0.38);
				GroundSpeed = ModifySpeed(Default.GroundSpeed * 0.39);
			}
			UpdateArmorAP(-1);
			break;

		default:
			if (IsInState('PlayerSprinting'))
			{
				FOPlayer(Controller).OldWaterSpeed = ModifySpeed(Default.WaterSpeed * 0.48);
				FOPlayer(Controller).OldAirSpeed = ModifySpeed(Default.AirSpeed * 0.47);
				FOPlayer(Controller).OldGroundSpeed = ModifySpeed(Default.GroundSpeed * 0.48);
			}
			else
			{
				WaterSpeed = ModifySpeed(Default.WaterSpeed * 0.48);
				AirSpeed = ModifySpeed(Default.AirSpeed * 0.47);
				GroundSpeed = ModifySpeed(Default.GroundSpeed * 0.48);
			}
			UpdateArmorAP(1);
			break;
	}
}

function ResetZoneDamage()
{
	local War_FOPlayer WFP;

	WFP = War_FOPlayer(Controller);
	if (WFP != None && WFP.bArmorChanged && WFP.GetStateName() != 'PreRound')
		return;
	
	if (WFP != None || (LastMiscItem != MiscItem && (MiscItem ~= "MED. Armor" || MiscItem ~= "HVY. Armor" || MiscItem ~= "LIT. Armor")))
	{
		// war
		if (WFP != None)
			WFP.bArmorChanged = true;
		
		HeadDamage = 0;
		TorsoDamage = 0;
		LeftArmDamage = 0;
		RightArmDamage = 0;
		LeftLegDamage = 0;
		RightLegDamage = 0;

		ResetSpeeds();

		if (IsInState('PlayerSprinting'))
		{
			GroundSpeed = FOPlayer(Controller).OldGroundSpeed * SprintMultiple;
			WaterSpeed = FOPlayer(Controller).OldWaterSpeed * SprintMultiple;
			AirSpeed = FOPlayer(Controller).OldAirSpeed * SprintMultiple;
		}

		// reset player's AP to max
		if (FOPlayer(Controller) != None && Controller.GetStateName() == 'PreRound')
			FOPlayer(Controller).ActionPoints = FOPlayer(Controller).GetMaxAP();

		LastMiscItem = MiscItem;
	}
}

function ResetWarDamage()
{
	local War_FOPlayer WFP;

	WFP = War_FOPlayer(Controller);
	if (WFP == None || (WFP.bArmorChanged && !Controller.IsInState('PreRound')))
		return;
	
	WFP.bArmorChanged = true;
	HeadDamage = 0;
	TorsoDamage = 0;
	LeftArmDamage = 0;
	RightArmDamage = 0;
	LeftLegDamage = 0;
	RightLegDamage = 0;
}

function UpdateArmorAP(int NewArmorAP)
{
	if (FOBot(Controller) != None)
		return;
	
	APArmorMod = NewArmorAP;
	if (FOPlayer(Controller).ActionPoints > FOPlayer(Controller).GetMaxAP())
		FOPlayer(Controller).ActionPoints = FOPlayer(Controller).GetMaxAP();
}

exec function SwitchToLastWeapon()
{
	if (Controller.IsInState('Building') || (Weapon == None) || (Weapon.ClientState != WS_ReadyToFire) || (Weapon != None && Weapon.IsFiring() && Weapon_Base(Weapon).GetAmmo(0) != None && Weapon.AmmoAmount(0) > 0) || Weapon_Base(Weapon).IsReloading() || (Weapon_GrenadeBase(Weapon) != None && Weapon_GrenadeBase(Weapon).bForceStay))
		return;

	if (Weapon.OldWeapon != None)
	{
		PendingWeapon = Weapon.OldWeapon;
		Weapon.PutDown();
	}
}

simulated function SwitchWeapon(byte F)
{
	local weapon newWeapon;

	if (Controller.IsInState('Building') || (Level.Pauser != None) || (Inventory == None) || (Weapon.ClientState != WS_ReadyToFire) || (Weapon != None && Weapon.IsFiring() && Weapon_Base(Weapon).GetAmmo(0) != None && Weapon.AmmoAmount(0) > 0) || Weapon_Base(Weapon).IsReloading() || (Weapon_GrenadeBase(Weapon) != None && Weapon_GrenadeBase(Weapon).bForceStay))
		return;

	newWeapon = FindWeapon(F);

	if (newWeapon == None)
		return;

	if (PendingWeapon != None && PendingWeapon.bForceSwitch)
		return;

	if (Weapon == None)
	{
		PendingWeapon = newWeapon;
		ChangedWeapon();
	}
	else if (Weapon != newWeapon || PendingWeapon != None)
	{
		PendingWeapon = newWeapon;
		Weapon.PutDown();
	}
	else if (Weapon == newWeapon)
		Weapon.Reselect(); // sjs
}

simulated function PrevWeapon()
{
	if (Controller.IsInState('Building'))
		return;
	
	if (Weapon == None)
	{
		if (FOPlayer(Controller) != None)
			FOPlayer(Controller).NewSwitchToBestWeapon();
		else
			Controller.SwitchToBestWeapon();
		return;
	}
	
	if (Level.Pauser !=None || Weapon_Base(Weapon).IsReloading() || (Weapon.ClientState != WS_ReadyToFire) || (Weapon.IsFiring() && Weapon_Base(Weapon).GetAmmo(0) != None && Weapon.AmmoAmount(0) > 0) || (Weapon_GrenadeBase(Weapon) != None && Weapon_GrenadeBase(Weapon).bForceStay))
		return;

	if (PendingWeapon != None)
	{
		if (PendingWeapon.bForceSwitch)
			return;
		PendingWeapon = Inventory.PrevWeapon(None, PendingWeapon);
	}
	else
		PendingWeapon = Inventory.PrevWeapon(None, Weapon);

	if (PendingWeapon != None)
		Weapon.PutDown();
}

simulated function NextWeapon()
{
	if (Controller.IsInState('Building'))
		return;
	
	if (Weapon == None)
	{
		if (FOPlayer(Controller) != None)
			FOPlayer(Controller).NewSwitchToBestWeapon();
		else
			Controller.SwitchToBestWeapon();
		return;
	}
	
	if (Level.Pauser !=None || Weapon_Base(Weapon).IsReloading() || (Weapon.ClientState != WS_ReadyToFire) || (Weapon.IsFiring() && Weapon_Base(Weapon).GetAmmo(0) != None && Weapon.AmmoAmount(0) > 0) || (Weapon_GrenadeBase(Weapon) != None && Weapon_GrenadeBase(Weapon).bForceStay))
		return;

	if (PendingWeapon != None)
	{
		if (PendingWeapon.bForceSwitch)
			return;
		PendingWeapon = Inventory.NextWeapon(None, PendingWeapon);
	}
	else
		PendingWeapon = Inventory.NextWeapon(None, Weapon);

	if (PendingWeapon != None)
		Weapon.PutDown();
}

simulated function ChangedWeapon()
{
	local Weapon OldWeapon;

	if (Weapon == PendingWeapon)
		ServerChangedWeapon(None, PendingWeapon);
	else
		ServerChangedWeapon(Weapon, PendingWeapon);
	if (Role < ROLE_Authority)
	{
		OldWeapon = Weapon;
		Weapon = PendingWeapon;
		PendingWeapon = None;
		if (Controller != None)
			Controller.ChangedWeapon();

		if (Weapon != None)
			Weapon.BringUp(OldWeapon);
	}

	// for war
	if (DrivenVehicle != None && War_FOVehiclePawnFree(DrivenVehicle) != None)
		DrivenVehicle.Weapon = Weapon;
}

function ServerChangedWeapon(Weapon OldWeapon, Weapon NewWeapon)
{
	Weapon = NewWeapon;

	if (Controller != None)
		Controller.ChangedWeapon();

	PendingWeapon = None;

	if (OldWeapon != None)
	{
		OldWeapon.SetDefaultDisplayProperties();
		OldWeapon.DetachFromPawn(self);
		OldWeapon.GotoState('Hidden');
		OldWeapon.NetUpdateFrequency = 30;
		Weapon_Base(OldWeapon).GetAmmo(0).NetUpdateFrequency = 10;
	}

	if (Weapon != None)
	{
		Weapon.AttachToPawn(self);
		Weapon.BringUp(OldWeapon);
		Weapon.NetUpdateFrequency = 100;
		Weapon.NetUpdateTime = Level.TimeSeconds - 1;
		PlayWeaponSwitch(NewWeapon);
	}

	if (Inventory != None)
		Inventory.OwnerEvent('ChangedWeapon'); // tell inventory that weapon changed (in case any effect was being applied)
}


simulated function Weapon GetDemoRecordingWeapon()
{
	local inventory Inv;
	local int Count;

	for (Inv=Inventory;Inv!=None;Inv=Inv.Inventory)
	{
		if ((Weapon(Inv) != None) && (Inv.ThirdPersonActor != None))
		{
			Weapon = Weapon(Inv);
			PendingWeapon = Weapon;
			Weapon.bSpectated = true;
			break;
		}
		Count++;
		if (Count > 1000)
			return None;
	}
	return Weapon;
}

simulated function Weapon FindWeapon(int NewWeaponGroup)
{
	local Inventory Inv;
	local int Count;

	if ((Inventory == None) || (NewWeaponGroup < 0))
		return None;

	if ((Weapon != None) && (Weapon.InventoryGroup == NewWeaponGroup))
		return Weapon;

	for (Inv=Inventory;Inv!=None;Inv=Inv.Inventory)
	{
		if (Inv.InventoryGroup == NewWeaponGroup)
			return Weapon(Inv);
		Count++;
		if (Count > 1000)
			return None;
	}
}

simulated function Reset()
{
	if (Role == ROLE_Authority && (Controller == None || Health <= 0) && !bEscaped)
	{
		Destroy();
		return;
	}
	else if (bEscaped)
	{
		if (EscapedController != None)
		{
			if (Role == ROLE_Authority)
			{
				EscapedController.Possess(self);
				if (FOPlayer(EscapedController) != None)
				{
					FOPlayer(EscapedController).bBehindView = false;
					FOPlayer(EscapedController).ClientSetBehindView(false);
				}
				if (FragOpsGame(Level.Game).PreRoundTime > 0 && !EscapedController.IsInState('PreRound'))
					EscapedController.GotoState('PreRound');
			}
			if (Level.NetMode != NM_DedicatedServer)
				FOPlayer(EscapedController).NewSwitchToBestWeapon();
		}
		else
		{
			Destroy();
			return;
		}
		SetPhysics(PHYS_Walking);
	}
	else
		Velocity = vect(0,0,0);
	bPoisoned = false;
	PoisonController = None;
	bHidden = false;
	EscapedController = None;
	CarriedObjective = None;
	War_CarriedObjective = None;
	bEscaped = false;
	bInjuredLeg = false;
}

simulated function Setup(xUtil.PlayerRecord rec, optional bool bLoadNow)
{
}

function bool CanGrabLadder()
{
	return (bCanClimbLadders && (Controller != None) && (Physics != PHYS_Flying) && (Level.TimeSeconds - LastLadderClimbTime > 1.0));
}

function ClimbLadder(LadderVolume L)
{
	OnLadder = L;
	SetRotation(OnLadder.WallDir);
	SetPhysics(PHYS_Flying);
	if (IsHumanControlled())
		Controller.GotoState('PlayerClimbing');
}

function EndClimbLadder(LadderVolume OldLadder)
{
	if (Controller != None)
		Controller.EndClimbLadder();
	if (Physics == PHYS_Flying)
		SetPhysics(PHYS_Falling);
	OnLadder = None;
	LastLadderClimbTime = Level.TimeSeconds;
}

function LadderLeap(eDoubleClickDir DoubleClickMove)
{
	local vector X,Y,Z, Dir, Cross;
	local rotator TurnRot;

	if (Physics != PHYS_Flying)
		return;

	TurnRot.Yaw = Rotation.Yaw;
	GetAxes(TurnRot,X,Y,Z);

	if (DoubleClickMove == DCLICK_Left)
	{
		Dir = -1 * Y;
		Cross = X;
	}
	else if (DoubleClickMove == DCLICK_Right)
	{
		Dir = Y;
		Cross = X;
	}
	else
		return;

	if (AIController(Controller) != None)
		Cross = vect(0,0,0);

	PerformDodge(DoubleClickMove, Dir,Cross);
}

function bool PerformDodge(eDoubleClickDir DoubleClickMove, vector Dir, vector Cross)
{
	local float VelocityZ;
	local name Anim;

        if (DoubleClickMove == DCLICK_Left)
            Anim = WallDodgeAnims[2];
        else if (DoubleClickMove == DCLICK_Right)
            Anim = WallDodgeAnims[3];

	if (PlayAnim(Anim, 1.0, 0.1))
		bWaitForAnim = true;

	AnimAction = Anim;
            
	VelocityZ = Velocity.Z;
	Velocity = DodgeSpeedFactor*GroundSpeed*Dir + (Velocity Dot Cross)*Cross;

	if (!bCanDodgeDoubleJump)
		MultiJumpRemaining = 0;
	if (bCanBoostDodge || (Velocity.Z < -100))
		Velocity.Z = VelocityZ + DodgeSpeedZ;
	else
		Velocity.Z = DodgeSpeedZ;

	CurrentDir = DoubleClickMove;
	SetPhysics(PHYS_Falling);
	OnLadder = None;
	PlaySound(GetSound(EST_Jump), SLOT_Pain, GruntVolume,,50);

	return true;
}

function PlayTeleportEffect(bool bOut, bool bSound) {}

function Died(Controller Killer, class<DamageType> damageType, vector HitLocation)
{
	local Vector TossVel;

	if (bDeleteMe || Level.bLevelChange)
		return; // already destroyed, or level is being cleaned up

	// mutator hook to prevent deaths
	// WARNING - don't prevent bot suicides - they suicide when really needed
	if (Level.Game != None && Level.Game.PreventDeath(self, Killer, damageType, HitLocation))
	{
		Health = max(Health, 1); //mutator should set this higher
		return;
	}
	Health = Min(0, Health);

	if (Weapon != None)
	{
		if (Controller != None)
			Controller.LastPawnWeapon = Weapon.Class;
		Weapon.HolderDied();
		TossVel = Vector(GetViewRotation());
		TossVel = TossVel * ((Velocity Dot TossVel) + 100) + Vect(0,0,200);
		TossWeapon(TossVel);
	}

	if (DrivenVehicle != None)
	{
		Velocity = DrivenVehicle.Velocity;
		bReplicateMovement = true;
		DrivenVehicle.DriverDied();
	}

	if (CriticalReplicationInfo != None)
		CriticalReplicationInfo.bLAWCarrier = false;

	if (CarriedObjective != None)
		DropObjective();

	if (War_CarriedObjective != None)
		DropWarObjective();

	if (Controller != None)
	{
		Controller.WasKilledBy(Killer);
		if (Level.Game != None)
			Level.Game.Killed(Killer, Controller, self, damageType);
	}
	else if (Level.Game != None)
		Level.Game.Killed(Killer, Controller(Owner), self, damageType);

	DrivenVehicle = None;

	if (Killer != None)
		TriggerEvent(Event, self, Killer.Pawn);
	else
		TriggerEvent(Event, self, None);

	// reset head rotation
	SetBoneRotation('Bip01 Head',rot(0,0,0));

	Velocity.Z *= 1.3;
	if (PlayerController(Controller) != None)
		PlayerController(Controller).ForceDeathUpdate();
	PlayDying(DamageType, HitLocation);
	if (Level.Game != None && Level.Game.bGameEnded)
		return;
	if (!bPhysicsAnimUpdate && !IsLocallyControlled())
		ClientDying(DamageType, HitLocation);
}

simulated function ChunkUp(Rotator HitRotation, float ChunkPerterbation) // gam
{
	if ((Level.NetMode != NM_Client) && (Controller != None))
	{
		OldController = Controller;
		if (Controller.bIsPlayer)
			Controller.PawnDied(self);
		else
			Controller.Destroy();
	}

	bTearOff = true;
	HitDamageType = class'DamType_Gibbed'; // make sure clients gib also
	GotoState('TimingOut','Begin');
	if (Level.NetMode == NM_DedicatedServer)
		return;
	if (class'GameInfo'.static.UseLowGore())
		return;
	SpawnGibs(HitRotation, ChunkPerterbation);
}

simulated state TimingOut
{
ignores BaseChange, Landed, AnimEnd, Trigger, Bump, HitWall, HeadVolumeChange, PhysicsVolumeChange, Falling, BreathTimer;

	simulated function TakeDamage(int Damage, Pawn instigatedBy, Vector hitlocation, Vector momentum, class<DamageType> damageType)
	{
	}

	event EncroachedBy(actor Other)
	{
	}

	simulated function BeginState()
	{
		bHidden = true;
		SetPhysics(PHYS_None);
		SetCollision(false,false,false);
		if (Controller != None)
		{
			OldController = Controller;
			if (Controller.bIsPlayer)
				Controller.PawnDied(self);
			else
				Controller.Destroy();
		}
	}

Begin:
	Sleep(5.0);
	if (OldController != None && FOPlayer(OldController) != None)
		FOPlayer(OldController).ServerViewNextPlayer();
	Destroy();
}

simulated function PlayDying(class<DamageType> DamageType, vector HitLoc)
{
	local vector shotDir, hitLocRel, deathAngVel, shotStrength;
	local float maxDim, HitBoneDist;
	local string RagSkelName;
	local KarmaParamsSkel skelParams;
	local name HitBone;
	local bool PlayersRagdoll;
	local PlayerController pc;


	AmbientSound = None;
	bCanTeleport = false; // sjs - fix karma going crazy when corpses land on teleporters
	bReplicateMovement = false;
	bTearOff = true;
	bPlayedDeath = true;
	TimeTillDeRes = Level.TimeSeconds + 5.0;
	bDeres = false;

	HitDamageType = DamageType; // these are replicated to other clients
	TakeHitLocation = HitLoc;

	if ((DamageType != None) && DamageType.default.bSkeletize && (SkeletonMesh != None))
	{
		if (!bSkeletized)
		{
			LinkMesh(SkeletonMesh, false);
			Skins[0] = SkeletonTexture;
			bSkeletized = true;
			RemoveHelmet(rotator(vect(0,0,1)));
			LastBone = 'spine';
		}
	}
	else if ((DamageType != None) && DamageType.default.bAlwaysGibs)
	{
		ChunkUp(Rotation, DamageType.default.GibPerterbation);
		FireState = FS_None;
		return;
	}

	// if this is war, bodies should disappear after a while
	if (bWarGame && bTeamDeathMatch)
		LifeSpan = 10;
	else if (bWarGame)
		LifeSpan = 60;
	else
		LifeSpan = 250;

	// do fun stuff with the attached helmet
	if (Level.NetMode != NM_DedicatedServer && LastBone == 'headshot')
		RemoveHelmet(rotator(Normal(TearOffMomentum)));

	// set deathtime
	TimeOfDeath = Level.TimeSeconds;
	GotoState('Dying');

	if (Level.NetMode != NM_DedicatedServer)
	{
		// Is this the local player's ragdoll?
		if (OldController != None)
			pc = PlayerController(OldController);
		if( pc != None && pc.ViewTarget == self )
			PlayersRagdoll = true;

		// In low physics detail, if we were not just controlling this pawn,
		// and it has not been rendered in 3 seconds, just destroy it.
		if(Level.PhysicsDetailLevel == PDL_Low && !PlayersRagdoll && (Level.TimeSeconds - LastRenderTime > 3) )
		{
			Destroy();
			return;
		}

		// Try and obtain a rag-doll setup. Use optional 'override' one out of player record first, then use the species one.
		RagSkelName = Default.RagdollType;

		// If we managed to find a name, try and make a rag-doll slot availbale.
		if( RagSkelName != "" )
		{
			KMakeRagdollAvailable();
		}
		if(KIsRagdollAvailable() && RagSkelName != "")
		{
			skelParams = KarmaParamsSkel(KParams);

			// getting sim errors with skeletons?
			if (bSkeletized)
				skelParams.bDestroyOnSimError = false;

			skelParams.KSkeleton = RagSkelName;
			StopAnimating(true);
			if(DamageType != None && DamageType.default.bKUseOwnDeathVel)
			{
				if ((DamageType.Name == 'DamType_DSR' || DamageType.Name == 'DamType_TRG') && (LastBone == 'rshoulder' || LastBone == 'lshoulder'))
				{
					RagDeathVel = 240;
					RagDeathUpKick = 25;
				}
				else if (DamageType.Name == 'DamType_MP5SR')
				{
					if (LastBone == 'headshot')
					{
						RagDeathVel = Rand(20);
						RagDeathUpKick = Rand(20);
					}
					else if (LastBone == 'rshoulder' || LastBone == 'lshoulder')
					{
						RagDeathVel = max(40, Rand(100));
						RagDeathUpKick = Rand(11);
					}
					else
					{
						RagDeathVel = max(15, Rand(75));
						RagDeathUpKick = max(5, Rand(20));
					}
				}
				else if (DamageType.Name == 'DamType_RedLabel')
				{
					CalcHitLoc(TakeHitLocation, vect(0,0,0), HitBone, HitBoneDist);
					if (HitBone == 'headshot' && fRand() < 0.6)
					{
						RagDeathVel = 0;
						RagDeathUpKick = 0;
					}
					else
					{
						RagDeathVel = max(120, Rand(DamageType.default.KDeathVel));
						RagDeathUpKick = max(40, Rand(DamageType.default.KDeathUpKick));
					}
				}
				else
				{
					RagDeathVel = (DamageType.default.KDeathVel*0.6) + Rand(DamageType.default.KDeathVel/2);
					RagDeathUpKick = (DamageType.default.KDeathUpKick*0.6) + Rand(DamageType.default.KDeathUpKick/2);
				}
			}

			// scale up velocity and upkick a little
			RagDeathVel *= 1.35;
			RagDeathUpKick *= 1.16;

			// Set the dude moving in direction he was shot in general
			shotDir = Normal(TearOffMomentum);
			shotStrength = RagDeathVel * shotDir;

			// Calculate angular velocity to impart, based on shot location.
			hitLocRel = TakeHitLocation - Location;

			// spin adjustments for sniper rifles
			if (DamageType != None && (DamageType.Name == 'DamType_DSR' || DamageType.Name == 'DamType_TRG'))
			{
				if (LastBone == 'rshoulder' || LastBone == 'lshoulder')
				{
					hitLocRel.X *= (RagSpinScale*14.0);
					hitLocRel.Y *= (RagSpinScale*14.0);
				}
				else
				{
					hitLocRel.X *= RagSpinScale*1.5;
					hitLocRel.Y *= RagSpinScale*1.5;
				}
			}
			else if (DamageType != None && (DamageType.Name == 'DamType_MP5SR') && (LastBone == 'rshoulder' || LastBone == 'lshoulder'))
			{
				hitLocRel.X *= RagSpinScale*7.0;
				hitLocRel.Y *= RagSpinScale*7.0;
			}
			else
			{
				hitLocRel.X *= RagSpinScale;
				hitLocRel.Y *= RagSpinScale;
			}


			// If the tear off momentum was very small for some reason, make up some angular velocity for the pawn
			if (VSize(TearOffMomentum) < 0.01)
				deathAngVel = VRand() * 18000.0;
			else
				deathAngVel = RagInvInertia * (hitLocRel Cross shotStrength);

			// Set initial angular and linear velocity for ragdoll.
			// Scale horizontal velocity for characters - they run really fast!
			skelParams.KStartLinVel.X = 0.6 * Velocity.X;
			skelParams.KStartLinVel.Y = 0.6 * Velocity.Y;
			skelParams.KStartLinVel.Z = Velocity.Z;
			skelParams.KStartLinVel += shotStrength;

			// If not moving downwards - give extra upward kick
			if(Velocity.Z > -10)
				skelParams.KStartLinVel.Z += RagDeathUpKick;

			skelParams.KStartAngVel = deathAngVel;

			// Set up deferred shot-bone impulse
			maxDim = Max(CollisionRadius, CollisionHeight);

			skelParams.KShotStart = TakeHitLocation - (1 * shotDir);
			skelParams.KShotEnd = TakeHitLocation + (2*maxDim*shotDir);
			skelParams.KShotStrength = RagShootStrength;

			// Turn on Karma collision for ragdoll.
			KSetBlockKarma(true);
			SetPhysics(PHYS_KarmaRagdoll);

			// If viewing this ragdoll, set the flag to indicate that it is 'important'
			if( PlayersRagdoll )
				skelParams.bKImportantRagdoll = true;

			return;
		}
	}

	// non-ragdoll death fallback
	Velocity += TearOffMomentum;
	BaseEyeHeight = Default.BaseEyeHeight;
	SetTwistLook(0, 0);
	SetInvisibility(0.0);
	PlayDirectionalDeath(HitLoc);
	SetPhysics(PHYS_Falling);
}

// remove helmet
simulated function RemoveHelmet(rotator BulletDir)
{
	local int i;
	local FragOps_ArmorPiece HelmetPiece;
	
	if (MiscItem ~= "LIT. Armor")
		return;

	for (i=0;i<ArmorPieces.Length;i++)
	{
		if (ArmorPieces[i].bHelmet)
		{
			HelmetPiece = ArmorPieces[i];
			break;
		}
	}

	if (HelmetPiece == None)
		return;

	HelmetPiece.BreakOff(BulletDir);
}

function CalcHitLoc(Vector hitLoc, Vector hitRay, out Name boneName, out float dist)
{
	local name HitBone;

	//Find the closest bone.
	HitBone = GetClosestBone(hitLoc, hitRay, dist, 'headshot', 12.0);
	if (HitBone == 'headshot')
		boneName = HitBone;
	else
		boneName = 'none';
}

event EncroachedBy(actor Other)
{
	//if (Pawn(Other) != None)
		//gibbedBy(Other);
}

function TakeFallingDamage()
{
	local float Shake, EffectiveSpeed;
	local vector TempVel;

	TempVel = Velocity;
	TempVel.X *= 0.5;
	TempVel.Y *= 0.5;
	EffectiveSpeed = VSize(TempVel);
	if (EffectiveSpeed > FallSpeedThreshold)
	{
		if (Role == ROLE_Authority)
		{
			MakeNoise(1.0);
			if (TouchingWaterVolume())
				EffectiveSpeed *= 0.8;
			EffectiveSpeed = FClamp((EffectiveSpeed-FallSpeedThreshold)/(FallSpeedThreshold*1.2), 0, 1.0);
			if (EffectiveSpeed >= 0.1)
			{
				if (FRand() < 0.5)
					LastBone = 'lfoot';
				else
					LastBone = 'rfoot';

				if (EffectiveSpeed*200.0 >= 20.0)
					TakeDamage(EffectiveSpeed*200.0, None, Location, vect(0,0,0), class'Fell');
			}
		}
		if (Controller != None)
		{
			Shake = FMin(1, -1 * Velocity.Z/FallSpeedThreshold);
			Controller.DamageShake(Shake);
		}
	}
	else if (Velocity.Z < -1.4 * JumpZ)
		MakeNoise(0.5);
}

simulated function bool IsControlledByMe()
{
	if (Controller == None)
		return false;
	if (PlayerController(Controller) == None || (PlayerController(Controller) != None) && (PlayerController(Controller).bBehindView))
		return false;

	return (Viewport(PlayerController(Controller).Player) != None);
}

State Dying
{
	simulated function AnimEnd(int Channel)
	{
		ReduceCylinder();
	}

	event FellOutOfWorld(eKillZType KillType)
	{
		local LavaDeath LD;

		// If we fall past a lava killz while dead- burn off skin.
		if (KillType == KILLZ_Lava)
		{
			if (!bSkeletized && (SkeletonMesh != None))
			{
				LinkMesh(SkeletonMesh, true);
				Skins.Length = 0;
				bSkeletized = true;

				LD = spawn(class'LavaDeath', , , Location + vect(0, 0, 10), Rotation );
				if (LD != None)
					LD.SetBase(self);
				// This should destroy itself once its finished.

				PlaySound( sound'WeaponSounds.BExplosion5', SLOT_None, 1.5*TransientSoundVolume );
			}

			return;
		}

		Super.FellOutOfWorld(KillType);
	}

	function LandThump()
	{
		// animation notify - play sound if actually landed, and animation also shows it
		if (Physics == PHYS_None)
		{
			bThumped = true;
			PlaySound(GetSound(EST_CorpseLanded));
		}
	}

	simulated function TakeDamage(int Damage, Pawn InstigatedBy, Vector Hitlocation, Vector Momentum, class<DamageType> damageType)
	{
		local vector shotDir;
		local vector PushLinVel, PushAngVel;
		
		// not on server, ever
		if (Level.NetMode == NM_DedicatedServer)
			return;

		if (Momentum == vect(0,0,0) && InstigatedBy != None)
			Momentum = (HitLocation-InstigatedBy.Location);
		
		PlayHit(Damage, InstigatedBy, hitLocation, damageType, Momentum);

		// turn karma back on if needed
		if (Physics == PHYS_None)
			ResetKarma();

		if (bPlayedDeath && Physics == PHYS_KarmaRagdoll)
		{
			if (damageType.Name == 'DamType_HEGrenade')
			{
				shotDir = Normal(Momentum*3);
				PushLinVel = (RagDeathVel * shotDir) +  vect(0, 0, 400);
				PushAngVel = Normal(shotDir Cross vect(0, 0, 1)) * -18000;
				KSetSkelVel(PushLinVel, PushAngVel);
			}
			else if (damageType.Name == 'DamType_RedLabel')
			{
				shotDir = Normal(Momentum);
				PushLinVel = RagShootStrength*Normal(Momentum);
				PushAngVel = Normal(shotDir Cross vect(-1, 0, 1)) * -18000;
				KAddImpulse(PushLinVel, PushAngVel);
			}
			else if (damageType.Name == 'DamType_DSR' || damageType.Name == 'DamType_TRG')
			{
				shotDir = Normal(Momentum);
				PushLinVel = (RagShootStrength*2)*Normal(Momentum);
				PushAngVel = Normal(shotDir Cross vect(-5, 0, 1)) * -18000;
				KAddImpulse(PushLinVel, PushAngVel);

			}
			else
			{
				PushLinVel = RagShootStrength*Normal(Momentum);
				KAddImpulse(PushLinVel, HitLocation);
			}
			return;
		}
	}

	simulated function BeginState()
	{
		Super(UnrealPawn).BeginState();
		AmbientSound = None;
		SetTimer(0.0, false);
	}

	simulated function Timer()
	{
		if (bDeRes)
			return;

		// hackish
		if ((Location.X < OldRagLocation.X - 10 || Location.X > OldRagLocation.X + 10) || (Location.Y < OldRagLocation.Y - 10 || Location.Y > OldRagLocation.Y + 10) || (Location.Z < OldRagLocation.Z - 10 || Location.Z > OldRagLocation.Z + 10))
		{
			OldRagLocation = Location;
			SetTimer(5.0, false);
			return;
		}
		StartDeRes();
	}

	// set the the initial timer
	event KVelDropBelow()
	{
		if (bDeRes)
			return;

		OldRagLocation = Location;
		SetTimer(5.0, false);
	}
}

event KImpact(actor other, vector pos, vector impactVel, vector impactNorm)
{
	local int numSounds, soundNum;

	numSounds = RagImpactSounds.Length;

	if(numSounds > 0 && Level.TimeSeconds > RagLastSoundTime + RagImpactSoundInterval)
	{
		soundNum = Rand(numSounds);
		PlaySound(RagImpactSounds[soundNum], SLOT_Pain, RagImpactVolume,true,50);
		RagLastSoundTime = Level.TimeSeconds;
	}
}

// reset karma
function ResetKarma()
{
	bDeRes = false;
	SetCollisionSize(Default.CollisionRadius, Default.CollisionHeight);
	LODBias = Default.LODBias;
	CullDistance = 3000.0;
	KSetBlockKarma(true);
	SetPhysics(PHYS_KarmaRagdoll);
}

function PlayDyingSound()
{
	if (Level.NetMode == NM_DedicatedServer || bSkeletized)
		return;

	if (bGibbed)
	{
		PlaySound(GibGroupClass.static.GibSound(), SLOT_Pain,3.4*TransientSoundVolume,true,200);
		return;
	}
	else if (HeadVolume.bWaterVolume)
	{
		PlaySound(GetSound(EST_Drown), SLOT_Pain,2.3*TransientSoundVolume,true,300);
		return;
	}
	else
		PlaySound(SoundGroupClass.static.GetDeathSound(), SLOT_Pain,2.3*TransientSoundVolume,true,300,0.97+(FRand()*0.06));
}

event Landed(vector HitNormal)
{
	ImpactVelocity = vect(0,0,0);
	TakeFallingDamage();
	if (Health > 0)
		PlayLanded(Velocity.Z);
	if ((Velocity.Z < -200) && (PlayerController(Controller) != None))
		bJustLanded = true;
	LastHitBy = None;

	// only increase airspeed after landing when enabling sprint
	if (Role == ROLE_Authority && NextAirSpeed > 0)
	{
		AirSpeed = NextAirSpeed;
		NextAirSpeed = 0;
	}
	if (Health > 0)
		PlayOwnedSound(GetSound(EST_Land), SLOT_Interact, FMin(1,-0.3 * Velocity.Z/JumpZ));
}

simulated event SetAnimAction(name NewAction)
{
	if (!bWaitForAnim && NewAction != '' && HasAnim(NewAction))
	{
		bPlayingAction = true;
		AnimAction = NewAction;
		if (AnimAction == 'vehicle_reload')
		{
			AnimBlendParams(1, 1.0, 0.0, 0.2, FireRootBone);
			PlayAnim('Weapon_Switch',0.6, 0.0, 1);
		}
		if (AnimAction == 'Weapon_Switch' || AnimAction == 'nadeprime')
		{
			AnimBlendParams(1, 1.0, 0.0, 0.2, FireRootBone);
			PlayAnim(NewAction,, 0.0, 1);
		}
		else if (Physics == PHYS_None && (AnimAction == 'WalkL' || AnimAction == 'WalkR') && GetAnimSequence() != AnimAction)
		{
			PlayAnim(NewAction, 1.1, 0.05);
			AnimBlendParams(1, 1.0, 0.0, 0.2, FireRootBone);
			LoopAnim(DrivenVehicle.DriveAnim,, 0.08, 1);
		}
		else if ((Physics == PHYS_None) || ((Level.Game != None) && Level.Game.IsInState('MatchOver')))
		{
			PlayAnim(AnimAction,,0.1);
			AnimBlendToAlpha(1,0.0,0.05);
		}
		else if ((Physics == PHYS_Falling) || ((Physics == PHYS_Walking) && (Velocity.Z != 0)))
		{
			if (CheckTauntValid(AnimAction) && GetAnimSeq() != FireAnim)
			{
				AnimBlendParams(1, 1.0, 0.0, 0.2, FireRootBone);
				PlayAnim(NewAction,, 0.1, 1);
			}
			else if (PlayAnim(AnimAction))
			{
				if (Physics != PHYS_None)
					bWaitForAnim = true;
			}
			else
				AnimAction = '';
		}
		else if (bIsIdle && !bIsCrouched && (Bot(Controller) == None) && GetAnimSeq() != FireAnim) // standing taunt
		{
			PlayAnim(AnimAction,,0.1);
			AnimBlendToAlpha(1,0.0,0.05);
		}
		else // running taunt
		{
			if (GetAnimSeq() != FireAnim)
			{
				AnimBlendParams(1, 1.0, 0.0, 0.2, FireRootBone);
				PlayAnim(NewAction,, 0.1, 1);
			}
		}
	}
}

// we need to modify weapon anims for different kinds
simulated function SetWeaponAttachment(xWeaponAttachment NewAtt)
{
	local rotator RelativeRot;
	
	if (SpectatorWeapon != None)
		SpectatorWeapon.Destroy();
	
	WeaponAttachment = NewAtt;

	SetBoneRotation('Bip01 L Forearm',rot(0,0,0));
	SetBoneRotation('Bip01 L Hand',rot(0,0,0));

	if (Attachment_Base(WeaponAttachment).bSidearm)
	{
		if (Attachment_Base(WeaponAttachment).bAkimbo)
			IdleWeaponAnim = IdleAkimboAnim;
		else
		{
			RelativeRot.Yaw = 1000;
			RelativeRot.Pitch = -1000;
			SetBoneRotation('Bip01 L Forearm',RelativeRot);
			IdleWeaponAnim = IdleSidearmAnim;
		}
	}
	else if (Attachment_Base(WeaponAttachment).bGrenade)
		IdleWeaponAnim = IdleGrenadeAnim;
	else if (Attachment_Base(WeaponAttachment).bMelee || Attachment_Base(WeaponAttachment).bFists)
		IdleWeaponAnim = IdleMeleeAnim;
	else if (Attachment_Base(WeaponAttachment).bHeavy)
		IdleWeaponAnim = IdleHeavyAnim;
	else
	{
		SetBoneRotation('Bip01 L Forearm',Attachment_Base(WeaponAttachment).ArmRotation);
		IdleWeaponAnim = IdleRifleAnim;
	}

	IdleRestAnim = IdleWeaponAnim;
	IdleTime = Level.TimeSeconds;
}

simulated function StartFiring(bool bHeavy, bool bRapid)
{
	local Attachment_Base FireAttach;

	FireAttach = Attachment_Base(WeaponAttachment);

	if (Physics == PHYS_Swimming || FireAttach == None)
		return;

	if (FireAttach.bSidearm)
	{
		if (FireAttach.bAkimbo || FireAttach.bIsAkimbo)
		{
			//if (HasAnim(FireAkimboAnim) && bSwungLast)
			//{
				//bSwungLast = false;
				//FireAnim = FireAkimboAnim;
			//}
			//else if (HasAnim(FireAkimboAltAnim) && !bSwungLast)
			//{
				//bSwungLast = true;
				//FireAnim = FireAkimboAltAnim;
			//}
			//else
				return;
		}
		else if (bRapid)
			FireAnim = FireSidearmAnim;
		else
			FireAnim = FireSidearmHeavyAnim;
	}
	else if (FireAttach.bGrenade)
		FireAnim = FireGrenadeAnim;
	else if (FireAttach.bFists)
	{
		if (FRand() < 0.5)
			FireAnim = FireMeleeAnim;
		else
			FireAnim = FireMeleeAltAnim;
	}
	else if (FireAttach.bMelee)
	{
		if (bSwungLast || !FireAttach.bAkimbo)
		{
			FireAnim = FireMeleeAnim;
			bSwungLast = false;
		}
		else
		{
			FireAnim = FireMeleeAltAnim;
			bSwungLast = true;
		}
	}
	else if (FireAttach.bHeavy)
		FireAnim = FireHeavyBurstAnim;
	else if (bRapid)
		FireAnim = FireRifleRapidAnim;
	else
		FireAnim = FireRifleBurstAnim;

	AnimBlendParams(1, 1.0, 0.0, 1.0, FireRootBone);
	PlayAnim(FireAnim,, 0.0, 1);
	FireState = FS_PlayOnce;

	IdleTime = Level.TimeSeconds;
}

simulated function name GetAnimSeq()
{
	local name anim;
	local float frame, rate;

	GetAnimParams(1, anim, frame, rate);
	return anim;
}

function Heal()
{
	if (bIsHealing && Level.TimeSeconds - LastHealTime >= 0.09 && HealthRestored < HealthRestoreMax && Health < 100)
	{
		Health++;
		HealthRestored++;
	}
	else
	{
		bIsHealing = false;
		HealthRestored = 0;
	}
}

// spawn an armor piece
simulated function FragOps_ArmorPiece SpawnArmorPieces(int ArmorType)
{
	local FragOps_ArmorPiece Piece;

	switch(ArmorType)
	{
		case 0:
			Piece = spawn(Helmets[0].ArmorActor,self);
			Piece.SetStaticMesh(Helmets[0].ArmorMesh);
			AttachToBone(Piece, Helmets[0].ArmorBone);
			Piece.SetRelativeLocation(Helmets[0].ArmorOffset);
			Piece.SetRelativeRotation(Helmets[0].ArmorRotation);
			break;

		case 1:
			Piece = spawn(Helmets[1].ArmorActor,self);
			Piece.SetStaticMesh(Helmets[1].ArmorMesh);
			AttachToBone(Piece, Helmets[1].ArmorBone);
			Piece.SetRelativeLocation(Helmets[1].ArmorOffset);
			Piece.SetRelativeRotation(Helmets[1].ArmorRotation);
			break;	

		case 2:
			Piece = spawn(ShoulderArmor[0].ArmorActor,self);
			Piece.SetStaticMesh(ShoulderArmor[0].ArmorMesh);
			AttachToBone(Piece, ShoulderArmor[0].ArmorBone);
			Piece.SetRelativeLocation(ShoulderArmor[0].ArmorOffset);
			Piece.SetRelativeRotation(ShoulderArmor[0].ArmorRotation);
			break;

		case 3:
			Piece = spawn(ShoulderArmor[1].ArmorActor,self);
			Piece.SetStaticMesh(ShoulderArmor[1].ArmorMesh);
			AttachToBone(Piece, ShoulderArmor[1].ArmorBone);
			Piece.SetRelativeLocation(ShoulderArmor[1].ArmorOffset);
			Piece.SetRelativeRotation(ShoulderArmor[1].ArmorRotation);
			break;

		case 4:
			Piece = spawn(LegArmor[0].ArmorActor,self);
			Piece.SetStaticMesh(LegArmor[0].ArmorMesh);
			AttachToBone(Piece, LegArmor[0].ArmorBone);
			Piece.SetRelativeLocation(LegArmor[0].ArmorOffset);
			Piece.SetRelativeRotation(LegArmor[0].ArmorRotation);
			break;

		case 5:
			Piece = spawn(LegArmor[1].ArmorActor,self);
			Piece.SetStaticMesh(LegArmor[1].ArmorMesh);
			AttachToBone(Piece, LegArmor[1].ArmorBone);
			Piece.SetRelativeLocation(LegArmor[1].ArmorOffset);
			Piece.SetRelativeRotation(LegArmor[1].ArmorRotation);
			break;

		default:
			break;
	}
	if (Piece != None)
		Piece.Instigator = self;
	return Piece;
}

// update attached armor pieces
simulated function UpdateArmor()
{
	local int i, HelmetPiece, BP;
	local array<int> BadPieces;
	local bool bGotShoulder, bGotLeg;
	
	HelmetPiece = 255;

	// give light armor
	if (MiscItem ~= "LIT. Armor")
	{
		// destroy armor pieces
		for (i=0;i<ArmorPieces.Length;i++)
		{
			if (ArmorPieces[i] != None)
				ArmorPieces[i].Destroy();
		}
		ArmorPieces.Remove(0,ArmorPieces.Length);
		return;
	}

	// give medium armor
	if (MiscItem ~= "MED. Armor")
	{
		// see if we have armor
		for (i=0;i<ArmorPieces.Length;i++)
		{
			if (ArmorPieces[i] == None)
			{
				BadPieces[BP] = i;
				BP++;
				continue;
			}
			if (ArmorPieces[i].bShoulder)
			{
				bGotShoulder = true;
				continue;
			}

			if (ArmorPieces[i].bHelmet)
			{
				HelmetPiece = i;
				continue;
			}

			ArmorPieces[i].Destroy();
			BadPieces[BP] = i;
			BP++;
		}

		// spawn missing pieces
		if (!bGotShoulder)
		{
			ArmorPieces[ArmorPieces.Length] = SpawnArmorPieces(2);
			ArmorPieces[ArmorPieces.Length] = SpawnArmorPieces(3);
		}
		if (HelmetPiece == 255)
			ArmorPieces[ArmorPieces.Length] = SpawnArmorPieces(0);
		else if (ArmorPieces[HelmetPiece].bHeavy)
		{
			ArmorPieces[HelmetPiece].Destroy();
			ArmorPieces[HelmetPiece] = SpawnArmorPieces(0);
		}

		// remove the unauthorized pieces
		for (i=0;i<BadPieces.Length;i++)
			ArmorPieces.Remove(BadPieces[i],1);
		return;
	}

	// reset
	BP = 0;
	BadPieces.Remove(0,BadPieces.Length);
	HelmetPiece = 255;

	// give heavy armor
	if (MiscItem ~= "HVY. Armor")
	{
		// see if we have armor
		for (i=0;i<ArmorPieces.Length;i++)
		{
			if (ArmorPieces[i] == None)
			{
				BadPieces[BP] = i;
				BP++;
				continue;
			}

			if (ArmorPieces[i].bShoulder)
			{
				bGotShoulder = true;
				continue;
			}

			if (ArmorPieces[i].bLeg)
			{
				bGotLeg = true;
				continue;
			}

			if (ArmorPieces[i].bHelmet)
			{
				HelmetPiece = i;
				continue;
			}
		}

		// spawn missing pieces		
		if (!bGotShoulder)
		{
			ArmorPieces[ArmorPieces.Length] = SpawnArmorPieces(2);
			ArmorPieces[ArmorPieces.Length] = SpawnArmorPieces(3);
		}
		if (!bGotLeg)
		{
			ArmorPieces[ArmorPieces.Length] = SpawnArmorPieces(4);
			ArmorPieces[ArmorPieces.Length] = SpawnArmorPieces(5);
		}
		if (HelmetPiece == 255)
			ArmorPieces[ArmorPieces.Length] = SpawnArmorPieces(1);
		else if (!ArmorPieces[HelmetPiece].bHeavy)
		{
			ArmorPieces[HelmetPiece].Destroy();
			ArmorPieces[HelmetPiece] = SpawnArmorPieces(1);
		}
		for (i=0;i<BadPieces.Length;i++)
			ArmorPieces.Remove(BadPieces[i],1);
	}
}

simulated function Tick(float delta)
{
	local name CurAnimName, LastAirAnim, LastTakeoffAnim;
	local rotator NewRotation;

	Super.Tick(delta);

	if (Health <= 0 || bPlayedDeath || Physics == PHYS_KarmaRagdoll)
		return;

	// update attached armor
	if (Level.NetMode != NM_DedicatedServer)
		UpdateArmor();

	if (DrivenVehicle != None && bDriving)
	{
		bHadVehicle = true;
		if (War_FOVehiclePawnFree(DrivenVehicle) != None)
		{
			CurAnimName = GetAnimSeq();
			if (!bAnimChanged && !bPlayingAction && (CurAnimName == '' || CurAnimName == MovementAnims[0] || CurAnimName == MovementAnims[1] || CurAnimName == MovementAnims[2] || CurAnimName == MovementAnims[3]))
			{
				AnimBlendParams(1, 0.9, 0.0, 0.2, FireRootBone);
				LoopAnim(IdleWeaponAnim,, 0.0, 1);
			}
		}
		else if (DrivenVehicle.bDrawDriverInTP && !bPlayingAction)
		{
			if (GetAnimSequence() != DrivenVehicle.DriveAnim)
				LoopAnim(DrivenVehicle.DriveAnim);			
			if (GetAnimSeq() != DrivenVehicle.DriveAnim)
				LoopAnim(DrivenVehicle.DriveAnim,,,1);
		}
		return;
	}

	// heal now
	if (Role == ROLE_Authority)
		Heal();

	if (Physics == PHYS_None || (DrivenVehicle != None && bDriving))
		return;

	// fix animation immediately
	if (bHadVehicle)
	{
		bHadVehicle = false;
		ChangeAnimation();
	}

	// this is actually for ladder climbing
	if (Physics == PHYS_Flying)
	{
		// we don't need no stinking rotation
		NewRotation = Rotation;
		NewRotation.Pitch = 0;
		NewRotation.Roll = 0;
		SetRotation(NewRotation);
		
		// change anims
		LastAirAnim = AirStillAnim;
		LastTakeoffAnim = TakeoffStillAnim;
		if (VSize(Velocity) > 10)
		{
			AirStillAnim = LadderAnim;
			TakeoffStillAnim = LadderAnim;
			if (GetAnimSeq() != AirStillAnim)
				LoopAnim(AirStillAnim,, 0.0, 0);
		}
		else
		{
			AirStillAnim = IdleWeaponAnim;
			TakeoffStillAnim = IdleWeaponAnim;
			ChangeAnimation();
		}		

		// initialize movement anims
		AirAnims[0] = Default.WalkAnims[0];
		AirAnims[1] = Default.WalkAnims[1];
		AirAnims[2] = Default.WalkAnims[2];
		AirAnims[3] = Default.WalkAnims[3];

		TakeoffAnims[0] = Default.WalkAnims[0];
		TakeoffAnims[1] = Default.WalkAnims[1];
		TakeoffAnims[2] = Default.WalkAnims[2];
		TakeoffAnims[3] = Default.WalkAnims[3];
	}
	else if (AirStillAnim != Default.AirStillAnim)
	{

		// reset movement anims
		AirStillAnim = Default.AirStillAnim;
		TakeoffStillAnim = Default.TakeoffStillAnim;
		
		AirAnims[0] = Default.AirAnims[0];
		AirAnims[1] = Default.AirAnims[1];
		AirAnims[2] = Default.AirAnims[2];
		AirAnims[3] = Default.AirAnims[3];

		TakeoffAnims[0] = Default.TakeoffAnims[0];
		TakeoffAnims[1] = Default.TakeoffAnims[1];
		TakeoffAnims[2] = Default.TakeoffAnims[2];
		TakeoffAnims[3] = Default.TakeoffAnims[3];

		// immediately use
		if (Physics == PHYS_Falling);
			ChangeAnimation();
	}

	// save some server resources
	if (Level.NetMode == NM_DedicatedServer)
		return;

	// upper body animation enforcement
	CurAnimName = GetAnimSeq();
	if (!bAnimChanged && !bPlayingAction && (CurAnimName == '' || CurAnimName == MovementAnims[0] || CurAnimName == MovementAnims[1] || CurAnimName == MovementAnims[2] || CurAnimName == MovementAnims[3]))
	{
		AnimBlendParams(1, 0.9, 0.0, 0.12, FireRootBone);
		LoopAnim(IdleWeaponAnim,, 0.0, 1);
	}

	// and this is a crouch rotation offset, because the start/endcrouch events only run on the local client.  Bonerotations don't replicate.
	if (Controller == None)
	{
		if (bIsCrouched && !bLastCrouched)
		{
			bLastCrouched = true;
			StartCrouch(0);
		}
		else if (!bIsCrouched && bLastCrouched)
		{
			bLastCrouched = false;
			EndCrouch(0);
		}
	}
}

simulated function AnimEnd(int Channel)
{
	local bool bWasFiring;

	// nothing here
	if (Physics == PHYS_KarmaRagdoll)
		return;
	
	bAnimChanged = true;

	if (DrivenVehicle != None && War_FOVehiclePawnFree(DrivenVehicle) == None && DrivenVehicle.bDrawDriverInTP)
	{
		if (GetAnimSequence() != DrivenVehicle.DriveAnim)
			LoopAnim(DrivenVehicle.DriveAnim);			
		if (GetAnimSeq() != DrivenVehicle.DriveAnim)
			LoopAnim(DrivenVehicle.DriveAnim,,,1);
		bAnimChanged = false;
		bPlayingAction = false;
		return;
	}
	else if (DrivenVehicle != None && War_FOVehiclePawnFree(DrivenVehicle) != None && Channel == 0)
		LoopAnim(DrivenVehicle.DriveAnim,,0.08,0);

	if (Health <= 0 || Physics == PHYS_KarmaRagdoll || bTearOff)
	{
		bAnimChanged = false;
		return;
	}

	if (Channel == 1)
	{
		if (FireState == FS_Ready)
		{
			FireState = FS_None;
		}
		else if (FireState == FS_PlayOnce)
		{
			FireState = FS_Ready;
			IdleTime = Level.TimeSeconds;
			bWasFiring = true;
		}

		// smooth certain anims
		if (bPlayingAction || bWasFiring)
		{
			bPlayingAction = false;
			LoopAnim(IdleWeaponAnim,, 0.12, 1);
		}
		else
		{
			AnimBlendParams(1, 1.0, 0.0, 0.1, FireRootBone);
			LoopAnim(IdleWeaponAnim,, 0.0, 1);
			bPlayingAction = false;
		}
	}
	else if (bKeepTaunting && (Channel == 0))
		PlayVictoryAnimation();

	bAnimChanged = false;
}

function PlayVictoryAnimation()
{
	SetAnimAction(TauntAnims[3]);
}

function TakeFlash(float FlashAmount, vector FlashLocation)
{
	local Controller TempController;
	
	if (DrivenVehicle == None && Controller != None && IsHumanControlled())
		TempController = Controller;
	else if (DrivenVehicle != None && DrivenVehicle.Controller != None && DrivenVehicle.IsHumanControlled())
		TempController = DrivenVehicle.Controller;

	if (TempController == None)
		return;

	FOPlayer(TempController).ClientTakeFlash(FlashAmount, FlashLocation);
}

function TakeDamage(int Damage, Pawn instigatedBy, Vector hitlocation, Vector Momentum, class<DamageType> DamageType)
{
	local int actualDamage;
	local bool bAlreadyDead;
	local Controller Killer;

	if (DrivenVehicle != None && DrivenVehicle.Health > 0 && !DrivenVehicle.bDriverCollideActors)
		return;

	if (bRestrictDamage || (Controller != None && Controller.IsInState('PreRound')) || Damage <= 0)
		return;

	if (DamageType == None)
	{
		if (InstigatedBy != None)
			warn("No damagetype for damage by "$instigatedby$" with weapon "$InstigatedBy.Weapon);
		DamageType = class'DamageType';
	}

	if (Role < ROLE_Authority)
	{
		log(self$" client damage type "$damageType$" by "$instigatedBy);
		return;
	}

	bAlreadyDead = ((Health <= 0) && (Controller == None || (PlayerReplicationInfo != None && PlayerReplicationInfo.bOutOfLives) || bPlayedDeath));

	if ((Physics == PHYS_None) && (DrivenVehicle == None))
		SetMovementPhysics();
	if (Physics == PHYS_Walking)
		Momentum.Z = FMax(Momentum.Z, 0.4 * VSize(Momentum));
	if (InstigatedBy == self)
		Momentum *= 0.6;

	if (Weapon != None)
	        Weapon.AdjustPlayerDamage(Damage, InstigatedBy, HitLocation, Momentum, DamageType);

	ActualDamage = int(ModifyArmorDamage(Damage));

	// better damage modifier
	if (Level.Game != None)
	{
		if (InstigatedBy != None)
			ActualDamage = FragOpsGame(Level.Game).ReduceDamagePlus(ActualDamage, self, InstigatedBy, InstigatedBy.Controller, HitLocation, Momentum, DamageType, LastBone);
		else if ((DamageType != None) && DamageType.Default.bDelayedDamage)
			ActualDamage = FragOpsGame(Level.Game).ReduceDamagePlus(ActualDamage, self, InstigatedBy, DelayedDamageInstigatorController, HitLocation, Momentum, DamageType, LastBone);
		else
			ActualDamage = FragOpsGame(Level.Game).ReduceDamagePlus(ActualDamage, self, InstigatedBy, None, HitLocation, Momentum, DamageType, LastBone);
	}

	if (class<WeaponDamageType>(DamageType) != None)
		ActualDamage = int(ModifyDamage(class<WeaponDamageType>(DamageType).Default.WeaponClass, ActualDamage));

	// don't take damage at all if very small or none
	if (ActualDamage < 2)
		return;

	Health -= ActualDamage;
	NetUpdateTime = Level.TimeSeconds - 1;

	if (HitLocation == vect(0,0,0))
		HitLocation = Location;

	PlayHit(actualDamage,InstigatedBy, hitLocation, DamageType, Momentum);

	if (bAlreadyDead)
		return;

	// add a hit if we can
	if (!DamageType.Default.bSpecial && InstigatedBy != None && InstigatedBy.PlayerReplicationInfo != None && InstigatedBy.PlayerReplicationInfo.Team != None && InstigatedBy.PlayerReplicationInfo.Team != PlayerReplicationInfo.Team)
	{
		if (LastBone == 'headshot')
			FragOps_PRI(InstigatedBy.PlayerReplicationInfo).AddHeadShot();
		FragOps_PRI(InstigatedBy.PlayerReplicationInfo).AddHit();
	}

	if (Health <= 0)
	{
		// pawn died
		if (DamageType.Name == 'DamType_Poison' && PoisonController != None)
			Killer = PoisonController;
		else if (instigatedBy != None)
			Killer = instigatedBy.GetKillerController();
		else if ((DamageType != None) && DamageType.Default.bDelayedDamage)
			Killer = DelayedDamageInstigatorController;
		if (((FOPlayer(Killer) != None && FOPlayer(Killer).SelectedTrait < class'Interface_TabPlayerTeam'.Default.Traits.Length && class'Interface_TabPlayerTeam'.Default.Traits[FOPlayer(Killer).SelectedTrait].Default.TraitName ~= "Bloody Mess") || bBloodyMess) && (DamageType.Name == 'DamType_RedLabel' || DamageType.Name == 'DamType_Super90GA' || DamageType.Name == 'DamType_HEGrenade'))
			DamageType = class'DamType_Gibbed';
		if (bPhysicsAnimUpdate)
			TearOffMomentum = Momentum;
		Died(Killer, DamageType, HitLocation);
	}
	else
	{
		if (DamageType.Name == 'DamType_MachetePoison' && (instigatedBy.PlayerReplicationInfo.Team.TeamIndex != PlayerReplicationInfo.Team.TeamIndex || FragOpsGame(Level.Game).FriendlyFireScale > 0.0))
		{
			bPoisoned = true;
			InitialPoisonTime = Level.TimeSeconds;
			SetTimer(1.0, false);
			PoisonController = instigatedBy.Controller;
		}
		if ((DrivenVehicle == None || Health < 0) && VSize(Momentum) > 1)
			AddVelocity(Momentum);
		if (Controller != None)
			Controller.NotifyTakeHit(instigatedBy, HitLocation, actualDamage, DamageType, Momentum);
	}
	MakeNoise(1.0);
}

// trait modifiers
//======================================================================================================
function float ModifyDamage(class<Weapon> ModWeapon, float InDamage)
{
	if (ModWeapon == None)
		return InDamage;
	
	if (ModWeapon.name == 'Weapon_GrenadeIncinerator')
		return InDamage*NapalmMod;

	switch (class<Weapon_Base>(ModWeapon).Default.WeaponType)
	{
		case Sniper:
			return InDamage*SniperMod;

		case Melee:
			return InDamage*MeleeMod;

		case Shotgun:
			return InDamage*ShotgunMod;	

		case Grenade:
			return InDamage*GrenadeMod;

		case Sidearm:
			return InDamage*SidearmMod;
		
		case Rifle:
			return InDamage*AutoMod;

		case SMG:
			return InDamage*AutoMod;

		default:
			break;
	}
	if (ModWeapon.name == 'Weapon_MachetePoison')
		return InDamage*PoisonMod;
	
	return InDamage;
}

function float ModifyArmorDamage(float InDamage)
{
	return InDamage*ArmorMod;
}

function float ModifyNapalmDamageDone(float InDamage)
{
	return InDamage*NapalmDamageMod;
}

function float ModifyDamageDone(class<Weapon> ModWeapon, float InDamage)
{
	if (ModWeapon == None)
		return InDamage;

	switch (class<Weapon_Base>(ModWeapon).Default.WeaponType)
	{
		case Sniper:
			return InDamage*SniperDamageMod;

		case Melee:
			return InDamage*MeleeDamageMod;

		case Shotgun:
			return InDamage*ShotgunDamageMod;	

		case Grenade:
			return InDamage*GrenadeDamageMod;

		case Sidearm:
			return InDamage*SidearmDamageMod;
		
		case Rifle:
			return InDamage*AutoDamageMod;

		case SMG:
			return InDamage*AutoDamageMod;

		default:
			break;
	}
	
	return InDamage;
}

function float ModifySpeed(float InSpeed)
{
	return InSpeed*SpeedMod;
}

function float ModifyRecoil(Weapon_Base ModWeapon, float InRecoil)
{
	if (ModWeapon == None)
		return InRecoil;
	
	if (ModWeapon.IsAkimbo() && ModWeapon.Default.WeaponType != Melee)
		return InRecoil*AkimboRecoilMod;

	switch (ModWeapon.Default.WeaponType)
	{
		case Sniper:
			return InRecoil*SniperRecoilMod;

		case Shotgun:
			return InRecoil*ShotgunRecoilMod;	

		case Sidearm:
			return InRecoil*SidearmRecoilMod;
		
		case Rifle:
			return InRecoil*AutoRecoilMod;

		case SMG:
			return InRecoil*AutoRecoilMod;

		default:
			return InRecoil;
	}
}

function float ModifyThrow(float InStrength)
{
	if (FragOpsWar(Level.Game) != None)
		return InStrength*WarThrowMod*ThrowMod;
	else
		return InStrength*ThrowMod;
}

simulated function float ModifyFootSteps(float InVolume)
{
	if (FootStepMod < 0.1)
		return InVolume;
	else
		return InVolume*FootStepMod;
}

function float ModifyJump(float InHeight)
{
	return InHeight*JumpMod;
}

// end trait modifiers
//======================================================================================================

function Timer()
{
	if (bPoisoned && Level.TimeSeconds - InitialPoisonTime <= PoisonTime && Health > 0 && !bRestrictDamage && Controller != None && !Controller.IsInState('PreRound'))
	{
		if (Level.TimeSeconds - LastGruntTime > 1.75 && Health > 0)
		{
			LastGruntTime = Level.TimeSeconds;
			PlaySound(PoisonChokeSound,SLOT_None,2.0*TransientSoundVolume,,180,1.1);
		}
		TakeDamage(PoisonDamage, None, Location, vect(0,0,0), class'DamType_Poison');
		SetTimer(1.9, false);
	}
	else
		bPoisoned = false;
}

simulated function SetDefaultMesh()
{
	if (Mesh != Default.Mesh)
		LinkMesh(Default.Mesh);
}

simulated function SpawnGiblet(class<Gib> GibClass, Vector Location, Rotator Rotation, float GibPerterbation)
{
	local Gib Giblet;
	local Vector Direction, Dummy;

	if ((GibClass == None) || class'GameInfo'.static.UseLowGore())
		return;

	Instigator = self;
	Giblet = Spawn(GibClass,,, Location, Rotation);

	if (Giblet == None)
		return;

	GibPerterbation *= 32768.0;
	Rotation.Pitch += ( FRand() * 2.0 * GibPerterbation ) - GibPerterbation;
	Rotation.Yaw += ( FRand() * 2.0 * GibPerterbation ) - GibPerterbation;
	Rotation.Roll += ( FRand() * 2.0 * GibPerterbation ) - GibPerterbation;

	GetAxes(Rotation, Dummy, Dummy, Direction);

	Giblet.Velocity = Velocity + Normal(Direction) * (40.0+(FRand()*30.0));
	Giblet.LifeSpan = Giblet.LifeSpan + 2 * FRand() - 1;
}

function PlayHeadShotSound()
{
	if (Level.TimeSeconds - LastHeadShotTime > 0.05)
	{
		LastHeadShotTime = Level.TimeSeconds;
		if (MiscItem ~= "LIT. Armor")
			PlaySound(HeadCrackSound,SLOT_None,2.25*TransientSoundVolume,,440,0.99+(FRand()*0.08));
		else
			PlaySound(HeadShotSound,SLOT_None,2.1*TransientSoundVolume,,440);
	}
}

function PlayBodyHitSound()
{
	if (Health > 0 && Level.TimeSeconds - LastGruntTime > 1.75 && FRand() < 0.3)
	{
		PlaySound(BodyGruntSound,,3.2*TransientSoundVolume,,420,1.15+(FRand()/20));
		LastGruntTime = Level.TimeSeconds;
	}
	else if (Level.TimeSeconds - LastImpactTime > 0.1)
	{
		LastImpactTime = Level.TimeSeconds;
		PlaySound(BodyHitSound,,3.2*TransientSoundVolume,,420,1.1+(FRand()/6));
	}
}

simulated function StartDeRes()
{
	local KarmaParamsSkel SkelParams;

	if (bDeRes)
		return;

	bDeRes = true;

	if ((GetKillerController() != None) && (FOPlayer(GetKillerController()) != None) && FOPlayer(GetKillerController()).IsDead() && FOPlayer(GetKillerController()).bAllowSwitch)
	{
		FOPlayer(GetKillerController()).bAllowSwitch = false;
		FOPlayer(GetKillerController()).ServerViewNextPlayer();
		FOPlayer(GetKillerController()).bAllowBV = true;
	}

	// remove shadow
	if (PlayerShadow != None)
		PlayerShadow.Destroy();

	if (RealtimeShadow != None)
		RealtimeShadow.Destroy();

	// change to new viewtarget after death
	if (SpectatorWeapon != None)
	{
		SpectatorWeapon.Destroy();
		Level.GetLocalPlayerController().ServerViewNextPlayer();
	}

	if (!bBodiesStay)
	{
		Destroy();
		return;
	}

	// Remove projectors
	//Projectors.Remove(0, Projectors.Length);

	if (class'GameInfo'.Default.GoreLevel > 1 && Mesh != SkeletonMesh)
		SpawnBloodPool();

	// Remove flames
	// RemoveFlamingEffects();

	// reset karma
	SkelParams = KarmaParamsSkel(KParams);
	SkelParams.bKImportantRagdoll = false;
	KSetBlockKarma(false);
	SetPhysics(PHYS_None);
	SetCollisionSize(35.0,12.0);

	// make sure dead models are rendered at less quality
	LODBias = 0.8;

	// if far away, dont even render
	CullDistance = 2000.0;
}

simulated function SpawnBloodPool()
{
	local vector LastHitLocation, BoneLoc;

	if (NumPools >= MaxPools)
		return;

	LastHitLocation = Location;
	// spawn pool closest to last bone hit
	if (LastBone == 'headshot' || LastBone == 'rshoulder' || LastBone == 'lshoulder' || LastBone == 'spine')
	{
		BoneLoc = GetBoneCoords(LastBone).Origin;
		LastHitLocation.X = BoneLoc.X;
		LastHitLocation.Y = BoneLoc.Y;
	}

	if (BloodPool.Length > 0 && NumPools < BloodPool.Length && BloodPool[NumPools] != None)
		BloodPool[NumPools].Destroy();
	BloodPool[NumPools] = spawn(class'Effect_BloodPool',self,,LastHitLocation,rotator(vect(0,0,-1)));
	if (BloodPool.Length > 0 && NumPools < BloodPool.Length && BloodPool[NumPools] != None)
	{
		BloodPool[NumPools].Instigator = self;
		NumPools++;
	}
}

// helper function for attaching emitters
simulated function AttachEmitter(class<Emitter> EmitterClass, Name BoneName)
{
	local Actor a;
	local coords BoneCoords;
	local int i;

	if (BoneName == 'None')
	        return;

	for (i = 0; i < Attached.Length; i++)
	{
		if (Attached[i] == None)
			continue;

		if (Attached[i].AttachmentBone != BoneName)
			continue;

		if (ClassIsChildOf(EmitterClass, Attached[i].Class))
			return;
	}

	BoneCoords = GetBoneCoords(BoneName);

	a = Spawn(EmitterClass,,, BoneCoords.Origin, rotator(BoneCoords.Origin));

	if (!AttachToBone(a, BoneName))
	{
		log( "Couldn't attach "$EmitterClass$" to "$BoneName, 'Error' );
		a.Destroy();
		return;
	}

	for (i=0;i<Attached.length;i++)
	{
		if (Attached[i] == a)
			break;
	}

	a.SetRelativeRotation(rotator(BoneCoords.Origin));
}

simulated function AttachFlamingEffects()
{

	if (Level.NetMode == NM_DedicatedServer)
		return;

	if (FRand() < 0.5)
		AttachEmitter(class'Effect_PlayerFlames','Bip01 L Calf');
	else
		AttachEmitter(class'Effect_PlayerFlames','Bip01 R Calf');

	if (FRand() < 0.5)
		AttachEmitter(class'Effect_PlayerFlames','Bip01 R Thigh');
	else
		AttachEmitter(class'Effect_PlayerFlames','Bip01 L Thigh');

	if (FRand() < 0.5)
		AttachEmitter(class'Effect_PlayerFlames','Bip01 R Forearm');
	else
		AttachEmitter(class'Effect_PlayerFlames','Bip01 L Forearm');
}



simulated function RemoveFlamingEffects()
{
	local int i;

	if (Level.NetMode == NM_DedicatedServer)
		return;

	for (i=0;i<Attached.length;i++)
	{
		if (Attached[i].IsA('xEmitter') && !Attached[i].IsA('BloodJet'))
			xEmitter(Attached[i]).mRegen = false;
		else if (Attached[i].IsA('Effect_PlayerFlames'))
			Effect_PlayerFlames(Attached[i]).StopFlames();
	}
}

simulated function CheckBob(float DeltaTime, vector Y)
{
	local float OldBobTime, Speed2D;
	local int m,n;

	OldBobTime = BobTime;

	if (!bWeaponBob || bJustLanded)
	{
		BobTime = 0;
		WalkBob = Vect(0,0,0);
		return;
	}
	Bob = FClamp(Bob, -0.01, 0.01);
	if (Physics == PHYS_Walking)
	{
		Speed2D = VSize(Velocity);
		if (Speed2D < 10)
			BobTime += 0.2 * DeltaTime;
		else
			BobTime += DeltaTime * (0.3 + 0.7 * Speed2D/GroundSpeed);
		WalkBob = Y * Bob * Speed2D * sin(8 * BobTime);
		AppliedBob = AppliedBob * (1 - FMin(1, 16 * deltatime));
		WalkBob.Z = AppliedBob;
		if (Speed2D > 10)
			WalkBob.Z = WalkBob.Z + 0.75 * Bob * Speed2D * sin(16 * BobTime);
	}
	else if (Physics == PHYS_Swimming)
	{
		BobTime += DeltaTime;
		Speed2D = Sqrt(Velocity.X * Velocity.X + Velocity.Y * Velocity.Y);
		WalkBob = Y * Bob *  0.5 * Speed2D * sin(4.0 * BobTime);
		WalkBob.Z = Bob * 1.5 * Speed2D * sin(8.0 * BobTime);
	}
	else
	{
		BobTime = 0;
		WalkBob = WalkBob * (1 - FMin(1, 8 * deltatime));
	}

	if ((Physics != PHYS_Walking) || (VSize(Velocity) < 10)	|| ((PlayerController(Controller) != None) && PlayerController(Controller).bBehindView))
		return;

	m = int(0.5*Pi + 9.0*OldBobTime/Pi);
	n = int(0.5*Pi + 9.0*BobTime/Pi);

	if ((m != n) && !bIsWalking && !bIsCrouched)
		FootSteppingNew(0);
	else if (!bWeaponBob && bPlayOwnFootSteps && !bIsWalking && !bIsCrouched && (Level.TimeSeconds - LastFootStepTime > 0.35))
	{
		LastFootStepTime = Level.TimeSeconds;
		FootSteppingNew(0);
	}
}

simulated function vector WeaponBob(float BobDamping)
{
	Local Vector WBob;

	WBob = (BobDamping - 0.15) * WalkBob;
	WBob.Z = (0.45 + 0.647 * BobDamping) * WalkBob.Z;
	WBob.Z *= 1.28;
	WBob.Z += LandBob;
	return WBob*0.9;
}

function ShouldCrouch(bool Crouch)
{
	if (Controller == None)
	{
		bWantsToCrouch = false;
		return;
	}
	
	if (Controller.IsInState('StickyObjective') || Controller.IsInstate('PreRound'))
		bWantsToCrouch = false;
	else
		bWantsToCrouch = Crouch;
}

simulated function FootStepping(int Side)
{
	if (Level.GetLocalPlayerController() != None && Level.GetLocalPlayerController().ViewTarget == Self && Level.GetLocalPlayerController().Pawn != Self && !Level.GetLocalPlayerController().bBehindView)
		return;

	FootSteppingNew(Side);
}

simulated function FootSteppingNew(int Side)
{
	local int SurfaceInt, i;
	local actor A;
	local material FloorMat;
	local vector HL, HN, Start, End, HitLocation, HitNormal;
	local Actor SoundPlayer;

	SurfaceInt = 0;

	if (WeaponAttachment == None || Level.GetLocalPlayerController() == None || (Level.GetLocalPlayerController().ViewTarget == self && !Level.GetLocalPlayerController().bBehindView))
		SoundPlayer = self;
	else
		SoundPlayer = WeaponAttachment;

	for (i=0;i<Touching.Length;i++)
		if (((PhysicsVolume(Touching[i]) != None) && PhysicsVolume(Touching[i]).bWaterVolume)
		|| (FluidSurfaceInfo(Touching[i]) != None))
		{
			if (FRand() < 0.5)
				SoundPlayer.PlaySound(sound'PlayerSounds.FootStepWater2', SLOT_Interact, ModifyFootSteps(FootstepVolume),, 300,,true);
			else
				SoundPlayer.PlaySound(sound'PlayerSounds.FootStepWater1', SLOT_Interact, ModifyFootSteps(FootstepVolume),, 300,,true);

			// add water ring spawner
			if (!Level.bDropDetail && (Level.DetailMode != DM_Low) 
				&& !Touching[i].TraceThisActor(HitLocation, HitNormal,Location - CollisionHeight*vect(0,0,1.1), Location) )
					Spawn(class'WaterRing',,,HitLocation,rot(16384,0,0));			
			return;
		}

	if (bIsCrouched || bIsWalking || bInStasis)
		return;

	if ((Base != None)  && (!Base.IsA('LevelInfo')) && (Base.SurfaceType!=0))
		SurfaceInt = Base.SurfaceType;
	else
	{
		Start = Location - Vect(0,0,1)*CollisionHeight;
		End = Start - Vect(0,0,16);
		A = Trace(hl,hn,End,Start,false,,FloorMat);
		if (FloorMat != None)
			SurfaceInt = FloorMat.SurfaceType;
	}

	// update the location
	if (SoundPlayer.Mesh != None)
		SoundPlayer.BoneRefresh();
	SoundPlayer.PlaySound(SoundFootsteps[SurfaceInt], SLOT_Interact, ModifyFootSteps(FootstepVolume),,150,,false);
}

function HandleAmmoPickup(Pickup pick)
{
	MakeNoise(0.2);
	if (Controller != None && FOPlayer(Controller) != None)
		FOPlayer(Controller).HandleAmmoPickup(pick);
}

// ever seen a fully armored special ops do a front flip?  Me neither.
function DoDoubleJump( bool bUpdating )
{
}

// or walldodge for that matter :)
function vector BotDodge(Vector Dir)
{
	return vect(0,0,0);
}

function bool Dodge(eDoubleClickDir DoubleClickMove)
{
	return false;
}

simulated function PlayTakeHit(vector HitLocation, int Damage, class<DamageType> DamageType)
{
	PlayDirectionalHit(HitLocation);
}

// true directional hits, even a headshot
simulated function PlayNewDirectionalHit(vector HitLoc, vector StartLoc, name HitBone)
{
	/*local vector X,Y,Z, Dir;

	// no hit anims if firing or vehicle
	if (DrivenVehicle != None || GetAnimSeq() == FireAnim)
		return;

	GetAxes(Rotation, X,Y,Z);
	HitLoc.Z = Location.Z;
	Dir = -Normal(HitLoc - StartLoc);

	AnimBlendParams(1, 1.0, 0.0, 0.1, FireRootBone);
	if (Dir Dot X > 0.7 || Dir == vect(0,0,0))
	{
		if (HitBone == 'headshot')
			PlayAnim('Hit_Head',,0.12, 1);
		else
			PlayAnim('HitF',,0.15, 1);
	}
	else if (Dir Dot X < -0.7)
		PlayAnim('HitB',,0.12, 1);
	else if (Dir Dot Y > 0)
		PlayAnim('HitR',,0.12, 1);
	else
		PlayAnim('HitL',,0.12, 1);*/
}

simulated function PlayHit(float Damage, Pawn InstigatedBy, vector HitLocation, class<DamageType> damageType, vector Momentum)
{
	local Vector HitNormal;
	local Vector HitRay;
	local Name HitBone;
	local float HitBoneDist;
	local PlayerController PC;
	local XPawn XInstigatedBy;
	local bool bShowEffects;
	local class<Effects> DesiredEffect;
	local class<Emitter> DesiredEmitter;
	local vector BloodOffset, Mo;
	local Effect_BloodHitGrow Grow;

	if ((Damage <= 0) && ((Controller == None) || !Controller.bGodMode))
		return;

	if (Damage > DamageType.Default.DamageThreshold) //spawn some blood
	{

		HitNormal = Normal(HitLocation - Location);

		// Play any set effect
		if (EffectIsRelevant(Location,true))
		{
			DesiredEffect = DamageType.static.GetPawnDamageEffect(HitLocation, Damage, Momentum, self, (Level.bDropDetail || Level.DetailMode == DM_Low));

			if (DesiredEffect != None)
			{
				BloodOffset = 0.2 * CollisionRadius * HitNormal;
				BloodOffset.Z = BloodOffset.Z * 0.5;

				Mo = Momentum;
				if (Mo.Z > 0)
					Mo.Z *= 0.5;
				spawn(DesiredEffect,self,,HitLocation + BloodOffset, rotator(Mo));
			}

			// Spawn any preset emitter

			DesiredEmitter = DamageType.Static.GetPawnDamageEmitter(HitLocation, Damage, Momentum, self, (Level.bDropDetail || Level.DetailMode == DM_Low));
			if (DesiredEmitter != None)
				spawn(DesiredEmitter,,,HitLocation+HitNormal, Rotator(HitNormal));
		}
	}
	if (Health <= 0)
	{
		if ( PhysicsVolume.bDestructive && (PhysicsVolume.ExitActor != None) )
			Spawn(PhysicsVolume.ExitActor);
	}

	// jdf ---
	if ((Level.NetMode != NM_DedicatedServer) && (Level.NetMode != NM_ListenServer) && Health > 0)
	{
		PC = PlayerController(Controller);
		if ( PC != None && PC.bEnableDamageForceFeedback )
			PC.ClientPlayForceFeedback("Damage");
	}
	// --- jdf

	if (Level.TimeSeconds - LastPainTime > 0.1 && Health > 0)
	{
		if (InstigatedBy != None)
			PlayNewDirectionalHit(HitLocation, InstigatedBy.Location, LastBone);
		else
			PlayTakeHit(HitLocation,Damage,damageType);
		LastPainTime = Level.TimeSeconds;
	}

	PC = PlayerController(Controller);
	bShowEffects = ((Level.NetMode != NM_Standalone) || (Level.TimeSeconds - LastRenderTime < 3)
					|| ((InstigatedBy != None) && (PlayerController(InstigatedBy.Controller) != None))
					|| (PC != None));
	if (!bShowEffects)
		return;

	XInstigatedBy = xPawn(InstigatedBy);

	HitRay = vect(0,0,0);
	if (InstigatedBy != None)
		HitRay = Normal(HitLocation-(InstigatedBy.Location+(vect(0,0,1)*InstigatedBy.EyeHeight)));

	if (DamageType.default.bLocationalHit)
		CalcHitLoc(HitLocation, HitRay, HitBone, HitBoneDist);
	else
	{
		HitLocation = Location;
		HitBone = 'None';
		HitBoneDist = 0.0f;
	}

	if (InstigatedBy != None)
		HitNormal = Normal(InstigatedBy.Location-HitLocation);
	else
		HitNormal = Normal(Vect(0,0,1) + VRand() * 0.2 + vect(0,0,2.8));

	if (DamageType.Default.bCausesBlood)
	{
		if (class'GameInfo'.static.UseLowGore())
			Spawn(GibGroupClass.default.LowGoreBloodHitClass,InstigatedBy,, HitLocation, Rotator(HitNormal));
		else
		{
			if (DamageType.Name != 'DamType_Fists' && DamageType.Name != 'DamType_FistsUppercut')
			{
				if (Health > 0)
				{
					if (DamageType.Name == 'DamType_RedLabel' || DamageType.Name == 'DamType_Super90GA')
						Grow = spawn(class'Effect_ShotgunBloodHitGrow',None,, HitLocation+(4*-HitNormal), Rotator(HitNormal));
					else
						Grow = spawn(class'Effect_BloodHitGrow',None,, HitLocation+(4*-HitNormal), Rotator(HitNormal));
					if (Grow != None)
						Grow.DoSplat(rotator(-HitNormal));
				}
				else
					spawn(class'Effect_BloodHitDead',None,, HitLocation+(4*-HitNormal), Rotator(HitNormal));
			}
			else if (LastBone == 'headshot')
				spawn(class'Effect_FistHitGrow',None,, HitLocation+(10*-HitNormal), Rotator(HitNormal));
			SpawnBloodGlobs(DamageType, HitLocation, Rotator(HitNormal), InstigatedBy);
		}
	}


	// red label head explosions, burning deaths, gibs
	if (((((DamageType.name == 'DamType_RedLabel') && (VSize(InstigatedBy.Location - Location) < 250)) || (DamageType.Name == 'DamType_IncinGrenade' || DamageType.Name == 'DamType_Napalm')) && (Health <= 0)) || DamageType.Default.bAlwaysGibs)
		DoDamageFX(HitBone, 8*Damage, DamageType, Rotator(HitNormal));

	if (DamageType.default.DamageOverlayMaterial != None && Damage > 0 ) // additional check in case shield absorbed
		SetOverlayMaterial(DamageType.default.DamageOverlayMaterial, DamageType.default.DamageOverlayTime, false);
}

simulated function SpawnBloodGlobs(class<DamageType> DamageType, vector SpawnLocation, rotator SpawnRotation, Pawn Attacker)
{
	local int i, RandGlobs;
	local rotator GlobRotation;

	// different amounts for different weapons
	if (DamageType.name == 'DamType_Fists')
		return;
	else if (DamageType.name == 'DamType_RedLabel' || DamageType.name == 'DamType_Super90GA'  || DamageType.Name == 'DamType_FistsUppercut' || IsInState('Dying'))
		RandGlobs = 1;
	else if (DamageType.name == 'DamType_DE' || DamageType.name == 'DamType_GE')
		RandGlobs = Rand(3)+2;
	else if (DamageType.name == 'DamType_HEGrenade')
		RandGlobs = Rand(7)+5;
	else
		RandGlobs = Rand(2);

	// spawn them globs
	for (i=0;i<RandGlobs;i++)
	{
		GlobRotation = SpawnRotation;
		GlobRotation.Yaw = (GlobRotation.Yaw-16385)+Rand(32767);
		GlobRotation.Yaw = (GlobRotation.Yaw-16385)+Rand(32767);
		GlobRotation.Pitch = (GlobRotation.Pitch-8196)+Rand(16385);
		GlobRotation.Pitch = (GlobRotation.Pitch-8196)+Rand(16385);
		spawn(class'Effect_BloodGlob',None,,SpawnLocation+(10*-vector(SpawnRotation)),GlobRotation);
	}
}

simulated function DoDamageFX( Name boneName, int Damage, class<DamageType> DamageType, Rotator r )
{
	if ((DamageType.Name == 'DamType_RedLabel' || DamageType.Name == 'DamType_AKBayonet' || DamageType.Name == 'DamType_Machete') && Health <= 0)
	{
		if (DamageType.Default.bAlwaysSevers && Damage > 2)
			HitFX[HitFxTicker].bSever = true;
	}

	HitFX[HitFxTicker].damtype = DamageType;
	HitFX[HitFxTicker].bone = boneName;
	HitFX[HitFxTicker].rotDir = r;
	HitFxTicker = HitFxTicker + 1;
	if( HitFxTicker > ArrayCount(HitFX)-1 )
		HitFxTicker = 0;
}

simulated function ProcessHitFX()
{
	local Coords boneCoords;
	local class<xEmitter> HitEffects[4];
	local int i,j;
	local float GibPerterbation;
	local name NewEffectBone;

	if (Level.NetMode == NM_DedicatedServer)
		return;

	for (SimHitFxTicker = SimHitFxTicker; SimHitFxTicker != HitFxTicker; SimHitFxTicker = (SimHitFxTicker + 1) % ArrayCount(HitFX))
	{
		j++;
		if (j > 30)
		{
			SimHitFxTicker = HitFxTicker;
			return;
		}

		if ((HitFX[SimHitFxTicker].damtype == None) || (Level.bDropDetail && (Level.TimeSeconds - LastRenderTime > 3) && !IsHumanControlled()))
			continue;

		if (HitFX[SimHitFxTicker].damtype.default.bAlwaysGibs)
			NewEffectBone = 'none';
		else if (HitFX[SimHitFxTicker].bone == 'headshot')
			NewEffectBone = 'head';

		boneCoords = GetBoneCoords(NewEffectBone);

		if (!Level.bDropDetail)
		{
			HitFX[SimHitFxTicker].damtype.static.GetHitEffects(HitEffects, Health);

			if (!PhysicsVolume.bWaterVolume) // don't attach effects under water
			{
				for(i=0;i<ArrayCount(HitEffects);i++)
				{
					if( HitEffects[i] == None )
						continue;

					AttachEffect( HitEffects[i], NewEffectBone, boneCoords.Origin, HitFX[SimHitFxTicker].rotDir );
				}
			}
		}
		if (class'GameInfo'.static.UseLowGore())
			HitFX[SimHitFxTicker].bSever = false;

		if (HitFX[SimHitFxTicker].DamType.Name == 'DamType_IncinGrenade' || HitFX[SimHitFxTicker].DamType.Name == 'DamType_Napalm')
			AttachFlamingEffects();

		if (HitFX[SimHitFxTicker].bSever || HitFX[SimHitFxTicker].damtype.default.bAlwaysGibs)
		{
			GibPerterbation = HitFX[SimHitFxTicker].DamType.default.GibPerterbation;

			switch(NewEffectBone)
			{

				case 'head':
					if (!HitFX[SimHitFxTicker].DamType.Default.bRubbery)
						SpawnGiblet(GetGibClass(EGT_Head), boneCoords.Origin, HitFX[SimHitFxTicker].rotDir, GibPerterbation );
					else
						SpawnHeadBurst(boneCoords.Origin, rotator(-vector(HitFX[SimHitFxTicker].rotDir)));
					GibCountHead--;
				break;

				case 'none':
					if (HitFX[SimHitFxTicker].damtype.default.bAlwaysGibs && !HitFX[SimHitFxTicker].damtype.default.bSpecial)
					{
						SpawnGiblet(GetGibClass(EGT_Torso), boneCoords.Origin, HitFX[SimHitFxTicker].rotDir, GibPerterbation);
							GibCountTorso--;
							bGibbed = true;
					}
					break;
			}
			if (NewEffectBone == 'head' && HitFX[SimHitFxTicker].bSever)
				HideBone('head');
		}
	}
}

simulated function SpawnHeadBurst(Vector SpawnLocation, Rotator SpawnRotation)
{
	if (class'GameInfo'.static.UseLowGore())
		return;

	AttachEffect(class'Effect_BrainChunk','head',SpawnLocation,SpawnRotation);
}

// Add Item to this pawn's inventory.
// Returns true if successfully added, false if not.
function bool AddInventory(inventory NewItem)
{
	// Skip if already in the inventory.
	local inventory Inv;
	local actor Last;
	local int Count;

	NewItem.SetOwner(Self);
	NewItem.NetUpdateTime = Level.TimeSeconds - 1;

	if (NewItem == None)
		log("tried to add none inventory to "$self);

	if (Weapon(Last) == None && Ammunition(NewItem) == None) // less cpu usage, shorter inventory chain
	{
		Last = self;
		for (Inv=Inventory;Inv!=None;Inv=Inv.Inventory)
		{
			if (Inv == NewItem)
				return false;
			Last = Inv;
			Count++;
			if (Count > 1000)
				break;
		}
	}

	// Add to back of inventory chain (so minimizes net replication effect).
	NewItem.Inventory = None;

	if (Ammunition(NewItem) == None)
	{
		Last.Inventory = NewItem;
		Last.NetUpdateTime = Level.TimeSeconds - 1;
		Last = NewItem;
	}

	if (Controller != None)
		Controller.NotifyAddInventory(NewItem);
	return true;
}

function Weapon AddNewInventory(class<Weapon_Base> CreatedWeapon, bool bSwitchGunNow, optional int InvOffset)
{
	local Weapon SpawnedWeapon;

	if (CreatedWeapon == None)
		return None;

	SpawnedWeapon = Spawn(CreatedWeapon);

	if (SpawnedWeapon != None)
	{
		if (SpawnedWeapon.Default.InventoryGroup == 4 && InvOffset > 3)
			SpawnedWeapon.InventoryGroup = 5;
		if (bWarGame && (SpawnedWeapon.InventoryGroup == 4 || SpawnedWeapon.InventoryGroup == 5) && FindBetterWeapon() || FOBot(Controller) != None || (Weapon != None && Weapon_Base(Weapon).IsReloading()))
			Weapon_Base(SpawnedWeapon).bDontSwitchNow = true;
		else if ((bSwitchGunNow && CriticalReplicationInfo.bLAWCarrier && Weapon_LAWBase(SpawnedWeapon) == None) || FindGreaterEquipItem(SpawnedWeapon.InventoryGroup) || FOBot(Controller) != None || (Weapon != None && Weapon_Base(Weapon).IsReloading()))
			Weapon_Base(SpawnedWeapon).bDontSwitchNow = true;
		else
			Weapon_Base(SpawnedWeapon).bJustBought = true;
		SpawnedWeapon.GiveTo(self);
	}
	else
		return None;

	if (Inventory != None && bSwitchGunNow)
		Inventory.OwnerEvent('LoadOut');

	if (ROLE == Role_Authority && FOBot(Controller) != None)
		Weapon_Base(SpawnedWeapon).GetAmmo(0).NumClips = Weapon_Base(SpawnedWeapon).GetAmmo(0).MaxClips;

	return SpawnedWeapon;
}

function GiveFists()
{
	local Weapon_Base SpawnedWeapon;
	
	EquipList[0] = class'Weapon_FistsBase';
	AttachList[0] = None;
	SpawnedWeapon = Spawn(EquipList[0]);
	SpawnedWeapon.bDontSwitchNow = FindBetterWeapon();
	SpawnedWeapon.GiveTo(self);
}

function bool FindGreaterEquipItem(int GroupOffset)
{
	local int i;

	for (i=GroupOffset;i<5;i++)
	{
		if (EquipList[i] != None && FindWeapon(i+1) == None && (!bWarGame || (!bJustSpawned || (GroupOffset > 0 && i != 3 && i != 4))))
			return true;
	}
	return false;
}

function bool FindBetterWeapon()
{
	local int i;

	for (i=1;i<3;i++)
	{
		if (EquipList[i] != None)
			return true;
	}
	return false;
}

// AI

simulated function Inventory FindStaticType( class DesiredClass )
{
	local Inventory Inv;
	local class<Weapon_Base> DesiredBase;
	local int Count;

	DesiredBase = class<Weapon_Base>(DesiredClass);

	if (DesiredBase == None)
		return None;

	for (Inv=Inventory;Inv!=None;Inv=Inv.Inventory)
	{
		if (Weapon_Base(Inv) == None)
			return None;
		if (Weapon_Base(Inv).StaticName ~= DesiredBase.Default.StaticName)
			return Inv;
		Count++;
		if (Count > 1000)
			return None;
	}
	return None;
}

simulated function bool FindInventoryGroup(int DesiredGroup)
{
	local Inventory Inv;
	local int Count;

	for (Inv=Inventory;Inv!=None;Inv=Inv.Inventory)
	{
		if (Inv.InventoryGroup == DesiredGroup)
			return true;
		Count++;
		if (Count > 1000)
			return false;
	}
	return true;
}

function name GetWeaponBoneFor(Inventory I)
{
	return 'righthand';
}

simulated function vector CalcDrawOffset(Inventory Inv)
{
	local vector DrawOffset;

	if (Controller == None)
		return (Inv.PlayerViewOffset >> Rotation) + BaseEyeHeight * vect(0,0,1);

	DrawOffset = ((0.9/Weapon.DisplayFOV * 100 * ModifiedPlayerViewOffset(Inv)) >> GetViewRotation() );
	if (!IsLocallyControlled())
		DrawOffset.Z += BaseEyeHeight;
	else
		DrawOffset.Z += EyeHeight;
	return DrawOffset;
}

function AddDefaultInventory()
{}

// tilt the head a little
simulated event StartCrouch(float HeightAdjust)
{
	EyeHeight += HeightAdjust;
	OldZ -= HeightAdjust;
	BaseEyeHeight = FMin(0.75 * CrouchHeight, CrouchHeight - 12);
	if (Role == ROLE_Authority || HeightAdjust == 0)
		SetBoneRotation('Bip01 Head',rot(4000,3000,0));
}

// undo tilt
simulated event EndCrouch(float HeightAdjust)
{
	EyeHeight -= HeightAdjust;
	OldZ += HeightAdjust;
	BaseEyeHeight = Default.BaseEyeHeight;
	if (Role == ROLE_Authority || HeightAdjust == 0)
		SetBoneRotation('Bip01 Head',rot(0,0,0));
}

function bool IsHeadShot(vector loc, vector ray, float AdditionalScale)
{
	local coords C;
	local vector HeadLoc, B, M, diff, X, Y, Z;
	local float t, DotMM, Distance;
	local int look;

	if (HeadBone == '')
		return false;

	// If we are a dedicated server estimate what animation is most likely playing on the client
	if (Level.NetMode == NM_DedicatedServer)
	{
		if (DrivenVehicle != None)
			PlayAnim(DrivenVehicle.DriveAnim, 1.0, 0.0);
		else if (Physics == PHYS_Falling)
			PlayAnim(AirAnims[0], 1.0, 0.0);
		else if (Physics == PHYS_Swimming)
		{
			GetAxes(Rotation, X, Y, Z);
			if (VSize(Velocity) > 40)		
			{
				if (Abs(X dot Velocity) > Abs(Y dot Velocity))
				{
					if ((X dot Velocity) > 0)
						PlayAnim(SwimAnims[0], 1.0, 0.0);
					else
						PlayAnim(SwimAnims[1], 1.0, 0.0);
				}
				else
				{
					if ((Y dot Velocity) < 0)
						PlayAnim(SwimAnims[2], 1.0, 0.0);
					else
						PlayAnim(SwimAnims[3], 1.0, 0.0);
				}
			}
			else
				PlayAnim(IdleSwimAnim, 1.0, 0.0);
		}
		else if (Physics == PHYS_Walking)
		{
			if (VSize(Velocity) > 40)		
			{
				GetAxes(Rotation, X,Y,Z);
				if (bIsCrouched)
				{
					if (Abs(X dot Velocity) > Abs(Y dot Velocity))
					{
						if ((X dot Velocity) > 0)
							PlayAnim(CrouchAnims[0], 1.0, 0.0);
						else
							PlayAnim(CrouchAnims[1], 1.0, 0.0);
					}
					else
					{
						if ((Y dot Velocity) < 0)
							PlayAnim(CrouchAnims[2], 1.0, 0.0);
						else
							PlayAnim(CrouchAnims[3], 1.0, 0.0);
					}
					//PlayAnim(IdleCrouchAnim, 1.0, 0.0);
				}
				else if (bIsWalking)
				{
					if (Abs(X dot Velocity) > Abs(Y dot Velocity))
					{
						if ((X dot Velocity) > 0)
							PlayAnim(WalkAnims[0], 1.0, 0.0);
						else
							PlayAnim(WalkAnims[1], 1.0, 0.0);
					}
					else
					{
						if ((Y dot Velocity) < 0)
							PlayAnim(WalkAnims[2], 1.0, 0.0);
						else
							PlayAnim(WalkAnims[3], 1.0, 0.0);
					}
				}
				else
				{
					if (Abs(X dot Velocity) > Abs(Y dot Velocity))
					{
						if ((X dot Velocity) > 0)
							PlayAnim(MovementAnims[0], 1.0, 0.0);
						else
							PlayAnim(MovementAnims[1], 1.0, 0.0);
					}
					else
					{
						if ((Y dot Velocity) < 0)
							PlayAnim(MovementAnims[2], 1.0, 0.0);
						else
							PlayAnim(MovementAnims[3], 1.0, 0.0);
					}
				}
			}
			else
			{
				if (bIsCrouched)
					PlayAnim(IdleCrouchAnim, 1.0, 0.0);
				else
					PlayAnim(IdleWeaponAnim, 1.0, 0.0);
			}

		}
		if (bDoTorsoTwist)
		{
			SmoothViewYaw = Rotation.Yaw;
			SmoothViewPitch = ViewPitch;

			look = (256 * ViewPitch) & 65535;
			if (look > 32768)
				look -= 65536;

			SetTwistLook(0, look);
		}
		SetAnimFrame(0.5);
	}
	else
		BoneRefresh();

	C = GetBoneCoords('headshot');
	HeadLoc = C.Origin + (HeadHeight * HeadScale * AdditionalScale * C.XAxis);

	// express snipe trace line in terms of B + tM
	B = loc;
	M = ray * (2.0 * CollisionHeight + 2.0 * CollisionRadius);

	// find point-line squared distance
	diff = HeadLoc - B;
	t = M Dot diff;
	if (t > 0)
	{
		DotMM = M dot M;
		if (t < DotMM)
		{
		    t = t / DotMM;
		    diff = diff - (t * M);
		}
		else
		{
		    t = 1;
		    diff -= M;
		}
	}
	else
		t = 0;

	Distance = Sqrt(diff Dot diff);
	return (Distance < (HeadRadius * HeadScale * AdditionalScale));
}

// trimmer and scalable for regular bone hits
function bool IsBoneHit(vector loc, vector ray, float AdditionalScale, name BoneHit, float InRadius, float InHeight)
{
	local coords C;
	local vector BoneLoc, B, M, diff;
	local float t, DotMM, Distance;

	C = GetBoneCoords(BoneHit);
	BoneLoc = C.Origin + (InHeight * AdditionalScale * C.XAxis);

	// express snipe trace line in terms of B + tM
	B = loc;
	M = ray * (2.0 * CollisionHeight + 2.0 * CollisionRadius);

	// find point-line squared distance
	diff = BoneLoc - B;
	t = M Dot diff;
	if (t > 0)
	{
		DotMM = M dot M;
		if (t < DotMM)
		{
		    t = t / DotMM;
		    diff = diff - (t * M);
		}
		else
		{
		    t = 1;
		    diff -= M;
		}
	}
	else
		t = 0;

	Distance = Sqrt(diff Dot diff);
	return (Distance < (InRadius * AdditionalScale));
}


simulated function Destroyed()
{
	local int i;
	
	Super.Destroyed();

	// remove any sticky knives
	if (StuckKnife != None)
		StuckKnife.Destroy();

	// kill the realtime shadows
	if (RealtimeShadow != None)
		RealtimeShadow.Destroy();

	// kill armor
	for (i=0;i<ArmorPieces.Length;i++)
	{
		if (ArmorPieces[i] != None)
			ArmorPieces[i].Destroy();
	}

	// kill the blood pools
	for (i=0;i<BloodPool.Length;i++)
	{
		if (BloodPool[i] != None)
			BloodPool[i].Destroy();
	}

	// kill any spectator remnants
	if (SpectatorWeapon != None)
		SpectatorWeapon.Destroy();
}

simulated function SetBaseEyeheight()
{
	if (!bIsCrouched)
		BaseEyeheight = Default.BaseEyeheight;
	else
		BaseEyeheight = FMin(0.75 * CrouchHeight, CrouchHeight - 12);

	EyeHeight = BaseEyeheight;
}

defaultproperties
{
	// skins
	MercGloveSkin=Material'magnum2.ArmTextures.glove_merc'
	MercSleaveSkins(0)=Material'magnum2.ArmTextures.Merc_green_camo'
	MercSleaveSkins(1)=Material'magnum2.ArmTextures.Merc_olive'
	MercSleaveSkins(2)=Material'magnum2.ArmTextures.Merc_red_camo'
	MercSleaveSkins(3)=Material'magnum2.ArmTextures.Merc_brown'
	MercSleaveSkins(4)=Material'magnum2.ArmTextures.Merc_blue_camo'
	MercSleaveSkins(5)=Material'magnum2.ArmTextures.Merc_urban_camo'

	SOGloveSkin=Material'magnum2.ArmTextures.glove_swat'
	SOSleaveSkins(0)=Material'magnum2.ArmTextures.SO_blue'
	SOSleaveSkins(1)=Material'magnum2.ArmTextures.SO_black'
	SOSleaveSkins(2)=Material'magnum2.ArmTextures.SO_urban_camo'
	SOSleaveSkins(3)=Material'magnum2.ArmTextures.SO_green_camo'
	SOSleaveSkins(4)=Material'magnum2.ArmTextures.SO_desert_camo'
	
	// animation
	// taunts
	TauntAnims(0)="gesture_point"
	TauntAnims(1)="gesture_beckon"
	TauntAnims(2)="gesture_halt"
	TauntAnims(3)="ThroatCut"

	// idles
	IdleSidearmAnim=Pistol_Idle
	IdleSMGAnim=SMGIdle
	IdleGrenadeAnim=GrenadeIdle
	IdleMeleeAnim=Fists_Hold
	IdleAkimboAnim=Pistol_Idle
	IdleHeavyAnim=LAWIdle
	IdleDriveAnim=driveg

	// fires
	FireSidearmAnim=small_fire
	FireSidearmHeavyAnim=large_fire
	FireSMGAnim=SMGFire
	FireGrenadeAnim=GrenadeFire
	FireMeleeAnim=Fists_Fire
	FireMeleeAltAnim=Fists_Fire2
	FireHeavyBurstAnim=LAWFire
	GrenadePrimeAnim=nadeprime
	FireAkimboAnim=Fists_Hold
	FireAkimboAltAnim=Fists_Hold
	//FireRifleRapidAnim=RifleFire

	// crouchfires
	FireSidearmCrouchAnim=SidearmCrouchFire
	FireSMGCrouchAnim=SMGCrouchFire
	FireGrenadeCrouchAnim=GrenadeCrouchFire
	FireMeleeCrouchAnim=MeleeCrouchFire
	FireRifleCrouchAnim=RifleCrouchFire

	// movement
	MovementAnims(0)=RunF
	MovementAnims(1)=RunB
	MovementAnims(2)=RunL
	MovementAnims(3)=RunR

	// body armor
	ShoulderArmor(0)=(ArmorActor=FragOps.Armor_Shoulder,ArmorMesh=StaticMesh'FOSMs.swatarmor.SWAT_Armor_L_shoulder',ArmorBone="Bip01 L UpperArm",ArmorOffset=(X=13,Y=0.6,Z=2.2),ArmorRotation=(Pitch=400,Yaw=0,Roll=0))
	ShoulderArmor(1)=(ArmorActor=FragOps.Armor_Shoulder,ArmorMesh=StaticMesh'FOSMs.swatarmor.SWAT_Armor_R_shoulder',ArmorBone="Bip01 R UpperArm",ArmorOffset=(X=13,Y=0,Z=-1.8),ArmorRotation=(Pitch=-400,Yaw=0,Roll=-32767))
	LegArmor(0)=(ArmorActor=FragOps.Armor_Leg,ArmorMesh=StaticMesh'FOSMs.swatarmor.SWAT_Armor_L_leg',ArmorBone="Bip01 L Calf",ArmorOffset=(X=25,Y=-1,Z=0),ArmorRotation=(Pitch=600,Yaw=-500,Roll=-16384))
	LegArmor(1)=(ArmorActor=FragOps.Armor_Leg,ArmorMesh=StaticMesh'FOSMs.swatarmor.SWAT_Armor_R_leg',ArmorBone="Bip01 R Calf",ArmorOffset=(X=25,Y=-1,Z=0),ArmorRotation=(Pitch=-600,Yaw=-500,Roll=-16384))
	Helmets(0)=(ArmorActor=FragOps.Armor_Helmet,ArmorMesh=StaticMesh'FOSMs.swatarmor.SF_Helm_medium',ArmorBone="Dummy13",ArmorOffset=(X=-9.0,Y=-1.4,Z=0),ArmorRotation=(Pitch=-16384,Yaw=0,Roll=1700))
	Helmets(1)=(ArmorActor=FragOps.Armor_Helmet_Heavy,ArmorMesh=StaticMesh'FOSMs.swatarmor.SF_Helm_heavy',ArmorBone="Dummy13",ArmorOffset=(X=-9.0,Y=-1.4,Z=0),ArmorRotation=(Pitch=-16384,Yaw=0,Roll=1700))
	ArmorSkins(0)=Material'FOCharacters.ArmorTextures.armor_final'
	ArmorSkins(1)=Material'FOCharacters.ArmorTextures.VisorFinal'

	// falling
	FallSpeedThreshold=790

	// ladder
	LadderAnim=LadUp

	// sprint anim
	SprintAnim=RunF

	// poison
	PoisonTime=12.0
	PoisonDamage=9
	PoisonChokeSound=sound'FOWeaponSounds.Misc.PoisonChoke'

	// skeleton
	SkeletonMesh=mesh'HumanMaleA.SkeletonMale'
	SkeletonTexture=Texture'PlayerSkins.Human_Skeleton'

	// fire root bone, for animation blending
	FireRootBone="Bip01 Spine"

	// action system
	SprintTime=5.85
	SprintMultiple=1.24
	SprintBreathSound=sound'FOGameSounds.PlayerSounds.HeavyBreathLong'
	RestingSound=sound'FOGameSounds.PlayerSounds.RestBreathShort'
	RestTime=2.25

	// breathing
	BreathHoldTime=6.2

	// healing
	HealthRestoreMax=40

	// master movement speeds
	AccelRate=+01300.000000
	GroundSpeed=476.0
	AirSpeed=488.0
	WaterSpeed=255.0
	ConstantAcceleration=(Z=-210)
	JumpZ=389.0

	// trait defaults
	PoisonMod=1.0
	SidearmMod=1.0
	GrenadeMod=1.0
	MeleeMod=1.0
	AutoMod=1.0
	ShotgunMod=1.0
	SniperMod=1.0
	NapalmMod=1.0

	SidearmDamageMod=1.0
	GrenadeDamageMod=1.0
	MeleeDamageMod=1.0
	AutoDamageMod=1.0
	ShotgunDamageMod=1.0
	SniperDamageMod=1.0
	NapalmDamageMod=1.0

	AkimboRecoilMod=1.0
	SidearmRecoilMod=1.0
	AutoRecoilMod=1.0
	ShotgunRecoilMod=1.0
	SniperRecoilMod=1.0

	ThrowMod=1.0
	GrenadeConeMod=0
	GrenadeRadiusMod=1.0
	FootStepMod=1.0
	SpeedMod=1.0
	JumpMod=1.0
	ArmorMod=1.0
	APMod=0
	HackMod=1.0
	RestMod=1.0

	// war
	WarThrowMod=0.85

	FootstepVolume=0.2
	Species=class'FOSpeciesType'
	GibGroupClass=class'Effect_GibGroupReal'
	GibCountTorso=1
	VoiceType="FragOps.Voice_MercMaleVoice"
	VoiceClass=class'FragOps.Voice_MercMaleVoice'
	SoundGroupClass=class'FragOps.Voice_MercMaleSounds'
	bNoTeamBeacon=true
	bCanWalkOffLedges=true
	bAllowDeres=true
	bNoDefaultInventory=true
	MaxMultiJump=0
	MultiJumpRemaining=0
	MultiJumpBoost=0
	bCanDodgeDoubleJump=false
	bCanWallDodge=false
	CrouchHeight=32.0
	CrouchRadius=21.5
	WalkingPct=+0.5
	CrouchedPct=+0.42
	BaseEyeHeight=33.0
	EyeHeight=33.0
	RagShootStrength=4000
	HeadCrackSound=sound'FOWeaponSounds.Misc.HeadCrack'
	HeadShotSound=sound'FOWeaponSounds.Misc.HeadShot'
	BodyHitSound=sound'FOWeaponSounds.Misc.BodyHit2'
	BodyGruntSound=sound'FOWeaponSounds.Misc.BodyHitGrunt'
	CollisionRadius=20.0
	CollisionHeight=42.4
	BloodEffect=class'Effect_BloodJetPlus'
	MiscItem="MED. Armor"
	LastMiscItem="MED. Armor"
	ForceType=FT_DragAlong
	ForceRadius=60
	ForceScale=1.0
	bBodiesStay=true
	MaxLights=5
	LODBias=1.5
        PitchUpLimit=18000
        PitchDownLimit=49153
	bForceSkelUpdate=false
	bDramaticLighting=false
	bRealtimeShadows=false
	Bob=0.0060

	HeadRadius=+11.0
	HeadHeight=10.0

	RagdollType="Male3"
	
	RagImpactSounds(0)=sound'FOGameSounds.PlayerSounds.Breakbone_01'
	RagImpactSounds(1)=sound'FOGameSounds.PlayerSounds.Breakbone_02'
	RagImpactSounds(2)=sound'FOGameSounds.PlayerSounds.Breakbone_03'
	RagImpactSounds(3)=sound'FOGameSounds.PlayerSounds.Breakbone_04'

	MaxPools=2

	Begin Object Class=KarmaParamsSkel Name=PawnKParams
		KFriction=0.98
		KRestitution=0.15
		KAngularDamping=0.06
		KLinearDamping=0.3
		KBuoyancy=0.98
		KStartEnabled=True
		KImpactThreshold=600
		KVelDropBelowThreshold=40
		bHighDetailOnly=False
	Name="PawnKParams"
	End Object
	KParams=KarmaParams'PawnKParams'

	// block bodies when alive, also lets us interact better with karma objects
	bBlockKarma=true
	bUseCollisionStaticMesh=true
	bNewStep=true
	bJustSpawned=true
	bAlwaysRelevant=false
}