//=============================================================================
// Frag.Ops  2003 Matt 'SquirrelZero' Farber
//=============================================================================

#exec OBJ LOAD FILE=FOGameSounds.uax

class FOPlayer extends xPlayer
	dependson(FragOps_PRI)
	config(User);

// config vars
var bool bAutoBriefing;

// currently selected character skins and mesh
var byte HeadSkin, BodySkin, SelectedRecord, SelectedTeam;

// misc vars -- will sort later
var int BeganRound, WaitForPreRound;
var bool bAllowSwitch, bAllowClick, bAllowBV, bSetWeapon, bBuyMenuOpen, bShouldStopObjective, bOpenBriefing;
var float LastSplatTime, ClickTime, AltClickTime, LastUseTime, RealFOV;
var FragOps_CRI CriticalReplicationInfo;
var Obj_InteractObjective CurrentObjective;

// mouse tracking for melee weapons
var bool bSlashLocked;
var float LastSlashTime, SlashLockTime;
var byte SlashDirection, LastSlashDir;

// variables for the stickyobjective state
var float OldAirSpeed, OldGroundSpeed, OldWaterSpeed;

var Pawn OldPawn;
var Actor LastViewTarget;

// actions for players
var bool bCanAction;
var float LastRestTime;
var byte ActionPoints, APBaseLimit;
var int APModTemp;
var array<Name> ActionTypes;
var array<int> APReq;

// first person spectating
var FOSpectatorController SpectatorController;
var rotator NewViewRotation;
var float NewEyeHeight;
var byte TargetZoomLevel, TargetAP, TargetMaxAP, TargetFireType, TargetClips, TargetAmmo, TargetAmmoSec, TargetHeadDamage, TargetTorsoDamage, TargetLeftArmDamage, TargetLeftLegDamage, TargetRightArmDamage, TargetRightLegDamage;
var bool bWasFirst, bTargetScopeZoomed, bTargetPoisoned, bTargetReloading, bTargetAkimboReloading, bTargetAkimbo, bTargetWeaponPickup, bTargetWeaponPutdown;
var string TargetMiscItem;
var int TargetHand, TargetMoney;
var FOPawn NewViewTarget;
var class<Weapon_Base> TargetWeapon;

// currently chosen trait
var byte SelectedTrait;

// flash grenade blur
var MotionBlur FlashBlur;
var bool bHaveBlur;
var globalconfig bool bTeamBeacons;

// keybind checking
var float LastCheckTime;

replication
{
	reliable if (Role < ROLE_Authority)
		SendVoiceMessagePlus, ServerUnUse, GetNewTeam, ServerWire;

	// can be lost due to packetloss
	unreliable if (Role < ROLE_Authority)
		ServerSwitchTrait, ServerAddToClip, ServerBuyWeapon, ServerSellWeapon, ServerUpdateSlash, ServerResetLevel, ServerEndRound, ServerAction, ServerUnAction;

	// server functions that can be lost
	unreliable if (Role == ROLE_Authority)
		ClientShake;

	reliable if (Role == ROLE_Authority)
		ReceiveKillMessage, ClientSetObjective, ClientTakeFlash, MoveOutAnnounce;

	reliable if ((!bDemoRecording || (bClientDemoRecording && bClientDemoNetFunc)) && Role == ROLE_Authority)
		ClientVoiceMessagePlus;

	reliable if ((bNetDirty) && (Role == ROLE_Authority))
		CriticalReplicationInfo;
	
	reliable if ((bNetDirty) && (bNetOwner) && (Role == ROLE_Authority))
		ActionPoints, APModTemp;

	reliable if (RemoteRole == ROLE_AutonomousProxy)
		ClientReceivePlayerStats, ClientPendingWeapon, ClientReceiveRoundStats;
}

//=============================================================================
// initialization functions
event PostBeginPlay()
{
	local int NewPCName;
	//local LadderVolume LV;
	
	Super.PostBeginPlay();
	if (!bDeleteMe && bIsPlayer && (Role == ROLE_Authority))
		CriticalReplicationInfo = Spawn(class'FragOps_CRI', Self,,vect(0,0,0),rot(0,0,0));
	NewPCName = rand(9999999);
	CriticalReplicationInfo.PCName = string(self)$NewPCName;
	FragOps_PRI(PlayerReplicationInfo).PCName = string(self)$NewPCName;
}

//=============================================================================
// execs

// functions we dont want
exec function ChangeCharacter(string newCharacter)
{
}
exec function SwitchTeam()
{
}
exec function ChangeTeam(int N)
{
}
function ServerChangeTeam( int N )
{
}

simulated function PlayRewardAnnouncement(name AName, byte AnnouncementLevel, optional bool bForce)
{
}

simulated function PlayStatusAnnouncement(name AName, byte AnnouncementLevel, optional bool bForce)
{
}

// debug function
exec function DumpActors()
{
	local Actor DActor;
	local string ActorString;

	foreach DynamicActors(class'Actor',DActor)
	{
		ActorString = "name: "$string(DActor)$", owner: "@string(DActor.Owner);
		log(ActorString);
	}
}

exec function Speech(name Type, int Index, string Callsign)
{
	if (Pawn != None)
	{
		if (FOPawn(Pawn).VoiceClass != None)
			FOPawn(Pawn).VoiceClass.static.PlayerSpeech(Type, Index, Callsign, Self);
	}
}

exec function AdminMenu(string CommandLine)
{
	if (PlayerReplicationInfo.bAdmin)
	{
		ClientOpenMenu("Xinterface.UT2AdminMenu");
	}
}

exec function DropFlag()
{
	ServerDropFlag();
}

function ServerDropFlag()
{
	if (Pawn == None || FOPawn(Pawn) == None || FOPawn(Pawn).CarriedObjective == None)
		return;

	FOPawn(Pawn).DropObjective();
}


exec function AdminReset()
{
	ServerResetLevel();
}

function ServerResetLevel()
{
	local Actor TempActor;
	
	if (PlayerReplicationInfo.bAdmin || (Level.NetMode == NM_StandAlone))
	{
		foreach AllActors(class'Actor',TempActor)
		{
			if (FOPlayer(TempActor) != None)
			{
				FOPlayer(TempActor).bDisplayWinner = false;
				FOPlayer(TempActor).bDisplayLoser = false;
			}
			if (GameInfo(TempActor) == None && FOPawn(TempActor) == None && LevelInfo(TempActor) == None && Controller(TempActor) == None && UnrealTeamInfo(TempActor) == None && xTeamRoster(TempActor) == None && DMRoster(TempActor) == None && TempActor != self)
				TempActor.Reset();
			if (Pawn(TempActor) != None)
			{
				if (FOPlayer(Pawn.Controller) != None)
					FOPlayer(Pawn.Controller).DetachPawn();
				Pawn.Destroy();
			}
		}
		Level.Game.Reset();
		FragOpsGame(Level.Game).StartRound();
	}
}

exec function GetWeapon(class<Weapon> NewWeaponClass )
{
	local Inventory Inv;
	local int Count;

	if ((Pawn.Inventory == None) || (NewWeaponClass == None))
		return;

	if ((Pawn.Weapon != None) && (Pawn.Weapon.Class == NewWeaponClass) && (Pawn.PendingWeapon == None))
	{
		Pawn.Weapon.Reselect();
		return;
	}

	if (Pawn.PendingWeapon != None && Pawn.PendingWeapon.bForceSwitch)
		return;

	for (Inv=Pawn.Inventory; Inv!=None; Inv=Inv.Inventory)
	{
		if (Inv.Class == NewWeaponClass)
		{
			Pawn.PendingWeapon = Weapon(Inv);
			Pawn.Weapon.PutDown();
			return;
		}
		Count++;
		if (Count > 1000)
			return;
	}
}

exec function BehindView(Bool B)
{
	if ((Level.NetMode == NM_Standalone) || (IsDead() && FragOps_GRI(GameReplicationInfo).bAllowBehindview) || PlayerReplicationInfo.bAdmin || IsA('Admin') || Vehicle(Pawn) != None)
	{
		if ((Vehicle(Pawn)==None) || (Vehicle(Pawn).bAllowViewChange))
		{
			ClientSetBehindView(B);
			bBehindView = B;
		}
	}
}

function ServerToggleBehindView()
{
	local bool B;

	if (Level.NetMode == NM_Standalone || (PlayerReplicationInfo.bOnlySpectator && FragOps_GRI(GameReplicationInfo).bAllowBehindview) || Vehicle(Pawn) != None || PlayerReplicationInfo.bAdmin || IsA('Admin'))
	{
		if ( (Vehicle(Pawn)==None) || (Vehicle(Pawn).bAllowViewChange) )	// Allow vehicles to limit view changes
		{
			B = !bBehindView;
			ClientSetBehindView(B);
			bBehindView = B;
		}
	}
}

function ClientSetBehindView(bool B)
{
	local bool bWasBehindView;

	if (ViewTarget == None)
		SetViewTarget(self);

	bWasBehindView = bBehindView;
	bBehindView = B;
	CameraDist = Default.CameraDist;

	if (bBehindView != bWasBehindView)
		ViewTarget.POVChanged(self, true);

	if (Vehicle(Pawn) != None)
	{
		Vehicle(Pawn).bDesiredBehindView = B;
		Pawn.SaveConfig();
	}
}


exec function SwitchTrait(byte NewTrait)
{
	if (PlayerReplicationInfo.bOnlySpectator || (!IsDead() && !FragOps_GRI(GameReplicationInfo).bPreRound))
		return;

	ServerSwitchTrait(NewTrait);
}

function ServerSwitchTrait(byte NewTrait)
{
	if ((!IsDead() && !FragOps_GRI(GameReplicationInfo).bPreRound) || NewTrait == SelectedTrait)
		return;

	if (Pawn != None && SelectedTrait < class'Interface_TabPlayerTeam'.Default.Traits.Length)
	{
		class'Interface_TabPlayerTeam'.Default.Traits[SelectedTrait].static.RemoveTrait(FOPawn(Pawn));
		FOPawn(Pawn).ResetSpeeds();
	}

	if (NewTrait < class'Interface_TabPlayerTeam'.Default.Traits.Length)
	{
		SelectedTrait = NewTrait;
		if (Pawn != None)
		{
			class'Interface_TabPlayerTeam'.Default.Traits[SelectedTrait].static.AddTrait(FOPawn(Pawn));
			ReceiveLocalizedMessage(class'Interface_TraitMessage',, None, None, class'Interface_TabPlayerTeam'.Default.Traits[SelectedTrait]);
			FOPawn(Pawn).ResetSpeeds();
		}
	}
	else
	{
		ReceiveLocalizedMessage(class'Interface_TraitMessage',, None, None, None);
		SelectedTrait = 255;
	}

	FragOps_PRI(PlayerReplicationInfo).TraitType = SelectedTrait;
}

exec function SwitchFire(optional int i)
{
	if ((Pawn != None) && (Pawn.Weapon != None) && (((Pawn.Weapon.ClientState != WS_BringUp && Pawn.Weapon.ClientState != WS_PutDown) || Pawn.Weapon.ClientState == WS_ReadyToFire) && !Weapon_Base(Pawn.Weapon).IsReloading() && !Weapon_Base(Pawn.Weapon).GetFM(0).bIsFiring))
	{
		if (i > 0)
			Weapon_Base(Pawn.Weapon).SwitchFireType(1);
		else
			Weapon_Base(Pawn.Weapon).SwitchFireType();
	}
}

exec function Reload()
{
	local Ammo_Base FOAmmoClass;

	if (Pawn == None || Pawn.Weapon == None || Weapon_Base(Pawn.Weapon).GetAmmo(0) == None)
		return;

	FOAmmoClass = Weapon_Base(Pawn.Weapon).GetAmmo(0);
	if (FOAmmoClass.NumClips > 0 && (FOAmmoClass.AmmoAmount < FOAmmoClass.MaxAmmo || (Weapon_Base(Pawn.Weapon).bIsAkimbo && Ammo_AkimboBase(FOAmmoClass).AmmoSec < FOAmmoClass.MaxAmmo)) && (Pawn.Weapon.ClientState != WS_PutDown && Pawn.Weapon.ClientState != WS_BringUp && Pawn.Weapon.ClientState != WS_Hidden) && Pawn.PendingWeapon == None && !Weapon_Base(Pawn.Weapon).IsReloading())
		Weapon_Base(Pawn.Weapon).PlayReload();
	else if (Weapon_Super90Base(Pawn.Weapon) != None && Weapon_Base(Pawn.Weapon).IsReloading())
		Weapon_Super90Base(Pawn.Weapon).CancelReload();
}

exec function Fire(optional float F)
{
	if ((Pawn != None) && (Pawn.Weapon != None))
	{
		if ((Weapon_Base(Pawn.Weapon).GetFM(0) != None) && (Weapon_Base(Pawn.Weapon).GetAmmo(0) != None) && (Pawn.Weapon.AmmoAmount(0) < Weapon_Base(Pawn.Weapon).GetFM(0).AmmoPerFire) && (Weapon_Base(Pawn.Weapon).GetFM(0) != None) && (Weapon_Base(Pawn.Weapon).GetFM(0).RecoilSize > 0) && (Level.TimeSeconds > ClickTime) && (!Weapon_Base(Pawn.Weapon).IsReloading()))
		{
			Weapon_Base(Pawn.Weapon).PlayEmptySound();
			ClickTime = Level.TimeSeconds + 0.15;
		}
	}
	Super.Fire(f);
}

exec function AltFire(optional float F)
{
	if ((Pawn != None) && (Pawn.Weapon != None) && (Weapon_Base(Pawn.Weapon).GetFM(1) != None) && (Weapon_Base(Pawn.Weapon).GetAmmo(1) != None) && (Weapon_Base(Pawn.Weapon).GetFM(1) != None) && (Weapon_Base(Pawn.Weapon).GetFM(1).RecoilSize > 0) && (Level.TimeSeconds > AltClickTime) && (!Weapon_Base(Pawn.Weapon).IsReloading()))
	{
		Weapon_Base(Pawn.Weapon).PlayEmptySound();
		AltClickTime = Level.TimeSeconds + 0.15;
	}
	Super.AltFire(f);
}

exec function SpecialFire()
{
	if ((Pawn != None) && (Pawn.Weapon != None) && (!Weapon_Base(Pawn.Weapon).IsReloading()) && (Pawn.Weapon.ClientState == WS_ReadyToFire))
	{
		Weapon_Base(Pawn.Weapon).SpecialFire();
	}
}

exec function BuyAmmo(optional bool bMaxAmmo)
{
	local Ammo_Base FOAmmoClass;
	
	if (Pawn == None || Pawn.Health <= 0 || Pawn.Weapon == None || !FOPawn(Pawn).bCanBuy)
		return;

	FOAmmoClass = Weapon_Base(Pawn.Weapon).GetAmmo(0);
	if (CriticalReplicationInfo.Money >= FOAmmoClass.Default.ClipPrice && Pawn.Weapon != None && FOAmmoClass.NumClips < FOAmmoClass.Default.MaxClips)
	{
		ServerAddToClip(bMaxAmmo);
	}
}

exec function BuyAmmoPrimary(optional bool bMaxAmmo)
{
	if (Pawn == None || Pawn.Health <= 0 || !FOPawn(Pawn).bCanBuy)
		return;

	ServerAddToClip(bMaxAmmo,3);
}

exec function BuyAmmoSidearm(optional bool bMaxAmmo)
{
	if (Pawn == None || Pawn.Health <= 0 || !FOPawn(Pawn).bCanBuy)
		return;

	ServerAddToClip(bMaxAmmo,2);
}


function ServerAddToClip(optional bool bShouldMax, optional byte AmmoGroup)
{
	local Ammo_Base FOAmmoClass;
	local int i, TotalClips, TotalMoney;

	if (Pawn == None || FOPawn(Pawn) == None || Pawn.Weapon == None || Pawn.Health <= 0 || !FOPawn(Pawn).bCanBuy)
		return;

	if (AmmoGroup == 3)
	{
		if (FOPawn(Pawn).FindWeapon(3) != None)
			FOAmmoClass = Weapon_Base(FOPawn(Pawn).FindWeapon(3)).GetAmmo(0);
		else
			return;
	}
	else if (AmmoGroup == 2)
	{
		if (FOPawn(Pawn).FindWeapon(2) != None)
			FOAmmoClass = Weapon_Base(FOPawn(Pawn).FindWeapon(2)).GetAmmo(0);
		else
			return;
	}
	else
		FOAmmoClass = Weapon_Base(Pawn.Weapon).GetAmmo(0);

	if (FOAmmoClass == None)
		return;

	AmmoSound();
	
	TotalClips = FOAmmoClass.NumClips;
	TotalMoney = CriticalReplicationInfo.Money;
	
	while (true)
	{
		if (TotalClips < FOAmmoClass.MaxClips && TotalMoney - FOAmmoClass.Default.ClipPrice >= 0)
		{
			i = 0;
			while(i < FOAmmoClass.Default.ClipMultiple && TotalClips < FOAmmoClass.Default.MaxClips)
			{
				i++;
				TotalClips++;
			}
			TotalMoney -= FOAmmoClass.Default.ClipPrice;
			if (!bShouldMax)
				break;
		}
		else
			break;
	}

	// set the ammo now
	FOAmmoClass.NumClips = TotalClips;
	CriticalReplicationInfo.Money = TotalMoney;
}

exec function BuyMelee(byte MeleeWeapon, optional byte MeleeAttachment)
{
	if (MeleeWeapon > ArrayCount(class'Interface_BuyMenuPlus'.Default.Melee))
		return;

	ServerBuyWeapon(class'Interface_BuyMenuPlus'.Default.Melee[MeleeWeapon],MeleeAttachment);
}

exec function BuySidearm(byte SidearmWeapon, optional byte SidearmAttachment)
{
	if ((SidearmWeapon > ArrayCount(class'Interface_BuyMenuPlus'.Default.Sidearms)) || (PlayerReplicationInfo.Team.TeamIndex == 0 && (SidearmWeapon % 2) != 0) || (PlayerReplicationInfo.Team.TeamIndex == 1 && (SidearmWeapon % 2) == 0))
		return;

	ServerBuyWeapon(class'Interface_BuyMenuPlus'.Default.Sidearms[SidearmWeapon],SidearmAttachment);
}

exec function BuyPrimary(byte PrimaryWeapon, optional byte PrimaryAttachment)
{
	if ((PrimaryWeapon > ArrayCount(class'Interface_BuyMenuPlus'.Default.Rifles)) || (PlayerReplicationInfo.Team.TeamIndex == 0 && (PrimaryWeapon % 2) != 0) || (PlayerReplicationInfo.Team.TeamIndex == 1 && (PrimaryWeapon % 2) == 0))
		return;

	ServerBuyWeapon(class'Interface_BuyMenuPlus'.Default.Rifles[PrimaryWeapon],PrimaryAttachment);
}

exec function BuyGrenade(byte GrenadeType)
{
	if (GrenadeType > ArrayCount(class'Interface_BuyMenuPlus'.Default.Grenades))
		return;

	ServerBuyWeapon(class'Interface_BuyMenuPlus'.Default.Grenades[GrenadeType],255);
}

exec function BuyArmor(byte ArmorType)
{
	if (ArmorType > ArrayCount(class'Interface_BuyMenuPlus'.Default.Misc))
		return;

	ServerBuyWeapon(class'Weapon_Base',ArmorType);
}

exec function SellWeapon()
{
	if (Pawn == None || FOPawn(Pawn) == None || Pawn.Weapon == None || Weapon_Base(Pawn.Weapon).WeaponType == Melee || !FOPawn(Pawn).bCanBuy || (Pawn.Weapon != None && Pawn.Weapon.ClientState != WS_ReadyToFire && Pawn.Weapon.ClientState != WS_Hidden))
		return;

	ServerSellWeapon();
}

function ServerSellWeapon()
{
	if (Pawn == None || FOPawn(Pawn) == None || Pawn.Weapon == None || Weapon_Base(Pawn.Weapon).WeaponType == Melee || !FOPawn(Pawn).bCanBuy || (Pawn.Weapon != None && Weapon_Base(Pawn.Weapon).IsReloading()))
		return;

	FOPawn(Pawn).EquipList[Pawn.Weapon.InventoryGroup - 1] = None;
	if (Weapon_Base(Pawn.Weapon).BaseWeaponClass != None && FOPawn(Pawn).AttachList[Pawn.Weapon.InventoryGroup - 1] != None)
		CriticalReplicationInfo.AddMoney(Weapon_Base(Pawn.Weapon).Default.GunCost + Weapon_Base(Pawn.Weapon).Default.BaseWeaponClass.Default.GunCost);
	else
		CriticalReplicationInfo.AddMoney(Weapon_Base(Pawn.Weapon).Default.GunCost);
	FOPawn(Pawn).AttachList[Pawn.Weapon.InventoryGroup - 1] = None;
	BuySound();
	Weapon_Base(Pawn.Weapon).DestroyWeapon();
	ClientPendingWeapon(FOPawn(Pawn).GetBestWeapon());
}

function ServerBuyWeapon(class<Weapon_Base> DesiredWeapon, optional byte DesiredAttachment, optional byte DesiredClips)
{
	local bool bShouldBeBought, bShouldBeRepaired, bNoMoneyBack;
	local int TotalCash, CopySidearmClips, CopyPrimaryClips;
	local string ArmorString, LastMiscItem;

	if (Pawn == None || FOPawn(Pawn) == None || (Pawn != None && !FOPawn(Pawn).bCanBuy))
		return;

	// armor
	if (DesiredWeapon.Name == 'Weapon_Base')
	{
		switch (DesiredAttachment)
		{
			case 0:
				ArmorString = "LIT. Armor";
				break;

			case 1:
				ArmorString = "MED. Armor";
				break;				
			
			case 2:
				ArmorString = "HVY. Armor";
				break;
		}

		bShouldBeBought = false;
		if (ArmorString != FOPawn(Pawn).MiscItem)
		{
			if (Pawn.IsInState('PlayerSprinting'))
				return;
			
			bShouldBeBought = true;
			LastMiscItem = FOPawn(Pawn).MiscItem;
			FOPawn(Pawn).MiscItem = ArmorString;
			if ((FOPawn(Pawn).HeadDamage > 0 || FOPawn(Pawn).TorsoDamage > 0 || FOPawn(Pawn).LeftArmDamage > 0 || FOPawn(Pawn).RightArmDamage > 0 || FOPawn(Pawn).LeftLegDamage > 0 || FOPawn(Pawn).RightLegDamage > 0))
				bNoMoneyBack = true;
			FOPawn(Pawn).ResetZoneDamage();
		}
		else if ((FOPawn(Pawn).HeadDamage > 0 || FOPawn(Pawn).TorsoDamage > 0 || FOPawn(Pawn).LeftArmDamage > 0 || FOPawn(Pawn).RightArmDamage > 0 || FOPawn(Pawn).LeftLegDamage > 0 || FOPawn(Pawn).RightLegDamage > 0))
		{
			bShouldBeBought = true;
			bShouldBeRepaired = true;
			FOPawn(Pawn).RepairArmor();
		}
		if (bShouldBeBought)
		{
			if (bShouldBeRepaired || bNoMoneyBack)
				TotalCash = class'Interface_BuyMenuPlus'.Default.RepairPrice;
			if (CriticalReplicationInfo.Money - TotalCash < 0)
				return;
			CriticalReplicationInfo.Money -= TotalCash;
		}
		return;
	}

	if (DesiredWeapon.Default.Unlocked > -1 && CriticalReplicationInfo.Medals[DesiredWeapon.Default.Unlocked] == 0)
		return;

	TotalCash += DesiredWeapon.Default.GunCost;
	if (DesiredWeapon.Default.GunCost > 0 && FOPawn(Pawn).FindWeapon(DesiredWeapon.Default.InventoryGroup) != None)
	{
		if (DesiredWeapon.Default.StaticName ~= Weapon_Base(FOPawn(Pawn).FindWeapon(DesiredWeapon.Default.InventoryGroup)).Default.StaticName && Weapon_Base(FOPawn(Pawn).FindWeapon(DesiredWeapon.Default.InventoryGroup)).GetAmmo(0) != None)
		{
			if (DesiredWeapon.Default.InventoryGroup == 3)
				CopyPrimaryClips = Weapon_Base(FOPawn(Pawn).FindWeapon(DesiredWeapon.Default.InventoryGroup)).GetAmmo(0).NumClips;
			else if (DesiredWeapon.Default.InventoryGroup == 2)
				CopySidearmClips = Weapon_Base(FOPawn(Pawn).FindWeapon(DesiredWeapon.Default.InventoryGroup)).GetAmmo(0).NumClips;
		}
		if (Weapon_Base(FOPawn(Pawn).FindWeapon(DesiredWeapon.Default.InventoryGroup)).BaseWeaponClass != None && FOPawn(Pawn).AttachList[DesiredWeapon.Default.InventoryGroup-1] != None)
		{
			TotalCash -= Weapon_Base(FOPawn(Pawn).FindWeapon(DesiredWeapon.Default.InventoryGroup)).Default.GunCost;
			TotalCash -= Weapon_Base(FOPawn(Pawn).FindWeapon(DesiredWeapon.Default.InventoryGroup)).Default.BaseWeaponClass.Default.GunCost;
		}
		else
		{
			if (DesiredWeapon.Default.InventoryGroup == 4 && FOPawn(Pawn).FindWeapon(4) != None && FOPawn(Pawn).FindWeapon(5) != None)
				TotalCash -= Weapon_Base(FOPawn(Pawn).FindWeapon(DesiredWeapon.Default.InventoryGroup)).Default.GunCost;
			else if ((DesiredWeapon.Default.InventoryGroup != 4) || (DesiredWeapon.Default.InventoryGroup == 4 && FOPawn(Pawn).FindWeapon(5) != None))
				TotalCash -= Weapon_Base(FOPawn(Pawn).FindWeapon(DesiredWeapon.Default.InventoryGroup)).Default.GunCost;
		}
	}

	if (CriticalReplicationInfo.Money - TotalCash < 0 || DesiredWeapon == FOPawn(Pawn).FindInventoryType(DesiredWeapon) || (((DesiredAttachment-1) > 0 && (DesiredAttachment-1) < 3)) &&  DesiredWeapon.Default.GunAttachment[DesiredAttachment-1] == FOPawn(Pawn).FindInventoryType(DesiredWeapon.Default.GunAttachment[DesiredAttachment-1]))
		return;

	if (DesiredWeapon.Default.InventoryGroup == 4 && FOPawn(Pawn).FindWeapon(4) != None && FOPawn(Pawn).FindWeapon(5) == None)
		FOPawn(Pawn).EquipList[DesiredWeapon.Default.InventoryGroup] = DesiredWeapon;
	else
		FOPawn(Pawn).EquipList[DesiredWeapon.Default.InventoryGroup-1] = DesiredWeapon;

	if ((DesiredAttachment-1) >= 0 && (DesiredAttachment-1) < 3 && DesiredWeapon.Default.GunAttachment[DesiredAttachment-1] != None && CriticalReplicationInfo.Money - (TotalCash + DesiredWeapon.Default.GunAttachment[DesiredAttachment-1].Default.GunCost) >= 0)
	{
		FOPawn(Pawn).AttachList[DesiredWeapon.Default.InventoryGroup-1] = DesiredWeapon.Default.GunAttachment[DesiredAttachment-1];
		TotalCash += DesiredWeapon.Default.GunAttachment[DesiredAttachment-1].Default.GunCost;
	}
	else
		FOPawn(Pawn).AttachList[DesiredWeapon.Default.InventoryGroup-1] = None;

	CriticalReplicationInfo.Money -= TotalCash;
	FOPawn(Pawn).SetNewInventory(CopySidearmClips,CopyPrimaryClips);
}

exec function BuyMenu()
{
	if ((Pawn != None) && (FOPawn(Pawn) != None) && (Pawn.Health > 0) && (FOPawn(Pawn).bCanBuy))
	{
		if (Weapon_Base(Pawn.Weapon).GetFM(0) != None && Weapon_Base(Pawn.Weapon).GetFM(0).bIsFiring)
			Pawn.Weapon.StopFire(0);
		if (Weapon_Base(Pawn.Weapon).GetFM(1) != None && Weapon_Base(Pawn.Weapon).GetFM(1).bIsFiring)
			Pawn.Weapon.StopFire(1);

		bFire = 0;
		bAltFire = 0;
		ClientReplaceMenu("FragOps.Interface_BuyMenuPlus");

		if (MyHUD != None && Interface_MainHUD(MyHUD).BriefingScreen != None)
		{
			Interface_MainHUD(MyHUD).bShowBriefing = false;
			Interface_MainHUD(MyHUD).BriefingScreen.bOpenTimed = false;
		}
	}
}

exec function ShowMenu()
{
	// Pause if not already
	if(Level.Pauser == None)
		SetPause(true);

	StopForceFeedback();  // jdf - no way to pause feedback

	// changed to our menu
	ClientOpenMenu("FragOps.Interface_MidGameMenu");
}

simulated function ShowMidTeamMenu()
{
	Player.GUIController.CloseMenu(false);
	ShowTeamMenu();
}

exec function ShowTeamMenu()
{
	if (Level.Pauser == None)
		SetPause(true);

	StopForceFeedback();

	ClientOpenMenu("FragOps.Interface_MidGameMenu");
}

exec function ShowVoteMenu()
{
	if (Level.NetMode != NM_StandAlone && VoteReplicationInfo != none && VoteReplicationInfo.MapVoteEnabled() && PlayerReplicationInfo != None && !PlayerReplicationInfo.bOnlySpectator  && PlayerReplicationInfo.Team != None)
		Player.GUIController.OpenMenu(Player.GUIController.GetPropertyText("MapVotingMenu"));
}

exec function EndRound()
{
	if (PlayerReplicationInfo.bAdmin || IsA('Admin') || Level.NetMode == NM_StandAlone)
		ServerEndRound();
}

function ServerEndRound()
{
	FragOpsGame(Level.Game).EndRound(3);
}

exec function Wire(string WireName, int WireAmount)
{
	ServerWire(caps(WireName), WireAmount);
}

function ServerWire(string WireName, int WireAmount)
{
	local Controller P;

	if (WireName ~= "" || WireAmount < 0 || WireAmount > CriticalReplicationInfo.Money)
		return;

	for (P=Level.ControllerList;P!=None;P=P.NextController)
	{
		if (PlayerController(P) != None && P.bIsPlayer)
		{
			if (P.PlayerReplicationInfo != None && caps(P.PlayerReplicationInfo.PlayerName) ~= WireName && ((IsDead() && FOPlayer(P).IsDead()) || (!IsDead() && !FOPlayer(P).IsDead())))
			{
				if (P.PlayerReplicationInfo.Team.TeamIndex == PlayerReplicationInfo.Team.TeamIndex)
				{
					CriticalReplicationInfo.AddMoney(-WireAmount);
					FOPlayer(P).CriticalReplicationInfo.AddMoney(WireAmount);
				}
				break;
			}
		}
	}
}

exec function Action()
{
	local byte ActionType;
	
	if (Pawn != None && FOPawn(Pawn) != None && !Pawn.IsInState('PlayerAction'))
	{
		if (Weapon_Base(Pawn.Weapon).bScopeZoomed)
			ActionType = 2;

		if (VSize(Pawn.Velocity) < 50 || Pawn.bIsWalking || Pawn.bIsCrouched) 
		{
			if (ActionType != 2)
				ActionType = 1;
		}
		else if (ActionType != 2)
			ActionType = 0;
		else
			return;

		if ((((ActionType == 1 && ActionPoints < GetMaxAP()) || (ActionType == 2 && ActionPoints > 0)) && !Pawn.Weapon.IsFiring()) || (ActionType == 0 && bCanAction))
			ServerAction(ActionType);
	}
}

function ServerAction(byte ActionType)
{
	if (ActionType >= ActionTypes.Length)
		return;

	if (Pawn != None && !Pawn.HeadVolume.bWaterVolume && !Pawn.IsInState('PlayerAction') && ActionPoints >= APReq[ActionType] && ((ActionType > 0 && ((ActionType == 1 && ActionPoints < GetMaxAP()) || (ActionType == 2 && ActionPoints > 0)) && !Pawn.Weapon.IsFiring() && (VSize(Pawn.Velocity) < 50 || ((Pawn.bIsWalking || Pawn.bIsCrouched) && Pawn.Physics != PHYS_Falling))) || ActionType < 1))
	{
		ActionPoints -= APReq[ActionType];
		if (ActionType == 1 && Level.TimeSeconds - LastRestTime > 1.0)
		{
			LastRestTime = Level.TimeSeconds;
			ReceiveLocalizedMessage(class'Interface_APMessage', 4, None, None, None);
		}
		else if (ActionType == 0)
			ReceiveLocalizedMessage(class'Interface_APMessage', 5, None, None, None);
		FOPawn(Pawn).GotoState(ActionTypes[ActionType],'Begin');
	}
}

exec function UnAction()
{
	if (Pawn != None)
		ServerUnAction();
}

function ServerUnAction()
{
	if (Pawn != None && Pawn.IsInState('PlayerResting'))
		FOPawn(Pawn).GotoState('');
}

exec function SetWeaponHand ( string S )
{
	if (caps(S) ~= "LEFT")
		Handedness = -1;
	else if (caps(S) ~= "RIGHT")
		Handedness = 1;
	else if (caps(S) ~= "HIDDEN")
		Handedness = 2;

	SetHand(Handedness);
}

function SetHand(int IntValue)
{
	if (IntValue == 0)
		IntValue = 1;

	class'FOPlayer'.default.Handedness = IntValue;
	class'FOPlayer'.static.StaticSaveConfig();

	Handedness = IntValue;
	if ((Pawn != None) && (Pawn.Weapon != None))
		Pawn.Weapon.SetHand(Handedness);

	ServerSetHandedness(Handedness);
}

// execs
exec function SetName(coerce string S)
{
	if (S ~= "" || mid(S, 0, 1) ~= " ")
		return;

	ChangeName(S);
	UpdateURL("Name", S, true);
	SaveConfig();
}

function ServerSetHandedness(float Hand)
{
	if (Hand == 0)
		Hand = 1;

	Handedness = hand;
	if ((Pawn != None) && (Pawn.Weapon != None))
		Pawn.Weapon.SetHand(Handedness);
}

exec function Use()
{
	if (Level.TimeSeconds - LastUseTime > 0.3 && !IsInState('PreRound'))
	{
		LastUseTime = Level.TimeSeconds;
		ServerUse();
	}
}

exec function UnUse()
{
	ServerUnUse();
}

function ServerUnUse()
{
	if (CurrentObjective == None || IsInState('PreRound') || FragOps_GRI(GameReplicationInfo).bPreRound)
		return;

	GotoState('PlayerWalking');
	ClientGotoState('PlayerWalking','');
}
// end execs

simulated event Destroyed()
{
	Super.Destroyed();
	// so people who get booted still trigger endround checking
	if (Role == ROLE_Authority && Level.NetMode != NM_StandAlone)
		FragOpsGame(Level.Game).CheckEndRound();
	if (CriticalReplicationInfo != None)
		CriticalReplicationInfo.Destroy();
	// kill spectatorcontroller
	if (SpectatorController != None)
		SpectatorController.Destroy();
}

function EndZoom()
{
	bZooming = false;
	FOVAngle = Default.DefaultFOV;
	DesiredFOV = Default.DefaultFOV;
}

function FragOps_LevelInfo GetLevelInfo()
{
	local FragOps_LevelInfo TempLevelInfo;

	foreach AllActors(class'FragOps_LevelInfo',TempLevelInfo)
		return TempLevelInfo;
}

// Possess a pawn
function Possess(Pawn aPawn)
{
	if (PlayerReplicationInfo.bOnlySpectator)
		return;

	// kill spectatorcontroller
	if (SpectatorController != None)
		SpectatorController.Destroy();

	SetBase(None);
	SetRotation(aPawn.Rotation);
	ResetFOV();
	aPawn.PossessedBy(self);
	Pawn = aPawn;
	Pawn.bStasis = false;
	CleanOutSavedMoves();  // don't replay moves previous to possession
	if (Vehicle(Pawn) != None && Vehicle(Pawn).Driver != None)
		PlayerReplicationInfo.bIsFemale = Vehicle(Pawn).Driver.bIsFemale;
	else
		PlayerReplicationInfo.bIsFemale = Pawn.bIsFemale;
	ServerSetHandedness(Handedness);
	ServerSetAutoTaunt(bAutoTaunt);
	Restart();
	if (FOPawn(aPawn) != None)
	{
		VoiceType = FOPawn(aPawn).VoiceType;
		PlayerReplicationInfo.VoiceType = class<VoicePack>(DynamicLoadObject(FOPawn(aPawn).VoiceType,Class'Class'));
	}
	SetViewTarget(Pawn);
	ClientSetViewTarget(Pawn);
	SetLocation(Pawn.Location);
	//SetBase(Pawn);
}

// detach an escaped pawn, but not unpossess it
function DetachPawn()
{
	if (Pawn != None)
	{
		SetLocation(Pawn.Location);
		Pawn.RemoteRole = ROLE_SimulatedProxy;
		Pawn.UnPossessed();
		CleanOutSavedMoves();  // don't replay moves previous to unpossession
		if (Viewtarget == Pawn)
			SetViewTarget(self);
	}
	Pawn = None;
}

function InitialStart()
{
	if (Role == ROLE_Authority)
	{
		CriticalReplicationInfo.bWaitingForTeam = false;
		FragOpsGame(Level.Game).InitializePlayer(self);
	}
}

simulated function TeamSelectMenu()
{
	ClientReplaceMenu("FragOps.Interface_MidGameMenu");
}

simulated function LoginMenu()
{
	if (!bDemoRecording)
		ClientReplaceMenu("FragOps.Interface_LoginMenu");
}

event InitInputSystem()
{
	Super.InitInputSystem();
	LoginMenu();
}

function ServerThrowWeapon()
{
	local Vector TossVel;

	if (Pawn.CanThrowWeapon())
	{
		TossVel = Vector(GetViewRotation());
		TossVel = TossVel * ((Pawn.Velocity Dot TossVel) + 500) + Vect(0,0,200);
		Pawn.TossWeapon(TossVel);
		if (Pawn.Weapon == None)
			ClientSwitchToBestWeapon();
	}
}

simulated function ForcePrevWeapon(weapon ForceWeapon)
{
	if (Pawn == None || Pawn.Inventory == None)
		return;

	Pawn.PendingWeapon = Pawn.Inventory.PrevWeapon(None, ForceWeapon);

	if (Pawn.PendingWeapon != None)
		Pawn.ChangedWeapon();
}

simulated function ForceBestWeapon()
{
	local float rating;

	if (Pawn == None || Pawn.Inventory == None)
		return;

	Pawn.PendingWeapon = Pawn.Inventory.RecommendWeapon(rating);

	StopFiring();

	if (Pawn.PendingWeapon != None)
		Pawn.ChangedWeapon();
}

simulated function ClientReceiveRoundStats(string aMostDamage, int aMostDamageScore, string aMinDamage, int aMinDamageScore, string aAccuracy, string aAccuracyScore, string aHeadShot, int aHeadShotScore, string aOverall, string aOverallScore, string aParseMedals, byte TeamWinner)
{
	local int i;
	local string sMedals;

	if (Interface_MainHUD(MyHUD) != None && Interface_MainHUD(MyHUD).RoundStats != None)
	{
		Interface_MainHUD(MyHUD).RoundStats.MostDamage = aMostDamage;
		Interface_MainHUD(MyHUD).RoundStats.MostDamageScore = aMostDamageScore;
		Interface_MainHUD(MyHUD).RoundStats.LeastDamage = aMinDamage;
		Interface_MainHUD(MyHUD).RoundStats.LeastDamageScore = aMinDamageScore;
		Interface_MainHUD(MyHUD).RoundStats.MostAccurate = aAccuracy;
		Interface_MainHUD(MyHUD).RoundStats.MostAccurateScore = aAccuracyScore;
		Interface_MainHUD(MyHUD).RoundStats.MostHeadshots = aHeadShot;
		Interface_MainHUD(MyHUD).RoundStats.MostHeadshotsScore = aHeadShotScore;
		Interface_MainHUD(MyHUD).RoundStats.BestOverall = aOverall;
		Interface_MainHUD(MyHUD).RoundStats.BestOverallScore = aOverallScore;

		Interface_MainHUD(MyHUD).RoundStats.ImpactMedals.Remove(0,Interface_MainHUD(MyHUD).RoundStats.ImpactMedals.Length);

		// definately wasn't expecting to have to do this =/
		while (aParseMedals != "")
		{
			if (i == 0)
			{
				if (InStr(aParseMedals,",") < 0)
				{
					Interface_MainHUD(MyHUD).RoundStats.ImpactMedals[i] = int(aParseMedals);
					break;
				}
				Interface_MainHUD(MyHUD).RoundStats.ImpactMedals[i] = int(Left(aParseMedals,InStr(aParseMedals,",")));
				sMedals = Mid(aParseMedals, (InStr(aParseMedals,",")+1));
			}
			else
			{
				if (InStr(sMedals,",") < 0)
				{
					Interface_MainHUD(MyHUD).RoundStats.ImpactMedals[i] = int(sMedals);
					break;
				}
				else
				{
					Interface_MainHUD(MyHUD).RoundStats.ImpactMedals[i] = int(Left(sMedals,InStr(sMedals,",")));
					sMedals = Mid(sMedals, (InStr(sMedals,",")+1));
				}
			}
			i++;
		}
		// announce the new stats
		Interface_MainHUD(MyHUD).RoundStats.bNewStats = true;
		Interface_MainHUD(MyHUD).bShowRoundStats = true;
	}
	WinLossAnnounce(TeamWinner);
}

// receive some stats for the round
function ReceivePlayerStats()
{
	ClientReceivePlayerStats(FragOps_PRI(PlayerReplicationInfo).StatsPacket, FragOps_PRI(PlayerReplicationInfo).RoundShots, FragOps_PRI(PlayerReplicationInfo).RoundShotsHit, FragOps_PRI(PlayerReplicationInfo).RoundHeadShots);
}

// print them to the console
simulated function ClientReceivePlayerStats(FragOps_PRI.NetPacket StatPacket, int TotalShots, int TotalHits, int TotalHeadShots)
{
	local string ConsoleString, ColorString;
	local array<FragOps_PRI.DamageCalc> CalcArray;
	local int i, TotalGiven, TotalTaken;
	local bool bComma;

	if (Player == None || Player.Console == None)
		return;

	// header
	Player.Console.Message(chr(27)$chr(255)$chr(255)$chr(90)$"-----------------------",5);

	// first print damage given
	for (i=0;i<ArrayCount(StatPacket.DamageTakers);i++)
	{
		if (StatPacket.DamageTakers[i].AssociatedPRI == None)
			continue;

		StatPacket.DamageTakers[i].Hits = StatPacket.DamageTakers[i].HeadHits + StatPacket.DamageTakers[i].ArmHits + StatPacket.DamageTakers[i].LegHits + StatPacket.DamageTakers[i].BodyHits;
		CalcArray[CalcArray.Length] = StatPacket.DamageTakers[i];
		TotalGiven += StatPacket.DamageTakers[i].Damage;
	}

	if (CalcArray.Length > 0)
	{
		ColorString = chr(27)$chr(20)$chr(230)$chr(20);
		Player.Console.Message(ColorString$"Damage Given ("$TotalGiven$")",5);
		for (i=0;i<CalcArray.Length;i++)
		{
			bComma = false;
			ConsoleString = CalcArray[i].AssociatedPRI.PlayerName$" ("$CalcArray[i].Damage$ " damage in "$CalcArray[i].Hits;
			if (CalcArray[i].Hits == 1)
				ConsoleString = ConsoleString$" hit)";
			else
				ConsoleString = ConsoleString$" hits)";

			ConsoleString = ConsoleString$" -- [";
			if (CalcArray[i].HeadHits > 0)
			{
				ConsoleString = ConsoleString$CalcArray[i].HeadHits$" Head";
				bComma = true;
			}
			if (CalcArray[i].BodyHits > 0)
			{
				if (bComma)
	 				ConsoleString = ConsoleString$", ";

				ConsoleString = ConsoleString$CalcArray[i].BodyHits$" Body";
				bComma = true;
			}
			if (CalcArray[i].ArmHits > 0)
			{
				if (bComma)
	 				ConsoleString = ConsoleString$", ";

				ConsoleString = ConsoleString$CalcArray[i].ArmHits$" Arm";
				bComma = true;
			}
			if (CalcArray[i].LegHits > 0)
			{
				if (bComma)
	 				ConsoleString = ConsoleString$", ";

				ConsoleString = ConsoleString$CalcArray[i].LegHits$" Leg";
			}
			ConsoleString = ConsoleString$"]";
			ConsoleString = chr(27)$chr(100)$chr(230)$chr(100)$ConsoleString;
			Player.Console.Message(ConsoleString,5);		
		}
	}

	// clear array
	CalcArray.Remove(0,CalcArray.Length);

	// now print damage taken
	for (i=0;i<ArrayCount(StatPacket.DamageGivers);i++)
	{
		if (StatPacket.DamageGivers[i].AssociatedPRI == None)
			continue;

		StatPacket.DamageGivers[i].Hits = StatPacket.DamageGivers[i].HeadHits + StatPacket.DamageGivers[i].ArmHits + StatPacket.DamageTakers[i].LegHits + StatPacket.DamageGivers[i].BodyHits;
		CalcArray[CalcArray.Length] = StatPacket.DamageGivers[i];
		TotalTaken += StatPacket.DamageGivers[i].Damage;
	}

	if (CalcArray.Length > 0)
	{
		ColorString = chr(27)$chr(230)$chr(20)$chr(20);
		Player.Console.Message(ColorString$"Damage Taken ("$TotalTaken$")",5);
		for (i=0;i<CalcArray.Length;i++)
		{
			bComma = false;
			ConsoleString = CalcArray[i].AssociatedPRI.PlayerName$" ("$CalcArray[i].Damage$ " damage in "$CalcArray[i].Hits;
			if (CalcArray[i].Hits == 1)
				ConsoleString = ConsoleString$" hit)";
			else
				ConsoleString = ConsoleString$" hits)";

			ConsoleString = ConsoleString$" -- [";
			if (CalcArray[i].HeadHits > 0)
			{
				ConsoleString = ConsoleString$CalcArray[i].HeadHits$" Head";
				bComma = true;
			}
			if (CalcArray[i].BodyHits > 0)
			{
				if (bComma)
	 				ConsoleString = ConsoleString$", ";

				ConsoleString = ConsoleString$CalcArray[i].BodyHits$" Body";
				bComma = true;
			}
			if (CalcArray[i].ArmHits > 0)
			{
				if (bComma)
	 				ConsoleString = ConsoleString$", ";

				ConsoleString = ConsoleString$CalcArray[i].ArmHits$" Arm";
				bComma = true;
			}
			if (CalcArray[i].LegHits > 0)
			{
				if (bComma)
	 				ConsoleString = ConsoleString$", ";

				ConsoleString = ConsoleString$CalcArray[i].LegHits$" Leg";
			}
			ConsoleString = ConsoleString$"]";
			ConsoleString = chr(27)$chr(230)$chr(100)$chr(100)$ConsoleString;
			Player.Console.Message(ConsoleString,5);		
		}
	}

	// now print summary
	ColorString = chr(27)$chr(245)$chr(245)$chr(90);
	ConsoleString = "You fired a total of "$TotalShots$" bullets, of which "$TotalHits;
	if (TotalHits == 1)
		ConsoleString = ConsoleString$" hit its mark.";
	else
		ConsoleString = ConsoleString$" hit their mark.";
	if (TotalHits >= 1)
	{
		ConsoleString = ConsoleString$" "$TotalHeadShots;
		if (TotalHeadShots == 1)
			ConsoleString = ConsoleString$" was a head shot.";
		else
			ConsoleString = ConsoleString$" were head shots.";
	}
	Player.Console.Message(ColorString$ConsoleString,5);
	ConsoleString = "Your overall accuracy was "$int((float(TotalHits)/float(TotalShots))*100)$"%, with head shots at "$int((float(TotalHeadShots)/float(TotalShots))*100)$"%.";
	Player.Console.Message(ColorString$ConsoleString,5);

	// footer
	Player.Console.Message(chr(27)$chr(255)$chr(255)$chr(90)$"-----------------------",5);
}

simulated function ClientPendingWeapon(weapon TempPendingWeapon)
{
	SetPendingWeapon(TempPendingWeapon);
}

function ChangedWeapon()
{
	if (Pawn != None && Pawn.Weapon != None )
	{
		Pawn.Weapon.SetHand(Handedness);
		LastPawnWeapon = Pawn.Weapon.Class;
	}
}

simulated function SetPendingWeapon(weapon TempPendingWeapon)
{
	if (Pawn == None)
	{
		bSetWeapon = true;
		return;
	}
	else if (Pawn.Weapon == None)
	{
		Pawn.PendingWeapon = TempPendingWeapon;
		Pawn.ChangedWeapon();
	}
}
// sound functions
function BuySound()
{
	if (Pawn != None)
		Pawn.PlaySound(Sound'WeaponSounds.BaseGunTech.BReload9',SLOT_None,TransientSoundVolume,,30,,false);
}

function AmmoSound()
{
	if (Pawn != None)
		Pawn.PlaySound(Sound'WeaponSounds.AssaultRifle.SwitchToAssaultRifle',SLOT_None,TransientSoundVolume,,30,,false);
}

simulated function StompSound()
{
	if (ViewTarget != None)
		ViewTarget.PlaySound(sound'FOGameSounds.Menu.MedalStomp',SLOT_None,TransientSoundVolume*1.4,,,,false);
}

simulated function DisableFlash()
{
	if (Pawn != None)
		Pawn.AmbientSound = None;
	//if (bHaveBlur)
	//{
		//FlashBlur.Alpha = 0;
		//RemoveCameraEffect(FlashBlur);
		//bHaveBlur = false;
	//}
}

simulated function MoveOutAnnounce()
{
	if (ViewTarget != None)
		ViewTarget.PlaySound(sound'FOMaleVoices.AmericanExtras.TakeEmDown',SLOT_Talk,TransientSoundVolume*1.8,,,0.94,false);
}

simulated function WinLossAnnounce(byte WinningTeam)
{
	if (ViewTarget == None || CriticalReplicationInfo.bWaitingForTeam)
		return;

	if (PlayerReplicationInfo.Team.TeamIndex == WinningTeam)
		ViewTarget.PlaySound(sound'FOMaleVoices.AmericanExtras.MissionComplete',SLOT_Talk,TransientSoundVolume*1.75,,,0.94,false);
	else
		ViewTarget.PlaySound(sound'FOMaleVoices.AmericanExtras.MissionFailed',SLOT_Talk,TransientSoundVolume*1.75,,,0.94,false);
}

simulated function UpdateFlashSound(float FlashVolumeSize, float InitialFlashMultiple)
{
	local FOPawn FlashPawn;

	// tone down blur towards end of flash
	//if (FlashBlur != None && bHaveBlur)
		//FlashBlur.Alpha = FClamp(FlashVolumeSize-0.08, 0, 1.0);

	// remove if 0
	//if (FlashBlur.Alpha == 0 && bHaveBlur)
	//{
		//bHaveBlur = false;
		//RemoveCameraEffect(FlashBlur);
	//}

	// update sound
	FlashPawn = FOPawn(Pawn);
	if (FlashPawn != None)
	{
		if (FlashPawn.Health <= 0 || InitialFlashMultiple < 0.9)
		{
			FlashPawn.AmbientSound = None;
			return;
		}
		FlashPawn.AmbientSound = Sound'FOWeaponSounds.Misc.FlashTone';
		FlashPawn.SoundVolume = Clamp(FlashVolumeSize*112,0.0,112);
	}

}

// no more announcements, yay
simulated function PlayAnnouncement(sound ASound, byte AnnouncementLevel, optional bool bForce){}

function GetNewTeam(byte TempTeam, byte NewRecord, byte NewBodySkin, byte NewHeadSkin, byte NewTrait, optional byte NewStructures, optional byte NewMechanic, optional byte NewWeapons) 
{
	local int OldTeam;
	local string NewPawnClass;

	if (Pawn != None && Pawn.Health > 0)
	{
		if (PlayerReplicationInfo.Team != None && PlayerReplicationInfo.Team.TeamIndex == 1)
			Pawn.Died(None, class'DamType_TeamChangeBlue', Pawn.Location);
		else
			Pawn.Died(None, class'DamType_TeamChange', Pawn.Location);
	}
	
	SelectedTrait = NewTrait;
	FragOps_PRI(PlayerReplicationInfo).TraitType = NewTrait;

	if (PlayerReplicationInfo.Team != none)
	{
		OldTeam = PlayerReplicationInfo.Team.TeamIndex;
		PlayerReplicationInfo.Team.RemoveFromTeam(self);
	}

	// initialize mesh/skins
	SelectedTeam = Clamp(TempTeam, 0, 1);

	switch(SelectedTeam)
	{
		case 1:
			SelectedRecord = Clamp(NewRecord, 0, (class'Interface_TabPlayerTeam'.Default.SORecord.Length-1));
			HeadSkin = Clamp(NewHeadSkin, 0, (class'Interface_TabPlayerTeam'.Default.SORecord[SelectedRecord].HeadSkin.Length-1));
			BodySkin = Clamp(NewBodySkin, 0, (class'Interface_TabPlayerTeam'.Default.SORecord[SelectedRecord].BodySkin.Length-1));
			NewPawnClass = class'Interface_TabPlayerTeam'.Default.SORecord[Clamp(SelectedRecord, 0, (class'Interface_TabPlayerTeam'.Default.SORecord.Length-1))].AssociatedClass;
			break;

		default:
			SelectedRecord = Clamp(NewRecord, 0, (class'Interface_TabPlayerTeam'.Default.MercRecord.Length-1));
			HeadSkin = Clamp(NewHeadSkin, 0, (class'Interface_TabPlayerTeam'.Default.MercRecord[SelectedRecord].HeadSkin.Length-1));
			BodySkin = Clamp(NewBodySkin, 0, (class'Interface_TabPlayerTeam'.Default.MercRecord[SelectedRecord].BodySkin.Length-1));
			NewPawnClass = class'Interface_TabPlayerTeam'.Default.MercRecord[Clamp(SelectedRecord, 0, (class'Interface_TabPlayerTeam'.Default.MercRecord.Length-1))].AssociatedClass;
			break;
	}

	// set the pawn class
	SetPawnClass(NewPawnClass, NewPawnClass);

	if (OldTeam != SelectedTeam)
	{
		ClientCloseMenu(true,true);
		CriticalReplicationInfo.bLAWCarrier = false;
	}

	FragOpsGame(Level.Game).Teams[SelectedTeam].AddToTeam(self);
	PlayerReplicationInfo.Team = FragOpsGame(Level.Game).Teams[SelectedTeam];
	PlayerReplicationInfo.Team.TeamIndex = SelectedTeam;
	if (!CriticalReplicationInfo.bWaitingForTeam)
		CriticalReplicationInfo.bChangedTeams = true;
	InitialStart();
}

function AdjustView(float DeltaTime)
{
	// adjust FOV for weapon zooming
	if (bZooming)
		DesiredFOV = FClamp(90.0 - (ZoomLevel * 88.0), 1, 170);
	else if (Level.bClassicView && (DesiredFOV == 85) && (DefaultFOV == 85))
		DesiredFOV = 90;

	// set to real FOV again
	if (RealFOV != 0)
		FOVAngle = RealFOV;

	// teleporters affect your FOV, so adjust it back down
	if (FOVAngle != DesiredFOV)
	{
		if (FOVAngle > DesiredFOV)
		{
			if (Pawn != None && Pawn.Weapon != None && Weapon_Base(Pawn.Weapon).WeaponType == Heavy)
				FOVAngle = FOVAngle - FMax(7, 1.0 * DeltaTime * (FOVAngle - DesiredFOV));
			else
				FOVAngle = FOVAngle - FMax(7, 1.3 * DeltaTime * (FOVAngle - DesiredFOV));
		}
		else
			if (Pawn != None && Pawn.Weapon != None && Weapon_Base(Pawn.Weapon).WeaponType == Heavy)
				FOVAngle = FOVAngle - FMin(-7, 1.0 * DeltaTime * (FOVAngle - DesiredFOV));
			else
				FOVAngle = FOVAngle - FMin(-7, 1.3 * DeltaTime * (FOVAngle - DesiredFOV));
		if (Abs(FOVAngle - DesiredFOV) <= 10)
			FOVAngle = DesiredFOV;
	}

	// scoped breathing
	//RealFOV = FOVAngle;
	//if (Pawn != None && Pawn.Weapon != None && Weapon_Base(Pawn.Weapon).bScopeZoomed)
		//FOVAngle = (RealFOV-(0.5*Weapon_Base(Pawn.Weapon).BreathFOV)) + Weapon_Base(Pawn.Weapon).BreathFOV;
}

// disable speech for dead players
function bool AllowVoiceMessage(name MessageType)
{
	if (Level.NetMode == NM_Standalone)
		return true;

	if (IsDead() || Pawn == None)
		return false;

	if (Level.TimeSeconds - OldMessageTime < 3 && OldMessageTime > 0)
	{
		if ((MessageType == 'TAUNT') || (MessageType == 'AUTOTAUNT'))
			return false;
		if (Level.TimeSeconds - OldMessageTime < 1)
			return false;
	}
	if (Level.TimeSeconds - OldMessageTime < 6)
		OldMessageTime = Level.TimeSeconds + 3;
	else
		OldMessageTime = Level.TimeSeconds;
	return true;
}

function SendVoiceMessagePlus(PlayerReplicationInfo Sender, PlayerReplicationInfo Recipient, name messagetype, byte messageID, name broadcasttype, class<VoicePack> VoiceClass)
{
	local Controller P;
		
	if (!AllowVoiceMessage(MessageType))
		return;

	if (VoiceClass == None)
		VoiceClass = class'FragOps.Voice_MercMaleVoice';

	for (P=Level.ControllerList; P!=None; P=P.NextController)
	{
		if (PlayerController(P) != None)
		{  
			if ((broadcasttype == 'GLOBAL') || (Sender.Team.TeamIndex == P.PlayerReplicationInfo.Team.TeamIndex))
				FOPlayer(P).ClientVoiceMessagePlus(Sender, Recipient, messagetype, messageID, VoiceClass);
		}
		else if ((messagetype == 'ORDER') && ((Recipient == None) || (Recipient == P.PlayerReplicationInfo)))
			P.BotVoiceMessage(messagetype, messageID, self);
	}
}

function ClientVoiceMessagePlus(PlayerReplicationInfo Sender, PlayerReplicationInfo Recipient, name messagetype, byte messageID, class<VoicePack> VoiceClass)
{
	local VoicePack V;

	if ((Sender == None) || (VoiceClass == None) || (Player.Console == None))
	        return;

	V = spawn(VoiceClass, self);
	if (V != None)
		V.ClientInitialize(Sender, Recipient, messagetype, messageID);
}

// lock the slash so the server can't update it again
simulated function LockSlash(float NewLockTime)
{
	bSlashLocked = true;
	LastSlashTime = Level.TimeSeconds;
	SlashLockTime = NewLockTime * 0.8;
}

event PlayerTick(float DeltaTime)
{
	Super(PlayerController).PlayerTick(DeltaTime);
	if (Level.NetMode != NM_DedicatedServer && Pawn != None && Pawn.Weapon != None && (Weapon_Base(Pawn.Weapon).WeaponType == Melee || Weapon_Base(Pawn.Weapon).WeaponType == Fists) && !bSlashLocked)
		UpdateSlash(LastSlashDir);
	else if (bSlashLocked && Level.TimeSeconds - LastSlashTime >= SlashLockTime)
		bSlashLocked = false;
	if (CriticalReplicationInfo != None && GameReplicationInfo != None && PlayerReplicationInfo != None && !PlayerReplicationInfo.bOnlySpectator)
	{
		if (!FragOps_GRI(GameReplicationInfo).bPreRound && BeganRound < 5 && bIsPlayer && !PlayerReplicationInfo.bOutOfLives && !CriticalReplicationInfo.bWaitingForTeam)
		{
			BeganRound++;
			GotoState('PlayerWalking');
		}
		else if (Level.Game != None && FragOpsGame(Level.Game).PreRoundTime > 0 && !IsInState('PreRound') && Pawn != None)
			GotoState('PreRound');
	}
	if (Pawn != None && bSetWeapon)
	{
		bSetWeapon = false;
		NewSwitchToBestWeapon();
	}
	if (Level.TimeSeconds - LastCheckTime > 9+Rand(5))
		CheckBadKeyBinds();
}

function CheckBadKeyBinds()
{
	local int i, j;
	local string Key, KeyContent, Alias;
	local Object Obj;
	local array<string> Aliases, NewAliases, AliasSplit;
	local bool bHasDoubleFire;

	// not on server
	if (Level.NetMode == NM_DedicatedServer)
		return;

	LastCheckTime = Level.TimeSeconds;

	// hax0r!
	for (i=0;i<256;i++)
	{
		Key = caps(ConsoleCommand("KEYNAME"@i));
		// capitalize, no sneaky stuff
		KeyContent = caps(ConsoleCommand("KEYBINDING"@Key));

		bHasDoubleFire = (InStr(KeyContent,"ALTFIRE") > -1 && InStr(KeyContent,"|") > -1);
		if (bHasDoubleFire)
			log("doublefire:"@keycontent);

		// no recoil?
		if (InStr(Key,"JOY") < 0 && InStr(Key,"MOUSEX") < 0 && InStr(Key,"MOUSEY") < 0 && (bHasDoubleFire || InStr(KeyContent,"MOUSEX") > -1 || InStr(KeyContent,"MOUSEY") > -1 || InStr(KeyContent,"LOOKUP") > -1 || InStr(KeyContent,"LOOKDOWN") > -1))
		{
			// haha.  Try it again!  This is fun!
			ConsoleCommand("set Input"@Key@"suicide");
		}
	}

	// now aliases. silly hax0r.
	// each alias looks like this: Command="Button bFire | Fire",Alias="Fire"
	Obj = FindObject("Package.Input", class'Object');
        if (Obj != none)
                Split(Obj.GetPropertyText("Aliases"),"),(",Aliases);

	// this finds an offending alias
	for (i=0;i<Aliases.Length;i++)
	{
		Aliases[i] = caps(Aliases[i]);

		bHasDoubleFire = (InStr(Aliases[i],"ALTFIRE") > -1 && InStr(Aliases[i],"|") > -1 && InStr(Aliases[i],"BUTTON BALTFIRE") < 0);
		if (bHasDoubleFire)
			log("doublefire:"@aliases[i]);

		if (bHasDoubleFire || InStr(Aliases[i],"MOUSEX") > -1 || InStr(Aliases[i],"MOUSEY") > -1 || InStr(Aliases[i],"LOOKUP") > -1 || InStr(Aliases[i],"LOOKDOWN") > -1)
		{
			Split(Aliases[i],",",AliasSplit);
			for (j=0;j<AliasSplit.Length;j++)
			{
				if (InStr(AliasSplit[j],"ALIAS=") > -1)
				{
					Alias = Mid(AliasSplit[j], InStr(AliasSplit[j],"ALIAS=")+7);
					Alias = Mid(Alias, 0, InStr(Alias,chr(34)));
					NewAliases[NewAliases.Length] = Repl(Alias, chr(34), "");
				}
			}
		}
	}

	// this now checks if a key is bound to these bad aliases
	for (i=0;i<256;i++)
	{
		Key = caps(ConsoleCommand("KEYNAME"@i));
		// capitalize, no sneaky stuff
		KeyContent = caps(ConsoleCommand("KEYBINDING"@Key));
		for (j=0;j<NewAliases.Length;j++)
		{
			// no recoil?
			if (InStr(Key,"JOY") < 0 && InStr(Key,"MOUSEX") < 0 && InStr(Key,"MOUSEY") < 0 && InStr(KeyContent,NewAliases[j]) > -1)
			{
				// haha.  Try it again!  This is fun!
				ConsoleCommand("set Input"@Key@"suicide");
			}
		}
	}
}

exec function SwitchToBestWeapon()
{
	if (!IsInState('Building') || Pawn == None || Pawn.Weapon == None || Pawn.Weapon.ClientState != WS_ReadyToFire || Weapon_Base(Pawn.Weapon).IsReloading() || (Weapon_GrenadeBase(Pawn.Weapon) != None && Weapon_GrenadeBase(Pawn.Weapon).bForceStay) || (Pawn.Weapon.IsFiring() && Weapon_Base(Pawn.Weapon).GetAmmo(0) != None && Pawn.Weapon.AmmoAmount(0) > 0))
		return;

	NewSwitchToBestWeapon();
}

function NewSwitchToBestWeapon()
{
	local float rating;

	if (Pawn == None || Pawn.Inventory == None)
		return;

	if (Pawn.PendingWeapon == None)
	{
		Pawn.PendingWeapon = Pawn.Inventory.RecommendWeapon(rating);
		if (Pawn.PendingWeapon == Pawn.Weapon)
		    Pawn.PendingWeapon = None;
		if (Pawn.PendingWeapon == None)
			return;
	}

	StopFiring();

	if (Pawn.Weapon == None)
		Pawn.ChangedWeapon();
	else if (Pawn.Weapon != Pawn.PendingWeapon)
		Pawn.Weapon.PutDown();
}

// server calls this to force client to switch
function ClientSwitchToBestWeapon()
{
	NewSwitchToBestWeapon();
}

function ServerChangeLoadout(string LoadoutName)
{
}

simulated function bool DodgingIsEnabled()
{
	return false;
}

function CalcFirstPersonView(out vector CameraLocation, out rotator CameraRotation)
{
	local vector x, y, z;
	local Weapon_Base TWeapon;

	GetAxes(Rotation, x, y, z);

	if (Level.NetMode != NM_DedicatedServer && Pawn != None && Pawn.Weapon != None)
	{
		TWeapon = Weapon_Base(Pawn.Weapon);
		if (TWeapon.bScopeZoomed)
		{
			CameraRotation = Normalize(Rotation + ShakeRot + TWeapon.ActualRot);
			CameraLocation = CameraLocation + Pawn.EyePosition() + Pawn.WalkBob + ShakeOffset.X * x + ShakeOffset.Y * y + ShakeOffset.Z * z + TWeapon.BreathLoc.X * X + TWeapon.BreathLoc.Y * Y + TWeapon.BreathLoc.Z * Z;
		}
		else
		{
			CameraRotation = Normalize(Rotation + ShakeRot); // amb
			CameraLocation = CameraLocation + Pawn.EyePosition() + Pawn.WalkBob + ShakeOffset.X * x + ShakeOffset.Y * y + ShakeOffset.Z * z;
		}
	}
	else
	{
		CameraRotation = Normalize(Rotation + ShakeRot); // amb
		CameraLocation = CameraLocation + Pawn.EyePosition() + Pawn.WalkBob + ShakeOffset.X * x + ShakeOffset.Y * y + ShakeOffset.Z * z;
	}
}

event PlayerCalcView(out actor ViewActor, out vector CameraLocation, out rotator CameraRotation )
{
    local Pawn PTarget;

	if (LastPlayerCalcView == Level.TimeSeconds && CalcViewActor != None && CalcViewActor.Location == CalcViewActorLocation)
	{
		ViewActor	= CalcViewActor;
		CameraLocation	= CalcViewLocation;
		CameraRotation	= CalcViewRotation;
		return;
	}

	// If desired, call the pawn's own special callview
	if (Pawn != None && Pawn.bSpecialCalcView && (ViewTarget == Pawn))
	{
		// try the 'special' calcview. This may return false if its not applicable, and we do the usual.
		if (Pawn.SpecialCalcView(ViewActor, CameraLocation, CameraRotation))
		{
			CacheCalcView(ViewActor,CameraLocation,CameraRotation);
			return;
		}
	}

	if ((ViewTarget == None) || ViewTarget.bDeleteMe)
	{
		if ((Pawn != None) && !Pawn.bDeleteMe)
		    SetViewTarget(Pawn);
		else if (RealViewTarget != None)
		    SetViewTarget(RealViewTarget);
		else
		    SetViewTarget(self);
	}

	ViewActor = ViewTarget;
	CameraLocation = ViewTarget.Location;

	if (ViewTarget == Pawn)
	{
		if (bBehindView) //up and behind
			CalcBehindView(CameraLocation, CameraRotation, CameraDist * Pawn.Default.CollisionRadius);
		else
			CalcFirstPersonView(CameraLocation, CameraRotation);
		return;
	}
	else if (ViewTarget == self)
	{
		CameraRotation = Rotation;
		CacheCalcView(ViewActor,CameraLocation,CameraRotation);
		return;
	}
	else if (ViewTarget.IsA('Projectile') && !bBehindView)
	{
		CameraLocation += (ViewTarget.CollisionHeight) * vect(0,0,1);
		CameraRotation = Rotation;
		CacheCalcView(ViewActor,CameraLocation,CameraRotation);
		return;
	}

	CameraRotation = ViewTarget.Rotation;
	PTarget = Pawn(ViewTarget);

	if (PTarget != None )
	{
		if ((Level.NetMode != NM_DedicatedServer) || (bDemoOwner && (Level.NetMode != NM_Standalone)) && !bBehindView)
		{
			PTarget.SetViewRotation(NewViewRotation);
			CameraRotation = BlendedTargetViewRotation;
			if (Level.NetMode != NM_StandAlone && Level.NetMode != NM_ListenServer)
				PTarget.EyeHeight = NewEyeHeight;

		}
		else if (PTarget.IsPlayerPawn())
			CameraRotation = PTarget.GetViewRotation();

		if (PTarget.bSpecialCalcView && PTarget.SpectatorSpecialCalcView(self, ViewActor, CameraLocation, CameraRotation))
		{
			CacheCalcView(ViewActor, CameraLocation, CameraRotation);
			return;
		}

		if (!bBehindView)
		{
			if (Role < ROLE_Authority)
				CameraLocation += NewEyeHeight*vect(0,0,1) + PTarget.WalkBob;
			else
				CameraLocation += PTarget.EyePosition();
		}
	}
	if (bBehindView)
	{
		CameraLocation = CameraLocation + (ViewTarget.Default.CollisionHeight - ViewTarget.CollisionHeight) * vect(0,0,1);
		CalcBehindView(CameraLocation, CameraRotation, CameraDist * ViewTarget.Default.CollisionRadius);
	}
	CacheCalcView(ViewActor,CameraLocation,CameraRotation);
}

function UpdateRotation(float DeltaTime, float maxPitch)
{
	local rotator NewRotation, ViewRotation;

	if (Level.NetMode != NM_DedicatedServer)
	{
		if (Pawn != None && Pawn.Weapon != None)
		{
			if (IsInState('PreRound'))
				Weapon_Base(Pawn.Weapon).SetTurnPivot(aTurn, 0);
			else if (IsInState('StickyObjective'))
				Weapon_Base(Pawn.Weapon).SetTurnPivot(0,0);
			else
				Weapon_Base(Pawn.Weapon).SetTurnPivot(aTurn, aLookUp);

		}
		LastSlashDir = UpdateSlashDir();
	}

	if (bInterpolating || ((Pawn != None) && Pawn.bInterpolating))
	{	
		ViewShake(deltaTime);
	        return;
	}
	
	// Added FreeCam control for better view control
	if (bFreeCam)
	{
		if (bFreeCamZoom)
			CameraDeltaRad += DeltaTime * 0.25 * aLookUp;
		else if (bFreeCamSwivel)
		{
			CameraSwivel.Yaw += 16.0 * DeltaTime * aTurn;
			CameraSwivel.Pitch += 16.0 * DeltaTime * aLookUp;
		}
		else
		{
			CameraDeltaRotation.Yaw += 32.0 * DeltaTime * aTurn;
			CameraDeltaRotation.Pitch += 32.0 * DeltaTime * aLookUp;
		}
	}
	else
	{
		ViewRotation = Rotation;
		if (Pawn != None && (Pawn.Physics != PHYS_Flying || Pawn.OnLadder != None)) // mmmmm
		{
			// Ensure we are not setting the pawn to a rotation beyond its desired
			if(Pawn.DesiredRotation.Roll < 65535 &&
				(ViewRotation.Roll < Pawn.DesiredRotation.Roll || ViewRotation.Roll > 0))
				ViewRotation.Roll = 0;
			else if(Pawn.DesiredRotation.Roll > 0 &&
				(ViewRotation.Roll > Pawn.DesiredRotation.Roll || ViewRotation.Roll < 65535))
				ViewRotation.Roll = 0;
		}

		DesiredRotation = ViewRotation; //save old rotation

		if (bTurnToNearest != 0)
			TurnTowardNearestEnemy();
		else if (bTurn180 != 0)
			TurnAround();
		else
		{
			TurnTarget = None;
			bRotateToDesired = false;
			bSetTurnRot = false;
			ViewRotation.Yaw += 32.0 * DeltaTime * aTurn;
			ViewRotation.Pitch += 32.0 * DeltaTime * aLookUp;
		}

		if (IsInState('StickyObjective') && bIsPlayer)
			ViewRotation = Rotation;
		else if (IsInState('PreRound') && bIsPlayer)
			ViewRotation.Pitch = 0;
		else if (Pawn != None)
			ViewRotation.Pitch = Pawn.LimitPitch(ViewRotation.Pitch); //amb

		SetRotation(ViewRotation);

		ViewShake(deltaTime);
		ViewFlash(deltaTime);

		NewRotation = ViewRotation;
		//NewRotation.Roll = Rotation.Roll;

		if (!bRotateToDesired && (Pawn != None) && (!bFreeCamera || !bBehindView))
			Pawn.FaceRotation(NewRotation, deltatime);
	}
}

function byte UpdateSlashDir()
{
	if (aTurn < 0.0)
		return 1;
	else if (aTurn > 0.0)
		return 2;
	else
		return 0;
}

function UpdateSlash(byte NewSlashDir)
{
	SlashDirection = NewSlashDir;
	if (Role < ROLE_Authority)
		ServerUpdateSlash(SlashDirection);
}

function ServerUpdateSlash(byte ServerSlashDir)
{
	SlashDirection = ServerSlashDir;
}

state PreRound
{
	function ServerViewNextPlayer()
	{
		if (Pawn != None)
		{
			SetViewTarget(Pawn);
			ClientSetViewTarget(Pawn);
		}
	}

	exec function Action() {}

	function ServerAction(byte ActionType) {}

	exec function Suicide() {}

	exec function Fire(optional float F) {}

	exec function AltFire(optional float F) {}

	function ProcessMove(float DeltaTime, vector NewAccel, eDoubleClickDir DoubleClickMove, rotator DeltaRot)
	{
		if (Pawn == None)
			return;

		Pawn.Acceleration = vect(0,0,0);
		Pawn.SetViewPitch(Rotation.Pitch);
	}

	function PlayerMove(float DeltaTime)
	{
		local vector X,Y,Z;
		local rotator OldRotation, ViewRotation;
		
		if (Pawn != None)
			GetAxes(Pawn.Rotation,X,Y,Z);

		GroundPitch = 0;
		ViewRotation = Rotation;

		if (Pawn != None)
			Pawn.CheckBob(DeltaTime, Y);

		// Update rotation.
		SetRotation(ViewRotation);
		OldRotation = Rotation;
		UpdateRotation(DeltaTime, 0);

		if (Role < ROLE_Authority) // then save this move and replicate it
			ReplicateMove(DeltaTime, vect(0,0,0), DCLICK_None, OldRotation - Rotation);
		else
			ProcessMove(DeltaTime, vect(0,0,0), DCLICK_None, OldRotation - Rotation);
	}

	function BeginState()
	{
		local Actor pActor;

		if (Level.NetMode == NM_DedicatedServer)
			ClientGotoState('PreRound','Begin');
		else if (myHUD != None) // turn off flash grenade effects
		{
			Interface_MainHUD(myHUD).FlashAmount = 0;
			Interface_MainHUD(myHUD).bFlashed = false;

			if (Interface_MainHUD(myHUD).SpectatorWeapon != None)
				Interface_MainHUD(myHUD).SpectatorWeapon.Destroy();
			if (Interface_MainHUD(myHUD).SpectatorWeaponAkimbo != None)
				Interface_MainHUD(myHUD).SpectatorWeaponAkimbo.Destroy();
			DisableFlash();
		}

		CurrentObjective = None;

		APModTemp = 0;
		
		if (Role == ROLE_Authority)
			ActionPoints = GetMaxAP();

		bJumpStatus = false;
		bPressedJump = false;
		GroundPitch = 0;

		bFire = 0;
		bAltFire = 0;
		BeganRound = 0;

		CleanOutSavedMoves();
		CheckBadKeyBinds();

		if (bOpenBriefing && bAutoBriefing && Level.NetMode != NM_DedicatedServer && MyHUD != None)
		{
			bOpenBriefing = false;
			Interface_MainHUD(MyHUD).OpenTimedBriefing();
		}

		foreach DynamicActors(class'Actor', pActor)
	        {
			if (FOPawn(pActor) != None)
			{
				pActor.Reset();
				if (FOPawn(pActor) != None)
					FOPawn(pActor).bInStasis = true;
			}
			else if ((MinigunMuzflash1st(pActor) != None || Effect_MuzSmoke3rd(pActor) != None || Effect_MeshFlash3rd(pActor) != None || Effect_BurstFlash3rd(pActor) != None || Effect_MuzSmoke(pActor) != None || Effect_ShellEjectorFirst(pActor) != None) && pActor.Owner == None)
				pActor.Destroy();
			else if (Effect_BulletShell(pActor) != None || Emitter(pActor) != None || XEmitter(pActor) != None || Effect_LAWChunk(pActor) != None || Effect_WaterRush(pActor) != None || FOAmbientCreatures(pActor) != None || Effect_BreakingGlass(pActor) != None)
				pActor.Reset();
		}

		if (Pawn != None)
		{
			//Pawn.bUpdateEyeHeight = true;
			Pawn.ShouldCrouch(false);
			if (Role == ROLE_Authority)
				Pawn.Velocity = vect(0,0,0);

			SetViewTarget(Pawn);
			bBehindview = false;

			if (ROLE == ROLE_Authority)
			{
				Pawn.bHidden = false;
				FOPawn(Pawn).StopAction();
				FOPawn(Pawn).CheckBuyZones();
			}

			Pawn.SetCollision(true,true,true);

			if (Pawn.Weapon != None)
			{
				if (Weapon_GrenadeBase(Pawn.Weapon) != None)
					Weapon_GrenadeBase(Pawn.Weapon).StopThrowing();
				else
				{
					if (Weapon_Base(Pawn.Weapon).GetFM(0).bIsFiring)
						Pawn.Weapon.StopFire(0);
					if (Weapon_Base(Pawn.Weapon).GetFM(1).bIsFiring)
						Pawn.Weapon.StopFire(1);
				}
			}
		}
	}

	function EndState()
	{
		local FOPawn P;

		bPressedJump = false;
		bJumpStatus = false;
		bBehindView = false;

		if (Role == ROLE_Authority && Pawn != None)
			FOPawn(Pawn).bRestrictDamage = false;

		// allow pawns to make sounds again
		foreach DynamicActors(class'FOPawn',P)
	        {
			if (P != None)
				P.bInStasis = false;
		}
	}
Begin:
}

function ClientTakeFlash(float FlashSize, vector FlashLocation)
{
	if (myHUD != None)
		Interface_MainHUD(MyHUD).DoFlash(FlashSize, FlashLocation);

	if (Level.NetMode == NM_DedicatedServer)
		return;
}

function NotifyTakeHit(pawn InstigatedBy, vector HitLocation, int Damage, class<DamageType> damageType, vector Momentum)
{
	local int iDam;
	local vector AttackLoc;

	Super.NotifyTakeHit(InstigatedBy,HitLocation,Damage,DamageType,Momentum);

	iDam = Clamp(Damage,0,250);
	if (InstigatedBy != None)
		AttackLoc = InstigatedBy.Location;

	NewClientPlayTakeHit(AttackLoc, hitLocation - Pawn.Location, iDam, damageType);

	// stop player from resting
	if (Pawn.IsInState('PlayerResting') && (InstigatedBy == None || InstigatedBy.PlayerReplicationInfo == None || InstigatedBy.PlayerReplicationInfo.Team == None || InstigatedBy.PlayerReplicationInfo.Team != PlayerReplicationInfo.Team))
		ServerUnAction();
	
	if (IsInState('StickyObjective'))
	{
		GotoState('PlayerWalking');
		ClientGotoState('PlayerWalking','');
	}
}

function NewClientPlayTakeHit(vector AttackLoc, vector HitLoc, byte Damage, class<DamageType> damageType)
{
	local vector HitDir;

	if ((myHUD != None) && ((Damage > 0) || bGodMode))
	{
		if (AttackLoc != vect(0,0,0))
			HitDir = Normal(AttackLoc - Pawn.Location);
		else
			HitDir = Normal(HitLoc);
		myHUD.DisplayHit(HitDir, Damage, DamageType);
	}
	HitLoc += Pawn.Location;

	if (bEnableDamageForceFeedback )        // jdf
		ClientPlayForceFeedback("Damage");  // jdf

	if (Level.NetMode == NM_Client )
		Pawn.PlayTakeHit(HitLoc, Damage, damageType);

	if (IsInState('StickyObjective'))
	{
		GotoState('PlayerWalking');
		ClientGotoState('PlayerWalking','');
	}
}

function ClientPlayTakeHit(vector HitLoc, byte Damage, class<DamageType> damageType)
{
	NewClientPlayTakeHit(Location, HitLoc, Damage, DamageType);
}

function PlayWinMessage(bool bWinner)
{
	if (bWinner)
		bDisplayWinner = true;
	else
		bDisplayLoser = true;
}

function ServerUse()
{
	local Actor A;

	if (Role < ROLE_Authority)
		return;

	if (Level.Pauser == PlayerReplicationInfo)
	{
		SetPause(false);
		return;
	}

	if (Pawn == None || !Pawn.bCanUse || IsInState('PreRound'))
		return;
	
	// only "use" items that we're looking at
	foreach Pawn.TouchingActors(class'Actor',A)
		A.UsedBy(Pawn);

	if (Pawn.Base != None )
		Pawn.Base.UsedBy( Pawn );
}

state StickyObjective
{
	exec function Fire(optional float F)
	{
		ServerUnUse();
		Global.Fire(F);
	}

	exec function AltFire(optional float F)
	{
		ServerUnUse();		
		Global.AltFire(F);
	}

	function ProcessMove(float DeltaTime, vector NewAccel, eDoubleClickDir DoubleClickMove, rotator DeltaRot)
	{
		if (Pawn == None)
			return;

		Pawn.Acceleration = NewAccel;
		Pawn.SetViewPitch(Rotation.Pitch);
	}

	function PlayerMove( float DeltaTime )
	{
		local vector X,Y,Z;
		local rotator OldRotation, ViewRotation;
		
		if (Pawn != None)
			GetAxes(Pawn.Rotation,X,Y,Z);

		GroundPitch = 0;
		ViewRotation = Rotation;

		if (Pawn != None)
			Pawn.CheckBob(DeltaTime, Y);

		// Update rotation.
		SetRotation(ViewRotation);
		OldRotation = Rotation;
		UpdateRotation(DeltaTime, 0);

		if (Role < ROLE_Authority) // then save this move and replicate it
			ReplicateMove(DeltaTime, vect(0,0,0), DCLICK_None, OldRotation - Rotation);
		else
			ProcessMove(DeltaTime, vect(0,0,0), DCLICK_None, OldRotation - Rotation);
	}

	exec function Action() {}

	function ServerAction(byte ActionType) {}

	function BeginState()
	{
		if (Role == ROLE_Authority && Level.NetMode != NM_StandAlone)
			ClientGotoState('StickyObjective','Begin');

		if (Role == ROLE_Authority)
		{
			Pawn.Velocity = vect(0,0,0);
			Pawn.Acceleration = vect(0,0,0);
		}

		if (Level.NetMode != NM_DedicatedServer)
			Interface_MainHUD(MyHUD).StartObjective(CurrentObjective.TriggerTime*FOPawn(Pawn).HackMod);
	}		

	function EndState()
	{
		if (!CurrentObjective.bCompleted)
		{
			if (Role == ROLE_Authority)
				CurrentObjective.Cancel();

			if (MyHUD == None)
				return;
		}

		if (Level.NetMode != NM_DedicatedServer && CurrentObjective != None)
			Interface_MainHUD(MyHUD).StopObjective();

		CurrentObjective = None;
	}

Begin:
	Sleep(CurrentObjective.TriggerTime*FOPawn(Pawn).HackMod);
	if (Role == ROLE_Authority)
		CurrentObjective.Completed(Pawn);
	if (Level.NetMode != NM_DedicatedServer)
		CurrentObjective = None;
	if (!FragOps_GRI(GameReplicationInfo).bPreRound)
		GotoState('PlayerWalking');
}

simulated function ClientSetObjective(Actor NewCurObjective)
{
	CurrentObjective = Obj_InteractObjective(NewCurObjective);
}

state Dead
{
	ignores SwitchWeapon, RestartLevel, ClientRestart, Suicide, ThrowWeapon, NotifyPhysicsVolumeChange, NotifyHeadVolumeChange;

	function bool IsDead()
	{
		return true;
	}

	function bool IsSpectating()
	{
		return true;
	}

	function ServerReStartPlayer()
	{
	}

	exec function Fire( optional float F )
	{
		if (!bAllowClick)
			return;

		ServerViewNextPlayer();
		bAllowSwitch = false;
		bAllowBV = true;
	}

	exec function AltFire( optional float F )
	{
		if (!bAllowClick)
			return;

		ServerViewSelf();
		bAllowSwitch = false;
		bAllowBV = true;
	}

	function ServerMove
	(
        float TimeStamp,
        vector Accel,
        vector ClientLoc,
        bool NewbRun,
        bool NewbDuck,
        bool NewbJumpStatus,
        bool NewbDoubleJump,
        eDoubleClickDir DoubleClickMove,
        byte ClientRoll,
        int View,
        optional byte OldTimeDelta,
        optional int OldAccel
	)
	{
        Global.ServerMove(
                    TimeStamp,
                    Accel,
                    ClientLoc,
                    false,
                    false,
                    false,
                    false,
                    DoubleClickMove,
                    ClientRoll,
                    View);
	}
	
	function PlayerMove(float DeltaTime)
	{
		local vector X,Y,Z;
		
		if (!bFrozen)
		{
			if (bPressedJump)
			{
				Fire(0);
				bPressedJump = false;
			}

			if (bWasFirst && ViewTarget != None && Pawn(ViewTarget) != None && LastViewTarget != None && ViewTarget != LastViewTarget)
			{
				bBehindView = false;
				bWasFirst = false;
			}

			// Update view rotation.
			if ((Pawn(ViewTarget) != None) && (Level.NetMode != NM_DedicatedServer) && (!bBehindView))
			{
				if (Pawn(ViewTarget).Health <= 0 || ViewTarget.IsInState('Dying'))
				{
					bWasFirst = true;
					bBehindView = true;
					LastViewTarget = ViewTarget;
				}
				if (Pawn(ViewTarget).bSimulateGravity)
					NewViewRotation.Roll = 0;
				BlendedTargetViewRotation.Pitch = BlendRot(DeltaTime, BlendedTargetViewRotation.Pitch, NewViewRotation.Pitch & 65535);
				BlendedTargetViewRotation.Yaw = BlendRot(DeltaTime, BlendedTargetViewRotation.Yaw, NewViewRotation.Yaw & 65535);
				BlendedTargetViewRotation.Roll = BlendRot(DeltaTime, BlendedTargetViewRotation.Roll, NewViewRotation.Roll & 65535);
				GetAxes(BlendedTargetViewRotation, X,Y,Z);
				Pawn(ViewTarget).CheckBob(DeltaTime,Y);
			}

			UpdateRotation(DeltaTime, 1);

			if (Role < ROLE_Authority) // then save this move and replicate it
				ReplicateMove(DeltaTime, vect(0,0,0), DCLICK_None, rot(0,0,0));
		        else
				ProcessMove(DeltaTime, vect(0,0,0), DCLICK_None, rot(0,0,0));
		}
		else if ((TimerRate <= 0.0) || (TimerRate > 1.0))
			bFrozen = false;

		ViewShake(DeltaTime);
		ViewFlash(DeltaTime);
	}

	function FindGoodView()
	{
		local vector cameraLoc;
		local rotator cameraRot, ViewRotation;
		local int tries, besttry;
		local float bestdist, newdist;
		local int startYaw;
		local actor ViewActor;

		////log("Find good death scene view");
		ViewRotation = Rotation;
		ViewRotation.Pitch = 56000;
		tries = 0;
		besttry = 0;
		bestdist = 0.0;
		startYaw = ViewRotation.Yaw;

		for (tries=0; tries<16; tries++)
		{
			cameraLoc = ViewTarget.Location;
			SetRotation(ViewRotation);
			PlayerCalcView(ViewActor, cameraLoc, cameraRot);
			newdist = VSize(cameraLoc - ViewTarget.Location);
			if (newdist > bestdist)
			{
				bestdist = newdist;
				besttry = tries;
			}
			ViewRotation.Yaw += 4096;
		}

		ViewRotation.Yaw = startYaw + besttry * 4096;
		SetRotation(ViewRotation);
	}

	function Timer()
	{
		bPressedJump = false;
		bAllowClick = true;
	}

	function BeginState()
	{
		local Actor A;
		
		if ( (Pawn != None) && (Pawn.Controller == self) )
			Pawn.Controller = None;
		if (bBuyMenuOpen)
			ClientCloseMenu(true,true);
		DisableFlash();
		bAllowSwitch = true;
		bAllowClick = false;
		bAllowBV = false;
		EndZoom();
		FOVAngle = DesiredFOV;
		Pawn = None;
		Enemy = None;
		bBehindView = true;
		bJumpStatus = false;
		bPressedJump = false;
		//bBlockCloseCamera = true;
		bValidBehindCamera = false;
		FindGoodView();
		SetTimer(1.0, false);
		StopForceFeedback();
		ClientPlayForceFeedback("Damage");  // jdf
		CleanOutSavedMoves();
		if (Viewport(Player) != None )
			ForEach DynamicActors(class'Actor',A)
				A.NotifyLocalPlayerDead(self);
	}

	function EndState()
	{
		EndZoom();
		FOVAngle = DesiredFOV;
		bBlockCloseCamera = false;
		CleanOutSavedMoves();
		Velocity = vect(0,0,0);
		Acceleration = vect(0,0,0);
	        if (Pawn != None)
				bBehindView = false;
		bPressedJump = false;
		if (myHUD != None)
			myHUD.bShowScoreBoard = false;

		StopViewShaking();
	}
Begin:
}

state JustJoined extends Dead
{
	exec function Fire(optional float F)
	{
		ServerViewNextPlayer();
		bAllowSwitch = false;
		bAllowBV = true;
	}

	exec function AltFire(optional float F)
	{
		ServerViewSelf();
	}
	
	function BeginState()
	{
		if (Level.NetMode == NM_DedicatedServer)
			ClientGotoState('JustJoined','Begin');

		EndZoom();
		FOVAngle = DesiredFOV;
		Pawn = None;
		Enemy = None;
		bBehindView = false;
		bJumpStatus = false;
		bPressedJump = false;
		bValidBehindCamera = false;
		StopForceFeedback();
		CleanOutSavedMoves();
		FindGoodView();
	}
	
Begin:
	if (Role == ROLE_Authority)
		Fire();
}

auto state PlayerWaiting
{
	exec function Fire(optional float F)
	{
	}
}

function ServerRestartPlayer()
{}

state Escaped extends JustJoined
{
	exec function Fire(optional float F)
	{
		ServerViewNextPlayer();
	}

	exec function AltFire(optional float F)
	{
		ServerViewSelf();
	}

	function BeginState()
	{
		bAllowClick = false;
		EndZoom();
		FOVAngle = DesiredFOV;
		Enemy = None;
		bBehindView = true;
		bJumpStatus = false;
		bPressedJump = false;
		bValidBehindCamera = false;
		StopForceFeedback();
		CleanOutSavedMoves();
	}

	function EndState()
	{
		bBlockCloseCamera = false;
		CleanOutSavedMoves();
		Velocity = vect(0,0,0);
		Acceleration = vect(0,0,0);
		bBehindView = false;
		bPressedJump = false;
		if (myHUD != None)
			myHUD.bShowScoreBoard = false;
		OldPawn = None;
	}

Begin:
	if (Level.Netmode != NM_DedicatedServer)
	{
		if (Pawn != None)
		{
			FOPawn(OldPawn).bEscaped = true;
			FOPawn(OldPawn).EscapedController = self;
		}
		else if (OldPawn != None)
		{
			FOPawn(OldPawn).bEscaped = true;
			FOPawn(OldPawn).EscapedController = self;
		}
	}
	if (Role == ROLE_Authority)
	{
		OldPawn = Pawn;
		OldPawn.Velocity = vect(0,0,0);
		OldPawn.Acceleration = vect(0,0,0);
		DetachPawn();
	}
	sleep(1.0);
	Pawn = None;
	if (Role == ROLE_Authority)
	{
		OldPawn.SetOwner(self); // stay relevant!
		OldPawn.bAlwaysRelevant = true; // just in case!
		ServerViewNextPlayer();
	}
}

state PlayerWalking
{
ignores SeePlayer, HearNoise;

	function bool NotifyPhysicsVolumeChange( PhysicsVolume NewVolume )
	{
		if (NewVolume.bWaterVolume)
			GotoState(Pawn.WaterMovementState);
		return false;
	}

	function ProcessMove(float DeltaTime, vector NewAccel, eDoubleClickDir DoubleClickMove, rotator DeltaRot)
	{
		local vector OldAccel;
		local bool OldCrouch;

		if (Pawn == None)
			return;

		OldAccel = Pawn.Acceleration;
		Pawn.Acceleration = NewAccel;
		if (bPressedJump)
			Pawn.DoJump(bUpdating);

		Pawn.SetViewPitch(Rotation.Pitch);

		if (Pawn.Physics != PHYS_Falling)
		{
			OldCrouch = Pawn.bWantsToCrouch;
			if (bDuck == 0)
				Pawn.ShouldCrouch(false);
			else if (Pawn.bCanCrouch)
				Pawn.ShouldCrouch(true);
		}
	}

	function PlayerMove(float DeltaTime)
	{
		local vector X,Y,Z, NewAccel;
		local rotator OldRotation, ViewRotation;
		local bool  bSaveJump;
		local float TempForward;

		if (Pawn == None)
		{
			GotoState('Dead'); // this was causing instant respawns in mp games
			return;
		}

		GetAxes(Pawn.Rotation,X,Y,Z);

		// Update acceleration.
		if (aForward < 0)
		{
			if (!FOPawn(Pawn).bSprinting)
				TempForward = aForward * 0.85;
			else
				TempForward = aForward * 0.7;
		}
		else
			TempForward = aForward;

		if (!FOPawn(Pawn).bSprinting)
			NewAccel = TempForward*X + (aStrafe*0.95)*Y;
		else
			NewAccel = TempForward*X + (aStrafe*0.7)*Y;
		NewAccel.Z = 0;
		if (VSize(NewAccel) < 1.0)
			NewAccel = vect(0,0,0);

		GroundPitch = 0;
		ViewRotation = Rotation;

		if (Pawn.Physics == PHYS_Walking)
		{
			// tell pawn about any direction changes to give it a chance to play appropriate animation
			// if walking, look up/down stairs - unless player is rotating view
			if ((bLook == 0) && (((Pawn.Acceleration != Vect(0,0,0)) && bSnapToLevel) || !bKeyboardLook))
			{
				if (bLookUpStairs || bSnapToLevel)
				{
					GroundPitch = FindStairRotation(deltaTime);
					ViewRotation.Pitch = GroundPitch;
				}
			}
		}
		Pawn.CheckBob(DeltaTime, Y);

		// Update rotation.
		SetRotation(ViewRotation);
		OldRotation = Rotation;
		UpdateRotation(DeltaTime, 1);
		bDoubleJump = false;

		bCanAction = (aForward != 0 || aStrafe != 0);

		if (bPressedJump && Pawn.CannotJumpNow())
		{
			bSaveJump = true;
			bPressedJump = false;
		}
		else
			bSaveJump = false;

		if (Role < ROLE_Authority) // then save this move and replicate it
			ReplicateMove(DeltaTime, NewAccel, DCLICK_None, OldRotation - Rotation);
		else
			ProcessMove(DeltaTime, NewAccel, DCLICK_None, OldRotation - Rotation);
		bPressedJump = bSaveJump;
	}

	function BeginState()
	{
		DoubleClickDir = DCLICK_None;
		bPressedJump = false;
		GroundPitch = 0;
		if (Pawn != None)
		{
			Pawn.ShouldCrouch(false);
			if (Pawn.Physics != PHYS_Falling && Pawn.Physics != PHYS_Karma) // FIXME HACK!!!
				Pawn.SetPhysics(PHYS_Walking);
		}
	}

	function EndState()
	{
		GroundPitch = 0;
		if (Pawn != None && bDuck == 0)
			Pawn.ShouldCrouch(false);
	}
}

function ServerMove
(
    float TimeStamp,
    vector InAccel,
    vector ClientLoc,
    bool NewbRun,
    bool NewbDuck,
    bool NewbJumpStatus,
    bool NewbDoubleJump,
    eDoubleClickDir DoubleClickMove,
    byte ClientRoll,
    int View,
    optional byte OldTimeDelta,
    optional int OldAccel
)
{
    local float DeltaTime, clientErr, OldTimeStamp;
    local rotator DeltaRot, Rot, ViewRot;
    local vector Accel, LocDiff;
    local int maxPitch, ViewPitch, ViewYaw;
    local bool NewbPressedJump, OldbRun, OldbDoubleJump;
    local eDoubleClickDir OldDoubleClickMove;

    // If this move is outdated, discard it.
    if ( CurrentTimeStamp >= TimeStamp )
        return;

	if ( AcknowledgedPawn != Pawn )
	{
		OldTimeDelta = 0;
		InAccel = vect(0,0,0);
		GivePawn(Pawn);
	}

    // if OldTimeDelta corresponds to a lost packet, process it first
    if (  OldTimeDelta != 0 )
    {
        OldTimeStamp = TimeStamp - float(OldTimeDelta)/500 - 0.001;
        if ( CurrentTimeStamp < OldTimeStamp - 0.001 )
        {
            // split out components of lost move (approx)
            Accel.X = OldAccel >>> 23;
            if ( Accel.X > 127 )
                Accel.X = -1 * (Accel.X - 128);
            Accel.Y = (OldAccel >>> 15) & 255;
            if ( Accel.Y > 127 )
                Accel.Y = -1 * (Accel.Y - 128);
            Accel.Z = (OldAccel >>> 7) & 255;
            if ( Accel.Z > 127 )
                Accel.Z = -1 * (Accel.Z - 128);
            Accel *= 20;

            OldbRun = ( (OldAccel & 64) != 0 );
            OldbDoubleJump = ( (OldAccel & 32) != 0 );
            NewbPressedJump = ( (OldAccel & 16) != 0 );
            if ( NewbPressedJump )
                bJumpStatus = NewbJumpStatus;
            switch (OldAccel & 7)
            {
                case 0:
                    OldDoubleClickMove = DCLICK_None;
                    break;
                case 1:
                    OldDoubleClickMove = DCLICK_Left;
                    break;
                case 2:
                    OldDoubleClickMove = DCLICK_Right;
                    break;
                case 3:
                    OldDoubleClickMove = DCLICK_Forward;
                    break;
                case 4:
                    OldDoubleClickMove = DCLICK_Back;
                    break;
            }
            //log("Recovered move from "$OldTimeStamp$" acceleration "$Accel$" from "$OldAccel);
            OldTimeStamp = FMin(OldTimeStamp, CurrentTimeStamp + MaxResponseTime);
            MoveAutonomous(OldTimeStamp - CurrentTimeStamp, OldbRun, (bDuck == 1), NewbPressedJump, OldbDoubleJump, OldDoubleClickMove, Accel, rot(0,0,0));
			CurrentTimeStamp = OldTimeStamp;
        }
    }

    // View components
    ViewPitch = View/32768;
    ViewYaw = 2 * (View - 32768 * ViewPitch);
    ViewPitch *= 2;
    // Make acceleration.
    Accel = InAccel * 0.1;

    NewbPressedJump = (bJumpStatus != NewbJumpStatus);
    bJumpStatus = NewbJumpStatus;

    // Save move parameters.
    DeltaTime = FMin(MaxResponseTime,TimeStamp - CurrentTimeStamp);

    CurrentTimeStamp = TimeStamp;
    ServerTimeStamp = Level.TimeSeconds;
    ViewRot.Pitch = ViewPitch;
    ViewRot.Yaw = ViewYaw;
    ViewRot.Roll = 0;

    if ( NewbPressedJump || (InAccel != vect(0,0,0)) )
		LastActiveTime = Level.TimeSeconds;

	if ( Pawn == None || Pawn.bServerMoveSetPawnRot )
		SetRotation(ViewRot);

	if ( AcknowledgedPawn != Pawn )
		return;

    if ( (Pawn != None) && Pawn.bServerMoveSetPawnRot )
    {
        Rot.Roll = 256 * ClientRoll;
        Rot.Yaw = ViewYaw;
        if ( (Pawn.Physics == PHYS_Swimming) || (Pawn.Physics == PHYS_Flying) )
            maxPitch = 2;
        else
            maxPitch = 0;
        if ( (ViewPitch > maxPitch * RotationRate.Pitch) && (ViewPitch < 65536 - maxPitch * RotationRate.Pitch) )
        {
            If (ViewPitch < 32768)
                Rot.Pitch = maxPitch * RotationRate.Pitch;
            else
                Rot.Pitch = 65536 - maxPitch * RotationRate.Pitch;
        }
        else
            Rot.Pitch = ViewPitch;
        DeltaRot = (Rotation - Rot);
        Pawn.SetRotation(Rot);
    }

    // Perform actual movement
    if ( (Level.Pauser == None) && (DeltaTime > 0) )
        MoveAutonomous(DeltaTime, NewbRun, NewbDuck, NewbPressedJump, NewbDoubleJump, DoubleClickMove, Accel, DeltaRot);

    // Accumulate movement error.
    if ( ClientLoc == vect(0,0,0) )
		return;		// first part of double servermove
    else if ( Level.TimeSeconds - LastUpdateTime > 0.3 )
        ClientErr = 10000;
    else if ( Level.TimeSeconds - LastUpdateTime > 180.0/Player.CurrentNetSpeed )
    {
        if ( Pawn == None )
            LocDiff = Location - ClientLoc;
        else
            LocDiff = Pawn.Location - ClientLoc;
        ClientErr = LocDiff Dot LocDiff;
    }

    // If client has accumulated a noticeable positional error, correct him.
    if ( ClientErr > 3 )
    {
        if ( Pawn == None )
        {
            PendingAdjustment.newPhysics = Physics;
            PendingAdjustment.NewLoc = Location;
            PendingAdjustment.NewVel = Velocity;
        }
        else
        {
            PendingAdjustment.newPhysics = Pawn.Physics;
            PendingAdjustment.NewVel = Pawn.Velocity;
            PendingAdjustment.NewBase = Pawn.Base;
            if ( (Mover(Pawn.Base) != None) || (Vehicle(Pawn.Base) != None) )
                PendingAdjustment.NewLoc = Pawn.Location - Pawn.Base.Location;
            else
                PendingAdjustment.NewLoc = Pawn.Location;
            PendingAdjustment.NewFloor = Pawn.Floor;
        }
	//log(" Client Error at "$TimeStamp$" is "$ClientErr$" with acceleration "$Accel$" LocDiff "$LocDiff$" Physics "$Pawn.Physics);
        LastUpdateTime = Level.TimeSeconds;

		PendingAdjustment.TimeStamp = TimeStamp;
		PendingAdjustment.newState = GetStateName();
    }
	//log("Server moved stamp "$TimeStamp$" location "$Pawn.Location$" Acceleration "$Pawn.Acceleration$" Velocity "$Pawn.Velocity);
}
state PlayerClimbing
{
ignores SeePlayer, HearNoise, Bump;

	function bool NotifyPhysicsVolumeChange( PhysicsVolume NewVolume )
	{
		if (NewVolume.bWaterVolume)
			GotoState(Pawn.WaterMovementState);
		else
			GotoState(Pawn.LandMovementState);
		return false;
	}

	function ProcessMove(float DeltaTime, vector NewAccel, eDoubleClickDir DoubleClickMove, rotator DeltaRot)
	{
		if (NewAccel == vect(0,0,0))
		{
			if (VSize(Pawn.Velocity) < 2.0)
				Pawn.Velocity = vect(0,0,0);
			else
				Pawn.Velocity *= FMin(DeltaTime*22, 0.9);
		}

		// more Z enforcement checks, hoping to figure this out soon
		if (NewAccel.Z == 0 && NewAccel != vect(0,0,0))
		{
			if (Abs(Pawn.Velocity.Z) < 2.0)
				Pawn.Velocity.Z = 0;
			else
				Pawn.Velocity.Z *= FMin(DeltaTime*16, 0.9);
		}

		if (NewAccel != Pawn.Acceleration)
			Pawn.Acceleration = NewAccel;

		// server limit velocity
		Pawn.Velocity.Z = FClamp(Pawn.Velocity.Z, -214.72, 214.72);
		Pawn.SetViewPitch(Rotation.Pitch);

		if (bPressedJump && (DoubleClickMove == DCLICK_Left || DoubleClickMove == DCLICK_Right))
		{
			// ladder jumping
			FOPawn(Pawn).LadderLeap(DoubleClickMove);
			if (Pawn.Physics == PHYS_Falling)
				GotoState('PlayerWalking');
		}
	}

	function PlayerMove(float DeltaTime)
	{
		local vector X,Y,Z, NewAccel, TempClimbDir;
		local eDoubleClickDir DoubleClickMove;
		local rotator OldRotation, ViewRotation;
		local bool bSaveJump;

		OldRotation = Rotation;
		OldRotation.Roll = 0;
		GetAxes(OldRotation,X,Y,Z);

		// Update acceleration.
		if (Pawn.OnLadder != None)
		{
			// ladder jumping
			if (aStrafe > 0)
				DoubleClickMove = DCLICK_Right;
			else if (aStrafe < 0)
				DoubleClickMove = DCLICK_Left;

			TempClimbDir = Pawn.OnLadder.ClimbDir;

			// reset Z accel after strafe, kills ladder launching
			NewAccel = (aStrafe*0.75)*Y;
			NewAccel.Z = 0;

			NewAccel += (aForward*1.15)*X; 
			NewAccel += (aUP*1.15)*TempClimbDir;
		}
		else
			NewAccel = aForward*X + aStrafe*Y;

		// limit velocity, no speedhackers
		NewAccel.Z = FClamp(NewAccel.Z, -7000, 7000);

		if (VSize(NewAccel) < 1.0)
			NewAccel = vect(0,0,0);

		ViewRotation = Rotation;

		// Update rotation.
		SetRotation(ViewRotation);
		OldRotation = Rotation;
		UpdateRotation(DeltaTime, 1);

		if (Role < ROLE_Authority) // then save this move and replicate it
			ReplicateMove(DeltaTime, NewAccel, DoubleClickMove, OldRotation - Rotation);
		else
			ProcessMove(DeltaTime, NewAccel, DoubleClickMove, OldRotation - Rotation);

		bPressedJump = bSaveJump;
	}

	function BeginState()
	{
		Pawn.ShouldCrouch(false);
		bPressedJump = false;
	}

	function EndState()
	{
		if (Pawn != None)
			Pawn.ShouldCrouch(false);
	}
}

function ServerViewNextPlayer()
{
	local Controller C, Pick;
	local bool bFound, bWasSpec;

	if (IsInState('PreRound'))
	{
		if (Pawn != None && ViewTarget != Pawn && Pawn.Health > 0)
		{
			SetViewTarget(Pawn);
			ClientSetViewTarget(Pawn);
			bBehindView = false;
			ClientSetBehindView(false);
		}
		return;
	}

	bWasSpec = !bBehindView && (ViewTarget != Pawn) && (ViewTarget != self);
    
	// view next player
	for (C=Level.ControllerList; C!=None; C=C.NextController)
	{
		if (FragOpsGame(Level.Game).CanSpectate(self,false,C) && C != None && C != self && C.Pawn != None && C.Pawn.Health > 0)
		{
			if (Pick == None)
				Pick = C;
			if (bFound)
			{
				Pick = C;
				break;
			}
			else
				bFound = ((RealViewTarget == C.Pawn) || (ViewTarget == C.Pawn));
		}
	}

	if (Pawn != None && Pawn.Health > 0)
	{
		SetViewTarget(Pawn);
		ClientSetViewTarget(Pawn);
	}
	else if (Pick != None && Pick.Pawn != None)
	{
		if (SpectatorController == None)
			SpectatorController = spawn(class'FOSpectatorController',self);
		SpectatorController.InitializeFor(FOPawn(Pick.Pawn));
		SetViewTarget(Pick.Pawn);
		ClientSetViewTarget(Pick.Pawn);
	}
	if ((ViewTarget == self) || (Pawn != None && Pawn.Health > 0 && ViewTarget == Pawn) || bWasSpec || (Pawn(ViewTarget) != None && ViewTarget != Pawn && !FragOps_GRI(GameReplicationInfo).bAllowBehindView))
		bBehindView = false;
	else if (FragOps_GRI(GameReplicationInfo).bAllowBehindView)
		bBehindView = true; //bChaseCam;

	ClientSetBehindView(bBehindView);
}

event ClientSetViewTarget(Actor a)
{
	local bool bNewViewTarget;
	local rotator TempRotator;

	if (A == None)
	{
		if (ViewTarget != self)
			SetLocation(CalcViewLocation);
		else if (ViewTarget != Pawn && ViewTarget != self)
			SetBase(ViewTarget);
		ServerVerifyViewTarget();
		if (ViewTarget != Pawn && Pawn(A) != None)
		{
			TempRotator = A.Rotation;
			TempRotator.Roll = 0;
			SetRotation(TempRotator);
		}
	}
	else
	{
		bNewViewTarget = (ViewTarget != a);
		SetViewTarget(a);
		if (bNewViewTarget)
			a.POVChanged(self, false);
	}
}

function ServerViewSelf()
{
	if (!FragOpsGame(Level.Game).bAllowFloatCam && !PlayerReplicationInfo.bOnlySpectator)
		return;

	bBehindView = false;
	ClientSetBehindview(false);
	SetViewTarget(self);
	ClientSetViewTarget(self);
	ClientMessage(OwnCamera, 'Event');
}

function ClientRestart(Pawn RestartPawn)
{
	bBehindView = false;
	Super.ClientRestart(RestartPawn);
	if (myHUD != None)
	{
		if (Interface_MainHUD(myHUD).SpectatorWeapon != None)
			Interface_MainHUD(myHUD).SpectatorWeapon.Destroy();
		if (Interface_MainHUD(myHUD).SpectatorWeaponAkimbo != None)
			Interface_MainHUD(myHUD).SpectatorWeaponAkimbo.Destroy();
	}
	ServerSetHandedness(Handedness);
	ShakeOffset = vect(0,0,0);
	ShakeRot = rot(0,0,0);
}

function HandleAmmoPickup(Pickup pick)
{
	ReceiveLocalizedMessage(pick.MessageClass,1,,,pick.class);
}

function ReceiveKillMessage(class<DamageType> KillDamType, PlayerReplicationInfo Killer, PlayerReplicationInfo Killed)
{
	if (MyHUD != None && Interface_MainHUD(MyHUD) != None)
		Interface_MainHUD(MyHUD).AddKillMessage(KillDamType,Killer,Killed);
}

event TeamMessage(PlayerReplicationInfo PRI, coerce string S, name Type)
{
	local string c;

	if (PlayerReplicationInfo == None || Player == None || Player.Console == None || S ~= "")
		return;
	
	if (MyHUD != None)
		MyHUD.Message(PRI, S, Type);

	if ((Type == 'Say' || Type == 'TeamSay') && PRI != None)
		S = PRI.PlayerName$": "$S;

	if (PRI != None && PRI.Team != None)
	{
		if (PRI.Team.TeamIndex == 0)
		{
			if (!FragOps_PRI(PRI).bEscaped && !PRI.bOutOfLives)
				c = chr(27)$chr(250)$chr(65)$chr(65);
			else
				c = chr(27)$chr(175)$chr(25)$chr(25);
		}
		else if (PRI.Team.TeamIndex == 1)
		{
			if (!FragOps_PRI(PRI).bEscaped && !PRI.bOutOfLives)
				c = chr(27)$chr(80)$chr(100)$chr(240);
			else
				c = chr(27)$chr(55)$chr(75)$chr(165);
		}
		else
			c = "";
	}

	if (PlayerReplicationInfo == None || PRI == None)
	{
		Player.Console.Chat(c$s, 6.0, PRI);
		return;
	}

	if (!IsDead() && (PRI.bOutOfLives || FragOps_PRI(PRI).bEscaped))
		return;

	Player.Console.Chat(c$s, 6.0, PRI);
}

function ViewFlash(float DeltaTime)
{
	local vector goalFog;
	local float goalscale, delta, Step;
	local PhysicsVolume ViewVolume;

	delta = FMin(0.1, DeltaTime);
	goalScale = 1; // + ConstantGlowScale;
	goalFog = vect(0,0,0); // ConstantGlowFog;

	if (bBehindView || IsDead())
		ViewVolume = Level.GetPhysicsVolume(CalcViewLocation);
	else if (Pawn != None)
		ViewVolume = Pawn.HeadVolume;

	if (ViewVolume != None)
	{
		goalScale += ViewVolume.ViewFlash.X;
		goalFog += ViewVolume.ViewFog;
	}

	Step = 0.7 * delta * ScreenFlashScaling;
	FlashScale.X = UpdateFlashComponent(FlashScale.X,step,goalScale);
	FlashScale = FlashScale.X * vect(1,1,1);

	FlashFog.X = UpdateFlashComponent(FlashFog.X,step,goalFog.X);
	FlashFog.Y = UpdateFlashComponent(FlashFog.Y,step,goalFog.Y);
	FlashFog.Z = UpdateFlashComponent(FlashFog.Z,step,goalFog.Z);
}

//=============================================================================
// misc functions

function StopFiring()
{
}

simulated function rotator GetAim()
{
	if (bBehindView)
		return Pawn.Rotation;
	else
		return Rotation;
}

function PawnDied(Pawn P)
{
	if (Pawn != P)
		return;

	bBehindview = true;
	Super.PawnDied(P);
}

simulated function float RateWeapon(Weapon w)
{
    return w.Default.Priority;
}

simulated function PlayBeepSound()
{
	if (ViewTarget != None)
		ViewTarget.PlaySound(sound'MenuSounds.selectJ', SLOT_None,,,,,false);
}

function bool AutoTaunt()
{
	return bAutoTaunt;
}

function bool DontReuseTaunt(int T)
{
	if (T == LastTaunt)
		return true;

	if (T == Level.LastTaunt[0] || T == Level.LastTaunt[1])
		return true;

	LastTaunt = T;

	Level.LastTaunt[1] = Level.LastTaunt[0];
	Level.LastTaunt[0] = T;

	return false;
}

// epic, if you want the redeemer shake to work, you probably need this function =P
function ClientShake(vector shRotMag, vector shRotRate, float shRotTime, vector shOffsetMag, vector shOffsetRate, float shOffsetTime)
{
	ShakeView(shRotMag, shRotRate, shRotTime, shOffsetMag, shOffsetRate, shOffsetTime);
}

function LocalShake(vector shRotMag, vector shRotRate, float shRotTime, vector shOffsetMag, vector shOffsetRate, float shOffsetTime)
{
	ShakeView(shRotMag, shRotRate, shRotTime, shOffsetMag, shOffsetRate, shOffsetTime);
}

function AddAP(int APToAdd)
{
	ActionPoints = Clamp(ActionPoints + APToAdd, 0, GetMaxAP());
}

function int GetMaxAP()
{
	if (Pawn != None && FOPawn(Pawn) != None)
		return Max(0, (APBaseLimit + FragOps_PRI(PlayerReplicationInfo).APModPerm + APModTemp + FOPawn(Pawn).APArmorMod + FOPawn(Pawn).APMod));
	else
		return Max(0, (APBaseLimit + FragOps_PRI(PlayerReplicationInfo).APModPerm + APModTemp));
}

function ModAP(int APToMod)
{
	APModTemp += APToMod; 
	if (ActionPoints > GetMaxAP())
		ActionPoints = GetMaxAP();
}

function ClientSetWeaponViewShake(Bool B)
{
}

defaultproperties
{
	PawnClass=Class'FragOps.FOPawn'
	InputClass=Class'FragOps.FOPlayerInput'
	PlayerReplicationInfoClass=Class'FragOps.FragOps_PRI'
	TeamBeaconTexture=Texture'TeamSymbols.TeamBeaconT'
	bAdrenalineEnabled=False
	bCenterView=False
	bAllowSwitch=true
	BeganRound=0
	WaitForPreRound=0
	ScreenFlashScaling=+1.0
	InitialState=JustJoined

	// action system
	APBaseLimit=2
	ActionPoints=2
	ActionTypes(0)=PlayerSprinting
	ActionTypes(1)=PlayerResting
	ActionTypes(2)=PlayerHoldingBreath
	APReq(0)=1
	APReq(1)=0
	APReq(2)=1

	bWeaponViewShake=true
	bOpenBriefing=true
	bAutoBriefing=true
	bTeamBeacons=true
	bEnableAmbientShake=false
}