//=============================================================================
// Frag.Ops  2004 Matt 'SquirrelZero' Farber
//=============================================================================
// Another slimmed down copy of the weapon class, this time for spectating in
// 1st person.  Nice because it only needs to exist on the client.
//=============================================================================
class FOSpectatorWeapon extends Inventory;

// the master weapon class, controls everything
var class<Weapon_Base> Master;
var class<Attachment_Base> OwnerAttachment;

var FOPlayer Controller;

// we'll use some settings here for easy access
var name IdleAnim, IdleEmptyAnim, ReloadAnim, SelectAnim, PutdownAnim, FireAnim, FireLoopAnim, FireAltAnim, FireLoopAltAnim;
var float IdleAnimRate, ReloadAnimRate, SelectAnimRate, PutDownAnimRate, FireAnimRate, FireLoopAnimRate, FireAltAnimRate, FireLoopAltAnimRate;
var sound SelectSound, ReloadSound, FireSound[2];
//var vector PlayerViewOffset;
var rotator RotationBase;
var float DisplayFOV, Hand, ReloadVolume, FireVolume[2], FirePitch[2];
var float YAWTurnOffset, PITCHTurnOffset, LastYaw, LastPitch;
var bool bCanBringUp, bInitiallyReloading, bInitiallyZoomed, bNoRender, bPlayLastFireAnim, bPlayIdleEmpty, bPlayedPutdown, bCanReload, bAkimbo, bZoomed, bMovingRight, bMovingLeft;
var actor FlashEmitter, SmokeEmitter, SmokeEmitter2nd;
var Effect_ShellEjectorFirst ShellEjector;
var byte LastZoomLevel, FiringMode, LastFireType;

function InitializeFor(class<Weapon_Base> WeaponClass, Pawn NewInstigator, FOPlayer RealController)
{
	local int i;
	
	// initialize master
	OwnerAttachment = class<Attachment_Base>(WeaponClass.Default.AttachmentClass);
	Instigator = NewInstigator;
	Controller = RealController;
	Master = Controller.TargetWeapon;
	FOPawn(Instigator).SpectatorWeapon = self;

	// set mesh
	LinkMesh(Master.Default.Mesh);

	// load all skins first
	for (i=0;i<WeaponClass.Default.Skins.Length;i++)
		Skins[i] = Master.Default.Skins[i];

	// set base attributes
	if (Instigator.PlayerReplicationInfo.Team.TeamIndex == 0)
	{
		Skins[0] = FOPawn(Instigator).Default.MercSleaveSkins[FOPawn(Instigator).BodySkin];
		Skins[1] = FOPawn(Instigator).Default.MercGloveSkin;
	}
	else
	{
		Skins[0] = FOPawn(Instigator).Default.SOSleaveSkins[FOPawn(Instigator).BodySkin];
		Skins[1] = FOPawn(Instigator).Default.SOGloveSkin;
	}

	SetDrawScale(Master.Default.DrawScale);
	DisplayFOV = Master.Default.DisplayFOV;
	PlayerViewOffset = Master.Default.PlayerViewOffset;
	PlayerViewPivot = Master.Default.PlayerViewPivot;
	RotationBase = Controller.BlendedTargetViewRotation;
	SelectSound = Master.Default.SelectSound;
	ReloadSound = Master.Default.FireModeClass[0].Default.ReloadSound;
	if (class<FireMode_Base>(Master.Default.FireModeClass[0]) != None)
		ReloadVolume = class<FireMode_Base>(Master.Default.FireModeClass[0]).Default.ReloadVolume;

	// fire sounds
	FireSound[0] = Master.Default.FireModeClass[0].Default.FireSound;
	FireVolume[0] = OwnerAttachment.Default.FireVolume[0];

	// initalize anims
	IdleAnim = Master.Default.IdleAnim;
	IdleAnimRate = Master.Default.IdleAnimRate;
	IdleEmptyAnim = Master.Default.IdleEmptyAnim;
	ReloadAnim = Master.Default.FireModeClass[0].Default.ReloadAnim;
	ReloadAnimRate = Master.Default.FireModeClass[0].Default.ReloadAnimRate;
	SelectAnim = Master.Default.SelectAnim;
	SelectAnimRate = Master.Default.SelectAnimRate;
	PutdownAnim = Master.Default.PutdownAnim;
	PutdownAnimRate = Master.Default.PutdownAnimRate;
	FireLoopAnim = Master.Default.FireModeClass[0].Default.FireLoopAnim;
	FireLoopAnimRate = Master.Default.FireModeClass[0].Default.FireLoopAnimRate;
	FireAnim = Master.Default.FireModeClass[0].Default.FireAnim;
	FireAnimRate = Master.Default.FireModeClass[0].Default.FireAnimRate;

	// alt anims
	if (Master.Default.FireModeClass[1] != None)
	{
		// fire sounds
		FireSound[1] = Master.Default.FireModeClass[1].Default.FireSound;
		FireVolume[1] = OwnerAttachment.Default.FireVolume[1];
		FireLoopAltAnim = Master.Default.FireModeClass[1].Default.FireLoopAnim;
		FireLoopAltAnimRate = Master.Default.FireModeClass[1].Default.FireLoopAnimRate;
		FireAltAnim = Master.Default.FireModeClass[1].Default.FireAnim;
		FireAltAnimRate = Master.Default.FireModeClass[1].Default.FireAnimRate;
	}

	// set initial firetype
	LastFireType = Controller.TargetFireType;

	// check if this player was already reloading.. if so, don't call
	// the first playreload, otherwise we could desynch.
	bInitiallyReloading = Controller.bTargetReloading;

	// now check if initially zoomed, so we can decide
	// to play the zoom click sound or not.
	bInitiallyZoomed = Controller.bTargetScopeZoomed;

	// allow reloading
	bCanReload = true;

	// bring into view
	BringUp();
}

// functions needed to fool player into thinking this is a normal weapon
function BringUp()
{
	if (bNoRender)
	{
		if (Controller.bTargetWeaponPickup)
		{
			PlaySound(SelectSound, SLOT_Interact,,,,, false);
			PlayAnim(SelectAnim,SelectAnimRate,0.0);
		}
		else
			PlayIdle();
		InitEffects();
		bNoRender = false;
	}
}

function BringIntoView()
{
	if (!bNoRender)
		return;

	bPlayedPutdown = false;
	bNoRender = false;
	PlayAnim(SelectAnim,SelectAnimRate,0.0);
}

function PutDown()
{
	if (bPlayedPutdown)
		return;

	bPlayedPutdown = true;
	PlayAnim(PutdownAnim,PutdownAnimRate,0.0);
	Controller.EndZoom();
	bZoomed = false;
	SetTimer(0.3,false);
}

function Timer()
{
	bNoRender = true;
}

event RenderOverlays(Canvas Canvas)
{
	local vector NewScale3D, CamPos;//, globalX,globalY,globalZ, localX,localY,localZ;
	local rotator CamRot;
	//local rotator WeaponPivot;
	local float OrigHand;

	if (Controller == None || bNoRender)
	{
		if (Controller != None)
			DrawAkimboInfo(Canvas);
		return;
	}

	OrigHand = Controller.TargetHand;
	if (OrigHand == 0)
		OrigHand = 1;
	
	if (bAkimbo)
		Hand = -OrigHand;
	else
		Hand = OrigHand;

	if ((Hand < -1.0) || (Hand > 1.0))
	{
		SetLocation(Instigator.Location);

		// draw scope
		if (Controller.bTargetScopeZoomed && Master.Default.ScopeHUDTex != None)
			DrawScope(Canvas);
		
		// draw extra ammo icon for akimbos
		if (bAkimbo)
			DrawAkimboInfo(Canvas);
		return;
	}

	if (!Controller.bTargetScopeZoomed || Master.Default.ScopeHUDTex == None)
	{
		// mirror
		newScale3D = Default.DrawScale3D;
		newScale3D.Y *= Hand;
		SetDrawScale3D(newScale3D);

		// adjust for hand
		PlayerViewPivot.Roll = Master.Default.PlayerViewPivot.Roll * Hand;
		PlayerViewPivot.Yaw = Master.Default.PlayerViewPivot.Yaw * Hand;
		PlayerViewOffset = Master.Default.PlayerViewOffset;
		PlayerViewOffset.Y *= Hand;

		// better way to draw weapons ;)
		Canvas.GetCameraLocation(CamPos,CamRot);	

		// update location/rotation
		SetLocation(CamPos + ((0.9/DisplayFOV*100*PlayerViewOffset) >> RotationBase) + (Instigator.WeaponBob(Master.Default.BobDamping)*0.14));
		SetRotation(RotationBase);

		// draw
		bDrawingFirstPerson = true;
		Canvas.DrawActor(self, false, false, DisplayFOV);
		bDrawingFirstPerson = false;
	}

	// draw scope
	if (Controller.bTargetScopeZoomed && Master.Default.ScopeHUDTex != None)
	{
		SetLocation(Instigator.Location);
		DrawScope(Canvas);
	}
	
	// draw extra ammo icon for akimbos
	if (bAkimbo)
		DrawAkimboInfo(Canvas);
}

// weapon scopes
function DrawScope(Canvas Canvas)
{
	Canvas.SetPos(0, 0);
	Canvas.DrawTile(Master.Default.ScopeHUDTex, Canvas.ClipX, Canvas.ClipY, 0, 0, 1024, 1024);
}

function DrawAkimboInfo(Canvas Canvas)
{
	local Interface_MainHUD TempMainHUD;

	if (Controller != None)
		TempMainHUD = Interface_MainHUD(Controller.MyHUD);

	if (TempMainHUD == None || Instigator.bHideRegularHUD || !TempMainHUD.bShowWeaponInfo || TempMainHUD.bHideHUD)
		return;

	TempMainHUD.AmmoSecCount.Value = Controller.TargetAmmoSec;
	TempMainHUD.DrawSpriteWidget(Canvas, TempMainHUD.AmmoSecIcon);
	TempMainHUD.DrawNumericWidget(Canvas, TempMainHUD.AmmoSecCount, TempMainHUD.DigitsBig);
}

function Tick(float dt)
{
	local int Multiplier;
	local FOSpectatorWeapon SpectatorWeaponAkimbo;

	Super.Tick(dt);

	if (Instigator == None || Controller == None)
		return;

	// check for akimbo reloads
	SpectatorWeaponAkimbo = Interface_MainHUD(Controller.MyHUD).SpectatorWeaponAkimbo;
	if (SpectatorWeaponAkimbo != None && !bAkimbo)
	{
		if (Controller.bTargetAkimboReloading)
		{
			bCanBringUp = true;
			PutDown();
		}
		else if (bCanBringUp)
		{
			bCanBringUp = false;
			BringIntoView();
		}
		else if (Controller.bTargetReloading && !Controller.bTargetAkimboReloading)
			SpectatorWeaponAkimbo.PutDown();
		else if (!Controller.bTargetReloading && !Controller.bTargetAkimboReloading)
			SpectatorWeaponAkimbo.BringIntoView();
	}

	// putdown check
	if (Controller.bTargetWeaponPutdown)
		PutDown();

	// reload check
	if (!bInitiallyReloading && (bAkimbo && Controller.bTargetAkimboReloading) || (!bAkimbo && Controller.bTargetReloading))
	{
		if (bCanReload)
			PlayReload();
		bCanReload = false;
	}
	else
	{
		bCanReload = true;
		if (!Controller.bTargetReloading)
			bInitiallyReloading = false;
	}	
	// scope zoom check
	if (Controller.bTargetScopeZoomed && Master.Default.ScopeHUDTex != None && !Controller.bTargetReloading)
		UpdateZoom();
	else if (bZoomed)
		DisableZoom();
	else if (Controller.TargetFireType != LastFireType)
		PlaySound(Master.Default.FireSwitchSound,,TransientSoundVolume*0.95,,30,,false);

	// adjustview
	Controller.AdjustView(dt);

	// update last firetype
	LastFireType = Controller.TargetFireType;

	// calc lazy weapon movement
	Multiplier = -1;
	RotationBase = Controller.BlendedTargetViewRotation;

	if (int(LastYaw) == RotationBase.Yaw)
	{
		YAWTurnOffset = YAWTurnOffset - (YAWTurnOffset*dt*7.0);
		if ((YAWTurnOffset < 2.0 && YAWTurnOffset > 0) || (YAWTurnOffset > -2.0 && YAWTurnOffset < 0))
			YAWTurnOffset = 0;
		bMovingRight = false;
		bMovingLeft = false;
	}
	else
	{
		bMovingLeft = true;
		bMovingRight = false;
		if (RotationBase.Yaw ClockWiseFrom LastYaw)
		{
			bMovingLeft = false;
			bMovingRight = true;
			Multiplier = 1;
		}
		YAWTurnOffset = fClamp(YAWTurnOffset - (Multiplier*750*dt*1.2), -Master.Default.WeaponTurnLimit, Master.Default.WeaponTurnLimit);
	}

	Multiplier = -1;

	if (int(LastPitch) == RotationBase.Pitch)
	{
		PITCHTurnOffset = PITCHTurnOffset - (PITCHTurnOffset*dt*5.0);
		if ((PITCHTurnOffset < 2.0 && PITCHTurnOffset > 0) || (PITCHTurnOffset > -2.0 && PITCHTurnOffset < 0))
			PITCHTurnOffset = 0;
	}
	else
	{
		if (RotationBase.Pitch ClockWiseFrom LastPitch)
			Multiplier = 1;
		PITCHTurnOffset = fClamp(PITCHTurnOffset - (Multiplier*750*dt*1.2), -Master.Default.WeaponPitchLimit, Master.Default.WeaponPitchLimit);
	}

	LastYaw = Controller.BlendedTargetViewRotation.Yaw;
	LastPitch = Controller.BlendedTargetViewRotation.Pitch;
	
	RotationBase.Yaw += YAWTurnOffset;
	RotationBase.Pitch += PITCHTurnOffset;
}

function UpdateZoom()
{
	if (!bZoomed && LastZoomLevel != Controller.TargetZoomLevel)
	{
		Controller.StartZoom();
		if (!bInitiallyZoomed)
			PlaySound(Sound'FOWeaponSounds.Misc.ZoomClick',,TransientSoundVolume,,,,false);
	}
	else if (LastZoomLevel != Controller.TargetZoomLevel && !bInitiallyZoomed)
		PlaySound(Master.Default.FireSwitchSound,,TransientSoundVolume*0.95,,30,,false);

	bZoomed = true;

	if (Master.Default.WeaponType != Sniper && Controller.TargetZoomLevel != LastZoomlevel)
		Controller.ZoomLevel = Master.Default.ZoomMultiple;
	else if (Controller.TargetZoomLevel != LastZoomlevel)
	{
		if (Controller.TargetZoomLevel == 1)
			Controller.ZoomLevel = Master.Default.ZoomMultiple;
		else if (Controller.TargetZoomLevel == 2)
			Controller.ZoomLevel = Master.Default.ZoomMultiple * Master.Default.SniperZoomLevels[0];
		else if (Controller.TargetZoomLevel == 3)
			Controller.ZoomLevel = Master.Default.ZoomMultiple * Master.Default.SniperZoomLevels[1];
	}

	// auto zoom
	if (bInitiallyZoomed)
	{
		Controller.DesiredFOV = FClamp(90.0 - (Controller.ZoomLevel * 88.0), 1, 170);
		Controller.FOVAngle = Controller.DesiredFOV;
	}

	LastZoomLevel = Controller.TargetZoomLevel;
	bInitiallyZoomed = false;
}

function DisableZoom()
{
	Controller.EndZoom();
	if (!Controller.bTargetReloading && Instigator.Health > 0 && GetAnimSeq() != FireAnim && GetAnimSeq() != FireAltAnim && GetAnimSeq() != ReloadAnim && GetAnimSeq() != PutdownAnim)
	{
		PlaySound(Sound'FOWeaponSounds.Misc.ZoomClick',,TransientSoundVolume,,,,false);
		if (HasAnim(Master.Default.ScopeDownAnim))
			PlayAnim(Master.Default.ScopeDownAnim, Master.Default.ScopeDownAnimRate, 0.0);
	}
	bZoomed = false;
	LastZoomLevel = 0;
}

function PlayReload()
{
	Controller.EndZoom();
	bZoomed = false;

	if (IsInState('ReloadWeapon'))
		return;

	// some weapons have special reloads, we can compensate for that here
	if (bAkimbo)
		GotoState('ReloadWeaponAkimbo','Begin');
	else if (Master.Name == 'Weapon_RedLabelBase' && Controller.TargetAmmo == 0 && Controller.TargetClips >= 2)
		GotoState('ReloadRedDouble','Begin');
	else if (Master.Name == 'Weapon_Super90Base')
		GotoState('ReloadSuperBegin','Begin');
	else
		GotoState('ReloadWeapon','Begin');
}

state ReloadWeapon
{
	function BeginState()
	{
		PlayAnim(ReloadAnim,ReloadAnimRate);
		PlaySound(ReloadSound,,ReloadVolume*Master.Default.TransientSoundVolume,,100,,false);
		bPlayIdleEmpty = false;
	}

	function EndState()
	{
		if (!bPlayedPutDown)
			PlayIdle();
	}

Begin:
	sleep(Master.Default.ReloadTime);
	GotoState('');
}

state ReloadWeaponAkimbo extends ReloadWeapon
{
	function BeginState()
	{
		bPlayIdleEmpty = false;
	}

	function EndState()
	{
		if (!bPlayedPutDown)
			PlayIdle();
	}

Begin:
	if (bNoRender)
	{
		BringIntoView();
		sleep(0.3);
	}
	PlayAnim(ReloadAnim,ReloadAnimRate);
	PlaySound(ReloadSound,,ReloadVolume*Master.Default.TransientSoundVolume,,100,,false);
	sleep(Master.Default.ReloadTime);
	GotoState('');
}

state ReloadRedDouble extends ReloadWeapon
{
	function BeginState()
	{
		PlayAnim(class<Weapon_RedLabelFire>(Master.Default.FireModeClass[0]).Default.ReloadAnimDouble, class<Weapon_RedLabelFire>(Master.Default.FireModeClass[0]).Default.ReloadAnimRateDouble);
		PlaySound(class<Weapon_RedLabelFire>(Master.Default.FireModeClass[0]).Default.ReloadSoundDouble,,ReloadVolume*Master.Default.TransientSoundVolume,,100,,false);
		bPlayIdleEmpty = false;
	}

	function EndState()
	{
		if (!bPlayedPutDown)
			PlayIdle();
	}

Begin:
	sleep(class<Weapon_RedLabelBase>(Master).Default.ReloadTimeDouble);
	GotoState('');
}

state ReloadSuperBegin extends ReloadWeapon
{
	function BeginState()
	{
		local class<Weapon_Super90Fire> FOFireClass;
		
		FOFireClass = class<Weapon_Super90Fire>(Master.Default.FireModeClass[0]);
		PlayAnim(FOFireClass.Default.ReloadBeginAnim, FOFireClass.Default.ReloadBeginAnimRate, 0.0);
		PlayOwnedSound(FOFireClass.Default.ReloadBeginSound,,ReloadVolume*TransientSoundVolume,,100,,false);
	}

	function EndState()
	{
	}

Begin:
	Sleep(class<Weapon_Super90Fire>(Master.Default.FireModeClass[0]).Default.ReloadBeginTime);
	GotoState('ReloadSuper','Begin');
}

state ReloadSuper extends ReloadWeapon
{
	function BeginState()
	{
		PlayAnim(ReloadAnim,ReloadAnimRate);
		PlaySound(ReloadSound,,ReloadVolume*Master.Default.TransientSoundVolume,,100,,false);
		bPlayIdleEmpty = false;

		SetTimer(Master.Default.ReloadTime,false);
	}

	function EndState()
	{
	}

	simulated function Timer()
	{
		if (Controller.bTargetReloading && Controller.TargetAmmo < Master.Default.FireModeClass[0].Default.AmmoClass.Default.MaxAmmo && Controller.TargetClips > 0)
			BeginState();
		else 
			GotoState('SuperReloadEnd','Begin');
	}
Begin:
}

state SuperReloadEnd extends ReloadWeapon
{
	function BeginState()
	{
		local class<Weapon_Super90Fire> FOFireClass;
		
		FOFireClass = class<Weapon_Super90Fire>(Master.Default.FireModeClass[0]);
		PlayAnim(FOFireClass.Default.ReloadEndAnim, FOFireClass.Default.ReloadEndAnimRate, 0.0);
		PlayOwnedSound(FOFireClass.Default.ReloadEndSound,,ReloadVolume*TransientSoundVolume,,100,,false);
	}
	
	function EndState()
	{
		if (!bPlayedPutDown)
			PlayIdle();
	}

Begin:
	Sleep(class<Weapon_Super90Fire>(Master.Default.FireModeClass[0]).Default.ReloadEndTime);
	GotoState('');
}

// some functions maliciously raided from the weapon class! :)
function name GetAnimSeq()
{
	local name anim;
	local float frame, rate;
    
	GetAnimParams(0, anim, frame, rate);
	return anim;
}

function AnimEnd(int channel)
{
	if (GetAnimSeq() != PutDownAnim)
		PlayIdle();
}

function PlayIdle()
{
	if (bPlayIdleEmpty && HasAnim(IdleEmptyAnim))
		PlayIdleEmpty();
	else
		LoopAnim(IdleAnim, IdleAnimRate, 0.2);
}

function PlayIdleEmpty()
{
	LoopAnim(IdleEmptyAnim, IdleAnimRate, 0.2);
}

function FireSpecWeapon(byte FireMode)
{
	FiringMode = FireMode;
	PlayFiring();

	if (Master.Default.FireModeClass[FireMode].Default.FlashEmitterClass != None)
		FlashMuzzleFlash();
	if (Master.Default.FireModeClass[FireMode].Default.SmokeEmitterClass != None)
		StartMuzzleSmoke();
}

function PrimeGrenade()
{
	if (class<Weapon_GrenadeBase>(Master) == None)
		return;

	PlayAnim(class<Weapon_GrenadeBase>(Master).Default.PrimeAnim, class<Weapon_GrenadeBase>(Master).Default.PrimeAnimRate, 0.0);	
	bPlayIdleEmpty = true;
}

function PlayFiring()
{
	// extra melee stuff
	if (Master.Default.WeaponType == Melee)
	{
		// reverse if akimbo
		if (bAkimbo)
		{
			if (bMovingRight)
			{
				bMovingRight = false;
				bMovingLeft = true;
			}
			else if (bMovingLeft)
			{
				bMovingRight = true;
				bMovingLeft = false;
			}
		}

		if (bMovingRight)
		{
			FireAnim = 'slasha';
			FireLoopAnim = FireAnim;
			FireLoopAnimRate = 1.1;
			FireAnimRate = FireLoopAnimRate;
			FireSound[0] = sound'FOWeaponSounds.Machete.LeftSlash';
		}
		else if (bMovingLeft)
		{
			FireAnim = 'slashb';
			FireLoopAnim = FireAnim;
			FireLoopAnimRate = 1.1;
			FireAnimRate = FireLoopAnimRate;
			FireSound[0] = sound'FOWeaponSounds.Machete.RightSlash';
		}
		else
		{
			FireAnim = 'slashb';
			FireLoopAnim = FireAnim;
			FireLoopAnimRate = 1.1;
			FireAnimRate = FireLoopAnimRate;
			FireSound[0] = sound'FOWeaponSounds.Machete.OverHead';
		}

		FireVolume[0] = 1.4;
		PlayFireSound(0);
		PlayAnim(FireLoopAnim, FireLoopAnimRate, 0.0);
		return;
	}

	if (FiringMode < 1)
	{
		// hack :(
		if (Master.Name == 'Weapon_TRGBase' && !Controller.bTargetScopeZoomed)
			PlayAnim(class<Weapon_TRGFire>(Master.Default.FireModeClass[0]).Default.FireRawAnim, class<Weapon_TRGFire>(Master.Default.FireModeClass[0]).Default.FireAnimRate, Master.Default.FireModeClass[0].Default.TweenTime);
		else if (HasAnim(FireLoopAnim))
			PlayAnim(FireLoopAnim, FireLoopAnimRate, 0.0);
		else
			PlayAnim(FireAnim, FireAnimRate, Master.Default.FireModeClass[0].Default.TweenTime);
	}
	else
	{
		if (HasAnim(FireLoopAltAnim))
			PlayAnim(FireLoopAltAnim, FireLoopAltAnimRate, 0.0);
		else
			PlayAnim(FireAltAnim, FireAltAnimRate, Master.Default.FireModeClass[1].Default.TweenTime);
	}
	PlayFireSound(FiringMode);
}

simulated function PlayFireSound(int ModeNum)
{
	if (Master.Default.FireModeClass[ModeNum] != None)
	{
		if (class<Weapon_KnifeBase>(Master) != None)
			PlaySound(FireSound[ModeNum],SLOT_Interact,TransientSoundVolume*FireVolume[ModeNum],,400,1.5,false);
		else
			PlaySound(FireSound[ModeNum],SLOT_Interact,TransientSoundVolume*FireVolume[ModeNum],,400,,false);
	}
}

// firemode raid!  some functions had to be modified
function DestroyEffects()
{
	if (SmokeEmitter != None)
		SmokeEmitter.Destroy();
	if (SmokeEmitter2nd != None)
		SmokeEmitter2nd.Destroy();
	if (ShellEjector != None)
	        ShellEjector.Destroy();
	if (FlashEmitter != None)
		FlashEmitter.Destroy();
}

function InitEffects()
{
	if ((Master.Default.FireModeClass[0].Default.FlashEmitterClass != None) && ((FlashEmitter == None) || FlashEmitter.bDeleteMe))
	{
		if (FlashEmitter != None)
			FlashEmitter.Destroy();
		FlashEmitter = Spawn(Master.Default.FireModeClass[0].Default.FlashEmitterClass,self);
	}

	if ((Master.Default.FireModeClass[0].Default.SmokeEmitterClass != None) && ((SmokeEmitter == None) || SmokeEmitter.bDeleteMe))
	{
		if (SmokeEmitter != None)
			SmokeEmitter.Destroy();
		SmokeEmitter = Spawn(Master.Default.FireModeClass[0].Default.SmokeEmitterClass,self);
	}

	if (class<FireMode_Base>(Master.Default.FireModeClass[0]) != None && (class<FireMode_Base>(Master.Default.FireModeClass[0]).Default.SmokeEmitter2ndClass != None) && ((SmokeEmitter2nd == None) || SmokeEmitter2nd.bDeleteMe))
	{
		if (SmokeEmitter2nd != None)
			SmokeEmitter2nd.Destroy();
		SmokeEmitter2nd = Spawn(class<FireMode_Base>(Master.Default.FireModeClass[0]).Default.SmokeEmitter2ndClass,self);
	}

	if (class<FireMode_Base>(Master.Default.FireModeClass[0]) != None && (class<FireMode_Base>(Master.Default.FireModeClass[0]).Default.ShellEjectorClass != None) && ((ShellEjector == None) || ShellEjector.bDeleteMe))
	{
		if (ShellEjector != None)
			ShellEjector.Destroy();
		ShellEjector = Spawn(class<FireMode_Base>(Master.Default.FireModeClass[0]).Default.ShellEjectorClass,self);
		ShellEjector.InitEjector();
	}
	AttachEffects();
}

function AttachEffects()
{
	if (FlashEmitter != None)
		AttachToBone(FlashEmitter, Master.Default.TracerBone);
	if (SmokeEmitter != None)
		AttachToBone(SmokeEmitter, Master.Default.SmokeBone);
	if (SmokeEmitter2nd != None)
		AttachToBone(SmokeEmitter2nd, Master.Default.SmokeBone);
}

function FlashMuzzleFlash()
{
	local rotator r;
	r.Roll = Rand(65536);
	r.Yaw = 25;
	r.Pitch = -50;
	AttachToBone(FlashEmitter, Master.Default.TracerBone);
	SetBoneRotation(Master.Default.TracerBone, r, 0, 1.f);
	if (FlashEmitter != None)
		FlashEmitter.Trigger(self, Instigator);
}

function EjectShells()
{
	if (ShellEjector == None)
		return;

	ShellEjector.Trigger(self, Instigator);
}

function StartMuzzleSmoke()
{
	local rotator r;
	r.Roll = 30000 + Rand(8000);
	r.Yaw = 4500;
	r.Pitch = -2550;
	SetBoneRotation(Master.Default.SmokeBone, r, 0, 1.f);
	if (SmokeEmitter2nd != None)
		SmokeEmitter2nd.Trigger(self, Instigator);
	if (SmokeEmitter != None && !Level.bDropDetail)
		SmokeEmitter.Trigger(self, Instigator);
	EjectShells();
}

function Destroyed()
{
	DestroyEffects();
	
	if (Instigator == None)
		return;
	
	if (FOPawn(Instigator).SpectatorWeapon == self)
		FOPawn(Instigator).SpectatorWeapon = None;

	if (bZoomed)
		Controller.EndZoom();
}

defaultproperties
{
	DrawType=DT_Mesh
	Style=STY_Normal
	bClientAnim=true
	bHidden=true
	bNoRender=true

	ScaleGlow=1.0
	AmbientGlow=11
	MaxLights=5

	RemoteRole=ROLE_None
}