//=============================================================================
// Frag.Ops  2004 Matt 'SquirrelZero' Farber
//=============================================================================
class FOWeaponPickup extends UTWeaponPickup
	notplaceable
	abstract;

var() int NumClips[2];
var int GBounces, DamageTaken;
var bool bGaveAmmo;
var bool bWeaponBroken, bAlreadyBroken;

// water splashes
var sound SplashSound;
var float SplashThreshold;

// impact sounds
var sound ImpactSound;
var float LastImpactTime;

replication
{
	reliable if (Role == ROLE_Authority)
		bWeaponBroken;
}

function PostBeginPlay()
{
	local vector NewVelocity;
	Super(Pickup).PostBeginPlay();
	SetWeaponStay();
	if (FragOpsWar(Level.Game) != None)
	{
		if (FragOpsWar(Level.Game).WarLevelInfo.bTeamDeathMatch)
			LifeSpan = 25;
		else
			LifeSpan = 60;
	}
	NewVelocity = Velocity * 0.6;
	NewVelocity.Z -= 15;
	Velocity = NewVelocity;
	MaxDesireability = 1.2 * class<Weapon>(InventoryType).Default.AIRating;
}

function InitDroppedPickupFor(Inventory Inv)
{
	local Weapon W;
	W = Weapon(Inv);
	if (W != None)
	{
		if (Weapon_Base(W).GetAmmo(0) != None)
		{
			AmmoAmount[0] = W.AmmoAmount(0);
			NumClips[0] = Weapon_Base(W).GetAmmo(0).NumClips;
		}
		if (Weapon_Base(W).GetAmmo(1) != None && Weapon_Base(W).GetAmmo(1) != Weapon_Base(W).GetAmmo(0))
		{
			AmmoAmount[1] = W.AmmoAmount(1);
			NumClips[1] = Weapon_Base(W).GetAmmo(1).NumClips;
		}
	}
	SetPhysics(PHYS_Falling);
	GotoState('NewFallingPickup');
	bAlwaysRelevant = false;
	bOnlyReplicateHidden = false;
	bUpdateSimulatedPosition = true;
	bDropped = true;
	bIgnoreEncroachers = false; // handles case of dropping stuff on lifts etc
}

function inventory SpawnCopy( pawn Other )
{
	local inventory Copy;
	local weapon CompWeapon;

	bGaveAmmo = false;

	if (Other.Inventory != None)
		CompWeapon = FOPawn(Other).FindWeapon(InventoryType.Default.InventoryGroup);

	// stop grenade pickups in war with a LAW
	if ((CompWeapon != None && War_Weapon_LAWBase(CompWeapon) != None && InventoryType.Default.InventoryGroup == 4) || (CompWeapon != None && (Weapon_GrenadeBase(CompWeapon) != None || FOPawn(Other).FindWeapon(5) != None) && class<War_Weapon_LAWBase>(InventoryType) != None))
		return None;

	if (CompWeapon != None && ((InventoryType.Default.InventoryGroup == 4 && class<War_Weapon_LAWBase>(InventoryType) == None && FOPawn(Other).FindWeapon(4) == None && FOPawn(Other).FindWeapon(5) == None) || (InventoryType.Default.InventoryGroup != 4)))
	{
		if (Weapon_Base(CompWeapon).Default.StaticName ~= class<Weapon_Base>(InventoryType).Default.StaticName)
		{
				while (NumClips[0] > 0 && Weapon_Base(CompWeapon).GetAmmo(0).NumClips < Weapon_Base(CompWeapon).GetAmmo(0).Default.MaxClips)
				{
					NumClips[0]--;
					Weapon_Base(CompWeapon).GetAmmo(0).NumClips++;
					bGaveAmmo = true;
				}
				while (NumClips[1] > 0 && Weapon_Base(CompWeapon).GetAmmo(1).NumClips < Weapon_Base(CompWeapon).GetAmmo(1).Default.MaxClips)
				{
					NumClips[1]--;
					Weapon_Base(CompWeapon).GetAmmo(1).NumClips++;
					bGaveAmmo = true;
				}
				if (bGaveAmmo)
					AnnouncePickup(Other);
		}
		return None;
	}

	Copy = spawn(InventoryType,Other,,,rot(0,0,0));
	if (Copy.Default.InventoryGroup == 4 && FOPawn(Other).FindWeapon(4) != None)
	{
		if (FOPawn(Other).FindWeapon(5) != None)
		{
			Copy.Destroy();
			return None;
		}
		Copy.InventoryGroup = 5;
	}
	Copy.GiveTo(Other, self);
	if (Weapon_Base(Copy).Default.BaseWeaponClass == None)
		FOPawn(Other).EquipList[(Copy.InventoryGroup-1)] = class<Weapon_Base>(InventoryType);
	else
	{
		FOPawn(Other).EquipList[(Copy.Default.InventoryGroup-1)] = Weapon_Base(Copy).Default.BaseWeaponClass;
		FOPawn(Other).AttachList[(Copy.Default.InventoryGroup-1)] = class<Weapon_Base>(InventoryType);
	}
	
	if (Weapon(Copy) != None && Weapon_Base(Copy).GetAmmo(0) != None)
	{
		AnnouncePickup(Other);
		Destroy();
	}
	return Copy;
}

auto state Pickup
{
	function bool ReadyToPickup(float MaxWait)
	{
		return true;
	}

	/* ValidTouch()
	 Validate touch (if valid return true to let other pick me up and trigger event).
	*/
	function bool ValidTouch( actor Other )
	{
		// make sure its a live player
		if ( (Pawn(Other) == None) || !Pawn(Other).bCanPickupInventory || (Pawn(Other).Health <= 0) )
			return false;

		// make sure not touching through wall, and weapon not about to be deleted
		if (!FastTrace(Other.Location, Location) || bWeaponBroken)
			return false;

		if (Pawn(Other).Inventory == none)
			return true;

		// make sure game will let player pick me up
		TriggerEvent(Event, self, Pawn(Other));

		return true;
	}
		
	// When touched by an actor.
	function Touch( actor Other )
	{
		local Inventory Copy;

		// If touched by a player pawn, let him pick this up.
		if (ValidTouch(Other))
		{
			Copy = SpawnCopy(Pawn(Other));
			if ( Copy != None )
				Copy.PickupFunction(Pawn(Other));
		}
	}

	// Make sure no pawn already touching (while touch was disabled in sleep).
	function CheckTouching()
	{
		local Pawn P;

		ForEach TouchingActors(class'Pawn', P)
			Touch(P);
	}

	function BeginState()
	{
		if ( bDropped )
			AddToNavigation();
	}
	
	function EndState()
	{
		if ( bDropped )
			RemoveFromNavigation();
	}

Begin:
	CheckTouching();
}

event EncroachedBy(actor Other)
{
	Touch(Other);
}

event HitWall(vector HitNormal, Actor HitActor)
{
	local float ImpactVolume;
	
	if (Level.TimeSeconds - LastImpactTime > 0.15)
	{
		ImpactVolume = FClamp(VSize(Velocity)/500.0, 0.1, 0.65);
		LastImpactTime = Level.TimeSeconds;
		PlaySound(ImpactSound,SLOT_Interact, ImpactVolume,,50,0.97+(FRand()*0.06));
	}
	Velocity = 0.38 * (Velocity - 2.0*HitNormal*(Velocity dot HitNormal));
	GBounces++;
	if (GBounces > 2)
		bBounce = false;

	if (!IsInState('Pickup'))
		GotoState('Pickup','Begin');
}

event Landed(vector HitNormal)
{
	Velocity = 0.3 * (Velocity - 2.0*HitNormal*(Velocity dot HitNormal));
	GBounces++;
	if (GBounces > 2)
		bBounce = false;
	GotoState('Pickup','Begin');
}

state NewFallingPickup extends Pickup
{
	function CheckTouching()
	{
	}

	function bool ValidTouch( actor Other )
	{
		return false;
	}

	function Timer()
	{
	}

	function BeginState()
	{
	}
}

static function string GetLocalString(
	optional int Switch,
	optional PlayerReplicationInfo RelatedPRI_1, 
	optional PlayerReplicationInfo RelatedPRI_2
	)
{
	local class<weapon_base> TempWeaponClass;

	if (Switch == 0)
		return Default.PickupMessage;
	else
	{
		TempWeaponClass = class<Weapon_Base>(Default.InventoryType);
		return "Picked up"@TempWeaponClass.Default.StaticName@"ammo";
	}
}

function AnnouncePickup( Pawn Receiver )
{
	if (!bGaveAmmo)
		Receiver.HandlePickup(self);
	else
		FOPawn(Receiver).HandleAmmoPickup(self);

	PlaySound(PickupSound,SLOT_Interact ); 
}

// tell the bot how much it wants this weapon pickup
// called when the bot is trying to decide which inventory pickup to go after next
function float BotDesireability(Pawn Bot)
{
	local Weapon_Base AlreadyHas;
	local float desire;

	// bots adjust their desire for their favorite weapons
	desire = MaxDesireability + Bot.Controller.AdjustDesireFor(self);

	// see if bot already has a weapon of this type
	// -- changed to check for the gun's static name instead, so attachment variations
	// -- will still be desireable
	AlreadyHas = Weapon_Base(FOPawn(Bot).FindStaticType(InventoryType));
	if (AlreadyHas != None)
	{
		if (Bot.Controller.bHuntPlayer)
			return 0;

		// bot wants this weapon for the ammo it holds
		// -- changed to magazines
		if (AlreadyHas.GetAmmo(0).NumClips < AlreadyHas.GetAmmo(0).Default.MaxClips && NumClips[0] > 0)
			return 0.5*desire;

		return 0;
	}

	// we don't want the bot to go after this gun at all if it has one in the same group
	if (FOPawn(Bot).FindInventoryGroup(InventoryType.Default.InventoryGroup))
		return 0;

	if (Bot.Controller.bHuntPlayer && (desire * 0.833 < Bot.Weapon.AIRating - 0.1))
		return 0;
	
	// incentivize bot to get this weapon if it doesn't have a good weapon already
	if (Bot.Weapon == None || Bot.Weapon.AIRating < 0.5)
		return 2*desire;

	return desire;
}

// updated so bots don't get "glued" to it
function float DetourWeight(Pawn Other,float PathWeight)
{
	local Weapon_Base AlreadyHas;

	AlreadyHas = Weapon_Base(FOPawn(Other).FindStaticType(InventoryType));
	if (AlreadyHas != None)
	{
		// bot wants this weapon for the ammo it holds
		// -- changed to magazines
		if (AlreadyHas.GetAmmo(0).NumClips > 0)
			return 0;
	}
	else if (FOPawn(Other).FindInventoryGroup(InventoryType.Default.InventoryGroup))
		return 0;

	if (AIController(Other.Controller).PriorityObjective()
		&& ((Other.Weapon.AIRating > 0.5) || (PathWeight > 400)) )
		return 0.2/PathWeight;

	return class<Weapon>(InventoryType).Default.AIRating/PathWeight;
}

function TakeDamage(int Damage, Pawn instigatedBy, Vector HitLocation, Vector Momentum, class<DamageType> damageType)
{
	local vector start, end, hl, HitNormal, VelVector;
	local actor A;

	if (bWeaponBroken)
		return;

	Damage *= 0.5;

	if (DamageType.Name == 'DamType_Machete' || DamageType.Name == 'DamType_AKBayonet' || DamageType.Name == 'DamType_MachetePoison')
		DamageTaken += Damage/3;
	else
		DamageTaken += Damage;

	// we want them to break apart if they take too much damage
	if (DamageTaken >= 50)
	{
		bHidden = true;
		LifeSpan = 2.0;
		bWeaponBroken = true;
		if (Level.NetMode != NM_DedicatedServer)
			PostNetReceive();

	}
	else
	{
		if (Level.NetMode != NM_DedicatedServer)
			Spawn(class'FragOps.Effect_KnifeSparkImpact',,, HitLocation, Rotator(-Momentum));

		// find out if we're on the ground, otherwise just apply normal momentum
		start = Location - vect(0,0,1)*CollisionHeight;
		end = start - vect(0,0,10);
		A = Trace(hl,HitNormal,end,start,false);
		if ((hl != Vect(0,0,0) && (A == None || A.bWorldGeometry)) || Physics != PHYS_Falling)
		{
			SetPhysics(PHYS_Falling);
			// pickups shouldnt get much lift, but skid instead
			VelVector = 35*(-(HitNormal - (Momentum*17)));

			// reset bounce
			bBounce = true;
			GBounces = 0;
			
			if (Velocity.Z < 100)
				VelVector.Z = max(20,Rand(50));
			else
				VelVector.Z = 0;

			Velocity += VelVector;
		}
		else
		{
			Velocity += 200*Momentum;
		}
		RandSpin(100000);
	}
}

// new randspin for weapon pickups
function RandSpin(float spinRate)
{
	//DesiredRotation.Yaw = fRand()*float(Rand(65536));
	//RotationRate.Yaw = spinRate * 2 *FRand() - spinRate;
}

simulated function Tick(float deltatime)
{
	if ((Velocity.Z > 5 || Velocity.Z < -5) && !PhysicsVolume.bWaterVolume)
	{
		Velocity.Z -= 9.0;
	}
	Super.Tick(deltatime);
}

// slow falling on impact, play splash effects
simulated event PhysicsVolumeChange(PhysicsVolume NewVolume)
{
	local float SplashVolume;
	local float Speed;

	Super.PhysicsVolumeChange(NewVolume);
	
	if (!NewVolume.bWaterVolume)
		return;

	if (Role == ROLE_Authority)
		Velocity *= 0.2;
	
	if (Level.NetMode == NM_DedicatedServer)
		return;

	Speed = VSize(Velocity);
	
	SplashVolume = FClamp(Speed/SplashThreshold, 0, 0.55);
	PlaySound(SplashSound, SLOT_Misc, SplashVolume,, 500, 0.92);
}

simulated function PostNetReceive()
{
	if (Level.NetMode != NM_DedicatedServer && bWeaponBroken)
	{
		bHidden = true;
		SpawnShards();
	}
}

simulated function SpawnShards()
{
	local rotator rot;
	local vector start;
	local int i;

	if (bAlreadyBroken)
		return;

	bAlreadyBroken = true;

	for (i=0; i<15; i++)
	{
		start = Location + vect(0,0,1);
		start.X += fRand()*5;
		start.Y += fRand()*5;
		start.Z += fRand()*5;
		rot = rotator(vect(0,0,1));
		rot.yaw += FRand()*32000-16000;
		rot.pitch += FRand()*32000-16000;
		rot.roll += FRand()*32000-16000;
		Spawn(class'FragOps.Effect_MetalShard',, '', Start, rot);
	}
	Spawn(class'FragOps.Effect_PickupCrap',,, Location, rotator(vect(0,0,1)));
}

defaultproperties
{
	Physics=PHYS_Falling
	RotationRate=(Yaw=0,Pitch=0,Roll=0)
	DesiredRotation=(Pitch=12000,Yaw=5666,Roll=2334)
	bUseCylinderCollision=true
	bFixedRotationDir=false
	//bShouldBaseAtStartup=false
	CollisionRadius=+20.0
	CollisionHeight=+3.8
	bAmbientGlow=true
	AmbientGlow=70
	bAcceptsProjectors=true
	bUnlit=false
	bOnlyReplicateHidden=false
	bBounce=true
	bCanBeDamaged=true
	bProjTarget=true
	bReplicateMovement=true
	bCollideActors=True
	bCollideWorld=True
	bDisturbFluidSurface=true
	bWorldGeometry=false
	bStatic=false
	bNoDelete=false
	bNetTemporary=false
	NetPriority=1.5
	NetUpdateFrequency=50
	bNetNotify=true
	bIgnoreEncroachers=false
	FluidSurfaceShootStrengthMod=3.f
	CullDistance=4000.0

	MessageClass=class'Interface_PickupMessage'

	// splashes
	SplashSound=sound'FOGameSounds.AmbientHit.Kerplunk'
	SplashThreshold=600

	// standard impacts
	ImpactSound=sound'FOWeaponSounds.Misc.WeaponImpact'
}