//=============================================================================
// Frag.Ops  2003 Matt 'SquirrelZero' Farber
//=============================================================================
class FireMode_Base extends FragOps_FireMode;

// hit detection struct
struct sBoneData
{
	var name BoneName;
	var float BoneHeight;
	var float BoneRadius;
};

// array of bones to use in hit detection
var array<sBoneData> BoneData;

var() name FireEmptyAnim;
var() float FireEmptyAnimRate;
var() class<Effect_BulletHit> BulletHitEffectClass;
var class<DamageType> DamageType;
var() class<Emitter> NewMuzFlashClass;
var() Emitter NewMuzFlash;
var() class<xEmitter> SmokeEmitter2ndClass;
var() xEmitter SmokeEmitter2nd;
var() class<Effect_ShellEjectorFirst> ShellEjectorClass;
var() Effect_ShellEjectorFirst ShellEjector;
var() float ConeMultiplier, ConeMax, WeaponRange, WeaponDamageShrink, WeaponRangeReduce, DeadBulletRange, BulletHeightLoss;
var int WeaponDamage, MinDamage;
var int NumFireTimes, WaitToRecoil;
var int TotalShotsFired;
var float LastFireTime, LastHeadShotTime, LastBodyHitTime, Momentum;
var float HeadShotRadius, TorsoShotRadius, ArmShotRadius, LegShotRadius, HandShotRadius;
var float ReloadVolume;
var bool bPlayClick, bPlayLastFireAnim, bWaitToRecoil, bClientApproved;
var Material MetalSkin;

simulated function AttachEjector();

simulated function DestroyEffects()
{
	if (NewMuzFlash != None)
		NewMuzFlash.Destroy();
	if (SmokeEmitter != None)
		SmokeEmitter.Destroy();
	if (SmokeEmitter2nd != None)
		SmokeEmitter2nd.Destroy();
	if (ShellEjector != None)
	        ShellEjector.Destroy();
	if (FlashEmitter != None)
		FlashEmitter.Destroy();
}

simulated function InitEffects()
{
	// don't even spawn on server
	if ((Weapon.Level.NetMode == NM_DedicatedServer) || (AIController(Instigator.Controller) != None))
		return;

	if ((NewMuzFlashClass != None) && (NewMuzFlash == None || NewMuzFlash.bDeleteMe))
	{
		if (NewMuzFlash != None)
			NewMuzFlash.Destroy();
		NewMuzFlash = Weapon.Spawn(NewMuzFlashClass,Weapon.Owner);
	}

	if ((FlashEmitterClass != None) && (FlashEmitter == None || FlashEmitter.bDeleteMe))
	{
		if (FlashEmitter != None)
			FlashEmitter.Destroy();
		FlashEmitter = Weapon.Spawn(FlashEmitterClass,Weapon.Owner);
	}

	if ((SmokeEmitterClass != None) && (SmokeEmitter == None || SmokeEmitter.bDeleteMe))
	{
		if (SmokeEmitter != None)
			SmokeEmitter.Destroy();
		SmokeEmitter = Weapon.Spawn(SmokeEmitterClass,Weapon.Owner);
	}

	if ((SmokeEmitter2ndClass != None) && (SmokeEmitter2nd == None || SmokeEmitter2nd.bDeleteMe))
	{
		if (SmokeEmitter2nd != None)
			SmokeEmitter2nd.Destroy();
		SmokeEmitter2nd = Weapon.Spawn(SmokeEmitter2ndClass,Weapon.Owner);
	}

	if ((ShellEjectorClass != None) && (ShellEjector == None || ShellEjector.bDeleteMe))
	{
		if (ShellEjector != None)
			ShellEjector.Destroy();
		ShellEjector = Weapon.Spawn(ShellEjectorClass,Weapon.Owner);
		ShellEjector.InitEjector();
		ShellEjector.Instigator = Weapon.Instigator;
	}
}

function DrawMuzzleFlash(Canvas Canvas)
{
}

function FlashMuzzleFlash()
{
	if (FlashEmitter != None)
		FlashEmitter.Trigger(Weapon, Instigator);
	else if (NewMuzFlash != None)
		NewMuzFlash.Trigger(Weapon, Instigator);
}

function StartMuzzleSmoke()
{
	if ((SmokeEmitter2nd != None))
		SmokeEmitter2nd.Trigger(Weapon, Instigator);
	if ((SmokeEmitter != None) && !Level.bDropDetail)
		SmokeEmitter.Trigger(Weapon, Instigator);
	EjectShells();
}

function EjectShells()
{
	if (ShellEjector == None && ShellEjectorClass == None)
		return;
	else if (ShellEjectorClass != None && FOBot(Instigator.Controller) == None && ShellEjector == None)
		ShellEjector = Weapon.Spawn(ShellEjectorClass);

	if (FOBot(Instigator.Controller) != None && ShellEjector != None)
	{
		ShellEjector.Destroy();
		return;
	}
	else if (ShellEjector != None)
		ShellEjector.Trigger(Weapon, Instigator);
}

simulated function bool AllowFire()
{
	return (Weapon.AmmoAmount(0) >= AmmoPerFire && Instigator.Controller != None && !Instigator.Controller.IsInState('PreRound') && !Instigator.Controller.IsInState('Building') && !Weapon_Base(Weapon).IsReloading());
}

simulated function PlayEmptySound()
{
	if (Super.AllowFire())
		bPlayClick = true;
	else if ((Weapon != None) && (bPlayClick))
	{
		Weapon_Base(Weapon).PlayEmptySound();
		bPlayClick = false;
	}
}

simulated function bool CheckLastFire()
{
	if (Instigator.IsLocallyControlled())
	{
		if (Weapon.AmmoAmount(ThisModeNum) > 1)
			return false;
	}
	return true;
}

event ModeDoFire()
{
	if ((Level.NetMode == NM_DedicatedServer || Level.NetMode == NM_ListenServer) && !bClientApproved && AIController(Instigator.Controller) == None)
		return;

	// if client deviates too far from prediction, let the native code trigger the next fire
	if (Level.TimeSeconds - LastFireTime < FireRate*0.85)
		return;

	bClientApproved = false;
	WaitToRecoil = 0;

	bPlayLastFireAnim = CheckLastFire();
	PlayEmptySound();

	if (AmmoPerFire <= 0 || !AllowFire())
	{
		if (!AllowFire() && Weapon_Base(Weapon).IsReloading())
		{
			if (bFireOnRelease)
			{
				if (bIsFiring)
					NextFireTime += MaxHoldTime + 0.02;
				else
					NextFireTime = Level.TimeSeconds + 0.02;
			}
			else
			{
				NextFireTime += 0.02;
				NextFireTime = FMax(NextFireTime, Level.TimeSeconds);
			}
		}
		return;
	}

	if (AIController(Instigator.Controller) == None && Level.NetMode != NM_DedicatedServer)
	{
		Weapon_Base(Weapon).NetUpdateTime = Weapon.Level.TimeSeconds - 1;
		Weapon_Base(Weapon).ServerSetFireData(Instigator.Controller.Rotation.Yaw, Instigator.Controller.Rotation.Pitch, ThisModeNum);
	}

	if (Weapon_Base(Weapon).WeaponType == Shotgun)
	{
		NumFireTimes = 20;
		Spread = Default.Spread;
	}
	else if ((Level.TimeSeconds - LastFireTime) > FireRate*2.0)
	{
		Spread = Default.Spread;
		if ((Level.TimeSeconds - LastFireTime) > FireRate*3.0)
			NumFireTimes = 0;
		NumFireTimes = NumFireTimes/2;
	}
	else
	{
		NumFireTimes++;
		Spread += NumFireTimes * ConeMultiplier;
		Spread = FMin(Spread, ConeMax);
	}

	if (MaxHoldTime > 0.0)
		HoldTime = FMin(HoldTime, MaxHoldTime);

	// client
	if (Instigator.IsLocallyControlled())
	{
		ShakeView();
		PlayFiring();
		FlashMuzzleFlash();
		StartMuzzleSmoke();
	}

	// server
	if (Weapon.Role == ROLE_Authority)
	{
		Weapon.ConsumeAmmo(0, Load);
		DoFireEffect();
		if ((Instigator == None) || (Instigator.Controller == None))
			return;
		// add recoil modifier
		if (AIController(Instigator.Controller) != None)
			AIController(Instigator.Controller).WeaponFireAgain(BotRefireRate, true);
		Instigator.SpawnTime = -100000;
	}

	if (Instigator.IsLocallyControlled() && RecoilSize > 0)
		Weapon_Base(Weapon).DoRecoil(FOPawn(Instigator).ModifyRecoil(Weapon_Base(Weapon),RecoilSize),NumFireTimes);

	Weapon.IncrementFlashCount(ThisModeNum);

	// set the next firing time. must be careful here so client and server do not get out of sync
	if (bFireOnRelease)
	{
	        if (bIsFiring)
			NextFireTime += MaxHoldTime + FireRate;
		else
			NextFireTime = Level.TimeSeconds + FireRate;
	}
	else
	{
		NextFireTime += FireRate;
		NextFireTime = FMax(NextFireTime, Level.TimeSeconds);
	}

	Load = AmmoPerFire;
	HoldTime = 0;

	if (Instigator.PendingWeapon != Weapon && Instigator.PendingWeapon != None)
	{
		bIsFiring = false;
		Weapon.PutDown();
	}
	LastFireTime = Level.TimeSeconds;
}

function DoFireEffect()
{
	local Vector StartTrace, End, XVec;
	local Rotator R, Aim;
	local Vector HitLocation, HitNormal, HitRay;
	local Actor Other;
	local float BulletDistance;
	local float FODamage;
	local int Damage, i;
	local vector BoneTestLocation;
	local name HitBone;
	local Material SurfaceMat;
	local bool bLastShot, bContinuedShot;
	local Vehicle FoundVehicle;
	local FOPawn FP;

	bLastShot = false;
	bContinuedShot = false;

	Instigator.MakeNoise(1.0);

	// the to-hit trace always starts right in front of the eye
	StartTrace = Instigator.Location + Instigator.EyePosition();

	// add vehicle passenger offset
	if (Instigator.DrivenVehicle != None && ONSWeaponPawn(Instigator.DrivenVehicle) != None)
		StartTrace = StartTrace + (Instigator.DrivenVehicle.FPCamPos >> ONSWeaponPawn(Instigator.DrivenVehicle).VehicleBase.Rotation);

	if (FOPlayer(Instigator.Controller) != None)
	{
		if (bWaitForRelease && Weapon_Base(Weapon).FireTimeStamp != 0 && Level.TimeSeconds-Weapon_Base(Weapon).FireTimeStamp < 0.09 && Weapon_Base(Weapon).FireRotation != rot(0,0,0))
			Aim = Weapon_Base(Weapon).FireRotation;
		else
			Aim = Instigator.Controller.Rotation;
	}
	else
	{
		if (Weapon_Base(Weapon).WeaponType != Shotgun)
			Spread = 0.16;
		Aim = AdjustAim(StartTrace, AimError);
	}

	// reset so no mistakes
	Weapon_Base(Weapon).FireRotation = rot(0,0,0);
	Weapon_Base(Weapon).FireTimeStamp = 0;

	// for accuracy stats
	if (RecoilSize > 0)
		FragOps_PRI(Instigator.PlayerReplicationInfo).AddShot();

	while (true)
	{
		if (!bLastShot && !bContinuedShot)
			R = rotator(vector(Aim) + VRand()*FRand()*Spread);
		else
			R = Aim;
		XVec = vector(R);
		End = StartTrace + DeadBulletRange * XVec;
		Weapon.bUseCollisionStaticMesh = true;
		Other = Weapon.Trace(HitLocation,HitNormal,End,StartTrace,true,,SurfaceMat);
		Weapon.bUseCollisionStaticMesh = false;
		BulletDistance = VSize(Instigator.Location - HitLocation);

		if (Other != None && (Other != Instigator))
		{
			// vehicle damage
			if (War_FOVehicleWheeled(Other) != None)
			{
				FoundVehicle = Vehicle(Other);
				Other = War_FOVehicleWheeled(FoundVehicle).GetHitPawn(HitLocation, R);
				if (Other == None)
				{
					FoundVehicle.TakeDamage(WeaponDamage, Instigator, HitLocation, vect(0,0,0), DamageType);
					break;
				}
			}
			else if (War_FOVehiclePawn(Other) != None)
			{
				FoundVehicle = Vehicle(Other);
				Other = War_FOVehiclePawn(FoundVehicle).GetHitPawn(HitLocation, R);
				if (Other == None)
				{
					FoundVehicle.TakeDamage(WeaponDamage, Instigator, HitLocation, vect(0,0,0), DamageType);
					break;
				}
			}

			// vehicle tweak
			if (Other == Instigator)
				break;
			
			FP = FOPawn(Other);
			if (FP != None)
			{
				Damage = FOPawn(Instigator).ModifyDamageDone(class<Weapon_Base>(Weapon.Class),WeaponDamage);
				Damage = Damage * DamageAtten;
				AdjustDamage(Damage);
				if (BulletDistance > WeaponRange)
				{
					FODamage = BulletDistance - WeaponRange;
					FODamage = FODamage / WeaponDamageShrink;
					Damage = Damage - int(FODamage);
				}

				// this new method works much better.
				BoneTestLocation = HitLocation + (Other.CollisionRadius*0.12)*XVec;
				HitRay = Normal(XVec);
								
				// find the closest bone.
				HitBone = 'none';
				// scale the head bone location to the center of the face
				if (Pawn(Other).IsHeadShot(HitLocation, XVec, 1.0))
					HitBone = 'headshot';
				else
				{
					for (i=0;i<BoneData.Length;i++)
					{
						if (FP.IsBoneHit(HitLocation, XVec, 1.0, BoneData[i].BoneName, BoneData[i].BoneRadius, BoneData[i].BoneHeight))
						{
							HitBone = BoneData[i].BoneName;
							break;
						}
					}
				}

				// if we hit no bone, move the shot beyond the player's collision radius and continue down fire path
				if (HitBone == 'none' || HitBone == '' && !bContinuedShot)
				{
					Aim = rotator(End - StartTrace);
					StartTrace = HitLocation + 0.5*(Other.CollisionRadius*vector(Aim));
					bContinuedShot = true;
				}
				else if (HitBone != 'none' && HitBone != '')
				{
					//log("hitbone: "@hitbone);
					//log("dist: "@dist);
					//log("old damage: "@damage);
					CalcDamage(Damage, HitBone, Other);
					//log("new damage: "@damage);

					if (HitBone == 'headshot' && FP.Health > 0)
					{
						if (FP.PlayerReplicationInfo.Team != Weapon.Instigator.PlayerReplicationInfo.Team && RecoilSize > 0)
						if (AllowHitSound(Pawn(Other)))
							FP.PlayHeadShotSound();
					}
					else if (HitBone != 'headshot' && AllowHitSound(Pawn(Other)))
						FP.PlayBodyHitSound();
					if (Damage < MinDamage)
						Damage = MinDamage;

					FP.LastBone = HitBone;
					Other.TakeDamage(Damage, Instigator, HitLocation, HitRay, DamageType);
					break;
				}
				else
					break;
			}
			else if (Other.Default.bCanBeDamaged || Other.Physics == PHYS_Karma || !Other.bWorldGeometry)
			{
				if (Vehicle(Other) != None)
					SurfaceMat = MetalSkin;
				if (Projectile(Other) == None)
					BulletHitEffect(StartTrace, R, HitLocation, HitNormal, SurfaceMat);
				Other.TakeDamage(WeaponDamage, Instigator, HitLocation, HitRay, DamageType);
				break;
			}
			else
			{
				BulletHitEffect(StartTrace, R, HitLocation, HitNormal, SurfaceMat);
				break;
			}
		}
		else if (Other == None && !bLastShot && RecoilSize > 0) // simulate gravity by pointing bullets downwards
		{
			Aim = rotator(End - StartTrace);
			StartTrace = End;
			Aim.Pitch -= BulletHeightLoss;
			bLastShot = true;
		}
		else
			break;
	}
}

function bool AllowHitSound(Pawn P)
{
	return ((FragOpsGame(Weapon.Level.Game).FriendlyFireScale > 0.0) || (P.PlayerReplicationInfo == None) || (Instigator.PlayerReplicationInfo.Team != P.PlayerReplicationInfo.Team));
}

function AdjustDamage(out int Damage);

function CalcDamage(out int Damage, name HitBone, actor VActor)
{
	local int VictimTeam;
	local float InDamage;
	local FOPawn FP;
	
	VictimTeam = 2;
	FP = FOPawn(VActor);
	if (FP != None)
	{
		if ((FP.Health > 0) && (FP.PlayerReplicationInfo.Team != None) )
			VictimTeam = FP.PlayerReplicationInfo.Team.TeamIndex;
		if (FP.MiscItem ~= "LIT. Armor" && HitBone == 'headshot')
			Damage *= 1.2;
	}

	InDamage = Damage;
	switch(HitBone)
	{
		// head hits, 1.9x dmg
		case 'headshot':
			InDamage *= 1.85;
			Weapon_Base(Weapon).AddMedalPoints(1,VictimTeam);
			break;

		// chest, shoulder hits, normal dmg
		case 'spine':
			InDamage *= 1.0;
			Weapon_Base(Weapon).AddMedalPoints(2,VictimTeam);
			break;

		case 'pelvis':
			InDamage *= 1.0;
			Weapon_Base(Weapon).AddMedalPoints(2,VictimTeam);
			break;

		case 'rshoulder':
			InDamage *= 1.1;
			Weapon_Base(Weapon).AddMedalPoints(2,VictimTeam);
			break;

		case 'lshoulder':
			InDamage *= 1.1;
			Weapon_Base(Weapon).AddMedalPoints(2,VictimTeam);
			break;

	// arms, thighs, 0.75x dmg
		case 'rfarm':
			InDamage *= 0.75;
			Weapon_Base(Weapon).AddMedalPoints(3,VictimTeam);
			break;

		case 'lfarm':
			InDamage *= 0.75;
			Weapon_Base(Weapon).AddMedalPoints(3,VictimTeam);
			break;

		case 'lthigh':
			InDamage *= 0.75;
			Weapon_Base(Weapon).AddMedalPoints(3,VictimTeam);
			break;

		case 'rthigh':
			Damage *= 0.75;
			Weapon_Base(Weapon).AddMedalPoints(3,VictimTeam);
			break;

	// feet, hands, 0.25x dmg
		case 'leftfoot':
			InDamage *= 0.4;
			Weapon_Base(Weapon).AddMedalPoints(3,VictimTeam);
			break;

		case 'rightfoot':
			InDamage *= 0.4;
			Weapon_Base(Weapon).AddMedalPoints(3,VictimTeam);
			break;

		case 'lefthand':
			InDamage *= 0.4;
			Weapon_Base(Weapon).AddMedalPoints(3,VictimTeam);
			break;

		case 'righthand':
			InDamage *= 0.4;
			Weapon_Base(Weapon).AddMedalPoints(3,VictimTeam);
			break;

	// otherwise we default to normal, eventually i want to change this so that shots can pass through
	// the invisible collision area around the head and in front of the feet.
		case 'none':
			InDamage *= 1.1;
	}
	Damage = InDamage;
}

function BulletHitEffect(Vector Start, Rotator Dir, Vector HitLocation, Vector HitNormal, Material SurfaceMat)
{
	SpawnBulletHit(Weapon.Instigator, BulletHitEffectClass, Start, Dir, HitLocation, HitNormal, SurfaceMat, DamageType);
}

function SpawnBulletHit(Pawn InstigatorPawn, class<Effect_BulletHit> HitEffectClass, vector Start, rotator Dir, vector HitLocation, vector HitNormal, Material SurfaceMat, class<DamageType> NewDamageType)
{
	local Effect_BulletHit BulletHit;

	BulletHit = Weapon.Spawn(HitEffectClass, None,, Start, Dir);
	BulletHit.ImpactAt(HitLocation, HitNormal, SurfaceMat, InstigatorPawn, NewDamageType);
}

function PlayFiring()
{
	if (FireCount > 0)
	{
		if (Weapon.HasAnim(FireEmptyAnim) && bPlayLastFireAnim)
		{
			if (Weapon.HasAnim(FireLoopAnim))
				Weapon.PlayAnim(FireEmptyAnim, FireEmptyAnimRate, 0.0);
			else if (Weapon_Base(Weapon).CurrentFireType != 1)
				Weapon.PlayAnim(FireEmptyAnim, FireEmptyAnimRate, 0.0);
			else
				Weapon.PlayAnim(FireEmptyAnim, FireEmptyAnimRate, TweenTime);
			Weapon_Base(Weapon).bPlayIdleEmpty = true;
		}
		else if (Weapon.HasAnim(FireLoopAnim))
		{
			Weapon.PlayAnim(FireLoopAnim, FireLoopAnimRate, 0.0);
		}
		else if (Weapon_Base(Weapon).CurrentFireType != 1)
			Weapon.PlayAnim(FireLoopAnim, FireLoopAnimRate, 0.0);
		else
			Weapon.PlayAnim(FireAnim, FireAnimRate, TweenTime);
	}
	else if (Weapon.HasAnim(FireEmptyAnim) && bPlayLastFireAnim)
	{
		if (Weapon.HasAnim(FireLoopAnim))
			Weapon.PlayAnim(FireEmptyAnim, FireEmptyAnimRate, 0.0);
		else if (Weapon_Base(Weapon).CurrentFireType != 1)
			Weapon.PlayAnim(FireLoopAnim, FireLoopAnimRate, 0.0);
		else
			Weapon.PlayAnim(FireEmptyAnim, FireEmptyAnimRate, TweenTime);
		Weapon_Base(Weapon).bPlayIdleEmpty = true;
	}
	else if (Weapon_Base(Weapon).CurrentFireType != 1 && Weapon.HasAnim(FireLoopAnim))
	{
		Weapon.PlayAnim(FireLoopAnim, FireLoopAnimRate, 0.0);
	}
	else
		Weapon.PlayAnim(FireAnim, FireAnimRate, TweenTime);

	PlayFireSound();
        
	ClientPlayForceFeedback(FireForce);  // jdf
    
	FireCount++;
}

function PlayFireSound()
{
	Weapon.PlayOwnedSound(FireSound,SLOT_Interact,TransientSoundVolume,,,(0.97)+(FRand()*0.06),false);
}

function StartBerserk()
{
}

function StopBerserk()
{
}

defaultproperties
{
	// hit detection data
	BoneData(0)=(BoneName=lshoulder,BoneHeight=6.0,BoneRadius=10.0)
	BoneData(1)=(BoneName=rshoulder,BoneHeight=6.0,BoneRadius=10.0)
	BoneData(2)=(BoneName=rfarm,BoneHeight=6.0,BoneRadius=8.0)
	BoneData(3)=(BoneName=lfarm,BoneHeight=6.0,BoneRadius=8.0)
	BoneData(4)=(BoneName=lthigh,BoneHeight=12.0,BoneRadius=10.0)
	BoneData(5)=(BoneName=rthigh,BoneHeight=12.0,BoneRadius=10.0)
	BoneData(6)=(BoneName=lefthand,BoneHeight=3.0,BoneRadius=5.0)
	BoneData(7)=(BoneName=righthand,BoneHeight=3.0,BoneRadius=5.0)
	BoneData(8)=(BoneName=rightfoot,BoneHeight=10.0,BoneRadius=8.0)
	BoneData(9)=(BoneName=leftfoot,BoneHeight=10.0,BoneRadius=8.0)
	BoneData(10)=(BoneName=rfoot,BoneHeight=5.0,BoneRadius=8.0)
	BoneData(11)=(BoneName=lfoot,BoneHeight=5.0,BoneRadius=8.0)
	BoneData(12)=(BoneName=spine,BoneHeight=14.0,BoneRadius=12.0)
	BoneData(13)=(BoneName=pelvis,BoneHeight=7.5,BoneRadius=13.0)

	MetalSkin=Material'FOHeavyWeaponSkins.jeep.m240body'
	MinDamage=6
	BulletHeightLoss=500
	FireEmptyAnim=fireempty
	FireEmptyAnimRate=1.0
	Momentum=0.0
	BulletHitEffectClass=class'Effect_BulletHit'
	FlashEmitterClass=None
	PreFireTime=0.0
	NumFireTimes=0
	TotalShotsFired=0
	WaitToRecoil=0
	bInstantHit=true
	bAttachFlashEmitter=true
	bAttachSmokeEmitter=true
	NoAmmoSound=Sound'WeaponSounds.P1Reload5'
	ReloadVolume=1.0
	ShellEjectorClass=None

	// most guns don't shake
	ShakeOffsetMag=(X=0.0,Y=0.0,Z=0.0)
	ShakeOffsetRate=(X=0.0,Y=0.0,Z=0.0)
	ShakeOffsetTime=0.0
	ShakeRotMag=(X=0.0,Y=0.0,Z=0.0)
	ShakeRotRate=(X=0.0,Y=0.0,Z=0.0)
	ShakeRotTime=0.0
}