//=============================================================================
// Frag.Ops  2004 Matt 'SquirrelZero' Farber
//=============================================================================
class FragOpsGame extends xTeamGame
	HideDropDown
	CacheExempt
	config;

var int WinningTeamMoney, LosingTeamMoney, DeathMoney, KillMoney, RoundTimeRemaining, PreRoundTime, TimeTillNextRound;
var int RoundTime;
var globalconfig int PreRoundWaitTime, FirstRoundWaitTime;
var bool bForceEndRound, bRoundEnded;
var FragOps_TimedTrigger TimedTrigger;
var globalconfig bool bFOCTGlobalBan;
var globalconfig bool bAllowFloatCam;
var globalconfig bool bAllowThirdPersonSpec;
var int RoundKills[2];
var string CrowdMaterial[6];
var array<string> FOHints;
var int OrigTimeLimit;
var localized string FOGoreLevelText[4];
var() string Version;

function RestartPlayer(Controller aPlayer)
{}

function FragOps_CRI PlayerCRI(Controller CRIController)
{
	if (CRIController == None)
		return None;

	if (FOBot(CRIController) != None && FOBot(CRIController).CriticalReplicationInfo != None)
		return FOBot(CRIController).CriticalReplicationInfo;
	else if (FOPlayer(CRIController) != None && FOPlayer(CRIController).CriticalReplicationInfo != None)
		return FOPlayer(CRIController).CriticalReplicationInfo;
	else
		return None;
}

function Reset()
{
	bGameEnded = false;
	bOverTime = false;
	ElapsedTime = NetWait - 3;
	RemainingTime = 60 * OrigTimeLimit;
	GameReplicationInfo.RemainingMinute = RemainingTime;
	GameReplicationInfo.RemainingTime = RemainingTime;
	FragOps_GRI(GameReplicationInfo).bAllowBehindview = bAllowThirdPersonSpec;
	if (!IsInState('MatchInProgress'))
		GotoState('MatchInProgress');
}

function AddMutator(string mutname, optional bool bUserAdded)
{
	local class<Mutator> mutClass;
	local Mutator mut;

	if (!Static.AllowMutator(MutName))
		return;

	mutClass = class<Mutator>(DynamicLoadObject(mutname, class'Class'));
	if (mutClass == None || class<FOMutator>(mutClass) == None)
		return;

	if ((mutClass.Default.GroupName != "") && (BaseMutator != None))
	{
		// make sure no mutators with same groupname
		for (mut=BaseMutator; mut!=None; mut=mut.NextMutator)
			if (mut.GroupName == mutClass.Default.GroupName)
			{
				log("Not adding "$mutClass$" because already have a mutator in the same group - "$mut);
				return;
			}
	}

	// make sure this mutator is not added already
	for (mut=BaseMutator;mut!=None;mut=mut.NextMutator)
		if (mut.Class == mutClass)
		{
			log("Not adding "$mutClass$" because this mutator is already added - "$mut);
			return;
		}

	mut = Spawn(mutClass);
	// mc, beware of mut being none
	if (mut == None)
		return;

	// Meant to verify if this mutator was from Command Line parameters or added from other Actors
	mut.bUserAdded = bUserAdded;

	if (BaseMutator == None)
		BaseMutator = mut;
	else
		BaseMutator.AddMutator(mut);
}

static function FillPlayInfo(PlayInfo PlayInfo)
{
	local int i;
	
	// from info
	PlayInfo.AddClass(default.Class);
	
	// from gameinfo
	PlayInfo.AddSetting(default.BotsGroup,   "GameDifficulty",			GetDisplayText("GameDifficulty"), 		0, 2, "Select", default.GIPropsExtras[0], "Xb");
	PlayInfo.AddSetting(default.GameGroup,   "TimeLimit",				GetDisplayText("TimeLimit"), 			0, 0, "Text",     "3;0:999");
	PlayInfo.AddSetting(default.RulesGroup,  "bAllowWeaponThrowing",	GetDisplayText("bAllowWeaponThrowing"), 1, 0, "Check",             ,            ,    ,True);
	PlayInfo.AddSetting(default.RulesGroup,  "bWeaponShouldViewShake",	GetDisplayText("bWeaponShouldViewShake"),1, 0, "Check",            ,            ,    ,True);
	PlayInfo.AddSetting(default.ServerGroup, "bEnableStatLogging",		GetDisplayText("bEnableStatLogging"), 	0, 1, "Check",             ,            ,True);
	PlayInfo.AddSetting(default.ServerGroup, "bAdminCanPause",			GetDisplayText("bAdminCanPause"), 		1, 1, "Check",             ,            ,True,True);
	PlayInfo.AddSetting(default.ServerGroup, "MaxSpectators",			GetDisplayText("MaxSpectators"), 		1, 1, "Text",      "3;0:32",            ,True,True);
	PlayInfo.AddSetting(default.ServerGroup, "MaxPlayers",				GetDisplayText("MaxPlayers"), 			0, 1, "Text",      "3;0:32",            ,True);
	PlayInfo.AddSetting(default.ServerGroup, "MaxIdleTime",			GetDisplayText("MaxIdleTime"), 			0, 1, "Text",      "3;0:300",            ,True,True);
	PlayInfo.AddSetting(default.RulesGroup,  "bAllowThirdPersonSpec",	"3rd Person Spectating", 1, 0, "Check",             ,            ,    ,True);


	// Add GRI's PIData
	if (default.GameReplicationInfoClass != None)
	{
		default.GameReplicationInfoClass.static.FillPlayInfo(PlayInfo);
		PlayInfo.PopClass();
	}

	if (default.VoiceReplicationInfoClass != None)
	{
		default.VoiceReplicationInfoClass.static.FillPlayInfo(PlayInfo);
		PlayInfo.PopClass();
	}

	if (default.BroadcastClass != None)
		default.BroadcastClass.static.FillPlayInfo(PlayInfo);
	else class'BroadcastHandler'.static.FillPlayInfo(PlayInfo);

	PlayInfo.PopClass();

	if (class'Engine.GameInfo'.default.VotingHandlerClass != None)
 	{
	 	class'Engine.GameInfo'.default.VotingHandlerClass.static.FillPlayInfo(PlayInfo);
	 	PlayInfo.PopClass();
	}
	else
		log("GameInfo::FillPlayInfo class'Engine.GameInfo'.default.VotingHandlerClass = None");

	// from unreal mp gameinfo
	PlayInfo.AddSetting(default.BotsGroup,  "MinPlayers",         default.MPGIPropsDisplayText[i++], 0,   0,   "Text",            "3;0:32");
	PlayInfo.AddSetting(default.GameGroup,  "EndTimeDelay",       default.MPGIPropsDisplayText[i++], 1,   1,   "Text",                    ,     ,     , True);
	PlayInfo.AddSetting(default.BotsGroup,  "BotMode",			   default.MPGIPropsDisplayText[i++], 30,  1, "Select", default.BotModeText);
	PlayInfo.AddSetting(default.RulesGroup, "bAllowPrivateChat",  default.MPGIPropsDisplayText[i++], 254, 1,  "Check",                    , "Xv", True, True);

	// from deathmatch
	class'MasterServerUplink'.static.FillPlayInfo(PlayInfo);
	PlayInfo.PopClass();

	i = 0;

	// from teamgame
	PlayInfo.AddSetting(default.BotsGroup,  "bBalanceTeams",        default.TGPropsDisplayText[i++], 0,   2, "Check");
	PlayInfo.AddSetting(default.RulesGroup, "bPlayersBalanceTeams", default.TGPropsDisplayText[i++], 0,   1, "Check",            ,    ,True);
	PlayInfo.AddSetting(default.GameGroup,  "FriendlyFireScale",    default.TGPropsDisplayText[i++], 20,  1,  "Text", "8;0.0:1.0",    ,    ,True);
	PlayInfo.AddSetting(default.ChatGroup,  "bAllowNonTeamChat",	 default.TGPropsDisplayText[i++], 60,  1, "Check",            ,"Xv",True,True);
}


function GetServerDetails( out ServerResponseLine ServerState )
{
	AddServerDetail(ServerState, "Frag.Ops Version", Version);
	Super(UnrealMPGameInfo).GetServerDetails(ServerState);

	AddServerDetail(ServerState, "TimeLimit", TimeLimit);
	AddServerDetail(ServerState, "3rd Person Spectating", bAllowThirdPersonSpec);
}

static function PrecacheGameTextures(LevelInfo myLevel)
{
	local int i, j;
	local string ObjectString, jstr;
	
	// precache playermodels
	DynamicLoadObject("FOCharacterModels.swatMale",Class'SkeletalMesh');
	DynamicLoadObject("FOCharacterModels.mercAMale",Class'SkeletalMesh');

	if (Default.bPreloadAllSkins || myLevel.bShouldPreload)
	{
		// precache playerskins
		// -merc
		myLevel.AddPrecacheMaterial(Material'FOCharacters.textures.mercAhead');
		myLevel.AddPrecacheMaterial(Material'FOCharacters.textures.mercAbody');
		myLevel.AddPrecacheMaterial(Material'FOCharacters.textures.mercAbody_jungle');
		myLevel.AddPrecacheMaterial(Material'FOCharacters.textures.mercAbody_olive');
		myLevel.AddPrecacheMaterial(Material'FOCharacters.textures.mercAbody_brown');
		myLevel.AddPrecacheMaterial(Material'FOCharacters.textures.mercAbody_urban');

		// -specops
		myLevel.AddPrecacheMaterial(Material'FOCharacters.textures.swatAhead');
		myLevel.AddPrecacheMaterial(Material'FOCharacters.textures.swatAhead_masked');
		myLevel.AddPrecacheMaterial(Material'FOCharacters.textures.swatAbody');
		myLevel.AddPrecacheMaterial(Material'FOCharacters.textures.swatAbody_grey');
		myLevel.AddPrecacheMaterial(Material'FOCharacters.textures.swatAbody_forest');
		myLevel.AddPrecacheMaterial(Material'FOCharacters.textures.swatAbody_urban');
		myLevel.AddPrecacheMaterial(Material'FOCharacters.textures.swatAbody_desert');
		myLevel.AddPrecacheMaterial(Material'FOCharacters.textures.swatBhead');
		myLevel.AddPrecacheMaterial(Material'FOCharacters.textures.swatBhead_masked');
	}

	// precache weaponskins
	myLevel.AddPrecacheMaterial(Material'FOHighDetailTextures.Weapons.eightbody_final');
	myLevel.AddPrecacheMaterial(Material'FOHighDetailTextures.Weapons.eightscope_shader');
	myLevel.AddPrecacheMaterial(Material'FOHighDetailTextures.Weapons.ak103body_shader');
	myLevel.AddPrecacheMaterial(Material'FOHighDetailTextures.Weapons.mp7body_shader');
	myLevel.AddPrecacheMaterial(Material'FOHighDetailTextures.Weapons.debody_shader');
	myLevel.AddPrecacheMaterial(Material'FOHighDetailTextures.Weapons.tmpbody_shader');
	myLevel.AddPrecacheMaterial(Material'FOHighDetailTextures.Weapons.famasbody_shader');
	myLevel.AddPrecacheMaterial(Material'FOHighDetailTextures.Weapons.knifebody_shader');
	myLevel.AddPrecacheMaterial(Material'FOHighDetailTextures.Weapons.machetebody_shader');
	myLevel.AddPrecacheMaterial(Material'FOHighDetailTextures.Weapons.xm8body_shader');
	myLevel.AddPrecacheMaterial(Material'FOHighDetailTextures.Weapons.xm8scope_shader');
	myLevel.AddPrecacheMaterial(Material'FOHighDetailTextures.Weapons.dsrbody_shader');
	myLevel.AddPrecacheMaterial(Material'FOHighDetailTextures.Weapons.trgbody_shader');
	myLevel.AddPrecacheMaterial(Material'FOHighDetailTextures.Weapons.m4body_shader');
	myLevel.AddPrecacheMaterial(Material'FOHighDetailTextures.Weapons.sigbody_shader');
	myLevel.AddPrecacheMaterial(Material'FOHighDetailTextures.Weapons.mp9body_shader');
	myLevel.AddPrecacheMaterial(Material'FOHighDetailTextures.Weapons.redbody_shader');
	myLevel.AddPrecacheMaterial(Material'FOHighDetailTextures.Weapons.umpbody_shader');
	myLevel.AddPrecacheMaterial(Material'FOHighDetailTextures.Weapons.uspbody_final');
	myLevel.AddPrecacheMaterial(Material'magnum2.Texture.p90_shader');

	myLevel.AddPrecacheMaterial(Material'magnum2.Texture.FI2');
	myLevel.AddPrecacheMaterial(Material'magnum2.Texture.Super90Skin');
	myLevel.AddPrecacheMaterial(Material'magnum2.Texture.descope');
	myLevel.AddPrecacheMaterial(Material'magnum2.Texture.LAWtex');

	myLevel.AddPrecacheMaterial(Material'dsrtextures.Cubes.scope1');

	myLevel.AddPrecacheMaterial(Material'trgtextures.trgskins.scope_skin');

	myLevel.AddPrecacheMaterial(Material'mpfive.mp5body');
	myLevel.AddPrecacheMaterial(Material'mpfive.Cubes.scope');


	// scopes
	myLevel.AddPrecacheMaterial(Material'FOMisc.HUD.xm8scope_fb');
	myLevel.AddPrecacheMaterial(Material'FOMisc.HUD.xm8base_fb');
	myLevel.AddPrecacheMaterial(Material'FOMisc.HUD.aimpoint_fb');
	myLevel.AddPrecacheMaterial(Material'FOMisc.HUD.dsrscope_fb');
	myLevel.AddPrecacheMaterial(Material'FOMisc.HUD.mp5scope_fb');
	myLevel.AddPrecacheMaterial(Material'FOMisc.HUD.trgscope_fb');
	myLevel.AddPrecacheMaterial(Material'FOMisc.HUD.LAWscope_fb');
	myLevel.AddPrecacheMaterial(Material'FOMisc.HUD.LAWAimer');
	myLevel.AddPrecacheMaterial(Material'FOMisc.Special.ScreenNoise');
	myLevel.AddPrecacheMaterial(Material'FOMisc.Projected.TacLight');

	// sprites / miscellaneous
	myLevel.AddPrecacheMaterial(Material'FOMisc.Sprites.BrainChunk');
	myLevel.AddPrecacheMaterial(Material'FOMisc.Sprites.ExplosionFlashNew');
	myLevel.AddPrecacheMaterial(Material'FOMisc.Sprites.MuzFlashNew_t');
	myLevel.AddPrecacheMaterial(Material'FOMisc.Sprites.MuzFlash3rd_t');	

	// special metro city precache -- ugly
	if (myLevel.Title ~= "Metro City")
	{
		for (i=1;i<7;i++)
		{
			for (j=0;j<17;j++)
			{
				if (j<10)
					jstr = "0"$j;
				else
					jstr = string(j);

				ObjectString = "intro_characters.CrowdMale"$i$"Sprite.CrowdMale"$i$"_a"$jstr;
				if (i == 3 && j > 10)
					continue;
	
				myLevel.AddPrecacheMaterial(Material(DynamicLoadObject(ObjectString,Class'Material')));
			}
		}
		myLevel.AddPrecacheMaterial(Material(DynamicLoadObject("intro_characters.CrowdMale6Sprite.CrowdMale6_a17",Class'Material')));
		for (i=0;i<6;i++)
		{
			myLevel.AddPrecacheMaterial(Material(DynamicLoadObject(Default.CrowdMaterial[i],Class'Material')));
		}
	}

	// load voice classes
	DynamicLoadObject("FragOps.Voice_MercMaleVoice",Class'Class');
	//DynamicLoadObject("FragOps.Voice_MercFemaleVoice",Class'Class');
}

event InitGame(string Options, out string Error)
{
	//local LadderVolume LV;

	//foreach AllActors(class'LadderVolume',LV)
		//LV.bAllowLadderStrafing = true;

	Super.InitGame(Options, Error);
	if (Level.NetMode == NM_DedicatedServer)
		log("FragOps: Waiting for human players...");
	bPlayersMustBeReady = true;
	bWaitForNetPlayers = true;
	Level.RagdollTimeScale = 1.2;
	MinNetPlayers = 100;
	MaxLives = 1; //1 Life
	GoalScore = 0;
}

function AddDefaultInventory(pawn PlayerPawn)
{
	local class<Weapon_Base> FOMeleeClass, FOPistolClass, FORifleClass;
	local Weapon FOTempWeapon;
	local Controller PC;

	PC = PlayerPawn.Controller;

	if (PlayerPawn != None)
	{
		if (PC.PlayerReplicationInfo.Team.TeamIndex < 1)
		{
			if (FOBot(PC) == None)
			{
				FOPawn(PlayerPawn).EquipList[0] = class'Interface_BuyMenuPlus'.Default.Melee[0];
				FOPawn(PlayerPawn).EquipList[1] = class'Interface_BuyMenuPlus'.Default.Sidearms[0];
			}
			else
			{
				if (PlayerCRI(PC).Money >= 900 && fRand() < 0.75)
				{

					if (fRand() < 0.75)
						FORifleClass = class'Interface_BuyMenuPlus'.Default.Rifles[8];
					else if (fRand() < 0.97)
						FORifleClass = class'Interface_BuyMenuPlus'.Default.Rifles[10];
					else
						FORifleClass = class'Interface_BuyMenuPlus'.Default.Rifles[12];
					PlayerCRI(PC).AddMoney(-(Clamp(600/FOBot(PC).Skill, 300, 800)));
				}
				else if (PlayerCRI(PC).Money >= 475)
				{
					if (fRand() < 0.5)
						FORifleClass = class'Interface_BuyMenuPlus'.Default.Rifles[0];
					else if (fRand() < 0.5)
						FORifleClass = class'Interface_BuyMenuPlus'.Default.Rifles[2];
					else
						FORifleClass = class'Interface_BuyMenuPlus'.Default.Rifles[6];
					PlayerCRI(PC).AddMoney(-(Clamp(400/FOBot(PC).Skill, 300, 500)));
				}
			}
			FOMeleeClass = class'Interface_BuyMenuPlus'.Default.Melee[0];
			FOPistolClass  = class'Interface_BuyMenuPlus'.Default.Sidearms[0];
		}
		else
		{
			if (FOBot(PC) == None)
			{
				FOPawn(PlayerPawn).EquipList[0] = class'Interface_BuyMenuPlus'.Default.Melee[1];
				FOPawn(PlayerPawn).EquipList[1] = class'Interface_BuyMenuPlus'.Default.Sidearms[1];
			}
			else
			{
				if (PlayerCRI(PC).Money >= 900 && fRand() < 0.75)
				{
					if (fRand() < 0.75)
						FORifleClass = class'Interface_BuyMenuPlus'.Default.Rifles[9];
					else if (fRand() < 0.97)
						FORifleClass = class'Interface_BuyMenuPlus'.Default.Rifles[11];
					else
						FORifleClass = class'Interface_BuyMenuPlus'.Default.Rifles[13];
					PlayerCRI(PC).AddMoney(-(Clamp(600/FOBot(PC).Skill, 300, 800)));
				}
				else if (PlayerCRI(PC).Money >= 475)
				{
					if (fRand() < 0.5)
						FORifleClass = class'Interface_BuyMenuPlus'.Default.Rifles[1];
					else if (fRand() < 0.5)
						FORifleClass = class'Interface_BuyMenuPlus'.Default.Rifles[3];
					else
						FORifleClass = class'Interface_BuyMenuPlus'.Default.Rifles[7];
					PlayerCRI(PC).AddMoney(-(Clamp(400/FOBot(PC).Skill, 300, 500)));
				}
			}
			FOMeleeClass = class'Interface_BuyMenuPlus'.Default.Melee[1];
			FOPistolClass  = class'Interface_BuyMenuPlus'.Default.Sidearms[1];
		}

		FOPawn(PlayerPawn).AddNewInventory(FOMeleeClass,false);
		FOTempWeapon = FOPawn(PlayerPawn).AddNewInventory(FOPistolClass,(FORifleClass == None));
		
		if (FORifleClass != None)
			FOTempWeapon = FOPawn(PlayerPawn).AddNewInventory(FORifleClass,true);
	}
	SetPlayerWeapon(FOTempWeapon,PC);
	SetPlayerDefaults(PlayerPawn);
}

function SetPlayerWeapon(weapon WTempWeapon, Controller WController)
{
	if (FOBot(WController) != None)
		WController.SwitchToBestWeapon();
}

function StartMatch()
{
	local Actor A;

	if (bWaitingToStartMatch)
		return;

	GotoState('MatchInProgress');

	if (Level.NetMode == NM_Standalone)
	        RemainingBots = InitialBots;
	else
		RemainingBots = 0;

	OrigTimeLimit = TimeLimit;

	RemainingTime = 60 * TimeLimit;

	GameReplicationInfo.RemainingMinute = RemainingTime;
	FragOps_GRI(GameReplicationInfo).bAllowBehindview = bAllowThirdPersonSpec;

	if (GameStats != None)
		GameStats.StartGame();

	// tell all actors the game is starting
	ForEach AllActors(class'Actor', A)
	{
		if (FragOps_PRI(A) != None)
			A.Reset();
		A.MatchStarting();
	}

	bWaitForNetPlayers = false;
	GameReplicationInfo.bMatchHasBegun = true;

	log("FragOps: START MAP");

	StartRound();
}

function StartRound()
{
	local Controller P;
        local int Num;
	local Actor aTempActor;
	local Mover aTempMover;

	if (IsInState('MatchOver'))
		return;

	log("FragOps: START ROUND");

	FragOps_GRI(GameReplicationInfo).bPreRound = true;
	FragOps_GRI(GameReplicationInfo).RoundCount++;

	RoundKills[0] = 0;
	RoundKills[1] = 0;

	bRoundEnded = false;

	foreach AllActors(class'Actor',aTempActor)
	{
		if ((StaticMeshActor(aTempActor) == None || War_Buildable(aTempActor) != None) && TriggerLight(aTempActor) == None && Objective_RetrievalVolume(aTempActor) == None && LevelInfo(aTempActor) == None && Effect_BreakingGlass(aTempActor) == None && FOAmbientCreatures(aTempActor) == None && FOPlayer(aTempActor) == None && FOBot(aTempActor) == None && FragOps_PRI(aTempActor) == None && FragOps_GRI(aTempActor) == None && FragOps_CRI(aTempActor) == None && UnrealTeamInfo(aTempActor) == None && xTeamRoster(aTempActor) == None && DMRoster(aTempActor) == None && aTempActor != self)
			aTempActor.Reset();
		if (FOWeaponPickup(aTempActor) != None || Weapon_GrenadeHEProjectile(aTempActor) != None || Weapon_GrenadeSmokeProj(aTempActor) != None || Weapon_GrenadeFlashProj(aTempActor) != None || Weapon_LAWProj(aTempActor) != None)
			aTempActor.Destroy();

		// reset all movers
		aTempMover = Mover(aTempActor);
		if (aTempMover != None)
			aTempMover.Reset();			

		// fix for triggerlights
		if (TriggerLight(aTempActor) != None && aTempActor.GetStateName() == 'TriggerTurnsOff')
		{
			aTempActor.GotoState('TriggerTurnsOn');
			TriggerEvent(aTempActor.Tag, aTempActor, None);
			aTempActor.GotoState('TriggerTurnsOff');			
		}
	}

	bMustJoinBeforeStart = false;
        FragOps_GRI(GameReplicationInfo).RoundTimeRemaining = 0;

	while (NeedPlayers() && Num < 16)
	{
		if (FragOps_GRI(GameReplicationInfo).RoundCount == 1)
		{
			if (AddBot("fuckingbugs"))
				RemainingBots--;
		}
		else
		{
			if (AddBot())
				RemainingBots--;
		}
		Num++;
	}

	SpecialPreRound();

	// start living players
	for (P=Level.ControllerList;P!=None;P=P.nextController)
		if (P.IsA('PlayerController') && P.bIsPlayer && !FOPlayer(P).IsDead() && (!P.PlayerReplicationInfo.bOutOfLives || (P.Pawn != None && P.Pawn.Health > 0)) && PlayerCRI(P) != None && !PlayerCRI(P).bWaitingForTeam )
		{
		        P.bFire = 0;
		        P.bAltFire = 0;
			if (P.Pawn.Weapon != None)
			{
				P.Pawn.Weapon.StopFire(0);
				P.Pawn.Weapon.StopFire(1);
			}
			RestartLivingPlayer(P);
		}

	// start non-living players next
	for (P=Level.ControllerList;P!=None;P=P.nextController)
		if (P.IsA('PlayerController') && P.bIsPlayer && PlayerCRI(P) != None && !PlayerCRI(P).bWaitingForTeam && (P.PlayerReplicationInfo.bOutOfLives || P.Pawn == None || P.Pawn.Health <= 0 || FOPlayer(P).IsDead()))
		{
			if (P.Pawn != None)
				P.Pawn.Destroy();

			P.PlayerReplicationInfo.bOutOfLives = False;
			P.PlayerReplicationInfo.NumLives = 0;
			RestartDeadPlayer(P);
		}

	// start AI players
	for (P=Level.ControllerList;P!=None;P=P.nextController)
		if (P.bIsPlayer && !P.IsA('PlayerController'))
		{
			if (P.Pawn != None)
				P.Pawn.Destroy();

		        P.PlayerReplicationInfo.bOutOfLives = False;
			P.PlayerReplicationInfo.NumLives = 0;
			RestartDeadPlayer(P);
		}

	// NOW put everyone in the preround state -- avoids client missing location updates before overriding movement in
	// the playercontroller 'PreRound' state
	for (P=Level.ControllerList;P!=None;P=P.nextController)
		if (P.IsA('PlayerController') && P.Pawn != None)
		{
			FragOps_PRI(P.PlayerReplicationInfo).ResetRoundPRI();
			if (PlayerCRI(P) != None)
				PlayerCRI(P).ResetRoundCRI();
			FOPlayer(P).GotoState('PreRound');
			FOPawn(P.Pawn).CheckBuyZones();
		}
		else if (FOBot(P) != None && P.Pawn != None && PlayerCRI(P) != None)
		{
			FragOps_PRI(P.PlayerReplicationInfo).ResetRoundPRI();
			PlayerCRI(P).ResetRoundCRI();
			FOBot(P).GotoState('PreRound');
		}
		if (FragOps_GRI(GameReplicationInfo).RoundCount == 1 && Level.NetMode == NM_DedicatedServer)
			PreRoundTime = FirstRoundWaitTime;
		else
			PreRoundTime = PreRoundWaitTime;
}

function CheckBuyZonePlayers(Pawn bzPawn)
{
	local FOPawn CheckPawn;

	CheckPawn = FOPawn(bzPawn);

	if (CheckPawn != None)
		CheckPawn.CheckBuyZones();
}

function SpecialPreRound();

function bool CheckEndGame(PlayerReplicationInfo Winner, string Reason)
{
	local Controller C;
	local PlayerController Player;

	if (TimeTillNextRound > 0 && RemainingTime <= 0)
	{
		// now we'll actually check for the end of the game ;)
		if (Teams[1].Score > Teams[0].Score)
			GameReplicationInfo.Winner = Teams[1];
		else
			GameReplicationInfo.Winner = Teams[0];
		
		if (Winner == None)
		{
			for (C=Level.ControllerList; C!=None; C=C.nextController )
				if ( (C.PlayerReplicationInfo != None) && (C.PlayerReplicationInfo.Team == GameReplicationInfo.Winner)
					&& ((Winner == None) || (C.PlayerReplicationInfo.Score > Winner.Score)))
				{
					Winner = C.PlayerReplicationInfo;
				}
		}

		EndTime = Level.TimeSeconds + EndTimeDelay;

		if (Winner != None)
			EndGameFocus = Controller(Winner.Owner).Pawn;
		if (EndGameFocus != None)
			EndGameFocus.bAlwaysRelevant = true;

		for (C=Level.ControllerList; C!=None; C=C.nextController)
		{
			player = PlayerController(C);
			if (Player != None)
			{
				PlayWinMessage(Player, (Player.PlayerReplicationInfo.Team == GameReplicationInfo.Winner));
				player.ClientSetBehindView(true);
				if ( EndGameFocus != None )
				{
					Player.ClientSetViewTarget(EndGameFocus);
					Player.SetViewTarget(EndGameFocus);
				}
				Player.ClientGameEnded();
				if (CurrentGameProfile != None)
					CurrentGameProfile.bWonMatch = (Player.PlayerReplicationInfo.Team == GameReplicationInfo.Winner);
			}
			C.GameHasEnded();
			C.PlayerReplicationInfo.bOutOfLives = false;
			
			// tally final scores
			FragOps_PRI(C.PlayerReplicationInfo).ResetRoundPRI();
		}
		return true;
	}
	else
		return false;
}

function CheckEndRound()
{
	local Controller C;
	local bool bTeam0Alive;
	local bool bTeam1Alive;
	local int TotalPlaying;
	local int TeamWinner;

	TotalPlaying = 0;
	TeamWinner = 2;

	//New round is about to start, don't do any round ending checking
	if (bRoundEnded || FragOps_GRI(GameReplicationInfo).bPreRound || TimeTillNextRound > 0)
		return;

	//Is a team out of action?
	bTeam0Alive = false;
	bTeam1Alive = false;
        for (C=Level.ControllerList; C!=None; C=C.NextController)
	{
		if (C.bIsPlayer || FOBot(C) != None)
			TotalPlaying++;

		if ((C.PlayerReplicationInfo != None) && (C.PlayerReplicationInfo.Team != None) && (C.bIsPlayer || FOBot(C) != None)
                && (!C.PlayerReplicationInfo.bOutOfLives && (C.Pawn != None) && (C.Pawn.Health > 0) ) && C.PlayerReplicationInfo.Team.TeamIndex == 0)
			bTeam0Alive = true;
			
		if ((C.PlayerReplicationInfo != None) && (C.PlayerReplicationInfo.Team != None) && (C.bIsPlayer || FOBot(C) != None)
                && (!C.PlayerReplicationInfo.bOutOfLives && (C.Pawn != None) && (C.Pawn.Health > 0) ) && C.PlayerReplicationInfo.Team.TeamIndex == 1)
			bTeam1Alive = true;
	}

	if (bTeam1Alive && !bTeam0Alive && TotalPlaying > 1)
		TeamWinner = 1;
	else if (bTeam0Alive && !bTeam1Alive && TotalPlaying > 1)
		TeamWinner = 0;

	if (TeamWinner < 2 && (!bTeam0Alive || !bTeam1Alive))
		EndRound(TeamWinner);
}

function EndRound(int WinningTeam, optional bool bNoAnnounce)
{
	local Controller C;
	local int LivingPlayers;
	local PlayerReplicationInfo SurvivorPRI;
	local FragOps_CRI SurvivorCRI;

	if (WinningTeam < 3 && !bNoAnnounce)
		BroadcastLocalizedMessage(class'Interface_GameMessage', (WinningTeam + 6));

        for (C=Level.ControllerList;C!=None;C=C.NextController)
	{
		if (PlayerCRI(C) == None || PlayerCRI(C).bWaitingForTeam || !C.bIsPlayer || C.PlayerReplicationInfo.bOnlySpectator)
			continue;

		if (WinningTeam < 2 && !FragOps_PRI(C.PlayerReplicationInfo).bOutOfLives)
		{
			LivingPlayers++;
			SurvivorCRI = PlayerCRI(C);
			SurvivorPRI = C.PlayerReplicationInfo;
		}
		if ((C.PlayerReplicationInfo.Team.TeamIndex == WinningTeam) && C.bIsPlayer)
			PlayerCRI(C).AddMoney(WinningTeamMoney);
		else if (C.bIsPlayer && !C.PlayerReplicationInfo.bOnlySpectator)
			PlayerCRI(C).AddMoney(LosingTeamMoney);
		CheckWeaponMedals(PlayerCRI(C),C.PlayerReplicationInfo,FOPlayer(C));
	}

	// award a survivor medal if applicable
	if ((SurvivorCRI != None) && LivingPlayers == 1 && NumPlayers+NumBots >= 4 && SurvivorPRI.Team.Size >= 2 && SurvivorPRI.Team.Size < NumPlayers+NumBots)
	{
		SurvivorCRI.AddMedal(0);
		SurvivorCRI.AddMoney(400);
	}	
	if (WinningTeam < 2)
	{
		Teams[WinningTeam].Score += 1;
	}
	else if (WinningTeam < 3)
	{
		Teams[0].Score += 1;
		Teams[1].Score += 1;
	}

	if (WinningTeam < 3)
	{
		BroadcastRoundStats(WinningTeam);
		for (C=Level.ControllerList; C!=None; C=C.NextController)
		{
			if (C.bIsPlayer && C.Pawn != None)
				FOPawn(C.Pawn).bRestrictDamage = true;
		}
		TimeTillNextRound = 5;
	}
	else
		StartRound();
}

function PlayWinMessage(PlayerController Player, bool bWinner)
{
	if (FOPlayer(Player) != None)
		FOPlayer(Player).PlayWinMessage(bWinner);
}

function CheckWeaponMedals(FragOps_CRI mPlayerCRI, PlayerReplicationInfo mPlayerPRI, FOPlayer APPlayer)
{
	local int NewMoney, OtherTeam;
	local Controller C;
	local int RoundPlayers[2];

	if (mPlayerPRI.Team.TeamIndex == 0)
		OtherTeam = 1;

        for (C=Level.ControllerList;C!=None;C=C.NextController)
	{	
		if (C.GetStateName() != 'JustJoined' && C.PlayerReplicationInfo != None && C.PlayerReplicationInfo.Team != None && PlayerCRI(C) != None && (!PlayerCRI(C).bWaitingForTeam || FOBot(C) != None))
		{
			RoundPlayers[C.PlayerReplicationInfo.Team.TeamIndex] += 1;
		}
	}

	// novice pistol medal, reqs: 1 pistol kill, 33% headshot accuracy
	if (mPlayerCRI.Medals[1] == 0 && mPlayerCRI.mPistolPoints[0] >= 1 && mPlayerCRI.mPistolPoints[1] >= ((mPlayerCRI.mPistolPoints[1] + mPlayerCRI.mPistolPoints[2] + mPlayerCRI.mPistolPoints[3]) * 0.33))
	{
		mPlayerCRI.AddMedal(1);
		NewMoney += 200;
	}

	// advanced pistol medal, reqs: 2 kills, and 50% headshot accuracy or better
	if (mPlayerCRI.Medals[2] == 0 && mPlayerCRI.mPistolPoints[0] >= 2 && mPlayerCRI.mPistolPoints[1] >= ((mPlayerCRI.mPistolPoints[1] + mPlayerCRI.mPistolPoints[2] + mPlayerCRI.mPistolPoints[3]) * 0.5))
	{
		mPlayerCRI.AddMedal(2);
		NewMoney += 300;
	}

	// expert pistol medal, reqs: 2 kills, and 90% headshot accuracy
	if (mPlayerCRI.Medals[3] == 0 && mPlayerCRI.mPistolPoints[0] >= 2 && mPlayerCRI.mPistolPoints[1] >= ((mPlayerCRI.mPistolPoints[1] + mPlayerCRI.mPistolPoints[2] + mPlayerCRI.mPistolPoints[3]) * 0.9))
	{
		mPlayerCRI.AddMedal(3);
	}

	// dont award major medals in the first round
	if (FragOps_GRI(GameReplicationInfo).RoundCount > 1 && RoundPlayers[OtherTeam] > 1)
	{
		// army cross, advanced, reqs: player must have killed 75% of the other team
		if (mPlayerCRI.Medals[4] == 0 && APPlayer != None && FragOps_PRI(mPlayerPRI).RoundKills > 0 && RoundPlayers[OtherTeam] > 0 && float(FragOps_PRI(mPlayerPRI).RoundKills)/float(RoundPlayers[OtherTeam]) >= 0.75)
		{
			mPlayerCRI.AddMedal(4);
			NewMoney += 1000;
		}
		
		// silver star, expert, reqs: player must have killed entire other team
		if (mPlayerCRI.Medals[5] == 0 && APPlayer != None && FragOps_PRI(mPlayerPRI).RoundKills > 0 && RoundPlayers[OtherTeam] > 0 && float(FragOps_PRI(mPlayerPRI).RoundKills)/float(RoundPlayers[OtherTeam]) >= 1.0)
		{
			mPlayerCRI.AddMedal(5);
			FragOps_PRI(mPlayerPRI).APModPerm++;
		}
	}

	if (NewMoney > 0)
		mPlayerCRI.AddMoney(NewMoney);
}

// for later use?  Don't know.
function CheatKick(Controller CheaterController, int CheatType)
{
}

function BroadCastRoundStats(byte NewWinner)
{
	local Controller C;
	local int MaxDamageRating, aMostDamageScore, MinDamageRating, aMinDamageScore, HeadShotRating, aHeadShotsScore, TempOverall, OverallRating;
	local float TempAccuracy, AccuracyRating;
	local bool bNoMinDamage, bNoMaxDamage, bNoAccuracy, bNoHeadShots, bNoOverall;
	local string aMostDamage, aMinDamage, aAccuracy, aHeadShot, aOverall, aOverallScore, aAccuracyScore;
	local FragOps_PRI FOPRI;

	// roundstats.  A nice secondary scoreboard for the end of each round.

	// collect stats
	for (C=Level.ControllerList;C!=None;C=C.NextController)
	{
		if (C.bIsPlayer)
		{
			FOPRI = FragOps_PRI(C.PlayerReplicationInfo);
			
			if (C.IsInState('JustJoined') || FOPRI == None || (FOPlayer(C) != None && FOPlayer(C).CriticalReplicationInfo != None && FOPlayer(C).CriticalReplicationInfo.bWaitingForTeam))
				continue;

			// get the most damaging player
			if (FOPRI.RoundDamageDone > MaxDamageRating || !bNoMaxDamage)
			{
				aMostDamage = FOPRI.PlayerName;
				MaxDamageRating = FOPRI.RoundDamageDone;
				aMostDamageScore = MaxDamageRating;
				bNoMaxDamage = true;
			}
			
			// get the least damaging player
			if (FOPRI.RoundDamageDone < MinDamageRating || !bNoMinDamage)
			{
				aMinDamage = FOPRI.PlayerName;
				MinDamageRating = FOPRI.RoundDamageDone;
				aMinDamageScore = MinDamageRating;
				bNoMinDamage = true;
			}

			// get the most accurate
			if (FOPRI.RoundShotsHit == 0 || FOPRI.RoundShots == 0)
				TempAccuracy = 0;
			else
				TempAccuracy = float(FOPRI.RoundShotsHit) / float(FOPRI.RoundShots);

			if (TempAccuracy > AccuracyRating || !bNoAccuracy)
			{
				AccuracyRating = TempAccuracy;
				aAccuracy = FOPRI.Playername;
				aAccuracyScore = Max(0,TempAccuracy*100)$"% [H/S: "$FOPRI.RoundShotsHit$"/"$FOPRI.RoundShots$"]";
				bNoAccuracy = true;
			}

			// get the most headshots
			if (FOPRI.RoundHeadshots > HeadshotRating || !bNoHeadShots)
			{
				HeadShotRating = FOPRI.RoundHeadShots;
				aHeadShot = FOPRI.PlayerName;
				aHeadShotsScore = FOPRI.RoundHeadShots;
				bNoHeadShots = true;
			}

			// get the best overall
			TempOverall = FOPRI.RoundDamageDone - FOPRI.RoundDamageTaken;
			TempOverall = TempOverall + (TempAccuracy*100);
			if (TempOverall > OverallRating || !bNoOverall)
			{
				OverallRating = TempOverall;
				aOverall = FOPRI.PlayerName;
				aOverallScore = (FOPRI.RoundDamageDone - FOPRI.RoundDamageTaken)$" [A: "$Max(0,TempAccuracy*100)$"%]";
				bNoOverall = true;
			}
		}
	}

	// broadcast stats
	for (C=Level.ControllerList;C!=None;C=C.NextController)
	{
		if (C.bIsPlayer && C.IsA('PlayerController'))
		{
			FOPlayer(C).ClientReceiveRoundStats(aMostDamage,aMostDamageScore,aMinDamage,aMinDamageScore,aAccuracy,aAccuracyScore,aHeadShot,aHeadShotsScore,aOverall,aOverallScore,FOPlayer(C).CriticalReplicationInfo.ParseMedals,NewWinner);
			if (!FOPlayer(C).IsDead())
				FOPlayer(C).ReceivePlayerStats();
		}
	}
}

state MatchInProgress
{   
	function Timer()
	{
		local Controller B;
		local Objective_RetrievalVolume ORV;
 
		Global.Timer();
		if (PreRoundTime > 0)
		{
			PreRoundTime--;
			FragOps_GRI(GameReplicationInfo).bPreRound = true;
			if (PreRoundTime <= 0)
			{
				FragOps_GRI(GameReplicationInfo).bPreRound = false;
				FragOps_GRI(GameReplicationInfo).RoundTimeRemaining = RoundTime * 60;
				RoundTimeRemaining = RoundTime * 60;
				BroadcastLocalizedMessage(class'Interface_GameMessage', (0));
				for (B = Level.ControllerList; B!=None; B=B.nextController)
				{
					if (B.bIsPlayer && !PlayerCRI(B).bWaitingForTeam && B.IsA('PlayerController'))
					{
						if (FOPawn(B.Pawn) != None)
							FOPawn(B.Pawn).bRestrictDamage = false;
						FOPlayer(B).GotoState('PlayerWalking');
						FOPlayer(B).MoveOutAnnounce();
					}
					else if (B.bIsPlayer && !B.IsA('PlayerController'))
					{
						if (FOPawn(B.Pawn) != None)
							FOPawn(B.Pawn).bRestrictDamage = false;
						FOBot(B).GotoState('Hunting','Begin');
					}
				}
				foreach AllActors(class'Objective_RetrievalVolume',ORV)
					ORV.Reset();
				PreRoundTime = 0;
			}
		}

		if (TimeTillNextRound > 0 || bForceEndRound)
		{
			if (TimeTillNextRound > 0)
				TimeTillNextRound--;
			if (TimeTillNextRound == 0 || bForceEndRound)
			{
				StartRound();
			}
			bForceEndRound = false;
		}

		if ( !bFinalStartup )
		{
			bFinalStartup = true;
			PlayStartupMessage();
		}

		if (NeedPlayers() && AddBot() && (RemainingBots > 0))
			RemainingBots--;

		if (TimeLimit > 0)
		{
			GameReplicationInfo.bStopCountDown = false;
			RemainingTime--;
			GameReplicationInfo.RemainingTime = RemainingTime;
			if (RemainingTime % 60 == 0)
				GameReplicationInfo.RemainingMinute = RemainingTime;
			if (RemainingTime <= 0 && RoundTimeRemaining <= 0)
				EndGame(None,"TimeLimit");
		}

		if (RoundTime > 0 && PreRoundTime <= 0)
		{
			if (TimeTillNextRound <= 0)
			{
				RoundTimeRemaining--;
				CheckTriggers((RoundTime*60)-RoundTimeRemaining);
			}
			else if (RemainingTime <= 0)
				RoundTimeRemaining = -5;

			FragOps_GRI(GameReplicationInfo).RoundTimeRemaining = RoundTimeRemaining;
			if (RoundTimeRemaining % 60 == 0)
				FragOps_GRI(GameReplicationInfo).RoundTimeMinute = RoundTimeRemaining;
			if (RoundTimeRemaining <= 0 && TimeTillNextRound <= 0)
				EndRound(2);
		}

		ElapsedTime++;
		GameReplicationInfo.ElapsedTime = ElapsedTime;
	}

	function Beginstate()
	{
		local PlayerReplicationInfo PRI;

		ForEach DynamicActors(class'PlayerReplicationInfo',PRI)
			PRI.StartTime = 0;
		ElapsedTime = 0;
		bWaitingToStartMatch = false;
		StartupStage = 5;
		StartupStage = 6;
	}
}

function CheckTriggers(int iElapsed)
{
	local int i;

	if (TimedTrigger == None)
		return;

	for (i=0;i<4;i++)
	{
		if (TimedTrigger.TriggerTime[i] != 0 && TimedTrigger.TriggerTime[i] == byte(iElapsed))
		{
			if (TimedTrigger.TriggerMessage[i] != "")
				BroadcastLocalizedMessage(class'Interface_TriggerMessage', i, None, None, TimedTrigger);
			TimedTrigger.TimedEventTrigger(i);
		}
	}
}

function PlayStartUpMessage()
{}

function bool AddBot(optional string botName)
{
	local Bot NewBot;
	
	if (bWaitingToStartMatch)
		return false;

	if (botName ~= "fuckingbugs")
		NewBot = SpawnBot("");
	else
		NewBot = SpawnBot(botName);

	if ( NewBot == None )
	{
		warn("Failed to spawn bot.");
		return false;
	}

	if (botName != "fuckingbugs")
		BroadcastLocalizedMessage(GameMessageClass, 1, NewBot.PlayerReplicationInfo);

	NewBot.PlayerReplicationInfo.PlayerID = CurrentID++;
	NumBots++;
	if (Level.NetMode == NM_Standalone)
		RestartDeadPlayer(NewBot);
	else
		NewBot.GotoState('Dead','MPStart');

	return true;
}

function EndGame(PlayerReplicationInfo Winner, string Reason )
{
	if ( (Reason ~= "triggered") ||
		(Reason ~= "LastMan") ||
		(Reason ~= "TimeLimit"))
	{

		if (!CheckEndGame(Winner, Reason))
		{
			return;
		}

		bGameEnded = true;
		TriggerEvent('EndGame', self, None);
		EndLogging(Reason);
		if (bGameEnded)
			GotoState('MatchOver');
	}
}

function Killed( Controller Killer, Controller Killed, Pawn KilledPawn, class<DamageType> damageType )
{
	local bool bEnemyKill;

	bEnemyKill = ( !bTeamGame || ((Killer != None) && (Killer != Killed) && (Killed != None)
	&& (Killer.PlayerReplicationInfo != None) && (Killed.PlayerReplicationInfo != None)
	&& (Killer.PlayerReplicationInfo.Team != Killed.PlayerReplicationInfo.Team)));

	if ((Killer != None) && Killer.bIsPlayer && (Killed != None) && Killed.bIsPlayer)
	{
		DamageType.static.IncrementKills(Killer);
	}

	if ((Killed != None) && Killed.bIsPlayer)
	{
		if (RoundTimeRemaining > 0 && !FragOps_GRI(GameReplicationInfo).bPreRound)
		{
			Killed.PlayerReplicationInfo.Deaths += 1;
			BroadcastKillMessage(damageType, Killer.PlayerReplicationInfo, Killed.PlayerReplicationInfo);
		}
		else
			return;
		if ( (Killer == Killed) || (Killer == None))
		{
			if (Killer == None )
				KillEvent("K", None, Killed.PlayerReplicationInfo, DamageType);	//"Kill"
			else
				KillEvent("K", Killer.PlayerReplicationInfo, Killed.PlayerReplicationInfo, DamageType);	//"Kill"
		}
		else
		{
			if (bTeamGame && (Killer.PlayerReplicationInfo != None)
				&& (Killer.PlayerReplicationInfo.Team == Killed.PlayerReplicationInfo.Team) )
			{
				KillEvent("TK", Killer.PlayerReplicationInfo, Killed.PlayerReplicationInfo, DamageType);	//"Teamkill"
			        PlayerCRI(Killer).AddMoney(-(8*KillMoney));
			}
			else
			{
				KillEvent("K", Killer.PlayerReplicationInfo, Killed.PlayerReplicationInfo, DamageType);	//"Kill"
				CheckKillType(Killer,damageType);
				PlayerCRI(Killer).AddMoney(KillMoney);
			}
		}
	}
	if (Killed != None)
	{
		FOPlayer(Killed).ReceivePlayerStats();
		Killed.PlayerReplicationInfo.bOutOfLives = true;
	}

	if (Killer != None)
		ScoreKill(Killer, Killed);	
	else if (Killed != None && Killer == None)
		FragOps_PRI(Killed.PlayerReplicationInfo).AddDamageTaken(100, 'spine', FragOps_PRI(Killed.PlayerReplicationInfo));

	DiscardInventory(KilledPawn);
	NotifyKilled(Killer,Killed,KilledPawn);
	CheckEndRound();
}

function BroadcastKillMessage(class<DamageType> bDamageType, PlayerReplicationInfo Killer, PlayerReplicationInfo Killed)
{
	local Controller C;
	
	for (C=Level.ControllerList;C!=None;C=C.NextController)
	{
		if (FOPlayer(C) == None)
			continue;

		FOPlayer(C).ReceiveKillMessage(bDamageType, Killer, Killed);
	}
}

// check for medal kill additions
function CheckKillType(Controller WeaponKiller, class<DamageType> KillDamageType)
{
	local string WeaponKillType;

	PlayerCRI(WeaponKiller).AddMoney(KillMoney);
	WeaponKillType = KillDamageType.default.DeathString;
	if ((KillDamageType.Name == 'DamType_DE' || KillDamageType.Name == 'DamType_USP' || KillDamageType.Name == 'DamType_8x' || KillDamageType.Name == 'DamType_Sig') && WeaponKiller.PlayerReplicationInfo != None)
		PlayerCRI(WeaponKiller).mPistolPoints[0]++;
}

/* Discard a player's inventory after he dies.
*/
function DiscardInventory(Pawn Other)
{
	local int i;

	Other.Weapon = None;
	Other.SelectedItem = None;

	for(i=0;i<ArrayCount(FOPawn(Other).EquipList);i++)
	{
		FOPawn(Other).EquipList[i] = None;
		FOPawn(Other).AttachList[i] = None;
	}

	FOPawn(Other).bCanBuy = false;
	if (FOPawn(Other).CriticalReplicationInfo.Money - DeathMoney > 0)
		FOPawn(Other).CriticalReplicationInfo.AddMoney(-DeathMoney);
	else
		FOPawn(Other).CriticalReplicationInfo.Money = 0;

	while (Other.Inventory != None)
		Other.Inventory.Destroy();
}

// player restarts, broken into living, dead and universal
function RestartLivingPlayer( Controller aPlayer )
{
	local TeamInfo BotTeam, OtherTeam;
	local NavigationPoint startSpot;
	local int TeamNum;

	if ( bBalanceTeams && (Bot(aPlayer) != None) )
	{
		BotTeam = aPlayer.PlayerReplicationInfo.Team;
		if ( BotTeam == Teams[0] )
			OtherTeam = Teams[1];
		else
			OtherTeam = Teams[0];

		if ( OtherTeam.Size < BotTeam.Size - 1 )
		{
			aPlayer.Destroy();
			return;
		}
	}

	TeamNum = aPlayer.PlayerReplicationInfo.Team.TeamIndex;

	if ( aPlayer.IsA('Bot') && TooManyBots(aPlayer) )
	{
		aPlayer.Destroy();
		return;
	}

	startSpot = FindPlayerStart(aPlayer, TeamNum);
	if (startSpot == None)
	{
		log(" Player start not found!!!");
		return;
	}

	aPlayer.Pawn.Anchor = startSpot;
	aPlayer.Pawn.LastStartSpot = PlayerStart(startSpot);
	aPlayer.Pawn.LastStartTime = Level.TimeSeconds;
	aPlayer.Pawn.SetLocation(StartSpot.Location);
	SetLivePlayerDefaults(aPlayer.Pawn);
	SetPlayerWeapon(None,aPlayer);
	TriggerEvent( StartSpot.Event, StartSpot, aPlayer.Pawn );
	CheckBuyZonePlayers(aPlayer.Pawn);
}

function RestartDeadPlayer( Controller aPlayer )
{
	local TeamInfo BotTeam, OtherTeam;
	local NavigationPoint startSpot;
	local int TeamNum;

	if (bBalanceTeams && (Bot(aPlayer) != None))
	{
		BotTeam = aPlayer.PlayerReplicationInfo.Team;
		if ( BotTeam == Teams[0] )
			OtherTeam = Teams[1];
		else
			OtherTeam = Teams[0];

		if ( OtherTeam.Size < BotTeam.Size - 1 )
		{
			aPlayer.Destroy();
			return;
		}
	}

	if (aPlayer.IsA('Bot') && TooManyBots(aPlayer))
	{
		aPlayer.Destroy();
		return;
	}

	if (bRestartLevel && Level.NetMode != NM_DedicatedServer && Level.NetMode != NM_ListenServer)
		return;

	if ((aPlayer.PlayerReplicationInfo == None) || (aPlayer.PlayerReplicationInfo.Team == None))
		TeamNum = 255;
	else
		TeamNum = aPlayer.PlayerReplicationInfo.Team.TeamIndex;

	if (TeamNum != 1 && TeamNum != 0)
	{
		aPlayer.GotoState('Dead');
		return;
	}

	startSpot = FindPlayerStart(aPlayer, TeamNum);
	if (startSpot == None)
	{
		log(" Player start not found!!!");
		return;
	}

	if (aPlayer.Pawn != None)
		aPlayer.Pawn.Destroy();

	if (aPlayer.PreviousPawnClass!=None && aPlayer.PawnClass != aPlayer.PreviousPawnClass)
		BaseMutator.PlayerChangedClass(aPlayer);

	if (aPlayer.PawnClass != None)
		aPlayer.Pawn = Spawn(aPlayer.PawnClass,,,StartSpot.Location,StartSpot.Rotation);

	if (aPlayer.Pawn == None)
	{
		log("Couldn't spawn player of type "$aPlayer.PawnClass$" at "$StartSpot);
		FOPlayer(aPlayer).GotoState('Dead');
		return;
	}
	
	// initialize pawn mesh and skins
	if (aPlayer.IsA('PlayerController'))
	{
		switch(FOPlayer(aPlayer).SelectedTeam)
		{
			case 1:
				aPlayer.Pawn.LinkMesh(class'Interface_TabPlayerTeam'.Default.SORecord[FOPlayer(aPlayer).SelectedRecord].AssociatedModel[FOPlayer(aPlayer).HeadSkin]);
				break;

			default:
				aPlayer.Pawn.LinkMesh(class'Interface_TabPlayerTeam'.Default.MercRecord[FOPlayer(aPlayer).SelectedRecord].AssociatedModel[FOPlayer(aPlayer).HeadSkin]);
				break;
		}
		FOPawn(aPlayer.Pawn).HeadSkin = FOPlayer(aPlayer).HeadSkin;
		FOPawn(aPlayer.Pawn).BodySkin = FOPlayer(aPlayer).BodySkin;
	}
	else
	{
		switch(FOBot(aPlayer).SelectedTeam)
		{
			case 1:
				aPlayer.Pawn.LinkMesh(class'Interface_TabPlayerTeam'.Default.SORecord[FOBot(aPlayer).SelectedRecord].AssociatedModel[FOBot(aPlayer).HeadSkin]);
				break;

			default:
				aPlayer.Pawn.LinkMesh(class'Interface_TabPlayerTeam'.Default.MercRecord[FOBot(aPlayer).SelectedRecord].AssociatedModel[FOBot(aPlayer).HeadSkin]);
				break;
		}
		FOPawn(aPlayer.Pawn).HeadSkin = FOBot(aPlayer).HeadSkin;
		FOPawn(aPlayer.Pawn).BodySkin = FOBot(aPlayer).BodySkin;
	}
	if (Level.NetMode != NM_DedicatedServer)
		aPlayer.Pawn.PostNetBeginPlay();

	aPlayer.Pawn.Anchor = startSpot;
	aPlayer.Pawn.LastStartSpot = PlayerStart(startSpot);
	aPlayer.Pawn.LastStartTime = Level.TimeSeconds;
	aPlayer.PreviousPawnClass = aPlayer.Pawn.Class;

	aPlayer.Possess(aPlayer.Pawn);
	aPlayer.PawnClass = aPlayer.Pawn.Class;
	TriggerEvent(StartSpot.Event, StartSpot, aPlayer.Pawn);

	if (aPlayer.IsA('PlayerController'))
	{
		FOPlayer(aPlayer).ClientSetBehindView(false);
		FOPlayer(aPlayer).SetViewTarget(aPlayer.Pawn);
		FOPlayer(aPlayer).ClientSetViewTarget(aPlayer.Pawn);
		aPlayer.ClientSetRotation(aPlayer.Pawn.Rotation);
	}
	AddDefaultInventory(aPlayer.Pawn);
	TriggerEvent(StartSpot.Event, StartSpot, aPlayer.Pawn);
	FragOps_PRI(aPlayer.PlayerReplicationInfo).bShowSpecHUD = true;
	FragOps_PRI(aPlayer.PlayerReplicationInfo).ResetStats();
	CheckBuyZonePlayers(aPlayer.Pawn);
}

function InitializePlayer( Controller aPlayer )
{
	local bool bTeamAlive;
	local Controller C;

	if (aPlayer.PlayerReplicationInfo.Team.TeamIndex == 0)
		BroadcastLocalizedMessage(class'Interface_GameMessageRed',0,aPlayer.PlayerReplicationInfo);
	else
		BroadcastLocalizedMessage(class'Interface_GameMessageBlue',0,aPlayer.PlayerReplicationInfo);

	for (C=Level.ControllerList;C!=None;C=C.NextController)
	{
		if ((C != aPlayer) && !C.IsA('Bot') && (C.PlayerReplicationInfo != None) && (C.PlayerReplicationInfo.Team != None && C.PlayerReplicationInfo.Team.TeamIndex == aPlayer.PlayerReplicationInfo.Team.TeamIndex))
		{
			bTeamAlive = true;
		}
	}

	if ((NumPlayers > 1 && (!bWaitingToStartMatch || IsInState('MatchInProgress')) && bTeamAlive) || IsInState('MatchOver'))
	{
		if (!FragOps_GRI(GameReplicationInfo).bPreRound)
		{
			FragOps_PRI(aPlayer.PlayerReplicationInfo).bShowSpecHUD = true;
			if (!IsInState('MatchOver'))
				aPlayer.PlayerReplicationInfo.bOutOfLives = true;
			aPlayer.PlayerReplicationInfo.NumLives = 0;
			SendToJoinState(aPlayer);
			return;
		}
	}

	if (aPlayer.PlayerReplicationInfo.bOnlySpectator)
		return;

	if (bWaitingToStartMatch || (!IsInState('MatchInProgress') && NumPlayers < 2))
	{
		bWaitingToStartMatch = false;
		aPlayer.PlayerReplicationInfo.bOutOfLives = true;
		aPlayer.PlayerReplicationInfo.bReadyToPlay = true;
		aPlayer.PlayerReplicationInfo.bWaitingPlayer = false;
		StartMatch();
		return;
	}

	if ((NumPlayers < 3 || !bTeamAlive) && IsInState('MatchInProgress') && !FragOps_GRI(GameReplicationInfo).bPreRound)
	{
		aPlayer.PlayerReplicationInfo.bOutOfLives = true;
		EndRound(3);
		return;
	}

	aPlayer.PlayerReplicationInfo.bOutOfLives = false;
	aPlayer.PlayerReplicationInfo.NumLives = 0;
	RestartDeadPlayer(aPlayer);
	if (FragOps_GRI(GameReplicationInfo).bPreRound)
		aPlayer.GotoState('PreRound','Begin');
}

function SpawnNewPlayer( Controller aPlayer )
{
	local NavigationPoint startSpot;
	local int TeamNum;

	if ( (aPlayer.PlayerReplicationInfo == None) || (aPlayer.PlayerReplicationInfo.Team == None) )
		TeamNum = 255;
	else
		TeamNum = rand(2);

	startSpot = FindPlayerStart(aPlayer, TeamNum);
	if( startSpot == None )
	{
		log(" Player start not found!!!");
		return;
	}

	PlayerController(aPlayer).ClientSetViewTarget(startSpot);
}

// Called after a successful login. This is the first place
// it is safe to call replicated functions on the PlayerPawn.
//
event PostLogin( PlayerController NewPlayer )
{
	local class<HUD> HudClass;
	local class<Scoreboard> ScoreboardClass;
	local String SongName;
	local int num;

	// Log player's login.
	if (GameStats!=None)
	{
		GameStats.ConnectEvent(NewPlayer.PlayerReplicationInfo);
		GameStats.GameEvent("NameChange",NewPlayer.PlayerReplicationInfo.playername,NewPlayer.PlayerReplicationInfo);
	}

	// set player's team to none
	SpawnNewPlayer(NewPlayer);
	NewPlayer.PlayerReplicationInfo.Team = none;
	GameEvent("TeamChange",""$num,NewPlayer.PlayerReplicationInfo);

	// Start player's music.
	SongName = Level.Song;
	if (SongName != "" && SongName != "None")
		NewPlayer.ClientSetInitialMusic( SongName, MTRAN_Fade );

	// set the hud to a more suitable one for team select
	HudClass = class<HUD>(DynamicLoadObject(HUDType, class'Class'));
        ScoreboardClass = class<Scoreboard>(DynamicLoadObject(ScoreBoardType, class'Class'));
	NewPlayer.ClientSetHUD(HudClass, ScoreboardClass);

	SetWeaponViewShake(NewPlayer);

	NewPlayer.ClientCapBandwidth(NewPlayer.Player.CurrentNetSpeed);
	NotifyLogin(NewPlayer.PlayerReplicationInfo.PlayerID);

	if (VotingHandler != None)
		VotingHandler.PlayerJoin(NewPlayer);

	log("New Player"@NewPlayer.PlayerReplicationInfo.PlayerName@"id="@NewPlayer.GetPlayerIDHash());
}

event PlayerController Login(string Portal, string Options, out string Error)
{
	local PlayerController NewPlayer;

	NewPlayer = FOLogin(Portal,Options,Error);

	// voice chat
	if (NewPlayer != None)
	{
		if (Level.NetMode == NM_DedicatedServer || Level.NetMode == NM_ListenServer)
		{
			NewPlayer.VoiceReplicationInfo = VoiceReplicationInfo;
			if (Level.NetMode == NM_ListenServer && Level.GetLocalPlayerController() == NewPlayer)
				NewPlayer.InitializeVoiceChat();
		}
	}
	
	return NewPlayer;
}

//
// Log a player in.
// Fails login if you set the Error string.
// PreLogin is called before Login, but significant game time may pass before
// Login is called, especially if content is downloaded.
//
function PlayerController FOLogin (string Portal, string Options, out string Error)
{
	local NavigationPoint StartSpot, NewPC;
	local array<NavigationPoint> StartSpots;
	local PlayerController NewPlayer;
	local Pawn      TestPawn;
	local string          InName, InAdminName, InPassword, InChecksum;
	local byte            InTeam;
	local bool bSpectator, bAdmin;
	local class<Security> MySecurityClass;
	local rotator TempRotation;
	local int i;

	bSpectator = (ParseOption(Options, "SpectatorOnly" ) ~= "true");
	if (AccessControl != None)
		bAdmin = AccessControl.CheckOptionsAdmin(Options);

	// Make sure there is capacity except for admins. (This might have changed since the PreLogin call).
	if (!bAdmin && AtCapacity(bSpectator))
	{
		Error = GameMessageClass.Default.MaxedOutMessage;
		return None;
	}

	// If admin, force spectate mode if the server already full of reg. players
	if (bAdmin && AtCapacity(false))
		bSpectator = true;

	BaseMutator.ModifyLogin(Portal, Options);

	// Get URL options.
	InName     = Left(ParseOption ( Options, "Name"), 20);
	InTeam     = GetIntOption(Options, "Team", 255); // default to "no team"
	InAdminName= ParseOption(Options, "AdminName");
	InPassword = ParseOption(Options, "Password" );
	InChecksum = ParseOption(Options, "Checksum" );

	log( "Login:" @ InName );

	// find a start spot.
	foreach AllActors(class'NavigationPoint', NewPC)
	{
		if (NewPC == None || PlayerStart(NewPC) == None)
			continue;
		
		StartSpots[i] = NewPC;
		i++;
	}

	StartSpot = StartSpots[Rand(StartSpots.Length)];

	if (PlayerControllerClass == None)
		PlayerControllerClass = class<PlayerController>(DynamicLoadObject(PlayerControllerClassName, class'Class'));

	TempRotation = StartSpot.Rotation;
	TempRotation.Pitch = 0;

	NewPlayer = spawn(PlayerControllerClass,,,StartSpot.Location,TempRotation);

	// Handle spawn failure.
	if (NewPlayer == None)
	{
		log("Couldn't spawn player controller of class "$PlayerControllerClass);
		Error = GameMessageClass.Default.FailedSpawnMessage;
		return None;
	}

	NewPlayer.StartSpot = StartSpot;

	// Init player's replication info
	NewPlayer.GameReplicationInfo = GameReplicationInfo;

	// Apply security to this controller
	MySecurityClass = class<Security>(DynamicLoadObject(SecurityClass,class'class'));
	if (MySecurityClass!=None)
	{
		NewPlayer.PlayerSecurity = spawn(MySecurityClass,NewPlayer);
		if (NewPlayer.PlayerSecurity == None)
		{
			log("Could not spawn security for player "$NewPlayer,'Security');
		}
	}
	else
		log("Unknown security class ["$SecurityClass$"] -- System is no secure.",'Security');

	if (!NewPlayer.IsInState('JustJoined') && !bSpectator && !NewPlayer.PlayerReplicationInfo.bOnlySpectator)
		SendToJoinState(NewPlayer);
	else if (bSpectator || NewPlayer.PlayerReplicationInfo.bOnlySpectator)
	{
		PlayerCRI(NewPlayer).bWaitingForTeam = false;
		NewPlayer.SetViewTarget(NewPlayer);
		NewPlayer.ClientSetViewTarget(NewPlayer);
		NewPlayer.GotoState('Dead');
		NewPlayer.ServerViewNextPlayer();
	}

	// Init player's name
	if (InName=="")
		InName=DefaultPlayerName;
	if (Level.NetMode != NM_Standalone || NewPlayer.PlayerReplicationInfo.PlayerName == DefaultPlayerName) 
		ChangeName( NewPlayer, InName, false );

	if (FOPlayer(NewPlayer).CriticalReplicationInfo != None && !bSpectator)
		FOPlayer(NewPlayer).CriticalReplicationInfo.Money = 400;

	if (bSpectator || NewPlayer.PlayerReplicationInfo.bOnlySpectator)
	{
		NewPlayer.PlayerReplicationInfo.bOnlySpectator = true;
		NewPlayer.PlayerReplicationInfo.bIsSpectator = true;
		NewPlayer.PlayerReplicationInfo.bOutOfLives = true;
		NewPlayer.PlayerReplicationInfo.PlayerID = CurrentID++;
		NumSpectators++;
		return NewPlayer;
	}

	// Init player's administrative privileges and log it
	if (AccessControl != None && AccessControl.AdminLogin(NewPlayer, InAdminName, InPassword))
	{
		AccessControl.AdminEntered(NewPlayer, InAdminName);
	}

	// Set the player's ID.
	NewPlayer.PlayerReplicationInfo.PlayerID = CurrentID++;
	NewPlayer.PlayerReplicationInfo.bReadyToPlay = false;
	NewPlayer.PlayerReplicationInfo.bWaitingPlayer = true;

	NumPlayers++;
	bWelcomePending = true;

	// if delayed start, don't give a pawn to the player yet
	// Normal for multiplayer games
	if ( bDelayedStart )
	{
		NewPlayer.GotoState('PlayerWaiting');
		return NewPlayer;
	}

	// Try to match up to existing unoccupied player in level,
	// for savegames and coop level switching.
	ForEach DynamicActors(class'Pawn', TestPawn )
	{
		if ( (TestPawn!=None) && (PlayerController(TestPawn.Controller)!=None) && (PlayerController(TestPawn.Controller).Player==None) && (TestPawn.Health > 0)
		&&  (TestPawn.OwnerName~=InName) )
		{
			NewPlayer.Destroy();
			TestPawn.SetRotation(TestPawn.Controller.Rotation);
			 TestPawn.bInitializeAnimation = false; // FIXME - temporary workaround for lack of meshinstance serialization
			TestPawn.PlayWaiting();
			return PlayerController(TestPawn.Controller);
		}
	}
	return newPlayer;
}

function SendToJoinState(Controller JPlayer)
{
	JPlayer.GotoState('JustJoined','Begin');
}

function Bot SpawnBot(optional string botName)
{
	local Bot NewBot;
	local RosterEntry Chosen;
	local UnrealTeamInfo BotTeam;

	BotTeam = GetBotTeam();
	Chosen = BotTeam.ChooseBotClass(botName);

	if (Chosen.PawnClass == None)
		Chosen.Init(); //amb
    
	NewBot = Spawn(class'FOBot');
	FOBot(NewBot).SetRandomChar(BotTeam.TeamIndex);

	if ( NewBot != None )
	{
		FragOps_PRI(NewBot.PlayerReplicationInfo).bNoTeamSkins = true;
		InitializeBot(NewBot,BotTeam,Chosen);
	}
	return NewBot;
}

function bool CanSpectate(PlayerController Viewer, bool bOnlySpectator, actor ViewTarget)
{
	if (ViewTarget == None)
		return false;
	if (bOnlySpectator)
	{
		if (Controller(ViewTarget) != None)
		{
			if (Controller(ViewTarget).PlayerReplicationInfo != None && Controller(ViewTarget).PlayerReplicationInfo.bOutOfLives && Viewer != PlayerController(ViewTarget))
				return false;
			return ( (Controller(ViewTarget).PlayerReplicationInfo != None)
				&& !Controller(ViewTarget).PlayerReplicationInfo.bOnlySpectator );
		}
		return true;
	}
	if (Controller(ViewTarget) != None)
		return ( (Controller(ViewTarget).PlayerReplicationInfo != None)
				&& !Controller(ViewTarget).PlayerReplicationInfo.bOnlySpectator
				&& (Controller(ViewTarget).PlayerReplicationInfo.Team != None && Viewer.PlayerReplicationInfo != None && Controller(ViewTarget).PlayerReplicationInfo.Team.TeamIndex == Viewer.PlayerReplicationInfo.Team.TeamIndex) && !Controller(ViewTarget).PlayerReplicationInfo.bOutOfLives );
	return ((Pawn(ViewTarget) != None) && Pawn(ViewTarget).IsPlayerPawn()
		&& (Pawn(ViewTarget).PlayerReplicationInfo.Team.TeamIndex == Viewer.PlayerReplicationInfo.Team.TeamIndex) && Pawn(ViewTarget).Health > 0);
}

function int ReduceDamagePlus(int Damage, Pawn Injured, Pawn InstigatedBy, Controller DamageController, vector HitLocation, out vector Momentum, class<DamageType> DamageType, name HitBone)
{
	local int TempDamage;
	local float NewArmorLevel;
	local TeamInfo InjuredTeam, InstigatorTeam;
	local FOPawn FOPlayerPawn;

	InjuredTeam = Injured.GetTeam();
	if (DamageController != None)
		InstigatorTeam = DamageController.PlayerReplicationInfo.Team;
	else
		InstigatorTeam = None;

	if ((InjuredTeam != None) && (InstigatorTeam != None))
	{
		if (InjuredTeam == InstigatorTeam && Injured != InstigatedBy)
		{
			if (Bot(Injured.Controller) != None && InstigatedBy != None)
				Bot(Injured.Controller).YellAt(InstigatedBy);
			FriendlyFireScale = class'FragOpsMission'.Default.FriendlyFireScale;
			if (FriendlyFireScale == 0.0)
			{
				if (GameRulesModifiers != None)
					return GameRulesModifiers.NetDamage(Damage,0,injured,instigatedBy,HitLocation,Momentum,DamageType);
				else
					return 0;
			}
			Damage *= FriendlyFireScale;
		}
	}

	// spawn protection
	if ((InstigatedBy != None) && (InstigatedBy != Injured) && (Level.TimeSeconds - Injured.SpawnTime < SpawnProtectionTime)
		&& (class<WeaponDamageType>(DamageType) != None))
		return 0;

	// get injured pawn
	FOPlayerPawn = FOPawn(Injured);
	
	// armor check at dmg for cheat protection
	switch (FOPlayerPawn.MiscItem)
	{
		case "MED. Armor":
			NewArmorLevel = 0.66;
			break;
		
		case "HVY. Armor":
			NewArmorLevel = 0.88;
			break;

		default:
			NewArmorLevel = 0.36;
	}

	if (((InjuredTeam == InstigatorTeam && FriendlyFireScale > 0.0) || InstigatedBy == None || InstigatedBy == Injured) || InjuredTeam != InstigatorTeam)
	{
		switch (HitBone)
		{
			case 'headshot':
				FOPlayerPawn.HeadDamage += Damage;
				Damage = CalcZoneDamage(Damage,FOPlayerPawn.HeadDamage,NewArmorLevel);
				break;

			case 'rshoulder':
				FOPlayerPawn.RightArmDamage += Damage;
				Damage = CalcZoneDamage(Damage,FOPlayerPawn.RightArmDamage,NewArmorLevel);
				break;

			case 'lshoulder':
				FOPlayerPawn.LeftArmDamage += Damage;
				Damage = CalcZoneDamage(Damage,FOPlayerPawn.LeftArmDamage,NewArmorLevel);
				break;

			case 'rfarm':
				FOPlayerPawn.RightArmDamage += Damage;
				Damage = CalcZoneDamage(Damage,FOPlayerPawn.RightArmDamage,NewArmorLevel);
				break;

			case 'lfarm':
				FOPlayerPawn.LeftArmDamage += Damage;
				Damage = CalcZoneDamage(Damage,FOPlayerPawn.LeftArmDamage,NewArmorLevel);
				break;

			case 'lthigh':
				FOPlayerPawn.LeftLegDamage += Damage;
				Damage = CalcZoneDamage(Damage,FOPlayerPawn.LeftLegDamage,NewArmorLevel);
				break;

			case 'rthigh':
				FOPlayerPawn.RightLegDamage += Damage;
				Damage = CalcZoneDamage(Damage,FOPlayerPawn.RightLegDamage,NewArmorLevel);
				break;

			case 'leftfoot':
				FOPlayerPawn.LeftLegDamage += Damage;
				Damage = CalcZoneDamage(Damage,FOPlayerPawn.LeftLegDamage,NewArmorLevel);
				break;

			case 'rightfoot':
				FOPlayerPawn.RightLegDamage += Damage;
				Damage = CalcZoneDamage(Damage,FOPlayerPawn.RightLegDamage,NewArmorLevel);
				break;

			case 'lfoot':
				FOPlayerPawn.LeftLegDamage += Damage;
				Damage = CalcZoneDamage(Damage,FOPlayerPawn.LeftLegDamage,NewArmorLevel);
				break;

			case 'rfoot':
				FOPlayerPawn.RightLegDamage += Damage;
				Damage = CalcZoneDamage(Damage,FOPlayerPawn.RightLegDamage,NewArmorLevel);
				break;

			case 'lefthand':
				FOPlayerPawn.LeftArmDamage += Damage;
				Damage = CalcZoneDamage(Damage,FOPlayerPawn.LeftArmDamage,NewArmorLevel);
				break;

			case 'righthand':
				FOPlayerPawn.RightArmDamage += Damage;
				Damage = CalcZoneDamage(Damage,FOPlayerPawn.RightArmDamage,NewArmorLevel);
				break;

			default:
				FOPlayerPawn.TorsoDamage += Damage;
				Damage = CalcZoneDamage(Damage,FOPlayerPawn.TorsoDamage,NewArmorLevel);
				break;
		}
	}

	if (FOPlayer(FOPlayerPawn.Controller) != None && (FOPlayerPawn.LeftLegDamage >= 35 || FOPlayerPawn.RightLegDamage >= 35) && !FOPlayerPawn.bInjuredLeg)
	{
		FOPlayerPawn.bInjuredLeg = true;
		FOPlayer(FOPlayerPawn.Controller).ModAP(-1);
		FOPlayer(FOPlayerPawn.Controller).ReceiveLocalizedMessage(class'Interface_APMessage', 1, None, None, None);
	}

	// for hitpoint score calculation
	if (Injured.Health <= 0 || DamageController == None)
		return Damage;
	else if (Injured.Health - Damage > 0)
		TempDamage = Damage;
	else
		TempDamage = (Damage + (Injured.Health - Damage));

	if (Damage > 1)
	{
		if (DamageController != None && DamageController.PlayerReplicationInfo != None)
		{
			if (InjuredTeam == InstigatorTeam && FriendlyFireScale > 0.0)
				FragOps_PRI(DamageController.PlayerReplicationInfo).AddDamageDone(Clamp(-TempDamage, -100, 100), HitBone, FragOps_PRI(Injured.PlayerReplicationInfo));
			else if (InjuredTeam != InstigatorTeam)
				FragOps_PRI(DamageController.PlayerReplicationInfo).AddDamageDone(Clamp(TempDamage, -100, 100), HitBone, FragOps_PRI(Injured.PlayerReplicationInfo));
		}
		FragOps_PRI(Injured.PlayerReplicationInfo).AddDamageTaken(Clamp(TempDamage, -100, 100), HitBone, FragOps_PRI(DamageController.PlayerReplicationInfo));
	}

	return Damage;
}

function int CalcZoneDamage(int AdjustDamage, int DamageMod, float ArmorLevel)
{
	local float NewDamageAdjust;
	NewDamageAdjust = AdjustDamage;
	NewDamageAdjust -= NewDamageAdjust * (ArmorLevel * (FClamp((60.0-float(DamageMod))/60.0, 0.0, 1.0)));
	return int(NewDamageAdjust);
}

function SetPlayerDefaults(Pawn FOPlayerPawn)
{
	if (FOPlayerPawn != None)
	{
		if (FOPlayerPawn.Controller != None && !FOPlayerPawn.IsHumanControlled())
		{
			FOPlayerPawn.GroundSpeed = FOPawn(FOPlayerPawn).ModifySpeed(FOPlayerPawn.Default.GroundSpeed * 0.45);
			FOPlayerPawn.WaterSpeed = FOPawn(FOPlayerPawn).ModifySpeed(FOPlayerPawn.Default.WaterSpeed * 0.45);
			FOPlayerPawn.AirSpeed = FOPawn(FOPlayerPawn).ModifySpeed(FOPlayerPawn.Default.AirSpeed * 0.44);
			FOPlayerPawn.Acceleration = FOPlayerPawn.Default.Acceleration * 0.50;
		}
		else
		{
			FOPlayerPawn.GroundSpeed = FOPawn(FOPlayerPawn).ModifySpeed(FOPlayerPawn.Default.GroundSpeed * 0.45);
			FOPlayerPawn.WaterSpeed = FOPawn(FOPlayerPawn).ModifySpeed(FOPlayerPawn.Default.WaterSpeed * 0.45);
			FOPlayerPawn.AirSpeed = FOPawn(FOPlayerPawn).ModifySpeed(FOPlayerPawn.Default.AirSpeed * 0.44);
			FOPlayerPawn.Acceleration = FOPlayerPawn.Default.Acceleration * 0.45;
		}
		FOPlayerPawn.AirControl = FOPlayerPawn.Default.AirControl * 0.85;
		FOPlayerPawn.JumpZ = FOPlayerPawn.Default.JumpZ * 0.99;
		FOPlayerPawn.HealthMax = 100;
		FOPawn(FOPlayerPawn).Health = 100;
		FOPawn(FOPlayerPawn).HeadDamage = 0;
		FOPawn(FOPlayerPawn).TorsoDamage = 0;
		FOPawn(FOPlayerPawn).LeftArmDamage = 0;
		FOPawn(FOPlayerPawn).RightArmDamage = 0;
		FOPawn(FOPlayerPawn).LeftLegDamage = 0;
		FOPawn(FOPlayerPawn).RightLegDamage = 0;
		BaseMutator.ModifyPlayer(FOPlayerPawn);
	}
}

function SetLivePlayerDefaults(Pawn FOPlayerPawn)
{
	if (FOPlayerPawn != None)
	{
		FOPlayerPawn.Acceleration = FOPlayerPawn.Default.Acceleration * 0.45;
		FOPlayerPawn.AirControl = FOPlayerPawn.Default.AirControl * 0.85;
		FOPlayerPawn.JumpZ = FOPlayerPawn.Default.JumpZ * 0.99;
		FOPlayerPawn.HealthMax = 100;
		FOPawn(FOPlayerPawn).Health = 100;
	}
}

static function array<string> GetAllLoadHints(optional bool bThisClassOnly)
{
	return Default.FOHints;
}

function NavigationPoint FindPlayerStart(Controller Player, optional byte InTeam, optional string incomingName)
{
	local NavigationPoint Best;

	if (Player != None && Player.StartSpot != None)
		LastPlayerStartSpot = Player.StartSpot;

	Best = GameInfoFindPlayerStart(Player, InTeam, incomingName);
	if (Best != None)
		LastStartSpot = Best;
	return Best;
}

function NavigationPoint GameInfoFindPlayerStart( Controller Player, optional byte InTeam, optional string IncomingName )
{
	local NavigationPoint N, BestStart;
	local Teleporter Tel;
	local float BestRating, NewRating;
	local byte Team;

	// always pick StartSpot at start of match
	if (Player != None && Player.StartSpot != None && Level.NetMode == NM_Standalone && (bWaitingToStartMatch || ((Player.PlayerReplicationInfo != None) && Player.PlayerReplicationInfo.bWaitingPlayer)))
		return Player.StartSpot;

	if (GameRulesModifiers != None)
	{
		N = GameRulesModifiers.FindPlayerStart(Player,InTeam,IncomingName);
		if (N != None)
			return N;
	}

	// if incoming start is specified, then just use it
	if (IncomingName != "")
		foreach AllActors(class 'Teleporter', Tel)
			if (string(Tel.Tag) ~= IncomingName)
				return Tel;

	// use InTeam if player doesn't have a team yet
	if (Player != None && Player.PlayerReplicationInfo != None)
	{
		if (Player.PlayerReplicationInfo.Team != None)
			Team = Player.PlayerReplicationInfo.Team.TeamIndex;
		else
			Team = InTeam;
	}
	else
		Team = InTeam;

	for (N=Level.NavigationPointList; N!=None; N=N.NextNavigationPoint)
	{
		NewRating = RatePlayerStart(N,Team,Player);
		if (NewRating > BestRating)
		{
			BestRating = NewRating;
			BestStart = N;
		}
	}

	if (PlayerStart(BestStart) == None)
	{
		BestRating = -100000000;
		foreach AllActors(class 'NavigationPoint', N)
		{
			NewRating = RatePlayerStart(N,0,Player);
			if (InventorySpot(N) != None)
					NewRating -= 50;
				NewRating += 20 * FRand();
			if (NewRating > BestRating)
			{
				BestRating = NewRating;
				BestStart = N;
			}
		}
	}

	return BestStart;
}

static function string GenerateBotOptions(bool bSinglePlayer)
{
	local string option;
	local byte value;

	if (bSinglePlayer)
	{
		// Get the value of MP botmode
		value = default.BotMode & 12;

		// Give three options - 0, 1, 2
		option =        value     $ ";" $ default.BotOptions[0];
		option $= ";" $ value | 1 $ ";" $ default.BotOptions[1];
		option $= ";" $ value | 2 $ ";" $ default.BotOptions[2];
	}

	else
	{
		// get the value of SP botmode
		value = default.BotMode & 3;

		// Give three options - 4, 8, 16
		option =        value | 4 $ ";" $ default.BotOptions[0];
		option $= ";" $ value | 8 $ ";" $ default.BotOptions[2];
	}
	return option;
}

// remove ALL law's from inventory
function RemoveLAW(Controller C)
{
	if (C.Pawn == None)
		return;

	if (FOPawn(C.Pawn).FindInventoryType(class'Weapon_LAWBase') != None)
		Weapon_LAWBase(FOPawn(C.Pawn).FindInventoryType(class'Weapon_LAWBase')).DestroyLAW();
}

static function bool AllowMutator( string MutatorClassName )
{
	if (MutatorClassName == Default.MutatorClass)
		return true;

	if (!Default.bAllowVehicles && static.IsVehicleMutator(MutatorClassName))
		return false;

	return !class'LevelInfo'.static.IsDemoBuild();
}

defaultproperties
{
	// version
	Version="v2.16"
	
	GameName="Frag.Ops RoundGame"
	MapPrefix="FO-RG"
	Acronym="FO-RG"
	FriendlyFireScale=0.0
	MapListType="FragOps.MapListFragOps"
	HUDType="FragOps.Interface_MainHUD"
	MutatorClass="FragOps.FOMutator"
	BroadcastHandlerClass="FragOps.Interface_BroadcastHandler"
	DeathMessageClass=None
	DefaultEnemyRosterClass="xGame.xTeamRoster"
	GameReplicationInfoClass=class'FragOps_GRI'
	ScoreBoardType="FragOps.Interface_ScoreBoard"
	VotingHandlerType="FragOps.FragOps_VotingHandler"
	bQuickStart=false
	bPlayersMustBeReady=false
	ScreenShotName="MapThumbnails.ShotTeamGame"
	DecoTextName="FragOps.FragOps"
	PlayerControllerClassName="FragOps.FOPlayer"
	SecurityClass="FragOps.Security_FOSecure"
	DefaultPlayerClassName="FragOps.FOPawn"
	bSpawnInTeamArea=true
        bScoreTeamKills=false
	PreRoundWaitTime=10
	FirstRoundWaitTime=25
	PreRoundTime=0
	RoundTime=4
	TimeLimit=25
	RoundTimeRemaining=0
	KillMoney=100
	DeathMoney=100
	bForceEndRound=false
	bAllowBehindView=false
	bModViewShake=true
	bWaitingToStartMatch=true
	bAllowThirdPersonSpec=false
	CountDown=0
	MinNetPlayers=100
	bFOCTGlobalBan=true

	// special metro crowd precaching - ugly
	CrowdMaterial(0)="intro_characters.CrowdMale1Sprite.CrowdMale1Sprite_S"
	CrowdMaterial(1)="intro_characters.CrowdMale2Sprite.CrowdMale2Sprite_S"
	CrowdMaterial(2)="intro_characters.CrowdMale3Sprite.CrowdMale3Sprite_S"
	CrowdMaterial(3)="intro_characters.CrowdMale4Sprite.CrowdMale4Sprite_S"
	CrowdMaterial(4)="intro_characters.CrowdMale5Sprite.CrowdMale5Sprite_S"
	CrowdMaterial(5)="intro_characters.CrowdMale6Sprite.CrowdMale6Sprite_S"

	// hints
	FOHints(0)="Grenades can be thrown underhand using %ALTFIRE%."
	FOHints(1)="You can shoot grenades to detonate them early."
	FOHints(2)="Armor levels can offer greater protection at the expense of speed."
	FOHints(3)="Nearly all weapons can be upgraded for expanded functionality."
	FOHints(4)="A Pistol Expert medal is awarded after 2 pistol kills with 9/10 headshot accuracy."
	FOHints(5)="An Advanced Pistol medal is awarded after 2 pistol kills with 1/2 headshot accuracy."
	FOHints(6)="A Pistol Novice medal is awarded after 1 pistol kill with 1/3 headshot accuracy."
	FOHints(7)="There is 1 expert medal which awards bonus AP to extend your maximum."
	FOHints(8)="Using your mousewheel allows you to quickly switch firemodes or zoom in/out."
	FOHints(9)="Tapping %RELOAD% again while reloading the Ultra90 will interrupt the reload process."

	// settings for rule page
	BotOptions(0)="Specify Number"
	BotOptions(1)="Use Map Defaults"
	BotOptions(2)="Players vs Bots"
	BotOptions(3)=""

	// gore settings
	FOGoreLevelText[0]="No Gore"
	FOGoreLevelText[1]="Reduced Gore"
	FOGoreLevelText[2]="Normal Gore"
	FOGoreLevelText[3]="Ultra Violence"
}