//=============================================================================
// Frag.Ops  2004 Matt 'SquirrelZero' Farber
//=============================================================================
class FragOpsMission extends FragOpsGame;

// money awarded for escaping, AP's rewarded
var() int EscapeMoney;

// master escape volumes for objective maps
var Objective_EscapeVolume MasterEscapeVolume[2];
var array<FragOps_ChainableObjective> SO_ObjectiveChain;
var array<FragOps_ChainableObjective> MC_ObjectiveChain;
var int LAWCount[2], RemainingLAWCount[2];
var int TeamLosses[2];

function LAWDestroyed()
{
	if (bRoundEnded)
		return;
	
	if (LAWCount[0] > 0 && LAWCount[1] > 0)
	{
		if (RemainingLAWCount[0] > 0)
			RemainingLAWCount[0] -= 1;
		else if (RemainingLAWCount[1] > 0)
			RemainingLAWCount[0] -= 1;
		else
		{
			BroadcastLocalizedMessage(class'Interface_ObjectiveMessage',6,None,None);
			EndRound(2);
		}
		return;
	}

	if (LAWCount[0] > 0)
	{
		RemainingLAWCount[0] -= 1;
		if (RemainingLAWCount[0] == 0)
		{
			EndRound(1);
		}
		BroadcastLocalizedMessage(class'Interface_ObjectiveMessage',4,None,None);
	}
	else if (LAWCount[1] > 0)
	{
		RemainingLAWCount[1] -= 1;
		if (RemainingLAWCount[1] == 0)
		{
			EndRound(0);
		}
		BroadcastLocalizedMessage(class'Interface_ObjectiveMessage',5,None,None);
	}
}

function UpdateChain(FragOps_ChainableObjective fcObjective)
{
	if (bRoundEnded)
		return;
	
	if (fcObjective.ObjectiveTeam == 0)
	{
		if (fcObjective.bCompleted && fcObjective.LinkNumber == (MC_ObjectiveChain.Length-1) && MasterEscapeVolume[0] == None)
			EndRound(0,true);
	}
	else if (fcObjective.ObjectiveTeam == 1)
	{
		if (fcObjective.bCompleted && fcObjective.LinkNumber == (SO_ObjectiveChain.Length-1) && MasterEscapeVolume[1] == None)
			EndRound(1,true);
	}
}

function SetPlayerWeapon(weapon WTempWeapon, Controller WController)
{
	if (FOPlayer(WController) != None && FOPlayer(WController).CriticalReplicationInfo.bLAWCarrier)
	{
		WTempWeapon = FOPawn(WController.Pawn).AddNewInventory(class'Weapon_LAWBase',true);
	}
	Super.SetPlayerWeapon(WTempWeapon,WController);
}

// discard LAW if carried
function DiscardInventory(Pawn Other)
{
	local Weapon_LAWBase TempLAW;
	local vector TossVel, X,Y,Z;
	
	TempLAW = Weapon_LAWBase(FOPawn(Other).FindInventoryType(class'Weapon_LAWBase'));
	if (TempLAW != None)
	{
		TossVel = Vector(Other.GetViewRotation());
		TossVel = TossVel * ((Other.Velocity Dot TossVel) + 500) + Vect(0,0,200);
		TempLAW.Velocity = TossVel;
		Other.GetAxes(Other.Rotation,X,Y,Z);
		TempLAW.DropFrom(Other.Location + 0.8 * Other.CollisionRadius * X - 0.5 * Other.CollisionRadius * Y); 
	}
	Super.DiscardInventory(Other);
}

function SpecialPreRound()
{
	local Controller P;
	local int LAWIncrement, LAWTeamPlayers, i, j, OldIncrement;
	local array<Controller> LAWHandOutList;

	RemainingLAWCount[0] = LAWCount[0];
	RemainingLAWCount[1] = LAWCount[1];

	for (P=Level.ControllerList;P!=None;P=P.nextController)
	{
		if (P.bIsPlayer)
		{
			RemoveLAW(P);
			PlayerCRI(P).bLAWCarrier = false;
		}
	}
	// here we'll hand out LAW's if necessary
	while (j < 2 && (LAWCount[0] > 0 || LAWCount[1] > 0))
	{
		if (LAWCount[j] > 0)
		{
			i = 0;
			LAWHandoutList.Remove(0,LAWHandoutList.Length);
			LAWHandoutList.Length = 0;
			LAWHandOutList[0] = None;
			for (P=Level.ControllerList;P!=None;P=P.nextController)
			{
				if (FOBot(P) == None && PlayerCRI(P) != None && !PlayerCRI(P).bWaitingForTeam && P.PlayerReplicationInfo != None && P.bIsPlayer && P.PlayerReplicationInfo.Team != None && P.PlayerReplicationInfo.Team.TeamIndex == j)				
				{
					i++;
					LAWHandoutList[i] = P;
				}
			}

			LAWTeamPlayers = i;
			LAWIncrement = Max(1, LAWTeamPlayers / LAWCount[j]);
			OLDIncrement = LAWIncrement;

			for (i=1;i<LAWHandoutList.Length;i++)
			{
				if (i == LAWIncrement && !PlayerCRI(LAWHandOutList[i]).bLAWCarrier)
				{
					PlayerCRI(LAWHandOutList[i]).bLAWCarrier = true;
					LAWIncrement += OLDIncrement;
				}
			}
		}
		j++;
	}
}			

function CheckEscapeWinner(byte EscapeTeam, Objective_EscapeVolume EscapeVolume)
{
	local Controller C;
	local int NumEscapePlayers,NumEscapees;
	local bool bEscapeAlive;

	if (bRoundEnded || PreRoundTime > 0)
		return;

	for (C=Level.ControllerList; C!=None; C=C.nextController )
	{
		if (FragOps_PRI(C.PlayerReplicationInfo).Team.TeamIndex == EscapeTeam)
		{
			NumEscapePlayers++;
			if (C.Pawn != None && !FOPawn(C.Pawn).bEscaped)
				bEscapeAlive = true;
			if (FragOps_PRI(C.PlayerReplicationInfo).bEscaped)
				NumEscapees++;
		}
	}
	if ((float(NumEscapees) / float(NumEscapePlayers)) > 0.5)
	{
		EndRound(int(EscapeTeam),true);
		BroadcastLocalizedMessage(class'Interface_GameMessage',0,None,None,EscapeVolume);
	}
	else if (!bEscapeAlive && NumEscapees > 0)
	{
		if (EscapeTeam == 0)
		{
			BroadcastLocalizedMessage(class'Interface_GameMessage',2,None,None,MasterEscapeVolume[0]);
			EndRound(1,true);
		}
		else
		{
			BroadcastLocalizedMessage(class'Interface_GameMessage',2,None,None,MasterEscapeVolume[1]);
			EndRound(0,true);
		}
	}
	else
		CheckEndRound();
}

function Killed(Controller Killer, Controller Killed, Pawn KilledPawn, class<DamageType> damageType)
{
	local bool bEnemyKill;

	bEnemyKill = (!bTeamGame || ((Killer != None) && (Killer != Killed) && (Killed != None)
	&& (Killer.PlayerReplicationInfo != None) && (Killed.PlayerReplicationInfo != None)
	&& (Killer.PlayerReplicationInfo.Team != Killed.PlayerReplicationInfo.Team)));

	if ((Killer != None) && Killer.bIsPlayer && (Killed != None) && Killed.bIsPlayer && damageType != None)
	{
		RoundKills[Killed.PlayerReplicationInfo.Team.TeamIndex] += 1;
		if (Killer.PlayerReplicationInfo.Team.TeamIndex != Killed.PlayerReplicationInfo.Team.TeamIndex)
			FragOps_PRI(Killer.PlayerReplicationInfo).RoundKills += 1;
		DamageType.static.IncrementKills(Killer);
	}

	if ((Killed != None) && Killed.bIsPlayer)
	{
		if (!FragOps_GRI(GameReplicationInfo).bPreRound)
		{
			Killed.PlayerReplicationInfo.Deaths += 1;
			if (Killer != None)
				BroadcastKillMessage(damageType, Killer.PlayerReplicationInfo, Killed.PlayerReplicationInfo);
			else
				BroadcastKillMessage(damageType, None, Killed.PlayerReplicationInfo);
		}
		else
		{
			Killed.PlayerReplicationInfo.bOutOfLives = true;
			DiscardInventory(KilledPawn);
			return;
		}
		if ((Killer == Killed) || (Killer == None))
		{
			if (Killer == None)
				KillEvent("K", None, Killed.PlayerReplicationInfo, DamageType);	//"Kill"
			else
				KillEvent("K", Killer.PlayerReplicationInfo, Killed.PlayerReplicationInfo, DamageType);	//"Kill"
		}
		else
		{
			if (bTeamGame && (Killer.PlayerReplicationInfo != None)
				&& (Killer.PlayerReplicationInfo.Team == Killed.PlayerReplicationInfo.Team) )
			{
				KillEvent("TK", Killer.PlayerReplicationInfo, Killed.PlayerReplicationInfo, DamageType);	//"Teamkill"
			        PlayerCRI(Killer).AddMoney(-(8*KillMoney));
				if (FOPlayer(Killer) != None)
				{
					FOPlayer(Killer).ModAP(-1);
					FOPlayer(Killer).ReceiveLocalizedMessage(class'Interface_APMessage', 2, None, None, None);
				}
			}
			else
			{
				KillEvent("K", Killer.PlayerReplicationInfo, Killed.PlayerReplicationInfo, DamageType);	//"Kill"
				CheckKillType(Killer,damageType);
				PlayerCRI(Killer).AddMoney(KillMoney);
			}
		}
	}
	if (Killed != None)
	{
		if (FOPlayer(Killed) != None)
			FOPlayer(Killed).ReceivePlayerStats();
		Killed.PlayerReplicationInfo.bOutOfLives = true;
	}

	if (Killer != None)
		ScoreKill(Killer, Killed);	

	DiscardInventory(KilledPawn);
	NotifyKilled(Killer,Killed,KilledPawn);

	if (MasterEscapeVolume[0] == None || MasterEscapeVolume[1] == None)
	{
		if (MasterEscapeVolume[0] != None && MasterEscapeVolume[1] == None)
			CheckEscapeWinner(0,MasterEscapeVolume[0]);
		else if (MasterEscapeVolume[1] != None && MasterEscapeVolume[0] == None)
			CheckEscapeWinner(1,MasterEscapeVolume[1]);
		else
			CheckEndRound();
	}
	else
		CheckEndRound();
}

function EndRound(int WinningTeam, optional bool bNoAnnounce)
{
	local Controller C;
	local int LivingPlayers;
	local PlayerReplicationInfo SurvivorPRI;
	local FragOps_CRI SurvivorCRI;
	local float LossIncrement;

	if (bRoundEnded)
		return;
	
	bRoundEnded = true;

	if (WinningTeam < 3 && !bNoAnnounce)
	{
		if (WinningTeam == 2)
		{
			if (MasterEscapeVolume[0] != None && MasterEscapeVolume[1] == None)
			{
				WinningTeam = 1;
				BroadcastLocalizedMessage(class'Interface_GameMessage',2,None,None,MasterEscapeVolume[0]);
			}
			else if (MasterEscapeVolume[1] != None && MasterEscapeVolume[0] == None)
			{
				WinningTeam = 0;
				BroadcastLocalizedMessage(class'Interface_GameMessage',2,None,None,MasterEscapeVolume[1]);
			}
			else if (MC_ObjectiveChain.Length > 0 && !MC_ObjectiveChain[MC_ObjectiveChain.Length-1].bCompletionOptional && (SO_ObjectiveChain.Length <= 0 || SO_ObjectiveChain[SO_ObjectiveChain.Length-1].bCompletionOptional))
			{
				WinningTeam = 1;
				BroadcastLocalizedMessage(class'Interface_GameMessage',1,None,None,MC_ObjectiveChain[(MC_ObjectiveChain.Length-1)]);
			}
			else if (SO_ObjectiveChain.Length > 0 && !SO_ObjectiveChain[SO_ObjectiveChain.Length-1].bCompletionOptional && (MC_ObjectiveChain.Length <= 0 || MC_ObjectiveChain[MC_ObjectiveChain.Length-1].bCompletionOptional))
			{
				WinningTeam = 0;
				BroadcastLocalizedMessage(class'Interface_GameMessage',1,None,None,SO_ObjectiveChain[(SO_ObjectiveChain.Length-1)]);
			}
			else
				BroadcastLocalizedMessage(class'Interface_GameMessage', (WinningTeam + 6));
		}
		else
				BroadcastLocalizedMessage(class'Interface_GameMessage', (WinningTeam + 6));
	}

	// reset loss incrementer to 0
	if (WinningTeam < 2)
		TeamLosses[WinningTeam] = 0;

	for (C=Level.ControllerList;C!=None;C=C.NextController)
	{
		if (PlayerCRI(C) == None || (PlayerCRI(C).bWaitingForTeam && FOBot(C) == None) || !C.bIsPlayer || C.PlayerReplicationInfo.bOnlySpectator)
			continue;

		// stop all objectives
		if (C.IsInState('StickyObjective'))
			FOPlayer(C).ServerUnUse();

		if (WinningTeam < 2 && ((!FragOps_PRI(C.PlayerReplicationInfo).bOutOfLives && C.Pawn != None) || (FragOps_PRI(C.PlayerReplicationInfo).bEscaped)))
		{
			LivingPlayers++;
			SurvivorCRI = PlayerCRI(C);
			SurvivorPRI = C.PlayerReplicationInfo;
		}
		if (C.PlayerReplicationInfo.Team.TeamIndex == WinningTeam)
		{
			PlayerCRI(C).AddMoney(WinningTeamMoney);
		}
		else
		{
			LossIncrement = 1.0 + float(Min(TeamLosses[C.PlayerReplicationInfo.Team.TeamIndex], 2))*0.17;
			PlayerCRI(C).AddMoney(LosingTeamMoney*LossIncrement);
		}
		CheckWeaponMedals(PlayerCRI(C),C.PlayerReplicationInfo,FOPlayer(C));
	}

	// award a survivor medal if applicable
	if ((SurvivorCRI != None) && SurvivorCRI.Medals[0] == 0 && LivingPlayers == 1 && NumPlayers+NumBots >= 4 && SurvivorPRI.Team.Size >= 2 && SurvivorPRI.Team.Size < NumPlayers+NumBots)
	{
		SurvivorCRI.AddMedal(0);
		SurvivorCRI.AddMoney(300);
	}
	if (WinningTeam < 2)
	{
		Teams[WinningTeam].Score += 1;
	}
	else if (WinningTeam < 3)
	{
		Teams[0].Score += 1;
		Teams[1].Score += 1;
	}

	// add a team loss
	if (WinningTeam == 0)
		TeamLosses[1]++;
	else if (WinningTeam == 1)
		TeamLosses[0]++;

	if (WinningTeam < 3)
		BroadcastRoundStats(WinningTeam);

	if (WinningTeam < 3)
	{
	        for (C=Level.ControllerList; C!=None;C=C.NextController)
		{
			if (C.bIsPlayer && C.Pawn != None)
				FOPawn(C.Pawn).bRestrictDamage = true;
		}
			TimeTillNextRound = 5;
	}
	else
		StartRound();
}

defaultproperties
{
	GameName="Frag.Ops Mission"
	Description="Tactical, Mission-based roundgame.  The team with the most rounds wins."
	HUDType="FragOps.Interface_MainHUD"
	BeaconName="MIS"
	Acronym="MIS"
	MapPrefix="MIS"
	DecoTextName="FragOps.FragOpsMission"
	MapListType="FragOps.MapListFragOps"
	EscapeMoney=500
	ScreenShotName="UT2004Thumbnails.DMShots"
	bAllowPlayerLights=false
	WinningTeamMoney=900
	LosingTeamMoney=775
}