//=============================================================================
// Frag.Ops  2004 Matt 'SquirrelZero' Farber
//=============================================================================
class FragOpsWar extends FragOpsGame;

// levelinfo for easy access
var War_LevelInfo WarLevelInfo;

// misc time settings
var int SOWaveTimeRemaining, MercWaveTimeRemaining, SOWaveTime, MercWaveTime, MercWaveWindowSeconds, SOWaveWindowSeconds, RejoiningMercs, RejoiningSO;
var config int SOWaveDelayOverride, MercWaveDelayOverride, RoundTimeOverride;
var config int WaveWindowTime;

// XP
var int LosingTeamXP, WinningTeamXP, KillXP, ObjectiveXP;

// initialize
event InitGame(string Options, out string Error)
{
	local War_LevelInfo WLevelInfo;
	
	Super.InitGame(Options, Error);

	// retrieve levelinfo
	foreach AllActors(class'War_LevelInfo',WLevelInfo)
	{
		WarLevelInfo = WLevelInfo;
		break;
	}

	// adjust timer settings
	if (SOWaveDelayOverride > 0)
		SOWaveTime = SOWaveDelayOverride;
	else
		SOWaveTime = WarLevelInfo.SOWaveDelayTime;

	if (MercWaveDelayOverride > 0)
		MercWaveTime = MercWaveDelayOverride;
	else
		MercWaveTime = WarLevelInfo.MercWaveDelayTime;

	if (RoundTimeOverride > 0)
		RoundTime = RoundTimeOverride;
	else
		RoundTime = WarLevelInfo.RoundTimeLimit;
}

// add some new details
function GetServerDetails(out ServerResponseLine ServerState)
{
	Super.GetServerDetails(ServerState);
	AddServerDetail(ServerState, "Respawn Window Time", WaveWindowTime);
	if (RoundTimeOverride > 0)
		AddServerDetail(ServerState, "Custom Round Time", RoundTimeOverride);
	if (MercWaveDelayOverride > 0)
		AddServerDetail(ServerState, "Custom Merc Wave Time", MercWaveDelayOverride);		
	if (SOWaveDelayOverride > 0)
		AddServerDetail(ServerState, "Custom SO Wave Time", SOWaveDelayOverride);		
}

function SpecialPreRound()
{
	local Controller C;

	// reset wave times
	War_GRI(GameReplicationInfo).SOWaveTimeRemaining = 0;	
        War_GRI(GameReplicationInfo).MercWaveTimeRemaining = 0;	

	// reset wave window times
	War_GRI(GameReplicationInfo).WaveWindowTime = WaveWindowTime;	
        War_GRI(GameReplicationInfo).MercWaveDelayTime = MercWaveTime;	
        War_GRI(GameReplicationInfo).SOWaveDelayTime = SOWaveTime;	

	// give everyone with a LAW 1 rocket
	for (C=Level.ControllerList;C!=None;C=C.NextController)
	{
		if (C.Pawn != None && C.Pawn.Health > 0 && FOPawn(C.Pawn).FindInventoryType(class'Weapon_LAWBase') != None)
			Weapon_Base(FOPawn(C.Pawn).FindInventoryType(class'Weapon_LAWBase')).GetAmmo(0).AmmoAmount = 1;
	}
}


function DiscardInventory(Pawn Other)
{
	local int i, Count;

	if (Other == None || Vehicle(Other) != None)
		return;

	Other.Weapon = None;
	Other.SelectedItem = None;

	for (i=0;i<ArrayCount(FOPawn(Other).EquipList);i++)
	{
		FOPawn(Other).EquipList[i] = None;
		FOPawn(Other).AttachList[i] = None;
	}
	while (Other.Inventory != None && Count < 1000)
	{
		Count++;
		Other.Inventory.Destroy();
	}
}

function AddDefaultInventory(pawn PlayerPawn)
{
	local class<Weapon_Base> FOMeleeClass, FOPistolClass, FORifleClass;
	local Weapon FOTempWeapon;
	local Controller PC;

	PC = PlayerPawn.Controller;

	if (PC == None || FOBot(PC) == None)
		return;
	
	if (PC.PlayerReplicationInfo.Team.TeamIndex < 1)
	{
		if (fRand() < 0.5)
		{
			if (fRand() < 0.75)
				FORifleClass = class'Interface_BuyMenuPlus'.Default.Rifles[8];
			else if (fRand() < 0.97)
				FORifleClass = class'Interface_BuyMenuPlus'.Default.Rifles[10];
			else
				FORifleClass = class'Interface_BuyMenuPlus'.Default.Rifles[12];
		}
		else
		{
			if (fRand() < 0.5)
				FORifleClass = class'Interface_BuyMenuPlus'.Default.Rifles[0];
			else if (fRand() < 0.5)
				FORifleClass = class'Interface_BuyMenuPlus'.Default.Rifles[2];
			else
				FORifleClass = class'Interface_BuyMenuPlus'.Default.Rifles[6];
		}
		FOMeleeClass = class'Interface_BuyMenuPlus'.Default.Melee[0];
		FOPistolClass  = class'Interface_BuyMenuPlus'.Default.Sidearms[0];
	}
	else
	{
		if (fRand() < 0.5)
		{
			if (fRand() < 0.75)
				FORifleClass = class'Interface_BuyMenuPlus'.Default.Rifles[9];
			else if (fRand() < 0.97)
				FORifleClass = class'Interface_BuyMenuPlus'.Default.Rifles[11];
			else
				FORifleClass = class'Interface_BuyMenuPlus'.Default.Rifles[13];
		}
		else
		{
			if (fRand() < 0.5)
				FORifleClass = class'Interface_BuyMenuPlus'.Default.Rifles[1];
			else if (fRand() < 0.5)
				FORifleClass = class'Interface_BuyMenuPlus'.Default.Rifles[3];
			else
				FORifleClass = class'Interface_BuyMenuPlus'.Default.Rifles[7];
		}
		FOMeleeClass = class'Interface_BuyMenuPlus'.Default.Melee[1];
		FOPistolClass  = class'Interface_BuyMenuPlus'.Default.Sidearms[1];
	}

	FOPawn(PlayerPawn).AddNewInventory(FOMeleeClass,false);
	FOTempWeapon = FOPawn(PlayerPawn).AddNewInventory(FOPistolClass,(FORifleClass == None));
	if (FORifleClass != None)
		FOTempWeapon = FOPawn(PlayerPawn).AddNewInventory(FORifleClass,true);
	SetPlayerWeapon(FOTempWeapon,PC);
	SetPlayerDefaults(PlayerPawn);
}

function Killed(Controller Killer, Controller Killed, Pawn KilledPawn, class<DamageType> damageType)
{
	local bool bEnemyKill;

	bEnemyKill = ( !bTeamGame || ((Killer != None) && (Killer != Killed) && (Killed != None)
	&& (Killer.PlayerReplicationInfo != None) && (Killed.PlayerReplicationInfo != None)
	&& (Killer.PlayerReplicationInfo.Team != Killed.PlayerReplicationInfo.Team)) );

	if ((Killer != None) && Killer.bIsPlayer && (Killed != None) && Killed.bIsPlayer && damageType != None)
	{
		RoundKills[Killed.PlayerReplicationInfo.Team.TeamIndex] += 1;
		if (Killer.PlayerReplicationInfo.Team.TeamIndex != Killed.PlayerReplicationInfo.Team.TeamIndex)
			FragOps_PRI(Killer.PlayerReplicationInfo).RoundKills += 1;
		DamageType.static.IncrementKills(Killer);

		// increment scores
		if (WarLevelInfo.bTeamDeathMatch && Killed.PlayerReplicationInfo.Team != Killer.PlayerReplicationInfo.Team)
			FragOps_PRI(Killer.PlayerReplicationInfo).Team.Score += 1;
	}
	if ((Killed != None) && Killed.bIsPlayer)
	{
		if (!FragOps_GRI(GameReplicationInfo).bPreRound)
		{
			Killed.PlayerReplicationInfo.Deaths += 1;
			if (Killer != None)
				BroadcastKillMessage(damageType, Killer.PlayerReplicationInfo, Killed.PlayerReplicationInfo);
			else
				BroadcastKillMessage(damageType, None, Killed.PlayerReplicationInfo);
		}
		else
		{
			Killed.PlayerReplicationInfo.bOutOfLives = true;
			DiscardInventory(KilledPawn);
			return;
		}
		if ((Killer == Killed) || (Killer == None))
		{
			if (Killer == None)
				KillEvent("K", None, Killed.PlayerReplicationInfo, DamageType);	//"Kill"
			else
				KillEvent("K", Killer.PlayerReplicationInfo, Killed.PlayerReplicationInfo, DamageType);	//"Kill"
		}
		else
		{
			if (bTeamGame && (Killer.PlayerReplicationInfo != None)
				&& (Killer.PlayerReplicationInfo.Team == Killed.PlayerReplicationInfo.Team) )
			{
				KillEvent("TK", Killer.PlayerReplicationInfo, Killed.PlayerReplicationInfo, DamageType);	//"Teamkill"
				War_PRI(Killer.PlayerReplicationInfo).AddXP(-8*KillXP);
				if (FOPlayer(Killer) != None)
				{
					FOPlayer(Killer).ModAP(-1);
					FOPlayer(Killer).ReceiveLocalizedMessage(class'Interface_APMessage', 2, None, None, None);
				}
			}
			else
			{
				KillEvent("K", Killer.PlayerReplicationInfo, Killed.PlayerReplicationInfo, DamageType);	//"Kill"
				CheckKillType(Killer,damageType);
				War_PRI(Killer.PlayerReplicationInfo).AddXP(KillXP);
			}
		}
	}
	if (Killed != None)
	{
		if (FOPlayer(Killed) != None)
			FOPlayer(Killed).ReceivePlayerStats();
		Killed.PlayerReplicationInfo.bOutOfLives = true;
	}

	if (Killer != None)
		ScoreKill(Killer, Killed);	

	DiscardInventory(KilledPawn);
	NotifyKilled(Killer,Killed,KilledPawn);

	// check for round end
	CheckEndRound();
}

function CheckEndRound()
{
	local Controller C;
	local bool bTeam0Alive;
	local bool bTeam1Alive;
	local int TotalPlaying;
	local int TeamWinner;

	TotalPlaying = 0;
	TeamWinner = 2;

	//New round is about to start, don't do any round ending checking
	if (bRoundEnded || FragOps_GRI(GameReplicationInfo).bPreRound || TimeTillNextRound > 0)
		return;

	//Is a team out of action?
	bTeam0Alive = false;
	bTeam1Alive = false;
        for (C=Level.ControllerList; C!=None; C=C.NextController)
	{
		if (C.bIsPlayer || FOBot(C) != None)
			TotalPlaying++;

		if ((C.PlayerReplicationInfo != None) && (C.PlayerReplicationInfo.Team != None) && (C.bIsPlayer || FOBot(C) != None)
                && (!C.PlayerReplicationInfo.bOutOfLives && (C.Pawn != None) && (C.Pawn.Health > 0) ) && C.PlayerReplicationInfo.Team.TeamIndex == 0)
			bTeam0Alive = true;
			
		if ((C.PlayerReplicationInfo != None) && (C.PlayerReplicationInfo.Team != None) && (C.bIsPlayer || FOBot(C) != None)
                && (!C.PlayerReplicationInfo.bOutOfLives && (C.Pawn != None) && (C.Pawn.Health > 0) ) && C.PlayerReplicationInfo.Team.TeamIndex == 1)
			bTeam1Alive = true;
	}


	if (bTeam1Alive && !bTeam0Alive && TotalPlaying > 1)
		TeamWinner = 1;
	else if (bTeam0Alive && !bTeam1Alive && TotalPlaying > 1)
		TeamWinner = 0;

	if ((TeamWinner < 2 && (!bTeam0Alive || !bTeam1Alive)) && (!WarLevelInfo.bTeamDeathMatch && ((TeamWinner == 0 && WarLevelInfo.bMercsCanWinByElimination) || (TeamWinner == 1 && WarLevelInfo.bSOCanWinByElimination))))
		EndRound(TeamWinner);
}

function EndRound(int WinningTeam, optional bool bNoAnnounce)
{
	local Controller C;
	local int LivingPlayers;
	local PlayerReplicationInfo SurvivorPRI;
	local FragOps_CRI SurvivorCRI;

	if (bRoundEnded)
		return;
	
	bRoundEnded = true;

	if (WinningTeam < 3)
	{
		if (!bNoAnnounce)
		{
			if (WinningTeam == 2)
			{
				if (WarLevelInfo.M_Obj.Length > 0 && WarLevelInfo.S_Obj.Length == 0)
					WinningTeam = 1;
				else if (WarLevelInfo.S_Obj.Length > 0 && WarLevelInfo.M_Obj.Length == 0)
					WinningTeam = 0;
			}			
			BroadcastLocalizedMessage(class'War_Interface_RoundMessage', (WinningTeam + 12));
		}
		else if (WinningTeam == 0)
			BroadcastLocalizedMessage(class'War_Interface_RoundMessage', 12);
		else if (WinningTeam == 1)
			BroadcastLocalizedMessage(class'War_Interface_RoundMessage', 13);
	}
	for (C=Level.ControllerList;C!=None;C=C.NextController)
	{
		if (PlayerCRI(C) == None || (PlayerCRI(C).bWaitingForTeam && FOBot(C) == None) || !C.bIsPlayer || C.PlayerReplicationInfo.bOnlySpectator)
			continue;

		// stop all objectives
		if (C.IsInState('StickyObjective'))
			FOPlayer(C).ServerUnUse();

		if (WinningTeam < 2 && ((!FragOps_PRI(C.PlayerReplicationInfo).bOutOfLives && C.Pawn != None) || (FragOps_PRI(C.PlayerReplicationInfo).bEscaped)))
		{
			LivingPlayers++;
			SurvivorCRI = PlayerCRI(C);
			SurvivorPRI = C.PlayerReplicationInfo;
		}
		if (C.PlayerReplicationInfo.Team.TeamIndex == WinningTeam)
			War_PRI(C.PlayerReplicationInfo).AddXP(WinningTeamXP);
		else
			War_PRI(C.PlayerReplicationInfo).AddXP(LosingTeamXP);
		CheckWeaponMedals(PlayerCRI(C),C.PlayerReplicationInfo,FOPlayer(C));
	}

	// award a survivor medal if applicable
	if ((SurvivorCRI != None) && SurvivorCRI.Medals[0] == 0 && LivingPlayers == 1 && NumPlayers+NumBots >= 4 && SurvivorPRI.Team.Size >= 2 && SurvivorPRI.Team.Size < NumPlayers+NumBots)
	{
		SurvivorCRI.AddMedal(0);
		War_PRI(SurvivorPRI).AddXP(20);
	}
	if (WinningTeam < 2)
	{
		Teams[WinningTeam].Score += 1;
	}
	else if (WinningTeam < 3)
	{
		Teams[0].Score += 1;
		Teams[1].Score += 1;
	}

	if (WinningTeam < 3)
		BroadcastRoundStats(WinningTeam);

	if (WinningTeam < 3)
	{
		RestrictPlayers();
		TimeTillNextRound = 5;
	}
	else
		StartRound();
}

// stop players from taking damage if this objective ends the round
function RestrictPlayers()
{
	local Pawn RoundPawn;
	
	foreach DynamicActors(class'Pawn',RoundPawn)
	{
		if (RoundPawn.Health > 0)
		{
			if (FOPawn(RoundPawn) != None)
				FOPawn(RoundPawn).bRestrictDamage = true;
			else if (War_FOVehicleWheeled(RoundPawn) != None)
				War_FOVehicleWheeled(RoundPawn).bRestrictDamage = true;
			else if (War_FOVehicleAir(RoundPawn) != None)
				War_FOVehicleAir(RoundPawn).bRestrictDamage = true;
		}
	}
}

// use a different bot class
function Bot SpawnBot(optional string botName)
{
	local Bot NewBot;
	local RosterEntry Chosen;
	local UnrealTeamInfo BotTeam;

	BotTeam = GetBotTeam();
	Chosen = BotTeam.ChooseBotClass(botName);

	if (Chosen.PawnClass == None)
		Chosen.Init(); //amb
    
	NewBot = Spawn(class'War_FOBot');
	FOBot(NewBot).SetRandomChar(BotTeam.TeamIndex);

	if (NewBot != None)
	{
		FragOps_PRI(NewBot.PlayerReplicationInfo).bNoTeamSkins = true;
		InitializeBot(NewBot,BotTeam,Chosen);
	}
	return NewBot;
}

state MatchInProgress
{   
	function Timer()
	{
		local Controller B;
		local Objective_RetrievalVolume ORV;
 
		Global.Timer();
		if (PreRoundTime > 0)
		{
			PreRoundTime--;
			FragOps_GRI(GameReplicationInfo).bPreRound = true;
			if (PreRoundTime <= 0)
			{
				// rounds
				FragOps_GRI(GameReplicationInfo).bPreRound = false;

				if (!WarLevelInfo.bTeamDeathMatch)
				{
					FragOps_GRI(GameReplicationInfo).RoundTimeRemaining = RoundTime * 60;
					RoundTimeRemaining = RoundTime * 60;
				}
				else
				{
					FragOps_GRI(GameReplicationInfo).RoundTimeRemaining = 0;
					FragOps_GRI(GameReplicationInfo).RoundTimeMinute = 0;
					RoundTimeRemaining = 0;
				}

				// waves
				War_GRI(GameReplicationInfo).MercWaveTimeRemaining = MercWaveTime;
				War_GRI(GameReplicationInfo).SOWaveTimeRemaining = SOWaveTime;
				MercWaveTimeRemaining = MercWaveTime;
				SOWaveTimeRemaining = SOWaveTime;
				
				// round starting, do stuff
				BroadcastLocalizedMessage(class'Interface_GameMessage', (0));
				for (B = Level.ControllerList; B!=None; B=B.nextController)
				{
					if (B.bIsPlayer && !PlayerCRI(B).bWaitingForTeam && B.IsA('PlayerController'))
					{
						if (FOPawn(B.Pawn) != None)
							FOPawn(B.Pawn).bRestrictDamage = false;
						FOPlayer(B).GotoState('PlayerWalking');
						FOPlayer(B).MoveOutAnnounce();
						War_FOPlayer(B).BroadcastInitialObjective();
					}
					else if (B.bIsPlayer && !B.IsA('PlayerController'))
					{
						if (FOPawn(B.Pawn) != None)
							FOPawn(B.Pawn).bRestrictDamage = false;
						FOBot(B).GotoState('Hunting','Begin');
					}
				}
				foreach AllActors(class'Objective_RetrievalVolume',ORV)
					ORV.Reset();
				PreRoundTime = 0;
			}
		}

		if (TimeTillNextRound > 0 || bForceEndRound)
		{
			if (TimeTillNextRound > 0)
				TimeTillNextRound--;
			if (TimeTillNextRound == 0 || bForceEndRound)
				StartRound();
			bForceEndRound = false;
		}

		if (!bFinalStartup)
		{
			bFinalStartup = true;
			PlayStartupMessage();
		}

		if (NeedPlayers() && AddBot() && (RemainingBots > 0))
			RemainingBots--;

		if (TimeLimit > 0)
		{
			GameReplicationInfo.bStopCountDown = false;
			RemainingTime--;
			GameReplicationInfo.RemainingTime = RemainingTime;
			if (RemainingTime % 60 == 0)
				GameReplicationInfo.RemainingMinute = RemainingTime;
			if (RemainingTime <= 0 && (RoundTimeRemaining <= 0 || WarLevelInfo.bTeamDeathMatch))
				EndGame(None,"TimeLimit");
		}

		if (RoundTime > 0 && PreRoundTime <= 0)
		{
			if (TimeTillNextRound <= 0)
			{
				// reduce wave time
				MercWaveTimeRemaining--;
				SOWaveTimeRemaining--;

				if (!WarLevelInfo.bTeamDeathMatch)
				{
					RoundTimeRemaining--;
					CheckTriggers((RoundTime*60)-RoundTimeRemaining);
				}
				else
					RoundTimeRemaining = 0;
			}
			else if (RemainingTime <= 0)
			{
				// wave
				MercWaveTimeRemaining = -5;
				SOWaveTimeRemaining = -5;

				RoundTimeRemaining = -5;
			}

			FragOps_GRI(GameReplicationInfo).RoundTimeRemaining = RoundTimeRemaining;

			// wave window
			if (MercWaveWindowSeconds > 0)
			{
				if (MercWaveWindowSeconds == 1)
					AnnounceRejoin(0);
				MercWaveWindowSeconds--;
			}
			War_GRI(GameReplicationInfo).MercWaveWindowSeconds = MercWaveWindowSeconds;

			if (SOWaveWindowSeconds > 0)
			{
				if (SOWaveWindowSeconds == 1)
					AnnounceRejoin(1);
				SOWaveWindowSeconds--;
			}
			War_GRI(GameReplicationInfo).SOWaveWindowSeconds = SOWaveWindowSeconds;

			// wave
			War_GRI(GameReplicationInfo).MercWaveTimeRemaining = MercWaveTimeRemaining;
			if (MercWaveTimeRemaining % 60 == 0)
				War_GRI(GameReplicationInfo).MercWaveTimeMinute = MercWaveTimeRemaining;
			if (MercWaveTimeRemaining <= 0 && TimeTillNextRound <= 0)
				SpawnWave(0);

			War_GRI(GameReplicationInfo).SOWaveTimeRemaining = SOWaveTimeRemaining;
			if (SOWaveTimeRemaining % 60 == 0)
				War_GRI(GameReplicationInfo).SOWaveTimeMinute = SOWaveTimeRemaining;
			if (SOWaveTimeRemaining <= 0 && TimeTillNextRound <= 0)
				SpawnWave(1);

			if (!WarLevelInfo.bTeamDeathMatch)
			{
				if (RoundTimeRemaining % 60 == 0)
					FragOps_GRI(GameReplicationInfo).RoundTimeMinute = RoundTimeRemaining;
				if (RoundTimeRemaining <= 0 && TimeTillNextRound <= 0)
					EndRound(2);
			}
		}

		ElapsedTime++;
		GameReplicationInfo.ElapsedTime = ElapsedTime;
	}
}

function bool CheckEndGame(PlayerReplicationInfo Winner, string Reason)
{
	local Controller C;
	local PlayerController Player;

	if ((TimeTillNextRound > 0 || WarLevelInfo.bTeamDeathMatch) && RemainingTime <= 0)
	{
		// now we'll actually check for the end of the game ;)
		if (Teams[1].Score > Teams[0].Score)
			GameReplicationInfo.Winner = Teams[1];
		else
			GameReplicationInfo.Winner = Teams[0];
		
		if (Winner == None)
		{
			for (C=Level.ControllerList; C!=None; C=C.nextController )
				if ( (C.PlayerReplicationInfo != None) && (C.PlayerReplicationInfo.Team == GameReplicationInfo.Winner)
					&& ((Winner == None) || (C.PlayerReplicationInfo.Score > Winner.Score)))
				{
					Winner = C.PlayerReplicationInfo;
				}
		}

		EndTime = Level.TimeSeconds + EndTimeDelay;

		if (Winner != None)
			EndGameFocus = Controller(Winner.Owner).Pawn;
		if (EndGameFocus != None)
			EndGameFocus.bAlwaysRelevant = true;

		for (C=Level.ControllerList; C!=None; C=C.nextController)
		{
			player = PlayerController(C);
			if (Player != None)
			{
				PlayWinMessage(Player, (Player.PlayerReplicationInfo.Team == GameReplicationInfo.Winner));
				player.ClientSetBehindView(true);
				if ( EndGameFocus != None )
				{
					Player.ClientSetViewTarget(EndGameFocus);
					Player.SetViewTarget(EndGameFocus);
				}
				Player.ClientGameEnded();
				if (CurrentGameProfile != None)
					CurrentGameProfile.bWonMatch = (Player.PlayerReplicationInfo.Team == GameReplicationInfo.Winner);
			}
			C.GameHasEnded();
			C.PlayerReplicationInfo.bOutOfLives = false;
			
			// tally final scores
			FragOps_PRI(C.PlayerReplicationInfo).ResetRoundPRI();
		}
		return true;
	}
	else
		return false;
}


// execute a new wave
function SpawnWave(byte iTeamNum)
{
	local War_BuildZone WaveBZ;
	local War_FOVehicleWeapon VehicleWeapon;
	local War_FOVehicleAir VehicleAir;
	local Controller C;

	// wave window timers
	if (iTeamNum == 1)
	{
		RejoiningSO = 0;
		SOWaveWindowSeconds = WaveWindowTime;
	}
	else
	{
		RejoiningMercs = 0;
		MercWaveWindowSeconds = WaveWindowTime;
	}

	// update GRI too
	War_GRI(GameReplicationInfo).MercWaveWindowSeconds = MercWaveWindowSeconds;
	War_GRI(GameReplicationInfo).SOWaveWindowSeconds = SOWaveWindowSeconds;

	foreach DynamicActors(class'War_BuildZone',WaveBZ)
	{
		if (WaveBZ.ControllingTeam != iTeamNum && WaveBZ.ControllingTeam != 255)
			continue;

		WaveBZ.WaveReset();
	}

	// restore ammunition to vehicles
	foreach DynamicActors(class'War_FOVehicleWeapon',VehicleWeapon)
	{
		if (VehicleWeapon.TeamIndex == iTeamNum && VehicleWeapon.bWaveAlternate)
		{
			VehicleWeapon.NumClips = VehicleWeapon.MaxClips;
			VehicleWeapon.bWaveAlternate = false;
		}
		else
			VehicleWeapon.bWaveAlternate = true;
	}

	// restore ammunition to aerial vehicles
	foreach DynamicActors(class'War_FOVehicleAir',VehicleAir)
	{
		if (VehicleAir.Team == iTeamNum)
			VehicleAir.AmmoCount = VehicleAir.Default.AmmoCount;
	}
	
	for (C=Level.ControllerList;C!=None;C=C.nextController)
	{
		if (C.PlayerReplicationInfo == None || C.PlayerReplicationInfo.Team == None || C.PlayerReplicationInfo.Team.TeamIndex != iTeamNum || !C.bIsPlayer || PlayerCRI(C) == None || (PlayerCRI(C).bWaitingForTeam && FOBot(C) == None))
			continue;

		// award any weapon medals
		CheckWeaponMedals(PlayerCRI(C),C.PlayerReplicationInfo,FOPlayer(C));

		if (War_FOBot(C) != None)
		{
			if (C.IsInState('Dead') || C.Pawn == None) 
			{
				C.PlayerReplicationInfo.bOutOfLives = False;
				C.PlayerReplicationInfo.NumLives = 0;
				RestartDeadPlayer(C);
				if (iTeamNum == 0)
					RejoiningMercs++;
				else
					RejoiningSO++;
			}
			War_FOBot(C).DoWaveCommands();
		}
		else if (War_FOPlayer(C) != None)
			War_FOPlayer(C).DoWaveCommands();

		// reset critical/player replication info
		PlayerCRI(C).ResetRoundCRI();
		War_PRI(C.PlayerReplicationInfo).ResetWavePRI();
	}

	// reset time
	if (iTeamNum == 0)
		MercWaveTimeRemaining = MercWaveTime;
	else
		SOWaveTimeRemaining = SOWaveTime;
}

function AnnounceRejoin(int AnnouncingTeam)
{
	local Controller C;
	local int JoiningPlayers;

	if (WarLevelInfo.bTeamDeathmatch)
		return;

	if (AnnouncingTeam == 0)
		JoiningPlayers = RejoiningMercs;
	else
		JoiningPlayers = RejoiningSO;

	for (C=Level.ControllerList;C!=None;C=C.nextController)
	{
		if (C.PlayerReplicationInfo == None || PlayerCRI(C) == None || C.PlayerReplicationInfo.Team == None || C.PlayerReplicationInfo.Team.TeamIndex != AnnouncingTeam || PlayerCRI(C).bWaitingForTeam || FOBot(C) != None)
			continue;

		PlayerController(C).ReceiveLocalizedMessage(class'War_Interface_GameMessage', JoiningPlayers);
	}
}

function AddRejoin(int JoinTeam)
{
	if (JoinTeam == 0)
		RejoiningMercs++;
	else
		RejoiningSO++;
}

function CheckBuyZonePlayers(Pawn BuildPawn)
{
	FOPawn(BuildPawn).CheckBuildZones();
}

function float RatePlayerStart(NavigationPoint N, byte Team, Controller Player)
{
	local float Score;
	local War_PlayerStart PS;

	// get the usual
	Score = Super.RatePlayerStart(N, Team, Player);
	PS = War_PlayerStart(N);
	
	// fall back
	if (PS == None || FragOps_GRI(GameReplicationInfo).bPreRound)
		return Score;

	// minimize score if not valid
	if (!PS.bEnabled || Team != PS.TeamNumber)
		Score -= 1000000;
	else // otherwise add desireability score
		Score += War_PlayerStart(N).SpawnDesireability * 1000;
	
	// return modified score
	return Score;
}

function bool InWaveWindow(byte WaveTeam)
{
	if (WaveTeam == 0 && MercWaveWindowSeconds > 0)
		return true;
	else if (WaveTeam == 1 && SOWaveWindowSeconds > 0)
		return true;
	
	return false;
}

// objective handlers
function bool CanTrigger(War_Objective DesiredObj, Pawn DesiredPawn)
{
	if (DesiredObj.bCompleted || (FOPawn(DesiredPawn) == None && !DesiredObj.bVehiclesAllowed) || DesiredPawn.Health <= 0 || DesiredPawn.PlayerReplicationInfo == None || DesiredPawn.PlayerReplicationInfo.Team.TeamIndex != DesiredObj.AttackingTeam || FragOps_GRI(GameReplicationInfo).bPreRound || bRoundEnded || (DesiredPawn.Weapon != None && DesiredPawn.Weapon.IsFiring()))
		return false;

	if (DesiredObj.bCanCompleteInAnyOrder)
		return true;

	if (WarLevelInfo.LinkComplete(DesiredObj.AttackingTeam,DesiredObj.LinkNumber) && DesiredObj.bDisableAfterLinkTrigger)
		return false;

	if (DesiredObj.LinkNumber < 1 || WarLevelInfo.LinkComplete(DesiredObj.AttackingTeam,DesiredObj.LinkNumber-1))
		return true;

	return false;
}

function CheckLinks(byte ObjTeam)
{
	local int i;
	local bool bAllComplete;

	bAllComplete = true;

	Switch(ObjTeam)
	{
		case 0:
			for (i=0;i<WarLevelInfo.M_Obj.Length;i++)
			{
				if (!WarLevelInfo.LinkComplete(0,i))
				{
					bAllComplete = false;
					break;
				}
			}
			for (i=0;i<WarLevelInfo.M_CaptureObj.Length;i++)
			{
				if (!WarLevelInfo.CaptureLinkComplete(0,i))
				{
					bAllComplete = false;
					break;
				}
			}
			break;

		default:
			for (i=0;i<WarLevelInfo.S_Obj.Length;i++)
			{
				if (!WarLevelInfo.LinkComplete(1,i))
				{
					bAllComplete = false;
					break;
				}
			}
			for (i=0;i<WarLevelInfo.S_CaptureObj.Length;i++)
			{
				if (!WarLevelInfo.CaptureLinkComplete(1,i))
				{
					bAllComplete = false;
					break;
				}
			}
			break;
	}
	if (bAllComplete)
		EndRound(ObjTeam,true);
}

function CheckWeaponMedals(FragOps_CRI mPlayerCRI, PlayerReplicationInfo mPlayerPRI, FOPlayer APPlayer)
{
	local int NewCXP, OtherTeam;
	local Controller C;
	local int RoundPlayers[2];

	if (mPlayerPRI.Team.TeamIndex == 0)
		OtherTeam = 1;

        for (C=Level.ControllerList;C!=None;C=C.NextController)
	{	
		if (C.GetStateName() != 'JustJoined' && C.PlayerReplicationInfo != None && C.PlayerReplicationInfo.Team != None && PlayerCRI(C) != None && (!PlayerCRI(C).bWaitingForTeam || FOBot(C) != None))
		{
			RoundPlayers[C.PlayerReplicationInfo.Team.TeamIndex] += 1;
		}
	}

	// novice pistol medal, reqs: 1 pistol kill, 33% headshot accuracy
	if (mPlayerCRI.Medals[1] == 0 && mPlayerCRI.mPistolPoints[0] >= 1 && mPlayerCRI.mPistolPoints[1] >= ((mPlayerCRI.mPistolPoints[1] + mPlayerCRI.mPistolPoints[2] + mPlayerCRI.mPistolPoints[3]) * 0.33))
	{
		mPlayerCRI.AddMedal(1);
		NewCXP += 20;
	}

	// advanced pistol medal, reqs: 2 kills, and 50% headshot accuracy or better
	if (mPlayerCRI.Medals[2] == 0 && mPlayerCRI.mPistolPoints[0] >= 2 && mPlayerCRI.mPistolPoints[1] >= ((mPlayerCRI.mPistolPoints[1] + mPlayerCRI.mPistolPoints[2] + mPlayerCRI.mPistolPoints[3]) * 0.5))
	{
		mPlayerCRI.AddMedal(2);
		NewCXP += 30;
	}

	// expert pistol medal, reqs: 2 kills, and 90% headshot accuracy
	if (mPlayerCRI.Medals[3] == 0 && mPlayerCRI.mPistolPoints[0] >= 2 && mPlayerCRI.mPistolPoints[1] >= ((mPlayerCRI.mPistolPoints[1] + mPlayerCRI.mPistolPoints[2] + mPlayerCRI.mPistolPoints[3]) * 0.9))
	{
		mPlayerCRI.AddMedal(3);
	}

	// dont award major medals in the first round
	if (FragOps_GRI(GameReplicationInfo).RoundCount > 1 && RoundPlayers[OtherTeam] > 1)
	{
		// army cross, advanced, reqs: player must have killed 75% of the other team
		if (mPlayerCRI.Medals[4] == 0 && APPlayer != None && FragOps_PRI(mPlayerPRI).RoundKills > 0 && RoundPlayers[OtherTeam] > 0 && float(FragOps_PRI(mPlayerPRI).RoundKills)/float(RoundPlayers[OtherTeam]) >= 0.75)
		{
			mPlayerCRI.AddMedal(4);
			NewCXP += 35;
		}
		
		// silver star, expert, reqs: player must have killed entire other team
		if (mPlayerCRI.Medals[5] == 0 && APPlayer != None && FragOps_PRI(mPlayerPRI).RoundKills > 0 && RoundPlayers[OtherTeam] > 0 && float(FragOps_PRI(mPlayerPRI).RoundKills)/float(RoundPlayers[OtherTeam]) >= 1.0)
		{
			mPlayerCRI.AddMedal(5);
			FragOps_PRI(mPlayerPRI).APModPerm++;
		}
	}

	if (NewCXP > 0)
		War_PRI(mPlayerPRI).AddXP(NewCXP);
}


defaultproperties
{
	GameName="Frag.Ops War"
	Description="The ultimate blend of strategy, action, and hardcore teamplay. Realism is heightened in this mode, and players must be prepared to defend and attack on foot, wheels and air. Building and deploying your own structures and battle weaponry must be second nature. Prepare for War."
	MapPrefix="WAR"
	Acronym="WAR"
	BeaconName="WAR"

	// XP
	KillXP=1
	LosingTeamXP=50
	WinningTeamXP=75
	ObjectiveXP=10

	FOHints(10)="Earn Combat Experience to master more powerful weaponry."	
	FOHints(11)="You can change your loadout while waiting to respawn."
	FOHints(12)="Completing objectives earns Combat Experience."
	FOHints(13)="Use %ALTFIRE% on Melee weapons for a more powerful (but slower) attack."
	FOHints(14)="You can resupply once per wave by entering a build zone."
	FOHints(15)="Use your QuickBar keys to change seats while in a vehicle."
	FOHints(16)="Use your %BUYMENU% key to choose your loadout while dead or during the warmup time."
	FOHints(17)="You can use %SWITCHFIRE 0% and %SWITCHFIRE 1% to rotate an object while building."
	FOHints(18)="Deployable weaponry will have its ammunition restored every other wave."

	// settings
	MapListType="FragOps.War_MapListFragOps"
	HUDType="FragOps.War_Interface_MainHUD"
	ScoreBoardType="FragOps.War_Interface_ScoreBoard"
	PlayerControllerClassName="FragOps.War_FOPlayer"
	DecoTextName="FragOps.FragOpsWar"
	GameReplicationInfoClass=class'War_GRI'
	bAllowPlayerLights=false
	WaveWindowTime=10
	bAllowVehicles=true
	PreRoundWaitTime=14
	SpawnProtectionTime=5.0
	FriendlyFireScale=0.0
}