//=============================================================================
// Frag.Ops  2003 Matt 'SquirrelZero' Farber
//=============================================================================
// FragOps_CRI.uc: CriticalReplicationInfo.
// Variables that are critical to gameplay are contained in this class, using 
// a higher updatefrequency, 3 times that of playerreplicationinfo.
//=============================================================================
class FragOps_CRI extends ReplicationInfo;

// replicated
var int Money, Medals[6];
var bool bWaitingForTeam, bChangedTeams;
var string PCName;

// not replicated
var int mPistolPoints[4];
var int NewParseMedals;
var string ParseMedals;
var bool bLAWCarrier;

replication
{
	reliable if ((bNetDirty) && (Role == ROLE_Authority))
		bWaitingForTeam, Medals, PCName;

	reliable if ((bNetDirty) && (bNetOwner) && (Role == ROLE_Authority))
		Money, bChangedTeams;
}

simulated function PostNetBeginPlay()
{
	local FragOps_GRI FOGRI;

	foreach DynamicActors(class'FragOps_GRI',FOGRI)
	{
		FOGRI.AddCRI(self);
		break;
	}
}

simulated function Destroyed()
{
	local FragOps_GRI FOGRI;
	
	foreach DynamicActors(class'FragOps_GRI',FOGRI)
		FOGRI.RemoveCRI(self);
}

function ResetRoundCRI()
{
	local int i;

	for (i=0;i<ArrayCount(mPistolPoints);i++)
		mPistolPoints[i] = 0;

	NewParseMedals = 0;
	ParseMedals = "";
	bChangedTeams = false;
}

// add a medal
function AddMedal(int i)
{
	if (Medals[i] > 0)
		return;

	Medals[i] = 1;
	if (ParseMedals != "")
		ParseMedals = ParseMedals$","$string(i);
	else
		ParseMedals = string(i);
	NewParseMedals++;
}

// add money
function AddMoney(int newCash)
{
	Money = Clamp(Money + newCash, -9999, 9999);
}

function Reset()
{
	local int i;

	Money = default.Money;
	bChangedTeams = false;

	for (i=0;i<ArrayCount(Medals);i++)
		Medals[i] = 0;
}

defaultproperties
{
	Money=400
	NetUpdateFrequency=50
	bWaitingForTeam=true
}