//=============================================================================
// Frag.Ops  2004 Matt 'SquirrelZero' Farber
//=============================================================================
class FragOps_PRI extends xPlayerReplicationInfo;

// for keeping track of damage
struct DamageCalc
{
	var FragOps_PRI AssociatedPRI;
	var byte Damage;
	var int Hits;
	var byte HeadHits, ArmHits, LegHits, BodyHits;
};

struct NetPacket
{
	var DamageCalc DamageGivers[7];
	var DamageCalc DamageTakers[8];
};

var NetPacket StatsPacket;

// some are replicated, some aren't
var int NumShots, RoundShots, RoundKills, NumShotsHit, RoundShotsHit, NumHeadShots, RoundHeadShots, NumChestShots, NumArmShots, NumLegShots, DamageDone, RoundDamageDone, DamageTaken, RoundDamageTaken;
var bool bShowSpecHUD;
var bool bEscaped;
var bool bHasObjective;
var string PCName;
var int APModPerm;
var byte TraitType;

replication
{
	reliable if ((bNetDirty) && (Role == ROLE_Authority))
		APModPerm, NumShots, NumShotsHit, NumHeadShots, DamageDone, DamageTaken, bShowSpecHUD, PCName, bHasObjective, TraitType;

	reliable if (Role == ROLE_Authority)
		bEscaped;
}

function Reset()
{
	DamageDone = 0;
	DamageTaken = 0;
	RoundDamageDone = 0;
	RoundDamageTaken = 0;
	NumShots = 0;
	NumShotsHit = 0;
	NumHeadShots = 0;
	RoundShots = 0;
	RoundShotsHit = 0;
	RoundHeadShots = 0;
	NumChestShots = 0;
	NumArmShots = 0;
	NumLegShots = 0;
	Score = 0;
	Deaths = 0;
	NumLives = 0;
	bEscaped = false;
	bOutOfLives = true;
	bHasObjective = false;
	APModPerm = 0;
	Super(ReplicationInfo).Reset();
	ResetStats();
}

// reset all medals/endround stuff
function ResetRoundPRI()
{
	// add to totals
	NumShots += RoundShots;
	NumShotsHit += RoundShotsHit;
	NumHeadShots += RoundHeadShots;
	
	// reset accuracy
	RoundShots= 0;
	RoundShotsHit = 0;
	RoundHeadShots = 0;
	RoundKills = 0;

	DamageDone += RoundDamageDone;
	DamageTaken += RoundDamageTaken;

	// reset damage
	RoundDamageDone = 0;
	RoundDamageTaken = 0;

	// reset escape
	bEscaped = false;

	// reset carried objective
	bHasObjective = false;
	ResetStats();
}

function ResetStats()
{
	local int i;
	
	// clear stats tracking arrays
	for (i=0;i<ArrayCount(StatsPacket.DamageTakers);i++)
	{
		StatsPacket.DamageTakers[i].Damage = 0;
		StatsPacket.DamageTakers[i].AssociatedPRI = None;
		StatsPacket.DamageTakers[i].Hits = 0;
		StatsPacket.DamageTakers[i].LegHits = 0;
		StatsPacket.DamageTakers[i].ArmHits = 0;
		StatsPacket.DamageTakers[i].BodyHits = 0;
		StatsPacket.DamageTakers[i].HeadHits = 0;
	}
	for (i=0;i<ArrayCount(StatsPacket.DamageGivers);i++)
	{
		StatsPacket.DamageGivers[i].Damage = 0;
		StatsPacket.DamageGivers[i].AssociatedPRI = None;
		StatsPacket.DamageGivers[i].Hits = 0;
		StatsPacket.DamageGivers[i].LegHits = 0;
		StatsPacket.DamageGivers[i].ArmHits = 0;
		StatsPacket.DamageGivers[i].BodyHits = 0;
		StatsPacket.DamageGivers[i].HeadHits = 0;
	}
}

// add to damagedone
function AddDamageDone(int dd, name HitBone, FragOps_PRI HurtPRI)
{
	local int i;

	RoundDamageDone += dd;
	
	for (i=0;i<ArrayCount(StatsPacket.DamageTakers);i++)
	{
		if (StatsPacket.DamageTakers[i].AssociatedPRI == None)
			break;

		if (StatsPacket.DamageTakers[i].AssociatedPRI == HurtPRI)
		{
			StatsPacket.DamageTakers[i].Damage += dd;
			StatsPacket.DamageTakers[i].Hits++;
			AddLocHit(StatsPacket.DamageTakers[i],HitBone);
			return;
		}
	}
	StatsPacket.DamageTakers[i].AssociatedPRI = HurtPRI;
	StatsPacket.DamageTakers[i].Damage = dd;
	StatsPacket.DamageTakers[i].Hits = 1;
	StatsPacket.DamageTakers[i].LegHits = 0;
	StatsPacket.DamageTakers[i].ArmHits = 0;
	StatsPacket.DamageTakers[i].HeadHits = 0;
	StatsPacket.DamageTakers[i].BodyHits = 0;
	AddLocHit(StatsPacket.DamageTakers[i],HitBone);
}

function AddDamageTaken(int dt, name HitBone, FragOps_PRI AttackerPRI)
{
	local int i;
	
	RoundDamageTaken += dt;

	for (i=0;i<ArrayCount(StatsPacket.DamageGivers);i++)
	{
		if (StatsPacket.DamageGivers[i].AssociatedPRI == None)
			break;

		if (StatsPacket.DamageGivers[i].AssociatedPRI == AttackerPRI)
		{
			StatsPacket.DamageGivers[i].Damage += dt;
			StatsPacket.DamageGivers[i].Hits++;
			AddLocHit(StatsPacket.DamageGivers[i],HitBone);
			return;
		}
	}
	StatsPacket.DamageGivers[i].AssociatedPRI = AttackerPRI;
	StatsPacket.DamageGivers[i].Damage = dt;
	StatsPacket.DamageGivers[i].LegHits = 0;
	StatsPacket.DamageGivers[i].ArmHits = 0;
	StatsPacket.DamageGivers[i].HeadHits = 0;
	StatsPacket.DamageGivers[i].BodyHits = 0;
	AddLocHit(StatsPacket.DamageGivers[i],HitBone);
}

// add a locational hit
function AddLocHit(out DamageCalc InCalc, name InBone)
{
	if (InBone == 'headshot')
		InCalc.HeadHits++;
	else if (InBone == 'rfarm' || InBone == 'lfarm' || InBone == 'lefthand' || InBone == 'righthand')
		InCalc.ArmHits++;
	else if (InBone == 'lthigh' || InBone == 'rthigh' || InBone == 'rfoot' || InBone == 'lfoot' || InBone == 'rightfoot' || InBone == 'leftfoot')	
		InCalc.LegHits++;
	else
		InCalc.BodyHits++;
}

// add a shot
function AddShot()
{
	RoundShots++;
}

// add a hit
function AddHit()
{
	RoundShotsHit++;
}

// add a headshot
function AddHeadShot()
{
	RoundHeadShots++;
}

simulated function SetCharacterName(string S)
{
	Super.SetCharacterName(S);
	UpdateCharacter();
	//log("charactername: "@charactername);
}

defaultproperties
{
	bNoTeamSkins=true
	bForceNoPlayerLights=true
	bShowSpecHUD=false
}