//=============================================================================
// Frag.Ops  2003 Matt 'SquirrelZero' Farber
//=============================================================================
class FragOps_VoicePack extends xVoicePack;

function Timer()
{
	local PlayerController PlayerOwner;
	local string Mesg;

	PlayerOwner = PlayerController(Owner);
	if ((Phrase[PhraseNum] != None) && (bDisplayNextMessage || (DisplayMessage[PhraseNum] != 0)))
	{
		Mesg = ClientParseMessageString(DelayedSender, PhraseString[PhraseNum]);
		PlayerOwner.TeamMessage(DelayedSender,Mesg,'TEAMSAY');
	}
	
	if ((Phrase[PhraseNum] != None) && ((Level.TimeSeconds - PlayerOwner.LastPlaySpeech > 2) || (PhraseNum > 0)))
	{
		PlayerOwner.LastPlaySpeech = Level.TimeSeconds;
		if (PlayerOwner.ViewTarget != None)
			PlayerOwner.ViewTarget.PlaySound(Phrase[PhraseNum], SLOT_Interface,1.7*TransientSoundVolume,,,Pitch,false);
		else
			PlayerOwner.PlaySound(Phrase[PhraseNum], SLOT_Interface,1.7*TransientSoundVolume,,,Pitch,false);

		if (MessageAnim != '')
			FOPlayer(PlayerOwner).Taunt(MessageAnim);

		if (Phrase[PhraseNum+1] == None)
			Destroy();
		else
		{
			SetTimer(GetSoundDuration(Phrase[PhraseNum]), false);
			PhraseNum++;
		}
	}
	else 
		Destroy();
}

static function PlayerSpeech(name Type, int Index, string Callsign, Actor PackOwner)
{
	local name SendMode;
	local PlayerReplicationInfo Recipient;
	local int i;
	local GameReplicationInfo GRI;

	switch (Type)
	{
		case 'ACK':					// Acknowledgements
		case 'FRIENDLYFIRE':		// Friendly Fire
		case 'OTHER':				// Other
			SendMode = 'TEAM';		// Only send to team.
			Recipient = None;		// Send to everyone.
			break;
		case 'ORDER':				// Orders
			SendMode = 'TEAM';		// Only send to team.

			Index = OrderToIndex(Index, PackOwner.Level.Game.Class);

			GRI = PlayerController(PackOwner).GameReplicationInfo;
			if (Callsign == "")
				Recipient = None;
			else 
			{
				for ( i=0; i<GRI.PRIArray.Length; i++ )
					if ( (GRI.PRIArray[i] != None) && (GRI.PRIArray[i].PlayerName == Callsign)
						&& (GRI.PRIArray[i].Team == PlayerController(PackOwner).PlayerReplicationInfo.Team))
					{
						Recipient = GRI.PRIArray[i];
						break;
					}
			}
			break;
		case 'TAUNT':				// Taunts
		case 'HIDDEN':				// Hidden Taunts
			SendMode = 'GLOBAL';	// Send to all teams.
			Recipient = None;		// Send to everyone.
			break;
		default:
			SendMode = 'GLOBAL';
			Recipient = None;
	}

	//Log("PlayerSpeech: "$Type$" Ix:"$Index$" Callsign:"$Callsign$" Recip:"$Recipient);
	if (FOPlayer(PackOwner) != None)
		FOPlayer(PackOwner).SendVoiceMessagePlus(Controller(PackOwner).PlayerReplicationInfo, Recipient, Type, Index, SendMode, Default.Class);
}

static function bool PlayDeathPhrase(Pawn P)
{
	local int pdNum, tryCount;
	local bool foundPhrase;

	if (Default.NumDeathPhrases == 0)
		return false;

	for (tryCount = 0; !foundPhrase && tryCount < 100; tryCount++)
	{
		pdNum = Rand(Default.NumDeathPhrases);

		if( !P.IsHumanControlled() &&  Default.HumanOnlyDeathPhrase[pdNum] == 1)
			continue;

		foundPhrase = true;
	}

	if (!foundPhrase)
	{
		Log("PlayDeathPhrase: Could Not Find Suitable Phrase.");
		return false;
	}

	P.PlaySound(Default.DeathPhrases[pdNum], SLOT_Pain,2.3*P.TransientSoundVolume, true,500,0.9);
	return true;
}

function SetClientAckMessage(int messageIndex, PlayerReplicationInfo Recipient, out Sound MessageSound)
{
	messageIndex = Clamp(messageIndex, 0, numAcks-1);
	MessageSound = AckSound[messageIndex];
	MessageString = AckString[messageIndex];
	MessageAnim = AckAnim[messageIndex];
}

function SetAckMessage(int messageIndex, PlayerReplicationInfo Recipient, out Sound MessageSound)
{
	SetTimer(3 + FRand(), false); // wait for initial order to be spoken
	Phrase[0] = AckSound[messageIndex];
	PhraseString[0] = AckString[messageIndex];
	MessageAnim = AckAnim[messageIndex];
}

function ClientInitialize(PlayerReplicationInfo Sender, PlayerReplicationInfo Recipient, name messagetype, byte messageIndex)
{
	local Sound MessageSound;

	DelayedSender = Sender;
	DisplayString = 0;
	bDisplayPortrait = false;
	bDisplayNextMessage = bShowMessageText && (MessageType != 'TAUNT') && (MessageType != 'AUTOTAUNT');
	if ((PlayerController(Owner).PlayerReplicationInfo != Sender) && ((messagetype == 'ORDER') || (messagetype == 'ACK')) && (Recipient != None))
	{
		Destroy();
		return;
	}

	if (PlayerController(Owner).bNoVoiceMessages || (PlayerController(Owner).bNoVoiceTaunts && (MessageType == 'TAUNT' || MessageType == 'AUTOTAUNT')) || (PlayerController(Owner).bNoAutoTaunts && MessageType == 'AUTOTAUNT'))
	{
		Destroy();
		return;
	}

	if (Sender.bBot)
	{
		BotInitialize(Sender, Recipient, messagetype, messageIndex);
		return;
	}

	SetTimer(0.6, false);

	if (messagetype == 'ACK')
		SetClientAckMessage(messageIndex, Recipient, MessageSound);
	else
	{
		if (messagetype == 'FRIENDLYFIRE')
			SetClientFFireMessage(messageIndex, Recipient, MessageSound);
		else if (messagetype == 'TAUNT')
			SetClientTauntMessage(messageIndex, Recipient, MessageSound);
		else if (messagetype == 'AUTOTAUNT')
		{
			SetClientTauntMessage(messageIndex, Recipient, MessageSound);
			SetTimer(1, false);
		}
		else if (messagetype == 'ORDER')
			SetClientOrderMessage(messageIndex, Recipient, MessageSound);
		else if (messagetype == 'HIDDEN')
			SetClientHiddenMessage(messageIndex, Recipient, MessageSound);
		else // messagetype == Other
			SetClientOtherMessage(messageIndex, Recipient, MessageSound);
	}
	Phrase[0] = MessageSound;
	PhraseString[0] = MessageString;
	DisplayMessage[0] = DisplayString;
	if (PlayerController(Owner).PlayerReplicationInfo == Sender )
		bForceMessageSound = true;
	else if ((PlayerController(Owner).PlayerReplicationInfo == Recipient)
			&& (MessageType != 'TAUNT') && (MessageType != 'AUTOTAUNT') )
		bForceMessageSound = true;
}

defaultproperties
{
	Pitch=+0.94
	
	numAcks=2
	AckString(0)="I copy."
	AckString(1)="Roger that."
	AckString(2)=""

	numFFires=1
	FFireString(0)="Friendly fire! Friendly fire!"
	FFireAbbrev(0)=""
	FFireString(1)=""
	FFireAbbrev(1)=""
	FFireString(2)=""
	FFireAbbrev(2)=""
	FFireString(3)=""
	FFireAbbrev(3)=""

	numTaunts=4
	TauntString(0)="I got one!"
	TauntString(1)="You're leaving here in a bodybag."
	TauntString(2)="There's no escape!"
	TauntString(3)="That's right, run you pansies!"
	TauntString(4)=""
	TauntString(5)=""
	TauntString(6)=""
	TauntString(7)=""
	TauntString(8)=""
	TauntString(9)=""

	OrderString(0)="Watch for cover!"
	OrderAbbrev(0)="Watch for cover!"
	OrderString(1)="Everybody down!"
	OrderAbbrev(1)="Everybody down!"
	OrderString(2)="Keep moving!"
	OrderAbbrev(2)="Keep moving!"
	OrderString(3)="Cover me."
	OrderString(4)="Continue to the next location."
	OrderAbbrev(4)="Next location."
	OrderString(10)="Let's move!"
	OrderString(11)=""
	OrderString(12)=""
	OrderString(13)=""
	OrderString(14)=""

	OtherString(0)="I'm advancing to the objective."
	OtherString(1)="They're attacking from the air!"
	OtherAbbrev(1)="They're attacking from the air!"
	OtherString(2)="Location cleared."
	OtherAbbrev(2)=""
	OtherString(3)="I've got your back."
	OtherAbbrev(3)="I've got your back."
	OtherString(4)="I could use some help here! %1"
	OtherAbbrev(4)="I could use some help!"
	OtherString(5)="Man down!"
	OtherString(6)="I'm taking heavy fire here! %l"
	OtherAbbrev(6)="I'm taking heavy fire!"
	OtherString(7)="I've got hostiles!"
	OtherAbbrev(7)="I've got hostiles!"
	OtherString(8)="I need some backup, fast! %1"
	OtherAbbrev(8)="I need some backup!"
	OtherString(9)="I'm going in!"
	OtherAbbrev(9)="I'm going in!"
	DisplayOtherMessage(9)=0
	OtherString(10)="I need cover, now dammit now! %1"
	OtherAbbrev(10)="I need cover!"
	OtherDelayed(10)=0
	OtherString(11)="Location cleared."
	OtherString(12)="I've got a lock!"
	OtherAbbrev(12)="I've got a lock!"
	DisplayOtherMessage(12)=0
	OtherString(13)="Enemy spotted."
	OtherAbbrev(13)=""
	OtherString(14)="Incoming, get down!"
	OtherAbbrev(14)="Incoming, get down!"
	OtherString(15)="Keep your head down!"
	OtherAbbrev(15)="Keep your head down!"
	DisplayOtherMessage(15)=0
	OtherString(16)="They've locked onto me!"
	OtherAbbrev(16)="Enemy lock!"
	OtherString(17)="Objective complete."
	OtherString(18)="I've got a bad feeling about this..."
	OtherAbbrev(18)="I've got a bad feeling..."
	OtherString(19)="Everybody, down!"
	OtherString(20)="Watch for wheeled attacks!"
	OtherAbbrev(20)="Watch for wheeled attacks!"
	DisplayOtherMessage(20)=0
	OtherString(21)="I can't hold em off anymore! %l!"
	OtherAbbrev(21)="I can't hold em!"
	DisplayOtherMessage(21)=0
	OtherString(22)="I'm heading north."
	OtherAbbrev(22)="I'm heading north."
	DisplayOtherMessage(22)=0
	OtherString(23)="I'm heading south."
	OtherString(24)="I'm heading east."
	OtherString(25)="I'm heading west."
	OtherString(26)="Fire in the hole!"
	OtherString(27)=""
	OtherString(28)=""
	OtherString(29)=""
	OtherString(30)=""
	OtherString(31)=""

	OtherMesgGroup(0)=""
	OtherMesgGroup(1)=""
	OtherMesgGroup(2)=""
	OtherMesgGroup(8)=""
	OtherMesgGroup(12)=""
	OtherMesgGroup(15)=""
	OtherMesgGroup(16)=""
	OtherMesgGroup(17)=""
	OtherMesgGroup(18)=""
	OtherMesgGroup(19)=""
	OtherMesgGroup(20)=""
	OtherMesgGroup(23)=""
	OtherMesgGroup(24)=""
	OtherMesgGroup(25)=""
	OtherMesgGroup(28)=""
	OtherMesgGroup(29)=""

	VoiceGender=VG_None

	NumDeathPhrases=0
}