//=============================================================================
// Frag.Ops  2003 Matt 'SquirrelZero' Farber
//=============================================================================
class Interface_BriefingScreen extends Info;

var() color BGColor, DescriptionColor;

var bool bOpenTimed;
var Interface_MainHUD MyHUD;

// objective arrays for quick access
var array<FragOps_ChainableObjective> SO_ObjectiveChain;
var array<FragOps_ChainableObjective> MC_ObjectiveChain;
var Objective_EscapeVolume MasterEscapeVolume[2];

// briefing image border
var Material BriefBorder, BriefBG;

simulated function PostBeginPlay()
{
	local FragOps_ChainableObjective TempObjective;
	local Objective_EscapeVolume TempEscapeVolume;

	Super.PostBeginPlay();

	// initialize HUD reference
	MyHUD = Interface_MainHUD(Owner);

	// initialize objective arrays
	foreach AllActors(class'FragOps_ChainableObjective',TempObjective)
	{
		if (TempObjective.ObjectiveTeam > 0)
			SO_ObjectiveChain[TempObjective.LinkNumber] = TempObjective;
		else
			MC_ObjectiveChain[TempObjective.LinkNumber] = TempObjective;
	}

	// initialize escape volumes
	foreach AllActors(class'Objective_EscapeVolume',TempEscapeVolume)
	{
		if (!TempEscapeVolume.bMasterEscapeVolume)
			continue;

		MasterEscapeVolume[TempEscapeVolume.EscapingTeam] = TempEscapeVolume;
	}
}

// for time-controlled opening and closing of briefing menu, used when changing teams
simulated function Timer()
{
	if (bOpenTimed)
		MyHUD.bShowBriefing = false;
}

simulated function DrawBriefing(Canvas C)
{
	local int i;
	local byte PlayerTeam;
	local float bgX,bgY, bgSizeX,bgSizeY, ImageSizeX,ImageSizeY, XL,YL;
	local float TextOffsetX, TextOffsetY, CompletionOffsetX, MissionOffsetY;
	local FragOps_LevelInfo MyLevelInfo;
	local string ObjectiveText;
	local Material BriefImage[3];
	local array<string> MissionArray;
	local array<string> RebuiltTextArray;
	local string RebuiltText;

	// dont draw briefing when waiting for a team or joining as spectator
	if (MyHUD == None || FOPlayer(MyHUD.PlayerOwner).CriticalReplicationInfo.bWaitingForTeam || !MyHUD.PlayerOwner.bIsPlayer || MyHUD.PlayerOwner.PlayerReplicationInfo.Team == None)
		return;

	// get player team
	PlayerTeam = MyHUD.PlayerOwner.PlayerReplicationInfo.Team.TeamIndex;

	// initialize box limits
	bgX = (C.ClipX/2) - (C.ClipX/4);
	bgY = (C.ClipY/2) - (C.ClipY/4.6);
	bgSizeX = C.ClipX/2;
	bgSizeY = C.ClipY*0.55;

	// setup margins
	TextOffsetX = bgSizeX * 0.03;
	TextOffsetY = bgSizeY * 0.04;
	
	// draw background box
	C.SetPos(bgX,bgY);
	C.DrawColor = Default.BGColor;
	C.DrawTileStretched(BriefBG,bgSizeX,bgSizeY);
	
	// get an image size for the briefing images
	ImageSizeX = C.ClipX * 0.15;
	ImageSizeY = C.ClipY * 0.15;
	
	// initialize "Completed" text offset
	C.Font = MyHUD.GetSmallerFontFor(C,3);
	C.StrLen("[OPTIONAL]",XL,YL);
	CompletionOffsetX = (C.ClipX*0.75) - TextOffsetX - XL;
	MissionOffsetY = (bgY + ImageSizeY) + (TextOffsetY*1.7);

	// retrieve the FO levelinfo
	MyLevelInfo = FOPlayer(MyHUD.PlayerOwner).GetLevelInfo();
	if (MyLevelInfo != None)
	{	
		for (i=0;i<3;i++)
		{
			if (MyLevelInfo.BriefingImage[i] == None)
				continue;
			
			BriefImage[i] = MyLevelInfo.BriefingImage[i];
		}

		// set to white
		C.DrawColor = MyHUD.Default.WhiteColor;

		// draw left box
		if (BriefImage[0] != None)
		{
			C.SetPos(bgX + TextOffsetX, bgY + TextOffsetY);
			C.DrawTile(BriefImage[0], ImageSizeX, ImageSizeY, 0, 0, BriefImage[0].MaterialUSize(), BriefImage[0].MaterialVSize());
			C.SetPos(bgX + TextOffsetX, bgY + TextOffsetY);
			C.DrawTileStretched(BriefBorder, ImageSizeX, ImageSizeY);
		}

		// draw center box
		if (BriefImage[1] != None)
		{
			C.SetPos((C.ClipX/2) - (ImageSizeX/2), bgY + TextOffsetY);
			C.DrawTile(BriefImage[1], ImageSizeX, ImageSizeY, 0, 0, BriefImage[1].MaterialUSize(), BriefImage[1].MaterialVSize());
			C.SetPos((C.ClipX/2) - (ImageSizeX/2), bgY + TextOffsetY);
			C.DrawTileStretched(BriefBorder, ImageSizeX, ImageSizeY);
		}

		// draw right box
		if (BriefImage[2] != None)
		{
			C.SetPos((C.ClipX-bgX) - TextOffsetX - ImageSizeX, bgY + TextOffsetY);
			C.DrawTile(BriefImage[2], ImageSizeX, ImageSizeY, 0, 0, BriefImage[2].MaterialUSize(), BriefImage[2].MaterialVSize());
			C.SetPos((C.ClipX-bgX) - TextOffsetX - ImageSizeX, bgY + TextOffsetY);
			C.DrawTileStretched(BriefBorder, ImageSizeX, ImageSizeY);
		}

		// retrieve briefing description
		if (PlayerTeam == 0)
			MissionArray = MyLevelInfo.MercBriefing;
		else
			MissionArray = MyLevelInfo.SpecOpsBriefing;

		for (i=0;i<MissionArray.Length;i++)
			RebuiltText = RebuiltText$" "$MissionArray[i];

		// set to turquoise
		C.DrawColor = DescriptionColor;
		C.Font = MyHUD.GetSmallerFontFor(C,4);
		C.WrapStringToArray(RebuiltText, RebuiltTextArray, bgSizeX-(TextOffsetX*2));	

		// draw description
		for (i=0;i<RebuiltTextArray.Length && i<8;i++)
		{
			C.StrLen(RebuiltTextArray[i],XL,YL);
			C.SetPos(bgX + TextOffsetX, (bgY + (TextOffsetY*1.7) + ImageSizeY) + (i*(YL*1.0)));
			C.DrawText(RebuiltTextArray[i], true);
		}
	
		MissionOffsetY = (bgY + ImageSizeY) + ((RebuiltTextArray.Length*YL)*1.03) + (TextOffsetY*1.8);
	}

	// retrieve objective chain
	if (((PlayerTeam == 0 && MC_ObjectiveChain.Length < 1) || (PlayerTeam == 1 && SO_ObjectiveChain.Length < 1)) && MasterEscapeVolume[PlayerTeam] == None)
		return;

	i = 0;

	// draw objective information
	switch (PlayerTeam)
	{
		case 0:
			for (i=0;i<MC_ObjectiveChain.Length;i++)
			{
				if (MC_ObjectiveChain[i] == None)
					continue;
				
				// get the short description
				ObjectiveText = (i+1)$". "$Left(MC_ObjectiveChain[i].ShortDescription, 34);

				// set up the font/text
				C.Font = MyHUD.GetSmallerFontFor(C,4);
				C.StrLen(ObjectiveText,XL,YL);
				C.SetPos(bgX + TextOffsetX, MissionOffsetY + (i*(YL*1.0)));
				C.DrawColor = MyHUD.Default.WhiteColor;

				// draw the description
				C.DrawText(ObjectiveText, false);

				// draw "complete" or "incomplete"
				C.SetPos(CompletionOffsetX, MissionOffsetY + (i*(YL*1.0)));
				if (MC_ObjectiveChain[i].bLinkComplete)
				{
					C.DrawColor = MyHUD.Default.GreenColor;
					C.DrawText("[COMPLETE]",false);
				}
				else if (!MC_ObjectiveChain[i].bCompletionOptional)
				{
					C.DrawColor = MyHUD.Default.RedColor;
					C.DrawText("[PENDING]",false);
				}
				else
				{
					C.DrawColor = MyHUD.Default.GoldColor;
					C.DrawText("[OPTIONAL]",false);
				}
			}
			break;

		case 1:
			for (i=0;i<SO_ObjectiveChain.Length;i++)
			{
				if (SO_ObjectiveChain[i] == None)
					continue;
				
				// get the short description
				ObjectiveText = (i+1)$". "$Left(SO_ObjectiveChain[i].ShortDescription, 34);

				// set up the font/text
				C.Font = MyHUD.GetSmallerFontFor(C,4);
				C.StrLen(ObjectiveText,XL,YL);
				C.SetPos(bgX + TextOffsetX, MissionOffsetY + (i*(YL*1.0)));
				C.DrawColor = MyHUD.Default.WhiteColor;

				// draw the description
				C.DrawText(ObjectiveText, false);

				// draw "complete" or "incomplete"
				C.SetPos(CompletionOffsetX, MissionOffsetY + (i*(YL*1.0)));
				if (SO_ObjectiveChain[i].bLinkComplete)
				{
					C.DrawColor = MyHUD.Default.GreenColor;
					C.DrawText("[COMPLETE]",false);
				}
				else if (!SO_ObjectiveChain[i].bCompletionOptional)
				{
					C.DrawColor = MyHUD.Default.RedColor;
					C.DrawText("[PENDING]",false);
				}
				else
				{
					C.DrawColor = MyHUD.Default.GoldColor;
					C.DrawText("[OPTIONAL]",false);
				}
			}
			break;
		}

		// quick hack for escapes	
		if (MasterEscapeVolume[PlayerTeam] == None)
			return;

		// get the short description				
		ObjectiveText = (i+1)$". "$Left(MasterEscapeVolume[PlayerTeam].ShortDescription, 32);

		// set up the font/text
		C.Font = MyHUD.GetSmallerFontFor(C,4);
		C.StrLen(ObjectiveText,XL,YL);
		C.SetPos(bgX + TextOffsetX, MissionOffsetY + (i*(YL*1.0)));
		C.DrawColor = MyHUD.Default.WhiteColor;

		// draw the description
		C.DrawText(ObjectiveText, false);

		// draw "complete" or "incomplete"
		C.SetPos(CompletionOffsetX, MissionOffsetY + (i*(YL*1.0)));
		C.DrawColor = MyHUD.Default.RedColor;
		C.DrawText("[PENDING]",false);
}

defaultproperties
{
	BriefBG=Material'FOWarTextures.HUD.ObjectiveBorder'
	BriefBorder=Material'FOMisc.HUD.BriefImageBorder'
	DescriptionColor=(R=240,G=240,B=240,A=255)
	BGColor=(R=255,G=255,B=255,A=255)
}