//=============================================================================
// Frag.Ops  2003 Matt 'SquirrelZero' Farber
//=============================================================================
class Interface_ControlBinder extends KeyBindMenu;

var localized string BindingLabel[150];

function LoadCommands()
{
	local int i;

	Super.LoadCommands();

	// Update the MultiColumnList's sortdata array to reflect the indexes of our Bindings array
	for (i=0;i<Bindings.Length;i++)
		li_Binds.AddedItem();
}

function MapBindings()
{
	LoadCustomBindings();
	Super.MapBindings();
}

protected function LoadCustomBindings()
{
	local int i;
	local array<string> KeyBindClasses;
	local class<GUIUserKeyBinding> CustomKeyBindClass;

	// Load custom keybinds from .int files
	PlayerOwner().GetAllInt("XInterface.GUIUserKeyBinding",KeyBindClasses);
	for (i=0;i<KeyBindClasses.Length;i++)
	{
		CustomKeyBindClass = class<GUIUserKeyBinding>(DynamicLoadObject(KeyBindClasses[i],class'Class'));
		if (CustomKeyBindClass != None)
			AddCustomBindings( CustomKeyBindClass.default.KeyData );
	}
}

function AddCustomBindings( array<GUIUserKeyBinding.KeyInfo> KeyData )
{
	local int i;

	for (i=0;i<KeyData.Length;i++)
		CreateAliasMapping(KeyData[i].Alias, KeyData[i].KeyLabel, KeyData[i].bIsSection);
}

function ClearBindings()
{
	local int i, max;

	Super.ClearBindings();
	Bindings = default.Bindings;
	max = Min(Bindings.Length, ArrayCount(BindingLabel));
	for (i=0;i<max;i++)
	{
		if (BindingLabel[i] != "")
			Bindings[i].KeyLabel = BindingLabel[i];
	}
}

defaultproperties
{
	Bindings(0)=(bIsSectionLabel=true,KeyLabel="Movement")
	BindingLabel(0)="Movement"
	Bindings(1)=(KeyLabel="Forward",Alias="MoveForward")
	BindingLabel(1)="Forward"
	Bindings(2)=(KeyLabel="Backward",Alias="MoveBackward")
	BindingLabel(2)="Backward"
	Bindings(3)=(KeyLabel="Strafe Left",Alias="StrafeLeft")
	BindingLabel(3)="Strafe Left"
	Bindings(4)=(KeyLabel="Strafe Right",Alias="StrafeRight")
	BindingLabel(4)="Strafe Right"
	Bindings(5)=(KeyLabel="Strafe Toggle",Alias="Strafe")
	BindingLabel(5)="Strafe Toggle"
	Bindings(6)=(KeyLabel="Jump",Alias="Jump")
	BindingLabel(6)="Jump"
	Bindings(7)=(KeyLabel="Walk",Alias="Walking")
	BindingLabel(7)="Walk"
	Bindings(8)=(KeyLabel="Crouch",Alias="Duck")
	BindingLabel(8)="Crouch"
	Bindings(9)=(KeyLabel="Sprint/Rest",Alias="DoAction")
	BindingLabel(9)="Sprint/Rest"
	Bindings(10)=(KeyLabel="Interact",Alias="Interact")
	BindingLabel(10)="Interact"

	Bindings(11)=(bIsSectionLabel=true,KeyLabel="Looking")
	BindingLabel(11)="Looking"
	Bindings(12)=(KeyLabel="Toggle \"BehindView\"",Alias="ToggleBehindView")
	BindingLabel(12)="Toggle \"BehindView\""
	Bindings(13)=(KeyLabel="Toggle Camera Mode",Alias="ToggleFreeCam")
	BindingLabel(13)="Toggle Camera Mode"

	Bindings(14)=(bIsSectionLabel=true,KeyLabel="Weapons")
	BindingLabel(14)="Weapons"
	Bindings(15)=(KeyLabel="Fire",Alias="Fire")
	BindingLabel(15)="Fire"
	Bindings(16)=(KeyLabel="Alt-Fire",Alias="AltFire")
	BindingLabel(16)="Alt-Fire"
	Bindings(17)=(KeyLabel="Special-Fire",Alias="SpecialFire")
	BindingLabel(17)="Special-Fire"
	Bindings(18)=(bIsSectionLabel=false,KeyLabel="FireMode/Zoom Up",Alias="SwitchFire")
	BindingLabel(18)="FireMode/Zoom Up"
	Bindings(19)=(bIsSectionLabel=false,KeyLabel="FireMode/Zoom Down",Alias="SwitchFire 1")
	BindingLabel(19)="FireMode/Zoom Down"
	Bindings(20)=(KeyLabel="Reload Weapon",Alias="Reload")
	BindingLabel(20)="Reload Weapon"
	Bindings(21)=(KeyLabel="Throw Weapon",Alias="ThrowWeapon")
	BindingLabel(21)="Throw Weapon"
	Bindings(22)=(KeyLabel="Best Weapon",Alias="SwitchToBestWeapon")
	BindingLabel(22)="Best Weapon"
	Bindings(23)=(KeyLabel="Next Weapon",Alias="NextWeapon")
	BindingLabel(23)="Next Weapon"
	Bindings(24)=(KeyLabel="Prev Weapon",Alias="PrevWeapon")
	BindingLabel(24)="Prev Weapon"
	Bindings(25)=(KeyLabel="Last Weapon",Alias="SwitchToLastWeapon")
	BindingLabel(25)="Last Weapon"

	Bindings(26)=(bIsSectionLabel=true,KeyLabel="Weapon Selection")
	BindingLabel(26)="Weapon Selection"
	Bindings(27)=(KeyLabel="Loadout Menu",Alias="BuyMenu")
	BindingLabel(27)="Loadout Menu"
	Bindings(28)=(KeyLabel="Buy Primary Ammo/Resupply",Alias="BuyAmmo")
	BindingLabel(28)="Buy Primary Ammo/Resupply"
	Bindings(29)=(KeyLabel="Primary",Alias="SwitchWeapon 3")
	BindingLabel(29)="Primary"
	Bindings(30)=(KeyLabel="Sidearm",Alias="SwitchWeapon 2")
	BindingLabel(30)="Sidearm"
	Bindings(31)=(KeyLabel="Melee",Alias="SwitchWeapon 1")
	BindingLabel(31)="Melee"
	Bindings(32)=(KeyLabel="Grenade #1",Alias="SwitchWeapon 4")
	BindingLabel(32)="Grenade #1"
	Bindings(33)=(KeyLabel="Grenade #2",Alias="SwitchWeapon 5")
	BindingLabel(33)="Grenade #2"
	Bindings(34)=(KeyLabel="LAW/C4",Alias="SwitchWeapon 6")
	BindingLabel(34)="LAW/C4"


	Bindings(35)=(bIsSectionLabel=true,KeyLabel="Communication")
	BindingLabel(35)="Communication"
	Bindings(36)=(KeyLabel="Say",Alias="Talk")
	BindingLabel(36)="Say"
	Bindings(37)=(KeyLabel="Team Say",Alias="TeamTalk")
	BindingLabel(37)="Team Say"
	Bindings(38)=(KeyLabel="In Game Chat",Alias="InGameChat")
	BindingLabel(38)="In Game Chat"
	Bindings(39)=(KeyLabel="Speech Menu",Alias="SpeechMenuToggle")
	BindingLabel(39)="Speech Menu"
	Bindings(40)=(KeyLabel="Activate Microphone",Alias="VoiceTalk")
	BindingLabel(40)="Activate Microphone"
	Bindings(41)=(KeyLabel="Speak in Public Channel",Alias="Speak Public")
	BindingLabel(41)="Speak in Public Channel"
	Bindings(42)=(KeyLabel="Speak in local Channel",Alias="Speak Local")
	BindingLabel(42)="Speak in local Channel"
	Bindings(43)=(KeyLabel="Speak in Team Channel",Alias="Speak Team")
	BindingLabel(43)="Speak in Team Channel"
	Bindings(44)=(KeyLabel="Toggle Public Chatroom",Alias="TogglePublicChat")
	BindingLabel(44)="Toggle Public Channel"
	Bindings(45)=(KeyLabel="Toggle Local Chatroom",Alias="ToggleLocalChat")
	BindingLabel(45)="Toggle Local Channel"
	Bindings(46)=(KeyLabel="Toggle Team Chatroom",Alias="ToggleTeamChat")
	BindingLabel(46)="Toggle Team Channel"

	Bindings(47)=(bIsSectionLabel=true,KeyLabel="Taunts")
	BindingLabel(47)="Taunts"
	Bindings(48)=(KeyLabel="Pelvic Thrust",Alias="taunt pthrust")
	BindingLabel(48)="Pelvic Thrust"
	Bindings(49)=(KeyLabel="Ass Smack",Alias="taunt asssmack")
	BindingLabel(49)="Ass Smack"
	Bindings(50)=(KeyLabel="Throat Cut",Alias="taunt throatcut")
	BindingLabel(50)="Throat Cut"
	Bindings(51)=(KeyLabel="Brag",Alias="taunt gesture_point")
	BindingLabel(51)="Brag"

	Bindings(52)=(bIsSectionLabel=true,KeyLabel="HUD")
	BindingLabel(52)="HUD"
	Bindings(53)=(KeyLabel="Grow HUD",Alias="GrowHud")
	BindingLabel(53)="Grow HUD"
	Bindings(54)=(KeyLabel="Shrink HUD",Alias="ShrinkHud")
	BindingLabel(54)="Shrink HUD"
	Bindings(55)=(KeyLabel="Show Radar Map",Alias="ToggleRadarMap")
	BindingLabel(55)="Show Radar Map"
	Bindings(56)=(KeyLabel="Mission Briefing",Alias="ShowBriefing")
	BindingLabel(56)="Mission Briefing"
	Bindings(57)=(KeyLabel="ScoreBoard",Alias="ShowScores")
	BindingLabel(57)="ScoreBoard"
	Bindings(58)=(KeyLabel="Roundstats",Alias="ShowRoundstats")
	BindingLabel(58)="RoundStats"

	Bindings(59)=(KeyLabel="ScoreBoard",Alias="ShowScores")
	BindingLabel(59)="ScoreBoard Toggle"
	Bindings(60)=(KeyLabel="ScoreBoard (QuickView)",Alias="ScoreToggle")
	BindingLabel(60)="ScoreBoard"

	Bindings(61)=(bIsSectionLabel=true,KeyLabel="Game")
	BindingLabel(61)="Game"
	Bindings(62)=(KeyLabel="Pause",Alias="Pause")
	BindingLabel(62)="Pause"
	Bindings(63)=(KeyLabel="Screenshot",Alias="shot")
	BindingLabel(63)="Screenshot"
	Bindings(64)=(KeyLabel="Next Inventory Item",Alias="InventoryNext")
	BindingLabel(64)="Next Inventory Item"
	Bindings(65)=(KeyLabel="Previous Inventory Item",Alias="InventoryPrevious")
	BindingLabel(65)="Previous Inventory Item"
	Bindings(66)=(KeyLabel="Activate Current Inventory Item",Alias="InventoryActivate")
	BindingLabel(66)="Activate Current Inventory Item"
	Bindings(67)=(KeyLabel="Show Personal Stats",Alias="ShowStats")
	BindingLabel(67)="Show Personal Stats"
	Bindings(68)=(KeyLabel="View Next Player's Stats",Alias="NextStats")
	BindingLabel(68)="View Next Player's Stats"
	Bindings(69)=(KeyLabel="Server Info",Alias="ServerInfo")
	BindingLabel(69)="Server Info"
	Bindings(70)=(KeyLabel="Vehicle Horn",Alias="playvehiclehorn 0")
	BindingLabel(70)="Vehicle Horn"

	Bindings(71)=(bIsSectionLabel=true,KeyLabel="War Quickbar")
	BindingLabel(71)="War QuickBar"
	Bindings(72)=(KeyLabel="Quickbar 1",Alias="QuickbarFunction 0")
	BindingLabel(72)="Quickbar 1"
	Bindings(73)=(KeyLabel="Quickbar 2",Alias="QuickbarFunction 1")
	BindingLabel(73)="Quickbar 2"
	Bindings(74)=(KeyLabel="Quickbar 3",Alias="QuickbarFunction 2")
	BindingLabel(74)="Quickbar 3"
	Bindings(75)=(KeyLabel="Quickbar 4",Alias="QuickbarFunction 3")
	BindingLabel(75)="Quickbar 4"
	Bindings(76)=(KeyLabel="Quickbar 5",Alias="QuickbarFunction 4")
	BindingLabel(76)="Quickbar 5"
	Bindings(77)=(KeyLabel="Quickbar 6",Alias="QuickbarFunction 5")
	BindingLabel(77)="Quickbar 6"
	Bindings(78)=(KeyLabel="Quickbar 7",Alias="QuickbarFunction 6")
	BindingLabel(78)="Quickbar 7"
	Bindings(79)=(KeyLabel="Quickbar 8",Alias="QuickbarFunction 7")
	BindingLabel(79)="Quickbar 8"
	Bindings(80)=(KeyLabel="Quickbar 9",Alias="QuickbarFunction 8")
	BindingLabel(80)="Quickbar 9"
	Bindings(81)=(KeyLabel="Quickbar 10",Alias="QuickbarFunction 9")
	BindingLabel(81)="Quickbar 10"

	Bindings(82)=(bIsSectionLabel=true,KeyLabel="Miscellaneous")
	BindingLabel(82)="Miscellaneous"
	Bindings(83)=(KeyLabel="Menu",Alias="ShowMenu")
	BindingLabel(83)="Menu"
	Bindings(84)=(KeyLabel="Music Player",Alias="MusicMenu")
	BindingLabel(84)="Music Player"
	Bindings(85)=(KeyLabel="Voting Menu",Alias="ShowVoteMenu")
	BindingLabel(85)="Voting Menu"
	Bindings(86)=(KeyLabel="Toggle Console",Alias="ConsoleToggle")
	BindingLabel(86)="Toggle Console"
	Bindings(87)=(KeyLabel="View Connection Status",Alias="Stat Net")
	BindingLabel(87)="View Connection Status"
	Bindings(88)=(KeyLabel="Cancel Pending Connection",Alias="Cancel")
	BindingLabel(88)="Cancel Pending Connection"
}