//=============================================================================
// Frag.Ops  2003 Matt 'SquirrelZero' Farber
//=============================================================================
class Interface_FOConsole extends ExtendedConsole;

event Message(coerce string Msg, float MsgLife)
{
	if (Msg ~= "")
		return;
	
	if (ScrollBack.Length==MaxScrollBackSize)	// if full, Remove Entry 0
	{
		ScrollBack.Remove(0,1);
		SBHead = MaxScrollBackSize-1;
	}
	else
		SBHead++;

	ScrollBack.Length = ScrollBack.Length + 1;

	Scrollback[SBHead] = Msg;
	Super(Console).Message(Msg,MsgLife);
}

exec function SpeechMenuToggle()
{
	local FOPawn FP;
	
	if (GetStateName() == 'SpeechMenuVisible')
	{
		GotoState('');
		return;
	}

	// Dont let spectators use the speech menu
	if (ViewportOwner.Actor.PlayerReplicationInfo.bOnlySpectator)
		return;

	if (Vehicle(ViewportOwner.Actor.Pawn) != None)
		FP = FOPawn(Vehicle(ViewportOwner.Actor.Pawn).Driver);
	else
		FP = FOPawn(ViewportOwner.Actor.Pawn);

	if (FP == None)
		return;

	// Don't show speech menu if no voice pack type
	if (FP.VoiceClass == None)
		return;

	GotoState('SpeechMenuVisible');
}

state SpeechMenuVisible
{
	function class<TeamVoicePack> GetVoiceClass()
	{
		local FOPawn FP;
		
		if (ViewportOwner == None || ViewportOwner.Actor == None || ViewportOwner.Actor.Pawn == None)
			return None;

		if (Vehicle(ViewportOwner.Actor.Pawn) != None)
			FP = FOPawn(Vehicle(ViewportOwner.Actor.Pawn).Driver);
		else
			FP = FOPawn(ViewportOwner.Actor.Pawn);

		if (FP == None)
			return None;

		return FP.VoiceClass;
	}
	
	function PostRender( canvas Canvas )
	{
		local float XL, YL;
		local int SelLeft, i;
		local float XMax, YMax;

		Canvas.Font = class'UT2MidGameFont'.static.GetMidGameFont(Canvas.ClipX); // Update which font to use.

		// Figure out max key name size
		XMax = 0;
		YMax = 0;
		for(i=0; i<10; i++)
		{
			Canvas.TextSize(NumberToString(i)$"- ", XL, YL);
			XMax = Max(XMax, XL);
			YMax = Max(YMax, YL);
		}
		SMLineSpace = YMax * 1.1;
		SMTab = XMax;

		SelLeft = SMArraySize - SMOffset;

		// First we figure out how big the bounding box needs to be
		XMax = 0;
		YMax = 0;
		DrawNumbers( canvas, Min(SelLeft, 9), SelLeft > 9, true, XMax, YMax);
		DrawCurrentArray( canvas, true, XMax, YMax);
		Canvas.TextSize(SMStateName[SMState], XL, YL);
		XMax = Max(XMax, Canvas.ClipX*(SMOriginX+SMMargin) + XL);
		YMax = Max(YMax, (Canvas.ClipY*SMOriginY) - (1.2*SMLineSpace) + YL);
		// XMax, YMax now contain to maximum bottom-right corner we drew to.

		// Then draw the box
		XMax -= Canvas.ClipX * SMOriginX;
		YMax -= Canvas.ClipY * SMOriginY;
		Canvas.SetDrawColor(255,255,255,255);
		Canvas.SetPos(Canvas.ClipX * SMOriginX, Canvas.ClipY * SMOriginY);
		Canvas.DrawTileStretched(Material'FOMisc.TeamMenu.SquareButtonDark', XMax + (SMMargin*Canvas.ClipX), YMax + (SMMargin*Canvas.ClipY));

		// Draw highlight
		if(bSpeechMenuUseMouseWheel)
		{
			Canvas.SetDrawColor(255,255,255,128);
			Canvas.SetPos( Canvas.ClipX*SMOriginX, Canvas.ClipY*(SMOriginY+SMMargin) + ((HighlightRow - 0.1)*SMLineSpace) );
			Canvas.DrawTileStretched(Material'FOMisc.TeamMenu.SquareButtonDark', XMax + (SMMargin*Canvas.ClipX), 1.1*SMLineSpace );
		}

		// Then actually draw the stuff
		DrawNumbers( canvas, Min(SelLeft, 9), SelLeft > 9, false, XMax, YMax);
		DrawCurrentArray( canvas, false, XMax, YMax);

		// Finally, draw a nice title bar.
		Canvas.SetDrawColor(255,255,255,255);
		Canvas.SetPos(Canvas.ClipX*SMOriginX, (Canvas.ClipY*SMOriginY) - (1.5*SMLineSpace));
		Canvas.DrawTileStretched(Material'FOMisc.TeamMenu.SquareButtonDark', XMax + (SMMargin*Canvas.ClipX), (1.5*SMLineSpace));

		Canvas.SetDrawColor(255,255,128,255);
		Canvas.SetPos(Canvas.ClipX*(SMOriginX+SMMargin), (Canvas.ClipY*SMOriginY) - (1.2*SMLineSpace));
		Canvas.DrawText(SMStateName[SMState]);
	}
}

state ConsoleVisible
{
	function PostRender( canvas Canvas )
	{

		local float fw,fh;
		local float yclip,y;
		local int idx;

		Canvas.Font = class'Interface_MainHUD'.static.GetSmallConsoleFont(Canvas);
		yclip = canvas.ClipY*0.5;
		Canvas.StrLen("X",fw,fh);

		Canvas.SetPos(0,0);
		Canvas.SetDrawColor(255,255,255,200);
		Canvas.Style=4;
		Canvas.DrawTileStretched(Material'FOMisc.MainMenu.Console_FB',Canvas.ClipX,yClip);
		Canvas.Style=1;

		Canvas.SetDrawColor(255,255,255,255);

		Canvas.SetPos(2,yclip-5-fh);
		Canvas.DrawText("(>"@TypedStr$"_");

		idx = SBHead - SBPos;
		y = yClip-y-5-(fh*2);

		if (ScrollBack.Length==0)
			return;

		Canvas.SetDrawColor(255,255,255,255);
		while (y>fh && idx>=0)
		{
			Canvas.SetPos(2,y);
			Canvas.DrawText(Scrollback[idx],false);
			idx--;
			y-=fh;
		}
	}
}

exec function ServerInfo()
{
	local GUIController GC;

	GC = GUIController(ViewportOwner.GUIController);

	if (GC==None)
		return;

	GC.OpenMenu("FragOps.Interface_ServerInfo");
}

defaultproperties
{
}