//=============================================================================
// Frag.Ops  2003 Matt 'SquirrelZero' Farber
//=============================================================================

#exec OBJ LOAD FILE=FOMisc.utx

class Interface_MainHUD extends HudBase;

// more HUD areas
var() globalconfig bool bShowArmorGuy;
var() globalconfig bool bShowMoney;

// more fonts
var localized string FontAltNames[9];
var Font FontAltFonts[9];

// kill messages
struct KillMessage
{
	var class<DamageType> mDamageType;
	var string mKiller;
	var string mKilled;
	var byte KillerTeam;
	var byte KilledTeam;
	var float mLife;
};
var KillMessage KillMessages[8];
var float KillMessagePosX, KillMessagePosY;

// scenemanager array
var array<SceneManager> MySceneManagers;

// gamereplicationinfo
var FragOps_GRI GRI;

// roundstats
var() class<Interface_RoundStats> RoundStatsClass;
var Interface_RoundStats RoundStats;

// briefing
var() class<Interface_BriefingScreen> BriefingClass;
var Interface_BriefingScreen BriefingScreen;

// level action, like loading the map, pausing, etc
var() Font LevelActionFontFont;
var() Color LevelActionFontColor;
var() float LevelActionPositionX, LevelActionPositionY;

// numeric digit set
var() DigitSet DigitsBig;

// numeric widgets
var() NumericWidget HealthCount, AmmoCount, ClipCount, RoundMinutes, RoundSeconds, MapMinutes, MapSeconds, MoneyCount, ProgressCountNum, APCount, APTotal;
var() NumericWidget AmmoSecCount; // for complex hud

// spritewidgets
// bottom left
var() SpriteWidget PlayerBG,  HealthCross, ArmorGuy, ArmorGuyBG, ArmorGuyOutline, ArmorTypeBG, APBG, APIcon, ArmorType, HeadZone, TorsoZone, LeftArmZone, RightArmZone, LeftLegZone, RightLegZone, PoisonLabel;
var() SpriteWidget ArmorGuySmall, ArmorGuyBGSmall, ArmorGuyOutlineSmall, HeadZoneSmall, TorsoZoneSmall, RightArmZoneSmall, LeftArmZoneSmall, LeftLegZoneSmall, RightLegZoneSmall;

// bottom right
var() SpriteWidget WeaponBG, ClipIcon, AmmoIcon, AmmoSecIcon, WeaponIcon, GrenadeCharge, GrenadeChargeTemp, GrenadeMaxIcon, AmmoInfoBG, FireInfoBG;

// top middle
var() SpriteWidget TimerBG, TimeSeperator[2];

// top left
var() SpriteWidget MoneyBG, MoneyIcon;

// others
var() SpriteWidget LaserHUDTexture, LaserTextureTemp;
var() SpriteWidget ProgressIcon, ProgressWiper, ProgressWiperBorder, ProgressFlashIcon;
var() SpriteWidget LAWCenterTex, LAWCenterIdleTex, LAWPointerTex, LAWAimerTex, LAWZoomTex;
var() SpriteWidget StopWatchBG, StopWatchHand, StopWatchHandSmall;

// spectating extras
var Material ReloadingIcon;

// colors
var() color TransparentColor, TranslucentColor;

// playername when pointed at playerpawn
var PlayerReplicationInfo NamedPlayer;
var float NameTime;

// various numerics
var float LastX, LastY, CurAmmoPrimary, MaxAmmoPrimary, CurHealth, ObjectiveTime, FlashTime, FlashAmount, InitialFlashTime, InitialFlashAmount, LastFlashSoundTime, ObjectiveStart, WiperStart, fPrevProgress;
var int CurInventoryGroup, LastInventoryGroup, ClipAmount;
var float HeadDamage, TorsoDamage, LeftArmDamage, LeftLegDamage, RightArmDamage, RightLegDamage;

// various booleans
var bool bShowRoundStats, bShowBriefing, bShowProgressBar, bShowStopWatch, bShowRestBar, bShowBreathBar, bFlashBar, bSecondStage, bThirdStage, bFlashed, bForceProgressStop;

// weapon name drawn when switched
var float CurrentWeaponPositionX, CurrentWeaponPositionY;
var string CurWeaponName;

// for first person spectating
var FOSpectatorWeapon SpectatorWeapon, SpectatorWeaponAkimbo;
var string LastName;

//=============================================================================
// initialization functions

function PostBeginPlay()
{
	local SceneManager SM;

	Super.PostBeginPlay();
	foreach AllActors(class'SceneManager',SM)
	{
		MySceneManagers.Length = MySceneManagers.Length+1;
		MySceneManagers[MySceneManagers.Length-1] = SM;
	}
}

static function Font LoadAltFontStatic(int i)
{
	if (default.FontAltFonts[i] == None)
	{
		default.FontAltFonts[i] = Font(DynamicLoadObject(default.FontAltNames[i], class'Font'));
		if (default.FontAltFonts[i] == None)
			Log("Warning: "$default.Class$" Couldn't dynamically load font "$default.FontAltNames[i]);
	}

	return default.FontAltFonts[i];
}

function Font LoadAltFont(int i)
{
	if (FontAltFonts[i] == None)
	{
		FontAltFonts[i] = Font(DynamicLoadObject(FontAltNames[i], class'Font'));
		if (FontAltFonts[i] == None)
			Log("Warning: "$Self$" Couldn't dynamically load font "$FontAltNames[i]);
	}
	return FontAltFonts[i];
}

function Font GetAltFontSizeIndex(Canvas C, int FontSize)
{
	if (C.ClipX >= 512)
		FontSize++;
	if (C.ClipX >= 640)
		FontSize++;
	if (C.ClipX >= 800)
		FontSize++;
	if (C.ClipX >= 1024)
		FontSize++;
	if (C.ClipX >= 1280)
		FontSize++;
	if (C.ClipX >= 1600)
		FontSize++;

	return LoadAltFont(Clamp( 8-FontSize, 0, 8));
}

function PrecacheFonts(Canvas C)
{
	FontsPrecached++;
	C.DrawColor = Default.TransparentColor;
	C.Font = GetConsoleFont(C);
	C.SetPos(0,0);
	C.DrawText("<>_Aa1");

	C.Font = GetFontSizeIndex(C,-2);
	C.SetPos(0,0);
	C.DrawText("Aa1");

	C.Font = GetFontSizeIndex(C,-4);
	C.SetPos(0,0);
	C.DrawText("Aa1");

	C.Font = GetFontSizeIndex(C,MessageFontOffset);
	C.SetPos(0,0);
	C.DrawText("Aa1");

	C.Font = GetFontSizeIndex(C,1 + MessageFontOffset);
	C.SetPos(0,0);
	C.DrawText("Aa1");

	C.Font = GetFontSizeIndex(C,2 + MessageFontOffset);
	C.SetPos(0,0);
	C.DrawText("Aa1");

	// alternate fonts now
	C.Font = GetAltFontSizeIndex(C,-2);
	C.SetPos(0,0);
	C.DrawText("Aa1");

	C.Font = GetAltFontSizeIndex(C,-4);
	C.SetPos(0,0);
	C.DrawText("Aa1");

	C.Font = GetAltFontSizeIndex(C,MessageFontOffset);
	C.SetPos(0,0);
	C.DrawText("Aa1");

	C.Font = GetAltFontSizeIndex(C,1 + MessageFontOffset);
	C.SetPos(0,0);
	C.DrawText("Aa1");

	C.Font = GetAltFontSizeIndex(C,2 + MessageFontOffset);
	C.SetPos(0,0);
	C.DrawText("Aa1");
}

function font LoadLevelActionFont()
{
	if (LevelActionFontFont == None)
	{
		LevelActionFontFont = Font(DynamicLoadObject(LevelActionFontName, class'Font'));
		if( LevelActionFontFont == None )
			Log("Warning: "$Self$" Couldn't dynamically load font "$LevelActionFontName);
	}
	return LevelActionFontFont;
}

function SetScoreBoardClass (class<Scoreboard> ScoreBoardClass)
{
	if (ScoreBoard != None )
		ScoreBoard.Destroy();

	if (ScoreBoardClass == None)
		ScoreBoard = None;
	else
	{
		ScoreBoard = Spawn(ScoreBoardClass, Owner);

		if (ScoreBoard == None)
			log("Hud::SetScoreBoard(): Could not spawn a scoreboard of class "$ScoreBoardClass, 'Error');
	}
	if (RoundStatsClass != None)
		RoundStats = Spawn(RoundStatsClass, Owner);
	if (RoundStats == None)
		log("Hud::SetScoreBoard(): Unable to spawn roundstats with a class "$RoundStatsClass, 'Error');
	if (BriefingClass != None)
		BriefingScreen = Spawn(BriefingClass, self);
	if (BriefingScreen == None)
		log("Hud::SetScoreBoard(): Unable to spawn briefingscreen with a class "$BriefingClass, 'Error');
}

//=============================================================================
// execs

exec function ShowRoundStats()
{
	bShowRoundStats = !bShowRoundStats;
}

exec function ShowBriefing()
{
	if (BriefingScreen != None)
		BriefingScreen.bOpenTimed = false;
	bShowBriefing = !bShowBriefing;
}

exec function ShowMoney()
{
	bShowMoney = !bShowMoney;
	SaveConfig();
}

exec function ShowArmorGuy()
{
	bShowArmorGuy = !bShowArmorGuy;
	SaveConfig();
}

exec function GrowHUD()
{
	if (!bShowWeaponInfo)
		bShowWeaponInfo = true;
	else if (!bShowWeaponBar)
		bShowWeaponBar = true;
	else if(!bShowPersonalInfo)
		bShowPersonalInfo = true;
	else if (!bShowArmorGuy)
		bShowArmorGuy = true;
	else if (!bShowMoney)
		bShowMoney = true;
	else if (!bShowPoints)
		bShowPoints = true;
	SaveConfig();
}

exec function ShrinkHUD()
{
	if (bShowPoints)
		bShowPoints = false;
	else if (bShowMoney)
		bShowMoney = false;	
	else if (bShowArmorGuy)
		bShowArmorGuy = false;
	else if(bShowPersonalInfo)
		bShowPersonalInfo = false;
	else if (bShowWeaponBar)
		bShowWeaponBar = false;
	else if (bShowWeaponInfo)
		bShowWeaponInfo = false;
	SaveConfig();
}

//=============================================================================
// message functions
function Message(PlayerReplicationInfo PRI, coerce string Msg, name MsgType)
{
	local Class<LocalMessage> MsgClass;

	if (Msg ~= "")
		return;

	switch(MsgType)
	{
		case 'Say':
			Msg = PRI.PlayerName$": "$Msg;
			if (PRI.bOutOfLives || FragOps_PRI(PRI).bEscaped)
				AddSayTextMessage(Msg,class'Interface_SayMessage',PRI,MsgType,true);
			else
				AddSayTextMessage(Msg,class'Interface_SayMessage',PRI,MsgType,false);
			return;
		case 'TeamSay':
			Msg = PRI.PlayerName$"("$PRI.GetLocationName()$"): "$Msg;
			if (PRI.bOutOfLives || FragOps_PRI(PRI).bEscaped)
				AddSayTextMessage(Msg,class'TeamSayMessagePlus',PRI,MsgType,true);
			else
				AddSayTextMessage(Msg,class'TeamSayMessagePlus',PRI,MsgType,false);
			return;
		case 'CriticalEvent':
			MsgClass = class'CriticalEventPlus';
			LocalizedMessage(MessageClass, 0, None, None, None, Msg);
			return;
		case 'DeathMessage':
			MsgClass = class'xDeathMessage';
			break;
		default:
			MsgClass = class'StringMessagePlus';
			break;
	}
	AddTextMessage(Msg,MsgClass,PRI);
}

function AddSayTextMessage(string M, class<LocalMessage> MessageClass, PlayerReplicationInfo PRI, name MsgType, bool bDead)
{
	local int i;

	if (bDead && !PlayerOwner.IsDead())
		return;

	if (bMessageBeep && MessageClass.Default.bBeep)
		PlayerOwner.PlayBeepSound();

	for (i=0;i<ConsoleMessageCount;i++)
	{
		if (TextMessages[i].Text == "")
			break;
	}

	if (i == ConsoleMessageCount)
	{
		for (i=0;i<ConsoleMessageCount-1;i++)
			TextMessages[i] = TextMessages[i+1];
	}

	TextMessages[i].Text = M;
	TextMessages[i].MessageLife = Level.TimeSeconds + MessageClass.Default.LifeTime;
	if (!bDead)
		TextMessages[i].TextColor = MessageClass.static.GetConsoleColor(PRI);
	else
		TextMessages[i].TextColor = (MessageClass.static.GetConsoleColor(PRI))*0.65;
	TextMessages[i].PRI = PRI;
}

function AddKillMessage(class<DamageType> KillDamageType, PlayerReplicationInfo Killer, PlayerReplicationInfo Killed)
{
	local int i;
	local string S;

	if (Killed == None)
		return;

	for (i=0;i<8;i++)
	{
		if (KillMessages[i].mDamageType == None)
			break;
	}

	if (i == 8)
	{
		for (i=0;i<7;i++)
			KillMessages[i] = KillMessages[i+1];
	}

	KillMessages[i].mDamageType = KillDamageType;
	KillMessages[i].mLife = Level.TimeSeconds + 5.0;
	KillMessages[i].KillerTeam = 255;
	KillMessages[i].KilledTeam = 255;

	if (Killer != None)
	{
		KillMessages[i].mKiller = Killer.PlayerName;
		KillMessages[i].KillerTeam = Killer.Team.TeamIndex;
	}

	KillMessages[i].mKilled = Killed.PlayerName;
	KillMessages[i].KilledTeam = Killed.Team.TeamIndex;

	// set console string
	if (Killer != Killed && Killer != None)
		S = Killer.PlayerName$" "$GetKillMethod(KillDamageType,KillMessages[i].mKiller,KillMessages[i].mKilled)$" "$KillMessages[i].mKilled$" "$GetWeaponKillType(KillDamageType);
	else
		S = Killed.PlayerName$" "$GetKillMethod(KillDamageType,KillMessages[i].mKiller,KillMessages[i].mKilled);

	// add console message
	if (PlayerOwner.Player != None)
		PlayerOwner.Player.Console.Message(S,5);
}

function DisplayMessages(Canvas C)
{
	local int i, j, XPos, YPos,MessageCount;
	local float XL, YL;

	for (i=0;i<ConsoleMessageCount;i++)
	{
		if (TextMessages[i].Text == "")
			break;
		else if (TextMessages[i].MessageLife < Level.TimeSeconds)
		{
			TextMessages[i].Text = "";

			if (i < ConsoleMessageCount-1 )
			{
				for (j=i;j<ConsoleMessageCount-1;j++)
					TextMessages[j] = TextMessages[j+1];
			}
			TextMessages[j].Text = "";
			break;
		}
		else
			MessageCount++;
	}

	XPos = (ConsoleMessagePosX * HudCanvasScale * C.SizeX) + (((1.0 - HudCanvasScale) / 2.0) * C.SizeX);
	YPos = (ConsoleMessagePosY * HudCanvasScale * C.SizeY) + (((1.0 - HudCanvasScale) / 2.0) * C.SizeY);

	C.Font = GetConsoleFont(C);
	C.DrawColor = ConsoleColor;

	C.TextSize("A", XL, YL);

	for (i=0;i<MessageCount;i++)
	{
		if ( TextMessages[i].Text == "" )
			break;

		C.StrLen( TextMessages[i].Text, XL, YL );
		C.SetPos( XPos, YPos );
		C.DrawColor = TextMessages[i].TextColor;
		C.DrawText( TextMessages[i].Text, false );
		YPos += YL;
	}

	DisplayKillMessages(C);
}

function DisplayKillMessages(Canvas C)
{
	local int i, j, XPos, YPos, MessageCount;
	local float XL, YL;
	local string KillMethod, WeaponKillType;

	for (i=0;i<8;i++)
	{
		if (KillMessages[i].mDamageType == None)
			break;
		else if (KillMessages[i].mLife < Level.TimeSeconds)
		{
			KillMessages[i].mDamageType = None;
			if (i<7)
			{
				for (j=i;j<7;j++)
					KillMessages[j] = KillMessages[j+1];
			}
			KillMessages[j].mDamageType = None;
			break;
		}
		else
			MessageCount++;
	}

	XPos = (KillMessagePosX * HudCanvasScale * C.SizeX) + (((1.0 - HudCanvasScale) / 2.0) * C.SizeX);
	YPos = (KillMessagePosY * HudCanvasScale * C.SizeY) + (((1.0 - HudCanvasScale) / 2.0) * C.SizeY);

	C.Font = GetConsoleFont(C);
	C.DrawColor = ConsoleColor;
	C.TextSize("A",XL,YL);

	for (i=0;i<MessageCount;i++)
	{
		if (KillMessages[i].mDamageType == None)
			break;

		// draw killer name
		C.SetPos(XPos, YPos);
		if (KillMessages[i].mKiller ~= "" || KillMessages[i].mKiller ~= KillMessages[i].mKilled)
		{
			C.DrawColor = GetConsoleColor(KillMessages[i].KilledTeam);
			C.StrLen(KillMessages[i].mKilled$" ",XL,YL);
			C.DrawText(KillMessages[i].mKilled,false);
		}
		else
		{
			C.DrawColor = GetConsoleColor(KillMessages[i].KillerTeam);
			C.StrLen(KillMessages[i].mKiller$" ",XL,YL);
			C.DrawText(KillMessages[i].mKiller,false);
		}

		// draw method of demise
		KillMethod = GetKillMethod(KillMessages[i].mDamageType,KillMessages[i].mKiller,KillMessages[i].mKilled);
		C.SetPos(XPos+XL, YPos);
		C.DrawColor = Default.WhiteColor;
		C.StrLen(KillMethod$" ",XL,YL);
		C.DrawText(KillMethod, false);

		// check if we need to draw killed name
		if (KillMessages[i].mKiller ~= "" || KillMessages[i].mKiller ~= KillMessages[i].mKilled)
		{
			YPos += YL;
			continue;
		}

		// draw killed name
		C.DrawColor = GetConsoleColor(KillMessages[i].KilledTeam);
		C.StrLen(KillMessages[i].mKiller$" "$KillMethod$" ",XL,YL);
		C.SetPos(XPos+XL, YPos);
		C.DrawText(KillMessages[i].mKilled, false);

		// draw weapon type
		WeaponKillType = GetWeaponKillType(KillMessages[i].mDamageType);
		C.DrawColor = class'Interface_SayMessage'.Default.DrawColor;
		C.StrLen(KillMessages[i].mKiller$" "$KillMethod$" "$KillMessages[i].mKilled$" ",XL,YL);
		C.SetPos(XPos+XL, YPos);
		C.DrawText(WeaponKillType, false);

		// increment Y for next message
		YPos += YL;
	}
}

static function color GetConsoleColor(byte ColorTeam)
{
	if (ColorTeam != 0 && ColorTeam != 1)
		return class'Interface_SayMessage'.Default.DrawColor;
		
	if (ColorTeam == 0)
		return class'Interface_SayMessage'.Default.RedTeamColor;
	else
		return class'Interface_SayMessage'.Default.BlueTeamColor;
}

function DisplayHit(vector HitDir, int Damage, class<DamageType> damageType)
{
	if (!DamageType.Default.bCausesBlood)
		PlayerOwner.ClientFlash(DamageType.Default.FlashScale,DamageType.Default.FlashFog);
	else
		Super.DisplayHit(HitDir,Damage,DamageType);
}

function string GetKillMethod(class<DamageType> KillDamageType, string Killer, string Killed)
{
	if (KillDamageType.Name == 'DamType_Turret')		
		return " blew to pieces";
	else if (Killer != Killed && Killer != "")
	{
		if (KillDamageType.Name == 'DamTypeExploBarrel')
			return "took out";
		if (KillDamageType.Name == 'DamType_Poison')
			return "murdered";
		else if (KillDamageType.Name == 'fell' || KillDamageType.Name == 'ConvoyGibbed')
			return "pushed";
		else if (KillDamageType.Name == 'Crushed' || KillDamageType.Name == 'DamTypeTelefragged' || KillDamageType.Name == 'DamType_RoadKill' || KillDamageType.Name == 'DamType_Jeep')
			return "crushed";
		else if (KillDamageType.Name == 'DamType_VehicleKill')
			return "junked";
		else if (KillDamageType.Name == 'Burned')
			return "fried";
		else if (KillDamageType.Name == 'DamType_GroundVehicle' || KillDamageType.Name == 'DamTypeRVRoadkill' || KillDamageType.Name == 'DamRanOver')
			return "ran over";
		else if (KillDamageType.Name == 'DamType_Machete' || KillDamageType.Name == 'DamType_MachetePoison')
			return "cut up";
		else if (KillDamageType.Default.DeathString != "[NADE]")
			return "killed";
		else
			return "blew up";
	}
	else if (Killer ~= Killed && (KillDamageType.Default.DeathString ~= "[NADE]" || KillDamageType.Name == 'DamTypeExploBarrel'))
		return "blew himself up";
	else if (((Killer ~= "" && Killed != "") || (Killer ~= Killed)) && (KillDamageType.Name == 'DamType_VehicleKill' || KillDamageType.Name == 'DamTypeONSVehicle'))
	{
		if (Killed != "" || Killer != "")
			return "died on impact";
	}
	else if (Killer ~= "" && Killed != "" && KillDamageType.Name == 'Drowned')
		return "took on water";
	else if (Killer ~= "" && Killed != "" && KillDamageType.Name == 'Burned')
		return "burned to death";
	else if (Killer ~= Killed || KillDamageType.Name == 'Suicide')
		return "put a gun to his head";
	else if (KillDamageType.Name == 'DamType_TeamChange')
		return "defected to SpecOps";
	else if (KillDamageType.Name == 'DamType_TeamChangeBlue')
		return "defected to Mercs";
	else if (KillDamageType.Name == 'fell')
		return "fell to his death";
	else if (Killer ~= "" && Killed != "")
	{
		if (KillDamageType.Name == 'DamType_TriggeredExplosion')
			return "became a victim";
		else
			return "fell to his death";
	}
	else
		return "error";
}

function string GetWeaponKillType(class<DamageType> KillDamageType)
{
	if (KillDamageType.Name == 'DamTypeExploBarrel')
		return "[BRRL]";
	else if (KillDamageType.Name == 'DamType_Turret')
		return "[AT]";
	else if (KillDamageType.Name == 'DamType_Poison')
		return "[POIS]";
	else if (KillDamageType.Name == 'Crushed' || KillDamageType.Name == 'DamTypeTelefragged')
		return "[SMITE]";
	else if (KillDamageType.Name == 'DamType_VehicleKill')
		return "[SCRAP]";
	else if (KillDamageType.Name == 'Burned')
		return "[FIRE]";
	else if (KillDamageType.Name == 'DamType_GroundVehicle')
		return "[VHCL]";
	else if (KillDamageType.Name == 'DamType_Machete' || KillDamageType.Name == 'DamType_MachetePoison')
		return "[BLADE]";
	else
		return KillDamageType.Default.DeathString;
}

//=============================================================================
// Render the HUD

event PostRender(canvas Canvas)
{
	local float XPos, YPos;
	local plane OldModulate,OM;
	local int OldOpacity;

	BuildMOTD();

	OldModulate = Canvas.ColorModulate;

	Canvas.ColorModulate.X = 1;
	Canvas.ColorModulate.Y = 1;
	Canvas.ColorModulate.Z = 1;
	Canvas.ColorModulate.W = 1;

	LinkActors();

	HUDScale = 1.0;
	HudOpacity = 255;

	ResScaleX = Canvas.SizeX / 640.0;
	ResScaleY = Canvas.SizeY / 480.0;

	LastX = Canvas.ClipX;
	LastY = Canvas.ClipY;

	Canvas.ColorModulate.W = HudOpacity/255;

	// determine if stopwatch should be drawn
	bShowStopWatch = (PawnOwner != None && FOPawn(PawnOwner) != None && PawnOwner.Health > 0 && FOPawn(PawnOwner).bSprinting);
	bShowRestBar = (PawnOwner != None && FOPawn(PawnOwner) != None && PawnOwner.Health > 0 && FOPawn(PawnOwner).IsInState('PlayerResting'));
	bShowBreathBar = (PawnOwner != None && FOPawn(PawnOwner) != None && PawnOwner.Health > 0 && FOPawn(PawnOwner).IsInState('PlayerHoldingBreath'));

	// draw weapon
	if (PawnOwner != None)
	{
		if (!PlayerOwner.bBehindView)
		{
			if (PlayerOwner.bDemoOwner)
				PawnOwner.GetDemoRecordingWeapon();
			else if (PlayerOwner.Pawn != None && War_FOVehiclePawnFree(PlayerOwner.Pawn) != None)
			{
				LastName = "";
				CanvasDrawActors(Canvas, false);
			}
			else if (Pawn(PlayerOwner.ViewTarget) != None && PlayerOwner.ViewTarget != PlayerOwner.Pawn)
				DrawSpectatorWeapon(Canvas);
			else
			{
				if (SpectatorWeapon != None)
					SpectatorWeapon.Destroy();
				if (SpectatorWeaponAkimbo != None)
					SpectatorWeaponAkimbo.Destroy();
				LastName = "";
				CanvasDrawActors(Canvas, false);
			}
		}
		else
			CanvasDrawActors(Canvas, false);

		if (bFlashed)
			DrawFlash(Canvas);
	}
	if (bShowDebugInfo)
	{
		Canvas.Font = GetConsoleFont(Canvas);
		Canvas.Style = ERenderStyle.STY_Alpha;
		Canvas.DrawColor = ConsoleColor;
		PlayerOwner.ViewTarget.DisplayDebug (Canvas, XPos, YPos);
	}
	else if (!bHideHud)
	{
		if (bShowLocalStats)
		{
			if (LocalStatsScreen == None)
				GetLocalStatsScreen();
			if (LocalStatsScreen != None)
			{
				OM = Canvas.ColorModulate;
				Canvas.ColorModulate = OldModulate;
				OldOpacity = Canvas.ColorModulate.W;
				Canvas.ColorModulate.W = 1;
				DisplayMessages(Canvas);
				LocalStatsScreen.DrawScoreboard(Canvas);
				Canvas.ColorModulate.W = OldOpacity;
				Canvas.ColorModulate = OM;
			}
		}
		else
		{
			if ((PlayerOwner == None) || (PawnOwner == None) || (PawnOwnerPRI == None) || ((PlayerOwner.IsInState('JustJoined') || PlayerOwner.IsInstate('Dead') || PlayerOwner.IsSpectating() || PlayerOwner.IsDead()) && PlayerOwner.bBehindView))
				DrawSpectatingHud(Canvas);
			else if (!PawnOwner.bHideRegularHUD)
			{
				DrawHud(Canvas);
				if (bShowProgressBar)
				{
					OM = Canvas.ColorModulate;
					Canvas.ColorModulate = OldModulate;
					DrawProgressBar(Canvas);
					Canvas.ColorModulate = OM;
				}
				if (bShowStopWatch)
				{
					OM = Canvas.ColorModulate;
					Canvas.ColorModulate = OldModulate;
					DrawStopWatch(Canvas);
					Canvas.ColorModulate = OM;
				}
				else if (bShowBreathBar)
				{
					OM = Canvas.ColorModulate;
					Canvas.ColorModulate = OldModulate;
					DrawBreathBar(Canvas);
					Canvas.ColorModulate = OM;
				}
				else if (bShowRestBar)
				{
					OM = Canvas.ColorModulate;
					Canvas.ColorModulate = OldModulate;
					DrawRestBar(Canvas);
					Canvas.ColorModulate = OM;
				}
			}
			else if (PawnOwner.bHideRegularHUD && FOPlayer(PlayerOwner).bBuyMenuOpen)
			{
				DisplayLocalMessages(Canvas);
				DrawSpecTimes(Canvas);
			}
			if (!DrawLevelAction(Canvas))
			{
				if (PlayerOwner!=None)
				{
					if (PlayerOwner.ProgressTimeOut > Level.TimeSeconds)
					{
						DisplayProgressMessages (Canvas);
					}
					else if (MOTDState==1)
						MOTDState=2;
				}
			}

			if (bShowBadConnectionAlert)
				DisplayBadConnectionAlert(Canvas);
			DisplayMessages(Canvas);
		}
		if (bShowScoreBoard)
		{
			if (ScoreBoard != None)
			{
				OM = Canvas.ColorModulate;
				Canvas.ColorModulate = OldModulate;
				if (Scoreboard.bDisplayMessages)
					DisplayMessages(Canvas);
				ScoreBoard.DrawScoreBoard(Canvas);
				Canvas.ColorModulate = OM;
			}
		}
		else if (bShowBriefing)
		{
			if (BriefingScreen != None)
			{
				bShowRoundStats = false;
				OM = Canvas.ColorModulate;
				Canvas.ColorModulate = OldModulate;
				BriefingScreen.DrawBriefing(Canvas);
				Canvas.ColorModulate = OM;
			}
		}		
		else if (bShowRoundStats)
		{
			if (FOPlayer(PlayerOwner).CriticalReplicationInfo == None || FOPlayer(PlayerOwner).CriticalReplicationInfo.bWaitingForTeam)
				bShowRoundStats = false;

			if (RoundStats != None)
			{
				OM = Canvas.ColorModulate;
				Canvas.ColorModulate = OldModulate;
				RoundStats.DrawRoundStats(Canvas);
				Canvas.ColorModulate = OM;
			}
		}
	}
	else if (PawnOwner != None)
		DrawInstructionGfx(Canvas);

	if ((PlayerConsole != None) && PlayerConsole.bTyping)
		DrawTypingPrompt(Canvas, PlayerConsole.TypedStr);

	Canvas.ColorModulate = OldModulate;
}

function DrawSpectatorWeapon(Canvas C)
{
	if (FOPlayer(PlayerOwner).NewViewTarget == None || FOPlayer(PlayerOwner).TargetWeapon == None || Level.Pauser != None)
		return;

	if (PlayerOwner.Pawn != None)
	{
		if (SpectatorWeaponAkimbo != None)
			SpectatorWeaponAkimbo.Destroy();
		if (SpectatorWeapon != None)
			SpectatorWeapon.Destroy();
		return;
	}

	if (SpectatorWeapon == None)
		SpawnSpecWeapon();
	else if (SpectatorWeapon.Master != FOPlayer(PlayerOwner).TargetWeapon || FOPlayer(PlayerOwner).NewViewTarget != SpectatorWeapon.Instigator)
	{
		SpectatorWeapon.bNoRender = true;
		SpectatorWeapon.Destroy();
		SpawnSpecWeapon();
	}

	if (SpectatorWeapon != None && FOPlayer(PlayerOwner).bTargetAkimbo)
	{
		if (SpectatorWeaponAkimbo == None)
			SpawnAkimboSpecWeapon();
	}
	else if (SpectatorWeaponAkimbo != None)
	{
		SpectatorWeaponAkimbo.bNoRender = true;
		SpectatorWeaponAkimbo.Destroy();
		SpectatorWeaponAkimbo = None;
	}

	C.DrawActor(None, false, true); // Clear the z-buffer here
	SpectatorWeapon.RenderOverlays(C);
	if (SpectatorWeaponAkimbo != None)
		SpectatorWeaponAkimbo.RenderOverlays(C);
}

function SpawnSpecWeapon()
{
	SpectatorWeapon = spawn(class'FOSpectatorWeapon',PlayerOwner.ViewTarget,,PlayerOwner.ViewTarget.Location,PlayerOwner.ViewTarget.Rotation);
	SpectatorWeapon.InitializeFor(FOPlayer(PlayerOwner).TargetWeapon, Pawn(PlayerOwner.ViewTarget), FOPlayer(PlayerOwner));
}

function SpawnAkimboSpecWeapon()
{	
	SpectatorWeaponAkimbo = spawn(class'FOSpectatorWeapon',PlayerOwner.ViewTarget,,PlayerOwner.ViewTarget.Location,PlayerOwner.ViewTarget.Rotation);
	SpectatorWeaponAkimbo.bAkimbo = true;
	SpectatorWeaponAkimbo.InitializeFor(FOPlayer(PlayerOwner).TargetWeapon, Pawn(PlayerOwner.ViewTarget), FOPlayer(PlayerOwner));
}

function CanvasDrawActors(Canvas C, bool bClearedZBuffer)
{
	local byte OldAlpha;
	
	if (!PlayerOwner.bBehindView && PawnOwner.Weapon != None)
	{
		if (!bClearedZBuffer)
			C.DrawActor(None, false, true); // Clear the z-buffer here

		OldAlpha = C.ColorModulate.W;
		C.ColorModulate.W = 1;
		C.DrawColor = Default.WhiteColor;
		if (PawnOwner.Weapon != None)
			PawnOwner.Weapon.RenderOverlays(C);
		C.ColorModulate.W = OldAlpha;
	}
}

function DrawHudPassA(Canvas C)
{
	if (bShowPoints && PlayerOwner != None)
		DrawSpriteWidget(C, TimerBG);

	if (bShowPersonalInfo && PawnOwner != None)
		DrawSpriteWidget(C, PlayerBG);

	if (bShowMoney && PawnOwner != None && GRI.Default.bUsesMoney)
		DrawSpriteWidget(C, MoneyBG);

	if (PawnOwner != None && (PawnOwner.Weapon != None || SpectatorWeapon != None || (ONSWeaponPawn(PawnOwner) != None && ONSWeaponPawn(PawnOwner).Gun != None) || War_Vehicle_M240MG(PawnOwner) != None || (War_FOVehicleAir(PawnOwner) != None && War_FOVehicleAir(PawnOwner).Default.AmmoCount > 0)))
	{
		if (bShowWeaponInfo)
			DrawSpriteWidget(C, WeaponBG);
		if (bShowWeaponBar && FinalBlend(WeaponIcon.WidgetTexture).Material != None)
			DrawSpriteWidget(C, WeaponIcon);
	}
	DisplayLocalMessages(C);
}

function DrawHudPassC(Canvas C)
{
	local float XL,YL,ZL, SmallH;
	local string TraitString;
	
	if (bShowPersonalInfo && PawnOwner != None)
	{
		DrawSpriteWidget(C, HealthCross);
		DrawNumericWidget(C, HealthCount, DigitsBig);
	}

	if (bShowArmorGuy && PawnOwner != None)
	{
		DrawSpriteWidget(C, ArmorTypeBG);
		DrawSpriteWidget(C, ArmorGuyBG);
		DrawSpriteWidget(C, HeadZone);
		DrawSpriteWidget(C, TorsoZone);
		DrawSpriteWidget(C, LeftArmZone);
		DrawSpriteWidget(C, RightArmZone);
		DrawSpriteWidget(C, LeftLegZone);
		DrawSpriteWidget(C, RightLegZone);
		DrawSpriteWidget(C, ArmorType);
		DrawSpriteWidget(C, ArmorGuy);
		DrawSpriteWidget(C, ArmorGuyOutline);
		if (FOPawn(PawnOwner) != None)
		{
			if ((PlayerOwner.Pawn != None && ((FOPawn(PlayerOwner.Pawn) != None && FOPawn(PlayerOwner.Pawn).bPoisoned) || (Vehicle(PlayerOwner.Pawn) != None && FOPawn(Vehicle(PlayerOwner.Pawn).Driver).bPoisoned))) || (PlayerOwner.Pawn == None && PlayerOwner.Viewtarget != None && FOPawn(PlayerOwner.ViewTarget) != None && FOPlayer(PlayerOwner).bTargetPoisoned))
				DrawSpriteWidget(C, PoisonLabel);
			DrawSpriteWidget(C, APBG);
			DrawSpriteWidget(C, APIcon);
			DrawAPCount(C);
		}
	}

	if (bShowPoints && PawnOwner != None)
	{
		DrawNumericWidget(C, MapMinutes, DigitsBig);
		DrawNumericWidget(C, MapSeconds, DigitsBig);
		DrawSpriteWidget(C, TimeSeperator[0]);
		DrawNumericWidget(C, RoundMinutes, DigitsBig);
		DrawNumericWidget(C, RoundSeconds, DigitsBig);
		DrawSpriteWidget(C, TimeSeperator[1]);
	}

	if (bShowMoney && GRI.Default.bUsesMoney)
	{
		DrawSpriteWidget(C, MoneyIcon);
		DrawNumericWidget(C, MoneyCount, DigitsBig);
	}
	
	if (bShowWeaponInfo && PawnOwner != None && (PawnOwner.Weapon != None || SpectatorWeapon != None || (ONSWeaponPawn(PawnOwner) != None && ONSWeaponPawn(PawnOwner).Gun != None) || War_Vehicle_M240MG(PawnOwner) != None || (War_FOVehicleAir(PawnOwner) != None && War_FOVehicleAir(PawnOwner).Default.AmmoCount > 0)))
	{
		C.Font = GetSmallerAltFontFor(C,4);
		C.SetDrawColor(255,255,255,255);
		if (War_FOVehiclePawn(PawnOwner) != None && War_FOVehiclePawn(PawnOwner).Gun != None)
			C.DrawScreenText(War_FOVehicleWeapon(ONSWeaponPawn(PawnOwner).Gun).FireType$" / "$War_FOVehicleWeapon(ONSWeaponPawn(PawnOwner).Gun).AmmoType, 0.985, 0.942, DP_MiddleRight);
		else if (War_Vehicle_M240MG(PawnOwner) != None && War_Vehicle_M240MG(PawnOwner).MGWeapon != None)
			C.DrawScreenText(War_FOVehicleWeapon(War_Vehicle_M240MG(PawnOwner).MGWeapon).FireType$" / "$War_FOVehicleWeapon(War_Vehicle_M240MG(PawnOwner).MGWeapon).AmmoType, 0.985, 0.942, DP_MiddleRight);
		else if (War_FOVehicleAir(PawnOwner) != None)
			C.DrawScreenText(War_FOVehicleAir(PawnOwner).FireType$" / "$War_FOVehicleAir(PawnOwner).AmmoType, 0.985, 0.942, DP_MiddleRight);
		else if (PlayerOwner.Pawn != None)
			C.DrawScreenText(Weapon_Base(PawnOwner.Weapon).FireTypeNames[Weapon_Base(PawnOwner.Weapon).CurrentFireType-1]$" / "$GetAmmoName(PawnOwner.Weapon.Default.FireModeClass[0]), 0.985, 0.942, DP_MiddleRight);
		else if (PlayerOwner.ViewTarget != None && FOPawn(PlayerOwner.ViewTarget) != None)
			C.DrawScreenText(SpectatorWeapon.Master.Default.FireTypeNames[FOPlayer(PlayerOwner).TargetFireType-1]$" / "$GetAmmoName(SpectatorWeapon.Master.Default.FireModeClass[0]), 0.985, 0.942, DP_MiddleRight);
		DrawSpriteWidget(C, AmmoIcon);
		DrawSpriteWidget(C, ClipIcon);
		DrawNumericWidget(C, AmmoCount, DigitsBig);
		DrawNumericWidget(C, ClipCount, DigitsBig);
	}

	if (War_FOVehiclePawnFree(PlayerOwner.Pawn) == None && (PawnOwner != PlayerOwner.Pawn) && (PawnOwner != None) && (PawnOwner.PlayerReplicationInfo != None))
	{
		// draw viewed player name
		C.SetDrawColor(255,255,255,255);

		// set last viewed playername
		LastName = PawnOwner.PlayerReplicationInfo.PlayerName;

		if (C.ClipX >= 640)
		{
			C.Font = GetConsoleFont(C);
			C.StrLen("NowViewing",XL,SmallH);
			C.SetPos(C.ClipX*0.5-XL*0.5,C.ClipY*0.84);
			C.DrawText(NowViewing,false);
		}
		C.Font = GetMediumFontFor(C);
		C.SetDrawColor(255,255,90,255);
		C.StrLen(PawnOwner.PlayerReplicationInfo.PlayerName,XL,YL);
		C.SetPos(C.ClipX*0.5-XL*0.5,C.ClipY*0.845 + SmallH);
		C.DrawText(PawnOwner.PlayerReplicationInfo.PlayerName,false);

		if (FragOps_PRI(Pawn(PlayerOwner.ViewTarget).PlayerReplicationInfo).TraitType >= class'Interface_TabPlayerTeam'.Default.Traits.Length)
			TraitString = "[NO TRAIT]";
		else 
			TraitString = "["$caps(class'Interface_TabPlayerTeam'.Default.Traits[FragOps_PRI(Pawn(PlayerOwner.ViewTarget).PlayerReplicationInfo).TraitType].Default.TraitName)$"]";
		C.Font = GetSmallerFontFor(C,2);
		C.StrLen(TraitString,XL,ZL);
		C.SetPos(C.ClipX*0.5-XL*0.5,C.ClipY*0.84 + SmallH + YL);
		C.DrawText(TraitString,false);
	}
	DrawCrosshair(C);
}

function DrawAPCount(Canvas C)
{
	//local string APString;

	DrawNumericWidget(C, APCount, DigitsBig);
	DrawNumericWidget(C, APTotal, DigitsBig);

	//APString = "AP: "$FOPlayer(PlayerOwner).ActionPoints$"/"$FOPlayer(PlayerOwner).GetMaxAP();

	//C.SetPos(C.ClipX*0.02,C.ClipY*0.68);
	//C.Font = GetSmallerFontFor(C,3);
	//C.DrawColor = Default.WhiteColor;
	//C.DrawText(APString);
}

function DrawBreathBar(Canvas C)
{
	local float fProgress;

	// decide how big bar is to be drawn
	fProgress = fClamp(1.0-((Level.TimeSeconds - FOPawn(PawnOwner).BreathHoldStart) / (FOPawn(PawnOwner).BreathHoldTime)), 0.0, 1.0)*C.ClipX*0.0683;

	C.SetDrawColor(255,255,255,220);
	C.SetPos(C.ClipX*0.011,C.ClipY*0.475);
	C.DrawTileStretched(material'FOMisc.HUD.ThickBorder',C.ClipX*0.0765,C.ClipY*0.09);
	C.SetDrawColor(255,255,255,140);
	C.SetPos(C.ClipX*0.0154,C.ClipY*0.481);
	C.DrawTileStretched(material'FOMisc.Special.WhiteBar_fb',fProgress,C.ClipY*0.077);
}

function DrawRestBar(Canvas C)
{
	local float fProgress;

	// decide how big bar is to be drawn
	fProgress = fClamp(((Level.TimeSeconds - FOPawn(PawnOwner).RestStart) / (FOPawn(PawnOwner).RestTime*FOPawn(PawnOwner).RestMod)), 0.0, 1.0)*C.ClipX*0.0683;

	C.SetDrawColor(255,255,255,220);
	C.SetPos(C.ClipX*0.011,C.ClipY*0.475);
	C.DrawTileStretched(material'FOMisc.HUD.ThickBorder',C.ClipX*0.0765,C.ClipY*0.09);
	C.SetDrawColor(255,255,255,140);
	C.SetPos(C.ClipX*0.0154,C.ClipY*0.481);
	C.DrawTileStretched(material'FOMisc.Special.WhiteBar_fb',fProgress,C.ClipY*0.077);
}

function bool DrawLevelAction(Canvas C)
{
	local String LevelActionText;
	local Plane OldModulate;

	if ((Level.LevelAction == LEVACT_None) && (Level.Pauser != none))
		LevelActionText = LevelActionPaused;
	else if ((Level.LevelAction == LEVACT_Loading) || (Level.LevelAction == LEVACT_Precaching))
		LevelActionText = LevelActionLoading;
	else
		LevelActionText = "";

	if (LevelActionText == "")
		return false;

	C.Font = LoadLevelActionFont();
	C.DrawColor = LevelActionFontColor;
	C.Style = ERenderStyle.STY_Alpha;

	OldModulate = C.ColorModulate;
	C.ColorModulate = C.default.ColorModulate;

	C.DrawScreenText (LevelActionText, LevelActionPositionX, LevelActionPositionY, DP_MiddleMiddle);

	C.ColorModulate = OldModulate;

	return true;
}

function DrawFlash(Canvas Canvas)
{
	local float OldAlpha, NewFlashAmount;
	local color FlashColor;

	// remember original modulation
	OldAlpha = Canvas.ColorModulate.W;
	Canvas.ColorModulate.W = 1;

	if (FlashAmount <= 0 || PlayerOwner.IsDead())
	{
		FlashAmount = 0;
		FOPlayer(PlayerOwner).DisableFlash();
		bFlashed = false;
		return;
	}

	// update flash sound/motion blur
	FOPlayer(PlayerOwner).UpdateFlashSound(FlashAmount,InitialFlashAmount);

	// a little less texture, a little more blur
	NewFlashAmount = FlashAmount;
	if (NewFlashAmount < 1.0)
		NewFlashAmount *= 0.95;

	// set up the opacity
	FlashColor = default.WhiteColor;
	FlashColor.A = 255*NewFlashAmount;
	
	// draw flashtile
	Canvas.SetPos(0,0);
	Canvas.DrawColor = FlashColor;
	Canvas.DrawTileStretched(Material'FOMisc.Special.WhiteFlash_fb',Canvas.ClipX,Canvas.ClipY);

	// return to original modulation
	Canvas.ColorModulate.W = OldAlpha;	
}

function DrawStopWatch(Canvas C)
{
	local float fProgress;

	// setup the rotating timer hand
	fProgress = fClamp(((Level.TimeSeconds - FOPawn(PawnOwner).SprintStart) / (FOPawn(PawnOwner).SprintTime)), 0.0, 1.0)*65535.0;
	TexRotator(StopWatchHand.WidgetTexture).Rotation.Yaw = -fProgress;

	// draw stopwatch
	DrawSpriteWidget(C,StopWatchBG);
	DrawSpriteWidget(C,StopWatchHandSmall);
	DrawSpriteWidget(C,StopWatchHand);
}

function DrawProgressBar(Canvas Canvas)
{
	local int ProgressPercent;
	local float fProgress;

	if (bForceProgressStop)
		return;

	// setup the flash if needed
	if (Level.TimeSeconds - ObjectiveStart > ObjectiveTime && !bFlashBar)
	{
		bFlashBar = true;
		FlashTime = Level.TimeSeconds;
	}

	// flash the bar if the objective was completed
	if (bFlashBar && Level.TimeSeconds - FlashTime < 1.0 && Level.TimeSeconds - ObjectiveStart > ObjectiveTime)
	{
		DrawSpriteWidget(Canvas,ProgressFlashIcon);
		TexRotator(ProgressWiper.WidgetTexture).Rotation.Yaw = 0;
		DrawSpriteWidget(Canvas,ProgressWiperBorder);
		DrawProgressCount(Canvas,100);
		DrawSpriteWidget(Canvas,ProgressWiper);
		return;
	}
	else if (Level.TimeSeconds - FlashTime > 1.0 && Level.TimeSeconds - ObjectiveStart > ObjectiveTime)
	{
		bShowProgressBar = false;
		bFlashBar = false;
		return;
	}
	else if (Level.TimeSeconds - ObjectiveStart <= ObjectiveTime)
		bFlashbar = false;

	// determine the percentage
	fProgress = (((Level.TimeSeconds - ObjectiveStart) / ObjectiveTime) * 100.0);
	ProgressPercent = fProgress;

	// setup the color of the background circle
	if (ProgressPercent < 33)
	{
		ProgressIcon.Tints[0].R = 255;
		ProgressIcon.Tints[0].G = 30;
		ProgressIcon.Tints[0].B = 30;
		ProgressIcon.Tints[1] = ProgressIcon.Tints[0];
	}
	else if (ProgressPercent >= 33 && ProgressPercent < 66)
	{
		if (ProgressIcon.Tints[0].G == 30)
			WiperStart = Level.TimeSeconds;			
		if (!bSecondStage)
		{
			bSecondStage = true;
			if (FOPlayer(PawnOwner.Controller).CurrentObjective != None)
				FOPlayer(PawnOwner.Controller).CurrentObjective.PlayIntervalSound();
		}
		ProgressIcon.Tints[0].R = 255;
		ProgressIcon.Tints[0].G = 255;
		ProgressIcon.Tints[0].B = 30;
		ProgressIcon.Tints[1] = ProgressIcon.Tints[0];

	}
	else if (ProgressPercent >= 66)
	{
		if (ProgressIcon.Tints[0].R == 255)
			WiperStart = Level.TimeSeconds;
		if (!bThirdStage)
		{
			bThirdStage = true;
			if (FOPlayer(PawnOwner.Controller).CurrentObjective != None)
				FOPlayer(PawnOwner.Controller).CurrentObjective.PlayIntervalSound();
		}
		ProgressIcon.Tints[0].R = 30;
		ProgressIcon.Tints[0].G = 255;
		ProgressIcon.Tints[0].B = 30;
		ProgressIcon.Tints[1] = ProgressIcon.Tints[0];
	}

	// setup the rotating wiper
	fProgress = (fClamp(((Level.TimeSeconds - WiperStart) / (ObjectiveTime/3.0)), 0.0, 1.0)*65535.0);
	TexRotator(ProgressWiper.WidgetTexture).Rotation.Yaw = fProgress;

	// draw progress bar here
	DrawSpriteWidget(Canvas,ProgressIcon);
	DrawSpriteWidget(Canvas,ProgressWiperBorder);
	DrawProgressCount(Canvas,ProgressPercent);
	DrawSpriteWidget(Canvas,ProgressWiper);
}

function DrawProgressCount(Canvas Canvas, int ObjProgress)
{
	ProgressCountNum.Value = ObjProgress;
	DrawNumericWidget(Canvas,ProgressCountNum,DigitsBig);
}

function DrawCrosshair(Canvas C)
{
	local float NormalScale, OldW, CurrentCrosshairScale;
	local int i, CurrentCrosshair;
	local color CurrentCrosshairColor;
	local SpriteWidget CHtexture;

	if (PawnOwner != None)
	{
		if ((PawnOwner.Weapon != None && Weapon_Base(PawnOwner.Weapon).bWeaponOwnsCross) || (FOPlayer(PlayerOwner).TargetWeapon != None && (FOPlayer(PlayerOwner).bTargetScopeZoomed || FOPlayer(PlayerOwner).TargetWeapon.Default.bWeaponOwnsCross) && PlayerOwner.ViewTarget != None && PlayerOwner.ViewTarget != PlayerOwner.Pawn))
		{
			if (PawnOwner.Weapon != None)
				DrawEnemyName(C);
			return;
		}
	}

	// no crosshair, escape
	if (!bCrosshairShow)
		return;

	// draw the special aiming reticle for air vehicles
	if (War_FOVehicleAir(PawnOwner) != None)
	{
		PawnOwner.SpecialDrawCrosshair(C);
		return;
	}

	if ((PlayerOwner.bBehindView && ONSWeaponPawn(PawnOwner) == None && War_Vehicle_M240MG(PawnOwner) == None) || (!PlayerOwner.bBehindView && War_FOVehicleWheeled(PawnOwner) != None))
		return;

	if (bUseCustomWeaponCrosshairs && (PlayerOwner.Pawn != None) && (PawnOwner.Weapon != None))
	{
		CurrentCrosshair = PawnOwner.Weapon.CustomCrosshair;
		if (CurrentCrosshair == -1 || CurrentCrosshair == Crosshairs.Length)
		{
			DrawEnemyName(C);
			return;
		}

		CurrentCrosshairColor = PawnOwner.Weapon.CustomCrosshairColor;
		CurrentCrosshairScale = PawnOwner.Weapon.CustomCrosshairScale;
		if (PawnOwner.Weapon.CustomCrosshairTextureName != "")
		{
			if (PawnOwner.Weapon.CustomCrosshairTexture == None)
			{
				PawnOwner.Weapon.CustomCrosshairTexture = Texture(DynamicLoadObject(PawnOwner.Weapon.CustomCrosshairTextureName,class'Texture'));
				if ( PawnOwner.Weapon.CustomCrosshairTexture == None )
				{
					log(PawnOwner.Weapon$" custom crosshair texture not found!");
					PawnOwner.Weapon.CustomCrosshairTextureName = "";
				}
			}
			CHTexture = Crosshairs[0];
			CHTexture.WidgetTexture = PawnOwner.Weapon.CustomCrosshairTexture;
		}
	}
	else
	{
		CurrentCrosshair = CrosshairStyle;
		CurrentCrosshairColor = CrosshairColor;
		CurrentCrosshairScale = CrosshairScale;
	}

	// set a default crosshair when spectating
	if (SpectatorWeapon != None)
		CurrentCrosshair = 7;
	else
		CurrentCrosshair = Clamp(CurrentCrosshair, 0, Crosshairs.Length - 1);

	NormalScale = Crosshairs[CurrentCrosshair].TextureScale;
	if (CHTexture.WidgetTexture == None)
		CHTexture = Crosshairs[CurrentCrosshair];
	CHTexture.TextureScale *= CurrentCrosshairScale;

	for (i = 0; i < ArrayCount(CHTexture.Tints); i++)
		CHTexture.Tints[i] = CurrentCrossHairColor;

	OldW = C.ColorModulate.W;
	C.ColorModulate.W = 1;
	DrawSpriteWidget (C, CHTexture);
	C.ColorModulate.W = OldW;
	CHTexture.TextureScale = NormalScale;

	// we don't want this for spectating
	if (PawnOwner.Weapon != None)
		DrawEnemyName(C);
}

function DrawWeaponName(Canvas C)
{
	local float XL,YL, Fade;

	if (bHideWeaponName)
		return;

	if (WeaponDrawTimer > Level.TimeSeconds)
	{
		C.Font = GetSmallerFontFor(C,2);
		C.DrawColor = WeaponDrawColor;

		Fade = WeaponDrawTimer - Level.TimeSeconds;

		if (Fade <=1 )
			C.DrawColor.A = 255 * Fade;

		C.Strlen(LastWeaponName,XL,YL);
		C.SetPos( (C.ClipX/2) - (XL/2), C.ClipY*0.9-YL);
		C.DrawText(LastWeaponName);
	}

	if (PawnOwner == None)
		return;

	if (CurWeaponName != LastWeaponName || CurInventoryGroup != LastInventoryGroup)
	{
		WeaponDrawTimer = Level.TimeSeconds+1.5;
		WeaponDrawColor = Default.WhiteColor;
	}

	LastInventoryGroup = CurInventoryGroup;
	LastWeaponName = CurWeaponName;
}

function DrawEnemyName(Canvas C)
{
	local actor HitActor;
	local vector HitLocation,HitNormal,ViewPos;

	if (bFlashed || PlayerOwner == None || PlayerOwner.PlayerReplicationInfo == None || PlayerOwner.PlayerReplicationInfo.Team == None || PawnOwner == None)
		return;

	ViewPos = PawnOwner.Location + PawnOwner.BaseEyeHeight * vect(0,0,1);
	HitActor = trace(HitLocation,HitNormal,ViewPos+2200*vector(PlayerOwner.Rotation),ViewPos,true);

	if (Pawn(HitActor) != None && Pawn(HitActor).PlayerReplicationInfo != None && HitActor != PawnOwner && Pawn(HitActor).PlayerReplicationInfo.Team != None && ((PlayerOwner.PlayerReplicationInfo.Team != Pawn(HitActor).PlayerReplicationInfo.Team && VSize(PawnOwner.Location - HitActor.Location) < 1200) || (PlayerOwner.PlayerReplicationInfo.Team == Pawn(HitActor).PlayerReplicationInfo.Team)) && !bNoEnemyNames)
	{
		if ((NamedPlayer != Pawn(HitActor).PlayerReplicationInfo) || (Level.TimeSeconds - NameTime > 0.5))
		{
			switch (Pawn(HitActor).PlayerReplicationInfo.Team.TeamIndex)
			{
				case 0:
					PlayerOwner.ReceiveLocalizedMessage(class'Interface_PlayerNameRed',0,Pawn(HitActor).PlayerReplicationInfo);
					break;

				case 1:
					PlayerOwner.ReceiveLocalizedMessage(class'Interface_PlayerNameBlue',0,Pawn(HitActor).PlayerReplicationInfo);
					break;

				default:
					PlayerOwner.ReceiveLocalizedMessage(class'PlayerNameMessage',0,Pawn(HitActor).PlayerReplicationInfo);
					break;
			}
			if (Pawn(HitActor).PlayerReplicationInfo.Team.TeamIndex == PlayerOwner.PlayerReplicationInfo.Team.TeamIndex)
			{
				switch (Pawn(HitActor).PlayerReplicationInfo.Team.TeamIndex)
				{
					case 0:
						PlayerOwner.ReceiveLocalizedMessage(class'Interface_PlayerHealthRed',0,None,None,HitActor);
						break;

					case 1:
						PlayerOwner.ReceiveLocalizedMessage(class'Interface_PlayerHealthBlue',0,None,None,HitActor);
						break;

					default:
						break;
				}
			}
			NameTime = Level.TimeSeconds;
		}
		NamedPlayer = Pawn(HitActor).PlayerReplicationInfo;
	}
}

function DrawSpectatingHud(Canvas C)
{
	local string InfoString;
	local float xl,yl,Full, Height, Top, TextTop, MedH, SmallH, Scale;
	local int i;

	if ((PlayerOwner.PlayerReplicationInfo != none) && (FOPlayer(PlayerOwner).CriticalReplicationInfo != None) && (!FOPlayer(PlayerOwner).CriticalReplicationInfo.bWaitingForTeam) && (FragOps_PRI(PlayerOwner.PlayerReplicationInfo).bShowSpecHUD))
	{
		DrawSpecTimes(C);
		DisplayLocalMessages(C);

		// Hack for tutorials. -- may use in future, will leave alone
		if (MySceneManagers.Length > 0)
		{
			for (i=0;i<MySceneManagers.Length;i++)
				if (MySceneManagers[i].bIsRunning)
					return;
		}

		// initialize scales
		C.Font = GetMediumFontFor(C);
		C.StrLen("W",xl,MedH);
		Height = MedH;
		C.Font = GetConsoleFont(C);
		C.StrLen("W",xl,SmallH);
		Height += SmallH;
		Full = Height;
		Top  = C.ClipY-8-Full;
		TextTop = Top + 4;
		Scale = (Full+16)/128;
		C.SetPos(0,Top-8);
		C.Style=1;

		// initialize GRI
		GRI = FragOps_GRI(PlayerController(Owner).GameReplicationInfo);

		// draw matchover text
		if (FOPlayer(Owner).bDisplayWinner ||  FOPlayer(Owner).bDisplayLoser)
		{
			if (FOPlayer(PlayerOwner).bDisplayWinner)
				InfoString = YouveWonTheMatch;
			else if (FOPlayer(PlayerOwner).bDisplayLoser)
				InfoString = YouveLostTheMatch;

			C.SetDrawColor(255,255,255,255);
			C.Font = GetMediumFontFor(C);
			C.StrLen(InfoString,XL,YL);
			C.SetPos((C.ClipX/2) - (XL/2), Top + (Full/2) - (YL/2));
			C.DrawText(InfoString,false);
		}
		else if (Pawn(PlayerOwner.ViewTarget) != None && Pawn(PlayerOwner.ViewTarget).PlayerReplicationInfo != None)
		{
			LastName = Pawn(PlayerOwner.ViewTarget).PlayerReplicationInfo.PlayerName;
			
			// draw some interesting information about our viewtarget
			C.SetDrawColor(255,255,255,255);

			// draw reloading icon
			if (FOPawn(PlayerOwner.ViewTarget) != None && FOPawn(PlayerOwner.ViewTarget).bIsReloading)
			{
				C.SetPos((C.ClipX*0.5)-(C.ClipX*0.075), C.ClipY*0.8);
				C.DrawTile(ReloadingIcon, C.ClipX*0.15, C.ClipY*0.04, 0, 0, ReloadingIcon.MaterialUSize(), ReloadingIcon.MaterialVSize());
			}

			if (C.ClipX < 640)
				SmallH = 0;
			else
			{
				// draw trait instead of "now viewing"
				if (FragOps_PRI(Pawn(PlayerOwner.ViewTarget).PlayerReplicationInfo).TraitType >= class'Interface_TabPlayerTeam'.Default.Traits.Length)
					InfoString = "[NO TRAIT]";
				else 
					InfoString = "["$caps(class'Interface_TabPlayerTeam'.Default.Traits[FragOps_PRI(Pawn(PlayerOwner.ViewTarget).PlayerReplicationInfo).TraitType].Default.TraitName)$"]";
				C.SetPos((128*Scale*0.75),TextTop);
				C.DrawText(InfoString,false);

				// draw "Health"
				C.StrLen("Health",Xl,Yl);
				C.SetPos(C.ClipX-150-XL,TextTop);
				C.DrawText("Health",false);

				// draw "Armor"
				C.StrLen("Armor",Xl,Yl);
				C.SetPos(C.ClipX-300-XL,TextTop);
				C.DrawText("Armor",false);
				
				// draw "Score"
				InfoString = ScoreText;
				C.StrLen(InfoString,Xl,Yl);
				C.SetPos(C.ClipX-5-XL-(128*Scale*0.75),TextTop);
				C.DrawText(InfoString);
			}

			// draw name 
			C.SetDrawColor(255,255,90,255);
			C.Font = GetMediumFontFor(C);
			C.SetPos((128*Scale*0.75),TextTop+SmallH);
			C.DrawText(Pawn(PlayerOwner.ViewTarget).PlayerReplicationInfo.PlayerName,false);

			// draw armor
			if (FOPawn(PlayerOwner.ViewTarget) != None)
			{
				InfoString = Left(FOPawn(PlayerOwner.ViewTarget).MiscItem, 3);
				C.StrLen(InfoString,Xl,Yl);
				C.SetPos(C.ClipX-300-XL,TextTop+SmallH);
				C.DrawText(InfoString);
			}
			
			// draw HP
			InfoString = string(Pawn(PlayerOwner.ViewTarget).Health);
			C.StrLen(InfoString,Xl,Yl);
			C.SetPos(C.ClipX-150-XL,TextTop+SmallH);
			C.DrawText(InfoString);

			// draw kills
			InfoString = GetScoreValue(Pawn(PlayerOwner.ViewTarget).PlayerReplicationInfo);
			C.StrLen(InfoString,xl,yl);
			C.SetPos(C.ClipX-5-XL-(128*Scale*0.75),TextTop+SmallH);
			C.DrawText(InfoString,false);

			// draw "fire to view next player"
			InfoString = GetScoreTagLine();
			C.Font = GetConsoleFont(C);
			C.StrLen(InfoString,xl,yl);
			C.SetPos((C.ClipX/2) - (XL/2),Top-3-YL);
			C.DrawText(InfoString);
		}
		else
		{
			// if no viewtarget, tell people to click to the next one
			if (PlayerOwner.IsDead())
			{
				if (FragOps_PRI(PlayerOwner.PlayerReplicationInfo).bEscaped)
					InfoString = class'Interface_ScoreBoard'.default.EscapeFireText;
				else if (Pawn(PlayerOwner.ViewTarget) != None && LastName != "" && LastName != PlayerOwner.PlayerReplicationInfo.PlayerName)
					InfoString = LastName$" is dead!  FIRE to view other players.";
				else
					InfoString = class'Interface_ScoreBoard'.default.OutFireText;
			}
			else
				InfoString = InitialViewingString;

			C.SetDrawColor(255,255,255,255);
			C.Font = GetSmallerFontFor(C,1);
			C.StrLen(InfoString,XL,YL);
			C.SetPos( (C.ClipX/2) - (XL/2), Top + (Full/2) - (YL/2));
			C.DrawText(InfoString,false);
		}
	}
}

function DrawSpecTimes(Canvas C)
{
	if (!bShowPoints || PlayerOwner == None)
		return;

	GRI = FragOps_GRI(PlayerController(Owner).GameReplicationInfo);

	if (GRI != None)
	{
		RoundMinutes.Value = GetMinutes(GRI.RoundTimeRemaining);
		RoundSeconds.Value = GetSeconds(GRI.RoundTimeRemaining);
		MapMinutes.Value = GetMinutes(GRI.RemainingTime);
		MapSeconds.Value = GetSeconds(GRI.RemainingTime);
	}

	DrawSpriteWidget (C, TimerBG);
	DrawNumericWidget (C, MapMinutes, DigitsBig);
	DrawNumericWidget (C, MapSeconds, DigitsBig);
	DrawSpriteWidget (C, TimeSeperator[0]);
	DrawNumericWidget (C, RoundMinutes, DigitsBig);
	DrawNumericWidget (C, RoundSeconds, DigitsBig);
	DrawSpriteWidget (C, TimeSeperator[1]);
}

//=============================================================================
// Update variables before drawing

function UpdateHud()
{
	// set GRI
	GRI = FragOps_GRI(PlayerController(Owner).GameReplicationInfo);

	// set Team
	if (PawnOwnerPRI != none && PawnOwnerPRI.Team != None)
		TeamIndex = Clamp(PawnOwnerPRI.Team.TeamIndex, 0, 1);

	// retrieve ammo counts
	CalculateAmmo();
	AmmoCount.Value = CurAmmoPrimary;
	ClipCount.Value = ClipAmount;

	// swap weapon icons if akimbo
	if (PawnOwner != None && PawnOwner.Weapon != None)
	{
		if (!Weapon_Base(PawnOwner.Weapon).IsAkimbo())
			FinalBlend(WeaponIcon.WidgetTexture).Material = Weapon_Base(PawnOwner.Weapon).WeaponIcon;
		else
			FinalBlend(WeaponIcon.WidgetTexture).Material = Weapon_Base(PawnOwner.Weapon).AkimboIcon;
	}

	// retrieve health
	CalculateHealth();
	HealthCount.Value = CurHealth;

	// set hitzones
	if (PlayerOwner.Pawn != None && FOPawn(PlayerOwner.Pawn) != None)
	{
		// retrieve AP counts
		APCount.Value = FOPlayer(PlayerOwner).ActionPoints;
		APTotal.Value = FOPlayer(PlayerOwner).GetMaxAP();

		// get head damage
		HeadDamage = FOPawn(PlayerOwner.Pawn).HeadDamage;
		TorsoDamage = FOPawn(PlayerOwner.Pawn).TorsoDamage;
		LeftArmDamage = FOPawn(PlayerOwner.Pawn).LeftArmDamage;
		LeftLegDamage = FOPawn(PlayerOwner.Pawn).LeftLegDamage;
		RightArmDamage = FOPawn(PlayerOwner.Pawn).RightArmDamage;
		RightLegDamage = FOPawn(PlayerOwner.Pawn).RightLegDamage;

		// retrieve money
		MoneyCount.Value = FOPlayer(PlayerOwner).CriticalReplicationInfo.Money;

		// swap weapon icons if akimbo
		if (PawnOwner != None && PawnOwner.Weapon != None)
		{
			if (!Weapon_Base(PawnOwner.Weapon).IsAkimbo())
				FinalBlend(WeaponIcon.WidgetTexture).Material = Weapon_Base(PawnOwner.Weapon).WeaponIcon;
			else
				FinalBlend(WeaponIcon.WidgetTexture).Material = Weapon_Base(PawnOwner.Weapon).AkimboIcon;
		}

		if (FOPawn(PawnOwner).MiscItem ~= "LIT. Armor")
			ArmorType.WidgetTexture = material'FOMisc.ArmorGuy.HUDArmorLit';
		else if (FOPawn(PawnOwner).MiscItem ~= "MED. Armor")
			ArmorType.WidgetTexture = material'FOMisc.ArmorGuy.HUDArmorMed';
		else
			ArmorType.WidgetTexture = material'FOMisc.ArmorGuy.HUDArmorHeavy';	
	}
	else if (PlayerOwner.ViewTarget != None && FOPawn(PlayerOwner.ViewTarget) != None)
	{
		// otherwise get the spectated viewtarget's information

		// AP counts
		APCount.Value = FOPlayer(PlayerOwner).TargetAP;
		APTotal.Value = FOPlayer(PlayerOwner).TargetMaxAP;
		
		// damage zones
		HeadDamage = FOPlayer(PlayerOwner).TargetHeadDamage;
		TorsoDamage = FOPlayer(PlayerOwner).TargetTorsoDamage;
		LeftArmDamage = FOPlayer(PlayerOwner).TargetLeftArmDamage;
		LeftLegDamage = FOPlayer(PlayerOwner).TargetLeftLegDamage;
		RightArmDamage = FOPlayer(PlayerOwner).TargetRightArmDamage;
		RightLegDamage = FOPlayer(PlayerOwner).TargetRightLegDamage;

		// money
		MoneyCount.Value = FOPlayer(PlayerOwner).TargetMoney;

		// weapon icons
		if (SpectatorWeapon != None)
		{
			if (SpectatorWeaponAkimbo == None)
				FinalBlend(WeaponIcon.WidgetTexture).Material = SpectatorWeapon.Master.Default.WeaponIcon;
			else
				FinalBlend(WeaponIcon.WidgetTexture).Material = SpectatorWeapon.Master.Default.AkimboIcon;
		}

		if (FOPlayer(PlayerOwner).TargetMiscItem ~= "LIT. Armor")
			ArmorType.WidgetTexture = material'FOMisc.ArmorGuy.HUDArmorLit';
		else if (FOPlayer(PlayerOwner).TargetMiscItem ~= "MED. Armor")
			ArmorType.WidgetTexture = material'FOMisc.ArmorGuy.HUDArmorMed';
		else
			ArmorType.WidgetTexture = material'FOMisc.ArmorGuy.HUDArmorHeavy';
	}
	else
		MoneyCount.Value = FOPlayer(PlayerOwner).CriticalReplicationInfo.Money;


	CalculateHitZones();

	if (FOPawn(PawnOwner) != None && FOPawn(PawnOwner).bCanBuy)
		MoneyIcon.WidgetTexture = FinalBlend'FOMisc.HUD2k4.HUDMoneyFlash';
	else
		MoneyIcon.WidgetTexture = material'FOMisc.HUD2k4.HUDMoneySign';

	// retrieve round/map times
	RoundMinutes.Value = GetMinutes(GRI.RoundTimeRemaining);
	RoundSeconds.Value = GetSeconds(GRI.RoundTimeRemaining);
	MapMinutes.Value = GetMinutes(GRI.RemainingTime);
	MapSeconds.Value = GetSeconds(GRI.RemainingTime);
}

function CalculateHitZones()
{
	local float DamageLimit;

	if (FOPawn(PawnOwner) == None)
		return;

	// update head zone
	DamageLimit = FClamp(HeadDamage/50.0, 0.0, 1.0);
	if (DamageLimit > 0.75)
	{
		HeadZone.WidgetTexture = material'FOMIsc.ArmorGuy.HeadFlash_fb';
		HeadZone.Tints[0].R = 215;
		HeadZone.Tints[0].B = 215;
		HeadZone.Tints[0].G = 215;
		HeadZone.Tints[0].A = 190;
	}
	else
	{
		HeadZone.WidgetTexture = material'FOMIsc.ArmorGuy.ZoneHead';
		HeadZone.Tints[0].R = 200;
		HeadZone.Tints[0].G = 0;
		HeadZone.Tints[0].B = 0;
		HeadZone.Tints[0].A = FMin(180.0, (200.0 * FClamp(HeadDamage/50.0, 0.0, 1.0)));
	}
	HeadZone.Tints[1] = HeadZone.Tints[0];

	// update torso zone
	DamageLimit = FClamp(TorsoDamage/50.0, 0.0, 1.0);
	if (DamageLimit > 0.75)
	{
		TorsoZone.WidgetTexture = material'FOMIsc.ArmorGuy.TorsoFlash_fb';
		TorsoZone.Tints[0].R = 215;
		TorsoZone.Tints[0].B = 215;
		TorsoZone.Tints[0].G = 215;
		TorsoZone.Tints[0].A = 190;
	}
	else
	{
		TorsoZone.WidgetTexture = material'FOMIsc.ArmorGuy.ZoneChest';
		TorsoZone.Tints[0].R = 200;
		TorsoZone.Tints[0].G = 0;
		TorsoZone.Tints[0].B = 0;
		TorsoZone.Tints[0].A = FMin(180.0, (200.0 * FClamp(TorsoDamage/50.0, 0.0, 1.0)));
	}
	TorsoZone.Tints[1] = TorsoZone.Tints[0];

	// update left arm zone
	DamageLimit = FClamp(LeftArmDamage/50.0, 0.0, 1.0);
	if (DamageLimit > 0.75)
	{
		LeftArmZone.WidgetTexture = material'FOMIsc.ArmorGuy.RightArmFlash_fb';
		LeftArmZone.Tints[0].R = 215;
		LeftArmZone.Tints[0].B = 215;
		LeftArmZone.Tints[0].G = 215;
		LeftArmZone.Tints[0].A = 190;
	}
	else
	{
		LeftArmZone.WidgetTexture = material'FOMIsc.ArmorGuy.ZoneLeftArm';
		LeftArmZone.Tints[0].R = 200;
		LeftArmZone.Tints[0].G = 0;
		LeftArmZone.Tints[0].B = 0;
		LeftArmZone.Tints[0].A = FMin(180.0, (200.0 * FClamp(LeftArmDamage/50.0, 0.0, 1.0)));
	}
	LeftArmZone.Tints[1] = LeftArmZone.Tints[0];

	// update right arm zone
	DamageLimit = FClamp(RightArmDamage/50.0, 0.0, 1.0);
	if (DamageLimit > 0.75)
	{
		RightArmZone.WidgetTexture = material'FOMIsc.ArmorGuy.LeftArmFlash_fb';
		RightArmZone.Tints[0].R = 215;
		RightArmZone.Tints[0].B = 215;
		RightArmZone.Tints[0].G = 215;
		RightArmZone.Tints[0].A = 190;
	}
	else
	{
		RightArmZone.WidgetTexture = material'FOMIsc.ArmorGuy.ZoneRightArm';
		RightArmZone.Tints[0].R = 200;
		RightArmZone.Tints[0].G = 0;
		RightArmZone.Tints[0].B = 0;
		RightArmZone.Tints[0].A = FMin(180.0, (200.0 * FClamp(RightArmDamage/50.0, 0.0, 1.0)));
	}
	RightArmZone.Tints[1] = RightArmZone.Tints[0];

	// update left leg zone
	DamageLimit = FClamp(LeftLegDamage/50.0, 0.0, 1.0);
	if (DamageLimit > 0.75)
	{
		LeftLegZone.WidgetTexture = material'FOMIsc.ArmorGuy.LeftLegFlash_fb';
		LeftLegZone.Tints[0].R = 215;
		LeftLegZone.Tints[0].B = 215;
		LeftLegZone.Tints[0].G = 215;
		LeftLegZone.Tints[0].A = 190;
	}
	else
	{
		LeftLegZone.WidgetTexture = material'FOMIsc.ArmorGuy.ZoneLeftLeg';
		LeftLegZone.Tints[0].R = 200;
		LeftLegZone.Tints[0].G = 0;
		LeftLegZone.Tints[0].B = 0;
		LeftLegZone.Tints[0].A = FMin(180.0, (200.0 * FClamp(LeftLegDamage/50.0, 0.0, 1.0)));
	}
	LeftLegZone.Tints[1] = LeftLegZone.Tints[0];

	DamageLimit = FClamp(RightLegDamage/50.0, 0.0, 1.0);
	if (DamageLimit > 0.75)
	{
		RightLegZone.WidgetTexture = material'FOMIsc.ArmorGuy.RightLegFlash_fb';
		RightLegZone.Tints[0].R = 215;
		RightLegZone.Tints[0].B = 215;
		RightLegZone.Tints[0].G = 215;
		RightLegZone.Tints[0].A = 190;
	}
	else
	{
		// update right leg zone
		RightLegZone.WidgetTexture = material'FOMIsc.ArmorGuy.ZoneRightLeg';
		RightLegZone.Tints[0].R = 200;
		RightLegZone.Tints[0].G = 0;
		RightLegZone.Tints[0].B = 0;
		RightLegZone.Tints[0].A = FMin(180.0, (200.0 * FClamp(RightLegDamage/50.0, 0.0, 1.0)));
	}
	RightLegZone.Tints[1] = RightLegZone.Tints[0];
}

function CalculateHealth()
{
	if (FOPawn(PlayerOwner.Pawn) != None)
		CurHealth = PawnOwner.Health;
	else if (Pawn(PlayerOwner.ViewTarget) != None)
		CurHealth = Pawn(PlayerOwner.ViewTarget).Health;
		
}

function CalculateAmmo()
{
	local int LastAmmoPrimary;

	LastAmmoPrimary = CurAmmoPrimary;

	MaxAmmoPrimary = 1;
	CurAmmoPrimary = 1;
	ClipAmount = 1;

	if ((PawnOwner != None) && (PawnOwner.Weapon != None))
	{
		if (Weapon_Base(PawnOwner.Weapon).bIsReloading && !Weapon_Base(PawnOwner.Weapon).bHasSpecialReload)
			CurAmmoPrimary = LastAmmoPrimary;
		else
			PawnOwner.Weapon.GetAmmoCount(MaxAmmoPrimary,CurAmmoPrimary);
		Weapon_Base(PawnOwner.Weapon).GetClipCount(ClipAmount);
	}
	else if (Pawn(PlayerOwner.ViewTarget) != None)
	{
		if (FOPlayer(PlayerOwner).bTargetReloading && SpectatorWeapon != None && !SpectatorWeapon.Master.Default.bHasSpecialReload)
			CurAmmoPrimary = LastAmmoPrimary;
		else
			CurAmmoPrimary = FOPlayer(PlayerOwner).TargetAmmo;
		ClipAmount = FOPlayer(PlayerOwner).TargetClips;
	}
}

function int GetMinutes( int Seconds )
{
	local int Minutes;
	local int Time;

	Minutes = Seconds / 60;

	if (Minutes < 0)
		Time = 0;
	else
		Time = Minutes;

	return Time;
}

function int GetSeconds( int Seconds )
{
	local int Minutes;
	local int Time;

	Minutes = Seconds / 60;
	Seconds -= Minutes * 60;

	if (Seconds < 0)
		Time = 0;
	else
		Time = Seconds;

	return Time;
}

function string GetAmmoName(class<WeaponFire> TempFMClass)
{
	local class<Ammunition> TempABClass;

	TempABClass = TempFMClass.Default.AmmoClass;
	return TempABClass.Default.ItemName;
}

function string GetScoreValue(PlayerReplicationInfo PRI)
{
	return ""$int(PRI.Score);
}

function string GetScoreTagLine()
{
	return InitialViewingString;
}

function Font GetSmallerFontFor(Canvas Canvas, int offset)
{
	local int i;

	for (i=0;i<8-offset;i++)
	{
		if (Default.FontScreenWidthMedium[i] <= Canvas.ClipX)
			return LoadFontStatic(i+offset);
	}
	return LoadFontStatic(8);
}

function Font GetSmallerAltFontFor(Canvas Canvas, int offset)
{
	local int i;

	for (i=0;i<8-offset;i++)
	{
		if (Default.FontScreenWidthMedium[i] <= Canvas.ClipX)
			return LoadAltFontStatic(i+offset);
	}
	return LoadAltFontStatic(8);
}

static function Font StaticGetSmallerAltFontFor(Canvas Canvas, int offset)
{
	local int i;

	for (i=0;i<8-offset;i++)
	{
		if (Default.FontScreenWidthMedium[i] <= Canvas.ClipX)
			return LoadAltFontStatic(i+offset);
	}
	return LoadAltFontStatic(8);
}

static function Font GetSmallConsoleFont(Canvas C)
{
	local int FontSize;

	if (default.OverrideConsoleFontName != "")
	{
		if( default.OverrideConsoleFont != None )
			return default.OverrideConsoleFont;
		default.OverrideConsoleFont = Font(DynamicLoadObject(default.OverrideConsoleFontName, class'Font'));
		if( default.OverrideConsoleFont != None )
			return default.OverrideConsoleFont;
		Log("Warning: HUD couldn't dynamically load font "$default.OverrideConsoleFontName);
		default.OverrideConsoleFontName = "";
	}

	FontSize = Default.ConsoleFontSize+1;
	if (C.ClipX < 640)
		FontSize++;
	if (C.ClipX < 800)
		FontSize++;
	if (C.ClipX < 1024)
		FontSize++;
	if (C.ClipX < 1280)
		FontSize++;
	if (C.ClipX < 1600)
		FontSize++;
	return LoadFontStatic(Min(8,FontSize));
}

//=============================================================================
// misc hud functions

function StartObjective(float ObjStartTime)
{
	bSecondStage = false;
	bThirdStage = false;
	bShowProgressBar = true;
	ObjectiveStart = Level.TimeSeconds;
	ObjectiveTime = ObjStartTime;
	WiperStart = ObjectiveStart;		
	bForceProgressStop = false;
}

function StopObjective()
{
	bForceProgressStop = true;
	bShowProgressBar = false;
	bFlashBar = false;
}

function DoFlash(float NewFlashSize, vector FlashLocation)
{
	local rotator ViewPath, RotationPath;

	// check if flash is offscreen
	ViewPath = Normalize(rotator(FlashLocation-PawnOwner.Location));
	RotationPath = Normalize(PawnOwner.GetViewRotation());
	if ((ViewPath.Yaw ClockWiseFrom (RotationPath.Yaw+5000) && !(ViewPath.Yaw ClockWiseFrom (RotationPath.Yaw-5000))) || (ViewPath.Pitch ClockWiseFrom (RotationPath.Pitch+5000) && !(ViewPath.Pitch ClockWiseFrom (RotationPath.Pitch-5000))))
		NewFlashSize = 0.75*NewFlashSize;
	else if ((ViewPath.Yaw ClockWiseFrom (RotationPath.Yaw+8000) && !(ViewPath.Yaw ClockWiseFrom (RotationPath.Yaw-8000))) || (ViewPath.Pitch ClockWiseFrom (RotationPath.Pitch+8000) && !(ViewPath.Pitch ClockWiseFrom (RotationPath.Pitch-8000))))
		NewFlashSize = 0.65*NewFlashSize;

	bFlashed = true;
	FlashAmount = (FMin(1.0,FlashAmount + NewFlashSize));
	InitialFlashAmount = FlashAmount;
	InitialFlashTime = Level.TimeSeconds;
}

function OpenTimedBriefing()
{
	if (BriefingScreen == None)
		return;

	bShowBriefing = true;
	BriefingScreen.bOpenTimed = true;
	BriefingScreen.SetTimer(9.0, false);
}


function Tick(float dt)
{
	Super.Tick(dt);
	if (bFlashed && Level.TimeSeconds - InitialFlashTime >= 3.6)
		// not deltatime friendly right now
		FlashAmount = FlashAmount - (dt*0.12);
}

function Destroyed()
{
	Super.Destroyed();
	if (SpectatorWeapon != None)
		SpectatorWeapon.Destroy();
	if (SpectatorWeaponAkimbo != None)
		SpectatorWeaponAkimbo.Destroy();
}

defaultproperties
{
	// roundstats/briefing classes
	RoundStatsClass=class'Interface_RoundStats'
	BriefingClass=class'Interface_BriefingScreen'

	// colors
	TranslucentColor=(R=255,G=255,B=255,A=130)
	TransparentColor=(R=255,G=255,B=255,A=0)
	WhiteColor=(R=255,G=255,B=255,A=255)
	RedColor=(R=255,G=0,B=0,A=255)
	BlueColor=(R=0,G=0,B=255,A=255)
	GreenColor=(R=145,G=242,B=159,A=255)
	GoldColor=(R=255,G=255,B=90,A=255)
	TurqColor=(R=0,G=128,B=255,A=255)
	GrayColor=(R=190,G=190,B=190,A=245)
	CyanColor=(R=0,G=255,B=255,A=255)
	PurpleColor=(R=255,G=0,B=255,A=255)

	Crosshairs[0]=(WidgetTexture=Material'Crosshairs.Hud.Crosshair_Bracket1',TextureCoords=(X1=0,Y1=0,X2=64,Y2=64),TextureScale=0.6,DrawPivot=DP_MiddleMiddle,PosX=0.5,PosY=0.5,OffsetX=0,OffsetY=0,ScaleMode=SM_None,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=255,A=255),Tints[1]=(R=255,G=255,B=255,A=255))
	Crosshairs[1]=(WidgetTexture=Material'Crosshairs.Hud.Crosshair_Bracket2',TextureCoords=(X1=0,Y1=0,X2=64,Y2=64),TextureScale=0.6,DrawPivot=DP_MiddleMiddle,PosX=0.5,PosY=0.5,OffsetX=0,OffsetY=0,ScaleMode=SM_None,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=255,A=255),Tints[1]=(R=255,G=255,B=255,A=255))
	Crosshairs[2]=(WidgetTexture=Material'Crosshairs.Hud.Crosshair_Circle1',TextureCoords=(X1=0,Y1=0,X2=64,Y2=64),TextureScale=0.4,DrawPivot=DP_MiddleMiddle,PosX=0.5,PosY=0.5,OffsetX=0,OffsetY=0,ScaleMode=SM_None,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=255,A=255),Tints[1]=(R=255,G=255,B=255,A=255))
	Crosshairs[3]=(WidgetTexture=Material'Crosshairs.Hud.Crosshair_Circle2',TextureCoords=(X1=0,Y1=0,X2=64,Y2=64),TextureScale=0.4,DrawPivot=DP_MiddleMiddle,PosX=0.5,PosY=0.5,OffsetX=0,OffsetY=0,ScaleMode=SM_None,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=255,A=255),Tints[1]=(R=255,G=255,B=255,A=255))
	Crosshairs[4]=(WidgetTexture=Material'ONSInterface-TX.MineLayerReticle',TextureCoords=(X1=0,Y1=0,X2=64,Y2=64),TextureScale=0.4,DrawPivot=DP_MiddleMiddle,PosX=0.5,PosY=0.5,OffsetX=0,OffsetY=0,ScaleMode=SM_None,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=255,A=255),Tints[1]=(R=255,G=255,B=255,A=255))
	Crosshairs[5]=(WidgetTexture=Material'ONSInterface-TX.grenadeLauncherReticle',TextureCoords=(X1=0,Y1=0,X2=64,Y2=64),TextureScale=0.4,DrawPivot=DP_MiddleMiddle,PosX=0.5,PosY=0.5,OffsetX=0,OffsetY=0,ScaleMode=SM_None,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=255,A=255),Tints[1]=(R=255,G=255,B=255,A=255))
	Crosshairs[6]=(WidgetTexture=Material'ONSInterface-TX.avrilRETICLEtrack',TextureCoords=(X1=0,Y1=0,X2=64,Y2=64),TextureScale=0.4,DrawPivot=DP_MiddleMiddle,PosX=0.5,PosY=0.5,OffsetX=0,OffsetY=0,ScaleMode=SM_None,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=255,A=255),Tints[1]=(R=255,G=255,B=255,A=255))
	Crosshairs[7]=(WidgetTexture=Material'Crosshairs.Hud.Crosshair_Cross1',TextureCoords=(X1=0,Y1=0,X2=64,Y2=64),TextureScale=0.75,DrawPivot=DP_MiddleMiddle,PosX=0.5,PosY=0.5,OffsetX=0,OffsetY=0,ScaleMode=SM_None,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=255,A=255),Tints[1]=(R=255,G=255,B=255,A=255))
	Crosshairs[8]=(WidgetTexture=Material'Crosshairs.Hud.Crosshair_Cross2',TextureCoords=(X1=0,Y1=0,X2=64,Y2=64),TextureScale=0.75,DrawPivot=DP_MiddleMiddle,PosX=0.5,PosY=0.5,OffsetX=0,OffsetY=0,ScaleMode=SM_None,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=255,A=255),Tints[1]=(R=255,G=255,B=255,A=255))
	Crosshairs[9]=(WidgetTexture=Material'Crosshairs.Hud.Crosshair_Cross3',TextureCoords=(X1=0,Y1=0,X2=64,Y2=64),TextureScale=0.75,DrawPivot=DP_MiddleMiddle,PosX=0.5,PosY=0.5,OffsetX=0,OffsetY=0,ScaleMode=SM_None,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=255,A=255),Tints[1]=(R=255,G=255,B=255,A=255))
	Crosshairs[10]=(WidgetTexture=Material'Crosshairs.Hud.Crosshair_Cross4',TextureCoords=(X1=0,Y1=0,X2=64,Y2=64),TextureScale=0.75,DrawPivot=DP_MiddleMiddle,PosX=0.5,PosY=0.5,OffsetX=0,OffsetY=0,ScaleMode=SM_None,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=255,A=255),Tints[1]=(R=255,G=255,B=255,A=255))
	Crosshairs[11]=(WidgetTexture=Material'Crosshairs.Hud.Crosshair_Cross5',TextureCoords=(X1=0,Y1=0,X2=64,Y2=64),TextureScale=0.75,DrawPivot=DP_MiddleMiddle,PosX=0.5,PosY=0.5,OffsetX=0,OffsetY=0,ScaleMode=SM_None,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=255,A=255),Tints[1]=(R=255,G=255,B=255,A=255))
	Crosshairs[12]=(WidgetTexture=Material'Crosshairs.Hud.Crosshair_Dot',TextureCoords=(X1=0,Y1=0,X2=64,Y2=64),TextureScale=0.75,DrawPivot=DP_MiddleMiddle,PosX=0.5,PosY=0.5,OffsetX=0,OffsetY=0,ScaleMode=SM_None,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=255,A=255),Tints[1]=(R=255,G=255,B=255,A=255))
	Crosshairs[13]=(WidgetTexture=Material'Crosshairs.Hud.Crosshair_Pointer',TextureCoords=(X1=0,Y1=0,X2=64,Y2=64),TextureScale=0.6,DrawPivot=DP_MiddleMiddle,PosX=0.5,PosY=0.5,OffsetX=0,OffsetY=0,ScaleMode=SM_None,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=255,A=255),Tints[1]=(R=255,G=255,B=255,A=255))
	Crosshairs[14]=(WidgetTexture=Material'Crosshairs.Hud.Crosshair_Triad1',TextureCoords=(X1=0,Y1=0,X2=64,Y2=64),TextureScale=0.7,DrawPivot=DP_MiddleMiddle,PosX=0.5,PosY=0.5,OffsetX=0,OffsetY=0,ScaleMode=SM_None,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=255,A=255),Tints[1]=(R=255,G=255,B=255,A=255))
	Crosshairs[15]=(WidgetTexture=Material'Crosshairs.Hud.Crosshair_Triad2',TextureCoords=(X1=0,Y1=0,X2=64,Y2=64),TextureScale=0.7,DrawPivot=DP_MiddleMiddle,PosX=0.5,PosY=0.5,OffsetX=0,OffsetY=0,ScaleMode=SM_None,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=255,A=255),Tints[1]=(R=255,G=255,B=255,A=255))
	Crosshairs[16]=(WidgetTexture=Material'Crosshairs.Hud.Crosshair_Triad3',TextureCoords=(X1=0,Y1=0,X2=64,Y2=64),TextureScale=0.7,DrawPivot=DP_MiddleMiddle,PosX=0.5,PosY=0.5,OffsetX=0,OffsetY=0,ScaleMode=SM_None,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=255,A=255),Tints[1]=(R=255,G=255,B=255,A=255))

	// - digits
	DigitsBig=(DigitTexture=Material'FOMisc.HUD2k4.HUDDigitsBig',TextureCoords[0]=(X1=11,Y1=0,X2=53,Y2=64),TextureCoords[1]=(X1=75,Y1=0,X2=117,Y2=64),TextureCoords[2]=(X1=139,Y1=0,X2=181,Y2=64),TextureCoords[3]=(X1=203,Y1=0,X2=245,Y2=64),TextureCoords[4]=(X1=11,Y1=64,X2=53,Y2=128),TextureCoords[5]=(X1=75,Y1=64,X2=117,Y2=128),TextureCoords[6]=(X1=139,Y1=64,X2=181,Y2=128),TextureCoords[7]=(X1=203,Y1=64,X2=245,Y2=128),TextureCoords[8]=(X1=11,Y1=128,X2=53,Y2=192),TextureCoords[9]=(X1=75,Y1=128,X2=117,Y2=192),TextureCoords[10]=(X1=138,Y1=128,X2=181,Y2=192)))//(X1=100,Y1=400,X2=199,Y2=499)))

	// - poison label
	PoisonLabel=(WidgetTexture=Material'FOMisc.Special.PoisonLabel_fb',TextureCoords=(X1=0,Y1=0,X2=128,Y2=32),TextureScale=0.44,DrawPivot=DP_UpperRight,PosX=0.1035,PosY=0.626,OffsetX=0,OffsetY=0,ScaleMode=SM_None,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=240,G=240,B=240,A=255),Tints[1]=(R=240,G=240,B=240,A=255))

	// - laser
	LaserHUDTexture=(WidgetTexture=Material'FOMisc.Projected.LaserPointerAlpha',TextureCoords=(X1=0,Y1=0,X2=64,Y2=64),TextureScale=0.1,DrawPivot=DP_MiddleMiddle,PosX=0.5,PosY=0.5,OffsetX=0,OffsetY=0,ScaleMode=SM_None,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=30,B=30,A=180),Tints[1]=(R=255,G=30,B=30,A=180))

	// - grenade charge bar
	GrenadeCharge=(WidgetTexture=Material'FOMisc.HUD.MiscTextures_fb',TextureCoords=(X1=0,Y1=127,X2=15,Y2=127),TextureScale=0.535,DrawPivot=DP_LowerRight,PosX=0.12,PosY=0.850,OffsetX=0,OffsetY=0,ScaleMode=SM_None,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=200,G=200,B=200,A=250),Tints[1]=(R=200,G=200,B=200,A=250))
	GrenadeMaxIcon=(WidgetTexture=Material'FOMisc.Special.MaxFinal',TextureCoords=(X1=0,Y1=0,X2=64,Y2=32),TextureScale=0.5,DrawPivot=DP_LowerRight,PosX=0.14,PosY=0.71,OffsetX=-1,OffsetY=-2,ScaleMode=SM_None,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=255,A=250),Tints[1]=(R=255,G=255,B=255,A=250))

	// - weapon info
	WeaponBG=(WidgetTexture=Material'FOMisc.HUD2k4.HUDWeaponBG',TextureCoords=(X1=0,Y1=0,X2=256,Y2=64),TextureScale=0.65,DrawPivot=DP_LowerRight,PosX=1.0,PosY=1.0,OffsetX=0,OffsetY=0,ScaleMode=SM_Right,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=255,A=255),Tints[1]=(R=255,G=255,B=255,A=255)) // -- Bottom Right Clip Icon
	WeaponIcon=(WidgetTexture=Material'FOMisc.WeaponHUD.WeapHUD_Final',TextureCoords=(X1=0,Y1=0,X2=512,Y2=128),TextureScale=0.3,DrawPivot=DP_LowerRight,PosX=0.991,PosY=0.917,OffsetX=0,OffsetY=0,ScaleMode=SM_Right,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=240,G=240,B=240,A=240),Tints[1]=(R=240,G=240,B=240,A=240))
	AmmoIcon=(WidgetTexture=Material'FOMisc.HUD2k4.HUDBulletIcon',TextureCoords=(X1=0,Y1=0,X2=64,Y2=32),TextureScale=0.65,DrawPivot=DP_UpperRight,PosX=0.88,PosY=0.948,OffsetX=0,OffsetY=0,ScaleMode=SM_Right,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=255,A=255),Tints[1]=(R=255,G=255,B=255,A=255)) 
	AmmoCount=(MinDigitCount=2,TextureScale=0.255,DrawPivot=DP_LowerLeft,PosX=0.876,PosY=0.986,OffsetX=0,OffsetY=0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=255,A=255),Tints[1]=(R=255,G=255,B=255,A=255),bPadWithZeroes=1)
	ClipIcon=(WidgetTexture=Material'FOMisc.HUD2k4.HUDMagIcon',TextureCoords=(X1=0,Y1=0,X2=64,Y2=32),TextureScale=0.65,DrawPivot=DP_UpperRight,PosX=0.958,PosY=0.949,OffsetX=0,OffsetY=0,ScaleMode=SM_Right,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=255,A=255),Tints[1]=(R=255,G=255,B=255,A=255)) 
	ClipCount=(MinDigitCount=2,TextureScale=0.245,DrawPivot=DP_LowerLeft,PosX=0.952,PosY=0.986,OffsetX=0,OffsetY=0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=255,A=255),Tints[1]=(R=255,G=255,B=255,A=255),bPadWithZeroes=1)

	// - akimbo
	AmmoSecIcon=(WidgetTexture=Material'FOMisc.HUD2k4.HUDBulletIcon',TextureCoords=(X1=0,Y1=0,X2=64,Y2=32),TextureScale=0.65,DrawPivot=DP_UpperRight,PosX=0.799,PosY=0.948,OffsetX=0,OffsetY=0,ScaleMode=SM_Right,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=255,A=255),Tints[1]=(R=255,G=255,B=255,A=255)) 
	AmmoSecCount=(MinDigitCount=2,TextureScale=0.255,DrawPivot=DP_LowerLeft,PosX=0.795,PosY=0.986,OffsetX=0,OffsetY=0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=255,A=255),Tints[1]=(R=255,G=255,B=255,A=255),bPadWithZeroes=1)

	// - player info
	PlayerBG=(WidgetTexture=Material'FOMisc.HUD2k4.HUDPlayerBG',TextureCoords=(X1=0,Y1=0,X2=256,Y2=64),TextureScale=0.65,DrawPivot=DP_LowerLeft,PosX=0.0,PosY=1.0,OffsetX=0,OffsetY=0,ScaleMode=SM_Right,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=255,A=255),Tints[1]=(R=255,G=255,B=255,A=255)) // -- Bottom Right Clip Icon
	HealthCross=(WidgetTexture=Material'FOMisc.HUD2k4.HUDHealthCross',TextureCoords=(X1=0,Y1=0,X2=32,Y2=32),TextureScale=0.65,DrawPivot=DP_LowerLeft,PosX=0.02,PosY=0.98,OffsetX=0,OffsetY=0,ScaleMode=SM_Right,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=0,B=0,A=166),Tints[1]=(R=0,G=0,B=255,A=166)) // -- Bottom Right Clip Icon
	HealthCount=(MinDigitCount=3,TextureScale=0.47,DrawPivot=DP_LowerLeft,PosX=0.059,PosY=0.99,OffsetX=0,OffsetY=0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=255,A=255),Tints[1]=(R=255,G=255,B=255,A=255),bPadWithZeroes=1)

	// armor guy
	ArmorTypeBG=(WidgetTexture=Material'FOMisc.ArmorGuy.HUDArmorTypeBG',TextureCoords=(X1=0,Y1=0,X2=128,Y2=256),TextureScale=0.65,DrawPivot=DP_LowerLeft,PosX=0.013,PosY=0.962,OffsetX=0,OffsetY=0,ScaleMode=SM_Right,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=255,A=255),Tints[1]=(R=255,G=255,B=255,A=255))
	ArmorGuyBG=(WidgetTexture=Material'FOMisc.ArmorGuy.HUDArmorGuyBack',TextureCoords=(X1=0,Y1=0,X2=128,Y2=256),TextureScale=0.65,DrawPivot=DP_LowerLeft,PosX=0.013,PosY=0.962,OffsetX=0,OffsetY=0,ScaleMode=SM_Right,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=200,G=200,B=200,A=80),Tints[1]=(R=255,G=255,B=255,A=80))
	ArmorGuy=(WidgetTexture=Material'FOMisc.ArmorGuy.HUDArmorGuyFinal',TextureCoords=(X1=0,Y1=0,X2=128,Y2=256),TextureScale=0.65,DrawPivot=DP_LowerLeft,PosX=0.013,PosY=0.962,OffsetX=0,OffsetY=0,ScaleMode=SM_Right,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=210,G=210,B=210,A=210),Tints[1]=(R=210,G=210,B=210,A=210))
	ArmorGuyOutline=(WidgetTexture=Material'FOMisc.ArmorGuy.HUDArmorGuyOutline',TextureCoords=(X1=0,Y1=0,X2=128,Y2=256),TextureScale=0.65,DrawPivot=DP_LowerLeft,PosX=0.013,PosY=0.962,OffsetX=0,OffsetY=0,ScaleMode=SM_Right,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=255,A=255),Tints[1]=(R=255,G=255,B=255,A=255))
	ArmorType=(WidgetTexture=Material'FOMisc.ArmorGuy.HUDArmorLit',TextureCoords=(X1=0,Y1=0,X2=32,Y2=16),TextureScale=0.65,DrawPivot=DP_LowerLeft,PosX=0.095,PosY=0.892,OffsetX=0,OffsetY=0,ScaleMode=SM_Right,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=255,A=255),Tints[1]=(R=255,G=255,B=255,A=255))
	HeadZone=(WidgetTexture=Material'FOMisc.ArmorGuy.ZoneHead',TextureCoords=(X1=0,Y1=0,X2=128,Y2=256),TextureScale=0.65,DrawPivot=DP_LowerLeft,PosX=0.013,PosY=0.962,OffsetX=0,OffsetY=0,ScaleMode=SM_Right,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=220,G=220,B=220,A=200),Tints[1]=(R=220,G=220,B=220,A=200))
	TorsoZone=(WidgetTexture=Material'FOMisc.ArmorGuy.ZoneChest',TextureCoords=(X1=0,Y1=0,X2=128,Y2=256),TextureScale=0.65,DrawPivot=DP_LowerLeft,PosX=0.013,PosY=0.962,OffsetX=0,OffsetY=0,ScaleMode=SM_Right,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=220,G=220,B=220,A=200),Tints[1]=(R=220,G=220,B=220,A=200))
	LeftArmZone=(WidgetTexture=Material'FOMisc.ArmorGuy.ZoneLeftArm',TextureCoords=(X1=0,Y1=0,X2=128,Y2=256),TextureScale=0.65,DrawPivot=DP_LowerLeft,PosX=0.013,PosY=0.962,OffsetX=0,OffsetY=0,ScaleMode=SM_Right,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=220,G=220,B=220,A=200),Tints[1]=(R=220,G=220,B=220,A=200))
	RightArmZone=(WidgetTexture=Material'FOMisc.ArmorGuy.ZoneRightArm',TextureCoords=(X1=0,Y1=0,X2=128,Y2=256),TextureScale=0.65,DrawPivot=DP_LowerLeft,PosX=0.013,PosY=0.962,OffsetX=0,OffsetY=0,ScaleMode=SM_Right,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=220,G=220,B=220,A=200),Tints[1]=(R=220,G=220,B=220,A=200))
	LeftLegZone=(WidgetTexture=Material'FOMisc.ArmorGuy.ZoneLeftLeg',TextureCoords=(X1=0,Y1=0,X2=128,Y2=256),TextureScale=0.65,DrawPivot=DP_LowerLeft,PosX=0.013,PosY=0.962,OffsetX=0,OffsetY=0,ScaleMode=SM_Right,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=220,G=220,B=220,A=200),Tints[1]=(R=220,G=220,B=220,A=200))
	RightLegZone=(WidgetTexture=Material'FOMisc.ArmorGuy.ZoneRightLeg',TextureCoords=(X1=0,Y1=0,X2=128,Y2=256),TextureScale=0.65,DrawPivot=DP_LowerLeft,PosX=0.013,PosY=0.962,OffsetX=0,OffsetY=0,ScaleMode=SM_Right,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=220,G=220,B=220,A=200),Tints[1]=(R=220,G=220,B=220,A=200))

	// - action system
	APBG=(WidgetTexture=Material'FOMisc.HUD2k4.HUDAPBG',TextureCoords=(X1=0,Y1=0,X2=128,Y2=64),TextureScale=0.65,DrawPivot=DP_LowerLeft,PosX=0.000,PosY=0.625,OffsetX=0,OffsetY=0,ScaleMode=SM_None,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=255,A=255),Tints[1]=(R=255,G=255,B=255,A=255))
	APIcon=(WidgetTexture=Material'FOMisc.HUD.HUDAPIcon',TextureCoords=(X1=0,Y1=0,X2=64,Y2=16),TextureScale=0.63,DrawPivot=DP_LowerLeft,PosX=0.010,PosY=0.6085,OffsetX=0,OffsetY=0,ScaleMode=SM_None,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=255,A=220),Tints[1]=(R=255,G=255,B=255,A=220))
	APCount=(MinDigitCount=1,TextureScale=0.19,DrawPivot=DP_UpperLeft,PosX=0.0421,PosY=0.5836,OffsetX=0,OffsetY=0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=255,A=255),Tints[1]=(R=255,G=255,B=255,A=255),bPadWithZeroes=1)
	APTotal=(MinDigitCount=1,TextureScale=0.19,DrawPivot=DP_UpperLeft,PosX=0.0692,PosY=0.5836,OffsetX=0,OffsetY=0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=255,A=255),Tints[1]=(R=255,G=255,B=255,A=255),bPadWithZeroes=1)
	StopWatchBG=(WidgetTexture=Material'FOMisc.HUD.StopWatchBG_fb',TextureCoords=(X1=0,Y1=0,X2=256,Y2=256),TextureScale=0.2,DrawPivot=DP_MiddleMiddle,PosX=0.048,PosY=0.5137,OffsetX=0,OffsetY=0,ScaleMode=SM_None,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=255,A=255),Tints[1]=(R=255,G=255,B=255,A=255))
	StopWatchHand=(WidgetTexture=Material'FOMisc.HUD.StopWatch_rot',TextureCoords=(X1=0,Y1=0,X2=256,Y2=256),TextureScale=0.2,DrawPivot=DP_MiddleMiddle,PosX=0.048,PosY=0.5137,OffsetX=0,OffsetY=0,ScaleMode=SM_None,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=255,A=200),Tints[1]=(R=255,G=255,B=255,A=210))
	StopWatchHandSmall=(WidgetTexture=Material'FOMisc.HUD.StopWatchSmall_rot',TextureCoords=(X1=0,Y1=0,X2=64,Y2=64),TextureScale=0.17,DrawPivot=DP_MiddleMiddle,PosX=0.048,PosY=0.5015,OffsetX=0,OffsetY=-7,ScaleMode=SM_None,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=255,A=200),Tints[1]=(R=255,G=255,B=255,A=210))

	// upper left
	MoneyBG=(WidgetTexture=Material'FOMisc.HUD2k4.HUDMoneyBG',TextureCoords=(X1=0,Y1=0,X2=256,Y2=64),TextureScale=0.65,DrawPivot=DP_UpperLeft,PosX=0,PosY=0,OffsetX=0,OffsetY=0,ScaleMode=SM_Right,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=255,A=255),Tints[1]=(R=255,G=255,B=255,A=255))
	MoneyIcon=(WidgetTexture=Material'FOMisc.HUD2k4.HUDMoneySign',TextureCoords=(X1=0,Y1=0,X2=32,Y2=32),TextureScale=0.65,DrawPivot=DP_UpperLeft,PosX=0.003,PosY=0.007,OffsetX=3,OffsetY=1,ScaleMode=SM_Right,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=0,A=166),Tints[1]=(R=255,G=255,B=0,A=166))
	MoneyCount=(MinDigitCount=4,TextureScale=0.265,DrawPivot=DP_UpperLeft,PosX=0.036,PosY=0.012,OffsetX=0,OffsetY=0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=255,A=255),Tints[1]=(R=255,G=255,B=255,A=255),bPadWithZeroes=1)

	// - armor (vehicle version)
	ArmorGuyBGSmall=(WidgetTexture=Material'FOMisc.ArmorGuy.HUDArmorGuyBack',TextureCoords=(X1=0,Y1=0,X2=128,Y2=256),TextureScale=0.27,DrawPivot=DP_UpperRight,PosX=0.08,PosY=0.57,OffsetX=0,OffsetY=0,ScaleMode=SM_Right,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=230,G=230,B=230,A=220),Tints[1]=(R=230,G=230,B=230,A=220))
	HeadZoneSmall=(WidgetTexture=Material'FOMisc.ArmorGuy.ZoneHead',TextureCoords=(X1=0,Y1=0,X2=128,Y2=256),TextureScale=0.27,DrawPivot=DP_UpperRight,PosX=0.08,PosY=0.57,OffsetX=0,OffsetY=0,ScaleMode=SM_Right,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=220,G=220,B=220,A=200),Tints[1]=(R=220,G=220,B=220,A=200))
	TorsoZoneSmall=(WidgetTexture=Material'FOMisc.ArmorGuy.ZoneChest',TextureCoords=(X1=0,Y1=0,X2=128,Y2=256),TextureScale=0.27,DrawPivot=DP_UpperRight,PosX=0.08,PosY=0.57,OffsetX=0,OffsetY=0,ScaleMode=SM_Right,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=220,G=220,B=220,A=200),Tints[1]=(R=220,G=220,B=220,A=200))
	LeftArmZoneSmall=(WidgetTexture=Material'FOMisc.ArmorGuy.ZoneLeftArm',TextureCoords=(X1=0,Y1=0,X2=128,Y2=256),TextureScale=0.27,DrawPivot=DP_UpperRight,PosX=0.08,PosY=0.57,OffsetX=0,OffsetY=0,ScaleMode=SM_Right,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=220,G=220,B=220,A=200),Tints[1]=(R=220,G=220,B=220,A=200))
	RightArmZoneSmall=(WidgetTexture=Material'FOMisc.ArmorGuy.ZoneRightArm',TextureCoords=(X1=0,Y1=0,X2=128,Y2=256),TextureScale=0.27,DrawPivot=DP_UpperRight,PosX=0.08,PosY=0.57,OffsetX=0,OffsetY=0,ScaleMode=SM_Right,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=220,G=220,B=220,A=200),Tints[1]=(R=220,G=220,B=220,A=200))
	LeftLegZoneSmall=(WidgetTexture=Material'FOMisc.ArmorGuy.ZoneLeftLeg',TextureCoords=(X1=0,Y1=0,X2=128,Y2=256),TextureScale=0.27,DrawPivot=DP_UpperRight,PosX=0.08,PosY=0.57,OffsetX=0,OffsetY=0,ScaleMode=SM_Right,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=220,G=220,B=220,A=200),Tints[1]=(R=220,G=220,B=220,A=200))
	RightLegZoneSmall=(WidgetTexture=Material'FOMisc.ArmorGuy.ZoneRightLeg',TextureCoords=(X1=0,Y1=0,X2=128,Y2=256),TextureScale=0.27,DrawPivot=DP_UpperRight,PosX=0.08,PosY=0.57,OffsetX=0,OffsetY=0,ScaleMode=SM_Right,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=220,G=220,B=220,A=200),Tints[1]=(R=220,G=220,B=220,A=200))

	// - map and round time
	TimerBG=(WidgetTexture=Material'FOMisc.HUD2k4.HUDTimerBG',TextureCoords=(X1=0,Y1=0,X2=256,Y2=64),TextureScale=0.65,DrawPivot=DP_UpperMiddle,PosX=0.5,PosY=0,OffsetX=0,OffsetY=-12,ScaleMode=SM_Right,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=255,A=255),Tints[1]=(R=255,G=255,B=255,A=255)) // -- Timers box 
	MapMinutes=(MinDigitCount=2,TextureScale=0.21,DrawPivot=DP_UpperRight,PosX=0.493,PosY=0,OffsetX=-110,OffsetY=30,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=255,A=255),Tints[1]=(R=255,G=255,B=255,A=255),bPadWithZeroes=1)
	MapSeconds=(MinDigitCount=2,TextureScale=0.21,DrawPivot=DP_UpperRight,PosX=0.493,PosY=0,OffsetX=0,OffsetY=30,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=255,A=255),Tints[1]=(R=255,G=255,B=255,A=255),bPadWithZeroes=1)
	TimeSeperator[0]=(WidgetTexture=Material'FOMisc.HUD2k4.HUDDigitsBig',TextureCoords=(X1=203,Y1=128,X2=245,Y2=192),TextureScale=0.21,DrawPivot=DP_UpperMiddle,PosX=0.461,PosY=0,OffsetX=0,OffsetY=30,ScaleMode=SM_Right,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=255,A=255),Tints[1]=(R=255,G=255,B=255,A=255)) // -- Timers semicolon
	RoundMinutes=(MinDigitCount=2,TextureScale=0.21,DrawPivot=DP_UpperLeft,PosX=0.507,PosY=0,OffsetX=0,OffsetY=30,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=255,A=255),Tints[1]=(R=255,G=255,B=255,A=255),bPadWithZeroes=1)
	RoundSeconds=(MinDigitCount=2,TextureScale=0.21,DrawPivot=DP_UpperLeft,PosX=0.507,PosY=0,OffsetX=110,OffsetY=30,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=255,A=255),Tints[1]=(R=255,G=255,B=255,A=255),bPadWithZeroes=1)
	TimeSeperator[1]=(WidgetTexture=Material'FOMisc.HUD2k4.HUDDigitsBig',TextureCoords=(X1=203,Y1=128,X2=245,Y2=192),TextureScale=0.21,DrawPivot=DP_UpperMiddle,PosX=0.539,PosY=0,OffsetX=0,OffsetY=30,ScaleMode=SM_Right,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=255,A=255),Tints[1]=(R=255,G=255,B=255,A=255)) // -- Timers semicolon

	// - LAW stuff
	LAWZoomTex=(WidgetTexture=Material'FOMisc.HUD.LAWZoom_fb',TextureCoords=(X1=0,Y1=0,X2=256,Y2=256),TextureScale=1.0,DrawPivot=DP_MiddleMiddle,PosX=0.5,PosY=0.5,OffsetX=0,OffsetY=0,ScaleMode=SM_None,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=255,A=245),Tints[1]=(R=255,G=255,B=255,A=245))
	LAWCenterTex=(WidgetTexture=Material'FOMisc.HUD.LAWCenter_fb',TextureCoords=(X1=0,Y1=0,X2=128,Y2=128),TextureScale=1.0,DrawPivot=DP_MiddleMiddle,PosX=0.5,PosY=0.5,OffsetX=0,OffsetY=0,ScaleMode=SM_None,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=255,A=245),Tints[1]=(R=255,G=255,B=255,A=245))
	LAWCenterIdleTex=(WidgetTexture=Material'FOMisc.HUD.LAWCenterLock',TextureCoords=(X1=0,Y1=0,X2=128,Y2=128),TextureScale=1.0,DrawPivot=DP_MiddleMiddle,PosX=0.5,PosY=0.5,OffsetX=0,OffsetY=0,ScaleMode=SM_None,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=5,B=5,A=245),Tints[1]=(R=255,G=5,B=5,A=255))
	LAWPointerTex=(WidgetTexture=Material'FOMisc.HUD.LAWPointer',TextureCoords=(X1=0,Y1=0,X2=128,Y2=128),TextureScale=1.0,DrawPivot=DP_MiddleMiddle,PosX=0.5,PosY=0.5,OffsetX=0,OffsetY=0,ScaleMode=SM_None,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=255,A=30),Tints[1]=(R=255,G=255,B=255,A=30))
	LAWAimerTex=(WidgetTexture=Material'FOMisc.HUD.LAWAimer',TextureCoords=(X1=0,Y1=0,X2=128,Y2=128),TextureScale=1.0,DrawPivot=DP_MiddleMiddle,PosX=0.5,PosY=0.5,OffsetX=0,OffsetY=0,ScaleMode=SM_None,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=10,G=10,B=10,A=245),Tints[1]=(R=10,G=10,B=10,A=245))

	// - interactive mission timer
	ProgressIcon=(WidgetTexture=Material'FOMisc.Special.ProgressCircle',TextureCoords=(X1=0,Y1=0,X2=128,Y2=128),TextureScale=1.0,DrawPivot=DP_MiddleMiddle,PosX=0.5,PosY=0.7,OffsetX=0,OffsetY=0,ScaleMode=SM_None,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=255,A=220),Tints[1]=(R=255,G=255,B=255,A=220))
	ProgressFlashIcon=(WidgetTexture=Material'FOMisc.Special.ProgressFlash',TextureCoords=(X1=0,Y1=0,X2=128,Y2=128),TextureScale=1.0,DrawPivot=DP_MiddleMiddle,PosX=0.5,PosY=0.7,OffsetX=0,OffsetY=0,ScaleMode=SM_None,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=255,A=220),Tints[1]=(R=255,G=255,B=255,A=220))
	ProgressWiper=(WidgetTexture=Material'FOMisc.Special.WiperRot',TextureCoords=(X1=16,Y1=16,X2=112,Y2=112),TextureScale=1.0,DrawPivot=DP_MiddleMiddle,PosX=0.5,PosY=0.7,OffsetX=0,OffsetY=0,ScaleMode=SM_None,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=100,G=100,B=100,A=255),Tints[1]=(R=100,G=100,B=100,A=255))
	ProgressWiperBorder=(WidgetTexture=Material'FOMisc.Special.ProgressWiperBorder',TextureCoords=(X1=0,Y1=0,X2=128,Y2=128),TextureScale=1.0,DrawPivot=DP_MiddleMiddle,PosX=0.5,PosY=0.7,OffsetX=0,OffsetY=0,ScaleMode=SM_None,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=100,G=100,B=100,A=255),Tints[1]=(R=100,G=100,B=100,A=255))
	ProgressCountNum=(MinDigitCount=3,TextureScale=0.25,DrawPivot=DP_MiddleMiddle,PosX=0.497,PosY=0.7,OffsetX=0,OffsetY=0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=255,A=255),Tints[1]=(R=255,G=255,B=255,A=255),bPadWithZeroes=1)

	// level action settings
	LevelActionLoading="LOADING..."
	LevelActionPaused="PAUSED"
	LevelActionFontName="UT2003Fonts.FontMedium"
	LevelActionFontColor=(R=255,G=255,B=255,A=255)

	// level action screenpos
	LevelActionPositionX=0.5
	LevelActionPositionY=0.25

	// console message screenpos
	ConsoleMessagePosX=0.011
	ConsoleMessagePosY=0.09

	// kill message screenpos
	KillMessagePosX=0.595
	KillMessagePosY=0.023

	// switched weapon screenpos
	CurrentWeaponPositionX=0.845
	CurrentWeaponPositionY=0.9

	// fonts
	FontArrayNames(0)="FOFonts.Gunplay45"
	FontArrayNames(1)="FOFonts.Gunplay40"
	FontArrayNames(2)="FOFonts.Gunplay35"
	FontArrayNames(3)="FOFonts.Gunplay29"
	FontArrayNames(4)="FOFonts.Gunplay24"
	FontArrayNames(5)="FOFonts.Gunplay20"
	FontArrayNames(6)="FOFonts.Gunplay15"
	FontArrayNames(7)="FOFonts.Gunplay12"
	FontArrayNames(8)="UT2003Fonts.FontNeuzeit8"

	FontAltNames(0)="FOFonts.LcdD26"
	FontAltNames(1)="FOFonts.LcdD24"
	FontAltNames(2)="FOFonts.LcdD22"
	FontAltNames(3)="FOFonts.LcdD20"
	FontAltNames(4)="FOFonts.LcdD18"
	FontAltNames(5)="FOFonts.LcdD16"
	FontAltNames(6)="FOFonts.LcdD14"
	FontAltNames(7)="UT2003Fonts.FontEurostile9"
	FontAltNames(8)="UT2003Fonts.FontSmallText"		//6

	// infolines
	WonMatchPostFix=" won the match!"
	YouveWonTheMatch="You've won the match!"
	YouveLostTheMatch="You've lost the match."
	InitialViewingString="Press Fire to View a different Player"
	NowViewing="Now viewing"
	Scoretext="Score"

	// spectator extras
	ReloadingIcon=Material'FOMisc.Special.Reloading_fb'
	
	// bools
	bShowMoney=true
	bShowArmorGuy=true
	bShowWeaponBar=true

	bUseCustomWeaponCrosshairs=false
}