//=============================================================================
// Frag.Ops  2004 Matt 'SquirrelZero' Farber
//=============================================================================
class Interface_MainMenu extends UT2k4GUIPage;

var automated BackgroundImage i_Background, i_BGSec;
var automated GUIImage SlideBottom, SlideTop, MenuLogo;
var automated GUIButton b_MultiPlayer, b_Host, b_InstantAction, b_ModsAndDemo, b_Settings, b_Quit;
var automated moEditBox PlayerName;
var automated GUILabel NameLabel, VersionLabel;

var bool bAllowClose;

var GUIButton Selected;
var() bool bNoInitDelay;
var() config string MenuSong;
var() string sName, sNameD;

function InitComponent(GUIController MyController, GUIComponent MyOwner)
{
	Super.InitComponent(MyController, MyOwner);

	PlayerName.MyEditBox.bConvertSpaces = true;
	PlayerName.MyEditBox.MaxWidth = 22;
	
	Background = MyController.DefaultPens[0];
	if (PlayerOwner().Level.IsDemoBuild())
		b_MultiPlayer.SetFocus(none);

	if (class'GameEngine'.Default.ConnectingMenuClass != "FragOps.Interface_ConnectingMenu")
	{
		class'GameEngine'.Default.ConnectingMenuClass = "FragOps.Interface_ConnectingMenu";
		class'GameEngine'.static.StaticSaveConfig();
	}
}

function InternalOnLoadINI(GUIComponent Sender, string s)
{
	local PlayerController PC;

	PC = PlayerOwner();
	if (GUIMenuOption(Sender) != None)
	{
		switch (GUIMenuOption(Sender))
		{
			case PlayerName:
				sName = PC.GetUrlOption("Name");
				sNameD = sName;
				PlayerName.SetText(sName);
				break;
		}
	}
}

function InternalOnChange(GUIComponent Sender)
{
	if (GUIMenuOption(Sender) != None)
	{
		switch (GUIMenuOption(Sender))
		{
			case PlayerName:
				sName = PlayerName.GetText();
				break;
		}
	}
}

function SaveSettings()
{
	local PlayerController PC;

	PC = PlayerOwner();
	if (sNameD != sName)
	{
		PC.ReplaceText(sName, "\"", "");
		sNameD = sName;
		PC.ConsoleCommand("SetName"@sName);
	}
}

event Timer()
{
	bNoInitDelay = true;
	if (!Controller.bQuietMenu)
		PlayerOwner().PlaySound(SlideInSound,SLOT_None);
	MenuLoaded();
}

function MenuLoaded()
{
	if (!Controller.bQuietMenu)
		PlayerOwner().PlaySound(FadeInSound);

	NameLabel.Animate(0.055,0.27,0.42);
	PlayerName.Animate(0.055,0.315,0.42);

	b_Multiplayer.Animate(0.055,0.38,0.3);
	b_Multiplayer.OnArrival = PlayPopSound;

	b_Host.Animate(0.055,0.43,0.32);
	b_Host.OnArrival = PlayPopSound;

	b_InstantAction.Animate(0.055,0.48,0.34);
	b_InstantAction.OnArrival = PlayPopSound;

	b_ModsAndDemo.Animate(0.055,0.53,0.36);
	b_ModsAndDemo.OnArrival = PlayPopSound;

	b_Settings.Animate(0.055,0.58,0.38);
	b_Settings.OnArrival = PlayPopSound;

	b_Quit.Animate(0.055,0.65,0.40);
	b_Quit.OnArrival = MenuIn_Done;

	SlideBottom.Animate(0,0.75,0.25);
	SlideTop.Animate(0,0,0.25);
	
	MenuLogo.Animate(0,0,0.4);

	VersionLabel.Animate(0.006,0.954,0.9);
}

event Opened(GUIComponent Sender)
{
	if (bDebugging)
		log(Name$".Opened()   Sender:"$Sender,'Debug');

	if (Sender != None && PlayerOwner().Level.IsPendingConnection())
	    	PlayerOwner().ConsoleCommand("CANCEL");

	Super.Opened(Sender);
	Selected = none;

	// Reset the animations of all components
	SlideBottom.Animate(1.0,0.75,0);
	SlideTop.Animate(-1.0,0,0);
	MenuLogo.Animate(1,0,0);
	NameLabel.Animate(1.3,0.967567,0);
	PlayerName.Animate(1.3,1.027567,0);
	VersionLabel.Animate(0.006,1.4,0);
	b_Multiplayer.Animate(1.3,0.449282,0);
	b_Host.Animate(1.3,0.534027,0);
	b_InstantAction.Animate(1.3,0.618619,0);
	b_ModsAndDemo.Animate(1.3,0.705859,0);
	b_Settings.Animate(1.3,0.800327,0);
	b_Quit.Animate(1.3,0.887567,0);
}

function MenuIn_Done(GUIComponent Sender, EAnimationType Type)
{
	Sender.OnArrival = none;
	PlayPopSound(Sender,Type);
}

function PlayPopSound(GUIComponent Sender, EAnimationType Type)
{
	if (!Controller.bQuietMenu)
		PlayerOwner().PlaySound(PopInSound);
}

function MainReopened()
{
	if (!PlayerOwner().Level.IsPendingConnection())
	{
		Opened(none);
		Timer();
	}
}

function InternalOnOpen()
{
	if (bNoInitDelay)
		Timer();
	else
		SetTimer(0.5,false);

	Controller.PerformRestore();
	PlayerOwner().ClientSetInitialMusic(MenuSong,MTRAN_Segue);
}

function OnClose(optional Bool bCancelled)
{
}

function bool MyKeyEvent(out byte Key,out byte State,float delta)
{
	if (Key == 0x1B && state == 1)	// Escape pressed
		bAllowClose = true;

	return false;
}

function bool CanClose(optional bool bCancelled)
{
	if (bAllowClose)
		ButtonClick(b_Quit);

	bAllowClose = False;
	return PlayerOwner().Level.IsPendingConnection();
}

function MoveOn()
{
	switch (Selected)
	{
		case b_MultiPlayer:
			Profile("ServerBrowser");
			Controller.OpenMenu(Controller.GetServerBrowserPage());
			Profile("ServerBrowser");
			return;

		case b_Host:
			Profile("MPHost");
			Controller.OpenMenu(Controller.GetMultiplayerPage());
			Profile("MPHost");
			return;

		case b_InstantAction:
			Profile("InstantAction");
			Controller.OpenMenu(Controller.GetInstantActionPage());
			Profile("InstantAction");
			return;

		case b_ModsAndDemo:
			Profile("ModsandDemos");
			Controller.OpenMenu(Controller.GetModPage());
			Profile("ModsandDemos");
			return;

		case b_Settings:
			Profile("Settings");
			Controller.OpenMenu(Controller.GetSettingsPage());
			Profile("Settings");
			return;

		case b_Quit:
			Profile("Quit");
			Controller.OpenMenu(Controller.GetQuitPage());
			Profile("Quit");
			return;

		default:
			StopWatch(True);
	}
}

function bool ButtonClick(GUIComponent Sender)
{
	if (GUIButton(Sender) != None)
		Selected = GUIButton(Sender);

	if (Selected == None)
	    	return false;

	InitAnimOut(b_Multiplayer,1.15,0.449282,0.3);
	InitAnimOut(b_Host,1.3,0.534027,0.35);
	InitAnimOut(b_InstantAction,1.45,0.618619,0.3);
	InitAnimOut(b_ModsAndDemo,1.6,0.705859,0.3);
	InitAnimOut(b_Settings,1.75,0.800327,0.3);
	InitAnimOut(b_Quit,1.9,0.887567,0.3);
	InitAnimOut(SlideBottom,1.0,0.75,0.36);
	InitAnimOut(SlideTop,-1.0,0,0.36);
	InitAnimOut(MenuLogo,1.0,0,0.36);
	InitAnimOut(PlayerName,1.3,1.0267567,0.3);
	InitAnimOut(NameLabel,1.3,0.967567,0.3);
	InitAnimOut(VersionLabel,0.006,1.4,0.3);

	// save name if needed
	SaveSettings();

	return true;
}

function InitAnimOut(GUIComponent C, float X, float Y, float Z)
{
	if (C == None)
	{
		Warn("UT2K4MainMenu.InitAnimOut called with null component!");
		return;
	}

	C.Animate(X,Y,Z);
	C.OnEndAnimation = MenuOut_Done;
}

function MenuOut_Done(GUIComponent Sender, EAnimationType Type)
{
	Sender.OnArrival = none;

	if (bAnimating)
		return;

	MoveOn();
}

function bool MyOnDraw(Canvas Canvas)
{
	local GUIButton FButton;
	local int i,x2;
	local float XL,YL;
	local float DeltaTime;

	if (bAnimating || !Controller.bCurMenuInitialized )
		return false;

	DeltaTime=Controller.RenderDelta;

	for (i=0;i<Controls.Length;i++)
	{
		if ((GUIButton(Controls[i])!=None))
		{
 			FButton = GUIButton(Controls[i]);
			if (FButton.Tag>0 && FButton.MenuState!=MSAT_Focused)
			{
				FButton.Tag -= 784*DeltaTime;
				if (FButton.Tag<0)
					FButton.Tag=0;
			}
			else if (FButton.MenuState==MSAT_Focused)
				FButton.Tag=200;

			if (FButton.Tag>0)
			{
				fButton.Style.TextSize(Canvas,MSAT_Focused, FButton.Caption,XL,YL,FButton.FontScale);
				x2 = FButton.ActualLeft() + XL + 16;
				Canvas.Style=5;
				Canvas.SetDrawColor(255,255,255,FButton.Tag);
				Canvas.SetPos(0,fButton.ActualTop());
				Canvas.DrawTilePartialStretched(material'Highlight',x2,FButton.ActualHeight());
			}
		}
	}
	return false;
}

event bool NotifyLevelChange()
{
	if (bDebugging)
		log(Name@"NotifyLevelChange  PendingConnection:"$PlayerOwner().Level.IsPendingConnection());

	return PlayerOwner().Level.IsPendingConnection();
}

defaultproperties
{
	bDebugging=false

	Begin Object Class=BackgroundImage Name=PageBackground
		Image=material'2K4Menus.Controls.mmbgnd'
		ImageStyle=ISTY_Scaled
		ImageRenderStyle=MSTY_Alpha
		X1=0
		Y1=0
		X2=1024
		Y2=768
		RenderWeight=0.01
	End Object

	Begin Object Class=BackgroundImage Name=PageScanLines
		ImageStyle=ISTY_Tiled
		Image=Material'2K4Menus.BkRenders.ScanLines'
		ImageRenderStyle=MSTY_Alpha
		ImageColor=(R=255,G=255,B=255,A=32)
		WinWidth=1.0
		WinHeight=1.0
		WinLeft=0.0
		WinTop=0.0
		X1=0
		Y1=0
		X2=32
		Y2=32
		RenderWeight=0.03
	End Object

	Begin Object Class=GUIImage Name=SlideBottomBox
		ImageStyle=ISTY_Stretched
		Image=Material'FOMisc.MainMenu.MainMenuOverlayBottom'
		ImageRenderStyle=MSTY_Alpha
		ImageColor=(R=255,G=255,B=255,A=220)
		WinWidth=1.0
		WinHeight=0.25
		WinLeft=0.0
		WinTop=0.75
		RenderWeight=0.05
	End Object
	SlideBottom=SlideBottomBox

	Begin Object Class=GUIImage Name=SlideTopBox
		ImageStyle=ISTY_Stretched
		Image=Material'FOMisc.MainMenu.MainMenuOverlayTop'
		ImageRenderStyle=MSTY_Alpha
		ImageColor=(R=255,G=255,B=255,A=220)
		WinWidth=1.0
		WinHeight=0.25
		WinLeft=0.0
		WinTop=0.25
		RenderWeight=0.05
	End Object
	SlideTop=SlideTopBox

	Begin Object Class=GUIImage Name=MenuLogoTex
		Image=material'2K4Menus.BkRenders.MainMenuGuy'
		ImageStyle=ISTY_Scaled
		ImageRenderStyle=MSTY_Alpha
		ImageColor=(R=255,G=255,B=255,A=225)
		WinWidth=1.0
		WinHeight=1.0
		WinLeft=0.0
		WinTop=0.0
		X1=0
		Y1=0
		X2=1024
		Y2=768
		RenderWeight=0.04
	End Object
	MenuLogo=MenuLogoTex

	Begin Object Class=GUIButton Name=MultiplayerButton
		FontScale=FNS_Small
		StyleName="MainButton"
		Caption="PLAY ONLINE"
		CaptionEffectStylename="MainButtonEffect"
		CaptionAlign=TXTA_Left
		Hint="Play with human opponents over a LAN or the internet"
		OnClick=ButtonClick
		WinWidth=0.593666
		WinHeight=0.075000
		WinLeft=0.363246
		WinTop=0.449282
		bFocusOnWatch=true
		TabOrder=0
		bUseCaptionHeight=true
		RenderWeight=0.04
	End Object

	Begin Object Class=GUIButton Name=HostButton
		FontScale=FNS_Small
		StyleName="MainButton"
		Caption="HOST GAME"
	        CaptionEffectStylename="MainButtonEffect"
		CaptionAlign=TXTA_Left
		Hint="Start a server and invite others to join your game"
		OnClick=ButtonClick
		WinWidth=0.593666
		WinHeight=0.075000
		WinLeft=0.395097
		WinTop=0.534027
		bFocusOnWatch=true
		TabOrder=1
		bUseCaptionHeight=true
		RenderWeight=0.04
	End Object

	Begin Object Class=GUIButton Name=InstantActionButton
		FontScale=FNS_Small
		StyleName="MainButton"
		Caption="BOTMATCH"
		CaptionEffectStylename="MainButtonEffect"
		CaptionAlign=TXTA_Left
		Hint="Play a practice match"
		OnClick=ButtonClick
		WinWidth=0.593666
		WinHeight=0.075000
		WinLeft=0.423640
		WinTop=0.618619
		bFocusOnWatch=true
		TabOrder=2
		bUseCaptionHeight=true
		RenderWeight=0.04
	End Object

	Begin Object Class=GUIButton Name=ModsAndDemosButton
		FontScale=FNS_Small
		StyleName="MainButton"
		Caption="COMMUNITY"
		CaptionEffectStylename="MainButtonEffect"
		CaptionAlign=TXTA_Left
		Hint="Get the latest news, demos and mods from the UT2004 community"
		OnClick=ButtonClick
		WinWidth=0.593666
		WinHeight=0.075000
		WinLeft=0.433406
		WinTop=0.705859
		bFocusOnWatch=true
		TabOrder=3
		bUseCaptionHeight=true
		RenderWeight=0.04
	End Object

	Begin Object Class=GUIButton Name=SettingsButton
		FontScale=FNS_Small
		StyleName="MainButton"
		Caption="SETTINGS"
		CaptionEffectStylename="MainButtonEffect"
		CaptionAlign=TXTA_Left
		Hint="Change your controls and settings"
		OnClick=ButtonClick
		WinWidth=0.593666
		WinHeight=0.075000
		WinLeft=0.434477
		WinTop=0.800327
		bFocusOnWatch=true
		TabOrder=4
		bUseCaptionHeight=true
		RenderWeight=0.04
	End Object

	Begin Object Class=GUIButton Name=QuitButton
		FontScale=FNS_Small
		StyleName="MainButton"
		Caption="QUIT"
		CaptionEffectStylename="MainButtonEffect"
		CaptionAlign=TXTA_Left
		Hint="Exit the game"
		OnClick=ButtonClick
		WinWidth=0.593666
		WinHeight=0.075000
		WinLeft=0.424711
		WinTop=0.887567
		bFocusOnWatch=true
		TabOrder=5
		bUseCaptionHeight=true
		RenderWeight=0.04
	End Object

	Begin Object class=moEditBox Name=PlayerNameBox
		WinWidth=0.23242
		WinLeft=0.101757
		WinTop=0.66042
		Caption=""
		INIOption="@INTERNAL"
		INIDefault="Player"
		OnLoadINI=InternalOnLoadINI
		OnChange=InternalOnChange
		Hint="Changes your player name."
		CaptionWidth=0
		bAutoSizeCaption=True
		TabOrder=6
		RenderWeight=0.04
	End Object
	PlayerName=PlayerNameBox

	Begin Object class=GUILabel Name=NameLabelctrl	
		Caption="Player Name"
		TextALign=TXTA_Left
		TextFont="UT2MidGameFont"
		TextColor=(R=255,G=255,B=255,A=210)
		WinWidth=0.16
		WinHeight=0.06
		WinLeft=0.05
		WinTop=0.829
		bBoundToParent=true
	End Object
	NameLabel=NameLabelctrl	

	Begin Object class=GUILabel Name=VersionLabelctrl
		Caption="Frag.Ops v2.16 beta (c) 2003-2004 Pandora Studios"
		TextALign=TXTA_Left
		TextFont="UT2ServerListFont"
		TextColor=(R=255,G=255,B=255,A=210)
		WinWidth=0.7
		WinHeight=0.06
		WinLeft=0.05
		WinTop=0.829
		bBoundToParent=true
	End Object
	VersionLabel=VersionLabelctrl	

	i_Background=PageBackground
	//i_BGSec=PageScanLines
	b_MultiPlayer=MultiplayerButton
	b_Host=HostButton
	b_InstantAction=InstantActionButton
	b_ModsAndDemo=ModsAndDemosButton
	b_Settings=SettingsButton
	b_Quit=QuitButton

	WinWidth=1.0
	WinHeight=1.0
	WinTop=0.0
	WinLeft=0.0

	bRenderWorld=False
	bAllowClose=False
	bAllowedAsLast=true
	bDisconnectOnOpen=true

	OnOpen=InternalOnOpen
	OnCanClose=CanClose
	OnKeyEvent=MyKeyEvent
	OnReopen=MainReopened
	MenuSong="fragops_main"
}