//=============================================================================
// Frag.Ops  2004 Matt 'SquirrelZero' Farber
//=============================================================================
// Interface_RoundStats.uc
// a special second scoreboard that updates itself only once per round.  This 
// announces what medals have been awarded, best players, most damage, most
// accurate, etc.
//=============================================================================
class Interface_RoundStats extends ScoreBoard;

var() material RoundStatsBG;
var() material MedalBoxBG;
var() color MedalBoxColor;

var bool bUpdateStatsInfo;

var string BestOverall;
var string BestOverallScore;
var string LeastDamage;
var int LeastDamageScore;
var string MostDamage;
var int MostDamageScore;
var string MostAccurate;
var string MostAccurateScore;
var string MostHeadshots;
var int MostHeadshotsScore;
var array<PlayerReplicationInfo> PRIArray;
var array<int> ImpactMedals;
var int PlayerCount, StompCount;
var float LastStompTime;
var bool bNewStats, bFirstStomp;
var class<Interface_ScoreBoard> ScoreBoardClass;

simulated function DrawRoundStats(Canvas Canvas)
{
	local float HeaderOffsetY;
	local float HeaderOffsetX;

	HeaderOffsetY = Canvas.ClipY*0.34;
	HeaderOffsetX = 0.15*Canvas.ClipX;

	if (bNewStats)
	{
		SetTimer(7.0,false);
		StompCount = -1;
		bNewStats = false;
		bFirstStomp = true;
	}
	DrawStatistics(Canvas, HeaderOffsetY, HeaderOffsetX);
	StompMedals(Canvas, HeaderOffsetY, HeaderOffsetX);
}

// let's draw some nice statistics
simulated function DrawStatistics(Canvas C, float HeaderOffsetY, float HeaderOffsetX)
{
	local float NameOffsetX, StatOffsetX;
	local float XL,YL;

	// set font
	C.Font = GetSmallerFontFor(C,4);

	// set up second column
	NameOffsetX = C.ClipX*0.31;

	// set up third column
	StatOffsetX = C.ClipX*0.52;

	// set up line heights
	C.StrLen("yTEST",XL,YL);

	// draw most damage
	C.SetPos(HeaderOffsetX, HeaderOffsetY);
	C.DrawColor = HUDClass.Default.GoldColor;
	C.DrawText("Most Damage:");
	C.SetPos(NameOffsetX, HeaderOffsetY);
	C.DrawColor = HUDClass.Default.WhiteColor;
	C.DrawText(MostDamage);
	C.SetPos(StatOffsetX, HeaderOffsetY);
	C.DrawText(MostDamageScore);

	// draw most accurate
	C.SetPos(HeaderOffsetX, HeaderOffsetY+YL);
	C.DrawColor = HUDClass.Default.GoldColor;
	C.DrawText("Most Accurate:");
	C.SetPos(NameOffsetX, HeaderOffsetY+YL);
	C.DrawColor = HUDClass.Default.WhiteColor;
	C.DrawText(MostAccurate);
	C.SetPos(StatOffsetX, HeaderOffsetY+YL);
	C.DrawText(MostAccurateScore);

	// draw most headshots
	C.SetPos(HeaderOffsetX, HeaderOffsetY+YL*2);
	C.DrawColor = HUDClass.Default.GoldColor;
	C.DrawText("Most Headshots:");
	C.SetPos(NameOffsetX, HeaderOffsetY+YL*2);
	C.DrawColor = HUDClass.Default.WhiteColor;
	C.DrawText(MostHeadshots);
	C.SetPos(StatOffsetX, HeaderOffsetY+YL*2);
	C.DrawText(MostHeadshotsScore);

	// draw most effective
	C.SetPos(HeaderOffsetX, HeaderOffsetY+YL*3);
	C.DrawColor = HUDClass.Default.GreenColor;
	C.DrawText("Most Effective:");
	C.SetPos(NameOffsetX, HeaderOffsetY+YL*3);
	C.DrawColor = HUDClass.Default.WhiteColor;
	C.DrawText(BestOverall);
	C.SetPos(StatOffsetX, HeaderOffsetY+YL*3);
	C.DrawText(BestOverallScore);

	// draw most useless
	C.SetPos(HeaderOffsetX, HeaderOffsetY+YL*4);
	C.DrawColor = HUDClass.Default.RedColor;
	C.DrawText("Most Useless:");
	C.SetPos(NameOffsetX, HeaderOffsetY+YL*4);
	C.DrawColor = HUDClass.Default.WhiteColor;
	C.DrawText(LeastDamage);
	C.SetPos(StatOffsetX, HeaderOffsetY+YL*4);
	C.DrawText(LeastDamageScore$" Score");
}

simulated function StompMedals(Canvas Canvas, float HeaderOffsetY, float HeaderOffsetX)
{
	local float MedalOffsetX, MedalBoxOffsetX, MedalNameOffsetX, MedalPrizeOffsetX, MedalOffsetY, XL, YL;
	local int NumMedals, i;

	MedalOffsetY = Canvas.ClipY*0.53;
	MedalOffsetX = HeaderOffsetX;
	MedalBoxOffsetX = HeaderOffsetX - Canvas.ClipX*0.005;;
	MedalNameOffsetX = HeaderOffsetX + Canvas.ClipX*0.05;
	MedalPrizeOffsetX = MedalNameOffsetX + Canvas.ClipX*0.22;
	Canvas.DrawColor = HUDClass.default.WhiteColor * 0.8;

	if (ImpactMedals.Length <= 0)
	{
		Canvas.SetPos(MedalOffsetX, MedalOffsetY);
		Canvas.DrawText("no new medals received.");
		return;
	}
	if (bFirstStomp)
	{
		LastStompTime = Level.TimeSeconds;
		bFirstStomp = false;
		return;
	}
	if (StompCount >= 0)
	{
		Canvas.Font = GetSmallerFontFor(Canvas,3);
		for (i=0;i<StompCount;i++)
		{
			// draw background box
			Canvas.DrawColor = Default.MedalBoxColor;
			Canvas.SetPos(MedalBoxOffsetX,(MedalOffsetY*0.995) + ((Canvas.ClipY / 19)*NumMedals));
			Canvas.DrawTileStretched(MedalBoxBG, 0.6*Canvas.ClipX, 0.05*Canvas.ClipY);

			// draw medal icon
			Canvas.DrawColor = HUDClass.default.WhiteColor * 0.8;
			Canvas.SetPos(MedalOffsetX,(MedalOffsetY) + ((Canvas.ClipY / 19)*NumMedals));
			Canvas.DrawIcon(ScoreBoardClass.Default.MedalMaterialBig[ImpactMedals[i]], Canvas.ClipY/2800);

			// draw medal name and prize
			Canvas.StrLen(ScoreBoardClass.Default.MedalNames[ImpactMedals[i]],XL,YL);
			// draw centered
			Canvas.SetPos(MedalNameOffsetX,(MedalOffsetY*0.995+((0.5*(0.05*Canvas.ClipY)) - (0.5*YL)))+((Canvas.ClipY / 19)*NumMedals));
			Canvas.DrawText(ScoreBoardClass.Default.MedalNames[ImpactMedals[i]],true);
			Canvas.SetPos(MedalPrizeOffsetX,(MedalOffsetY*0.995+((0.5*(0.05*Canvas.ClipY)) - (0.5*YL)))+((Canvas.ClipY / 19)*NumMedals));
			Canvas.DrawText(ScoreBoardClass.Default.MedalPrizes[ImpactMedals[i]],true);

			// increment to the next medal
			NumMedals++;
		}
	}
	if (Level.TimeSeconds - LastStompTime >= 0.3 && StompCount < ImpactMedals.Length)
	{
		LastStompTime = Level.TimeSeconds;
		if (StompCount < 0)
			StompCount = 0;
		StompCount++;
		FOPlayer(Owner).StompSound();
	}
}

simulated function Timer()
{
	if (Interface_MainHUD(PlayerController(Owner).MyHUD) != None && Interface_MainHUD(PlayerController(Owner).MyHUD).bShowRoundStats)
		Interface_MainHUD(PlayerController(Owner).MyHUD).bShowRoundStats = false;
}

defaultproperties
{
	HUDClass=class'FragOps.Interface_MainHUD'
	ScoreBoardClass=class'FragOps.Interface_ScoreBoard'
	RoundStatsBG=material'FOMisc.TeamMenu.SquareButtonDark'
	MedalBoxBG=material'FOMisc.Menu.MedalBoxBG'
	MedalBoxColor=(R=0,B=0,G=0,A=110)
}