//=============================================================================
// Frag.Ops  2003 Matt 'SquirrelZero' Farber
//=============================================================================
class Interface_ScoreBoard extends ScoreBoardDeathMatch;

var Material TeamBoxMaterial[2];
var Texture MedalMaterial[6], MedalMaterialBig[6], ScoreText[2], MedalText;
var string MedalNames[6], MedalPrizes[6];
var float MedalWidth[6];
var Material ScoreBack;
var Material ScoreboardU;
var Texture CarriedObjectiveIcon;
var Color TeamColors[2], DarkenColor;
var localized string RoundTime;
var FragOps_CRI CRIArray[MAXPLAYERS];
var string EscapeFireText;
var font OldFont;

simulated function UpdatePrecacheMaterials()
{
	Level.AddPrecacheMaterial(TeamBoxMaterial[0]);
	Level.AddPrecacheMaterial(TeamBoxMaterial[1]);
	Level.AddPrecacheMaterial(ScoreText[0]);
	Level.AddPrecacheMaterial(ScoreText[1]);
	UpdatePrecacheFonts();
}

function DrawTitle(Canvas Canvas, float HeaderOffsetY, float PlayerAreaY, float PlayerBoxSizeY)
{
	local string TitleString, WaitingString;
	local float MedalOffsetX, TitleXL, TitleYL;
	local FragOps_CRI MedalCRI;
	local int i, NumMedals, j, PrevMedal;
	local array<int> ScoreMedals;
	local bool bDrawWaiting;
	
	// darken background
	Canvas.Style = ERenderStyle.STY_Alpha;
	Canvas.SetPos(0,0);
	Canvas.DrawColor = Default.DarkenColor;
	Canvas.DrawTileStretched(TeamBoxMaterial[0], Canvas.ClipX, Canvas.ClipY);
	
	if (Canvas.ClipX < 512)
		return;	

	// draw medal info text, bottom
	MedalCRI = FOPlayer(Owner).CriticalReplicationInfo;
	MedalOffsetX = 0.12*Canvas.ClipX;
	Canvas.DrawColor = HUDClass.default.WhiteColor * 0.75;
	Canvas.SetPos(MedalOffsetX, (HeaderOffsetY) + PlayerAreaY);
	Canvas.DrawIcon(MedalText, (Canvas.ClipX / 1400));
	NumMedals = 0;
	Canvas.DrawColor = HUDClass.default.WhiteColor * 0.8;

	// draw medals, bottom
	for(j=0;j<ArrayCount(MedalCRI.Medals);j++)
	{
		if (MedalCRI.Medals[j] > 0)
		//if (MedalCRI.Medals[j] == 0)
		{
			ScoreMedals[NumMedals] = j;
			NumMedals++;
		}
	}
	MedalOffsetX = MedalOffsetX + Canvas.ClipX/10.12;
	for(j=0;j<NumMedals;j++)
	{
		MedalOffsetX = (MedalOffsetX + (Canvas.ClipX/MedalWidth[ScoreMedals[j]])) - 0.5*(Canvas.ClipX/MedalWidth[ScoreMedals[j]]);
		Canvas.SetPos(MedalOffsetX, (Canvas.ClipY*0.161) + PlayerAreaY);
		Canvas.DrawIcon(MedalMaterialBig[ScoreMedals[j]], Canvas.ClipX/2300);
		MedalOffsetX = MedalOffsetX + 0.55*(Canvas.ClipX/MedalWidth[ScoreMedals[j]]);
		PrevMedal = ScoreMedals[j];
	}

	// draw scoreboard header, top
	TitleString = "Round #"$string(FragOps_GRI(GRI).RoundCount)$" in "$Level.Title;
	OldFont = Canvas.Font;
	Canvas.DrawColor = HUDClass.default.WhiteColor;
	Canvas.Font = GetSmallerFontFor(Canvas,4);
	Canvas.StrLen(TitleString,TitleXL,TitleYL);
	Canvas.SetPos(0.5 * (Canvas.ClipX-TitleXL),0.086 * Canvas.ClipY);
	Canvas.DrawText(TitleString,true);

	// draw waiting players, bottom
	WaitingString = "No Team Yet: ";
	for (i=0;i<GRI.PRIArray.Length;i++)
	{
		if (GRI.PRIArray[i].Team == None && !GRI.PRIArray[i].bOnlySpectator)
		{
			WaitingString = WaitingString$GRI.PRIArray[i].PlayerName$", ";
			bDrawWaiting = true;
		}
	}
	WaitingString = Left(WaitingString, Len(WaitingString)-2);
	if (bDrawWaiting)
	{
		Canvas.DrawColor = HUDClass.Default.WhiteColor;
		Canvas.SetPos(Canvas.ClipX*0.029, PlayerAreaY+(Canvas.ClipY*0.12));
		Canvas.DrawText(WaitingString,true);
	}
	Canvas.Font = OldFont;
}

simulated event UpdateScoreBoard(Canvas Canvas)
{
	local PlayerReplicationInfo PRI, OwnerPRI;
	local int i, FontReduction,HeaderOffsetY,HeadFoot,PlayerBoxSizeY, BoxSpaceY;
	local float XL,YL,XXL,YYL, IconSize, MessageFoot;
	local int BluePlayerCount, RedPlayerCount, RedOwnerOffset, BlueOwnerOffset, MaxPlayerCount;
	local PlayerReplicationInfo RedPRI[MAXPLAYERS], BluePRI[MaxPlayers];
		
	OwnerPRI = PlayerController(Owner).PlayerReplicationInfo;
	RedOwnerOffset = -1;
	BlueOwnerOffset = -1;
	for (i=0; i<GRI.PRIArray.Length; i++)
	{
		PRI = GRI.PRIArray[i];
		if ((PRI.Team != None) && (!PRI.bIsSpectator || PRI.bWaitingPlayer))
		{
			if (PRI.Team.TeamIndex == 0)
			{
				if (RedPlayerCount < MAXPLAYERS)
				{
					RedPRI[RedPlayerCount] = PRI;
					if (PRI == OwnerPRI)
						RedOwnerOffset = RedPlayerCount;
					RedPlayerCount++;
				}
			}
			else if (BluePlayerCount < MAXPLAYERS)
			{
				BluePRI[BluePlayerCount] = PRI;
				if ( PRI == OwnerPRI )
					BlueOwnerOffset = BluePlayerCount;
				BluePlayerCount++;
			}
		}
	}
	MaxPlayerCount = Max(RedPlayerCount,BluePlayerCount);
	IconSize = FMax(2 * YL, 128 * Canvas.ClipX/1024);

	bDisplayMessages = true;

	// reduce font if necessary
	FontReduction = 1;

	// header/footer stuff
	HeadFoot = 4*YL + IconSize;
	MessageFoot = 1.5 * HeadFoot;

	// space between boxes
	BoxSpaceY = Canvas.ClipY*(0.008-(0.002*FontReduction));

	// text Y offset
	HeaderOffsetY = 0.165*Canvas.ClipY;

	// Select best font size and box size to fit as many players as possible on screen
	Canvas.Font = GetSmallFontFor(Canvas.ClipX,FontReduction);
	Canvas.StrLen("Test", XL, YL);
	Canvas.Font = GetSmallFontFor(Canvas.ClipX,FontReduction+1);
	Canvas.StrLen("Test", XXL, YYL);

	// set box size
	PlayerBoxSizeY = YL*1.008 + YYL*1.008 + 0.006*Canvas.ClipY;

	// set playercounts
	MaxPlayerCount = Min(MaxPlayerCount, 1+(Canvas.ClipY - HeadFoot)/(PlayerBoxSizeY + BoxSpaceY) );
	RedPlayerCount = Min(RedPlayerCount,MaxPlayerCount);
	BluePlayerCount = Min(BluePlayerCount,MaxPlayerCount);
	if (RedOwnerOffset >= RedPlayerCount)
		RedPlayerCount -= 1;
	if (BlueOwnerOffset >= BluePlayerCount)
		BluePlayerCount -= 1;

	// init criticalreplication array
	for (i=0;i<FragOps_GRI(GRI).CRIArray.Length;i++)
		CRIArray[i] = FragOps_GRI(GRI).CRIArray[i];

	// draw title
	Canvas.Style = ERenderStyle.STY_Normal;
	DrawTitle(Canvas,HeaderOffsetY,(MaxPlayerCount+1)*(PlayerBoxSizeY + BoxSpaceY),PlayerBoxSizeY);

	if (MaxPlayerCount > 6)
		FontReduction = 1;

	// draw red team
	for (i=0; i<32; i++ )
		PRIArray[i] = RedPRI[i];
	DrawTeam(0,RedPlayerCount,RedOwnerOffset,Canvas,FontReduction,BoxSpaceY,PlayerBoxSizeY,HeaderOffsetY);
	
	// draw blue team
	for (i=0; i<32; i++ )
		PRIArray[i] = BluePRI[i];
	DrawTeam(1,BluePlayerCount,BlueOwnerOffset,Canvas,FontReduction,BoxSpaceY,PlayerBoxSizeY,HeaderOffsetY);

	if (Level.NetMode != NM_Standalone)
		DrawMatchID(Canvas,FontReduction);	
}

function DrawTeam(int TeamNum, int PlayerCount, int OwnerOffset, Canvas Canvas, int FontReduction, int BoxSpaceY, int PlayerBoxSizeY, int HeaderOffsetY)
{
	local int i, j, OrderXPos, ObjectiveXPos, OwnerPos, NetXPos, NameXPos, BoxTextOffsetY, NumMedals, ScoreXPos, ScoreYPos, BoxXPos, BoxWidth, DeathXPos, LineCount;
	local float XL,YL,NameHeight,MaxNamePos,MedalScale,TraitLength,TeamIconSizeX,TeamIconSizeY;
	local string PlayerName[MAXPLAYERS], AccText, DamageText, TraitLine;
	local int OtherTeam;
	local FragOps_CRI MedalCRI;

	BoxWidth = 0.491 * Canvas.ClipX;
	BoxXPos = 0.5 * (0.5 * Canvas.ClipX - BoxWidth);
	BoxWidth = 0.5 * Canvas.ClipX - 2*BoxXPos;
	BoxXPos = BoxXPos + TeamNum * 0.5 * Canvas.ClipX; 
	NameXPos = BoxXPos + 0.012 * BoxWidth;
	ScoreXPos = BoxXPos + 0.83 * BoxWidth;
	OrderXPos = BoxXPos + 0.627 * BoxWidth;
	DeathXPos = BoxXPos + 0.915 * BoxWidth;
	NetXPos = BoxXPos + 0.50 * BoxWidth;
	MedalScale = Canvas.ClipX/8192;
	ObjectiveXPos = BoxXPos + 0.578 * BoxWidth;

	// draw team header
	Canvas.Style = ERenderStyle.STY_Normal;
	Canvas.StrLen("0",XL,YL);

	// set size of header icons
	TeamIconSizeX = Canvas.ClipX*0.2;
	TeamIconSizeY = Canvas.ClipY*0.03;
	ScoreYPos = HeaderOffsetY - TeamIconSizeY - (Canvas.ClipY*0.02);

	// draw icon
	Canvas.DrawColor = HUDClass.default.WhiteColor * 0.8;
	Canvas.SetPos(Canvas.ClipX*0.25 + ((0.5*Canvas.ClipX)*TeamNum) - ((TeamIconSizeX+XL)*0.5), ScoreYPos);
	Canvas.DrawTile(ScoreText[TeamNum], TeamIconSizeX, TeamIconSizeY, 0, 0, ScoreText[TeamNum].MaterialUSize(), ScoreText[TeamNum].MaterialVSize());

	// draw score
	Canvas.Font = GetSmallFontFor(Canvas.ClipX, 0);
	Canvas.DrawColor = HUDClass.default.WhiteColor;
	Canvas.SetPos(Canvas.ClipX*0.25 + ((0.5*Canvas.ClipX)*TeamNum) - ((TeamIconSizeX+XL)*0.5) + TeamIconSizeX, ScoreYPos);
	Canvas.DrawText(int(GRI.Teams[TeamNum].Score));

	// draw BG box
	Canvas.Style = ERenderStyle.STY_Alpha;
	Canvas.SetPos(BoxXPos, HeaderOffsetY);
	Canvas.DrawColor = TeamColors[TeamNum];
	Canvas.DrawTileStretched(Material'FOMisc.Special.WhiteFlash', BoxWidth, PlayerCount*(PlayerBoxSizeY + BoxSpaceY));	
	Canvas.Style = ERenderStyle.STY_Normal;

	// set font
	Canvas.Font = GetSmallFontFor(Canvas.ClipX, FontReduction);

	// set maximum name length
	MaxNamePos = 0.80 * (OrderXPos - NameXPos);
	for (i=0;i<PlayerCount;i++)
	{
		PlayerName[i] = PRIArray[i].PlayerName;
		Canvas.StrLen(PlayerName[i], XL, YL);
		if (XL > MaxNamePos)
			PlayerName[i] = left(PlayerName[i], MaxNamePos/XL * len(PlayerName[i]));
	}
	if (OwnerOffset >= PlayerCount)
	{
		PlayerName[OwnerOffset] = PRIArray[OwnerOffset].PlayerName;
		Canvas.StrLen(PlayerName[OwnerOffset], XL, YL);
		if (XL > MaxNamePos)
			PlayerName[OwnerOffset] = left(PlayerName[OwnerOffset], MaxNamePos/XL * len(PlayerName[OwnerOffset]));
	}	

	// initialize some settings
	Canvas.Style = ERenderStyle.STY_Normal;
	BoxTextOffsetY = HeaderOffsetY;
	NameHeight = YL;

	for (i=0;i<PlayerCount;i++)
		if (i != OwnerOffset)
		{
			Canvas.SetPos(NameXPos, (PlayerBoxSizeY + BoxSpaceY)*i + BoxTextOffsetY);
			if (PRIArray[i].bAdmin)
			{
				if (PRIArray[i].bOutOfLives || FragOps_PRI(PRIArray[i]).bEscaped)
					Canvas.DrawColor = HUDClass.default.RedColor * 0.7;				
				else
					Canvas.DrawColor = HUDClass.default.RedColor;
			}
			else
			{
				if (PRIArray[i].bOutOfLives || FragOps_PRI(PRIArray[i]).bEscaped)
					Canvas.DrawColor = HUDClass.default.GrayColor * 0.7;				
				else
					Canvas.DrawColor = HUDClass.default.WhiteColor;
			}
			Canvas.DrawText(PlayerName[i],true);
		}

	// draw scores
	Canvas.DrawColor = HUDClass.default.WhiteColor;
	for (i=0;i<PlayerCount;i++)
		if (i != OwnerOffset)
		{
			if (PRIArray[i].bOutOfLives || FragOps_PRI(PRIArray[i]).bEscaped)
				Canvas.DrawColor = HUDClass.default.GrayColor  * 0.7;
			else
				Canvas.DrawColor = HUDClass.default.WhiteColor;
			Canvas.SetPos(ScoreXPos, (PlayerBoxSizeY + BoxSpaceY)*i + BoxTextOffsetY);
			Canvas.DrawText(int(PRIArray[i].Score),true);
		}

	// draw deaths
	for (i=0;i<PlayerCount;i++)
		if (i != OwnerOffset)
		{
			if (PRIArray[i].bOutOfLives || FragOps_PRI(PRIArray[i]).bEscaped)
				Canvas.DrawColor = HUDClass.default.GrayColor * 0.7;				
			else
				Canvas.DrawColor = HUDClass.default.WhiteColor;
			Canvas.SetPos(DeathXPos, (PlayerBoxSizeY + BoxSpaceY)*i + BoxTextOffsetY);
			Canvas.DrawText(int(PRIArray[i].Deaths),true);
		}

	// draw objective icon
	for (i=0;i<PlayerCount;i++)
		if (i != OwnerOffset && FragOps_PRI(PRIArray[i]).bHasObjective && PRIArray[i].Team.TeamIndex == PlayerController(Owner).PlayerReplicationInfo.Team.TeamIndex)
		{
			Canvas.SetDrawColor(255,255,255,200);
			Canvas.SetPos(ObjectiveXPos, (PlayerBoxSizeY + BoxSpaceY)*i + (HeaderOffsetY + 0.23*PlayerBoxSizeY));
			Canvas.DrawIcon(CarriedObjectiveIcon, Canvas.ClipX/1500);
		}

	// draw player traits and medals
	for (i=0;i<PlayerCount;i++)
		if (i != OwnerOffset)
		{
			// traits		
			Canvas.Font = GetSmallFontFor(Canvas.ClipX, FontReduction+1);
			if (FragOps_PRI(PRIArray[i]).bEscaped)
				TraitLine = "[ESCAPED]";
			else if (PRIArray[i].bOutOflives)
				TraitLine = "[DEAD]";
			else if (FragOps_PRI(PRIArray[i]).TraitType >= class'Interface_TabPlayerTeam'.Default.Traits.Length)
				TraitLine = "No Trait";
			else
				TraitLine = class'Interface_TabPlayerTeam'.Default.Traits[FragOps_PRI(PRIArray[i]).TraitType].Default.TraitName;
			Canvas.StrLen(TraitLine, XL, YL);
			if (PRIArray[i].bOutOfLives || FragOps_PRI(PRIArray[i]).bEscaped)
				Canvas.DrawColor = HUDClass.default.WhiteColor * 0.5;
			else
				Canvas.DrawColor = HUDClass.default.WhiteColor * 0.7;
			Canvas.SetPos(NameXPos, (PlayerBoxSizeY + BoxSpaceY)*i + BoxTextOffsetY + (NameHeight*1.1));
			Canvas.DrawText(TraitLine);
			TraitLength = XL;

			// medals
			for (j=0;j<FragOps_GRI(GRI).CRIArray.Length;j++)
			{
				if (CRIArray[j].PCName ~= FragOps_PRI(PRIArray[i]).PCName)
				{
					MedalCRI = CRIArray[j];
					break;
				}
			}
			if (PRIArray[i].bOutOfLives || FragOps_PRI(PRIArray[i]).bEscaped)
				Canvas.DrawColor = HUDClass.default.WhiteColor * 0.6;
			else
				Canvas.DrawColor = HUDClass.default.WhiteColor * 0.8;

			Canvas.SetPos((NameXPos + (Canvas.ClipX*0.005) + TraitLength), (PlayerBoxSizeY + BoxSpaceY)*i + BoxTextOffsetY + (NameHeight*1.04));
			NumMedals = 0;
			for(j=0;j<ArrayCount(MedalCRI.Medals);j++)
				//if (j == 0 || j == 1)
				if (MedalCRI.Medals[j] > 0)
				{
					Canvas.DrawTile(MedalMaterial[j], PlayerBoxSizeY*(0.40-(0.08*FontReduction)), PlayerBoxSizeY*(0.40-(0.08*FontReduction)), 0, 0, MedalMaterial[j].MaterialUSize(), MedalMaterial[j].MaterialVSize());
					NumMedals++;
					Canvas.SetPos((NameXPos + (Canvas.ClipX*0.005) + TraitLength) + (PlayerBoxSizeY*(0.45-(0.11*FontReduction))*NumMedals), (PlayerBoxSizeY + BoxSpaceY)*i + BoxTextOffsetY + (NameHeight*1.04));
				}		
		}

	Canvas.Font = GetSmallFontFor(Canvas.ClipX, FontReduction);
	
	// draw owner line
	if (OwnerOffset >= 0)
	{
		for (j=0;j<FragOps_GRI(GRI).CRIArray.Length;j++)
		{
			if (CRIArray[j].PCName == FragOps_PRI(PRIArray[OwnerOffset]).PCName)
			{
				MedalCRI = CRIArray[j];
				break;
			}
		}
		if (OwnerOffset >= PlayerCount)
		{
			OwnerPos = (PlayerBoxSizeY + BoxSpaceY)*PlayerCount + BoxTextOffsetY;
			Canvas.Style = ERenderStyle.STY_Normal;	
		}
		else
			OwnerPos = (PlayerBoxSizeY + BoxSpaceY)*OwnerOffset + BoxTextOffsetY;


		if (PRIArray[OwnerOffset].bAdmin)
		{
			if (PRIArray[OwnerOffset].bOutOfLives || FragOps_PRI(PRIArray[OwnerOffset]).bEscaped)
				Canvas.DrawColor = HUDClass.default.RedColor * 0.7;				
			else
				Canvas.DrawColor = HUDClass.default.RedColor;
		}
		else
		{
			if (PRIArray[OwnerOffset].bOutOfLives || FragOps_PRI(PRIArray[OwnerOffset]).bEscaped)
				Canvas.DrawColor = HUDClass.default.GoldColor * 0.65;
			else
				Canvas.DrawColor = HUDClass.default.GoldColor;
		}

		Canvas.SetPos(NameXPos, OwnerPos);
		Canvas.DrawText(playername[OwnerOffset],true);
		Canvas.StrLen(playername[OwnerOffset],XL,YL);
		NameHeight = YL;

		if (PRIArray[OwnerOffset].bOutOfLives || FragOps_PRI(PRIArray[OwnerOffset]).bEscaped)
			Canvas.DrawColor = HUDClass.default.GoldColor * 0.65;
		else
			Canvas.DrawColor = HUDClass.default.GoldColor;
		
		Canvas.SetPos(ScoreXPos, OwnerPos);
		Canvas.DrawText(int(PRIArray[OwnerOffset].Score),true);
		Canvas.SetPos(DeathXPos, OwnerPos);
		Canvas.DrawText(int(PRIArray[OwnerOffset].Deaths),true);

		// draw objectiveicon		
		if (FragOps_PRI(PRIArray[OwnerOffset]).bHasObjective)
		{
			Canvas.SetDrawColor(255,255,255,200);
			Canvas.SetPos(ObjectiveXPos, OwnerPos + 0.23*PlayerBoxSizeY);
			Canvas.DrawIcon(CarriedObjectiveIcon, Canvas.ClipX/1500);
		}

		// draw owner trait
		Canvas.Font = GetSmallFontFor(Canvas.ClipX, FontReduction+1);
		if (FragOps_PRI(PRIArray[OwnerOffset]).bEscaped)
			TraitLine = "[ESCAPED]";
		else if (PRIArray[OwnerOffset].bOutOflives)
			TraitLine = "[DEAD]";
		else if (FragOps_PRI(PRIArray[OwnerOffset]).TraitType >= class'Interface_TabPlayerTeam'.Default.Traits.Length)
			TraitLine = "No Trait";
		else
			TraitLine = class'Interface_TabPlayerTeam'.Default.Traits[FragOps_PRI(PRIArray[OwnerOffset]).TraitType].Default.TraitName;
		Canvas.StrLen(TraitLine, XL, YL);
		if (PRIArray[OwnerOffset].bOutOfLives || FragOps_PRI(PRIArray[OwnerOffset]).bEscaped)
			Canvas.DrawColor = HUDClass.default.WhiteColor * 0.5;
		else
			Canvas.DrawColor = HUDClass.default.WhiteColor * 0.7;
		Canvas.SetPos(NameXPos, OwnerPos + (NameHeight*1.1));
		Canvas.DrawText(TraitLine);
		TraitLength = XL;

		// draw medals
		NumMedals = 0;
		if (PRIArray[OwnerOffset].bOutOfLives || FragOps_PRI(PRIArray[OwnerOffset]).bEscaped)
			Canvas.DrawColor = HUDClass.default.WhiteColor * 0.6;
		else
			Canvas.DrawColor = HUDClass.default.WhiteColor * 0.8;

		Canvas.SetPos((NameXPos + (Canvas.ClipX*0.005) + TraitLength), OwnerPos + (NameHeight*1.04));
		for(j=0;j<ArrayCount(MedalCRI.Medals);j++)
		{
			//if (j == 0 || j == 1)
			if (MedalCRI.Medals[j] > 0)
			{
				Canvas.DrawTile(MedalMaterial[j], PlayerBoxSizeY*(0.40-(0.08*FontReduction)), PlayerBoxSizeY*(0.40-(0.08*FontReduction)), 0, 0, MedalMaterial[j].MaterialUSize(), MedalMaterial[j].MaterialVSize());
				NumMedals++;
				Canvas.SetPos((NameXPos + (Canvas.ClipX*0.005) + TraitLength) + (PlayerBoxSizeY*(0.40-(0.1*FontReduction))*NumMedals), OwnerPos + (NameHeight*1.04));
			}
		}
	}

	if (TeamNum == 0)
		OtherTeam = 1;
		
	// draw accuracy/hitpoint score and/or orders
	if ((Canvas.ClipX >= 512))
	{
		//BoxTextOffsetY = HeaderOffsetY + 0.5*PlayerBoxSizeY;
		Canvas.DrawColor = HUDClass.default.GreenColor;
		Canvas.Font = GetSmallFontFor(Canvas.ClipX, FontReduction+1);
		Canvas.StrLen("TEST", XL, YL);
		for (i=0;i<PlayerCount;i++)
		{
			LineCount = 0;
			if (!PRIArray[i].bBot)
			{
				Canvas.SetPos(OrderXPos, ((PlayerBoxSizeY + BoxSpaceY)*i + BoxTextOffsetY)+Canvas.ClipY*(0.005-(float(FontReduction)*0.002)));
				Canvas.DrawColor = HUDClass.default.GreenColor;
				// fix for mac math bug
				AccText = "A: "$(string(max(0,int((float(FragOps_PRI(PRIArray[i]).NumShotsHit) / float(FragOps_PRI(PRIArray[i]).NumShots)) * 100))))$"/"$(string(max(0,int((float(FragOps_PRI(PRIArray[i]).NumHeadShots) / float(FragOps_PRI(PRIArray[i]).NumShotsHit)) * 100))))$"%";
				Canvas.DrawText(AccText);
			}
			Canvas.SetPos(ScoreXPos, (PlayerBoxSizeY + BoxSpaceY)*i + BoxTextOffsetY + (NameHeight*1.1));
			Canvas.DrawColor = HUDClass.default.WhiteColor;
			DamageText = "S: "$FragOps_PRI(PRIArray[i]).DamageDone;
			Canvas.DrawText(DamageText);
		}
		if (OwnerOffset >= PlayerCount && PRIArray[OwnerOffset].Team.TeamIndex == TeamNum) 
		{	
			Canvas.SetPos(OrderXPos, OwnerPos+Canvas.ClipY*(0.005-(float(FontReduction)*0.002)));
			Canvas.DrawColor = HUDClass.default.GreenColor;
			// fix for mac math bug
			AccText = "A: "$(string(max(0,int((float(FragOps_PRI(PRIArray[OwnerOffset]).NumShotsHit) / float(FragOps_PRI(PRIArray[OwnerOffset]).NumShots)) * 100))))$"/"$(string(max(0,int((float(FragOps_PRI(PRIArray[OwnerOffset]).NumHeadShots) / float(FragOps_PRI(PRIArray[OwnerOffset]).NumShotsHit)) * 100))))$"%";
			Canvas.DrawText(AccText);
			Canvas.SetPos(ScoreXPos, OwnerPos + (NameHeight*1.1));
			Canvas.DrawColor = HUDClass.default.WhiteColor;
			DamageText = "S: "$FragOps_PRI(PRIArray[OwnerOffset]).DamageDone;
			Canvas.DrawText(DamageText);
		}
	}
				
	// no ping/time in botmatch
	if (Level.NetMode == NM_Standalone)
		return;

	DrawNetInfo(Canvas,FontReduction,HeaderOffsetY,PlayerBoxSizeY,BoxSpaceY,BoxTextOffsetY,OwnerOffset,PlayerCount,NetXPos);
}

function DrawNetInfo(Canvas Canvas,int FontReduction,int HeaderOffsetY,int PlayerBoxSizeY,int BoxSpaceY,int BoxTextOffsetY,int OwnerOffset,int PlayerCount, int NetXPos)
{
	local int i;
	local float XL, NameHeight;

	Canvas.Font = GetSmallFontFor(Canvas.ClipX, FontReduction);
	Canvas.StrLen("TEST",XL,NameHeight);
	
	// draw time and ping
	if (Canvas.ClipX < 512)
		return;
	else
		PingText = Default.PingText;

	if (((FPHTime == 0) || (!FOPlayer(Owner).bDisplayLoser && !FOPlayer(Owner).bDisplayWinner))
		&& (GRI.ElapsedTime > 0) )
		FPHTime = GRI.ElapsedTime;

	Canvas.Font = GetSmallFontFor(Canvas.ClipX, FontReduction+1);

	for (i=0;i<PlayerCount;i++)
	{
		if (PRIArray[i].bOutOfLives || FragOps_PRI(PRIArray[i]).bEscaped)
			Canvas.DrawColor = HUDClass.default.GrayColor * 0.8;				
		else
			Canvas.DrawColor = HUDClass.default.GrayColor;
		Canvas.SetPos(NetXPos, ((PlayerBoxSizeY + BoxSpaceY)*i + BoxTextOffsetY)+Canvas.ClipY*(0.005-(float(FontReduction)*0.002)));
		Canvas.DrawText(PingText@Min(999,4*PRIArray[i].Ping),true);
		Canvas.SetPos(NetXPos, 	(PlayerBoxSizeY + BoxSpaceY)*i + BoxTextOffsetY + (NameHeight*1.1));
		Canvas.DrawText(FormatTime(Max(0,FPHTime - PRIArray[i].StartTime)),true);
	}
	if ((OwnerOffset >= PlayerCount))
	{	
		if (PRIArray[OwnerOffset].bOutOfLives || FragOps_PRI(PRIArray[OwnerOffset]).bEscaped)
			Canvas.DrawColor = HUDClass.default.GrayColor * 0.8;				
		else
			Canvas.DrawColor = HUDClass.default.GrayColor;
		Canvas.SetPos(NetXPos, ((PlayerBoxSizeY + BoxSpaceY)*i + BoxTextOffsetY)+Canvas.ClipY*(0.005-(float(FontReduction)*0.002)));
		Canvas.DrawText(PingText@Min(999,4*PRIArray[OwnerOffset].Ping),true);
		Canvas.SetPos(NetXPos, 	(PlayerBoxSizeY + BoxSpaceY)*i + BoxTextOffsetY + (NameHeight*1.1));
		Canvas.DrawText(FormatTime(Max(0,FPHTime - PRIArray[OwnerOffset].StartTime)),true);
	}
}

defaultproperties
{
	HUDClass=class'Interface_MainHUD'
	ScoreBoardU=Material'MenuEffects.ScoreboardU_FB'
	ScoreBack=Material'MenuEffects.ScoreIcon_FB'
	TeamBoxMaterial(0)=Material'FOMisc.Special.WhiteFlash'
	TeamBoxMaterial(1)=Material'FOMisc.Special.WhiteFlash'
	CarriedObjectiveIcon=Texture'FOMisc.HUD.ObjectiveIcon'
	TeamColors(0)=(R=51,G=20,B=20,A=135)
	TeamColors(1)=(R=20,G=24,B=45,A=135)
	DarkenColor=(R=0,B=0,G=0,A=60)
	ScoreText(0)=Texture'FOMisc.Menu.Merc_ScoreText'
	ScoreText(1)=Texture'FOMisc.Menu.SO_ScoreText'
	MedalText=Texture'FOMisc.Menu.MedalText'
	FragLimit="SCORE LIMIT:"
	TimeLimit="MAP TIME: "
	RoundTime="ROUND TIME: "
	PingText="P:"
	OutFireText="You were killed!  FIRE to view other players."
	EscapeFireText="You escaped!  FIRE to view other players."
	MedalMaterial(0)=Texture'FOMedals.Novice.SurvivorMedal_Small'
	MedalMaterial(1)=Texture'FOMedals.Novice.PistolMarksman_Small'
	MedalMaterial(2)=Texture'FOMedals.Advanced.PistolSharpShooter_Small'
	MedalMaterial(3)=Texture'FOMedals.Advanced.PistolExpert_Small'
	MedalMaterial(4)=Texture'FOMedals.Novice.ArmyCross_Small'
	MedalMaterial(5)=Texture'FOMedals.Advanced.SilverStar_Small'
	MedalWidth(0)=32
	MedalWidth(1)=19
	MedalWidth(2)=19
	MedalWidth(3)=19
	MedalWidth(4)=32
	MedalWidth(5)=28
	MedalMaterialBig(0)=Texture'FOMedals.Novice.SurvivorMedal'
	MedalMaterialBig(1)=Texture'FOMedals.Novice.PistolMarksman'
	MedalMaterialBig(2)=Texture'FOMedals.Advanced.PistolSharpShooter'
	MedalMaterialBig(3)=Texture'FOMedals.Advanced.PistolExpert'
	MedalMaterialBig(4)=Texture'FOMedals.Novice.ArmyCross'
	MedalMaterialBig(5)=Texture'FOMedals.Advanced.SilverStar'
	MedalNames(0)="Survivor"
	MedalNames(1)="P. Marksman"
	MedalNames(2)="P. SharpShooter"
	MedalNames(3)="P. Expert"
	MedalNames(4)="Army Cross"
	MedalNames(5)="Silver Star"
	MedalPrizes(0)="$300 Bonus"
	MedalPrizes(1)="$200 Bonus"
	MedalPrizes(2)="$300 Bonus"
	MedalPrizes(3)="Golden Eagle Unlock!"
	MedalPrizes(4)="$1000 Bonus"
	MedalPrizes(5)="+1 AP Bonus"
}