//=============================================================================
// Frag.Ops  2003 Matt 'SquirrelZero' Farber
//=============================================================================
class Interface_TabGameSettings extends UT2k4Tab_GameSettings;

var automated moComboBox WeaponHand;
var localized string HandNames[3];
var() int iHand, iHandD;

function InitComponent(GUIController MyController, GUIComponent MyOwner)
{
	local int i;

	Super(Settings_Tabs).InitComponent(MyController, MyOwner);
	if (class'GameInfo'.Default.bAlternateMode )
		RemoveComponent(co_GoreLevel);
	else
	{
		for (i=0;i<ArrayCount(class'GameInfo'.default.GoreLevelText);i++)
			co_GoreLevel.AddItem(class'GameInfo'.default.GoreLevelText[i]);
	}

	LastGameOption = ch_LandShake;

	// Network
	for (i=0;i<ArrayCount(NetSpeedText);i++)
		co_Netspeed.AddItem(NetSpeedText[i]);

	ed_Name.MyEditBox.bConvertSpaces = true;
	ed_Name.MyEditBox.MaxWidth=14;

	ed_Password.MyEditBox.MaxWidth=14;

	ed_Password.MaskText(true);
	l_ID.Caption = FormatID(PlayerOwner().GetPlayerIDHash());
	LastGameOption = WeaponHand;
	WeaponHand.AddItem(HandNames[0]);
	WeaponHand.AddItem(HandNames[1]);
	WeaponHand.AddItem(HandNames[2]);
	WeaponHand.ReadOnly(true);
}

function InternalOnLoadINI(GUIComponent Sender, string s)
{
	local int i;
    local PlayerController PC;

    if (GUIMenuOption(Sender) != None)
    {
        PC = PlayerOwner();

        switch (GUIMenuOption(Sender))
        {
            case ch_AutoSwitch:
                bAuto = !PC.bNeverSwitchOnPickup;
                ch_AutoSwitch.Checked(bAuto);
                break;

            case ch_WeaponBob:
            	if ( PC.Pawn != None )
            		bBob = PC.Pawn.bWeaponBob;
            	else bBob = class'Pawn'.default.bWeaponBob;
                ch_WeaponBob.Checked(bBob);
                break;

            case co_GoreLevel:
            	if (PC.Level.Game != None)
            		iGore = PC.Level.Game.GoreLevel;
                else iGore = class'GameInfo'.default.GoreLevel;
                co_GoreLevel.SetIndex(iGore);
                break;

            case ch_Dodging:
                bDodge = class'Effect_BulletHit'.Default.bShowTracers;
                ch_Dodging.Checked(bDodge);
                break;

            case ch_AutoAim:
                bAim = class'FOPawn'.Default.bBodiesStay;
                ch_AutoAim.Checked(bAim);
                break;

            case ch_ClassicTrans:
       		bClassicTrans = class'FOPlayer'.Default.bAutoBriefing;
                ch_ClassicTrans.Checked(bClassicTrans);
                break;

			case ch_LandShake:
				bLandShake = PC.bLandingShake;
				ch_LandShake.Checked(bLandShake);
				break;

		case WeaponHand:
			iHand = PC.Handedness;
			iHandD = iHand;
			if (iHandD == -1)
				WeaponHand.SetIndex(0);
			else if (iHandD == 1)
				WeaponHand.SetIndex(1);
			else
				WeaponHand.SetIndex(2);
			break;

            case ch_Speech:
            	bSpeechRec = bool(PC.ConsoleCommand("get ini:Engine.Engine.ViewportManager UseSpeechRecognition"));
            	ch_Speech.SetComponentValue(bSpeechRec, True);
            	break;

        // Network
        	case ch_Precache:
        		bPrecache = PC.Level.bDesireSkinPreload;
        		ch_Precache.Checked(bPrecache);
        		break;

            case co_Netspeed:
            	if ( PC.Player != None )
            		i = PC.Player.ConfiguredInternetSpeed;
            	else i = class'Player'.default.ConfiguredInternetSpeed;

                if (i <= 2600)
                    iNetSpeed = 0;

                else if (i <= 5000)
                    iNetSpeed = 1;

                else if (i <= 10000)
                    iNetSpeed = 2;

                else iNetSpeed = 3;

				iNetSpeedD = iNetSpeed;
                co_NetSpeed.SetIndex(iNetSpeed);
                break;

            case ed_Name:
                sName = PC.StatsUserName;
                ed_Name.SetText(sName);
                break;

            case ed_Password:
                sPassword = PC.StatsPassword;
                ed_Password.SetText(sPassword);
                break;

            case ch_TrackStats:
            	bStats = PC.bEnableStatsTracking;
                ch_TrackStats.Checked(bStats);
                UpdateStatsItems();
                break;

            case ch_DynNetspeed:
                bDynNet = PC.bDynamicNetSpeed;
                ch_DynNetspeed.Checked(bDynNet);
                break;

        }
    }
}

function SaveSettings()
{
    local PlayerController PC;
    local bool bSave;

	Super.SaveSettings();
    PC = PlayerOwner();

    if ( bSpeechRec != bool(PC.ConsoleCommand("get ini:Engine.Engine.ViewportManager UseSpeechRecognition")) )
    	PC.ConsoleCommand("set ini:Engine.Engine.ViewportManager UseSpeechRecognition"@bSpeechRec);


	if (class'FOPlayer'.Default.bAutoBriefing != bClassicTrans)
	{
		class'FOPlayer'.Default.bAutoBriefing = bClassicTrans;
		class'FOPlayer'.static.StaticSaveConfig();
	}

	if (class'Effect_BulletHit'.Default.bShowTracers != bDodge)
	{
		class'Effect_BulletHit'.Default.bShowTracers = bDodge;
		class'Effect_BulletHit'.static.StaticSaveConfig();
	}

    if (PC.bNeverSwitchOnPickup == bAuto)
    {
        PC.bNeverSwitchOnPickup = !bAuto;
        bSave = True;
    }

	if (class'FOPawn'.Default.bBodiesStay != bAim)
	{
		class'FOPawn'.Default.bBodiesStay = bAim;
		class'FOPawn'.static.StaticSaveConfig();
	}

    if (PC.Pawn != None)
    {

        PC.Pawn.bWeaponBob = bBob;
        PC.Pawn.SaveConfig();
    }
    else if (class'Engine.Pawn'.default.bWeaponBob != bBob)
    {
        class'Engine.Pawn'.default.bWeaponBob = bBob;
        class'Engine.Pawn'.static.StaticSaveConfig();
    }

    if (PC.Level != None && PC.Level.Game != None)
    {
    	if ( PC.Level.Game.GoreLevel != iGore )
    	{
	        PC.Level.Game.GoreLevel = iGore;
	        PC.Level.Game.SaveConfig();
	    }
    }
    else
    {
		if ( class'Engine.GameInfo'.default.GoreLevel != iGore )
		{
	        class'Engine.GameInfo'.default.GoreLevel = iGore;
	        class'Engine.GameInfo'.static.StaticSaveConfig();
	    }
    }

	// Network
	if ( bPrecache != PC.Level.bDesireSkinPreload )
	{
		PC.Level.bDesireSkinPreload = bPrecache;
		PC.Level.SaveConfig();
	}

	if ( iNetSpeed != iNetSpeedD || (class'Player'.default.ConfiguredInternetSpeed == 9636) )
	{
		if ( PC.Player != None )
		{
			switch (iNetSpeed)
			{
				case 0: PC.Player.ConfiguredInternetSpeed = 2600; break;
				case 1: PC.Player.ConfiguredInternetSpeed = 5000; break;
				case 2: PC.Player.ConfiguredInternetSpeed = 10000; break;
				case 3: PC.Player.ConfiguredInternetSpeed = 20000; break;
			}

			PC.Player.SaveConfig();
		}

		else
		{
			switch (iNetSpeed)
			{
				case 0: class'Player'.default.ConfiguredInternetSpeed = 2600; break;
				case 1: class'Player'.default.ConfiguredInternetSpeed = 5000; break;
				case 2: class'Player'.default.ConfiguredInternetSpeed = 10000; break;
				case 3: class'Player'.default.ConfiguredInternetSpeed = 15000; break;
			}

			class'Player'.static.StaticSaveConfig();
		}
	}

	if ( bStats != PC.bEnableStatsTracking )
	{
		PC.bEnableStatsTracking = bStats;
		bSave = True;
	}

	if ( sName != PC.StatsUserName )
	{
		PC.StatsUserName = sName;
		bSave = True;
	}

	if ( PC.StatsPassword != sPassword )
	{
		PC.StatsPassword = sPassword;
		bSave = True;
	}

	if ( PC.bDynamicNetSpeed != bDynNet )
	{
		PC.bDynamicNetSpeed = bDynNet;
		bSave = True;
	}


    if (bSave)
        PC.SaveConfig();
}

function ResetClicked()
{
    local class<Client> ViewportClass;
    local bool bTemp;
    local PlayerController PC;
    local int i;

    Super.ResetClicked();

	PC = PlayerOwner();
    ViewportClass = class<Client>(DynamicLoadObject(GetNativeClassName("Engine.Engine.ViewportManager"), class'Class'));

    ViewportClass.static.ResetConfig("UseSpeechRecognition");
    ViewportClass.static.ResetConfig("ScreenFlashes");
    class'XGame.xPlayer'.static.ResetConfig("bClassicTrans");

    PC.ResetConfig("bNeverSwitchOnPickup");
    class'Effect_BulletHit'.static.ResetConfig("bShowTracers");
	class'Controller'.static.ResetConfig("Handedness");
	class'FOPawn'.static.ResetConfig("bBodiesStay");

    class'Engine.Pawn'.static.ResetConfig("bWeaponBob");
    class'Engine.GameInfo'.static.ResetConfig("GoreLevel");

    // Network
    class'Engine.Player'.static.ResetConfig("ConfiguredInternetSpeed");
    class'Engine.LevelInfo'.static.ResetConfig("bDesireSkinPreload");
    PC.ResetConfig("bEnableStatsTracking");
    PC.ClearConfig("StatsUserName");
    PC.ClearConfig("StatsPassword");
    PC.ResetConfig("bDynamicNetSpeed");

    bTemp = Controller.bCurMenuInitialized;
    Controller.bCurMenuInitialized = False;

    for (i = 0; i < Components.Length; i++)
        Components[i].LoadINI();

    Controller.bCurMenuInitialized = bTemp;
}

function InternalOnChange(GUIComponent Sender)
{
    local PlayerController PC;

	Super.InternalOnChange(Sender);
    if (GUIMenuOption(Sender) != None)
    {
        PC = PlayerOwner();

        switch (GUIMenuOption(Sender))
        {
            case ch_Speech:
            	bSpeechRec = ch_Speech.IsChecked();
            	break;

            case ch_AutoSwitch:
                bAuto = ch_AutoSwitch.IsChecked();
                break;

            case ch_WeaponBob:
                bBob = ch_WeaponBob.IsChecked();
                break;

            case co_GoreLevel:
                iGore = co_GoreLevel.GetIndex();
                break;

            case ch_Dodging:
                bDodge = ch_Dodging.IsChecked();
                break;

            case ch_AutoAim:
                bAim = ch_AutoAim.IsChecked();
                break;

            case ch_ClassicTrans:
                bClassicTrans = ch_ClassicTrans.IsChecked();
                break;

		case WeaponHand:
			iHand = WeaponHand.GetIndex();
			if (iHand != iHandD)
			{
				if (iHand == 0)
				{
					PC.Handedness = -1;
					PC.SetHand(-1);
				}
				else if (iHand == 1)
				{
					PC.Handedness = 1;
					PC.SetHand(1);
				}
				else
				{
					PC.Handedness = 2;
					PC.SetHand(2);
				}
				PC.SaveConfig();
			}
			break;

		// Network
        	case ch_Precache:
        		bPrecache = ch_Precache.IsChecked();
        		break;

            case co_Netspeed:
                iNetSpeed = co_NetSpeed.GetIndex();
                break;

            case ed_Name:
                sName = ed_Name.GetText();
                break;

            case ed_Password:
                sPassword = ed_Password.GetText();
                break;

            case ch_TrackStats:
            	bStats = ch_TrackStats.IsChecked();
                UpdateStatsItems();
                break;

            case ch_DynNetspeed:
                bDynNet = ch_DynNetspeed.IsChecked();
                break;

        }
    }

    l_Warning.SetVisibility(!ValidStatConfig());
}

function ShowPanel(bool bShow)
{
	Super.ShowPanel(bShow);
	if (bShow)
	{
		if (bInit)
		{
			i_BG1.ManageComponent(ch_WeaponBob);
			i_BG1.Managecomponent(ch_AutoSwitch);
			i_BG1.ManageComponent(ch_Dodging);
			i_BG1.ManageComponent(ch_AutoAim);
			i_BG1.ManageComponent(ch_ClassicTrans);
			i_BG1.ManageComponent(WeaponHand);
			i_BG1.Managecomponent(co_GoreLevel);

			// No speech recognition except on win32... --ryan.
			if ((!PlatformIsWindows()) || (PlatformIs64Bit()))
				ch_Speech.DisableMe();
		}
			UpdateStatsItems();
	}
}

defaultproperties
{
	Begin Object class=moCheckBox Name=GameDodging
		WinWidth=0.400000
		WinLeft=0.050000
		WinTop=0.541563
		Caption="Bullet Tracers"
		INIOption="@Internal"
		OnLoadINI=InternalOnLoadINI
		OnChange=InternalOnChange
		Hint="Check to enable ingame bullet tracers"
		ComponentJustification=TXTA_Right
		LabelJustification=TXTA_Left
		ComponentWidth=-1
		CaptionWidth=0.8
		RenderWeight=1.04
		TabOrder=3
	End Object
	ch_Dodging=GameDodging

	Begin Object class=moCheckBox Name=GameAutoAim
		WinWidth=0.400000
		WinLeft=0.050000
		WinTop=0.692344
		Caption="Bodies Stay"
		INIOption="@Internal"
		OnLoadINI=InternalOnLoadINI
		OnChange=InternalOnChange
		Hint="Uncheck to make dead bodies disappear after a short time"
		ComponentJustification=TXTA_Right
		LabelJustification=TXTA_Left
		ComponentWidth=-1
		CaptionWidth=0.8
		RenderWeight=1.04
		TabOrder=4
	End Object
	ch_AutoAim=GameAutoAim

	Begin Object class=moCheckBox Name=GameClassicTrans
		Caption="Auto Briefing"
		IniOption="@Internal"
		OnChange=InternalOnChange
		OnLoadINI=InternalOnLoadINI
		Hint="Check to enable automatic briefing screen when joining team"
		bSquare=True
		ComponentJustification=TXTA_Right
		LabelJustification=TXTA_Left
		ComponentWidth=-1
		CaptionWidth=0.8
		RenderWeight=1.04
		TabOrder=5
	End Object
	ch_ClassicTrans=GameClassicTrans

	Begin Object class=moCheckBox Name=LandShaking
		WinWidth=0.266797
		WinLeft=0.705430
		WinTop=1.5
		Caption="Landing Viewshake"
		OnLoadINI=InternalOnLoadINI
		OnChange=InternalOnChange
		INIOption="@Internal"
		Hint="Enable view shaking upon landing."
		CaptionWidth=0.9
		bSquare=True
		bHeightFromComponent=False
		ComponentJustification=TXTA_Right
		LabelJustification=TXTA_Left
		bAcceptsInput=false
		bVisible=false
	End Object
	ch_LandShake=LandShaking

	Begin Object class=moComboBox Name=WeaponHandBox
		WinWidth=0.266797
		WinLeft=0.705430
		WinTop=0.150261
		Caption="Weapon Hand"
		INIOption="@Internal"
		OnLoadINI=InternalOnLoadINI
		OnChange=InternalOnChange
		Hint="Configure the hand you hold your weapon in"
		ComponentJustification=TXTA_Right
		LabelJustification=TXTA_Left
		ComponentWidth=-1
		CaptionWidth=0.5
		RenderWeight=1.04
		TabOrder=7
		bReadOnly=True
	End Object
	WeaponHand=WeaponHandBox

	HandNames(0)="Left"
	HandNames(1)="Right"
	HandNames(2)="Hidden"
}