//=============================================================================
// Frag.Ops  2003 Matt 'SquirrelZero' Farber
//=============================================================================
class Interface_TabPlayerTeam extends MidGamePanel
	config(User);

struct SaveRecord
{
	var config string SaveName;
	var config byte GroupType;
	var config byte SoldierType;
	var config byte BodyType;
	var config byte Trait;
	var config byte S_Structures, S_Mechanic, S_Weapons;
};

// controls
var automated GUIListBox CharacterList;
var automated GUISectionBackground CharacterName, ModelBG;
var automated GUIButton DeleteButton, SaveButton, JoinButton, JoinRandomButton, SOButton, MercButton;
var automated moEditBox PlayerSave;
var automated moComboBox PlayerLoad;
var automated GUIButton HeadLeft, HeadRight, BodyLeft, BodyRight;

// trait system
var automated GUISectionBackground TraitBG;
var automated GUIListBox TraitList;
var automated GUIScrollTextBox TraitScroll, TeamDescription;
var const array<class<FOTraitRecord> > Traits;
var byte SelectedTrait;

// configurable save records
var config SaveRecord SOFavorite[10], MercFavorite[10];

// the dynamic array of records kept
var const array<FOPlayerRecord> MercRecord, SORecord;

// everything else
var string EmptyString, SavedString, DeletedString, ListFullString, NameString, CustomString;
var bool bRenderDude, bSaved, bDeleted, bListFull, bAwaitingInitialization, bJoinClicked;
var string MercChar[4], SOChar[4], MercDesc[4], SODesc[4];
var byte SelectedTeam, SelectedRecord, HeadSkinCount, BodySkinCount, NewIndex;
var color DarkenColor, BrightenColor;
var Interface_TeamMenuDude MenuDude;

function InitComponent(GUIController MyController, GUIComponent MyOwner)
{
	local GameReplicationInfo GRI;
	local int SOPlayers, MercPlayers, i;

	bAwaitingInitialization = true;
	Super.InitComponent(MyController,MyOwner);

	HeadSkinCount = 0;
	BodySkinCount = 0;

	BuildTraitList();

	// new -- choose initial team based on players for easy balance
	if (PlayerOwner().GameReplicationInfo != None)
		GRI = PlayerOwner().GameReplicationInfo;

	if ((PlayerOwner().PlayerReplicationInfo == None || PlayerOwner().PlayerReplicationInfo.Team == None) && GRI != None)
	{
		for (i=0;i<GRI.PRIArray.Length;i++)
		{
			if (GRI.PRIArray[i].Team != None && GRI.PRIArray[i].Team.TeamIndex == 0)
			{
				MercPlayers++;
			}
			else if (GRI.PRIArray[i].Team != None && GRI.PRIArray[i].Team.TeamIndex == 1)
			{
				SOPlayers++;
			}
		}	
		if (MercPlayers < SOPlayers)
			LoadTeamLists(MercButton);
		else if (SOPlayers < MercPlayers)
			LoadTeamLists(SOButton);		
		else if (FRand() < 0.5)
			LoadTeamLists(MercButton);
		else
			LoadTeamLists(SOButton);
	}	
	else
		LoadTeamLists(MercButton);
	// -

	PlayerSave.MyEditBox.OnClick = ClearSave;
	PlayerLoad.ReadOnly(true);
	
	bSaved = false;
	bDeleted = false;
	bListFull = false;

	CharacterList.List.OnChange = ListClick;
	CharacterList.List.TextAlign = TXTA_Left;
	TraitList.List.OnChange = TraitClick;

	PlayerLoad.OnChange = InternalOnChange;
	SetTimer(0.03,false);
	bJoinClicked = false;
}

function BuildTraitList()
{
	local int i;
	
	TraitList.List.OnChange = NullChange;
	TraitList.List.Clear();
	TraitList.List.Add("None",None,"Play with no additional traits.");

	for (i=0;i<Traits.Length;i++)
	{
		TraitList.List.Add(Traits[i].Default.TraitName,None,Traits[i].Default.Description);
	}
	TraitList.List.OnChange = TraitClick;
}

function TraitClick(GUIComponent Sender)
{
	TraitScroll.SetContent(TraitList.List.GetExtra());
	SelectedTrait = TraitList.List.Index-1;
	if (TraitList.List.Get() ~= "None")
		SelectedTrait = 255;
}

// dude rendering
function InternalRendered(Canvas canvas)
{
	if (bRenderDude && MenuDude != None && Controller.bCurMenuInitialized && !bAwaitingInitialization)
	{
		MenuDude.TabOwner = self;
		MenuDude.GUICanvas = canvas;
		Canvas.DrawActor(None,false,true);
		Canvas.DrawActor(MenuDude,false,false,90.0);
		if (MenuDude.AttachedWeapon != None)
			Canvas.DrawActor(MenuDude.AttachedWeapon,false,false,90.0);
	}
}

function Timer()
{
	if (Controller == None || (Controller != None && !Controller.bCurMenuInitialized))
	{
		SetTimer(0.03,false);
		return;
	}
	else if (bAwaitingInitialization)
	{
		MenuDude = PlayerOwner().spawn(class'FragOps.Interface_TeamMenuDude');
		MenuDude.SetRotation(PlayerOwner().Rotation);
		MenuDude.bPlayRandomAnims = false;
		MenuDude.SpinRate = 12000;
		bAwaitingInitialization = false;
		SetupTeamDude();
	}
	
	if (bListFull)
	{
		bDeleted = false;
		if (PlayerSave.GetText() ~= DeletedString)
			PlayerSave.SetText(NameString);
		return;
	}
	else if (bDeleted)
	{
		bDeleted = false;
		if (PlayerSave.GetText() ~= DeletedString)
			PlayerSave.SetText(NameString);
		return;
	}
	else if (bSaved)
	{
		bSaved = false;
		if (PlayerSave.GetText() ~= SavedString)
			PlayerSave.SetText(NameString);
		return;
	}		
}

function ListClick(GUIComponent Sender)
{
	local int i;

	i = CharacterList.List.Index;

	// change record selection
	SelectedRecord = i;

	if (SelectedTeam == 1 && SelectedRecord < ArrayCount(SODesc))
		TeamDescription.SetContent(SODesc[SelectedRecord]);
	else if (SelectedRecord < ArrayCount(MercDesc))
		TeamDescription.SetContent(MercDesc[SelectedRecord]);
	HeadSkinCount = 0;
	BodySkinCount = 0;

	// turn off model rendering
	bRenderDude = false;

	// apply settings to model mockup
	SetupTeamDude();

	// set to custom
	PlayerLoad.MyComboBox.Edit.SetText(CustomString);

	// set caption on model box
	ModelBG.Caption = CharacterList.List.Get();
}

function bool LoadTeamLists(GUIComponent Sender)
{
	local int i, RandFavorite;

	CharacterList.List.Clear();

	if (Sender == MercButton)
	{
		SelectedTeam = 0;
		for (i=0;i<ArrayCount(MercChar);i++)
		{
			if (MercChar[i] != "")
				CharacterList.List.Add(MercChar[i],MercRecord[i]);
		}
	}
	else
	{
		SelectedTeam = 1;
		for (i=0;i<ArrayCount(SOChar);i++)
		{
			if (SOChar[i] != "")
				CharacterList.List.Add(SOChar[i],SORecord[i]);
		}
	}

	// reset
	SelectedTrait = 255;
	SelectedRecord = 0;
	HeadSkinCount = 0;
	BodySkinCount = 0;

	// rebuild the list
	RebuildList(SelectedTeam);

	// pick a random favorite
	switch(SelectedTeam)
	{
		case 1:
			CharacterName.Caption = "Special Ops";
			for(i=0;i<ArrayCount(SOFavorite);i++)
			{
				if (SOFavorite[i].SaveName ~= EmptyString)
					break;
			}
			if (i == 0)
			{
				CharacterList.List.OnChange = NullChange;
				CharacterList.List.SetIndex(0);
				CharacterList.List.OnChange = ListClick;
				PlayerLoad.MyComboBox.Edit.SetText(CustomString);
				break;
			}

			RandFavorite = Rand(i);
			SelectedRecord = Clamp(SOFavorite[RandFavorite].GroupType, 0, SORecord.Length-1);
			HeadSkinCount = Clamp(SOFavorite[RandFavorite].SoldierType, 0, SORecord[SelectedRecord].HeadSkin.Length-1);
			BodySkinCount = Clamp(SOFavorite[RandFavorite].BodyType, 0, SORecord[SelectedRecord].BodySkin.Length-1);
			if (SOFavorite[RandFavorite].Trait < Traits.Length)
				SelectedTrait = SOFavorite[RandFavorite].Trait;
			else
				SelectedTrait = 255;

			// set the combobox index to our chosen favorite
			PlayerLoad.OnChange = NullChange;
			PlayerLoad.SetIndex(RandFavorite);
			PlayerLoad.OnChange = InternalOnChange;			
			CharacterList.List.OnChange = NullChange;
			CharacterList.List.SetIndex(SelectedRecord);
			CharacterList.List.OnChange = ListClick;
			break;

		default:
			CharacterName.Caption = "Mercenaries";
			for(i=0;i<ArrayCount(MercFavorite);i++)
			{
				if (MercFavorite[i].SaveName ~= EmptyString)
					break;
			}
			if (i == 0)
			{
				CharacterList.List.OnChange = NullChange;
				CharacterList.List.SetIndex(SelectedRecord);
				CharacterList.List.OnChange = ListClick;
				PlayerLoad.MyComboBox.Edit.SetText(CustomString);
				break;
			}

			RandFavorite = Rand(i);
			SelectedRecord = Clamp(MercFavorite[RandFavorite].GroupType, 0, MercRecord.Length-1);
			HeadSkinCount = Clamp(MercFavorite[RandFavorite].SoldierType, 0, MercRecord[SelectedRecord].HeadSkin.Length-1);
			BodySkinCount = Clamp(MercFavorite[RandFavorite].BodyType, 0, MercRecord[SelectedRecord].BodySkin.Length-1);
			if (MercFavorite[RandFavorite].Trait < Traits.Length)
				SelectedTrait = MercFavorite[RandFavorite].Trait;
			else
				SelectedTrait = 255;

			// set the combobox index to our chosen favorite
			PlayerLoad.OnChange = NullChange;
			PlayerLoad.SetIndex(RandFavorite);
			PlayerLoad.OnChange = InternalOnChange;
			CharacterList.List.OnChange = NullChange;
			CharacterList.List.SetIndex(SelectedRecord);
			CharacterList.List.OnChange = ListClick;
			break;
	}

	// reset name box 
	PlayerSave.SetText(NameString);	

	// set caption on model box
	ModelBG.Caption = CharacterList.List.Get();

	// turn off model rendering
	bRenderDude = false;

	// set traitlist index
	if (SelectedTrait == 255)
		TraitList.List.SetIndex(0);
	else
		TraitList.List.SetIndex(SelectedTrait+1);

	if (SelectedTeam == 1 && SelectedRecord < ArrayCount(SODesc))
		TeamDescription.SetContent(SODesc[SelectedRecord]);
	else if (SelectedRecord < ArrayCount(MercDesc))
		TeamDescription.SetContent(MercDesc[SelectedRecord]);

	// set box content
	TraitScroll.SetContent(TraitList.List.GetExtra());
	
	// apply settings to model mockup
	SetupTeamDude();
	return false;
}

function SetupTeamDude()
{
	if (MenuDude == None || bAwaitingInitialization) 
		return;

	if (FOPlayerRecord(CharacterList.List.GetObject()) == None)
		return;

	if (HeadSkinCount >= FOPlayerRecord(CharacterList.List.GetObject()).HeadSkin.Length || BodySkinCount >= FOPlayerRecord(CharacterList.List.GetObject()).BodySkin.Length)
		return;

	// link to to new mesh
	MenuDude.LinkMesh(FOPlayerRecord(CharacterList.List.GetObject()).AssociatedModel[HeadSkinCount]);

	// apply skins
	MenuDude.Skins[0] = FOPlayerRecord(CharacterList.List.GetObject()).BodySkin[BodySkinCount];
	MenuDude.Skins[1] = FOPlayerRecord(CharacterList.List.GetObject()).HeadSkin[HeadSkinCount];
	MenuDude.Skins[2] = FOPlayerRecord(CharacterList.List.GetObject()).GlassesSkin[HeadSkinCount];
	MenuDude.Skins[3] = FOPlayerRecord(CharacterList.List.GetObject()).HelmetSkin[HeadSkinCount];
	MenuDude.Skins[4] = FOPlayerRecord(CharacterList.List.GetObject()).MiscSkin[HeadSkinCount];

	// adjust scale
	MenuDude.SetDrawScale(0.73);
	MenuDude.CenterWeaponOffset = vect(150, 23, 4.6);

	// old" (X=150,Y=51.5,Z=4.7)

	// attach a weapon
	MenuDude.AttachToBone(MenuDude.AttachedWeapon,'righthand');

	// refresh bones
	MenuDude.BoneRefresh();

	// loop an idle anim
	MenuDude.LoopAnim('Idle_Rest', 1.0);
	bRenderDude = true;
}

function bool JoinClick(GUIComponent Sender)
{
	bJoinClicked = true;
	if (!FOPlayer(PlayerOwner()).CriticalReplicationInfo.bChangedTeams)
	{
		if (PlayerOwner().PlayerReplicationInfo.Team == None || SelectedTeam != PlayerOwner().PlayerReplicationInfo.Team.TeamIndex)
			FOPlayer(PlayerOwner()).bOpenBriefing = true;
		FOPlayer(PlayerOwner()).GetNewTeam(SelectedTeam,SelectedRecord,BodySkinCount,HeadSkinCount,SelectedTrait);
	}

	if (Interface_LoginMenu(GUIController(PlayerOwner().Player.GUIController).ActivePage) != None)
		Controller.CloseMenu();
	return true;
}

function bool JoinRandomClick(GUIComponent Sender)
{
	local int MercPlayers, SOPlayers, i;
	local float RandSeed;
	local GameReplicationInfo GRI;
	local FOPlayerRecord RandRecord;

	if (PlayerOwner().GameReplicationInfo != None)
		GRI = PlayerOwner().GameReplicationInfo;

	SelectedRecord = 0;
	if (!FOPlayer(PlayerOwner()).CriticalReplicationInfo.bChangedTeams)
	{
		for (i=0;i<GRI.PRIArray.Length;i++)
		{
			if (GRI.PRIArray[i].Team != None && GRI.PRIArray[i].Team.TeamIndex == 0)
				MercPlayers++;
			else if (GRI.PRIArray[i].Team != None && GRI.PRIArray[i].Team.TeamIndex == 1)
				SOPlayers++;
		}
		RandSeed = FRand();
		if (SOPlayers > MercPlayers)
		{
			SelectedTeam = 0;
			RandRecord = MercRecord[SelectedRecord];
		}
		else
		{
			if (SOPlayers == MercPlayers)
			{
				if (FRand() < 0.5)
				{
					SelectedTeam = 0;
					RandRecord = MercRecord[SelectedRecord];
				}
				else
				{
					SelectedTeam = 1;
					RandRecord = SORecord[SelectedRecord];
				}
			}
			else
			{
					SelectedTeam = 1;
					RandRecord = SORecord[SelectedRecord];
			}
		}

		// pick a random head/bodyskin
		HeadSkinCount = Rand(RandRecord.HeadSkin.Length);
		BodySkinCount = Rand(RandRecord.BodySkin.Length);
		
		// now a random trait
		SelectedTrait = Rand(Traits.Length);

		if (PlayerOwner().PlayerReplicationInfo.Team == None || SelectedTeam != PlayerOwner().PlayerReplicationInfo.Team.TeamIndex)
			FOPlayer(PlayerOwner()).bOpenBriefing = true;

		FOPlayer(PlayerOwner()).GetNewTeam(SelectedTeam,SelectedRecord,BodySkinCount,HeadSkinCount,SelectedTrait);
	}

	bJoinClicked = true;

	// close the menu
	if (Interface_LoginMenu(GUIController(PlayerOwner().Player.GUIController).ActivePage) != None)
		Controller.CloseMenu();
	return true;
}

function bool ArrowClick(GUIComponent Sender)
{
	if (Sender == HeadLeft)
	{
		if (HeadSkinCount > 0)
			HeadSkinCount--;
	}
	else if (Sender == HeadRight)
	{
		if (HeadSkinCount < (FOPlayerRecord(CharacterList.List.GetObject()).HeadSkin.Length-1))
			HeadSkinCount++;
	}
	else if (Sender == BodyLeft)
	{
		if (BodySkinCount > 0)
			BodySkinCount--;
	}
	else if (Sender == BodyRight)
	{
		if (BodySkinCount < (FOPlayerRecord(CharacterList.List.GetObject()).BodySkin.Length-1))
			BodySkinCount++;
	}
	else return false;

	// let people know they're making a custom player
	PlayerLoad.MyComboBox.Edit.SetText(CustomString);

	// turn off model rendering
	bRenderDude = false;

	// apply settings to model mockup
	SetupTeamDude();
	return true;
}

function bool ClearSave(GUIComponent Sender)
{
	PlayerSave.SetText("");
	return true;
}

function bool PlayerClick(GUIComponent Sender)
{
	local int i, j;
	
	if (Sender == SaveButton)
	{
		if (PlayerSave.GetText() ~= NameString)
			return false;
		switch(SelectedTeam)
		{
			case 1:
				// check for dupes first
				for (i=0;i<ArrayCount(SOFavorite);i++)
				{
					if (SOFavorite[i].SaveName ~= PlayerSave.GetText())
					{
						SOFavorite[i].SaveName = PlayerSave.GetText();
						SOFavorite[i].GroupType = SelectedRecord;
						SOFavorite[i].SoldierType = HeadSkinCount;
						SOFavorite[i].BodyType = BodySkinCount;
						SOFavorite[i].Trait = SelectedTrait;
						PlayerSave.SetText(SavedString);
						bSaved = true;
						SaveConfig();
						NewIndex = i;
						RebuildList(1);
						SetTimer(2.0, false);
						return true;
					}
				}
				
				// otherwise find an empty slot
				for (i=0;i<ArrayCount(SOFavorite);i++)
				{
					if (SOFavorite[i].SaveName ~= EmptyString)
					{
						SOFavorite[i].SaveName = PlayerSave.GetText();
						SOFavorite[i].GroupType = SelectedRecord;
						SOFavorite[i].SoldierType = HeadSkinCount;
						SOFavorite[i].BodyType = BodySkinCount;
						SOFavorite[i].Trait = SelectedTrait;
						PlayerSave.SetText(SavedString);
						bSaved = true;
						SaveConfig();
						NewIndex = i;
						RebuildList(1);
						SetTimer(2.0, false);
						return true;
					}
				}

				// if we made it here, the list is full
				PlayerSave.SetText(ListFullString);
				bListFull = true;
				SetTimer(2.0, false);
				break;

			default:
				// check for dupes first
				for (i=0;i<ArrayCount(MercFavorite);i++)
				{
					if (MercFavorite[i].SaveName ~= PlayerSave.GetText())
					{
						MercFavorite[i].SaveName = PlayerSave.GetText();
						MercFavorite[i].GroupType = SelectedRecord;
						MercFavorite[i].SoldierType = HeadSkinCount;
						MercFavorite[i].BodyType = BodySkinCount;
						MercFavorite[i].Trait = SelectedTrait;
						PlayerSave.SetText(SavedString);
						bSaved = true;
						SaveConfig();
						NewIndex = i;
						RebuildList(0);
						SetTimer(2.0, false);
						return true;
					}
				}
				
				// otherwise find an empty slot
				for (i=0;i<ArrayCount(MercFavorite);i++)
				{
					if (MercFavorite[i].SaveName ~= EmptyString)
					{
						MercFavorite[i].SaveName = PlayerSave.GetText();
						MercFavorite[i].GroupType = SelectedRecord;
						MercFavorite[i].SoldierType = HeadSkinCount;
						MercFavorite[i].BodyType = BodySkinCount;
						MercFavorite[i].Trait = SelectedTrait;
						PlayerSave.SetText(SavedString);
						bSaved = true;
						SaveConfig();
						NewIndex = i;
						RebuildList(0);
						SetTimer(2.0, false);
						return true;
					}
				}

				// if we made it here, the list is full
				PlayerSave.SetText(ListFullString);
				bListFull = true;
				SetTimer(2.0, false);
				break;		
		}
	}
	else if (Sender == DeleteButton)
	{
		// so we don't get an endless loop
		PlayerLoad.OnChange = NullChange;
		
		// if player clicked an empty box, don't do anything
		if (PlayerLoad.GetItem(PlayerLoad.GetIndex()) ~= EmptyString)
			return true;

		// delete the favorite
		i = PlayerLoad.GetIndex();
		switch(SelectedTeam)
		{
			case 1:
				for(j=i;j<ArrayCount(SOFavorite);j++)
				{
					if (SOFavorite[j+1].SaveName ~= EmptyString)
						break;
					SOFavorite[j] = SOFavorite[j+1];
				}
				SOFavorite[j].SaveName = EmptyString;
				SOFavorite[j].GroupType = 255;
				SOFavorite[j].SoldierType = 255;
				SOFavorite[j].BodyType = 255;
				SOFavorite[j].Trait = 255;
				break;

			default:
				for(j=i;j<ArrayCount(MercFavorite);j++)
				{
					if (MercFavorite[j+1].SaveName ~= EmptyString)
						break;
					MercFavorite[j] = MercFavorite[j+1];
				}
				MercFavorite[j].SaveName = EmptyString;
				MercFavorite[j].GroupType = 255;
				MercFavorite[j].SoldierType = 255;
				MercFavorite[j].BodyType = 255;
				MercFavorite[j].Trait = 255;
				break;
		}
		
		PlayerLoad.RemoveItem(i);
		PlayerLoad.AddItem(EmptyString);
		PlayerLoad.SetIndex(9);
		PlayerSave.SetText(DeletedString);
		bDeleted = true;
		SaveConfig();
		PlayerLoad.OnChange = InternalOnChange;	
		SetTimer(2.0,false);
	}
	
	return true;
}

function RebuildList(byte TeamList)
{
	local int i;

	// no endless loop
	PlayerLoad.OnChange = NullChange;

	// clear the entire list
	PlayerLoad.MyComboBox.List.Clear();

	// rebuild
	switch(SelectedTeam)
	{
		case 1:
			for (i=0;i<ArrayCount(SOFavorite);i++)
				PlayerLoad.AddItem(SOFavorite[i].SaveName);
			break;

		default:
			for (i=0;i<ArrayCount(MercFavorite);i++)
				PlayerLoad.AddItem(MercFavorite[i].SaveName);
			break;
	}

	// set to the new index
	PlayerLoad.SetIndex(NewIndex);
	
	// reset
	PlayerLoad.OnChange = InternalOnChange;
}

singular function NullChange(GUIComponent Sender);

singular function InternalOnChange(GUIComponent Sender)
{
	local int i;

	if (PlayerLoad != Sender)
		return;

	// no endless loop
	PlayerLoad.OnChange = NullChange;

	// if player clicked an empty string, do nothing
	if (PlayerLoad.GetText() ~= EmptyString)
	{
		PlayerLoad.SetIndex(0);

		// reset
		ModelBG.Caption = CharacterList.List.Get();
		PlayerLoad.OnChange = InternalOnChange;
		return;
	}
	else if (PlayerLoad.GetText() ~= CustomString)
	{
		// reset
		ModelBG.Caption = CharacterList.List.Get();
		PlayerLoad.OnChange = InternalOnChange;
		return;
	}

	// retrieve a matching favorite and apply it to model mockup
	switch(SelectedTeam)
	{
		case 1:
			for (i=0;i<ArrayCount(SOFavorite);i++)
			{
				if (PlayerLoad.GetText() ~= SOFavorite[i].SaveName)
				{
					SelectedRecord = Clamp(SOFavorite[i].GroupType, 0, SORecord.Length-1);
					if (SOFavorite[i].Trait < Traits.Length)
						SelectedTrait = SOFavorite[i].Trait;
					else
						SelectedTrait = 255;
					HeadSkinCount = Clamp(SOFavorite[i].SoldierType, 0, SORecord[SelectedRecord].HeadSkin.Length-1);
					BodySkinCount = Clamp(SOFavorite[i].BodyType, 0, SORecord[SelectedRecord].BodySkin.Length-1);
					CharacterList.List.OnChange = NullChange;
					CharacterList.List.SetIndex(SelectedRecord);
					CharacterList.List.OnChange = ListClick;
					ModelBG.Caption = CharacterList.List.Get();
					bRenderDude = false;
					SetupTeamDude();
					break;
				}
			}
			break;

		default:
			for (i=0;i<ArrayCount(MercFavorite);i++)
			{
				if (PlayerLoad.GetText() ~= MercFavorite[i].SaveName)
				{
					SelectedRecord = Clamp(MercFavorite[i].GroupType, 0, MercRecord.Length-1);
					if (MercFavorite[i].Trait < Traits.Length)
						SelectedTrait = MercFavorite[i].Trait;
					else
						SelectedTrait = 255;
					HeadSkinCount = Clamp(MercFavorite[i].SoldierType, 0, MercRecord[SelectedRecord].HeadSkin.Length-1);
					BodySkinCount = Clamp(MercFavorite[i].BodyType, 0, MercRecord[SelectedRecord].BodySkin.Length-1);
					CharacterList.List.OnChange = NullChange;
					CharacterList.List.SetIndex(SelectedRecord);
					CharacterList.List.OnChange = ListClick;
					ModelBG.Caption = CharacterList.List.Get();
					bRenderDude = false;
					SetupTeamDude();
					break;
				}
			}
			break;
	}

	// set traitlist index
	TraitList.List.OnChange = NullChange;
	if (SelectedTrait == 255)
		TraitList.List.SetIndex(0);
	else
		TraitList.List.SetIndex(SelectedTrait+1);
	TraitScroll.SetContent(TraitList.List.GetExtra());
	TraitList.List.OnChange = TraitClick;

	// reset
	PlayerLoad.OnChange = InternalOnChange;
}

function OnDestroyPanel(optional bool bCancelled)	// Always call Super.OnDestroyPanel()
{
	if (MenuDude != None)
		MenuDude.Destroy();

	Super.OnDestroyPanel(bCancelled);
}

defaultproperties
{
	Begin Object Class=GUIListBox Name=CharacterListBox
		WinWidth=0.33
		WinHeight=0.360000
		WinLeft=0.070000
		WinTop=0.200000
		bVisibleWhenEmpty=True
		FontScale=FNS_Medium
		StyleName="NoBackground"
		SelectedStyleName="ListSelection"
		TabOrder=2
	End Object
	CharacterList=CharacterListBox

	Begin Object Class=GUISectionBackground Name=CharacterBG
		WinWidth=0.3961
		WinHeight=0.424078
		WinLeft=0.036600
		LeftPadding=0.01
		RightPadding=0.01
		WinTop=0.133066
		Caption=""
		bFillClient=true
	End Object
	CharacterName=CharacterBG

	Begin Object Class=GUISectionBackground Name=ModelBox
		WinWidth=0.501
		WinHeight=0.600
		WinLeft=0.462
		LeftPadding=0.01
		RightPadding=0.01
		WinTop=0.133066
		Caption=""
		bFillClient=true
	End Object
	ModelBG=ModelBox

	Begin Object Class=GUIButton Name=MercButtonBox
		Caption="Mercenaries"
		Hint="View Mercenary skins"
		StyleName="SquareButton"
		WinWidth=0.190100
		WinLeft=0.036600
		WinTop=0.053
		TabOrder=1
		bStandardized=true
		bBoundToParent=true
		bScaleToParent=true
		OnClick=LoadTeamLists
	End Object
	MercButton=MercButtonBox

	Begin Object Class=GUIButton Name=SOButtonBox
		Caption="Special Ops"
		StyleName="SquareButton"
		Hint="View Special Ops skins"
		WinWidth=0.190100
		WinLeft=0.242039
		WinTop=0.053
		TabOrder=1
		bStandardized=true
		bBoundToParent=true
		bScaleToParent=true
		OnClick=LoadTeamLists
	End Object
	SOButton=SOButtonBox

	Begin Object Class=GUIButton Name=JoinButtonBox
		Caption="JOIN"
		StyleName="SquareButton"
		Hint="Join the game!"
		OnClick=JoinClick
		WinHeight=0.07
		WinWidth=0.140100
		WinLeft=0.652039
		WinTop=0.045
		FontScale=FNS_Medium
		TabOrder=3
		bStandardized=false
		bBoundToParent=true
		bScaleToParent=true
	End Object
	JoinButton=JoinButtonBox

	Begin Object Class=GUIButton Name=RandomButtonBox
		Caption="Random"
		Hint="Join using a random skin.  No trait will be used."
		StyleName="SquareButton"
		OnClick=JoinRandomClick
		WinWidth=0.15
		WinLeft=0.81
		WinTop=0.053
		TabOrder=4
		bStandardized=true
		bBoundToParent=true
		bScaleToParent=true
	End Object
	JoinRandomButton=RandomButtonBox

	Begin Object Class=GUIButton Name=SaveButtonBox
		Caption="SAVE"
		StyleName="SquareButton"
		Hint="Save this player"
		OnClick=PlayerClick
		WinWidth=0.110100
		WinLeft=0.036600
		WinHeight=0.047
		WinTop=0.57
		TabOrder=5
		bStandardized=false
		bBoundToParent=true
		bScaleToParent=true
	End Object
	SaveButton=SaveButtonBox

	Begin Object class=moEditBox Name=SaveBox
		WinWidth=0.2784
		WinLeft=0.1512
		WinTop=0.574
		Hint="Enter a name to save this player as"
		CaptionWidth=0
		TabOrder=6
		bBoundToParent=true
		bScaleToParent=true
	End Object
	PlayerSave=SaveBox

	Begin Object Class=GUIButton Name=DeleteButtonBox
		Caption="DELETE"
		StyleName="SquareButton"
		Hint="Delete a saved player"
		WinWidth=0.138
		WinLeft=0.036600
		WinTop=0.64
		WinHeight=0.047
		TabOrder=7
		bStandardized=false
		bBoundToParent=true
		bScaleToParent=true
		OnClick=PlayerClick
	End Object
	DeleteButton=DeleteButtonBox

	Begin Object class=moComboBox Name=LoadBox
		WinWidth=0.391430
		WinLeft=0.038
		WinTop=0.689
		Hint="Loads a saved player"
		ComponentJustification=TXTA_Left
		ComponentWidth=-1
		CaptionWidth=0
		TabOrder=8
		bReadOnly=True
		bBoundToParent=true
		bScaleToParent=true
		OnChange=InternalOnChange
	End Object
	PlayerLoad=LoadBox

	Begin Object class=GUIButton Name=HeadLeftBox
		StyleName="LeftArrow"
		Hint="Head Skin -"
		WinWidth=0.035
		WinHeight=0.045
		WinLeft=0.52
		WinTop=0.19
		OnClick=ArrowClick
		bNeverFocus=true
		bRepeatClick=true
	End Object
	HeadLeft=HeadLeftBox

	Begin Object class=GUIButton Name=HeadRightBox
		StyleName="RightArrow"
		Hint="Head Skin +"
		WinWidth=0.035
		WinHeight=0.045
		WinLeft=0.625
		WinTop=0.19
		OnClick=ArrowClick
		bNeverFocus=true
		bRepeatClick=true
	End Object
	HeadRight=HeadRightBox

	Begin Object class=GUIButton Name=BodyLeftBox
		StyleName="LeftArrow"
		Hint="Body Skin -"
		WinWidth=0.035
		WinHeight=0.045
		WinLeft=0.52
		WinTop=0.67
		OnClick=ArrowClick
		bNeverFocus=true
		bRepeatClick=true
	End Object
	BodyLeft=BodyLeftBox

	Begin Object class=GUIButton Name=BodyRightBox
		StyleName="RightArrow"
		Hint="Body Skin +"
		WinWidth=0.035
		WinHeight=0.045
		WinLeft=0.625
		WinTop=0.67
		OnClick=ArrowClick
		bNeverFocus=true
		bRepeatClick=true
	End Object
	BodyRight=BodyRightBox

	Begin Object Class=GUIListBox Name=TraitListBox
		WinWidth=0.24
		WinHeight=0.111000
		WinLeft=0.070000
		WinTop=0.818000
		bVisibleWhenEmpty=True
		StyleName="NoBackground"
		SelectedStyleName="ListSelection"
		TabOrder=3
	End Object
	TraitList=TraitListBox

	Begin Object class=GUIScrollTextBox Name=TraitScrollBox
		WinWidth=0.57
		WinHeight=0.111000
		WinLeft=0.35000
		WinTop=0.818000
		bNoTeletype=true
		CharDelay=0.000001
		EOLDelay=0.000001
		FontScale=FNS_Small
		bVisibleWhenEmpty=true
		StyleName="SmallScroll"
		bAcceptsInput=false
	End Object
	TraitScroll=TraitScrollBox

	Begin Object Class=GUISectionBackground Name=TraitBackground
		WinWidth=0.9261
		WinHeight=0.214078
		WinLeft=0.036600
		WinTop=0.74566
		LeftPadding=0.01
		RightPadding=0.01
		Caption="Pick a Trait"
		bFillClient=true
	End Object
	TraitBG=TraitBackground

	Begin Object class=GUIScrollTextBox Name=TeamDesc
		WinWidth=0.215
		WinHeight=0.45
		WinLeft=0.701
		WinTop=0.205
		bNoTeletype=true
		CharDelay=0.000001
		EOLDelay=0.000001
		FontScale=FNS_Small
		bVisibleWhenEmpty=true
		StyleName="SmallScroll"
		bAcceptsInput=false
	End Object
	TeamDescription=TeamDesc

	// Trait records
	Traits(0)=class'Trait_POW'
	Traits(1)=class'Trait_Ninja'
	Traits(2)=class'Trait_Rambo'
	Traits(3)=class'Trait_Thug'
	Traits(4)=class'Trait_Pyro'
	Traits(5)=class'Trait_Cadet'
	Traits(6)=class'Trait_Sniper'
	Traits(7)=class'Trait_Assassin'
	Traits(8)=class'Trait_Genius'
	Traits(9)=class'Trait_Pitcher'
	Traits(10)=class'Trait_Police'
	Traits(11)=class'Trait_Tank'
	Traits(12)=class'Trait_BloodyMess'
	Traits(13)=class'Trait_Sprinter'
	Traits(14)=class'Trait_Convict'
	Traits(15)=class'Trait_Outcast'

	// Character records
	// - merc
	Begin Object Class=FOPlayerRecord Name=MercRecord0
		AssociatedClass="FragOps.FOPawnDarkGlory"
		AssociatedModel(0)=Mesh'FOCharacterModels.mercAmale'
		AssociatedModel(1)=Mesh'FOCharacterModels.mercAmale'
		HeadSkin(0)=Material'FOCharacters.textures.mercAhead_blonde'
		HeadSkin(1)=Material'FOCharacters.textures.mercAhead'
		HelmetSkin(0)=None
		HelmetSkin(1)=None
		GlassesSkin(0)=None
		GlassesSkin(1)=None
		BodySkin(0)=Material'FOCharacters.textures.mercAbody_jungle'
		BodySkin(1)=Material'FOCharacters.textures.mercAbody_olive'
		BodySkin(2)=Material'FOCharacters.textures.mercAbody_urban'
		BodySkin(3)=Material'FOCharacters.textures.mercAbody_brown'
		BodySkin(4)=Material'FOCharacters.textures.mercAbody'
		BodySkin(5)=Material'FOCharacters.textures.mercAbody_darkurban'
		MiscSkin(0)=None
		MiscSkin(1)=None
	Name="MercRecord0"
	End Object
	MercRecord(0)=FOPlayerRecord'MercRecord0'

	//Begin Object Class=FOPlayerRecord Name=MercRecord1
		//AssociatedClass="FragOps.FOPawnSiren"
		//AssociatedModel(0)=Mesh'FOCharacterFemales.mercAfemale'
		//AssociatedModel(1)=Mesh'FOCharacterFemales.mercBfemale'
		//AssociatedModel(2)=Mesh'FOCharacterFemales.mercBfemale'
		//AssociatedModel(3)=Mesh'FOCharacterFemales.mercCfemale'
		//AssociatedModel(4)=Mesh'FOCharacterFemales.mercDfemale'
		//AssociatedModel(5)=Mesh'FOCharacterFemales.mercDfemale'
		//AssociatedModel(6)=Mesh'FOCharacterFemales.mercAfemale'
		//AssociatedModel(7)=Mesh'FOCharacterFemales.mercBfemale'
		//AssociatedModel(8)=Mesh'FOCharacterFemales.mercBfemale'
		//AssociatedModel(9)=Mesh'FOCharacterFemales.mercCfemale'
		//AssociatedModel(10)=Mesh'FOCharacterFemales.mercDfemale'
		//AssociatedModel(11)=Mesh'FOCharacterFemales.mercDfemale'
		//HeadSkin(0)=Material'FOCharacters.textures.mercAfemalehead_blond'
		//HeadSkin(1)=Material'FOCharacters.textures.mercAfemalehead_blond'
		//HeadSkin(2)=Material'FOCharacters.textures.mercAfemalehead_blond'
		//HeadSkin(3)=Material'FOCharacters.textures.mercAfemalehead_blondmask'
		//HeadSkin(4)=Material'FOCharacters.textures.mercAfemalehead_blondmask'
		//HeadSkin(5)=Material'FOCharacters.textures.mercAfemalehead_blondmask'
		//HeadSkin(6)=Material'FOCharacters.textures.mercAfemalehead_black'
		//HeadSkin(7)=Material'FOCharacters.textures.mercAfemalehead_black'
		//HeadSkin(8)=Material'FOCharacters.textures.mercAfemalehead_black'
		//HeadSkin(9)=Material'FOCharacters.textures.mercAfemalehead_blackmask'
		//HeadSkin(10)=Material'FOCharacters.textures.mercAfemalehead_blackmask'
		//HeadSkin(11)=Material'FOCharacters.textures.mercAfemalehead_blackmask'
		//GlassesSkin(0)=Material'FOCharacters.textures.mercAfemale_blondhair'
		//GlassesSkin(1)=Material'FOCharacters.textures.mercAfemale_blondhair'
		//GlassesSkin(2)=Material'FOCharacters.textures.mercAfemale_blondhair'
		//GlassesSkin(3)=None
		//GlassesSkin(4)=Material'FOCharacters.textures.mercAfemalehead_blond'
		//GlassesSkin(5)=Material'FOCharacters.textures.mercAfemalehead_blackmask'
		//GlassesSkin(6)=Material'FOCharacters.textures.mercAfemale_blackhair'
		//GlassesSkin(7)=Material'FOCharacters.textures.mercAfemale_blackhair'
		//GlassesSkin(8)=Material'FOCharacters.textures.mercAfemale_blackhair'
		//GlassesSkin(9)=None
		//GlassesSkin(10)=Material'FOCharacters.textures.mercAfemalehead_blond'
		//GlassesSkin(11)=Material'FOCharacters.textures.mercAfemalehead_blackmask'
		//HelmetSkin(0)=None
		//HelmetSkin(1)=Material'FOCharacters.textures.mercAfemalehead_blond'
		//HelmetSkin(2)=Material'FOCharacters.textures.mercAfemalehead_blackmask'
		//HelmetSkin(3)=None
		//HelmetSkin(4)=None
		//HelmetSkin(5)=Material'FOCharacters.textures.mercAfemalehead_blond'
		//HelmetSkin(6)=Material'FOCharacters.textures.mercAfemalehead_blackmask'
		//HelmetSkin(7)=Material'FOCharacters.textures.mercAfemalehead_blond'
		//HelmetSkin(8)=Material'FOCharacters.textures.mercAfemalehead_blackmask'
		//HelmetSkin(9)=None
		//HelmetSkin(10)=None
		//HelmetSkin(11)=None
		//BodySkin(0)=Material'FOCharacters.textures.mercAbody_jungle'
		//BodySkin(1)=Material'FOCharacters.textures.mercAbody_olive'
		//BodySkin(2)=Material'FOCharacters.textures.mercAbody_urban'
		//BodySkin(3)=Material'FOCharacters.textures.mercAbody_brown'
		//BodySkin(4)=Material'FOCharacters.textures.mercAbody'
		//BodySkin(5)=Material'FOCharacters.textures.mercAbody_darkurban'
		//MiscSkin(0)=None
		//MiscSkin(1)=None
		//MiscSkin(2)=None
		//MiscSkin(3)=None
		//MiscSkin(4)=None
		//MiscSkin(5)=None
		//MiscSkin(6)=None
		//MiscSkin(7)=None
		//MiscSkin(8)=None
		//MiscSkin(9)=None
		//MiscSkin(10)=None
		//MiscSkin(11)=None
	//Name="MercRecord1"
	//End Object
	//MercRecord(1)=FOPlayerRecord'MercRecord1'

	// - specops
	Begin Object Class=FOPlayerRecord Name=SORecord0
		AssociatedClass="FragOps.FOPawnSWAT"
		AssociatedModel(0)=Mesh'FOCharacterModels.swatMale'
		AssociatedModel(1)=Mesh'FOCharacterModels.swatMale'
		HeadSkin(0)=Material'FOCharacters.textures.swatAhead'
		HeadSkin(1)=Material'FOCharacters.textures.swatBhead'
		HelmetSkin(0)=None
		HelmetSkin(1)=None
		GlassesSkin(0)=None
		GlassesSkin(1)=None
		MiscSkin(0)=None
		MiscSkin(1)=None
		BodySkin(0)=Material'FOCharacters.textures.swatAbody'
		BodySkin(1)=Material'FOCharacters.textures.swatAbody_grey'
		BodySkin(2)=Material'FOCharacters.textures.swatAbody_urban'
		BodySkin(3)=Material'FOCharacters.textures.swatAbody_forest'
		BodySkin(4)=Material'FOCharacters.textures.swatAbody_desert'
	Name="SORecord0"
	End Object
	SORecord(0)=FOPlayerRecord'SORecord0'

	//Begin Object Class=FOPlayerRecord Name=SORecord1
		//AssociatedClass="FragOps.FOPawnJo"
		//AssociatedModel(0)=Mesh'FOCharacterFemales.swatAfemale'
		//AssociatedModel(1)=Mesh'FOCharacterFemales.swatBfemale'
		//AssociatedModel(2)=Mesh'FOCharacterFemales.swatCfemale'
		//AssociatedModel(3)=Mesh'FOCharacterFemales.swatBfemale'
		//AssociatedModel(4)=Mesh'FOCharacterFemales.swatAfemale'
		//AssociatedModel(5)=Mesh'FOCharacterFemales.swatBfemale'
		//AssociatedModel(6)=Mesh'FOCharacterFemales.swatCfemale'
		//AssociatedModel(7)=Mesh'FOCharacterFemales.swatBfemale'
		//HeadSkin(0)=Material'FOCharacters.textures.swatAfemalehead_red'
		//HeadSkin(1)=Material'FOCharacters.textures.swatAfemalehead_red'
		//HeadSkin(2)=Material'FOCharacters.textures.swatAfemalehead_redmask'
		//HeadSkin(3)=Material'FOCharacters.textures.swatAfemalehead_redmask'
		//HeadSkin(4)=Material'FOCharacters.textures.swatAfemalehead_black'
		//HeadSkin(5)=Material'FOCharacters.textures.swatAfemalehead_black'
		//HeadSkin(6)=Material'FOCharacters.textures.swatAfemalehead_blackmask'
		//HeadSkin(7)=Material'FOCharacters.textures.swatAfemalehead_blackmask'
		//HelmetSkin(0)=None
		//HelmetSkin(1)=Material'FOCharacters.textures.swathelmet'
		//HelmetSkin(2)=None
		//HelmetSkin(3)=Material'FOCharacters.textures.swathelmet'
		//HelmetSkin(4)=None
		//HelmetSkin(5)=Material'FOCharacters.textures.swathelmet'
		//HelmetSkin(6)=None
		//HelmetSkin(7)=Material'FOCharacters.textures.swathelmet'
		//GlassesSkin(0)=Material'FOCharacters.textures.swatAfemale_redhair'
		//GlassesSkin(1)=None
		//GlassesSkin(2)=Material'glasses.1.glases'
		//GlassesSkin(3)=None
		//GlassesSkin(4)=Material'FOCharacters.textures.mercAfemale_blackhair'
		//GlassesSkin(5)=None
		//GlassesSkin(6)=Material'glasses.1.glases'
		//GlassesSkin(7)=None
		//BodySkin(0)=Material'FOCharacters.textures.swatAbody'
		//BodySkin(1)=Material'FOCharacters.textures.swatAbody_grey'
		//BodySkin(2)=Material'FOCharacters.textures.swatAbody_urban'
		//BodySkin(3)=Material'FOCharacters.textures.swatAbody_forest'
		//BodySkin(4)=Material'FOCharacters.textures.swatAbody_desert'
		//MiscSkin(0)=Material'glasses.1.glases'
		//MiscSkin(1)=Material'glasses.1.glases'
		//MiscSkin(2)=None
		//MiscSkin(3)=Material'glasses.1.glases'
		//MiscSkin(4)=Material'glasses.1.glases'
		//MiscSkin(5)=Material'glasses.1.glases'
		//MiscSkin(6)=None
		//MiscSkin(7)=Material'glasses.1.glases'
	//Name="SORecord1"
	//End Object
	//SORecord(1)=FOPlayerRecord'SORecord1'

	// Character names
	MercChar(0)="Dark Glory"
	MercDesc(0)="The richest and most well-armed Mercenary force in the world. Specializing in weapon theft and creation, they are known to strike where the bounty is most valuable."
	//MercChar(1)="Dark Glory Female"
	SOChar(0)="SWAT"
	SODesc(0)="The elite police of the modern day, the SWAT are called in for every job imaginable. Like most of the world's security forces, they are now part of the Last Few alliance."
	//SOChar(1)="SWAT Female"

	// Saved records
	MercFavorite(0)=(SaveName="- Empty -",GroupType=255,SoldierType=255,BodyType=255,Trait=255,S_Structures=8,S_Mechanic=8,S_Weapons=8)
	MercFavorite(1)=(SaveName="- Empty -",GroupType=255,SoldierType=255,BodyType=255,Trait=255,S_Structures=8,S_Mechanic=8,S_Weapons=8)
	MercFavorite(2)=(SaveName="- Empty -",GroupType=255,SoldierType=255,BodyType=255,Trait=255,S_Structures=8,S_Mechanic=8,S_Weapons=8)
	MercFavorite(3)=(SaveName="- Empty -",GroupType=255,SoldierType=255,BodyType=255,Trait=255,S_Structures=8,S_Mechanic=8,S_Weapons=8)
	MercFavorite(4)=(SaveName="- Empty -",GroupType=255,SoldierType=255,BodyType=255,Trait=255,S_Structures=8,S_Mechanic=8,S_Weapons=8)
	MercFavorite(5)=(SaveName="- Empty -",GroupType=255,SoldierType=255,BodyType=255,Trait=255,S_Structures=8,S_Mechanic=8,S_Weapons=8)
	MercFavorite(6)=(SaveName="- Empty -",GroupType=255,SoldierType=255,BodyType=255,Trait=255,S_Structures=8,S_Mechanic=8,S_Weapons=8)
	MercFavorite(7)=(SaveName="- Empty -",GroupType=255,SoldierType=255,BodyType=255,Trait=255,S_Structures=8,S_Mechanic=8,S_Weapons=8)
	MercFavorite(8)=(SaveName="- Empty -",GroupType=255,SoldierType=255,BodyType=255,Trait=255,S_Structures=8,S_Mechanic=8,S_Weapons=8)
	MercFavorite(9)=(SaveName="- Empty -",GroupType=255,SoldierType=255,BodyType=255,Trait=255,S_Structures=8,S_Mechanic=8,S_Weapons=8)

	SOFavorite(0)=(SaveName="- Empty -",GroupType=255,SoldierType=255,BodyType=255,Trait=255,S_Structures=8,S_Mechanic=8,S_Weapons=8)
	SOFavorite(1)=(SaveName="- Empty -",GroupType=255,SoldierType=255,BodyType=255,Trait=255,S_Structures=8,S_Mechanic=8,S_Weapons=8)
	SOFavorite(2)=(SaveName="- Empty -",GroupType=255,SoldierType=255,BodyType=255,Trait=255,S_Structures=8,S_Mechanic=8,S_Weapons=8)
	SOFavorite(3)=(SaveName="- Empty -",GroupType=255,SoldierType=255,BodyType=255,Trait=255,S_Structures=8,S_Mechanic=8,S_Weapons=8)
	SOFavorite(4)=(SaveName="- Empty -",GroupType=255,SoldierType=255,BodyType=255,Trait=255,S_Structures=8,S_Mechanic=8,S_Weapons=8)
	SOFavorite(5)=(SaveName="- Empty -",GroupType=255,SoldierType=255,BodyType=255,Trait=255,S_Structures=8,S_Mechanic=8,S_Weapons=8)
	SOFavorite(6)=(SaveName="- Empty -",GroupType=255,SoldierType=255,BodyType=255,Trait=255,S_Structures=8,S_Mechanic=8,S_Weapons=8)
	SOFavorite(7)=(SaveName="- Empty -",GroupType=255,SoldierType=255,BodyType=255,Trait=255,S_Structures=8,S_Mechanic=8,S_Weapons=8)
	SOFavorite(8)=(SaveName="- Empty -",GroupType=255,SoldierType=255,BodyType=255,Trait=255,S_Structures=8,S_Mechanic=8,S_Weapons=8)
	SOFavorite(9)=(SaveName="- Empty -",GroupType=255,SoldierType=255,BodyType=255,Trait=255,S_Structures=8,S_Mechanic=8,S_Weapons=8)

	// Settings
	EmptyString="- Empty -"
	SavedString="Saved!"
	DeletedString="Deleted!"
	ListFullString="No empty slots!"
	NameString="Enter a name"
	CustomString="[Custom]"

	// Colors
	DarkenColor=(R=150,G=150,B=150,A=255)
	BrightenColor=(R=255,G=255,B=255,A=255)

	OnRendered=InternalRendered
}