//=============================================================================
// Frag.Ops  2003 Matt 'SquirrelZero' Farber
//=============================================================================
class Interface_TabWeaponSettings extends UT2k4Tab_WeaponPref;

var rotator CenterWeaponPivot;

function ReallyInitializeWeaponList( Canvas C )
{
	local int i;
	local array<CacheManager.WeaponRecord> Records;

	if ( Controller.ActivePage.Tag != 55 )
	{
		Controller.ActivePage.Tag = 55;
		return;
	}

	// Initialise weapon list. Sort based on current priority - highest priority first
	class'CacheManager'.static.GetWeaponList(Records);

	// Disable the combo list's OnChange()
	lb_Weapons.List.bNotify = False;
	lb_Weapons.List.Clear();


	for(i=0;i<Records.Length;i++)
	{
		if (class<Weapon_Base>(DynamicLoadObject(Records[i].ClassName,class'Class')) != None)
			lb_Weapons.List.Add(Records[i].FriendlyName, DynamicLoadObject(Records[i].ClassName,class'Class'), Records[i].Description);
	}

	lb_Weapons.List.SortList();

	// Spawn spinny weapon actor
	if (SpinnyWeap == None)
		SpinnyWeap = PlayerOwner().spawn(class'XInterface.SpinnyWeap');

	SpinnyWeap.SetRotation(PlayerOwner().Rotation);
	SpinnyWeap.SetStaticMesh(None);

	// Start with first item on list selected
	lb_Weapons.List.SetIndex(0);
	WeaponListInitialized();

	lb_Weapons.List.bNotify = True;

	if (Controller.ActivePage != PageOwner)
		Controller.CloseMenu(true);

	FocusFirst(none);
}

function InternalDraw(Canvas canvas)
{
	local vector CamPos,X,Y,Z,WX,WY,WZ,localX,localY,localZ,globalX,globalY,globalZ;
	local rotator CamRot;

	if (WeaponIndex < 0)
		return;

	Canvas.GetCameraLocation(CamPos, CamRot);
	GetAxes(CamRot,X,Y,Z);

	GetAxes(CenterWeaponPivot,globalX,globalY,globalZ);
	localX = globalX >> CamRot;
	localY = globalY >> CamRot;
	localZ = globalZ >> CamRot;
	CamRot = OrthoRotation(localX,localY,localZ);
	SpinnyWeap.SetRotation(CamRot);
	SpinnyWeap.Disable('Tick');

	if(SpinnyWeap.DrawType == DT_Mesh)
	{
		GetAxes(SpinnyWeap.Rotation, WX, WY, WZ);
		SpinnyWeap.SetLocation(CamPos + (SpinnyWeapOffset.X * X) + (SpinnyWeapOffset.Y * Y) + (SpinnyWeapOffset.Z * Z) + (30 * WX));
	}
	else
	{
		SpinnyWeap.SetLocation(CamPos + (SpinnyWeapOffset.X * X) + (SpinnyWeapOffset.Y * Y) + (SpinnyWeapOffset.Z * Z));
	}
	Canvas.DrawActorClipped(SpinnyWeap, false, i_BG.ClientBounds[0], i_BG.ClientBounds[1], i_BG.ClientBounds[2] - i_BG.ClientBounds[0], (i_BG.ClientBounds[3] - i_BG.ClientBounds[1])/2, true, 90.0);
}

defaultproperties
{
	SpinnyWeapOffset=(X=80,Y=1.50,Z=-10.00)
	CenterWeaponPivot=(Pitch=0,Yaw=-16384,Roll=49152)

	Begin Object Class=moCheckBox Name=WeaponMesh
		Caption="Classic Model"
		Hint="Enable to use the classic model for this weapon"
		OnChange=InternalOnChange
		LabelJustification=TXTA_Left
		ComponentJustification=TXTA_Right
		ComponentWidth=-1
		CaptionWidth=0.8
		RenderWeight=1.03
		WinWidth=0.224062
		WinHeight=0.040000
		WinLeft=0.542070
		WinTop=1.537279
		bSquare=True
		TabOrder=12
		bAutoSizeCaption=True
		bVisible=false
		bAcceptsInput=false
	End Object
	ch_ClassicModel=WeaponMesh
}