//=============================================================================
// Frag.Ops  2003 Matt 'SquirrelZero' Farber
//=============================================================================
// Obj_AutoTurret.uc
// A fearsome automatic turret cannon, with 3 pieces of independant movement.
// Fires giant 30mm shells with dead perfect accuracy.  Using a specialized
// sensor, it can track any visible movement and weigh its threat on a number
// of variables.  Don't get in the way of this monster!  Turret can be turned
// off by triggering its event tag, as in power-cutting objectives.  For a good
// example of its inner workings, see the official map FO-MIS-OpCan. 
//=============================================================================
class Obj_AutoTurret extends Obj_LAWObjective;

var vector TurretOffset;
var Turret_Rotator TurretRotator;

// how far this turret can "see"
var() float VisualRadius;

// delay between each cannon shot
var() float FireRate;

var Sound OffSound;

function Trigger(actor Other, pawn EventInstigator)
{
	TurnOff();
}

function PostBeginPlay()
{
	Super.PostBeginPlay();
	if (Role == ROLE_Authority)
	{
		TurretRotator = spawn(class'Turret_Rotator',None,, Location + TurretOffset, Rotation);
		TurretRotator.EnemyTeam = ObjectiveTeam;
		TurretRotator.VisualRadius = VisualRadius;
		TurretRotator.FireRate = FireRate;
		TurretRotator.SetBase(self);
	}
}

function Completed(Pawn Other)
{
	TurretRotator.HideTurret();
	Super.Completed(Other);
}

function TurnOff()
{
	// turning off
	PlaySound(OffSound,,2.5*TransientSoundVolume,,150,,false);
	TurretRotator.bTurretDisabled = true;
}

defaultproperties
{
	StaticMesh=StaticMesh'OpCanNiSM.CannonSMs.cannon_base'
	OffSound=Sound'IndoorAmbience.door12'
	TurretOffset=(Z=0)
	VisualRadius=5000.0
	FireRate=0.3
	AmbientGlow=15
}