//=============================================================================
// Frag.Ops  2003 Matt 'SquirrelZero' Farber
//=============================================================================
// Frag.Ops Objective_EscapeVolume
// --
// Adds an Escape Objective to your level.
// You can place as many as you want, and even allows for both teams to have
// them in the same map.  Make sure they don't intersect each other, or it
// could cause a world of problems.
// --
//=============================================================================
class Objective_EscapeVolume extends Volume;

// Is this a Master Escape Volume?  The master volume is where the game will read 
// options that it only uses once.  For example, the EscapeMessage and 
// FailedEscapeMessage are only needed once, so they only need to be configured once.
// Also, for maps where there is only one side escaping, YOU MUST SET THIS TO TRUE 
// FOR ONLY ONE VOLUME.  If you don't, then the game will not know that a team must
// lose if they don't escape, and will always result in a draw game.  The EscapingTeam
// option will also only be read once in single-side escape maps, and only from the 
// Master Escape Volume.  In maps where both teams can escape, you must ALWAYS have 
// a total of TWO Master Escape Volumes, ONE PER SIDE.  This is so the game can read the 
// FailedEscapeMessage twice, once for each team.
var() bool bMasterEscapeVolume;

// 0 = Mercenaries, 1 = Special Ops, anything else = stupid
var() byte EscapingTeam;

// the message the game will broadcast when enough people have escaped on EscapingTeam,
// only read once
var() string EscapeMessage;

// the message the game will broadcast when EscapingTeam fails to escape
var() string FailedEscapeMessage;

// the message the game will broadcast to each individual player when he escapes using
// this volume
var() string PlayerEscapeMessage;

// a short description for this objective
var() string ShortDescription;

event PostBeginPlay()
{
	Super.PostBeginPlay();
	if (FragOpsMission(Level.Game) == None || !bMasterEscapeVolume)
		return;

	FragOpsMission(Level.Game).MasterEscapeVolume[EscapingTeam] = self;
}

event Touch(Actor Other)
{
	local Pawn P;

	// extra check -- we dont want escaping players after round is over
	if (FragOpsGame(Level.Game).TimeTillNextRound > 0 || FragOpsGame(Level.Game).PreRoundTime > 0)
		return;	

	Super.Touch(Other);
	if (EscapingTeam == 1)
	{
		if (FragOpsMission(Level.Game).SO_ObjectiveChain.Length > 0 && !FragOpsMission(Level.Game).SO_ObjectiveChain[(FragOpsMission(Level.Game).SO_ObjectiveChain.Length-1)].bCompleted && !FragOpsMission(Level.Game).SO_ObjectiveChain[(FragOpsMission(Level.Game).SO_ObjectiveChain.Length-1)].bLinkComplete && !FragOpsMission(Level.Game).SO_ObjectiveChain[(FragOpsMission(Level.Game).SO_ObjectiveChain.Length-1)].bCompletionOptional)
			return;
	}
	else if (EscapingTeam == 0)
	{
		if (FragOpsMission(Level.Game).MC_ObjectiveChain.Length > 0 && !FragOpsMission(Level.Game).MC_ObjectiveChain[(FragOpsMission(Level.Game).MC_ObjectiveChain.Length-1)].bCompleted && !FragOpsMission(Level.Game).MC_ObjectiveChain[(FragOpsMission(Level.Game).MC_ObjectiveChain.Length-1)].bLinkComplete && !FragOpsMission(Level.Game).MC_ObjectiveChain[(FragOpsMission(Level.Game).MC_ObjectiveChain.Length-1)].bCompletionOptional)
			return;
	}

	P = Pawn(Other);
	if (P == None)
		return;

	if (P.Health > 0 && P.PlayerReplicationInfo != None && P.PlayerReplicationInfo.Team.TeamIndex == EscapingTeam && !P.PlayerReplicationInfo.bOutOfLives && !FOPawn(P).bEscaped)
		DoEscape(P);
}

function DoEscape(Pawn Other)
{
	FOPawn(Other).DoEscape(self);
	if (FragOpsMission(Level.Game) != None)
		FragOpsMission(Level.Game).CheckEscapeWinner(EscapingTeam,self);
}

defaultproperties
{
	bNoDelete=true
	bAlwaysRelevant=true
	EscapingTeam=0
	EscapeMessage="Mercenaries have successfully escaped!"
	PlayerEscapeMessage="You successfully escaped!"
	FailedEscapeMessage="Mercenaries failed to escape."
}