//=============================================================================
// Frag.Ops  2003 Matt 'SquirrelZero' Farber
//=============================================================================
class ProjectileFire_Base extends FragOps_FireMode;

var() class<Emitter> NewMuzFlashClass;
var() Emitter NewMuzFlash;

var() int ProjPerFire;
var() Vector ProjSpawnOffset; // +x forward, +y right, +z up
                                        
function DoFireEffect()
{
	local Vector StartProj, StartTrace, X,Y,Z;
	local Rotator R, Aim;
	local Vector HitLocation, HitNormal;
	local Actor Other;
	local int p;
	local int SpawnCount;
	local float theta;

	Instigator.MakeNoise(1.0);
	Weapon.GetViewAxes(X,Y,Z);

	StartTrace = Instigator.Location + Instigator.EyePosition();// + X*Instigator.CollisionRadius;
	StartProj = StartTrace + X*ProjSpawnOffset.X;
	StartProj = StartProj + Weapon.Hand * Y*ProjSpawnOffset.Y + Z*ProjSpawnOffset.Z;

	// check if projectile would spawn through a wall and adjust start location accordingly
	Other = Weapon.Trace(HitLocation, HitNormal, StartProj, StartTrace, false);
	if (Other != None)
	{
		StartProj = HitLocation;
	}
    
	Aim = AdjustAim(StartProj, AimError);
	SpawnCount = Max(1, ProjPerFire * int(Load));

	switch (SpreadStyle)
	{
		case SS_Random:
			X = Vector(Aim);
			for (p = 0; p < SpawnCount; p++)
			{
				R.Yaw = Spread * (FRand()-0.5);
				R.Pitch = Spread * (FRand()-0.5);
				R.Roll = Spread * (FRand()-0.5);
				SpawnProjectile(StartProj, Rotator(X >> R));
			}
			break;

		case SS_Line:
			for (p = 0; p < SpawnCount; p++)
			{
				theta = Spread*PI/32768*(p - float(SpawnCount-1)/2.0);
				X.X = Cos(theta);
				X.Y = Sin(theta);
				X.Z = 0.0;
				SpawnProjectile(StartProj, Rotator(X >> Aim));
			}
			break;
		default:
			SpawnProjectile(StartProj, Aim);
	}
}

function projectile SpawnProjectile(Vector Start, Rotator Dir)
{
	local Projectile p;
    
	if (ProjectileClass != None )
		p = Weapon.Spawn(ProjectileClass,,, Start, Dir);

	if( p == None )
		return None;

	p.Damage *= DamageAtten;
	return p;
}

simulated function vector GetFireStart(vector X, vector Y, vector Z)
{
	return Instigator.Location + Instigator.EyePosition() + X*ProjSpawnOffset.X + Y*ProjSpawnOffset.Y + Z*ProjSpawnOffset.Z;
}

simulated function InitEffects()
{
	// don't even spawn on server
	if ( (Level.NetMode == NM_DedicatedServer) || (AIController(Instigator.Controller) != None) )
		return;

	if ( (NewMuzFlashClass != None) && ((NewMuzFlash == None) || NewMuzFlash.bDeleteMe) )
		NewMuzFlash = Spawn(NewMuzFlashClass);
}

function FlashMuzzleFlash()
{
	if (NewMuzFlash != None)
		NewMuzFlash.Trigger(Weapon, Instigator);
}

simulated event ModeDoFire()
{
	if (AmmoPerFire <= 0 || !NewAllowFire())
		return;

	if (MaxHoldTime > 0.0)
		HoldTime = FMin(HoldTime, MaxHoldTime);

	// client
	if (Instigator.IsLocallyControlled())
	{
		ShakeView();
		PlayFiring();
		FlashMuzzleFlash();
		StartMuzzleSmoke();
	}

	Weapon.IncrementFlashCount(ThisModeNum);

	// set the next firing time. must be careful here so client and server do not get out of sync
	if (bFireOnRelease)
	{
	        if (bIsFiring)
			NextFireTime += MaxHoldTime + FireRate;
		else
			NextFireTime = Level.TimeSeconds + FireRate;
	}
	else
	{
		NextFireTime += FireRate;
		NextFireTime = FMax(NextFireTime, Level.TimeSeconds);
	}


	Load = AmmoPerFire;

	// server
	if (Weapon.Role == ROLE_Authority)
	{
		DoFireEffect();
		if (Instigator == None || Instigator.Controller == None)
			return;
		if (AIController(Instigator.Controller) != None)
			AIController(Instigator.Controller).WeaponFireAgain(BotRefireRate, true);
		Instigator.SpawnTime = -100000;
	}

	HoldTime = 0;

	if (Instigator.PendingWeapon != Weapon && Instigator.PendingWeapon != None)
	{
		bIsFiring = false;
		Weapon.PutDown();
	}
}

// destroyed
simulated function Destroyed()
{
	DestroyEffects();
}

function StartBerserk()
{
}

function StopBerserk()
{
}

defaultproperties
{
	ProjPerFire=1
	ProjSpawnOffset=(Z=-10.000000)
	bLeadTarget=True
	bInstantHit=False
	NoAmmoSound=ProceduralSound'WeaponSounds.PReload5.P1Reload5'
	WarnTargetPct=0.500000
}