//=============================================================================
// Frag.Ops  2003 Matt 'SquirrelZero' Farber
//=============================================================================
// FOSecure, inspired by UTSecure.  Some code borrowed from Dr.Sin and UTSecure
//=============================================================================
class Security_FOSecure extends UnrealSecurity;

struct CheckData
{
	var globalconfig string FileName;
	var globalconfig string GUID;
	var globalconfig int    MaxGenerations;
	var globalconfig string MD5;
	var globalconfig int	MD5Type;
	var globalconfig bool   Optional;
};

var globalconfig array<CheckData> Checks;
var globalconfig int SecondsBeforeTimeout;
var globalconfig bool bKickTimedoutPlayers;
var globalconfig int VengeanceLevel;

var int Index;

replication
{
	reliable if (Role == ROLE_Authority)
    		ClientAnnounceInsecure;
}

auto state StartUp
{
	function Timer()
	{
		// spectators don't need repeated checks
		if (PlayerController(Owner).PlayerReplicationInfo.bOnlySpectator)
		{
			GotoState('Disabled');
			return;
		}

		// if we have a file to check, do it here
		if (Checks.Length > 0)
			ClientPerform(Checks[Index].MD5Type,Checks[Index].FileName,"");

		GotoState('CheckTimeout','Begin');
	}

	function BeginState()
	{
		SetTimer(FRand()+1,false);
	}
}

// check for timeouts, if client exceeds timeout limit, kick
state CheckTimeout
{
	function Timer()
	{
		if (bKickTimedoutPlayers)
			PlayerController(Owner).Destroy();
	}

	function BeginState()
	{
	}

Begin:
	SetTimer(SecondsBeforeTimeout,false);
}

// disable for spectators
state Disabled
{
	ignores Timer, Tick;

	function BeginState()
	{
		log("FOSecure: Player passed cheat check:"@PlayerController(Owner).PlayerReplicationInfo.PlayerName);
	}
}

// check the next file in the index
function CheckNextFile()
{
	Index++;
	if (Index > Checks.Length)
	{
		GotoState('Disabled');
	}
	else
	{
		ClientPerform(Checks[Index].MD5Type,Checks[Index].FileName,"");
		GotoState('CheckTimeout','Begin');
	}
}

// after the clientperform, data is sent back to the server, deal with it here
event ServerCallback(int SecType, string Data)
{
	if (SecType == Checks[Index].MD5Type)
	{
		if (Data ~= Checks[Index].MD5)
	        	CheckNextFile();
		else
		{
			log("FOSecure: INSECURE CLIENT!"@PlayerController(Owner).PlayerReplicationInfo.PlayerName$"["@PlayerController(Owner).GetPlayerIDHash()@"]");
			log("FOSecure: INSECURE CLIENT!"$Checks[Index].FileName);
			log("FOSecure: INSECURE CLIENT! expected ["$Checks[Index].MD5$"] got ["$Data$"]");

			if (VengeanceLevel > 0)
				ClientAnnounceInsecure(0,Checks[Index].FileName,Level.NetMode);

			switch(VengeanceLevel)
			{
				case 1:
					Level.Game.AccessControl.KickPlayer(PlayerController(Owner));
					break;

				case 2:
					Level.Game.AccessControl.BanPlayer(PlayerController(Owner), true);
					break;

				case 3:
					Level.Game.AccessControl.BanPlayer(PlayerController(Owner), false);
					break;
			}
		}
	}
	else if (SecType == 255 && Data ~= "File was not found")
	{
		if (Checks[Index].GUID!="" && Checks[Index].MaxGenerations >= 0)
		{
			if (Checks[Index].MaxGenerations > 0)
				ClientPerform(Checks[Index].MD5Type,"..\\Cache\\"$Checks[Index].GUID$"-"$Checks[Index].MaxGenerations$".uxx","");
			else
				ClientPerform(Checks[Index].MD5Type,"..\\Cache\\"$Checks[Index].GUID$".uxx","");

			Checks[Index].MaxGenerations--;
			GotoState('CheckTimeout','Begin');
		}
		else if (Checks[Index].Optional)	// Ignore optional files if they aren't loaded
		{
			CheckNextFile();
		}
		else
		{
			log("FOSecure: INSECURE CLIENT!"@PlayerController(Owner).PlayerReplicationInfo.PlayerName$"["@PlayerController(Owner).GetPlayerIDHash()@"]");
			log("FOSecure: INSECURE CLIENT!  Failed to find the file"@Checks[Index].FileName);

			if (VengeanceLevel > 0)
				ClientAnnounceInsecure(2,Checks[Index].FileName,Level.NetMode);

			switch(VengeanceLevel)
			{
				case 1:
					Level.Game.AccessControl.KickPlayer(PlayerController(Owner));
					break;

				case 2:
					Level.Game.AccessControl.BanPlayer(PlayerController(Owner), true);
					break;

				case 3:
					Level.Game.AccessControl.BanPlayer(PlayerController(Owner), false);
					break;
			}
		}
	}
	else if (SecType == 255 && Data ~= "Package Not Loaded" && Checks[Index].Optional)
		CheckNextFile();
	else
	{
		log("FOSecure: SUSPICIOUS CLIENT"@PlayerController(Owner).PlayerReplicationInfo.PlayerName$"["@PlayerController(Owner).GetPlayerIDHash()@"]");
		log("FOSecure: SUSPICIOUS CLIENT Returned Security code "$SecType$" with message ["$Data$"]");
	}
}

simulated function ClientAnnounceInsecure(int ErrorType, string s, byte Role)
{
	local string st;
	local int i;
	local PlayerController PC;
	//local GUIController GUIC;
	//local Interface_SecurityMenu SecurityMenu;

	// handle bad calls
	if (ROLE == 3)
	{
		log("FOSecure: Invalid call to the security announcement function.");
		return;
	}

	// clean up message string
	i = Instr(S,"\\");
	while (i >=0 )
	{
		st = Right(s,Len(s)-i-1);
		s = st;
		i = Instr(S,"\\");
	}

	// find out if this playercontroller is still valid
	foreach AllActors(class'PlayerController',PC)
	{	
		if (PC.Player != None)
			break;
	}

/*	// open an announcement menu
	PC.ClientOpenMenu("FragOps.Interface_SecurityMenu");

	GUIC = GUIController(PC.Player.GUIController);
	if (GUIC != None)
	{
		SecurityMenu = Interface_SecurityMenu(GUIC.ActivePage);
		if (SecurityMenu != None)
		{
        		Switch(ErrorType)
			{
				case 0:
					st = "The server found file "$Caps(S)$" to be insecure.  Make sure you are running an unmodified version, if you feel this is an error, try a different server.";
					break;


				case 1:
					st = "The server found a set hack ["$s$"] and has disconnected you.  SetHacking is not allowed here.";
					break;

				case 2:
					st = "The server was unable to verify file "$s$"!";
					break;

				case 3:
					st = "This server restricts access to unverified files ["$s$"] during the game!";
					break;

				default:				
					SecurityMenu.bAllowedAsLast = true;
					log("FOSecure: INVALID ERROR CODE"@ErrorType);
					GUIC.CloseMenu(true);
					return;
			}
			SecurityMenu.SetText(st);
		}
	}*/
}

defaultproperties
{
	VengeanceLevel=1
	bKickTimedOutPlayers=true
	SecondsBeforeTimeout=150
	InitialState=StartUp
}