//=============================================================================
// Frag.Ops  2003 Matt 'SquirrelZero' Farber
//=============================================================================
// The rotating YAW piece of the autoturret.  Everything is controlled here.
//=============================================================================
class Turret_Rotator extends Actor;

// how far this turret can "see"
var float VisualRadius;

// delay between each cannon shot
var float FireRate;

var Turret_CannonPiece TurretGun[2];
var vector GunOffset[2];
var int EnemyTeam;
var Pawn CurrentThreat;
var bool bTurretDisabled, bFiredTopLast;
var float LastFireTime;
var Sound OnSound, TurnSound;

var Effect_TurretEye TurretEye;
var vector EyeOffset;

replication
{
	reliable if (bNetInitial && Role == ROLE_Authority)
		EnemyTeam, VisualRadius, FireRate;

	reliable if (bNetDirty && Role == ROLE_Authority)
		bTurretDisabled, TurretGun;
}

function BeginPlay()
{
	Super.BeginPlay();

	// spawn the cannons
	if (Role == ROLE_Authority)
	{
		TurretGun[0] = spawn(class'Turret_CannonPiece',None,, Location + GunOffset[0], Rotation);
		TurretGun[0].SetBase(self);
		TurretGun[1] = spawn(class'Turret_CannonPiece',None,, Location + GunOffset[1], Rotation);
		TurretGun[1].SetBase(self);
		AmbientSound = OnSound;
	}
}

simulated function PostBeginPlay()
{
	Super.PostBeginPlay();
	SetTimer(1.0, false);

	if (Level.NetMode == NM_DedicatedServer)
		return;

	// spawn the eye corona
	TurretEye = spawn(class'Effect_TurretEye',self,, Location + EyeOffset, Rotation);
	TurretEye.SetBase(self);
}

simulated function Timer()
{
	local Pawn Victims;
	local array<Pawn> Enemies;
	local float ThreatDistance, GreatestThreat;
	local int i;

	if (TurretEye != None)
	{
		if (bTurretDisabled && TurretEye.bCorona)
			TurretEye.bCorona = false;
		else if (!bTurretDisabled && !TurretEye.bCorona)
			TurretEye.bCorona = true;
	}

	// build array of enemies
	foreach CollidingActors(class'Pawn', Victims, VisualRadius)
	{
		if (Victims.Health > 0 && Victims.PlayerReplicationInfo.Team.TeamIndex == EnemyTeam && FastTrace(Victims.GetBoneCoords('head').Origin, Location))
			Enemies[Enemies.Length] = Victims;
	}

	// decide which enemy poses greatest threat
	for (i=0;i<Enemies.Length;i++)
	{
		ThreatDistance = VSize(Enemies[i].Location - (Location + vect(0,0,25)));
		if (ThreatDistance > GreatestThreat || (ThreatDistance < 280 && Enemies.Length == 1))
		{
			GreatestThreat = ThreatDistance;
			CurrentThreat = Enemies[i];
		}
	}

	// update every 1/10 of a second
	SetTimer(0.1, false);
}

simulated function Tick(float delta)
{
	local rotator DirtyRotation, TurretRotation, CurrentRotation;
	local int TurretYaw, TurretPitch, LastRotationDir;

	// dont do anything if turret guns havent spawned yet
	if (TurretGun[0] == None || TurretGun[1] == None)
		return;

	if (CurrentThreat == None || bTurretDisabled)
	{
		DesiredRotation = rot(0,0,0);
		TurretGun[0].DesiredRotation = rot(0,0,0);
		TurretGun[1].DesiredRotation = rot(0,0,0);

		RotationRate.Yaw = 0;
		TurretGun[0].RotationRate.Pitch = 0;
		TurretGun[1].RotationRate.Pitch = 0;
		return;
	}

	// check if player's head is visible
	if (!FastTrace(CurrentThreat.GetBoneCoords('head').Origin, Location) || CurrentThreat.Health <= 0)
	{
		CurrentThreat = None;
		return;
	}

	// get rotation to target
	DirtyRotation = rotator(CurrentThreat.GetBoneCoords('head').Origin - Location);
	TurretRotation = Normalize(DirtyRotation);

	// normalize current rotation
	CurrentRotation = Normalize(DesiredRotation);

	TurretYaw = TurretRotation.Yaw;
	TurretPitch = TurretRotation.Pitch;

	LastRotationDir = RotationRate.Yaw;

	if (CurrentRotation.Yaw ClockWiseFrom TurretYaw)
		RotationRate.Yaw = -Default.RotationRate.Yaw;
	else if (!(CurrentRotation.Yaw ClockWiseFrom TurretYaw))
		RotationRate.Yaw = Default.RotationRate.Yaw;

	// play a turning sound
	if (LastRotationDir != RotationRate.Yaw && Level.NetMode != NM_DedicatedServer)
		PlaySound(TurnSound,SLOT_Interact,2.2*TransientSoundVolume,,150,,false);

	DesiredRotation.Yaw = TurretYaw;

	// normalize current rotation
	CurrentRotation = Normalize(TurretGun[0].DesiredRotation);

	// find out whether to turn turret guns up or down
	if (CurrentRotation.Pitch ClockWiseFrom TurretPitch)
		TurretGun[0].RotationRate.Pitch = -TurretGun[0].Default.RotationRate.Pitch;
	else if (!(CurrentRotation.Pitch ClockWiseFrom TurretPitch))
		TurretGun[0].RotationRate.Pitch = TurretGun[0].Default.RotationRate.Pitch;
	TurretGun[0].DesiredRotation.Pitch = FClamp(TurretPitch, -8000, 32768);
	TurretGun[1].DesiredRotation.Pitch = TurretGun[0].DesiredRotation.Pitch;

	// rotate to desired
	bRotateToDesired = true;
	TurretGun[0].bRotateToDesired = true;
	TurretGun[1].bRotateToDesired = true;

	if (Role < Role_Authority)
		return;

	CurrentRotation = Normalize(Rotation);
	DirtyRotation = Normalize(TurretGun[0].Rotation);

	// decide if we should fire a shell or not
	if (Level.TimeSeconds - LastFireTime >= FireRate && VSize(CurrentThreat.Location - Location) > 280)
	{
		// is the turret in firing position?
		if (CurrentRotation.Yaw > (TurretYaw-3000) && CurrentRotation.Yaw < (TurretYaw+3000) && DirtyRotation.Pitch > (TurretPitch-3000) && DirtyRotation.Pitch < (TurretPitch+3000))
		{
			// alternate shot locations
			if (bFiredTopLast)
			{
				TurretGun[0].FireShell(CurrentThreat);
				bFiredTopLast = false;
			}
			else
			{
				TurretGun[1].FireShell(CurrentThreat);
				bFiredTopLast = true;
			}
			LastFireTime = Level.TimeSeconds;
		}
	}
}

function HideTurret()
{
	bTurretDisabled = true;
	AmbientSound = None;

	SetCollision(false,false,false);
	SetStaticMesh(None);
	TurretGun[0].SetCollision(false,false,false);
	TurretGun[0].SetStaticMesh(None);
	TurretGun[1].SetCollision(false,false,false);
	TurretGun[1].SetStaticMesh(None);
}

function Reset()
{
	SetStaticMesh(Default.StaticMesh);
	SetCollision(true,true,true);
	bTurretDisabled = false;
	AmbientSound = OnSound;
}

defaultproperties
{
	// settings
	OnSound=Sound'IndoorAmbience.machinery32'
	TurnSound=Sound'IndoorAmbience.door7'
	GunOffset(0)=(Z=8)
	GunOffset(1)=(Z=34)
	EyeOffset=(X=37,Y=7,Z=38)
	VisualRadius=5000.0
	FireRate=0.5

	// actor properties
	DrawType=DT_StaticMesh
	StaticMesh=StaticMesh'OpCanNiSM.CannonSMs.cannon_turretnew'
	Physics=PHYS_Projectile
	RemoteRole=ROLE_SimulatedProxy
	bGameRelevant=true
	bAlwaysRelevant=true
	bBlockActors=true
	bBlockPlayers=true
	bCollideActors=true
	bCollideWorld=false
	bStatic=false
	bHardAttach=true
	bRotateToDesired=false
	bFixedRotationDir=true
	bDirectional=true
	bNetInitialRotation=true
	//bReplicateMovement=false
	RotationRate=(Yaw=80000)
	AmbientGlow=15
}