//=============================================================================
// Frag.Ops  2004 Matt 'SquirrelZero' Farber
//=============================================================================
// DO NOT! put two of the same item into the buildable items array.  It will
// cause problems.
//=============================================================================
class War_BuildZone extends Actor
	placeable;

enum eItemName
{
	NO_ITEM,
	Sandbag_Wall,
	Mongoose_Vehicle,
	M240_MachineGun,
	Sniper_Tower,
	Stryker,
	Medical_Station,
	RAH66_Helicopter
};

// items
struct sBuildableItems
{
	var() byte ResourceAmount;
	var() eItemName ItemName;
	var class<War_FOQuickbarFunction> ItemType;
	var array<War_FOQuickbarFunction> ItemUpgrades;
};

var() sBuildableItems BuildableItems[10];

// current resources
var byte ResourceCount[10];

// resource usage
var byte ResourceUsage[10];

// this is only used by clients
var float LastUseTime[10];
var float RealUseTime[10];
var float ClientUseTime[10];

// team that controls this objective
// 255 = neutral
var() byte ControllingTeam;

// is this build zone enabled by trigger?
var() bool bIsTriggered;

// enabled?
var bool bEnabled;

// server enforce resources
replication
{
	reliable if (bNetDirty && Role == ROLE_Authority)
		ResourceCount, LastUseTime, bEnabled;
}

// init
simulated function BeginPlay()
{
	Super.BeginPlay();
	SetDrawType(DT_StaticMesh);
}

simulated function PostBeginPlay()
{
	local int i;

	Super.PostBeginPlay();

	for (i=0;i<ArrayCount(BuildableItems);i++)
	{
		if (BuildableItems[i].ItemName != NO_ITEM)
			BuildableItems[i].ItemType = GetItemType(BuildableItems[i].ItemName);
	}
}

// set item types
simulated function class<War_FOQuickbarFunction> GetItemType(eItemName QueryName)
{
	switch(QueryName)
	{
		case Sandbag_Wall:
			return class'War_QuickBar_Sandbag';
			break;

		case Mongoose_Vehicle:
			return class'War_QuickBar_Mongoose';
			break;

		case M240_MachineGun:
			return class'War_QuickBar_M240LMG';
			break;

		case Sniper_Tower:
			return class'War_QuickBar_Sniper';
			break;

		case Stryker:
			return class'War_QuickBar_Stryker';
			break;

		case Medical_Station:
			return class'War_QuickBar_MedStation';
			break;

		case RAH66_Helicopter:
			return class'War_QuickBar_Comanche';
			break;

		default:
			return None;
			break;
	}
}

function Trigger(Actor Other, Pawn EventInstigator)
{
	if (bIsTriggered)
		bEnabled = true;
}

// when an actor enters this build zone
event Touch(Actor Other)
{
	local FOPawn P;

	Super.Touch(Other);

	if (bIsTriggered && !bEnabled)
		return;

	P = FOPawn(Other);
	if (P == None || P.PlayerReplicationInfo == None || (P.PlayerReplicationInfo.Team.TeamIndex != ControllingTeam && ControllingTeam != 255))
		return;

	P.CurrentBuildZone = self;

	if (P.Controller != None && PlayerController(P.Controller) != None && !War_FOPlayer(P.Controller).bAmmoRestored)
		PlayerController(P.Controller).ReceiveLocalizedMessage(class'War_Interface_BuildZoneMessage',0,,,Level);
}

// when an actor leaves this build zone
event UnTouch(Actor Other)
{
	local FOPawn P;

	Super.UnTouch(Other);

	P = FOPawn(Other);
	if (P == None)
		return;
	
	if (P.Controller != None && P.Controller.IsInState('Building'))
		War_FOPlayer(P.Controller).ServerRequestItem();
	P.CurrentBuildZone = None;
}

function WaveReset()
{
	local int i;

	for (i=0;i<ArrayCount(BuildableItems);i++)
		ResourceCount[i] = BuildableItems[i].ResourceAmount - ResourceUsage[i];
}

function Reset()
{
	local int i;

	Super.Reset();
	
	for (i=0;i<ArrayCount(BuildableItems);i++)
	{
		if (bIsTriggered)
			bEnabled = false;
		LastUseTime[i] = 1;
		ResourceCount[i] = BuildableItems[i].ResourceAmount;
	}
}


// turns on a nifty grid so you can see the boundaries of the build zone
simulated function EnableGrid()
{
	local vector Scale3D;
	
	bHidden = false;
	Scale3D.X = FMax(CollisionRadius*0.028, 0);
	Scale3D.Y = Scale3D.X;
	Scale3D.Z = FMax(CollisionHeight*0.03, 0);
	SetDrawScale(1.0);
	SetDrawScale3D(Scale3D);
	SetRotation(rot(0,0,0));
}

simulated function DisableGrid()
{
	bHidden = true;
}

// retrieve item number
simulated function int GetBuildNum(class<War_FOQuickbarFunction> InFunc)
{
	local int i;

	for (i=0;i<ArrayCount(BuildableItems);i++)
	{
		if (BuildableItems[i].ItemType == InFunc)
			return i;
	}
	return -1;
}

// bit of a mistake on my part.. forgot that level.timeseconds is not synched between client and server. We compensate here
simulated event PostNetReceive()
{
	local int i;

	for (i=0;i<ArrayCount(LastUseTime);i++)
	{
		if (RealUseTime[i] != LastUseTime[i])
		{
			if (LastUseTime[i] == 1)
				ClientUseTime[i] = 1;
			else
				ClientUseTime[i] = Level.TimeSeconds;
		}
		RealUseTime[i] = LastUseTime[i];
	}
}

defaultproperties
{
	// items
	BuildableItems(0)=(ItemName=Sandbag_Wall,ResourceAmount=1)
	BuildableItems(1)=(ItemName=Sniper_Tower,ResourceAmount=1)
	BuildableItems(2)=(ItemName=Mongoose_Vehicle,ResourceAmount=1)
	BuildableItems(3)=(ItemName=M240_MachineGun,ResourceAmount=1)

	// internal
	ControllingTeam=255

	// settings
	RemoteRole=ROLE_SimulatedProxy
	StaticMesh=StaticMesh'FOWarSMs.Build.BuildZoneGrid'
	Texture=S_TerrainInfo
	DrawScale=3.0
	DrawType=DT_Sprite
	bNetNotify=true
	bAlwaysRelevant=true
	bStatic=false
	bNoDelete=true
	bHidden=true
	bDirectional=true
	bCollideActors=true
	bCollideWhenPlacing=false
	bBlockZeroExtentTraces=false
	bSkipActorPropertyReplication=true
	bUseCylinderCollision=true
	bUnlit=true
	CollisionRadius=700.0
	CollisionHeight=100.0
}