//=============================================================================
// Frag.Ops  2004 Matt 'SquirrelZero' Farber
//=============================================================================
class War_Buildable extends StaticMeshActor
	abstract;

// Karma Object type
enum EKarmaType
{
	KLead,
	KSteel,
	KRubber,
	KWood,
	KFeather
};

// Chunkrecord
struct ChunkRecord
{
	var() StaticMesh ChunkMesh;
	var() editinline float ChunkScale;
	var() Sound ChunkBounceSound;
	var() float ChunkVelocity;
	var() float ChunkLifeSpan;
	var() EKarmaType KarmaType;
};

// these chunks will fly out everywhere when object is destroyed
var() editinline array<ChunkRecord> ExplodedChunks;

// explosion emitters
var() editinline class<Emitter> ExplodedEmitter;
var() editinline class<xEmitter> ExplodedXEmitter;

// location offset to spawn emitter/chunks
var() vector EffectSpawnOffset;

// chunk offset
var() vector ChunkSpawnOffset;

// shake?
var() bool bShakeView;
var() float ShakeRadius;

// placement sound
var sound PlacingSound;

// buyzone stuff
var War_BuildZone MyBZ;
var int BZSlot;
var array<Actor> FoundActors;
var float NewCollisionRadius;

// health
var int Health;

// team
var int TeamIndex;

replication
{
	reliable if (bNetDirty && Role == ROLE_Authority)
		Health;
}

simulated function PostBeginPlay()
{
	Super.PostBeginPlay();

	// add to buildables array
	if (Level.NetMode != NM_DedicatedServer && Level.GetLocalPlayerController() != None)
		War_FOPlayer(Level.GetLocalPlayerController()).AddBuildable(self);

	// nice placement sound
	if (Level.NetMode != NM_DedicatedServer)
		PlaySound(PlacingSound,,,,80,0.7,true);
}

simulated function Destroyed()
{
	Super.Destroyed();

	// release resources
	if (MyBZ != None)
		MyBZ.ResourceUsage[BZSlot] = Max(MyBZ.ResourceUsage[BZSlot]-1, 0);

	// remove from buildables array
	if (Level.NetMode != NM_DedicatedServer && Level.GetLocalPlayerController() != None)
		War_FOPlayer(Level.GetLocalPlayerController()).DelBuildable(self);
}

// save me jebus!
simulated function Tick(float dt)
{
	local PlayerController PC;
	local Actor WPI, TA;
	local array<Actor> OldActors;
	local int i, j;
	local bool bFound, bFoundTouching;
	
	PC = Level.GetLocalPlayerController();

	if (Level.NetMode == NM_DedicatedServer || PC == None || !PC.IsInState('Building') || War_FOPlayer(PC).LastBuildZone != MyBZ)
		return;

	// duplicate
	for (i=0;i<FoundActors.Length;i++)
		OldActors[i] = FoundActors[i];
	
	// clear array
	FoundActors.Remove(0,FoundActors.Length);

	// start at 0 again
	i = 0;

	// fill array and touch necessary actors
	foreach CollidingActors(class'Actor',WPI,NewCollisionRadius)
	{
		if (War_PlacedItem(WPI) == None && War_ColChecker(WPI) == None)
			continue;
		
		bFound = false;
		bFoundTouching = false;
		for (j=0;j<OldActors.Length;j++)
		{
			if (OldActors[j] == WPI)
			{
				bFound = true;
				break;
			}
		}
		foreach TouchingActors(class'Actor',TA)
		{
			if (TA == WPI)
			{
				bFoundTouching = true;
				break;
			}
		}
		if (!bFound && !bFoundTouching)
			WPI.Touch(self);
		if (!bFoundTouching)
		{
			FoundActors[i] = WPI;
			i++;
		}
	}
	
	// if no longer touching a collision zone, untouch
	for (i=0;i<OldActors.Length;i++)
	{
		bFound = false;
		for (j=0;j<FoundActors.Length;j++)
		{
			if (FoundActors[j] == OldActors[i])
			{
				bFound = true;
				break;
			}
		}
		if (!bFound)
			OldActors[i].UnTouch(self);
	}
}

function bool TryToUse(Pawn User)
{
	return false;
}

// damage
event TakeDamage(int Damage, Pawn EventInstigator, vector HitLocation, vector Momentum, class<DamageType> DamageType)
{
	// reduce fist damage
	if (DamageType.Name == 'DamType_Machete' || DamageType.Name == 'DamType_Fists' || DamageType.Name == 'DamType_AKBayonet')
		Damage *= 0.1;

	if (EventInstigator != None && EventInstigator.PlayerReplicationInfo.Team.TeamIndex == MyBZ.ControllingTeam)
		return;

	Health -= Damage;	
	NetUpdateTime = Level.TimeSeconds - 1;

	if (Health <= 0)
		Explode(EventInstigator, HitLocation, Momentum, DamageType);
}

// blow apart
function Explode(Pawn Killer, vector HitLoc, vector Momentum, class<DamageType> DamageType)
{
	if (bShakeView)
		ShakeView();
	SpawnChunks();
	Destroy();
}

function SpawnChunks()
{
	local Effect_LAWChunk NewChunk;
	local Effect_LAWEmitterSpawner LAWEmitterSpawner;
	local vector start, newstart;
	local rotator rot;
	local int i;

	// spawn the explosion emitter
	if (ExplodedEmitter != None || ExplodedXEmitter != None)
	{
		LAWEmitterSpawner = Spawn(class'Effect_LAWEmitterSpawner',,, (Location + EffectSpawnOffset), rotator(vect(0,0,1)));
		LAWEmitterSpawner.EmitterClass = ExplodedEmitter;
		LAWEmitterSpawner.XEmitterClass = ExplodedXEmitter;
		LAWEmitterSpawner.DoSpawn();
	}

	// spawn some chunks
	Start = Location;
	for (i=0;i<ExplodedChunks.Length;i++)
	{
		rot = rotator(vect(0,0,1));
		rot.yaw += FRand()*64000-32000;
		rot.pitch += FRand()*16000-8000;
		rot.roll += FRand()*64000-32000;
		NewStart.X = (Start.X - 2) + (FRand()*4);
		NewStart.Y = (Start.Y - 2) + (FRand()*4);
		NewChunk = Spawn(class'Effect_LAWChunk',,, Start + ChunkSpawnOffset, rot);
		if (ExplodedChunks[i].ChunkMesh != None && NewChunk != None)
		{
			NewChunk.SetStaticMesh(ExplodedChunks[i].ChunkMesh);
			NewChunk.SetDrawScale(ExplodedChunks[i].ChunkScale);
			NewChunk.ImpactSound = ExplodedChunks[i].ChunkBounceSound;
			NewChunk.NewLifeSpan = ExplodedChunks[i].ChunkLifeSpan;
			if (ExplodedChunks[i].ChunkVelocity >= 1.0)
				NewChunk.Speed = ExplodedChunks[i].ChunkVelocity;
			NewChunk.SetKarma(true,int(ExplodedChunks[i].KarmaType));
		}
	}
}

function ShakeView()
{
	local Controller C;
	local PlayerController PC;
	local float Dist, Scale;

	for (C=Level.ControllerList;C!=None;C=C.NextController)
	{
		PC = PlayerController(C);
		if (PC != None && PC.ViewTarget != None)
		{
			Dist = VSize(Location - PC.ViewTarget.Location);
			if (Dist < ShakeRadius * 2.0)
			{
				if (Dist < ShakeRadius)
					Scale = 1.0;
				else
					Scale = (ShakeRadius*2.0 - Dist) / (ShakeRadius);
				FOPlayer(PC).ClientShake(vect(250,250,250)*Scale,vect(7000,7000,7000),3.5, vect(10,10,10)*Scale,vect(2000,2000,2000),3.5);
			}
		}
	}
}

// remove at round start
function Reset()
{
	Destroy();
}

defaultproperties
{
	// display
	DrawType=DT_StaticMesh

	// sound
	PlacingSound=sound'FOGameSounds.Menu.MedalStomp'

	// collision
	bBlockZeroExtentTraces=true
	bBlockNonZeroExtentTraces=true
	bBlockKarma=true
	bBlockActors=true
	bBlockPlayers=true
	bCollideActors=true

	// health
	Health=600
	
	// destroyed effects properties
	ExplodedEmitter=class'War_Effect_StructureExplosion'
	ChunkSpawnOffset=(Z=50)
	ShakeRadius=300.0

	// settings
	Physics=PHYS_None
	RemoteRole=ROLE_SimulatedProxy
	bWorldGeometry=true
	bUseDynamicLights=false
	bCanBeDamaged=true
	bProjTarget=true
	bAcceptsProjectors=false
	bStatic=false
	bNoDelete=false
	bShadowCast=false
	bOwnerNoSee=false
	bAlwaysRelevant=true
	MaxLights=4
}