class War_Effect_BulletHit extends Effect_BulletHit;

var actor TracerActor;

replication
{
	reliable if (Role == ROLE_Authority)
		TracerActor;
}

simulated function SpawnTracer()
{
	local Effect_BulletTracer BulletTracer;
	local class<Effect_BulletTracer> BulletTracerClass;
	local int TrailOffset;
	local rotator TrailRotation;

	if (Level.NetMode == NM_DedicatedServer || FOPawn(Instigator) == None || TracerActor == None)
		return;

	// get a random offset
	TrailOffset = Rand(40);

	// setup tracer
	BulletTracerClass = class'War_Effect_BulletTracer';
	SetLocation(TracerActor.GetBoneCoords(ONSWeapon(TracerActor).Default.WeaponFireAttachmentBone).Origin);

	// so dead bodies aren't so... boring
	CheckBodyHits();

	// get the rotation
	TrailRotation = rotator(SpawnLoc - Location);
	
	// do splashes
	CheckSplash(SpawnLoc);

	if (!bShowTracers)
		return;

	// only spawn tracers randomly
	if (FRand() > 0.5)
		return;

	// don't spawn if through wall
	if (!FastTrace(Location,Location + vector(TrailRotation)*TrailOffset))
		return;

	// spawn the tracer
	BulletTracer = spawn(BulletTracerClass,,, Location + vector(TrailRotation)*(100+TrailOffset), TrailRotation);
	
	// project it
	if (BulletTracer != None)
		BulletTracer.ProjectBeam(Location,SpawnLoc);
}

// this makes dead bodies more interesting
simulated function CheckBodyHits()
{
	local vector HitLocation, NewNormal;
	local Actor Other;

	// don't do in listen or standalone games
	if (Level.NetMode == NM_StandAlone || Level.NetMode == NM_ListenServer)
		return;

	// find a hit on a body first
	Other = Trace(HitLocation,NewNormal,SpawnLoc,Location,true);
	if (FOPawn(Other) == None || !Other.IsInState('Dying'))
		return;

	// take damage
	bHitBody = true;
	FOPawn(Other).PlayBodyHitSound();
	Other.TakeDamage(50, Instigator, HitLocation, 2.5*(SpawnLoc-Location), MyDamageType);
}

defaultproperties
{
}