//=============================================================================
// Frag.Ops  2004 Matt 'SquirrelZero' Farber
//=============================================================================
class War_FOPlayer extends FOPlayer;

// desired weaponry
var class<Weapon_Base> DesiredEquip[6], DesiredAttach[6];
var string DesiredMiscItem;

// building
var class<War_FOQuickbarFunction> BuildFunction;
var War_BuildZone LastBuildZone;
var War_PlacedItem PlacedItem;

// buildable arrays, for easy access.  Maybe a linked list later.
var array<War_Buildable> BuiltItems;
var array<War_BuildZone> BuildZones;

// wave flags
var bool bAmmoRestored, bArmorChanged, bAlreadyRespawned, bHaveSpawned, bFirstLoadout;

// objectives
var War_Objective_Interactive CurWarObjective;
var War_Objective Objective;

// special skills
var byte S_Structures, S_Mechanic, S_Weapons;

replication
{
	reliable if (Role == ROLE_Authority)
		ClientDoWaveCommands, ClientApproveItem, ClientSplashObjective;

	reliable if (bNetOwner && bNetDirty && Role == ROLE_Authority)
		S_Structures, S_Mechanic, S_Weapons, bAlreadyRespawned;

	reliable if (bNetOwner && Role == ROLE_Authority)
		DesiredEquip, DesiredAttach, DesiredMiscItem, Objective;

	unreliable if (Role < ROLE_Authority)
		ServerRequestItem, ServerBuyMenu, ServerSetGear;

	reliable if (Role < ROLE_Authority)
		ServerPlaceItem;
}

// initialize radar arrays
simulated function PostNetBeginPlay()
{
	local War_Buildable Buildables;
	local War_BuildZone BuildZone;

	Super.PostNetBeginPlay();

	if (Level.NetMode == NM_DedicatedServer)
		return;

	foreach DynamicActors(class'War_Buildable',Buildables)
		BuiltItems[BuiltItems.Length] = Buildables;

	foreach AllActors(class'War_BuildZone',BuildZone)
		BuildZones[BuildZones.Length] = BuildZone;
}

// quickbar execution
exec function QuickbarFunction(byte FunctionNum)
{
	// dont execute anything if no buildzone, resources, or still within used time
	if (Pawn == None || MyHUD == None || (FunctionNum >= War_Interface_MainHUD(MyHUD).Quickbar.BarFunction.Length && Vehicle(Pawn) == None))
		return;

	// vehicle stuff
	if (Vehicle(Pawn) != None)
	{
		Pawn.SwitchWeapon(FunctionNum+1);
		return;
	}
	else if (FOPawn(Pawn).CurrentBuildZone == None)
		return;

	ServerRequestItem(War_Interface_MainHUD(MyHUD).Quickbar.BarFunction[FunctionNum]);
}

// request permission to build an item from the server
function ServerRequestItem(optional class<War_FOQuickbarFunction> BarFunction)
{
	local int i;
	local bool bFoundItem;

	// fallback 1
	if (IsInState('PreRound') || IsDead() || BarFunction == None || Pawn == None || Pawn.Physics == PHYS_Swimming || Vehicle(Pawn) != None || (Pawn.Weapon != None && Pawn.Weapon.IsFiring()) || FOPawn(Pawn).CurrentBuildZone == None)
		return;

	// try to find the requested item
	for (i=0;i<ArrayCount(FOPawn(Pawn).CurrentBuildZone.BuildableItems);i++)
	{
		if (FOPawn(Pawn).CurrentBuildZone.BuildableItems[i].ItemType == BarFunction && FOPawn(Pawn).CurrentBuildZone.ResourceCount[i] > 0 && (Level.TimeSeconds - FOPawn(Pawn).CurrentBuildZone.LastUseTime[i] >= BarFunction.static.GetLevel(self).Default.UseDelayTime || FOPawn(Pawn).CurrentBuildZone.LastUseTime[i] == 1))
		{
			bFoundItem = true;
			break;
		}
	}

	// fallback 2
	if (!bFoundItem)
		return;

	// item approved, send back
	if (Level.NetMode == NM_ListenServer || Level.NetMode == NM_DedicatedServer)
		ClientApproveItem(BarFunction);

	BuildFunction = BarFunction;
	StartBuilding(BarFunction);
}

// approve item
function ClientApproveItem(class<War_FOQuickbarFunction> BarFunction)
{
	BuildFunction = BarFunction;
	StartBuilding(BarFunction);
}

// start building item
function StartBuilding(class<War_FOQuickbarFunction> BarFunction)
{
	if (Pawn == None)
		return;

	Pawn.LandMovementState = 'Building';
	GotoState('Building');
}

// not necessary outside of state right now
function ServerPlaceItem(vector ItemLoc, int ItemYaw);

simulated state Building extends PlayerWalking
{
	// place the object
	exec function Fire(optional float F)
	{
		bFire = 0;
		if (PlacedItem.bCanBuild)
			ServerPlaceItem(PlacedItem.Location, PlacedItem.Rotation.Yaw);
	}

	// cancel the object
	exec function AltFire(optional float F)
	{
		bAltFire = 0;
		ServerRequestItem();
	}

	// rotate the object
	exec function SwitchFire(optional int i)
	{
		if (PlacedItem == None)
			return;
		
		if (i > 0)
		{
			PlacedItem.RotationOffset += (65535/8);
			if (PlacedItem.RotationOffset > 65500)
				PlacedItem.RotationOffset = 0;
		}
		else
		{
			PlacedItem.RotationOffset -= (65535/8);
			if (PlacedItem.RotationOffset < 35)
				PlacedItem.RotationOffset = 65535;
		}
	}

	// this cancels in the build state
	function ServerRequestItem(optional class<War_FOQuickbarFunction> BarFunction)
	{
		GotoState(Pawn.Default.LandMovementState,'');
		ClientGotoState(Pawn.Default.LandMovementState,'');
	}

	// finalization of item placement
	function ServerPlaceItem(vector ItemLoc, int ItemYaw)
	{
		local int i;
		local bool bItemAllowed;
		local rotator ItemRot;
		local vector BZLoc;
		local Actor Build;

		// no pawn
		if (Pawn == None || Pawn.Health <= 0)
		{
			ServerRequestItem();
			return;
		}

		// final check, to avoid abuse
		for (i=0;i<ArrayCount(LastBuildZone.BuildableItems);i++)
		{
			if (LastBuildZone.BuildableItems[i].ItemType == BuildFunction && LastBuildZone.ResourceCount[i] > 0 && (Level.TimeSeconds - LastBuildZone.LastUseTime[i] >= BuildFunction.static.GetLevel(self).Default.UseDelayTime || LastBuildZone.LastUseTime[i] == 1))
			{
				bItemAllowed = true;
				break;
			}
		}

		// location security check
		ItemLoc.Z = (LastBuildZone.Location.Z - LastBuildZone.CollisionHeight) + BuildFunction.static.GetLevel(self).Default.PlacingClass.Default.ZOffset;
		BZLoc = LastBuildZone.Location;
		BZLoc.Z = ItemLoc.Z;
		if (VSize(ItemLoc-BZLoc) > LastBuildZone.CollisionRadius*1.1 || !bItemAllowed)
		{
			ServerRequestItem();
			return;
		}

		// rotation enforcement
		ItemRot.Yaw = ItemYaw;

		// spawn/initialize item
		Build = Pawn.Spawn(BuildFunction.static.GetLevel(self).Default.BuildClass, None,, ItemLoc, ItemRot);
		if (War_Buildable(Build) != None)
		{
			Build.Instigator = None;
			War_Buildable(Build).MyBZ = LastBuildZone;
			War_Buildable(Build).BZSlot = i;
			War_Buildable(Build).TeamIndex = PlayerReplicationInfo.Team.TeamIndex;
			LastBuildZone.ResourceCount[i] -= 1;
			LastBuildZone.ResourceUsage[i] += 1;
			LastBuildZone.LastUseTime[i] = Level.TimeSeconds;
		}
		else if (War_FOVehicleWheeled(Build) != None)
		{
			War_FOVehicleWheeled(Build).MyBZ = LastBuildZone;
			War_FOVehicleWheeled(Build).BZSlot = i;
			War_FOVehicleWheeled(Build).SetTeamNum(PlayerReplicationInfo.Team.TeamIndex);
			LastBuildZone.ResourceCount[i] -= 1;
			LastBuildZone.ResourceUsage[i] += 1;
			LastBuildZone.LastUseTime[i] = Level.TimeSeconds;
		}
		else if (War_FOVehicleAir(Build) != None)
		{
			War_FOVehicleAir(Build).MyBZ = LastBuildZone;
			War_FOVehicleAir(Build).BZSlot = i;
			War_FOVehicleAir(Build).SetTeamNum(PlayerReplicationInfo.Team.TeamIndex);
			LastBuildZone.ResourceCount[i] -= 1;
			LastBuildZone.ResourceUsage[i] += 1;
			LastBuildZone.LastUseTime[i] = Level.TimeSeconds;
		}
		else if (War_Vehicle_M240MG(Build) != None)
		{
			War_Vehicle_M240MG(Build).MyBZ = LastBuildZone;
			War_Vehicle_M240MG(Build).BZSlot = i;
			War_Vehicle_M240MG(Build).SetTeamNum(PlayerReplicationInfo.Team.TeamIndex);
			LastBuildZone.ResourceCount[i] -= 1;
			LastBuildZone.ResourceUsage[i] += 1;
			LastBuildZone.LastUseTime[i] = Level.TimeSeconds;
		}
		ServerRequestItem();
	}

	function EndState()
	{
		Super.EndState();
		
		// reset movement state
		if (Pawn != None)
			Pawn.LandMovementState = Pawn.Default.LandMovementState;

		if (Level.NetMode != NM_DedicatedServer)
		{
			if (LastBuildZone != None)
				LastBuildZone.DisableGrid();
			if (Pawn != None && Pawn.Weapon != None)
				Weapon_Base(Pawn.Weapon).EndBuild();
		}
		LastBuildZone = None;
		BuildFunction = None;

		// remove temp item
		if (PlacedItem != None)
			PlacedItem.Destroy();
	}

	function BeginState()
	{
		Super.BeginState();
		
		if (BuildFunction == None)
			ServerRequestItem();
		
		LastBuildZone = FOPawn(Pawn).CurrentBuildZone;
		if (Level.NetMode != NM_DedicatedServer)
		{
			LastBuildZone.EnableGrid();
			Weapon_Base(Pawn.Weapon).StartBuild();

			// spawn placement item
			PlacedItem = Pawn.Spawn(BuildFunction.static.GetLevel(self).Default.PlacingClass);
			if (PlacedItem != None)
			{
				PlacedItem.FP = self;
				PlacedItem.BuildNum = LastBuildZone.GetBuildNum(BuildFunction);
				PlacedItem.BuildZone = LastBuildZone;
				PlacedItem.UpdateLocation();
			}
			else
				ServerRequestItem();
		}
	}
}

// add to buildables array
simulated function AddBuildable(War_Buildable Built)
{
	BuiltItems[BuiltItems.Length] = Built;
}

// delete
simulated function DelBuildable(optional War_Buildable Demolished)
{
	local int i;

	for (i=0;i<BuiltItems.Length;i++)
	{
		if (BuiltItems[i] == None || (Demolished != None && BuiltItems[i] == Demolished))
		{
			BuiltItems.Remove(i,1);
			break;
		}
	}
}

// set desired gear for next respawn
function ServerSetGear(byte AddNewMelee, byte AddNewSidearm, byte AddNewRifle, byte AddNewGrenade, byte AddNewGrenadeSec, byte AddNewMisc, class<Weapon_Base> AddSidearmAttach, class<Weapon_Base> AddRifleAttach, class<Weapon_Base> AddMeleeAttach)
{
	// Add weapons
	if (AddNewMelee == 1 || AddNewMelee == 0)
		DesiredEquip[0] = class'FragOps.Interface_BuyMenuPlus'.Default.Melee[AddNewMelee];
	else
		DesiredEquip[0] = class'Weapon_FistsBase';

	if (AddNewSidearm != 255)
		DesiredEquip[1] = class'FragOps.Interface_BuyMenuPlus'.Default.Sidearms[AddNewSidearm];
	else
		DesiredEquip[1] = None;

	if (AddNewRifle != 255)
		DesiredEquip[2] = class'FragOps.Interface_BuyMenuPlus'.Default.Rifles[AddNewRifle];
	else
		DesiredEquip[2] = None;

	if (AddNewGrenade == 254)
		DesiredEquip[3] = class'FragOps.War_Weapon_LAWBase';
	else if (AddNewGrenade != 255)
		DesiredEquip[3] = class'FragOps.Interface_BuyMenuPlus'.Default.Grenades[AddNewGrenade];
	else
		DesiredEquip[3] = None;

	if (AddNewGrenadeSec != 255 && AddNewGrenade != 254)
		DesiredEquip[4] = class'FragOps.Interface_BuyMenuPlus'.Default.Grenades[AddNewGrenadeSec];
	else
		DesiredEquip[4] = None;

	if (AddNewMisc != 255)
		DesiredMiscItem = class'FragOps.Interface_BuyMenuPlus'.Default.Misc[AddNewMisc];
	else
		DesiredMiscItem = "MED. Armor";

	// Add attachments
	DesiredAttach[0] = AddMeleeAttach;
	DesiredAttach[1] = AddSidearmAttach;
	DesiredAttach[2] = AddRifleAttach;

	if (Pawn != None && FOPawn(Pawn) != None)
	{
		FOPawn(Pawn).UpdateWeapons(self);
		FOPawn(Pawn).SetNewInventory(0,0);
	}
}

// vehicle zoom
function ToggleZoom()
{
	ToggleZoomWithMax(0.4);
}

function VehicleShakeView(vector shRotMag, vector shRotRate, float shRotTime, vector shOffsetMag, vector shOffsetRate, float shOffsetTime)
{
	ShakeView(shRotMag * (1.0 - ZoomLevel), shRotRate, shRotTime, shOffsetMag * (1.0 - ZoomLevel), shOffsetRate, shOffsetTime);
}

// adjust FOV
function AdjustView(float DeltaTime )
{
	// adjust FOV for weapon zooming
	if (Pawn != None && Pawn.Weapon != None)
	{
		if (bZooming)
			DesiredFOV = FClamp(90.0 - (ZoomLevel * 88.0), 1, 170);
		else if (Level.bClassicView && (DesiredFOV == 85) && (DefaultFOV == 85))
			DesiredFOV = 90;
	}

	// set to real FOV again
	if (RealFOV != 0)
		FOVAngle = RealFOV;

	// teleporters affect your FOV, so adjust it back down
	if (FOVAngle != DesiredFOV)
	{
		if (FOVAngle > DesiredFOV)
		{
			if (Pawn != None && ((Pawn.Weapon != None && Weapon_Base(Pawn.Weapon).WeaponType == Heavy) || Vehicle(Pawn) != None))
				FOVAngle = FOVAngle - FMax(7, 0.9 * DeltaTime * (FOVAngle - DesiredFOV));
			else
				FOVAngle = FOVAngle - FMax(7, 1.3 * DeltaTime * (FOVAngle - DesiredFOV));
		}
		else
			if (Pawn != None && ((Pawn.Weapon != None && Weapon_Base(Pawn.Weapon).WeaponType == Heavy) || Vehicle(Pawn) != None))
				FOVAngle = FOVAngle - FMin(-7, 0.9 * DeltaTime * (FOVAngle - DesiredFOV));
			else
				FOVAngle = FOVAngle - FMin(-7, 1.3 * DeltaTime * (FOVAngle - DesiredFOV));;
		if (Abs(FOVAngle - DesiredFOV) <= 10)
			FOVAngle = DesiredFOV;
	}

	// adjust FOV for weapon zooming
	if (Pawn != None && Vehicle(Pawn) != None && bZooming)
	{
		ZoomLevel = FMin(ZoomLevel + DeltaTime, DesiredZoomLevel);
		DesiredFOV = FClamp(90.0 - (ZoomLevel * 88.0), 1, 170);
	}

	// scoped breathing
	RealFOV = FOVAngle;
	if (Pawn != None && Pawn.Weapon != None && Weapon_Base(Pawn.Weapon).bScopeZoomed)
		FOVAngle = (RealFOV-(0.5*Weapon_Base(Pawn.Weapon).BreathFOV)) + Weapon_Base(Pawn.Weapon).BreathFOV;
}

// update rotation
function UpdateRotation(float DeltaTime, float maxPitch)
{
	local rotator NewRotation, ViewRotation, LastRotation;
	local float NewLookUp, NewTurn;

	if (bInterpolating || ((Pawn != None) && Pawn.bInterpolating))
	{	
		ViewShake(deltaTime);
	        return;
	}
	
	// Added FreeCam control for better view control
	if (bFreeCam)
	{
		if (bFreeCamZoom)
			CameraDeltaRad += DeltaTime * 0.25 * aLookUp;
		else if (bFreeCamSwivel)
		{
			CameraSwivel.Yaw += 16.0 * DeltaTime * aTurn;
			CameraSwivel.Pitch += 16.0 * DeltaTime * aLookUp;
		}
		else
		{
			CameraDeltaRotation.Yaw += 32.0 * DeltaTime * aTurn;
			CameraDeltaRotation.Pitch += 32.0 * DeltaTime * aLookUp;
		}
	}
	else
	{
		ViewRotation = Rotation;
		if (Pawn != None && (Pawn.Physics != PHYS_Flying || Pawn.OnLadder != None)) // mmmmm
		{
			// Ensure we are not setting the pawn to a rotation beyond its desired
			if(Pawn.DesiredRotation.Roll < 65535 &&
				(ViewRotation.Roll < Pawn.DesiredRotation.Roll || ViewRotation.Roll > 0))
				ViewRotation.Roll = 0;
			else if(Pawn.DesiredRotation.Roll > 0 &&
				(ViewRotation.Roll > Pawn.DesiredRotation.Roll || ViewRotation.Roll < 65535))
				ViewRotation.Roll = 0;
		}

		DesiredRotation = ViewRotation; //save old rotation

		TurnTarget = None;
		bRotateToDesired = false;
		bSetTurnRot = false;
		ViewRotation.Yaw += 32.0 * DeltaTime * aTurn;
		ViewRotation.Pitch += 32.0 * DeltaTime * aLookUp;

		LastRotation.Yaw = ViewRotation.Yaw;
		LastRotation.Pitch = ViewRotation.Pitch;

		if (IsInState('StickyObjective') && bIsPlayer)
			ViewRotation = Rotation;
		else if (IsInState('PreRound') && bIsPlayer)
			ViewRotation.Pitch = 0;
		else if (Pawn != None)
		{
			// limit yaw too
			if (War_FOVehiclePawn(Pawn) != None)
				ViewRotation.Yaw = War_FOVehiclePawn(Pawn).LimitYaw(ViewRotation.Yaw);
			else if (War_Vehicle_M240MG(Pawn) != None)
				ViewRotation.Yaw = War_Vehicle_M240MG(Pawn).LimitYaw(ViewRotation.Yaw);
			ViewRotation.Pitch = Pawn.LimitPitch(ViewRotation.Pitch); //amb
		}

		if (Level.NetMode != NM_DedicatedServer)
		{
			NewTurn = aTurn;
			NewLookUp = aLookup;
			if (LastRotation.Pitch != ViewRotation.Pitch)
				NewLookUp = 0;
			if (LastRotation.Yaw != ViewRotation.Yaw)
				NewTurn = 0;

			if (Pawn != None && Pawn.Weapon != None)
			{
				if (IsInState('PreRound'))
					Weapon_Base(Pawn.Weapon).SetTurnPivot(NewTurn, 0);
				else if (IsInState('StickyObjective'))
					Weapon_Base(Pawn.Weapon).SetTurnPivot(0,0);
				else
					Weapon_Base(Pawn.Weapon).SetTurnPivot(NewTurn, NewLookUp);

			}
			LastSlashDir = UpdateSlashDir();
		}		

		SetRotation(ViewRotation);

		ViewShake(deltaTime);
		ViewFlash(deltaTime);

		NewRotation = ViewRotation;
		//NewRotation.Roll = Rotation.Roll;

		if ((Pawn != None) && (!bFreeCamera || !bBehindView))
		{
			if (!bRotateToDesired)
				Pawn.FaceRotation(NewRotation, deltatime);
		}
	}
}

// no throw when building
function ServerThrowWeapon()
{
	local Vector TossVel;

	if (Pawn.CanThrowWeapon() && !IsInState('Building'))
	{
		TossVel = Vector(GetViewRotation());
		TossVel = TossVel * ((Pawn.Velocity Dot TossVel) + 500) + Vect(0,0,200);
		Pawn.TossWeapon(TossVel);
		if (Pawn.Weapon == None)
			ClientSwitchToBestWeapon();
	}
}

// loadout
exec function BuyMenu()
{
	if (PlayerReplicationInfo.Team == None || CriticalReplicationInfo.bWaitingForTeam)
		return;
	
	bFire = 0;
	bAltFire = 0;
	ServerBuyMenu();
}

function ServerBuyMenu()
{
	if (PlayerReplicationInfo.Team == None || CriticalReplicationInfo.bWaitingForTeam)
		return;
	
	if ((Pawn != None && FOPawn(Pawn) == None) || (!IsDead() && !FragOps_GRI(GameReplicationInfo).bPreRound))
		return;
	
	if (Pawn != None && FOPawn(Pawn) != None && Pawn.Weapon != None)
	{
		if (Weapon_Base(Pawn.Weapon).IsFiring())
			return;
	}
	ClientReplaceMenu("FragOps.War_Interface_LoadOutMenu");
}

// team menu
exec function ShowMenu()
{
	// Pause if not already
	if(Level.Pauser == None)
		SetPause(true);

	StopForceFeedback();  // jdf - no way to pause feedback

	// changed to our menu
	ClientOpenMenu("FragOps.War_Interface_MidGameMenu");
}

exec function ShowTeamMenu()
{
	if (Level.Pauser == None)
		SetPause(true);

	StopForceFeedback();

	ClientOpenMenu("FragOps.War_Interface_MidGameMenu");
}

simulated function TeamSelectMenu()
{
	ClientReplaceMenu("FragOps.War_Interface_MidGameMenu");
}

simulated function LoginMenu()
{
	ClientReplaceMenu("FragOps.War_Interface_LoginMenu");
}

function GetNewTeam(byte TempTeam, byte NewRecord, byte NewBodySkin, byte NewHeadSkin, byte NewTrait, optional byte NewStructures, optional byte NewMechanic, optional byte NewWeapons) 
{
	local int OldTeam, i;
	local string NewPawnClass;

	if (Pawn != None && Pawn.Health > 0)
	{
		if (PlayerReplicationInfo.Team != None && PlayerReplicationInfo.Team.TeamIndex == 1)
			Pawn.Died(None, class'DamType_TeamChangeBlue', Pawn.Location);
		else
			Pawn.Died(None, class'DamType_TeamChange', Pawn.Location);
	}
	
	SelectedTrait = NewTrait;
	FragOps_PRI(PlayerReplicationInfo).TraitType = NewTrait;

	if (PlayerReplicationInfo.Team != none)
	{
		OldTeam = PlayerReplicationInfo.Team.TeamIndex;
		PlayerReplicationInfo.Team.RemoveFromTeam(self);
	}

	// initialize mesh/skins
	SelectedTeam = Clamp(TempTeam, 0, 1);

	if (OldTeam != SelectedTeam)
	{
		for (i=1;i<ArrayCount(DesiredEquip);i++)
		{
			DesiredEquip[i] = None;
			DesiredAttach[i] = None;
		}
		CriticalReplicationInfo.bLAWCarrier = false;
	}

	switch(SelectedTeam)
	{
		case 1:
			if (OldTeam != SelectedTeam || bFirstLoadout)
			{
				DesiredEquip[0] = class'War_Interface_LoadOutMenu'.Default.Melee[1];
				DesiredEquip[1] = class'War_Interface_LoadOutMenu'.Default.Sidearms[0];
				DesiredEquip[2] = class'War_Interface_LoadOutMenu'.Default.Rifles[1];
				DesiredEquip[3] = class'War_Interface_LoadOutMenu'.Default.Grenades[0];
				DesiredEquip[4] = class'War_Interface_LoadOutMenu'.Default.Grenades[0];
			}
			SelectedRecord = Clamp(NewRecord, 0, (class'Interface_TabPlayerTeam'.Default.SORecord.Length-1));
			HeadSkin = Clamp(NewHeadSkin, 0, (class'Interface_TabPlayerTeam'.Default.SORecord[SelectedRecord].HeadSkin.Length-1));
			BodySkin = Clamp(NewBodySkin, 0, (class'Interface_TabPlayerTeam'.Default.SORecord[SelectedRecord].BodySkin.Length-1));
			NewPawnClass = class'Interface_TabPlayerTeam'.Default.SORecord[Clamp(SelectedRecord, 0, (class'Interface_TabPlayerTeam'.Default.SORecord.Length-1))].AssociatedClass;
			break;

		default:
			if (OldTeam != SelectedTeam || bFirstLoadout)
			{
				DesiredEquip[0] = class'War_Interface_LoadOutMenu'.Default.Melee[0];
				DesiredEquip[1] = class'War_Interface_LoadOutMenu'.Default.Sidearms[0];
				DesiredEquip[2] = class'War_Interface_LoadOutMenu'.Default.Rifles[0];
				DesiredEquip[3] = class'War_Interface_LoadOutMenu'.Default.Grenades[0];
				DesiredEquip[4] = class'War_Interface_LoadOutMenu'.Default.Grenades[0];
			}
			SelectedRecord = Clamp(NewRecord, 0, (class'Interface_TabPlayerTeam'.Default.MercRecord.Length-1));
			HeadSkin = Clamp(NewHeadSkin, 0, (class'Interface_TabPlayerTeam'.Default.MercRecord[SelectedRecord].HeadSkin.Length-1));
			BodySkin = Clamp(NewBodySkin, 0, (class'Interface_TabPlayerTeam'.Default.MercRecord[SelectedRecord].BodySkin.Length-1));
			NewPawnClass = class'Interface_TabPlayerTeam'.Default.MercRecord[Clamp(SelectedRecord, 0, (class'Interface_TabPlayerTeam'.Default.MercRecord.Length-1))].AssociatedClass;
			break;
	}

	bFirstLoadout = false;

	// fast update
	NetUpdateTime = Level.TimeSeconds - 1;

	// set the pawn class
	SetPawnClass(NewPawnClass, NewPawnClass);

	// set skills
	S_Structures = Clamp(NewStructures, 0, class'War_Interface_TabPlayerTeam'.Default.MaxPoints) + 80;
	S_Mechanic = Clamp(NewMechanic, 0, class'War_Interface_TabPlayerTeam'.Default.MaxPoints-NewStructures) + 80;
	S_Weapons = (class'War_Interface_TabPlayerTeam'.Default.MaxPoints-(NewStructures+NewMechanic)) + 80;
	
	// initialize
	FragOpsGame(Level.Game).Teams[SelectedTeam].AddToTeam(self);
	PlayerReplicationInfo.Team = FragOpsGame(Level.Game).Teams[SelectedTeam];
	PlayerReplicationInfo.Team.TeamIndex = SelectedTeam;
	if (!CriticalReplicationInfo.bWaitingForTeam)
		CriticalReplicationInfo.bChangedTeams = true;
	InitialStart();
}

// todo: make a functional buy bind for War
function ServerBuyWeapon(class<Weapon_Base> DesiredWeapon, optional byte DesiredAttachment, optional byte DesiredClips)
{
}

// waves
function DoWaveCommands()
{
	bAmmoRestored = false;
	bArmorChanged = false;
	bAlreadyRespawned = false;
	
	if (APModTemp < 0)
		APModTemp = 0;

	if (Pawn != None && FOPawn(Pawn) != None)
	{
		if (FOPawn(Pawn).CurrentBuildZone != None)
			ReceiveLocalizedMessage(class'War_Interface_BuildZoneMessage',0,,,Level);
		FOPawn(Pawn).bCanHeal = true;
	}
	ClientDoWaveCommands();
}

// when wave hits, do this
function ClientDoWaveCommands()
{
	local FOPawn FOP;
	local Actor EActor;

	foreach DynamicActors(class'Actor', EActor)
	{
		if ((MinigunMuzflash1st(EActor) != None || Effect_MuzSmoke3rd(EActor) != None || Effect_MeshFlash3rd(EActor) != None || Effect_BurstFlash3rd(EActor) != None || Effect_MuzSmoke(EActor) != None || Effect_ShellEjectorFirst(EActor) != None) && EActor.Owner == None)
			EActor.Destroy();
		
		FOP = FOPawn(EActor);
		if (FOP == None || FOP.Health > 0 || FOP.Physics == PHYS_KarmaRagdoll || Level.TimeSeconds - FOP.TimeOfDeath < 10)
			continue;

		FOP.Destroy();
	}
}

// objective-related
function ServerUnUse()
{
	if (CurWarObjective == None || IsInState('PreRound') || FragOps_GRI(GameReplicationInfo).bPreRound)
		return;

	GotoState('PlayerWalking');
	ClientGotoState('PlayerWalking','');
}

function ClientSetObjective(Actor NewCurObjective)
{
	CurWarObjective = War_Objective_Interactive(NewCurObjective);
}

function ClientSplashObjective(string ObjSplashString, Material ObjSplashMaterial, optional Sound ObjAnnouncement)
{	
	if (MyHUD != None)
		War_Interface_MainHUD(MyHUD).SplashObjective(ObjSplashString, ObjSplashMaterial);

	if (ObjAnnouncement != None && !IsDead())
		PlayObjAnnouncement(ObjAnnouncement);
}

simulated function PlayObjAnnouncement(sound AnnounceSound)
{
	if (ViewTarget == None)
		return;

	ViewTarget.PlaySound(AnnounceSound,SLOT_Talk,TransientSoundVolume*2.55,,,,false);
}

state StickyObjective
{
	exec function Fire(optional float F)
	{
		ServerUnUse();
		Global.Fire(F);
	}

	exec function AltFire(optional float F)
	{
		ServerUnUse();		
		Global.AltFire(F);
	}

	function ProcessMove(float DeltaTime, vector NewAccel, eDoubleClickDir DoubleClickMove, rotator DeltaRot)
	{
		if (Pawn == None)
			return;

		Pawn.Acceleration = NewAccel;
		Pawn.SetViewPitch(Rotation.Pitch);
	}

	function PlayerMove( float DeltaTime )
	{
		local vector X,Y,Z;
		local rotator OldRotation, ViewRotation;
		
		if (Pawn != None)
			GetAxes(Pawn.Rotation,X,Y,Z);

		GroundPitch = 0;
		ViewRotation = Rotation;

		if (Pawn != None)
			Pawn.CheckBob(DeltaTime, Y);

		// Update rotation.
		SetRotation(ViewRotation);
		OldRotation = Rotation;
		UpdateRotation(DeltaTime, 0);

		if (Role < ROLE_Authority) // then save this move and replicate it
			ReplicateMove(DeltaTime, vect(0,0,0), DCLICK_None, OldRotation - Rotation);
		else
			ProcessMove(DeltaTime, vect(0,0,0), DCLICK_None, OldRotation - Rotation);
	}

	exec function Action() {}

	function ServerAction(byte ActionType) {}

	function BeginState()
	{
		if (Role == ROLE_Authority && Level.NetMode != NM_StandAlone)
			ClientGotoState('StickyObjective','Begin');

		if (Role == ROLE_Authority)
		{
			Pawn.Velocity = vect(0,0,0);
			Pawn.Acceleration = vect(0,0,0);
		}

		if (Level.NetMode != NM_DedicatedServer)
			Interface_MainHUD(MyHUD).StartObjective(CurWarObjective.TriggerTime*FOPawn(Pawn).HackMod);
	}		

	function EndState()
	{
		if (CurWarObjective != None && !CurWarObjective.bCompleted)
		{
			if (Role == ROLE_Authority)
				CurWarObjective.Cancel();

			if (MyHUD == None)
				return;
		}

		if (Level.NetMode != NM_DedicatedServer && CurWarObjective != None)
			Interface_MainHUD(MyHUD).StopObjective();

		CurWarObjective = None;
	}

Begin:
	Sleep(CurWarObjective.TriggerTime*FOPawn(Pawn).HackMod);
	if (Role == ROLE_Authority)
		CurWarObjective.Completed(Pawn);
	if (Level.NetMode != NM_DedicatedServer)
		CurWarObjective = None;
	if (!FragOps_GRI(GameReplicationInfo).bPreRound)
		GotoState('PlayerWalking');
}

// vehicle related
function ServerUse()
{
	local Actor A;
	local Vehicle DrivenVehicle, EntryVehicle, V;
	local War_Buildable WB;

	if (Role < ROLE_Authority)
		return;

	if (Level.Pauser == PlayerReplicationInfo)
	{
		SetPause(false);
		return;
	}

	if (Pawn == None || !Pawn.bCanUse || IsInState('PreRound'))
		return;

	// exit vehicle we're already in
	DrivenVehicle = Vehicle(Pawn);
	if (DrivenVehicle != None )
	{
		DrivenVehicle.KDriverLeave(false);
		return;
	}

	foreach Pawn.CollidingActors(class'War_Buildable',WB,VehicleCheckRadius)
	{
		if (WB.TryToUse(Pawn))
			return;
	}

	// check for nearby buildables/vehicles
	foreach Pawn.VisibleCollidingActors(class'Actor',A,VehicleCheckRadius)
	{
		// vehicle check
		V = Vehicle(A);
		if (V == None)
			continue;

		// found a vehicle within radius
		EntryVehicle = V.FindEntryVehicle(Pawn);
		if (EntryVehicle != None && EntryVehicle.TryToDrive(Pawn))
			return;
	}
	
	// only "use" items that we're looking at
	foreach Pawn.TouchingActors(class'Actor',A)
		A.UsedBy(Pawn);

	// now use base (for movers?)
	if (Pawn.Base != None)
		Pawn.Base.UsedBy(Pawn);
}

function AmmoSound()
{
	if (Pawn != None)
		Pawn.PlaySound(Sound'FOGameSounds.Misc.ResupplySound',SLOT_None,TransientSoundVolume*1.2,,30,,false);
}

exec function BuyAmmo(optional bool bMaxAmmo)
{
	if (Pawn == None || FOPawn(Pawn) == None || FOPawn(Pawn).CurrentBuildZone == None)
		return;

	ServerAddToClip(false);
}

function RoundResupply()
{
	local Weapon TempWeapon;

	if (Pawn == None || FOPawn(Pawn) == None)
		return;

	TempWeapon = FOPawn(Pawn).FindWeapon(4);

	// restore ammunition
	if (TempWeapon != None && War_Weapon_LAWBase(TempWeapon) != None)
		Weapon_Base(TempWeapon).GetAmmo(0).AmmoAmount = Weapon_Base(TempWeapon).GetAmmo(0).Default.MaxAmmo;

	TempWeapon = FOPawn(Pawn).FindWeapon(3);
	if (TempWeapon != None)
	{
		Weapon_Base(TempWeapon).GetAmmo(0).NumClips = Weapon_Base(TempWeapon).GetAmmo(0).Default.MaxClips;
		Weapon_Base(TempWeapon).GetAmmo(0).AmmoAmount = Weapon_Base(TempWeapon).GetAmmo(0).Default.MaxAmmo;
	}

	TempWeapon = FOPawn(Pawn).FindWeapon(2);
	if (TempWeapon != None)
	{
		Weapon_Base(TempWeapon).GetAmmo(0).NumClips = Weapon_Base(TempWeapon).GetAmmo(0).Default.MaxClips;
		Weapon_Base(TempWeapon).GetAmmo(0).AmmoAmount = Weapon_Base(TempWeapon).GetAmmo(0).Default.MaxAmmo;
		if (Ammo_AkimboBase(Weapon_Base(TempWeapon).GetAmmo(0)) != None)
			Ammo_AkimboBase(Weapon_Base(TempWeapon).GetAmmo(0)).AmmoSec = Weapon_Base(TempWeapon).GetAmmo(0).Default.MaxAmmo;
	}

	// restore grenades
	if (DesiredEquip[3].Name != 'War_Weapon_LAWBase' && DesiredEquip[3] != None && FOPawn(Pawn).FindWeapon(4) == None)
	{
		FOPawn(Pawn).EquipList[3] = DesiredEquip[3];
		FOPawn(Pawn).AddNewInventory(DesiredEquip[3],false,0);
	}
	if (DesiredEquip[3].Name == 'War_Weapon_LAWBase')
		DesiredEquip[4] = None;
	else if (DesiredEquip[4] != None && FOPawn(Pawn).FindWeapon(5) == None)
	{
		FOPawn(Pawn).EquipList[4] = DesiredEquip[4];
		FOPawn(Pawn).AddNewInventory(DesiredEquip[4],false,5);
	}

	// restore armor
	FOPawn(Pawn).ResetWarDamage();
}

function ServerAddToClip(optional bool bMaxed, optional byte AmmoGroup)
{
	local Weapon TempWeapon;
	
	if (Pawn == None || FOPawn(Pawn) == None || FOPawn(Pawn).CurrentBuildZone == None)
		return;
	
	if (!bAmmoRestored)
	{
		bAmmoRestored = true;

		// restore ammunition
		TempWeapon = FOPawn(Pawn).FindWeapon(4);

		if (TempWeapon != None && War_Weapon_LAWBase(TempWeapon) != None)
			Weapon_Base(TempWeapon).GetAmmo(0).AmmoAmount = Weapon_Base(TempWeapon).GetAmmo(0).Default.MaxAmmo;

		TempWeapon = FOPawn(Pawn).FindWeapon(3);
		if (TempWeapon != None)
			Weapon_Base(TempWeapon).GetAmmo(0).NumClips = Weapon_Base(TempWeapon).GetAmmo(0).Default.MaxClips;

		TempWeapon = FOPawn(Pawn).FindWeapon(2);
		if (TempWeapon != None)
			Weapon_Base(TempWeapon).GetAmmo(0).NumClips = Weapon_Base(TempWeapon).GetAmmo(0).Default.MaxClips;

		// restore grenades
		if (DesiredEquip[3].Name != 'War_Weapon_LAWBase' && DesiredEquip[3] != None && FOPawn(Pawn).FindWeapon(4) == None)
		{
			FOPawn(Pawn).EquipList[3] = DesiredEquip[3];
			FOPawn(Pawn).AddNewInventory(DesiredEquip[3],false,0);
		}
		if (DesiredEquip[3].Name == 'War_Weapon_LAWBase')
			DesiredEquip[4] = None;
		else if (DesiredEquip[4] != None && FOPawn(Pawn).FindWeapon(5) == None)
		{
			FOPawn(Pawn).EquipList[4] = DesiredEquip[4];
			FOPawn(Pawn).AddNewInventory(DesiredEquip[4],false,5);
		}

		// sound
		AmmoSound();

		// restore armor
		FOPawn(Pawn).ResetWarDamage();
		ReceiveLocalizedMessage(class'War_Interface_BuildZoneMessage',1);
	}
}

state PlayerDriving
{
	// Set the throttle, steering etc. for the vehicle based on the input provided
	function ProcessDrive(float InForward, float InStrafe, float InUp, bool InJump)
	{
		local Vehicle CurrentVehicle;
		local War_FOVehicleAir WVA;

		CurrentVehicle = Vehicle(Pawn);
		WVA = War_FOVehicleAir(CurrentVehicle);

		if (CurrentVehicle == None || (WVA != None && !WVA.bEngineStarted))
			return;

		//log("Forward:"@InForward@" Strafe:"@InStrafe@" Up:"@InUp);

		CurrentVehicle.Throttle = FClamp( InForward/5000.0, -1.0, 1.0 );
		CurrentVehicle.Steering = FClamp( -InStrafe/5000.0, -1.0, 1.0 );
		CurrentVehicle.Rise = FClamp( InUp/5000.0, -1.0, 1.0 );
	}

	exec function SpecialFire()
	{
		if (War_FOVehicleWheeled(Pawn) != None)
			War_FOVehicleWheeled(Pawn).SpecialFire();
		else if (War_FOVehicleAir(Pawn) != None)
			War_FOVehicleAir(Pawn).SpecialFire();
	}
}

state PlayerTurreting
{
	exec function Reload()
	{
		if (War_FOVehicleWeapon(War_FOVehiclePawn(Pawn).Gun) != None)
			War_FOVehicleWeapon(War_FOVehiclePawn(Pawn).Gun).PlayReload();
	}
}

state PlayerStaticTurreting extends PlayerTurreting
{
	function PlayerMove(float DeltaTime)
	{
		local Vehicle CurrentVehicle;
		
		if (Pawn == None)
		{
			GotoState('dead');
			return;
		}

		Pawn.UpdateRocketAcceleration(DeltaTime, aTurn, aLookUp);
		if (!bFreeCamera)
			SetRotation(War_Vehicle_M240MG(Pawn).TurretRotation);

		ViewShake(DeltaTime);
		ViewFlash(DeltaTime);

		if (Role < ROLE_Authority) // then save this move and replicate it
			ReplicateMove(DeltaTime, Pawn.Acceleration, DCLICK_None, rot(0,0,0));
		else
			ProcessMove(DeltaTime, Pawn.Acceleration, DCLICK_None, rot(0,0,0));

		CurrentVehicle = Vehicle(Pawn);
		if (CurrentVehicle != None )
		{
			if (bFire == 0 && CurrentVehicle.bWeaponIsFiring)
				CurrentVehicle.ClientVehicleCeaseFire(false);

			if (bAltFire == 0 && CurrentVehicle.bWeaponIsAltFiring)
				CurrentVehicle.ClientVehicleCeaseFire(true);
		}	
	}	

	exec function Reload()
	{
		if (War_Vehicle_M240MG(Pawn) != None && War_Vehicle_M240MG(Pawn).MGWeapon != None)
			War_FOVehicleWeapon(War_Vehicle_M240MG(Pawn).MGWeapon).PlayReload();
	}
}

state PlayerGunning extends PlayerTurreting
{
	function bool NotifyPhysicsVolumeChange(PhysicsVolume NewVolume)
	{
		return false;
	}

	exec function Reload()
	{
		Global.Reload();
	}
	
	function PlayerMove(float DeltaTime)
	{
		if (Pawn == None)
		{
			GotoState('Dead');
			return;
		}

		Pawn.UpdateRocketAcceleration(DeltaTime, aTurn, aLookUp);

		UpdateRotation(DeltaTime,2);

		if (Role < ROLE_Authority) // then save this move and replicate it
		    ReplicateMove(DeltaTime, Pawn.Acceleration, DCLICK_None, rot(0,0,0));
		else
		    ProcessMove(DeltaTime, Pawn.Acceleration, DCLICK_None, rot(0,0,0));
	}

	function UpdateRotation(float DeltaTime, float maxPitch)
	{
		local rotator NewRotation, ViewRotation, LastRotation;
		local float NewLookUp, NewTurn;

		if (bInterpolating || ((Pawn != None) && Pawn.bInterpolating))
		{	
			ViewShake(deltaTime);
			return;
		}
		
		// Added FreeCam control for better view control
		if (bFreeCam)
		{
			if (bFreeCamZoom)
				CameraDeltaRad += DeltaTime * 0.25 * aLookUp;
			else if (bFreeCamSwivel)
			{
				CameraSwivel.Yaw += 16.0 * DeltaTime * aTurn;
				CameraSwivel.Pitch += 16.0 * DeltaTime * aLookUp;
			}
			else
			{
				CameraDeltaRotation.Yaw += 32.0 * DeltaTime * aTurn;
				CameraDeltaRotation.Pitch += 32.0 * DeltaTime * aLookUp;
			}
		}
		else
		{
			ViewRotation = Rotation;
			if (Pawn != None && (Pawn.Physics != PHYS_Flying || Pawn.OnLadder != None)) // mmmmm
			{
				// Ensure we are not setting the pawn to a rotation beyond its desired
				if (Pawn.DesiredRotation.Roll < 65535 &&
					(ViewRotation.Roll < Pawn.DesiredRotation.Roll || ViewRotation.Roll > 0))
					ViewRotation.Roll = 0;
				else if (Pawn.DesiredRotation.Roll > 0 &&
					(ViewRotation.Roll > Pawn.DesiredRotation.Roll || ViewRotation.Roll < 65535))
					ViewRotation.Roll = 0;
			}

			DesiredRotation = ViewRotation; //save old rotation

			TurnTarget = None;
			bRotateToDesired = false;
			bSetTurnRot = false;

			ViewRotation.Yaw += 32.0 * DeltaTime * aTurn;
			ViewRotation.Pitch += 32.0 * DeltaTime * aLookUp;

			LastRotation.Yaw = ViewRotation.Yaw;
			LastRotation.Pitch = ViewRotation.Pitch;

			if (Pawn != None)
			{
				// limit yaw too
				if (War_FOVehiclePawn(Pawn) != None)
					ViewRotation.Yaw = War_FOVehiclePawn(Pawn).LimitYaw(ViewRotation.Yaw);
				else if (War_Vehicle_M240MG(Pawn) != None)
					ViewRotation.Yaw = War_Vehicle_M240MG(Pawn).LimitYaw(ViewRotation.Yaw);
				ViewRotation.Pitch = Pawn.LimitPitch(ViewRotation.Pitch);
			}

			if (Level.NetMode != NM_DedicatedServer)
			{
				NewTurn = aTurn;
				NewLookUp = aLookup;
				if (LastRotation.Pitch != ViewRotation.Pitch)
					NewLookUp = 0;
				if (LastRotation.Yaw != ViewRotation.Yaw)
					NewTurn = 0;

				if (Pawn != None && Pawn.Weapon != None)
					Weapon_Base(Pawn.Weapon).SetTurnPivot(NewTurn, NewLookUp);

				LastSlashDir = UpdateSlashDir();
			}

			SetRotation(ViewRotation);

			ViewShake(deltaTime);
			ViewFlash(deltaTime);

			NewRotation = ViewRotation;
		}
	}

	exec function BehindView(bool B)
	{
		if ((Level.NetMode == NM_Standalone) || (IsDead() && FragOps_GRI(GameReplicationInfo).bAllowBehindview) || PlayerReplicationInfo.bAdmin || IsA('Admin') || Vehicle(Pawn) != None)
		{
			if ((Vehicle(Pawn) == None) || (Vehicle(Pawn).bAllowViewChange) || (Level.NetMode == NM_StandAlone))
			{
				ClientSetBehindView(B);
				bBehindView = B;
			}
		}
	}

	exec function SwitchWeapon(byte F)
	{
		War_FOVehiclePawnFree(Pawn).NewSwitchWeapon(F);
	}

	function BeginState()
	{
		Super.BeginState();
		bZeroRoll = false;
	}

	function EndState()
	{
		Super.BeginState();
		bZeroRoll = true;
	}
}

function ClientRestart(Pawn RestartPawn)
{
	Super.ClientRestart(RestartPawn);
	if (bBuyMenuOpen && RestartPawn != None && !IsInState('PreRound'))
		Player.GUIController.CloseMenu(false);
	if (FOPawn(RestartPawn) != None)
		bBehindView = false;
}

exec function Speech(name Type, int Index, string Callsign)
{
	local FOPawn FP;
	
	if (Pawn != None)
	{
		if (Vehicle(Pawn) != None && Vehicle(Pawn).Driver != None)
			FP = FOPawn(Vehicle(Pawn).Driver);
		else
			FP = FOPawn(Pawn);
		
		// fallback
		if (FP == None)
			return;

		if (FP.VoiceClass != None)
			FP.VoiceClass.static.PlayerSpeech(Type, Index, Callsign, Self);
	}
}

exec function NextWeapon()
{
	if (Vehicle(Pawn) == None)
		Super.NextWeapon();
}

exec function PrevWeapon()
{
	if (Vehicle(Pawn) == None)
		Super.PrevWeapon();
}

exec function SwitchWeapon(byte F)
{
	if (Vehicle(Pawn) == None)
		Super.SwitchWeapon(F);
}

exec function SwitchFire(optional int i)
{
	if (Vehicle(Pawn) == None || War_FOVehiclePawnFree(Pawn) != None)
	{
		Super.SwitchFire(i);
		return;
	}

	if (i > 0)
		Pawn.NextWeapon();
	else
		Pawn.PrevWeapon();
}

exec function BehindView(Bool B)
{
	if ((Level.NetMode == NM_Standalone) || (IsDead() && FragOps_GRI(GameReplicationInfo).bAllowBehindview) || PlayerReplicationInfo.bAdmin || IsA('Admin') || Vehicle(Pawn) != None)
	{
		if ((Vehicle(Pawn) == None) || (Vehicle(Pawn).bAllowViewChange) || (Level.NetMode == NM_StandAlone))
		{
			ClientSetBehindView(B);
			bBehindView = B;
		}
	}
}

// special action assignment for helicopters
exec function Action()
{
	if (Pawn != None && War_FOVehicleAir(Pawn) != None)
			War_FOVehicleAir(Pawn).ServerActivateFunction(1);
	else
		Super.Action();
}

function BroadcastInitialObjective()
{
	local War_LevelInfo WLI;
	local string ObjString;
	local Material ObjMaterial;
	
	WLI = FragOpsWar(Level.Game).WarLevelInfo;
	if (WLI.bTeamDeathMatch)
	{
		if (PlayerReplicationInfo.Team.TeamIndex == 0)
			ObjString = WLI.TDMMercBriefing;
		else
			ObjString = WLI.TDMSOBriefing;
		ObjMaterial = WLI.TDMBriefIcon;
	}
	else if (PlayerReplicationInfo.Team.TeamIndex == 0)
	{
		if (WLI.M_Obj.Length > 0)
		{
			ObjString = WLI.M_Obj[0].Objective.AttackTeam_ObjectiveInfo.StartedBriefing;
			ObjMaterial = WLI.M_Obj[0].Objective.AttackTeam_ObjectiveInfo.StartedPicture;
			Objective = WLI.M_Obj[0].Objective;
		}
		else if (WLI.M_CaptureObj.Length > 0)
		{
			ObjString = WLI.M_CaptureObj[0].Objective.AttackTeam_ObjectiveInfo.StartedBriefing;
			ObjMaterial = WLI.M_CaptureObj[0].Objective.AttackTeam_ObjectiveInfo.StartedPicture;
			Objective = WLI.M_CaptureObj[0].Objective;
		}
		else if (WLI.S_Obj.Length > 0)
		{
			ObjString = WLI.S_Obj[0].Objective.DefendTeam_ObjectiveInfo.StartedBriefing;
			ObjMaterial = WLI.S_Obj[0].Objective.AttackTeam_ObjectiveInfo.StartedPicture;
			Objective = WLI.S_Obj[0].Objective;
		}
		else
			return;
	}
	else
	{
		if (WLI.S_Obj.Length > 0)
		{
			ObjString = WLI.S_Obj[0].Objective.AttackTeam_ObjectiveInfo.StartedBriefing;
			ObjMaterial = WLI.S_Obj[0].Objective.AttackTeam_ObjectiveInfo.StartedPicture;
			Objective = WLI.S_Obj[0].Objective;
		}
		else if (WLI.S_CaptureObj.Length > 0)
		{
			ObjString = WLI.S_CaptureObj[0].Objective.AttackTeam_ObjectiveInfo.StartedBriefing;
			ObjMaterial = WLI.S_CaptureObj[0].Objective.AttackTeam_ObjectiveInfo.StartedPicture;
			Objective = WLI.S_CaptureObj[0].Objective;
		}
		else if (WLI.M_Obj.Length > 0)
		{
			ObjString = WLI.M_Obj[0].Objective.DefendTeam_ObjectiveInfo.StartedBriefing;
			ObjMaterial = WLI.M_Obj[0].Objective.AttackTeam_ObjectiveInfo.StartedPicture;
			Objective = WLI.M_Obj[0].Objective;
		}
		else
			return;
	}
	NetUpdateTime = Level.TimeSeconds - 1;
	ClientSplashObjective(ObjString, ObjMaterial);
}

function Possess(Pawn aPawn)
{
	Super.Possess(aPawn);
	if (Vehicle(aPawn) == None)
		bBehindView = false;

	if (aPawn.bSetPCRotOnPossess)
		ActionPoints = GetMaxAP();
}

state PlayerWalking
{
	function BeginState()
	{
		Super.BeginState();
		bBehindView = false;
	}

	function PlayerMove(float DeltaTime)
	{
		Super.PlayerMove(DeltaTime);
		if (Level.NetMode != NM_StandAlone)
			bBehindView = false;
	}
	function ProcessMove(float DeltaTime, vector NewAccel, eDoubleClickDir DoubleClickMove, rotator DeltaRot)
	{
		Super.ProcessMove(DeltaTime, NewAccel, DoubleClickMove, DeltaRot);

		if (Level.NetMode != NM_StandAlone)
			bBehindView = false;
	}
}

function ServerViewNextPlayer()
{
	local Controller C, Pick;
	local bool bFound, bWasSpec;

	if (Pawn != None && ViewTarget != Pawn && Pawn.Health > 0)
	{
		SetViewTarget(Pawn);
		ClientSetViewTarget(Pawn);
		bBehindView = false;
		ClientSetBehindView(false);
	}

	bWasSpec = !bBehindView && (ViewTarget != Pawn) && (ViewTarget != self);
    
	// view next player
	for (C=Level.ControllerList; C!=None; C=C.NextController)
	{
		if (FragOpsGame(Level.Game).CanSpectate(self,false,C) && C != None && C != self && C.Pawn != None && C.Pawn.Health > 0)
		{
			if (Pick == None)
				Pick = C;
			if (bFound)
			{
				Pick = C;
				break;
			}
			else
				bFound = ((Vehicle(C.Pawn) == None && ((RealViewTarget == C.Pawn) || (ViewTarget == C.Pawn))) || (Vehicle(C.Pawn) != None && ((RealViewTarget == Vehicle(C.Pawn).Driver) || (ViewTarget == Vehicle(C.Pawn).Driver))));
		}
	}

	if (Pawn != None && Pawn.Health > 0)
	{
		SetViewTarget(Pawn);
		ClientSetViewTarget(Pawn);
	}
	else if (Pick != None && Pick.Pawn != None)
	{
		if (Vehicle(Pick.Pawn) == None)
		{
			if (SpectatorController == None)
				SpectatorController = spawn(class'FOSpectatorController',self);
			SpectatorController.InitializeFor(FOPawn(Pick.Pawn));
			SetViewTarget(Pick.Pawn);
			ClientSetViewTarget(Pick.Pawn);
		}
		else
		{
			if (SpectatorController != None)
				SpectatorController.Destroy();
			SetViewTarget(Vehicle(Pick.Pawn).Driver);
			ClientSetViewTarget(Vehicle(Pick.Pawn).Driver);
		}
	}
	if (ViewTarget != None && Pawn(ViewTarget) != None && Pawn(ViewTarget).DrivenVehicle != None)
		bBehindView = true;
	else if ((ViewTarget == self) || (Pawn != None && Pawn.Health > 0 && ViewTarget == Pawn) || bWasSpec || (Pawn(ViewTarget) != None && ViewTarget != Pawn && !FragOps_GRI(GameReplicationInfo).bAllowBehindView))
		bBehindView = false;
	else if (FragOps_GRI(GameReplicationInfo).bAllowBehindView)
		bBehindView = true; //bChaseCam;

	ClientSetBehindView(bBehindView);
}


// misc states
state PreRound
{
	function BeginState()
	{
		Super.BeginState();

		if (Role == ROLE_Authority)
		{
			Objective = None;
			RoundResupply();
			bAmmoRestored = true;
			bArmorChanged = true;
			bAlreadyRespawned = true;
		}
		CurWarObjective = None;
	}

	function ServerThrowWeapon()
	{
	}
}

function SplashCurrentObjective()
{
	local War_LevelInfo WLI;
	local string ObjString;
	local Material ObjMaterial;
	local int LastMObjLink, LastSObjLink, LastMCaptureLink, LastSCaptureLink;

	WLI = FragOpsWar(Level.Game).WarLevelInfo;
	LastMObjLink = Max(WLI.LastMObjLink, 0);
	LastSObjLink = Max(WLI.LastSObjLink, 0);
	LastMCaptureLink = Max(WLI.LastMCaptureLink, 0);
	LastSCaptureLink = Max(WLI.LastSCaptureLink, 0);
	if (WLI.bTeamDeathMatch)
	{
		if (!bHaveSpawned)
		{
			if (PlayerReplicationInfo.Team.TeamIndex == 0)
				ObjString = WLI.TDMMercBriefing;
			else
				ObjString = WLI.TDMSOBriefing;
			ObjMaterial = WLI.TDMBriefIcon;
		}
		bHaveSpawned = true;
	}
	else if (PlayerReplicationInfo.Team.TeamIndex == 0)
	{
		if (WLI.M_Obj.Length > 0)
		{
			if (WLI.M_Obj[LastMObjLink].Objective.bCompleted)
			{
				ObjString = WLI.M_Obj[LastMObjLink].Objective.AttackTeam_ObjectiveInfo.CompletedBriefing;
				ObjMaterial = WLI.M_Obj[LastMObjLink].Objective.AttackTeam_ObjectiveInfo.CompletedPicture;
			}
			else
			{
				ObjString = WLI.M_Obj[LastMObjLink].Objective.AttackTeam_ObjectiveInfo.StartedBriefing;
				ObjMaterial = WLI.M_Obj[LastMObjLink].Objective.AttackTeam_ObjectiveInfo.StartedPicture;
			}
			Objective = WLI.M_Obj[LastMObjLink].Objective;
		}
		else if (WLI.M_CaptureObj.Length > 0)
		{
			if (WLI.M_CaptureObj[LastMCaptureLink].Objective.bCompleted)
			{
				ObjString = WLI.M_CaptureObj[LastMCaptureLink].Objective.AttackTeam_ObjectiveInfo.CompletedBriefing;
				ObjMaterial = WLI.M_CaptureObj[LastMCaptureLink].Objective.AttackTeam_ObjectiveInfo.CompletedPicture;
			}
			else
			{
				ObjString = WLI.M_CaptureObj[LastMCaptureLink].Objective.AttackTeam_ObjectiveInfo.StartedBriefing;
				ObjMaterial = WLI.M_CaptureObj[LastMCaptureLink].Objective.AttackTeam_ObjectiveInfo.StartedPicture;
			}
			Objective = WLI.M_CaptureObj[LastMCaptureLink].Objective;
		}
		else if (WLI.S_Obj.Length > 0)
		{
			if (WLI.S_Obj[LastSObjLink].Objective.bCompleted)
			{
				ObjString = WLI.S_Obj[LastSObjLink].Objective.DefendTeam_ObjectiveInfo.CompletedBriefing;
				ObjMaterial = WLI.S_Obj[LastSObjLink].Objective.DefendTeam_ObjectiveInfo.CompletedPicture;
			}
			else
			{
				ObjString = WLI.S_Obj[LastSObjLink].Objective.DefendTeam_ObjectiveInfo.StartedBriefing;
				ObjMaterial = WLI.S_Obj[LastSObjLink].Objective.DefendTeam_ObjectiveInfo.StartedPicture;
			}
			Objective = WLI.S_Obj[LastSObjLink].Objective;
		}
		else
			return;
	}
	else
	{
		if (WLI.S_Obj.Length > 0)
		{
			if (WLI.S_Obj[LastSObjLink].Objective.bCompleted)
			{
				ObjString = WLI.S_Obj[LastSObjLink].Objective.AttackTeam_ObjectiveInfo.CompletedBriefing;
				ObjMaterial = WLI.S_Obj[LastSObjLink].Objective.AttackTeam_ObjectiveInfo.CompletedPicture;
			}
			else
			{
				ObjString = WLI.S_Obj[LastSObjLink].Objective.AttackTeam_ObjectiveInfo.StartedBriefing;
				ObjMaterial = WLI.S_Obj[LastSObjLink].Objective.AttackTeam_ObjectiveInfo.StartedPicture;
			}
			Objective = WLI.S_Obj[LastSObjLink].Objective;
		}
		else if (WLI.S_CaptureObj.Length > 0)
		{
			if (WLI.S_CaptureObj[LastSCaptureLink].Objective.bCompleted)
			{
				ObjString = WLI.S_CaptureObj[LastSCaptureLink].Objective.AttackTeam_ObjectiveInfo.CompletedBriefing;
				ObjMaterial = WLI.S_CaptureObj[LastSCaptureLink].Objective.AttackTeam_ObjectiveInfo.CompletedPicture;
			}
			else
			{
				ObjString = WLI.S_CaptureObj[LastSCaptureLink].Objective.AttackTeam_ObjectiveInfo.StartedBriefing;
				ObjMaterial = WLI.S_CaptureObj[LastSCaptureLink].Objective.AttackTeam_ObjectiveInfo.StartedPicture;
			}
			Objective = WLI.S_CaptureObj[LastSCaptureLink].Objective;
		}
		else if (WLI.M_Obj.Length > 0)
		{
			if (WLI.M_Obj[LastMObjLink].Objective.bCompleted)
			{
				ObjString = WLI.M_Obj[LastMObjLink].Objective.DefendTeam_ObjectiveInfo.CompletedBriefing;
				ObjMaterial = WLI.M_Obj[LastMObjLink].Objective.DefendTeam_ObjectiveInfo.CompletedPicture;
			}
			else
			{
				ObjString = WLI.M_Obj[LastMObjLink].Objective.DefendTeam_ObjectiveInfo.StartedBriefing;
				ObjMaterial = WLI.M_Obj[LastMObjLink].Objective.DefendTeam_ObjectiveInfo.StartedPicture;
			}
			Objective = WLI.M_Obj[LastMObjLink].Objective;
		}
		else
			return;
	}
	NetUpdateTime = Level.TimeSeconds - 1;
	if (ObjMaterial != None && ObjString != "")
		ClientSplashObjective(ObjString, ObjMaterial);
}

state JustJoined
{
	exec function AltFire(optional float F)
	{
		ServerRestartPlayer();
	}

	// request restart from server
	function ServerRestartPlayer()
	{
		if ((PlayerReplicationInfo == None || PlayerReplicationInfo.Team == None) || (!FragOpsWar(Level.Game).WarLevelInfo.bTeamDeathMatch && (!FragOpsWar(Level.Game).InWaveWindow(PlayerReplicationInfo.Team.TeamIndex) || bAlreadyRespawned)))
			return;
			
		bAmmoRestored = true;
		bArmorChanged = true;
		PlayerReplicationInfo.bOutOfLives = false;
		PlayerReplicationInfo.NumLives = 0;
		FragOpsWar(Level.Game).RestartDeadPlayer(self);
		FragOpsWar(Level.Game).AddRejoin(PlayerReplicationinfo.Team.TeamIndex);
		bAlreadyRespawned = true;
		NetUpdateTime = Level.TimeSeconds - 1;
		SplashCurrentObjective();
	}

	function ProcessMove(float DeltaTime, vector NewAccel, eDoubleClickDir DoubleClickMove, rotator DeltaRot)
	{
		Super.ProcessMove(DeltaTime, NewAccel, DCLICK_None, DeltaRot);
		if (FOPawn(ViewTarget) != None && Pawn(ViewTarget).Health > 0 && Pawn(ViewTarget).DrivenVehicle == None && bBehindView)
		{
			bBehindView = false;
			ClientSetBehindView(false);
			if (SpectatorController == None)
				SpectatorController = spawn(class'FOSpectatorController',self);
			SpectatorController.InitializeFor(FOPawn(ViewTarget));
		}
	}

	function BeginState()
	{
		Super.BeginState();
		if (Level.NetMode != NM_StandAlone && Level.NetMode != NM_DedicatedServer && PlayerReplicationInfo != None && PlayerReplicationInfo.Team != None)
			BuyMenu();
	}
}

state Dead
{
	exec function AltFire(optional float F)
	{
		ServerRestartPlayer();
	}

	// request restart from server
	function ServerRestartPlayer()
	{
		if ((PlayerReplicationInfo == None || PlayerReplicationInfo.Team == None) || (!FragOpsWar(Level.Game).WarLevelInfo.bTeamDeathMatch && (!FragOpsWar(Level.Game).InWaveWindow(PlayerReplicationInfo.Team.TeamIndex) || bAlreadyRespawned)))
			return;
			
		bAmmoRestored = true;
		bArmorChanged = true;
		PlayerReplicationInfo.bOutOfLives = False;
		PlayerReplicationInfo.NumLives = 0;
		FragOpsWar(Level.Game).RestartDeadPlayer(self);
		FragOpsWar(Level.Game).AddRejoin(PlayerReplicationinfo.Team.TeamIndex);
		bAlreadyRespawned = true;
		NetUpdateTime = Level.TimeSeconds - 1;
		SplashCurrentObjective();
	}

	function ProcessMove(float DeltaTime, vector NewAccel, eDoubleClickDir DoubleClickMove, rotator DeltaRot)
	{
		Super.ProcessMove(DeltaTime, NewAccel, DCLICK_None, DeltaRot);
		if (FOPawn(ViewTarget) != None && Pawn(ViewTarget).Health > 0 && Pawn(ViewTarget).DrivenVehicle == None && bBehindView)
		{
			bBehindView = false;
			ClientSetBehindView(false);
			if (SpectatorController == None)
				SpectatorController = spawn(class'FOSpectatorController',self);
			SpectatorController.InitializeFor(FOPawn(ViewTarget));
		}
	}
}

defaultproperties
{
	// default skill settings
	S_Structures=88
	S_Mechanic=88
	S_Weapons=88
	
	// standard equipment
	DesiredEquip(0)=class'Weapon_FistsBase'
	DesiredMiscItem="MED. Armor"
	
	PlayerReplicationInfoClass=class'FragOps.War_PRI'

	TeamBeaconMaxDist=2000.f
	TeamBeaconPlayerInfoMaxDist=1000.f
	TeamBeaconTeamColors(0)=(R=250,G=65,B=65,A=255)
	TeamBeaconTeamColors(1)=(R=80,G=100,B=240,A=255)
	TeamBeaconCustomColor=(R=255,G=240,B=170,A=255)

	bFirstLoadout=true
	bAutoBriefing=true
}