//=============================================================================
// Frag.Ops  2004 Matt 'SquirrelZero' Farber
//=============================================================================
// The Frag.Ops helicopter.
//=============================================================================

#exec OBJ LOAD FILE=IndoorAmbience.uax

class War_FOVehicleAir extends ONSChopperCraft
	abstract;

// placement sound
var sound PlacingSound;

// buyzone stuff
var War_BuildZone MyBZ;
var int BZSlot;

// copter rotor
var War_FOVehicleAirRotor Rotor;
var int RotorSpeed, MaxRotorSpeed;
var rotator RotorRot, RearRotorRot;
var class<Projectile> ProjClass;
var array<vector> RotorDustOffset;
var float RotorDustTraceDistance, StartupMod;
var array<ONSHoverBikeHoverDust> RotorDust;
var array<vector> RotorDustLastNormal;
var bool bOverWater;

// booleans
var bool bEngineStarted, bLightsOn, bGearOut;

// -- weaponry --
var float FireDelay, AltFireDelay;
var int AmmoCount;
var string FireType, AmmoType;
var bool bFiredLast;
var sound FireSound;
var Material ReticleTex;
var TexRotator AimerTex;
var Material LockTex;

// restrict damage to this vehicle
var bool bRestrictDamage;

// size when destroyed
var float DestroyedScale;

// destroy after a while
var float TimeOutSpan;

// engine pitch
var float MaxPitchSpeed;

// gravity control
var float GravScale;

// alternate, lower-detail kparams for destruction
var KarmaParams AltKParams;

// sound limiters for running over
var float LastRanIntoTime;
var Actor LastRanInto;

replication
{
	reliable if (Role < ROLE_Authority)
		ServerActivateFunction;

	reliable if (bNetDirty && Role == ROLE_Authority)
		bLightsOn, bEngineStarted, bGearOut, Rotor;

	reliable if (bNetOwner && bNetDirty && Role == ROLE_Authority)
		AmmoCount;
}

simulated function PostNetBeginPlay()
{
	Super.PostNetBeginPlay();
	if (HeadLightProjector != None)
		HeadLightProjector.MaxTraceDistance = 4048;
}

simulated function SpecialFire()
{
	ServerActivateFunction(2);
}

function ServerActivateFunction(byte Func)
{
	if (Func == 0 || Func == 255 || DrawType == DT_StaticMesh)
		return;

	if (Func == 1)
		ToggleEngine();
	else if (Func == 2)
		bLightsOn = !bLightsOn;
	else if (Func == 3 && !IsInState('StartingUp') && !IsInState('Firing'))
	{
		ToggleLandingGear();
		GotoState('AltFiring','Begin');	
	}
}

// Do some server-side vehicle firing stuff
function VehicleFire(bool bWasAltFire)
{
	if (!IsInState('Firing') && !IsInState('StartingUp') && !bGearOut && AmmoCount > 0 && !bWasAltFire)
	{
		FireWeapon();
		bWeaponIsFiring = true;
	}
}

function FireWeapon()
{
	AmmoCount -= 1;
	GotoState('Firing','Begin');
	DoFire();
}

function DoFire()
{
	local name WeaponBoneName;
	local vector FireLoc;
	local rotator FireRot;
	local Projectile P;

	// get fire location
	if (bFiredLast)
	{
		WeaponBoneName = 'Pod_Effect_Left';
		bFiredLast = false;
	}
	else
	{
		WeaponBoneName = 'Pod_Effect_Right';
		bFiredLast = true;
	}

	// set final rotation and location
	FireRot = Rotation;
	FireLoc = GetBoneCoords(WeaponBoneName).Origin + vector(FireRot)*5;

	// spawn
	PlaySound(FireSound,SLOT_Interact,TransientSoundVolume*3.0,,,1.25,false);
	P = spawn(ProjClass, self,, FireLoc, FireRot);
}

state Firing
{

Begin:
	Sleep(FireDelay);
	GotoState('');
}

function AltFire(optional float F)
{
	ServerActivateFunction(3);
}

state AltFiring extends Firing
{

Begin:
	Sleep(AltFireDelay);
	GotoState('');
}

function ToggleLandingGear()
{
	if (!bEngineStarted)
		return;
	
	if (bGearOut)
	{
		PlayAnim('TakeOff');
		PlaySound(Sound'IndoorAmbience.door13',,0.9,,210,1.05,true);
		GroundSpeed = Default.GroundSpeed*1.2;
	}
	else
	{
		PlayAnim('Land');
		PlaySound(Sound'IndoorAmbience.door2',,0.9,,210,1.3,true);
		GroundSpeed = Default.GroundSpeed*0.9;
	}
	bGearOut = !bGearOut;
}

event AnimEnd(int Channel)
{
}

simulated function UpdatePrecacheStaticMeshes()
{
	Super(SVehicle).UpdatePrecacheStaticMeshes();

	if (DestroyedVehicleMesh != None)
		Level.AddPrecacheStaticMesh(DestroyedVehicleMesh);
}

static function StaticPrecache(LevelInfo L)
{
	local int i;

	for(i=0;i<Default.DriverWeapons.Length;i++)
		Default.DriverWeapons[i].WeaponClass.static.StaticPrecache(L);

	for(i=0;i<Default.PassengerWeapons.Length;i++)
		Default.PassengerWeapons[i].WeaponPawnClass.static.StaticPrecache(L);

	if (Default.DestroyedVehicleMesh != None)
		L.AddPrecacheStaticMesh(Default.DestroyedVehicleMesh);

	if (Default.HeadlightCoronaMaterial != None)
		L.AddPrecacheMaterial(Default.HeadLightCoronaMaterial);

	if (Default.HeadlightProjectorMaterial != None)
		L.AddPrecacheMaterial(Default.HeadLightProjectorMaterial);

	L.AddPrecacheMaterial(default.VehicleIcon.Material);

	L.AddPrecacheMaterial(Material'EmitterTextures.MultiFrame.LargeFlames');
	L.AddPrecacheMaterial(Material'EmitterTextures.MultiFrame.fire3');
	L.AddPrecacheMaterial(Texture'AW-2004Particles.Weapons.DustSmoke');
}

simulated function PostBeginPlay()
{
	Super.PostBeginPlay();

	// nice placement sound
	if (Level.NetMode != NM_DedicatedServer)
	{
		if (!class'FOPlayer'.default.bTeamBeacons)
			bNoTeamBeacon = true;
		PlaySound(PlacingSound,,,,80,0.7,true);
	}

	// attach rotor blade
	if (Role == ROLE_Authority)
	{
		Rotor = spawn(class'War_Vehicle_ComancheRotor',self);
		if (Rotor != None)
			AttachToBone(Rotor,'Rotor_Blades_Bone');
	}
	SetBoneRotation('Rotor_Blades_Bone',RotorRot);
	SetBoneRotation('Object02',RearRotorRot);

	// start with landing gear
	PlayAnim('Land',3.0);
	MaxRiseForce = 0;
	MaxThrustForce = 0;
	bFollowLookDir = false;
	TurnTorqueMax = 0;
	RollTorqueMax = 0;
	bEngineStarted = false;
}

function bool TryToDrive(Pawn P)
{
	if (P.Weapon != None && (P.Weapon.IsFiring() || (Weapon_Base(P.Weapon).ZoomLevel > 0 && Weapon_Base(P.Weapon).ScopeHUDTex != None)))
		return false;

	return Super.TryToDrive(P);
}

// when engines turn on
function ToggleEngine()
{
	if (bEngineStarted)
	{
		AmbientSound = None;
		if (ShutDownSound != None && AmbientSound != None)
			PlaySound(ShutDownSound, SLOT_None, 1.0,,120);
		bEngineStarted = false;
		return;
	}

	bEngineStarted = true;	
	if (IdleSound != None)
		AmbientSound = IdleSound;

	if (StartUpSound != None)
		PlaySound(StartUpSound, SLOT_Misc, 1.1,,120);
	SoundVolume = 0;
	GotoState('StartingUp');
}

// disabled until i figure that weird bug out
//simulated function SwitchWeapon(byte F)
//{
//}

state StartingUp
{

Begin:
	Sleep(2.1);
	SoundVolume = Default.SoundVolume;
	GotoState('');
}

// enter
function KDriverEnter(Pawn P)
{
	local int x;

	ResetTime = Level.TimeSeconds - 1;
	Instigator = self;

	Super(SVehicle).KDriverEnter(P);

	if (Weapons.Length > 0)
		Weapons[ActiveWeapon].bActive = True;

	Driver.bSetPCRotOnPossess = false; //so when driver gets out he'll be facing the same direction as he was inside the vehicle

	for (x = 0; x < Weapons.length; x++)
	{
		if (Weapons[x] == None)
		{
			Weapons.Remove(x, 1);
			x--;
		}
		else
		{
			Weapons[x].NetUpdateFrequency = 20;
			ClientRegisterVehicleWeapon(Weapons[x], x);
		}
	}

	// save player replicationinfo
	P.PlayerReplicationInfo = Controller.PlayerReplicationInfo;
	SetTimer(0.1,false);
	LifeSpan = 0;
	KSetActorGravScale(GravScale);
	if (AIController(Controller) != None && !bEngineStarted)
		ToggleEngine();
}

function DriverLeft()
{
	if (ActiveWeapon < Weapons.Length)
	{
		Weapons[ActiveWeapon].bActive = False;
		Weapons[ActiveWeapon].AmbientSound = None;
	}

	if (ShutDownSound != None && AmbientSound != None)
		PlaySound(ShutDownSound, SLOT_Misc, 1.0,,120);

	if (AmbientSound != None)
		AmbientSound = None;

	Super(SVehicle).DriverLeft();
}

simulated function ClientKDriverEnter(PlayerController PC)
{
	Super.ClientKDriverEnter(PC);
	PC.EndZoom();
}

event bool KDriverLeave(bool bForceLeave)
{
	local Pawn OldDriver;

	// cannot leave while engine is started!
	if (bEngineStarted && !bForceLeave && Controller != None && AIController(Controller) == None)
		return false;

	// turn off if forced out
	if (bEngineStarted)
		ResetNonDrivingParams();

	OldDriver = Driver;
	if (Super.KDriverLeave(bForceLeave))
	{
		if (OldDriver != None && OldDriver.Base == self)
			OldDriver.SetBase(None);
		LifeSpan = Default.TimeOutSpan;
		return true;
	}
	return false;
}

event PhysicsVolumeChange(PhysicsVolume NewVolume)
{
	Super.PhysicsVolumeChange(NewVolume);
	if (NewVolume.bWaterVolume && Role == ROLE_Authority)
		TakeImpactDamage(50000);
}

simulated function Destroyed()
{
	local int i;
	
	Super.Destroyed();

	if (AltKParams != None)
		AltKParams = None;

	// release resources
	if (MyBZ != None)
		MyBZ.ResourceUsage[BZSlot] = Max(MyBZ.ResourceUsage[BZSlot]-1, 0);

	if (Level.NetMode != NM_DedicatedServer)
	{
		for (i=0;i<RotorDust.Length;i++)
			RotorDust[i].Destroy();

		RotorDust.Length = 0;
	}

	// destroy rotor
	if (Rotor != None)
		Rotor.Destroy();
}

event bool IsVehicleEmpty()
{
	local int i;

	if (Driver != None)
		return false;

	for (i=0; i<WeaponPawns.length; i++)
		if (WeaponPawns[i] != None && WeaponPawns[i].Driver != None )
			return false;

	return true;
}

simulated function Tick(float dt)
{
	local int i;
	local float EnginePitch, HitDist;
	local float TorqueMod;
	local vector TraceStart, TraceEnd, HitLocation, HitNormal;
	local actor HitActor;
	local KarmaParams kp;

	Super.Tick(dt);

	if (Health <= 0 || DrawType == DT_StaticMesh)
		return;

	// manually update driver
	if (Driver != None && FOPawn(Driver).bDriving && Driver.Health > 0 && !Driver.IsInState('Dying'))
	{
		Driver.bDoTorsoTwist = false;
		Driver.bRotateToDesired = false;

		Driver.SetPhysics(PHYS_None);
		Driver.SetLocation(Location + (DrivePos >> Rotation));
		Driver.SetRotation(Rotation);
	}

	// engine sounds
	EnginePitch = 64.0 + VSize(Velocity)/MaxPitchSpeed * 12.0;
        SoundPitch = FClamp(EnginePitch, 64, 76);

	// headlights
	if (Level.NetMode != NM_DedicatedServer)
	{
		for (i=0;i<HeadlightCorona.Length;i++)
		{
			if (!bLightsOn)
				HeadlightCorona[i].Skins[0] = None;
			else
				HeadlightCorona[i].Skins[0] = HeadlightCoronaMaterial;
		}

		if (HeadlightProjector != None)
		{
			if (!bLightsOn)
			{
				HeadLightProjector.MaxTraceDistance = 0;
				HeadLightProjector.ProjTexture = None;
			}
			else
			{
				HeadLightProjector.MaxTraceDistance = HeadLightProjector.Default.MaxTraceDistance;
				HeadLightProjector.ProjTexture = HeadlightProjectorMaterial;
			}
		}
	}

	// modify turning torque
	TorqueMod = FClamp(1.0-(VSize(Velocity)/(MaxRotorSpeed*0.1)), 0.33, 1.0);

	// add lift slowly
	if (bEngineStarted)
	{
		GravScale = KGetActorGravScale();
		StartupMod = FMin(StartupMod+(dt*0.15), 1.0);
		if (GravScale > 0)
		{
			GravScale = FClamp(GravScale-dt, 0, 3.7);
			KSetActorGravScale(GravScale);
		}
		RotorSpeed = FClamp(RotorSpeed + (dt*650), 0, MaxRotorSpeed);
		if (Rotor != None)
		{
			if (Level.NetMode != NM_DedicatedServer)
			{
				Rotor.EnableTrails();
				if (RotorSpeed == MaxRotorSpeed && Rotor.Mesh != Rotor.PlaneMesh)
				{
					Rotor.LinkMesh(Rotor.PlaneMesh);
					Rotor.bActorShadows = false;
				}

				bOverWater = false;
				kp = KarmaParams(KParams);
				for (i=0;i<kp.Repulsors.Length;i++)
				{
					if (kp.Repulsors[i].bRepulsorOnWater)
					{
						bOverWater = true;
						break;
					}
				}

				for (i=0;i<RotorDust.Length;i++)
				{
					RotorDust[i].bDustActive = false;

					TraceStart = Location + (RotorDustOffset[i] >> Rotation);
					TraceEnd = TraceStart - (RotorDustTraceDistance * vect(0,0,1));

					HitActor = Trace(HitLocation, HitNormal, TraceEnd, TraceStart, true);

					if (HitActor == None)
						RotorDust[i].UpdateHoverDust(false, 0);
					else
					{
						if (bOverWater)
							RotorDust[i].SetDustColor(Level.WaterDustColor);
						else
							RotorDust[i].SetDustColor(Level.DustColor);

						HitDist = VSize(HitLocation - TraceStart);

						RotorDust[i].SetLocation(HitLocation + 10*HitNormal);
						RotorDustLastNormal[i] = Normal(3*RotorDustLastNormal[i] + HitNormal);
						RotorDust[i].SetRotation(Rotator(RotorDustLastNormal[i]));

						// ramp up dust
						if (StartupMod != 1.0)
							RotorDust[i].UpdateHoverDust(true, FMin((HitDist/RotorDustTraceDistance)+(1.0-StartUpMod), 1.0));
						else
							RotorDust[i].UpdateHoverDust(true, (HitDist/RotorDustTraceDistance));

						// If dust is just turning on, set OldLocation to current Location to avoid spawn interpolation.
						if (!RotorDust[i].bDustActive)
							RotorDust[i].OldLocation = RotorDust[i].Location;

						RotorDust[i].bDustActive = true;
					}
				}
			}
		}
		MaxRiseForce = FClamp(MaxRiseForce+(dt*4.0), 0, Default.MaxRiseForce);
		MaxThrustForce = FClamp(MaxThrustForce+(dt*5.0), 0, Default.MaxThrustForce);
		TurnTorqueMax = FClamp(TurnTorqueMax+(dt*8.0), 0, Default.TurnTorqueMax*TorqueMod);
		RollTorqueMax = FClamp(RollTorqueMax+(dt*2.0), 0, Default.RollTorqueMax);
		bFollowLookDir = true;
	}
	else // shut off and kill lift quickly
	{
		StartupMod = 0;
		if (KGetActorGravScale() != 3.7)
		{
			GravScale = 3.7;
			KSetActorGravScale(GravScale);
		}
		if (Rotor != None)
		{
			if (Level.NetMode != NM_DedicatedServer)
			{
				Rotor.DisableTrails();
				if (Rotor.Mesh != Rotor.Default.Mesh)
				{
					Rotor.LinkMesh(Rotor.Default.Mesh);
					Rotor.bActorShadows = true;
				}

				for(i=0;i<RotorDust.Length;i++)
				{
					RotorDust[i].bDustActive = false;
					RotorDust[i].UpdateHoverDust(false, 0);
				}
			}
		}
		RotorSpeed = FClamp(RotorSpeed - (dt*1800), 0, MaxRotorSpeed);
		MaxRiseForce = FClamp(MaxRiseForce-(dt*30.0), 0, Default.MaxRiseForce);
		MaxThrustForce = FClamp(MaxThrustForce-(dt*35.0), 0, Default.MaxThrustForce);
		TurnTorqueMax = FClamp(TurnTorqueMax-(dt*80.0), 0, Default.TurnTorqueMax*TorqueMod);
		RollTorqueMax = FClamp(RollTorqueMax-(dt*20.0), 0, Default.RollTorqueMax);
		bFollowLookDir = false;
	}

	// spin top rotor
	RotorRot.Yaw += RotorSpeed;
	RotorRot.Yaw = RotorRot.Yaw & 65535;
	SetBoneRotation('Rotor_Blades_Bone',RotorRot);

	// now rear rotor
	RearRotorRot.Pitch += RotorSpeed*1.3;
	RearRotorRot.Pitch = RotorRot.Yaw & 65535;
	SetBoneRotation('Object02',RearRotorRot);

	// update impact threshold based on landing gear params
	if (bGearOut && KGetImpactThreshold() != 480)
		KSetImpactThreshold(480);
	if (!bGearOut)
	{
		if (bEngineStarted)
		{
			if (KGetImpactThreshold() != 175)
				KSetImpactThreshold(175);
		}
		else if (KGetImpactThreshold() != 20)
			KSetImpactThreshold(20);
	}
}

// some damage modifiers
event TakeDamage(int Damage, Pawn EventInstigator, vector HitLocation, vector Momentum, class<DamageType> DamageType)
{
	if (bRestrictDamage)
		return;

	Super.TakeDamage(Damage, EventInstigator, HitLocation, Momentum, DamageType);
}

event TakeImpactDamage(float AccelMag)
{
	local BlockingVolume BV;
	
	if (BlockingVolume(ImpactInfo.Other) != None)
	{
		BV = BlockingVolume(ImpactInfo.Other);
		if (BV.bClassBlocker && BV.BlockedClasses.Length == 0)
			return;
	}
	
	if (!bGearOut)
	{
		ONSTakeImpactDamage(AccelMag);
		return;
	}
	
	AccelMag *= 0.8;
	if (Location.Z-ImpactInfo.Pos.Z > 7)
	{
		if (AccelMag < 3000)
			return;
		AccelMag *= 0.8;
	}
	ONSTakeImpactDamage(AccelMag);
}

event ONSTakeImpactDamage(float AccelMag)
{
	local int Damage;

	Damage = int(AccelMag * ImpactDamageModifier());
	TakeDamage(Damage, Self, ImpactInfo.Pos, vect(0,0,0), class'DamType_VehicleKill');

	//FIXME - Scale sound volume to damage amount
	if (ImpactDamageSounds.Length > 0)
		PlaySound(ImpactDamageSounds[Rand(ImpactDamageSounds.Length-1)],,TransientSoundVolume*2.5);

	if (Health < 0 && (Level.TimeSeconds - LastImpactExplosionTime) > TimeBetweenImpactExplosions)
	{
		VehicleExplosion(Normal(ImpactInfo.ImpactNorm), 0.5);
		LastImpactExplosionTime = Level.TimeSeconds;
	}

	if ((Controller != None) && (KarmaBoostDest(Controller.MoveTarget) != None) && Controller.InLatentExecution(Controller.LATENT_MOVETOWARD) )
		Controller.MoveTimer = -1;
}

function Timer()
{
	if (Abs(Velocity.Z) > 50 || VSize(Velocity) > 700)
		Velocity.Z -= 40;
	if (Driver != None)
		SetTimer(0.1,false);
}

function Pawn GetHitPawn(vector HitLocation, rotator HitRotation)
{
	return None;
}

simulated event DrivingStatusChanged()
{
	local int i;
	local PlayerController PC;

	PC = Level.GetLocalPlayerController();
	if (bDriving && PC != None && (PC.ViewTarget == None || !(PC.ViewTarget.IsJoinedTo(self))))
		bDropDetail = (Level.bDropDetail || (Level.DetailMode == DM_Low));
	else
		bDropDetail = false;

	// always enable tick!
	Enable('Tick');
	if (!bDriving)
		ResetNonDrivingParams();
	else if (Level.NetMode != NM_DedicatedServer && RotorDust.Length == 0)
	{
		RotorDust.Length = RotorDustOffset.Length;
		RotorDustLastNormal.Length = RotorDustOffset.Length;

		for(i=0; i<RotorDustOffset.Length; i++)
			if (RotorDust[i] == None)
			{
				RotorDust[i] = spawn(class'War_Effect_RotorDust', self,, Location + (RotorDustOffset[i] >> Rotation));
				RotorDust[i].SetDustColor(Level.DustColor);
				RotorDustLastNormal[i] = vect(0,0,1);
				RotorDust[i].bDustActive = false;
				RotorDust[i].UpdateHoverDust(false, 0);
			}
	}
	else if (Level.NetMode != NM_DedicatedServer)
	{
		for(i=0;i<RotorDust.Length;i++)
			RotorDust[i].Destroy();

		RotorDust.Length = 0;
	}
}

simulated function ResetNonDrivingParams()
{
	if (Level.NetMode != NM_DedicatedServer && Rotor != None && Rotor.Mesh != Rotor.Default.Mesh)
	{
		Rotor.LinkMesh(Rotor.Default.Mesh);
		Rotor.bActorShadows = true;
	}

	// reset gravity scale
	if (KGetActorGravScale() != 3.7)
	{
		GravScale = 3.7;
		KSetActorGravScale(GravScale);
	}

	// update impact threshold based on landing gear params
	if (bGearOut && KGetImpactThreshold() != 480)
		KSetImpactThreshold(480);
	if (!bGearOut)
	{
		if (KGetImpactThreshold() != 20)
			KSetImpactThreshold(20);
	}

	AmbientSound = None;
	bEngineStarted = false;
}

simulated event DestroyAppearance()
{
	local int i;
	local KarmaParamsRBFull KP;

	if (Role == ROLE_Authority)
		PlaySound(sound'FOWeaponSounds.Misc.ShellHit8', SLOT_Misc, 0.1,,5);

	// For replication
	bDestroyAppearance = true;
	AmbientSound = None;

	// Put brakes on
	Throttle	= 0;
	Steering	= 0;
	Rise		= 0;

	// turn off
	bEngineStarted = false;

	// Destroy the weapons
	if (Role == ROLE_Authority)
	{
		for(i=0;i<Weapons.Length;i++)
			Weapons[i].Destroy();
		for(i=0;i<WeaponPawns.Length;i++)
			WeaponPawns[i].Destroy();
	}
	Weapons.Length = 0;
	WeaponPawns.Length = 0;

	// destroy the effects
	if (Level.NetMode != NM_DedicatedServer)
	{
		for (i=0;i<RotorDust.Length;i++)
			RotorDust[i].Destroy();

		RotorDust.Length = 0;
		bNoTeamBeacon = true;

		for(i=0;i<HeadlightCorona.Length;i++)
			HeadlightCorona[i].Destroy();
		HeadlightCorona.Length = 0;

		if(HeadlightProjector != None)
			HeadlightProjector.Destroy();
	}

	//  destroy rotor
	if (Rotor != None)
		Rotor.Destroy();

	// new karma params
	KP = KarmaParamsRBFull(KParams);
	if (KP != None)
		KP.KStartLinVel = Velocity;
	
	// Become the dead vehicle mesh
	SetPhysics(PHYS_None);
	KSetBlockKarma(False);
	SetDrawType(DT_StaticMesh);
	SetStaticMesh(DestroyedVehicleMesh);
	SetDrawScale(DestroyedScale);
	KSetBlockKarma(True);
	KSetActorGravScale(1.0);
	SetPhysics(PHYS_Karma);
	Skins.Length = 0;
	NetPriority = 2.5;
	NetUpdateTime = Level.TimeSeconds - 1;
	LifeSpan = 20.0;
}

event RanInto(Actor Other)
{
	local vector Momentum;
	local float Speed;
	local Pawn HitPawn;

	Momentum = Normal(Other.Location-Location);
	// in case of emergency...
	if (Effect_BreakingGlass(Other) != None)
	{
		Other.TakeDamage(Effect_BreakingGlass(Other).MaxDamage, Instigator, Other.Location, Momentum, RanOverDamageType);
		return;
	}
	else if (Pawn(Other) == None || Vehicle(Other) != None || Other == Instigator || Other.Role != ROLE_Authority)
		return;

	HitPawn = Pawn(Other);
	Speed = VSize(Velocity);
	if (HitPawn != None && HitPawn.PlayerReplicationInfo != None && HitPawn.PlayerReplicationInfo.Team.TeamIndex == GetTeamNum())
		MinRunOverSpeed = 500;
	else
		MinRunOverSpeed = 0;
	if (Speed > MinRunOverSpeed)
	{
		Momentum = Normal(Velocity);
		if (RanOverSound != None && (HitPawn == None || HitPawn.Health > 0) && Level.TimeSeconds - LastRanIntoTime > 0.5)
		{
			LastRanIntoTime = Level.TimeSeconds;
			if (MinRunOverSpeed != 500 || class'FragOpsMission'.Default.FriendlyFireScale > 0)
				PlaySound(RanOverSound,,TransientSoundVolume*2.5,,100);
		}
		Other.TakeDamage(int(Speed * 0.075), Instigator, Other.Location, Momentum, RanOverDamageType);
	}
	else
		Other = None;
}

simulated event Touch(Actor Other)
{
	local vector Momentum;

	Super.Touch(Other);

	// not on client
	if (Role < ROLE_Authority || Effect_BreakingGlass(Other) == None)
		return;

	Momentum = Normal(Other.Location-Location);
	Other.TakeDamage(Effect_BreakingGlass(Other).MaxDamage, Instigator, Other.Location, Momentum, RanOverDamageType);
}

// special reticle
simulated function SpecialDrawCrosshair(Canvas C)
{
	local vector V, L, X, CamPos, ReticleOffset, Start, End, HitNormal, HitLocation;
	local rotator CamRot;
	local rotator VehicleRot;
	local float fX, fY, MX, MY, LockScale;
	local Actor HitActor;

	// fallback
	if (Controller == None)
		return;

	// retrieve camera location and rotation
	C.GetCameraLocation(CamPos,CamRot);

	// draw elevation
	DrawElevation(C);

	// get a scale base
	MX = FClamp(C.ClipX/1024.0, 0.25, 2.0);
	MY = FClamp(C.ClipY/768.0, 0.25, 2.0);

	// only draw rocket indicators if landing gear has been retracted
	if (!bGearOut)
	{
		// set draw color
		X = vector(Rotation);
		C.DrawColor.G = 255;
		C.DrawColor.R = 210;
		C.DrawColor.B = 210;
		C.DrawColor.A = 220;
		
		// draw rocket indicators -- right
		Start = GetBoneCoords('Pod_Effect_Right').Origin + X*200;
		End = Start + X*7000;
		HitActor = Trace(HitLocation, HitNormal, End, Start, true, vect(30,30,30));

		if (HitActor == None)
			HitLocation = End;
		
		V = FOPlayer(Controller).Player.Console.WorldToScreen(HitLocation, CamPos, CamRot);
		LockScale = FClamp(1.0-VSize(Location-HitLocation)/5000.0, 0.25, 1.0);
		fX = V.X;
		fY = V.Y;
		fX = fX - 0.5*(30.0*(MX*LockScale));
		fY = fY - 0.5*(30.0*(MY*LockScale));
		if (fX < C.ClipX*0.99 && fX >= C.ClipX*0.01 && fY < C.ClipY*0.99 && fY >= C.ClipY*0.01)
		{
			C.SetPos(fX, fY);
			C.DrawTile(LockTex, 30.0*(MX*LockScale), 30.0*(MY*LockScale), 0.0, 0.0, 32.0, 32.0);
		}

		// draw rocket indicators -- left
		Start = GetBoneCoords('Pod_Effect_Left').Origin + X*200;
		End = Start + X*7000;
		HitActor = Trace(HitLocation, HitNormal, End, Start, true, vect(30,30,30));

		if (HitActor == None)
			HitLocation = End;
		
		L = Start;
		V = FOPlayer(Controller).Player.Console.WorldToScreen(HitLocation, CamPos, CamRot);
		LockScale = FClamp(1.0-VSize(Location-HitLocation)/5000.0, 0.25, 1.0);
		fX = V.X;
		fY = V.Y;
		fX = fX - 0.5*(30.0*(MX*LockScale));
		fY = fY - 0.5*(30.0*(MY*LockScale));
		if (fX < C.ClipX*0.99 && fX >= C.ClipX*0.01 && fY < C.ClipY*0.99 && fY >= C.ClipY*0.01)
		{
			C.SetPos(fX, fY);
			C.DrawTile(LockTex, 30.0*(MX*LockScale), 30.0*(MY*LockScale), 0.0, 0.0, 32.0, 32.0);
		}
	}

	// get the reticle location
	VehicleRot = Rotation;
	VehicleRot.Pitch = 0;
	ReticleOffset.X = 1800.0+(VSize(Velocity)*7.0);
	L = Location + (ReticleOffset >> VehicleRot);
	V = FOPlayer(Controller).Player.Console.WorldToScreen(L, CamPos, CamRot);
	fX = V.X;
	fY = V.Y;
	fX = fX - 0.5*(64.0*MX);
	fY = fY - 0.5*(256.0*MY);

	// set draw color
	C.DrawColor.G = 255;
	C.DrawColor.R = 150;
	C.DrawColor.B = 150;
	C.DrawColor.A = 155;

	// don't draw at edges
	if (V.X < C.ClipX && V.X >= 0 && V.Y < C.ClipY && V.Y >= 0)
	{
		// draw
		C.SetPos(fX, fY);
		C.DrawTile(ReticleTex, 64.0*MX, 256.0*MY, 0.0, 0.0, 128.0, 512.0);
	}

	// now the tricky part :)
	L = Location + (ReticleOffset >> Rotation);
	V = FOPlayer(Controller).Player.Console.WorldToScreen(L, CamPos, CamRot);
	fX = V.X;
	fY = V.Y;
	fX = fX - 0.5*(128.0*MX);
	fY = fY - 0.5*(128.0*MY);

	if (V.X < C.ClipX && V.X >= 0 && V.Y < C.ClipY && V.Y >= 0)
	{
		// set reticle rotation
		AimerTex.Rotation.Yaw = -(Rotation.Roll & 65535);

		// draw
		C.SetPos(fX, fY);
		C.DrawColor.R = 200;
		C.DrawColor.B = 200;
		C.DrawColor.A = 160;
		C.DrawTile(AimerTex, 128.0*MX, 128.0*MY, 0.0, 0.0, 128.0, 128.0);
	}
}

simulated function DrawElevation(Canvas C)
{
	local vector HitLocation, HitNormal, Start, End;
	local float GroundDist;

	// trace to ground
	Start = Location - vect(0,0,60);
	End = Start - vect(0,0,15000);
	bTraceWater = true;
	Trace(HitLocation, HitNormal, End, Start, false);
	bTraceWater = false;
	GroundDist = FClamp(VSize(Location-HitLocation)/15.0, 0, 15000.0/15.0);
	C.SetPos(C.ClipX*0.875, C.ClipY*0.88);
	C.DrawColor = class'Interface_MainHUD'.Default.WhiteColor;
	C.Font = class'Interface_MainHUD'.static.StaticGetSmallerAltFontFor(C,2);

	// draw text
	C.DrawText(int(GroundDist)$" feet", false);
}

// overridden -- unwanted
simulated event OnDaredevil()
{
}

simulated function PrecacheAnnouncer(AnnouncerVoice V, bool bRewardSounds)
{
}

function SpawnBuildEffect()
{
}

simulated function SetOverlayMaterial( Material mat, float time, bool bOverride)
{
}

simulated function POVChanged(PlayerController PC, bool bBehindViewChanged)
{
	if (PC.bBehindView)
	{
		bOwnerNoSee = false;

		if (Driver != None)
		{
			if (bDrawDriverInTP)
				Driver.bOwnerNoSee = false;
			else
				Driver.bOwnerNoSee = true;
		}

		if (PC == Controller)   // No overlays for spectators
		    ActivateOverlay(False);
	}
	else
	{
		if (bDrawMeshInFP)
			bOwnerNoSee = false;
		else
			bOwnerNoSee = true;

		if (Driver != None)
			Driver.bOwnerNoSee = true;

		if (bDriving && PC == Controller)   // No overlays for spectators
		    ActivateOverlay(True);
	}
}

defaultproperties
{
	// display
	Mesh=Mesh'FOVehicles.Comanche'
	VehicleIcon=(Material=Material'FOWarTextures.VehicleIcons.JeepIcon')
	DrawScale=1.0
	StolenSound=None

	// team beacon
	TeamBeaconTexture=Texture'FOWarTextures.HUD.HealthIcon'
	TeamBeaconBorderMaterial=Texture'FOWarTextures.HUD.HealthBorder'

	// health
	Health=400
	HealthMax=400

	// weaponry
	FireDelay=0.5

	// rotor
	MaxRotorSpeed=2400

	// sound
	IdleSound=sound'FOVehicleSounds.Comanche.ComancheRotor'
	StartUpSound=sound'ONSVehicleSounds-S.AttackCraft.AttackCraftStartUp'
	ShutDownSound=sound'ONSVehicleSounds-S.AttackCraft.AttackCraftShutDown'
	MaxPitchSpeed=2000
	SoundVolume=255
	SoundRadius=255
	
	// headlight defaults
	HeadlightProjectorMaterial=Texture'VMVehicles-TX.RVGroup.RVProjector'
	HeadlightProjectorScale=0.3

	// readables
	VehiclePositionString="in an RAH-66"
	VehicleNameString="RAH-66"
	
	// entry
	EntryPosition=(X=50,Y=0,Z=0)
	EntryRadius=210.0

	// exit
	ExitPositions(0)=(X=0,Y=-145,Z=10)
	ExitPositions(1)=(X=0,Y=145,Z=10)

	// yaw/turn properties
	TurnTorqueFactor=400.0
	TurnTorqueMax=155.0
	TurnDamping=60.0
	MaxYawRate=1.5
	
	// pitch properties
	PitchTorqueFactor=160.0
	PitchTorqueMax=70.0
	PitchDamping=15.0

	// roll properties
	RollTorqueTurnFactor=450.0
	RollTorqueStrafeFactor=50.0
	RollTorqueMax=50.0
	RollDamping=30.0	

	// other physics properties
	AccelRate=+0500.000000
	UprightStiffness=400
	UprightDamping=300
	MaxThrustForce=11.0
	LongDamping=0.015
	MaxStrafeForce=12.0
	LatDamping=0.02
	MaxRiseForce=5.5
	UpDamping=0.05

	MaxRandForce=0.600000
	RandForceInterval=1.25000

	// misc
	bPCRelativeFPRotation=true
	bCanStrafe=false
	bDrawDriverInTP=true
	bDrawMeshInFP=true
	bShowDamageOverlay=false
	bGearOut=true
	bCanBeBaseForPawns=false
	bFollowLookDir=false
	bHasAltFire=true
	bClientAnim=false
	bReplicateAnimations=true
	bDriverCollideActors=false

	// collision
	CollisionHeight=+70.0
	CollisionRadius=150.0

	// destruction
	DamagedEffectOffset=(X=0,Y=0,Z=40)
	ExplosionMomentum=1000
	DestructionEffectClass=class'Onslaught.ONSSmallVehicleExplosionEffect'
	DestructionLinearMomentum=(Min=50000,Max=70000)
	DestructionAngularMomentum=(Min=25,Max=50)
	DisintegrationEffectClass=class'FragOps.War_Effect_VehicleDeath'
	DisintegrationHealth=-6000
	RanOverDamageType=class'DamType_Roadkill'
	CrushedDamageType=class'DamType_Roadkill'
	ExplosionDamageType=class'DamType_DestroyedVehicle'
	DestroyedRoadKillDamageType=class'DamType_DestroyedVehicle'

	// impacts, very fragile
	ImpactDamageMult=0.036
	TimeOutSpan=75

	// driver-related
	MaxViewYaw=19000
	MaxViewPitch=19000
	DrivePos=(X=40.0,Y=0.0,Z=25.0)
	DriveAnim=driveg
	DriverDamageMult=0.30

	// dust
	RotorDustOffset(0)=(X=0,Y=0,Z=-20)
	RotorDustTraceDistance=300

	Begin Object Class=KarmaParamsRBFull Name=KParams0
		KStartEnabled=True
		KFriction=0.5
		KLinearDamping=0.0
		KAngularDamping=0.0
		KImpactThreshold=520
		bKNonSphericalInertia=True
		bHighDetailOnly=False
		bClientOnly=False
		bKDoubleTickRate=True
		bKStayUpright=True
		bKAllowRotate=True
		KInertiaTensor(0)=2.0
		KInertiaTensor(1)=0.0
		KInertiaTensor(2)=0.0
		KInertiaTensor(3)=4.0
		KInertiaTensor(4)=0.0
		KInertiaTensor(5)=4.5
		KCOMOffset=(X=-0.25,Y=0.0,Z=0.0)
		KActorGravScale=1.1
		bDestroyOnWorldPenetrate=true
		bDoSafetime=true
		KMaxAngularSpeed=0.400000
	Name="KParams0"
	End Object
	KParams=KarmaParams'KParams0'

	Begin Object Class=KarmaParamsRBFull Name=KParams1
		KStartEnabled=True
		KFriction=0.5
		KLinearDamping=0.0
		KAngularDamping=0.0
		KImpactThreshold=520
		bKNonSphericalInertia=True
		bHighDetailOnly=False
		bClientOnly=False
		bKDoubleTickRate=False
		bKStayUpright=False
		bKAllowRotate=True
		KInertiaTensor(0)=2.0
		KInertiaTensor(1)=0.0
		KInertiaTensor(2)=0.0
		KInertiaTensor(3)=4.0
		KInertiaTensor(4)=0.0
		KInertiaTensor(5)=4.5
		KCOMOffset=(X=-0.25,Y=0.0,Z=0.0)
		KActorGravScale=1.0
		bDestroyOnWorldPenetrate=true
		bDoSafetime=false
		KMaxSpeed=2500.0
	Name="KParams1"
	End Object
	AltKParams=KarmaParams'KParams1'
}