//=============================================================================
// Frag.Ops  2004 Matt 'SquirrelZero' Farber
//=============================================================================
class War_FOVehiclePawn extends ONSWeaponPawn;

function UpdatePawnLook(rotator NewRotation);

simulated function PostBeginPlay()
{
	Super.PostBeginPlay();

	// nice placement sound
	if (Level.NetMode != NM_DedicatedServer)
	{
		if (!class'FOPlayer'.default.bTeamBeacons)
			bNoTeamBeacon = true;
	}
}

// enter
function KDriverEnter(Pawn P)
{
	Super.KDriverEnter(P);

	// save player replicationinfo
	P.PlayerReplicationInfo = Controller.PlayerReplicationInfo;
	if (VehicleBase != None)
		VehicleBase.LifeSpan = 0;
}

event bool KDriverLeave(bool bForceLeave)
{
	local Pawn OldDriver;
	
	OldDriver = Driver;
	if (Super.KDriverLeave(bForceLeave))
	{
		if (OldDriver != None && OldDriver.Base == VehicleBase)
			OldDriver.SetBase(None);
		if (VehicleBase != None)
			VehicleBase.LifeSpan = War_FOVehicleWheeled(VehicleBase).Default.TimeOutSpan;
		return true;
	}
	return false;
}

simulated function ClientKDriverEnter(PlayerController PC)
{
	Super.ClientKDriverEnter(PC);
	PC.EndZoom();
}

function bool TryToDrive(Pawn P)
{
	if (P.Weapon != None && P.Weapon.IsFiring())
		return false;

	return Super.TryToDrive(P);
}

simulated function int LimitYaw(int InYaw)
{
	if (VehicleBase == None || Gun == None)
		return InYaw;
		
	return War_FOVehicleWeapon(Gun).LimitYaw(InYaw, VehicleBase.Rotation);	
}

simulated function AttachDriver(Pawn P)
{
	local vector AttachPos;

	AttachPos = Location + (DrivePos >> VehicleBase.Rotation);
	P.SetLocation(AttachPos);
	P.SetPhysics(PHYS_None);
	P.SetBoneRotation('Bip01 Spine', rot(0,0,0));
}

simulated function DetachDriver(Pawn P)
{
	P.SetBase(None);
}

// update rotation
function UpdateRocketAcceleration(float deltaTime, float YawChange, float PitchChange)
{
	local rotator NewRotation;
	local float PrevRecoil;
	local War_FOVehicleWeapon MyGun;

	NewRotation = Rotation;
	NewRotation.Yaw += 32.0 * deltaTime * YawChange;
	NewRotation.Pitch += 32.0 * deltaTime * PitchChange;
	NewRotation.Yaw = LimitYaw(NewRotation.Yaw);

	if (Gun != None)
	{
		MyGun = War_FOVehicleWeapon(Gun);
		NewRotation.Pitch += MyGun.InitialRecoil;
		PrevRecoil = MyGun.InitialRecoil;
		MyGun.InitialRecoil -= (PrevRecoil*deltaTime)*19.0;
		if (MyGun.InitialRecoil <= 0.0)
			MyGun.bHasRecoiled = false;
	}
	NewRotation.Pitch = LimitPitch(NewRotation.Pitch);
	SetRotation(NewRotation);
}

// see if driver was shot
function Pawn GetHitPawn(vector HitLocation, rotator HitRotation)
{
	local vector HitLoc;
	
	HitLoc = HitLocation;
	if (Driver != None)
	{
		HitLoc = HitLocation;
		HitLoc.Z = Driver.Location.Z;
		if (VSize(HitLocation-Driver.Location) <= Driver.CollisionRadius)
			return Driver;
	}
	return None;
}

function SetTeamNum(byte T)
{
	Super.SetTeamNum(T);
	if (Gun != None)
		War_FOVehicleWeapon(Gun).TeamIndex = T;
}

simulated function POVChanged(PlayerController PC, bool bBehindViewChanged)
{
	if (PC.bBehindView)
	{
		bOwnerNoSee = false;

		if (Driver != None)
		{
			if (bDrawDriverInTP)
				Driver.bOwnerNoSee = false;
			else
				Driver.bOwnerNoSee = true;
		}

		if (PC == Controller)   // No overlays for spectators
		    ActivateOverlay(False);
	}
	else
	{
		if (bDrawMeshInFP)
			bOwnerNoSee = false;
		else
			bOwnerNoSee = true;

		if (Driver != None)
			Driver.bOwnerNoSee = true;

		if (bDriving && PC == Controller)   // No overlays for spectators
		    ActivateOverlay(True);
	}
}

simulated function Tick(float dt)
{
	Super.Tick(dt);

	// manually update driver
	if (Driver != None && FOPawn(Driver).bDriving && Driver.Health > 0 && !Driver.IsInState('Dying') && VehicleBase != None && VehicleBase.Health > 0 && VehicleBase.DrawType != DT_StaticMesh)
	{
		Driver.bDoTorsoTwist = false;
		Driver.bRotateToDesired = false;

		Driver.SetPhysics(PHYS_None);
		Driver.SetLocation(VehicleBase.Location + (DrivePos >> VehicleBase.Rotation));
		Driver.SetRotation(VehicleBase.Rotation);
	}
}

defaultproperties
{
	DriveAnim=driveg
	bDesiredBehindview=true
	bDriverCollideActors=true
	TPCamDistRange=(Min=50,Max=350)
}