//=============================================================================
// Frag.Ops  2004 Matt 'SquirrelZero' Farber
//=============================================================================
class War_FOVehiclePawnFree extends War_FOVehiclePawn;

var int YawLeftLimit, YawRightLimit;

simulated function PostBeginPlay()
{
	Super.PostBeginPlay();

	// nice placement sound
	if (Level.NetMode != NM_DedicatedServer)
	{
		if (!class'FOPlayer'.default.bTeamBeacons)
			bNoTeamBeacon = true;
	}
}

// enter
function KDriverEnter(Pawn P)
{
	local rotator NewRotation;
	local Controller C;

	C = P.Controller;

	Super(Vehicle).KDriverEnter(P);

	if (VehicleBase != None && VehicleBase.bTeamLocked && VehicleBase.bEnterringUnlocks)
		VehicleBase.bTeamLocked = false;

	if (!bHasOwnHealth && VehicleBase == None)
	{
		Health = Driver.Health;
		HealthMax = Driver.HealthMax;
	}

	NewRotation = C.Rotation;
	NewRotation.Pitch = LimitPitch(NewRotation.Pitch);
	SetRotation(NewRotation);
	Driver.bSetPCRotOnPossess = false; //so when driver gets out he'll be facing the same direction as he was inside the vehicle
	Driver.SetOwner(Controller); // This keeps the driver relevant.

	// set inventory to driver's
	if (FOPawn(P) != None)
	{
		if (P.Inventory != None)
		{
			Inventory = P.Inventory;
			Weapon = P.Weapon;
		}
	}

	P.Controller = C;
	P.PlayerReplicationInfo = C.PlayerReplicationInfo;
	PlayerController(Controller).BehindView(false);
	P.SetOwner(Controller);
	if (VehicleBase != None)
		VehicleBase.LifeSpan = 0;
}

function AttachToVehicle(ONSVehicle VehiclePawn, name WeaponBone)
{
	if (Level.NetMode != NM_Client)
		VehicleBase = VehiclePawn;
}

event bool KDriverLeave(bool bForceLeave)
{
	local Pawn OldDriver;
	
	if (Driver != None)
	{
		OldDriver = Driver;
		Driver.Controller = None;
		Driver.PlayerReplicationInfo = None;
		Inventory = None;
		Weapon = None;
	}
	if (!Super.KDriverLeave(bForceLeave))
	{
		if (Driver != None)
		{
			Inventory = Driver.Inventory;
			Weapon = Driver.Weapon;
			Driver.Controller = Controller;
			Driver.PlayerReplicationInfo = PlayerReplicationInfo;
		}
		return false;
	}
	if (OldDriver != None)
		OldDriver.SetBoneRotation('Bip01 Spine', rot(0,0,0));
	return true;
}

simulated function ClientKDriverEnter(PlayerController PC)
{
	Super.ClientKDriverEnter(PC);

	// set inventory to driver's
	if (PC.Pawn != None && FOPawn(PC.Pawn) != None)
	{
		if (PC.Pawn.Inventory != None)
		{
			Inventory = PC.Pawn.Inventory;
			Weapon = PC.Pawn.Weapon;
		}
		PC.Pawn.SetOwner(PC);
	}
	else if (Driver != None)
	{
		if (Driver.Inventory != None)
		{
			Inventory = Driver.Inventory;
			Weapon = Driver.Weapon;
		}
		Driver.SetOwner(PC);
	}
}

simulated function ClientKDriverLeave(PlayerController PC)
{
	Super.ClientKDriverLeave(PC);
	Inventory = None;
	Weapon = None;
}

// no spawning of gun
function BeginPlay()
{
	Super(Vehicle).BeginPlay();
}

// limit yaw and pitch
simulated function int LimitYaw(int InYaw)
{
	local int VehicleYaw;

	if (VehicleBase == None)
		return InYaw;

	VehicleYaw = VehicleBase.Rotation.Yaw & 65535;
	InYaw = InYaw & 65535;

	if ((InYaw ClockwiseFrom (VehicleYaw + YawRightLimit)) && (InYaw ClockwiseFrom VehicleYaw))
		return VehicleYaw + YawRightLimit;
	else if (!(InYaw ClockwiseFrom (VehicleYaw + YawLeftLimit)) && !(InYaw ClockwiseFrom VehicleYaw))
		return VehicleYaw + YawLeftLimit;
	else
		return InYaw;
}

simulated function int LimitPitch(int InPitch)
{
	local int VehiclePitch;

	if (VehicleBase == None)
		return InPitch;

	VehiclePitch = VehicleBase.Rotation.Pitch & 65535;
	InPitch = InPitch & 65535;

	if ((InPitch ClockwiseFrom (VehiclePitch + PitchUpLimit)) && (InPitch ClockwiseFrom VehiclePitch))
		return VehiclePitch + PitchUpLimit;
	else if (!(InPitch ClockwiseFrom (VehiclePitch + PitchDownLimit)) && !(InPitch ClockwiseFrom VehiclePitch))
		return VehiclePitch + PitchDownLimit;
	else
		return InPitch;
}

// weapon control
simulated function Fire(optional float F)
{
	if (Weapon != None)
	{
		if (Weapon.bDebugging)
			Log(self$" fire weapon "$weapon$" with role "$Role);
		Weapon.Fire(F);
	}
}

simulated function AltFire(optional float F)
{
	if (Weapon != None)
	        Weapon.AltFire(F);
}

simulated function NewSwitchWeapon(byte F)
{
	Driver.SwitchWeapon(F);
}

simulated function NextWeapon()
{
	Driver.NextWeapon();
}

simulated function PrevWeapon()
{
	Driver.PrevWeapon();
}

exec function SwitchToLastWeapon()
{
	Driver.SwitchToLastWeapon();
}

function ChangedWeapon()
{
	Driver.PendingWeapon = PendingWeapon;
	PendingWeapon = None;
	Driver.ChangedWeapon();
}

// update rotation
function UpdateRocketAcceleration(float deltaTime, float YawChange, float PitchChange)
{
	local rotator NewRotation;

	NewRotation = Rotation;
	NewRotation.Yaw += 32.0 * deltaTime * YawChange;
	NewRotation.Pitch += 32.0 * deltaTime * PitchChange;
	NewRotation.Pitch = LimitPitch(NewRotation.Pitch);
	NewRotation.Yaw = LimitYaw(NewRotation.Yaw);

	SetRotation(NewRotation);
}

simulated function bool SpecialCalcView(out actor ViewActor, out vector CameraLocation, out rotator CameraRotation )
{
	local PlayerController pc;

	pc = PlayerController(Controller);

	// Only do this mode we have a playercontroller
	if ((pc == None) || (pc.Viewtarget != self))
		return false;

	if (Level.NetMode != NM_StandAlone)
		PC.bBehindView = false;
	
	if (PC.bBehindView)
		SpecialCalcBehindView(PC,ViewActor,CameraLocation,CameraRotation);
	else
		SpecialCalcFirstPersonView(PC,ViewActor,CameraLocation,CameraRotation);

	LastCameraCalcTime = Level.TimeSeconds;
	return true;
}

simulated function SpecialCalcFirstPersonView(PlayerController PC, out actor ViewActor, out vector CameraLocation, out rotator CameraRotation )
{
	local vector x, y, z;
	local vector VehicleZ, CamViewOffsetWorld;
	local float CamViewOffsetZAmount;
	local Weapon_Base TWeapon;

	GetAxes(CameraRotation, x, y, z);
	ViewActor = Driver;
	CamViewOffsetWorld = FPCamViewOffset >> CameraRotation;
	CameraLocation = GetCameraLocationStart() + (FPCamPos >> VehicleBase.Rotation);
	if (bFPNoZFromCameraPitch)
	{
		VehicleZ = vect(0,0,1) >> VehicleBase.Rotation;
		CamViewOffsetZAmount = CamViewOffsetWorld Dot VehicleZ;
		CameraLocation -= CamViewOffsetZAmount * VehicleZ;
	}

	if (Driver != None && Driver.Weapon != None && Weapon_Base(Driver.Weapon).bScopeZoomed)
	{
		TWeapon = Weapon_Base(Driver.Weapon);
		CameraRotation = Normalize(CameraRotation + PC.ShakeRot + TWeapon.ActualRot);
		CameraLocation = CameraLocation + PC.ShakeOffset.X * x + PC.ShakeOffset.Y * y + PC.ShakeOffset.Z * z + TWeapon.BreathLoc.X * X + TWeapon.BreathLoc.Y * Y + TWeapon.BreathLoc.Z * Z;
	}
	else
	{
		CameraRotation = Normalize(CameraRotation + PC.ShakeRot);
		CameraLocation = CameraLocation + PC.ShakeOffset.X * x + PC.ShakeOffset.Y * y + PC.ShakeOffset.Z * z;
	}
	Driver.bOwnerNoSee = true;
}

simulated function vector GetCameraLocationStart()
{
	if (Driver != None)
		return Driver.Location + Driver.EyePosition();
	else
		return Super.GetCameraLocationStart();
}

simulated function Tick(float dt)
{
	local rotator ControllerRotation, TestRotation, SpineRotation;

	Super(ONSWeaponPawn).Tick(dt);

	if (Driver == None || !FOPawn(Driver).bDriving || Driver.Health <= 0 || Driver.IsInState('Dying') || VehicleBase == None || VehicleBase.Health <= 0 || VehicleBase.DrawType == DT_StaticMesh)
		return;

	Driver.bDoTorsoTwist = false;
	Driver.bRotateToDesired = false;
	if (Role == ROLE_Authority && Controller != None)
	{
		Driver.SetTwistLook(0,0);
		ControllerRotation = Controller.Rotation;
		ControllerRotation.Roll = 0;
		TestRotation.Yaw = VehicleBase.Rotation.Yaw & 65535;
		TestRotation.Pitch = VehicleBase.Rotation.Pitch & 65535;
		SpineRotation.Roll = (float((Normalize(ControllerRotation - TestRotation).Yaw))*0.95)-900;
		SpineRotation.Yaw = fClamp(float((-Normalize(ControllerRotation - TestRotation).Pitch)), -5000.0, 5000.0);
		Driver.SetBoneRotation('Bip01 Spine', SpineRotation);
	}

	Driver.SetPhysics(PHYS_None);
	Driver.SetLocation(VehicleBase.Location + (DrivePos >> VehicleBase.Rotation));
	Driver.SetRotation(VehicleBase.Rotation);
	Driver.NetUpdateTime = Level.TimeSeconds - 1;
}

defaultproperties
{
	PitchUpLimit=9000
	PitchDownLimit=-9000
	YawLeftLimit=-17000
	YawRightLimit=17000

	VehiclePositionString="riding shotgun in a Mongoose"
	VehicleNameString="Mongoose Passenger"

	DriveAnim=driveg
	DrivePos=(X=-10.8,Y=25.0,Z=-2.0)
	DriveRot=(Pitch=-800)
	EntryPosition=(X=0,Y=0,Z=0)
	EntryRadius=100.0
	ExitPositions(0)=(X=0,Y=155,Z=50)
	ExitPositions(1)=(X=0,Y=-155,Z=50)
	ExitPositions(2)=(X=0,Y=155,Z=-50)
	ExitPositions(3)=(X=0,Y=-155,Z=-50)

	FPCamPos=(X=11,Y=0,Z=0)
	FPCamViewOffset=(X=6,Y=0,Z=0)
	bFPNoZFromCameraPitch=false
	bAllowViewChange=false

	bDesiredBehindView=false
	bDrawDriverInTP=true
	bDrawMeshInFP=false
	DriverDamageMult=0.0
	bHasAltFire=false
	bSpecialCalcView=true

	GunClass=None
	LandMovementState=PlayerGunning
}