//=============================================================================
// Frag.Ops  2004 Matt 'SquirrelZero' Farber
//=============================================================================

#exec OBJ LOAD FILE=FOVehicles.ukx
#exec OBJ LOAD FILE=FOWarVehicleSkins.utx

class War_FOVehicleWheeled extends ONSWheeledCraft
	abstract;

// placement sound
var sound PlacingSound;

// buyzone stuff
var War_BuildZone MyBZ;
var int BZSlot;

// headlights
var bool bLightsOn;

// restrict damage to this vehicle
var bool bRestrictDamage;

// size when destroyed
var float DestroyedScale;

// sound limiters for running over
var float LastRanIntoTime;
var Actor LastRanInto;

// alternate, lower-detail kparams for destruction
var KarmaParams AltKParams;

// destroy after a while
var float TimeOutSpan;
var bool bAlreadyDestroyed;

replication
{
	unreliable if (Role < ROLE_Authority)
		SpecialFire;

	reliable if (bNetDirty && Role == ROLE_Authority)
		bLightsOn;
}

static function StaticPrecache(LevelInfo L)
{
	local int i;

	for(i=0;i<Default.DriverWeapons.Length;i++)
		Default.DriverWeapons[i].WeaponClass.static.StaticPrecache(L);

	for(i=0;i<Default.PassengerWeapons.Length;i++)
		Default.PassengerWeapons[i].WeaponPawnClass.static.StaticPrecache(L);

	if (Default.DestroyedVehicleMesh != None)
		L.AddPrecacheStaticMesh(Default.DestroyedVehicleMesh);

	if (Default.HeadlightCoronaMaterial != None)
		L.AddPrecacheMaterial(Default.HeadLightCoronaMaterial);

	if (Default.HeadlightProjectorMaterial != None)
		L.AddPrecacheMaterial(Default.HeadLightProjectorMaterial);

	L.AddPrecacheMaterial(default.VehicleIcon.Material);

	L.AddPrecacheMaterial(Material'EmitterTextures.MultiFrame.LargeFlames');
	L.AddPrecacheMaterial(Material'EmitterTextures.MultiFrame.fire3');
	L.AddPrecacheMaterial(Texture'AW-2004Particles.Weapons.DustSmoke');
}

simulated function UpdatePrecacheStaticMeshes()
{
	Super(SVehicle).UpdatePrecacheStaticMeshes();

	if (DestroyedVehicleMesh != None)
		Level.AddPrecacheStaticMesh(DestroyedVehicleMesh);
}

simulated function UpdatePrecacheMaterials()
{
	if (HeadlightCoronaMaterial != None)
		Level.AddPrecacheMaterial(HeadLightCoronaMaterial);

	if (HeadlightProjectorMaterial != None)
		Level.AddPrecacheMaterial(HeadLightProjectorMaterial);

	Level.AddPrecacheMaterial(Material'EmitterTextures.MultiFrame.LargeFlames');
	Level.AddPrecacheMaterial(Material'EmitterTextures.MultiFrame.fire3');
	Level.AddPrecacheMaterial(Texture'AW-2004Particles.Weapons.DustSmoke');

	Level.AddPrecacheMaterial(VehicleIcon.Material);

	Super(SVehicle).UpdatePrecacheMaterials();
}

function SpecialFire()
{
	bLightsOn = !bLightsOn;
}

simulated function PostBeginPlay()
{
	Super.PostBeginPlay();

	// nice placement sound
	if (Level.NetMode != NM_DedicatedServer)
	{
		if (!class'FOPlayer'.default.bTeamBeacons)
			bNoTeamBeacon = true;
		PlaySound(PlacingSound,,,,80,0.7,true);
	}
}

function bool TryToDrive(Pawn P)
{
	if (P.Weapon != None && (P.Weapon.IsFiring() || (Weapon_Base(P.Weapon).ZoomLevel > 0 && Weapon_Base(P.Weapon).ScopeHUDTex != None)))
		return false;

	return Super.TryToDrive(P);
}

// enter
function KDriverEnter(Pawn P)
{
	local int x;

	ResetTime = Level.TimeSeconds - 1;
	Instigator = self;

	Super(SVehicle).KDriverEnter(P);

	if (Weapons.Length > 0)
		Weapons[ActiveWeapon].bActive = True;

	if (IdleSound != None)
		AmbientSound = IdleSound;

	if (StartUpSound != None)
		PlaySound(StartUpSound, SLOT_None, 1.0,,120);

	Driver.bSetPCRotOnPossess = false; //so when driver gets out he'll be facing the same direction as he was inside the vehicle

	for (x = 0; x < Weapons.length; x++)
	{
		if (Weapons[x] == None)
		{
			Weapons.Remove(x, 1);
			x--;
		}
		else
		{
			Weapons[x].NetUpdateFrequency = 20;
			ClientRegisterVehicleWeapon(Weapons[x], x);
		}
	}

	// save player replicationinfo
	P.PlayerReplicationInfo = Controller.PlayerReplicationInfo;
	SetTimer(0.1,false);
	LifeSpan = 0;
}

function DriverLeft()
{
	if (ActiveWeapon < Weapons.Length)
	{
		Weapons[ActiveWeapon].bActive = False;
		Weapons[ActiveWeapon].AmbientSound = None;
	}

	if (AmbientSound != None)
		AmbientSound = None;

	if (ShutDownSound != None)
		PlaySound(ShutDownSound, SLOT_None, 1.0,,120);

	Super(SVehicle).DriverLeft();
}

simulated function ClientKDriverEnter(PlayerController PC)
{
	Super.ClientKDriverEnter(PC);
	PC.EndZoom();
}

event bool KDriverLeave(bool bForceLeave)
{
	local Pawn OldDriver;
	
	OldDriver = Driver;
	if (Super.KDriverLeave(bForceLeave))
	{
		if (OldDriver != None && OldDriver.Base == self)
			OldDriver.SetBase(None);
		LifeSpan = Default.TimeOutSpan;
		return true;
	}
	return false;
}

simulated function Destroyed()
{
	Super.Destroyed();

	if (AltKParams != None)
		AltKParams = None;

	// release resources
	if (MyBZ != None)
		MyBZ.ResourceUsage[BZSlot] = Max(MyBZ.ResourceUsage[BZSlot]-1, 0);
}

// see if driver was shot
function Pawn GetHitPawn(vector HitLocation, rotator HitRotation)
{
	local vector HitLoc;
	local int i;
	
	HitLoc = HitLocation;
	if (Driver != None)
	{
		HitLoc.Z = Driver.Location.Z;
		if (VSize(HitLoc-Driver.Location) <= Driver.CollisionRadius)
			return Driver;
	}

	for (i=0;i<WeaponPawns.Length;i++)
	{	
		if (WeaponPawns[i].Driver != None)
		{
			HitLoc.Z = WeaponPawns[i].Driver.Location.Z;
			if (VSize(HitLoc-WeaponPawns[i].Driver.Location) <= WeaponPawns[i].Driver.CollisionRadius)
				return WeaponPawns[i].Driver;
		}
	}
	return None;
}

event bool IsVehicleEmpty()
{
	local int i;

	if (Driver != None)
		return false;

	for (i=0; i<WeaponPawns.length; i++)
		if (WeaponPawns[i] != None && WeaponPawns[i].Driver != None )
			return false;

	return true;
}

simulated function Tick(float dt)
{
	local int i;

	Super.Tick(dt);

	if (Health <= 0 || DrawType == DT_StaticMesh)
		return;

	// manually update
	if (Driver != None && FOPawn(Driver).bDriving && Driver.Health > 0 && !Driver.IsInState('Dying'))
	{
		Driver.SetBase(None);
		Driver.bHardAttach = true;
		Driver.SetPhysics(PHYS_None);
		Driver.SetBase(self);
		Driver.SetRelativeLocation(DrivePos);
		Driver.SetRotation(Rotation);
	}

	// This code just for making it easy to position coronas etc.
	if (Level.NetMode == NM_DedicatedServer)
		return;

	for (i=0;i<HeadlightCorona.Length;i++)
	{
		if (!bLightsOn)
			HeadlightCorona[i].Skins[0] = None;
		else
			HeadlightCorona[i].Skins[0] = HeadlightCoronaMaterial;
	}

	if (HeadlightProjector != None)
	{
		if (!bLightsOn)
		{
			HeadLightProjector.MaxTraceDistance = 0;
			HeadLightProjector.ProjTexture = None;
		}
		else
		{
			HeadLightProjector.MaxTraceDistance = HeadLightProjector.Default.MaxTraceDistance;
			HeadLightProjector.ProjTexture = HeadlightProjectorMaterial;
		}
	}
}

// some damage modifiers
event TakeDamage(int Damage, Pawn EventInstigator, vector HitLocation, vector Momentum, class<DamageType> DamageType)
{
	if (bRestrictDamage)
		return;

	Super.TakeDamage(Damage, EventInstigator, HitLocation, Momentum, DamageType);
}

// overridden -- unwanted
simulated event OnDaredevil()
{
}

simulated function PrecacheAnnouncer(AnnouncerVoice V, bool bRewardSounds)
{
}

function SpawnBuildEffect()
{
}

simulated function SetOverlayMaterial( Material mat, float time, bool bOverride)
{
}

function Timer()
{
	if (Abs(Velocity.Z) > 50 || VSize(Velocity) > 700)
		Velocity.Z -= 40;
	if (Driver != None)
		SetTimer(0.1,false);
}

simulated event DestroyAppearance()
{
	local int i;
	local KarmaParamsRBFull KP;

	// For replication
	bDestroyAppearance = true;

	if (bAlreadyDestroyed)
		return;

	bAlreadyDestroyed = true;

	// Put brakes on
	Throttle	= 0;
	Steering	= 0;
	Rise		= 0;

	// Destroy the weapons
	if (Role == ROLE_Authority)
	{
		for (i=0;i<Weapons.Length;i++)
			Weapons[i].Destroy();
		for (i=0;i<WeaponPawns.Length;i++)
			WeaponPawns[i].Destroy();
	}
	Weapons.Length = 0;
	WeaponPawns.Length = 0;

	// Destroy the effects
	if (Level.NetMode != NM_DedicatedServer)
	{
		bNoTeamBeacon = true;

		for (i=0;i<HeadlightCorona.Length;i++)
			HeadlightCorona[i].Destroy();
		HeadlightCorona.Length = 0;

		if (HeadlightProjector != None)
			HeadlightProjector.Destroy();
	}

	// copy linear velocity from actor so it doesn't just stop.
	KP = KarmaParamsRBFull(KParams);
	if (KP != None)
		KP.KStartLinVel = Velocity;
	
	// reduce wheel size to tiny, crank up friction
	WheelLongFrictionScale = 2.6;
	Super(SVehicle).SetWheelsScale(0.01);
	for(i=0;i<Wheels.Length;i++)
	{
		SetBoneScale(i, 0.0001, Wheels[i].BoneName);
		Wheels[i].WheelRadius = 0;
	}
	
	// Become the dead vehicle mesh
	SetPhysics(PHYS_None);
	KSetBlockKarma(False);
	SetDrawType(DT_StaticMesh);
	SetStaticMesh(DestroyedVehicleMesh);
	SetDrawScale(DestroyedScale);
	KSetBlockKarma(True);
	SetPhysics(PHYS_Karma);
	Skins.Length = 0;
	NetPriority = 2.5;
	NetUpdateTime = Level.TimeSeconds - 1;
	LifeSpan = 20.0;
}

event RanInto(Actor Other)
{
	local vector Momentum;
	local float Speed;
	local Pawn HitPawn;

	Momentum = Normal(Other.Location-Location);
	// in case of emergency...
	if (Effect_BreakingGlass(Other) != None)
	{
		Other.TakeDamage(Effect_BreakingGlass(Other).MaxDamage, Instigator, Other.Location, Momentum, RanOverDamageType);
		return;
	}
	else if (Pawn(Other) == None || Vehicle(Other) != None || Other == Instigator || Other.Role != ROLE_Authority)
		return;

	HitPawn = Pawn(Other);
	Speed = VSize(Velocity);
	if (HitPawn != None && HitPawn.PlayerReplicationInfo != None && HitPawn.PlayerReplicationInfo.Team.TeamIndex == GetTeamNum())
		MinRunOverSpeed = 1000;
	else
		MinRunOverSpeed = 0;
	if (Speed > MinRunOverSpeed)
	{
		Momentum = Normal(Velocity);
		if (RanOverSound != None && (HitPawn == None || HitPawn.Health > 0) && Level.TimeSeconds - LastRanIntoTime > 0.5)
		{
			LastRanIntoTime = Level.TimeSeconds;
			if (MinRunOverSpeed != 1000 || class'FragOpsMission'.Default.FriendlyFireScale > 0)
				PlaySound(RanOverSound,,TransientSoundVolume*2.5,,100);
		}
		Other.TakeDamage(int(Speed * 0.075), Instigator, Other.Location, Momentum, RanOverDamageType);
	}
	else
		Other = None;
}

simulated event Touch(Actor Other)
{
	local vector Momentum;

	Super.Touch(Other);

	// not on client
	if (Role < ROLE_Authority || Effect_BreakingGlass(Other) == None)
		return;

	Momentum = Normal(Other.Location-Location);
	Other.TakeDamage(Effect_BreakingGlass(Other).MaxDamage, Instigator, Other.Location, Momentum, RanOverDamageType);
}

simulated function POVChanged(PlayerController PC, bool bBehindViewChanged)
{
	if (PC.bBehindView)
	{
		bOwnerNoSee = false;

		if (Driver != None)
		{
			if (bDrawDriverInTP)
				Driver.bOwnerNoSee = false;
			else
				Driver.bOwnerNoSee = true;
		}

		if (PC == Controller)   // No overlays for spectators
		    ActivateOverlay(False);
	}
	else
	{
		if (bDrawMeshInFP)
			bOwnerNoSee = false;
		else
			bOwnerNoSee = true;

		if (Driver != None)
			Driver.bOwnerNoSee = true;

		if (bDriving && PC == Controller)   // No overlays for spectators
		    ActivateOverlay(True);
	}
}

defaultproperties
{
	// display
	Mesh=Mesh'FOVehicles.jeepb'
	VehiclePositionString="in a Mongoose"
	VehicleNameString="Mongoose"
	DrawScale=1.0
	DrawScale3D=(X=1.0,Y=1.0,Z=1.0)
	DestroyedScale=0.85

	// team beacon
	TeamBeaconTexture=Texture'FOWarTextures.HUD.HealthIcon'
	TeamBeaconBorderMaterial=Texture'FOWarTextures.HUD.HealthBorder'

	// sound
	PlacingSound=sound'FOGameSounds.Menu.MedalStomp'
	IdleSound=sound'ONSVehicleSounds-S.RV.RVEng01'
	StartUpSound=sound'ONSVehicleSounds-S.RV.RVStart01'
	ShutDownSound=sound'ONSVehicleSounds-S.RV.RVStop01'
	EngineRPMSoundRange=9000
	SoundVolume=180
	IdleRPM=500
	RevMeterScale=4000

	// speed
	GroundSpeed=940

	// physics - wheels
	WheelPenScale=1.2
	WheelPenOffset=0.01
	WheelSoftness=0.025
	WheelRestitution=0.1
	WheelAdhesion=0.0
	WheelLongFrictionFunc=(Points=((InVal=0,OutVal=0.0),(InVal=100.0,OutVal=1.0),(InVal=200.0,OutVal=0.9),(InVal=10000000000.0,OutVal=0.9)))
	WheelLongFrictionScale=1.1
	WheelLatFrictionScale=1.35
	WheelLongSlip=0.001
	WheelLatSlipFunc=(Points=((InVal=0.0,OutVal=0.0),(InVal=30.0,OutVal=0.009),(InVal=45.0,OutVal=0.00),(InVal=10000000000.0,OutVal=0.00)))
	WheelHandbrakeSlip=0.01
	WheelHandbrakeFriction=0.1
	WheelSuspensionTravel=15.0
	WheelSuspensionOffset=0.0
	WheelSuspensionMaxRenderTravel=15.0
	
	// physics - control
	MinBrakeFriction=4.0
	MaxBrakeTorque=20.0
	MaxSteerAngleCurve=(Points=((InVal=0,OutVal=25.0),(InVal=1500.0,OutVal=11.0),(InVal=1000000000.0,OutVal=11.0)))
	TurnDamping=35
	SteerSpeed=160
	StopThreshold=100
	TorqueCurve=(Points=((InVal=0,OutVal=9.0),(InVal=200,OutVal=10.0),(InVal=1500,OutVal=11.0),(InVal=2800,OutVal=0.0)))
	EngineBrakeFactor=0.0001
	EngineBrakeRPMScale=0.1
	EngineInertia=0.1
	WheelInertia=0.1
	
	// physics - gears
	TransRatio=0.15
	GearRatios[0]=-0.5
	GearRatios[1]=0.4
	GearRatios[2]=0.65
	GearRatios[3]=0.85
	GearRatios[4]=1.1
	ChangeUpPoint=2000
	ChangeDownPoint=1000
	LSDFactor=1.0

	// physics - misc
	CenterSpringForce="SpringONSSRV"
	HandbrakeThresh=200
	FTScale=0.03
	ChassisTorqueScale=0.4
	VehicleMass=3.5

	// collision
	CollisionHeight=+60.0
	CollisionRadius=+90.0

	// force
	StartUpForce="RVStartUp"
	
	// cameras
	FPCamPos=(X=3,Y=-6,Z=7)
	TPCamLookat=(X=0,Y=0,Z=0)
	TPCamWorldOffset=(X=0,Y=0,Z=100)
	TPCamDistance=150
	TPCamDistRange=(Min=50,Max=220)

	// destruction
	ExplosionMomentum=1000
	DestructionEffectClass=class'Onslaught.ONSSmallVehicleExplosionEffect'
	DestructionLinearMomentum=(Min=200000,Max=300000)
	DestructionAngularMomentum=(Min=100,Max=150)
	DisintegrationEffectClass=class'FragOps.War_Effect_VehicleDeath'
	DisintegrationHealth=-2000

	// driver-related
	MaxViewYaw=19000
	MaxViewPitch=19000
	DrivePos=(X=2.0,Y=0.0,Z=38.0)
	DriveAnim=driveg
	bDriverCollideActors=true
	DriverDamageMult=0.90
	RanOverDamageType=class'DamType_Roadkill'
	CrushedDamageType=class'DamType_Roadkill'
	ExplosionDamageType=class'DamType_DestroyedVehicle'
	DestroyedRoadKillDamageType=class'DamType_DestroyedVehicle'

	// steering wheel
	SteerBoneName="SteeringWheel"
	SteerBoneAxis=AXIS_Z
	SteerBoneMaxAngle=90

	// horn
	HornSounds(0)=sound'ONSVehicleSounds-S.Horn06'
	HornSounds(1)=sound'ONSVehicleSounds-S.Dixie_Horn'

	// stunt-related, reduce for realism
	AirTurnTorque=15.0
	AirPitchTorque=25.0
	AirPitchDamping=15.0
	AirRollTorque=15.0
	AirRollDamping=15.0

	// ai
	MaxDesireability=0.4
	ObjectiveGetOutDist=1500.0

	// settings
	TimeOutSpan=75
	SparkEffectClass=None
	SparkAdvanceFactor=0
	bHasHandbrake=true
	bAllowBigWheels=true
	bDrawDriverInTP=True
	bDrawMeshInFP=True
	bDoStuntInfo=false
	bHasAltFire=false
	bSeparateTurretFocus=true

	// tweaks
	bTeamLocked=false
	MaxLights=4
	bUseDynamicLights=false
}