//=============================================================================
// Frag.Ops  2004 Matt 'SquirrelZero' Farber
//=============================================================================

#exec OBJ LOAD FILE=FOWarTextures.utx

class War_Interface_MainHUD extends Interface_MainHUD
	config(User);

struct Quickbars
{
	var array<class<War_FOQuickbarFunction> > BarFunction;
	var array<byte> ResourceCount;
	var array<byte> ResourceMax;
	var array<float> ClientUseTime;
};

// new timer widgets
var() NumericWidget WaveMinutes, WaveSeconds;
var() SpriteWidget MapTimeIcon, RoundTimeIcon, WaveTimeIcon, WaveTimeSep;

// quickbar widget
var() SpriteWidget QuickbarBG;

// vehicle stuff
var() SpriteWidget VehicleBG, VehicleIcon;
var() NumericWidget VehicleHealth;
var float YBump;

// quickbar arrays
var Quickbars Quickbar;

// bar icon variables
var float IconSize, IconSpace, IconOffsetX, IconOffsetY;
var float MeterOffsetX, MeterOffsetY;
var float ItemNumberOffsetX, ItemNumberOffsetY;

// icon colors
var config color IconActiveColor, IconInActiveColor, MeterColor, WiperColor, IconBorderColor, IconEmptyColor;
var Combiner IconFlasher;

// wave messages
var float WaveDelay;
var float WaveMessageTime;
var float WaveMessageXPos, WaveMessageYPos;
var string WaveMessage;
var color WaveFlashColor;
var bool bReversed, bFadeOut;
var string RespawnKeyName, LoadOutKeyName;
var float LastKeyTime;

//  radar
var float RadarPulse, RadarScale, LastDrawRadar, MinEnemyDist;
var float RadarPosX, RadarPosY;

// objectives
var string ObjMessageSingle;
var array<string> ObjSplashMessage;
var Material ObjSplashPicture;
var Material ObjPicBorder;
var bool bObjSplash;
var float ObjSplashTime;
var float ObjPosX, ObjPosY;
var byte ObjAlpha;

// objective splash BG
var Material ObjBG;
var float ObjBGScaleX, ObjBGScaleY;

// objective picture/text
var float ObjPicOffsetX, ObjPicOffsetY, ObjPicScale;
var float ObjTextOffsetX, ObjTextOffsetY, ObjTextWidthX;

// levelinfo
var War_LevelInfo WarLevelInfo;

simulated function PostBeginPlay()
{
	Super.PostBeginPlay();
	foreach AllActors(class'War_LevelInfo',WarLevelInfo)
	{
		break;
	}
}

simulated function UpdatePrecacheMaterials()
{
	Level.AddPrecacheMaterial(Material'InterfaceContent.HUD.SkinA');
	Level.AddPrecacheMaterial(Material'AS_FX_TX.AssaultRadar');
	Super.UpdatePrecacheMaterials();
}

// radar is obviously a variation on the assault radar
simulated function DrawRadar(Canvas C)
{
	local float RadarWidth;

	if (PawnOwner == None || !bShowMoney)
		return;

	RadarScale = Default.RadarScale * HUDScale;
	RadarWidth = 0.5 * RadarScale * C.ClipX;

	C.Style = ERenderStyle.STY_Alpha;
	C.SetDrawColor(0,0,0,255);
	C.SetPos(RadarPosX*C.ClipX - RadarWidth,RadarPosY*C.ClipY+RadarWidth);
	C.DrawTile(Material'AS_FX_TX.AssaultRadar', RadarWidth, RadarWidth, 0, 512, 512, -512);
	C.SetPos(RadarPosX*C.ClipX,RadarPosY*C.ClipY+RadarWidth);
	C.DrawTile(Material'AS_FX_TX.AssaultRadar', RadarWidth, RadarWidth, 512, 512, -512, -512);
	C.SetPos(RadarPosX*C.ClipX - RadarWidth,RadarPosY*C.ClipY);
	C.DrawTile(Material'AS_FX_TX.AssaultRadar', RadarWidth, RadarWidth, 0, 0, 512, 512);
	C.SetPos(RadarPosX*C.ClipX,RadarPosY*C.ClipY);
	C.DrawTile(Material'AS_FX_TX.AssaultRadar', RadarWidth, RadarWidth, 512, 0, -512, 512);
}

simulated function DrawRadarPassC(Canvas C)
{
	local Pawn P;
	local Vehicle VP;
	local array<War_Buildable> FBuildables;
	local array<War_BuildZone> FBuildZones;
	local Actor CurrentObjective, CaptureObjective;
	local Pawn RealPawnOwner;
	local War_Objective_RetrievalVolume WRV;
	local float Dist,RadarWidth,Angle,DotSize,OffsetY,OffsetScale,ColorScale;
	local rotator Dir;
	local vector Start;
	local Color DotColor;	
	local int i;

	if (!bShowMoney)
		return;

	if (Vehicle(PawnOwner) != None)
		RealPawnOwner = Vehicle(PawnOwner).Driver;
	else
		RealPawnOwner = PawnOwner;

	LastDrawRadar = Level.TimeSeconds;
	RadarWidth = 0.5 * RadarScale * C.ClipX;
	Start = RealPawnOwner.Location;
	
	C.Style = ERenderStyle.STY_Translucent;
	OffsetY = RadarPosY + RadarWidth/C.ClipY;
	MinEnemyDist = 3000;

	// draw build zones
	FBuildZones = War_FOPlayer(PlayerOwner).BuildZones;
	for (i=0;i<FBuildZones.Length;i++)
	{
		if ((!FBuildZones[i].bIsTriggered || FBuildZones[i].bEnabled) && (FBuildZones[i].ControllingTeam == 255 || FBuildZones[i].ControllingTeam == TeamIndex))
		{
			Dist = VSize(Start - FBuildZones[i].Location);
			if (Dist < (3000 - FBuildZones[i].CollisionRadius*0.5))
			{
				DotColor.R = 150;
				DotColor.G = 150;
				DotColor.B = 150;
				ColorScale = 1.0-FClamp(((Dist + FBuildZones[i].CollisionRadius) - 3000)/(FBuildZones[i].CollisionRadius*0.5), 0, 1.0);
				DotColor = DotColor * ColorScale;
				DotSize = (6*C.ClipX*HUDScale/1600)*FClamp(FBuildZones[i].CollisionRadius/55.0, 1.0, 10.0);
				C.DrawColor = DotColor;

				Dir = rotator(FBuildZones[i].Location - Start);
				OffsetScale = RadarScale*Dist*0.000167;
				Angle = ((Dir.Yaw - RealPawnOwner.GetViewRotation().Yaw) & 65535) * 6.2832/65536;
				C.SetPos(RadarPosX * C.ClipX + OffsetScale * C.ClipX * sin(Angle) - 0.5*DotSize,
					OffsetY * C.ClipY - OffsetScale * C.ClipX * cos(Angle) - 0.5*DotSize);
				if (FBuildZones[i].CollisionRadius < 200)
					C.DrawTile(Material'FOWarTextures.HUD.RadarCircleSmall',DotSize,DotSize,0,0,32,32);
				else
					C.DrawTile(Material'FOWarTextures.HUD.RadarCircle',DotSize,DotSize,0,0,64,64);
			}
		}
	}

	// draw buildables
	FBuildables = War_FOPlayer(PlayerOwner).BuiltItems;
	for (i=0;i<FBuildables.Length;i++)
	{
		if (FBuildables[i].TeamIndex == TeamIndex)
		{
			Dist = VSize(Start - FBuildables[i].Location);
			if (Dist < (3000 - FBuildables[i].NewCollisionRadius*0.5))
			{
				DotColor.R = 210;
				DotColor.G = 210;
				DotColor.B = 30;
				ColorScale = 1.0-FClamp(((Dist + FBuildables[i].NewCollisionRadius) - 3000)/(FBuildables[i].NewCollisionRadius*0.5), 0, 1.0);
				DotColor = DotColor * ColorScale;
				DotSize = (8*C.ClipX*HUDScale/1600)*FClamp(FBuildables[i].NewCollisionRadius/150.0, 1.0, 2.0);
				C.DrawColor = DotColor;

				Dir = rotator(FBuildables[i].Location - Start);
				OffsetScale = RadarScale*Dist*0.000167;
				Angle = ((Dir.Yaw - RealPawnOwner.GetViewRotation().Yaw) & 65535) * 6.2832/65536;
				C.SetPos(RadarPosX * C.ClipX + OffsetScale * C.ClipX * sin(Angle) - 0.5*DotSize,
					OffsetY * C.ClipY - OffsetScale * C.ClipX * cos(Angle) - 0.5*DotSize);
				C.DrawTile(Material'FOMisc.Special.WhiteFlash',DotSize,DotSize,0,0,4,4);
			}
		}
	}

	// setup for objectives
	CurrentObjective = War_FOPlayer(PlayerOwner).Objective;
	DotColor.R = 240;
	DotColor.G = 30;
	DotColor.B = 30;
	DotColor.A = 255;
	C.DrawColor = DotColor;
	C.Style = ERenderStyle.STY_Normal;
	DotSize = 10*C.ClipX*HUDScale/1600;

	// draw capture objectives
	if (TeamIndex == 0)
	{
		for (i=0;i<WarLevelInfo.M_CaptureObj.Length;i++)
		{
			if (WarLevelInfo.M_CaptureObj[i].Objective == CurrentObjective || WarLevelInfo.M_CaptureObj[i].Objective.bCompleted)
				continue;

			CaptureObjective = WarLevelInfo.M_CaptureObj[i].Objective;

			// hack for carried objectives
			if (FOPawn(RealPawnOwner).War_CarriedObjective == CaptureObjective)
			{
				foreach AllActors(class'War_Objective_RetrievalVolume',WRV,CaptureObjective.Tag)
				{
					CaptureObjective = WRV;
					break;
				}
			}
			Dist = FMin(VSize(Start - CaptureObjective.Location), 2825);
			Dir = rotator(CaptureObjective.Location - Start);
			OffsetScale = RadarScale*Dist*0.000167;
			Angle = ((Dir.Yaw - RealPawnOwner.GetViewRotation().Yaw) & 65535) * 6.2832/65536;
			C.SetPos(RadarPosX * C.ClipX + OffsetScale * C.ClipX * sin(Angle) - 0.5*DotSize,
				OffsetY * C.ClipY - OffsetScale * C.ClipX * cos(Angle) - 0.5*DotSize);
			C.DrawTile(Material'FOWarTextures.HUD.RadarDotFinal',DotSize,DotSize,0,0,64,64);
		}
	}
	else
	{
		for (i=0;i<WarLevelInfo.S_CaptureObj.Length;i++)
		{
			if (WarLevelInfo.S_CaptureObj[i].Objective == CurrentObjective || WarLevelInfo.S_CaptureObj[i].Objective.bCompleted)
				continue;

			CaptureObjective = WarLevelInfo.S_CaptureObj[i].Objective;

			// hack for carried objectives
			if (FOPawn(RealPawnOwner).War_CarriedObjective == CaptureObjective)
			{
				foreach AllActors(class'War_Objective_RetrievalVolume',WRV,CaptureObjective.Tag)
				{
					CaptureObjective = WRV;
					break;
				}
			}
			Dist = FMin(VSize(Start - CaptureObjective.Location), 2825);
			Dir = rotator(CaptureObjective.Location - Start);
			OffsetScale = RadarScale*Dist*0.000167;
			Angle = ((Dir.Yaw - RealPawnOwner.GetViewRotation().Yaw) & 65535) * 6.2832/65536;
			C.SetPos(RadarPosX * C.ClipX + OffsetScale * C.ClipX * sin(Angle) - 0.5*DotSize,
				OffsetY * C.ClipY - OffsetScale * C.ClipX * cos(Angle) - 0.5*DotSize);
			C.DrawTile(Material'FOWarTextures.HUD.RadarDotFinal',DotSize,DotSize,0,0,64,64);
		}
	}

	// draw primary objective
	if (CurrentObjective != None && !War_Objective(CurrentObjective).bCompleted)
	{
		// hack for carried objectives
		if (FOPawn(RealPawnOwner).War_CarriedObjective == CurrentObjective)
		{
			foreach AllActors(class'War_Objective_RetrievalVolume',WRV,CurrentObjective.Tag)
			{
				CurrentObjective = WRV;
				break;
			}
		}
		Dist = FMin(VSize(Start - CurrentObjective.Location), 2825);
		Dir = rotator(CurrentObjective.Location - Start);
		OffsetScale = RadarScale*Dist*0.000167;
		Angle = ((Dir.Yaw - RealPawnOwner.GetViewRotation().Yaw) & 65535) * 6.2832/65536;
		C.SetPos(RadarPosX * C.ClipX + OffsetScale * C.ClipX * sin(Angle) - 0.5*DotSize,
			OffsetY * C.ClipY - OffsetScale * C.ClipX * cos(Angle) - 0.5*DotSize);
		C.DrawTile(Material'FOWarTextures.HUD.RadarDotFinal',DotSize,DotSize,0,0,64,64);
	}
	C.Style = ERenderStyle.STY_Translucent;

	// draw pawns/vehicles
	foreach DynamicActors(class'Pawn',P)
	{
		VP = Vehicle(P);
		if (P.Health > 0 && ONSWeaponPawn(P) == None && ((VP != None && VP.Team == TeamIndex) || (VP == None && P.PlayerReplicationInfo != None && P.PlayerReplicationInfo.Team == PlayerOwner.PlayerReplicationInfo.Team)))
		{
			if (Vehicle(PawnOwner) == None)
				Dist = VSize(Start - P.Location);
			else
				Dist = FMin(VSize(Start - P.Location), 2550);
			if (Dist < (3000 - P.CollisionRadius*0.5))
			{
				if (VP != None)
				{
					DotColor.R = 210;
					DotColor.G = 210;
					DotColor.B = 30;					
					DotSize = 10*C.ClipX*HUDScale/1600;
				}
				else
				{
					DotColor.R = 35;
					if (RealPawnOwner == P)
						DotColor.G = 255;
					else
						DotColor.G = 180;
					DotColor.B = 35;
					DotSize = 6*C.ClipX*HUDScale/1600;
				}

				ColorScale = 1.0-FClamp(((Dist + P.CollisionRadius) - 3000)/(P.CollisionRadius*0.5), 0, 1.0);
				DotColor = DotColor * ColorScale;

				C.DrawColor = DotColor;

				Dir = rotator(P.Location - Start);
				OffsetScale = RadarScale*Dist*0.000167;
				Angle = ((Dir.Yaw - RealPawnOwner.GetViewRotation().Yaw) & 65535) * 6.2832/65536;
				C.SetPos(RadarPosX * C.ClipX + OffsetScale * C.ClipX * sin(Angle) - 0.5*DotSize,
					OffsetY * C.ClipY - OffsetScale * C.ClipX * cos(Angle) - 0.5*DotSize);
				C.DrawTile(Material'FOWarTextures.HUD.RadarDot',DotSize,DotSize,0,0,64,64);
			}
		}
	}
	C.Style = ERenderStyle.STY_Alpha;
}

// draw the compass
function DrawCompass(Canvas C)
{
}

// slide/fade in objective splash screen
function SlideObjIn(float dt)
{
	if (PawnOwner == None || !bObjSplash)
		return;
	
	// slide in bg
	ObjPosX = FClamp(ObjPosX - dt*ObjPosX, 0.66, Default.ObjPosX);
	ObjAlpha = Clamp(ObjAlpha + dt*float(ObjAlpha)*30.0, 20, 255);
}

// slide/fade out objective splash screen
function SlideObjOut(float dt)
{
	if (PawnOwner == None || bObjSplash)
		return;
	
	// slide out bg
	ObjPosX = FClamp(ObjPosX + dt*(1.1-ObjPosX)*3.8, 0.66, Default.ObjPosX);
	ObjAlpha = Clamp(ObjAlpha - dt*float(255-ObjAlpha)*60.0, 20, 254);
}

// receive a new objective splash
function SplashObjective(string ObjMessage, Material ObjPicture)
{
	// clear message
	ObjSplashMessage.Remove(0,ObjSplashMessage.Length);

	// set for drawing
	ObjMessageSingle = ObjMessage;

	// set splash time
	ObjSplashTime = Default.ObjSplashTime;

	// initialize content
	ObjSplashPicture = ObjPicture;

	// splash
	bObjSplash = true;
}

// objectives
function DrawObjectiveSplash(Canvas C)
{
	local int i;
	local float XL, YL;
	local float OldClip;

	if (ObjPosX == Default.ObjPosX)
		return;

	OldClip = C.ClipX;
	C.SpaceX = 0.3*C.ClipX;

	// set colors
	C.DrawColor = Default.WhiteColor;
	C.DrawColor.A = ObjAlpha;

	// draw background
	C.SetPos(C.ClipX*ObjPosX, C.ClipY*ObjPosY);
	C.DrawTileStretched(ObjBG, ObjBGScaleX*C.ClipX, ObjBGScaleY*C.ClipY);	

	// draw picture
	C.SetPos(C.ClipX*ObjPosX + C.ClipX*ObjPicOffsetX, C.ClipY*ObjPosY + C.ClipY*ObjPicOffsetY);
	C.DrawTile(ObjSplashPicture, ObjPicScale*C.ClipX, ObjPicScale*C.ClipY, 0, 0, ObjSplashPicture.MaterialUSize(), ObjSplashPicture.MaterialVSize());
	C.SetPos(C.ClipX*ObjPosX + C.ClipX*ObjPicOffsetX, C.ClipY*ObjPosY + C.ClipY*ObjPicOffsetY);
	C.DrawTileStretched(ObjPicBorder, ObjPicScale*C.ClipX, ObjPicScale*C.ClipY);	

	C.SpaceX = 0;

	// draw text
	C.Font = GetSmallerFontFor(C,4);
	C.StrLen("TEST",XL,YL);

	// wrap new objective string
	if (ObjSplashMessage.Length == 0)
		C.WrapStringToArray(ObjMessageSingle, ObjSplashMessage, ObjTextWidthX*C.ClipX);
	for (i=0;i<ObjSplashMessage.Length;i++)
	{
		C.ClipX = OldClip;
		C.SetPos(C.ClipX*ObjPosX + C.ClipX*ObjPicScale + C.ClipX*ObjPicOffsetX + C.ClipX*ObjTextOffsetX, C.ClipY*ObjPosY + C.ClipY*ObjTextOffsetY + (i*(YL*0.97)));
		C.ClipX = 1.6*OldClip;
		C.DrawText(ObjSplashMessage[i], true);
	}
	C.ClipX = OldClip;
}

// draw extras
function DrawHUDPassA(Canvas C)
{
	Super.DrawHUDPassA(C);
	DrawQuickbar(C);
	DrawRadar(C);
	DrawCompass(C);
	DrawObjectiveSplash(C);
	if (bShowPersonalInfo && Vehicle(PawnOwner) != None)
		DrawSpriteWidget(C, VehicleBG);
}

function DrawHUDPassC(Canvas C)
{
	Super.DrawHUDPassC(C);
	if (bShowPoints && PawnOwner != None)
	{
		DrawNumericWidget(C, WaveMinutes, DigitsBig);
		DrawNumericWidget(C, WaveSeconds, DigitsBig);
		DrawSpriteWidget(C, WaveTimeSep);
		DrawSpriteWidget(C, MapTimeIcon);
		DrawSpriteWidget(C, RoundTimeIcon);
		DrawSpriteWidget(C, WaveTimeIcon);
	}
	if (bShowPersonalInfo && Vehicle(PawnOwner) != None)
	{
		DrawSpriteWidget(C, VehicleIcon);		
		DrawNumericWidget(C, VehicleHealth, DigitsBig);
	}
	if (Level.TimeSeconds - WaveMessageTime < WaveDelay && !PlayerOwner.IsInState('GameEnded'))
		DrawWaveMessage(C);
	if ((PlayerOwner.Pawn == None || PlayerOwner.IsDead()) && !FOPlayer(PlayerOwner).bDisplayWinner && !FOPlayer(PlayerOwner).bDisplayLoser)
		DrawWaveInfo(C);
	else if (PlayerOwner.Pawn != None && PlayerOwner.IsInState('PreRound') && !FOPlayer(PlayerOwner).bBuyMenuOpen)
		DrawPreRoundInfo(C);
	DrawRadarPassC(C);
}

// draw respawn information for dead players
function DrawWaveInfo(Canvas C)
{
	local string ReString, KeyContent;
	local int WindowSeconds, i;

	// retrieve window seconds for team wave
	if (PlayerOwner.PlayerReplicationInfo.Team.TeamIndex == 1)
		WindowSeconds = War_GRI(GRI).SOWaveWindowSeconds;
	else
		WindowSeconds = War_GRI(GRI).MercWaveWindowSeconds;

	if ((!War_FOPlayer(PlayerOwner).bAlreadyRespawned && WindowSeconds > 0) || ((TeamIndex == 0 && War_GRI(GRI).MercWaveDelayTime <= War_GRI(GRI).WaveWindowTime) || (TeamIndex == 1 && War_GRI(GRI).SOWaveDelayTime <= War_GRI(GRI).WaveWindowTime)))
	{
		// make a flashy color
		if (bReversed)
		{
			WaveFlashColor.R = Min(WaveFlashColor.R+14, 255);
			if (WaveFlashColor.R == 255)
				bReversed = false;
		}
		else
		{
			WaveFlashColor.R = Max(WaveFlashColor.R-8, 48);
			if (WaveFlashColor.R == 48)
				bReversed = true;
		}
		WaveFlashColor.G = WaveFlashColor.R;
		WaveFlashColor.B = WaveFlashColor.R;
		C.DrawColor = WaveFlashColor;
		
		// get key for respawn
		if ((Level.TimeSeconds - LastKeyTime > 2.0) || LastKeyTime == 0)
		{
			LastKeyTime = Level.TimeSeconds;;
			for (i=0;i<256;i++)
			{
				RespawnKeyName = caps(PlayerOwner.ConsoleCommand("KEYNAME"@i));
				KeyContent = caps(PlayerOwner.ConsoleCommand("KEYBINDING"@RespawnKeyName));
				if (InStr(KeyContent,"ALTFIRE") > -1)
					break;
			}
			if ((TeamIndex == 0 && War_GRI(GRI).MercWaveDelayTime <= War_GRI(GRI).WaveWindowTime) || (TeamIndex == 1 && War_GRI(GRI).SOWaveDelayTime <= War_GRI(GRI).WaveWindowTime))
			{
				for (i=0;i<256;i++)
				{

					LoadOutKeyName = caps(PlayerOwner.ConsoleCommand("KEYNAME"@i));
					KeyContent = caps(PlayerOwner.ConsoleCommand("KEYBINDING"@LoadOutKeyName));
					if (InStr(KeyContent,"BUYMENU") > -1)
						break;
				}
			}
		}
		
		// draw
		if ((TeamIndex == 0 && War_GRI(GRI).MercWaveDelayTime <= War_GRI(GRI).WaveWindowTime) || (TeamIndex == 1 && War_GRI(GRI).SOWaveDelayTime <= War_GRI(GRI).WaveWindowTime))
			ReString = "Press '"$RespawnKeyName$"' to join the battle or '"$LoadOutKeyName$"' to change loadouts!";
		else
			ReString = "Press '"$RespawnKeyName$"' to join the battle! "$WindowSeconds$" seconds left!";
		C.Font = GetSmallerFontFor(C,3);
		C.SetPos(WaveMessageXPos*C.ClipX, WaveMessageYPos*C.ClipY);
		C.DrawText(ReString,false);
	}
	else if (WaveSeconds.Value <= 10)
	{
		if (WaveSeconds.Value <= 5)
		{
			// flashy
			if (bReversed)
			{
				WaveFlashColor.R = Min(WaveFlashColor.R+7, 255);
				if (WaveFlashColor.R == 255)
					bReversed = false;
			}
			else
			{
				WaveFlashColor.R = Max(WaveFlashColor.R-4, 48);
				if (WaveFlashColor.R == 48)
					bReversed = true;
			}
			WaveFlashColor.G = WaveFlashColor.R;
			WaveFlashColor.B = WaveFlashColor.R;
			C.DrawColor = WaveFlashColor;
			ReString = "Prepare!  Next wave in "$WaveSeconds.Value$" seconds!";
		}
		else
		{
			C.DrawColor = Default.WhiteColor;
			ReString = "Next wave in "$WaveSeconds.Value$" seconds";
		}
		C.Font = GetSmallerFontFor(C,3);
		C.SetPos(WaveMessageXPos*C.ClipX, WaveMessageYPos*C.ClipY);
		C.DrawText(ReString,false);
	}
	else if (!FOPlayer(PlayerOwner).bBuyMenuOpen)
	{
		if ((Level.TimeSeconds - LastKeyTime > 2.0) || LastKeyTime == 0)
		{
			for (i=0;i<256;i++)
			{
				LoadOutKeyName = caps(PlayerOwner.ConsoleCommand("KEYNAME"@i));
				KeyContent = caps(PlayerOwner.ConsoleCommand("KEYBINDING"@LoadOutKeyName));
				if (InStr(KeyContent,"BUYMENU") > -1)
					break;
			}
			ReString = "Press '"$LoadOutKeyName$"' to change loadouts";
			C.DrawColor = Default.WhiteColor;
			C.Font = GetSmallerFontFor(C,3);
			C.SetPos(WaveMessageXPos*C.ClipX, WaveMessageYPos*C.ClipY);
			C.DrawText(ReString,false);
		}
	}
}

// draw respawn information for dead players
function DrawPreRoundInfo(Canvas C)
{
	local string ReString, KeyContent;
	local int i;
	
	if ((Level.TimeSeconds - LastKeyTime > 2.0) || LastKeyTime == 0)
	{
		for (i=0;i<256;i++)
		{
			LoadOutKeyName = caps(PlayerOwner.ConsoleCommand("KEYNAME"@i));
			KeyContent = caps(PlayerOwner.ConsoleCommand("KEYBINDING"@LoadOutKeyName));
			if (InStr(KeyContent,"BUYMENU") > -1)
				break;
		}
		ReString = "Press '"$LoadOutKeyName$"' to change loadouts";
		C.DrawColor = Default.WhiteColor;
		C.Font = GetSmallerFontFor(C,4);
		C.SetPos(WaveMessageXPos*C.ClipX, WaveMessageYPos*C.ClipY);
		C.DrawText(ReString,false);
	}
}

function LocalizedMessage(class<LocalMessage> Message, optional int Switch, optional PlayerReplicationInfo RelatedPRI_1, optional PlayerReplicationInfo RelatedPRI_2, optional Object OptionalObject, optional string CriticalString)
{
	if (Message.Name == 'War_Interface_BuildZoneMessage')
		PrintWaveMessage(GetWaveMessage(Switch));
	else if (Message.Name == 'War_Interface_GameMessage' && Switch == 0)
		return;
	else
		Super.LocalizedMessage(Message, Switch, RelatedPRI_1, RelatedPRI_2, OptionalObject, CriticalString);
}

function string GetWaveMessage(byte MessageNum)
{
	local int i;
	local string KeyName, KeyContent;

	Switch(MessageNum)
	{
		case 0:
			for (i=0;i<256;i++)
			{
				KeyName = caps(PlayerOwner.ConsoleCommand("KEYNAME"@i));
				KeyContent = caps(PlayerOwner.ConsoleCommand("KEYBINDING"@KeyName));
				if (InStr(KeyContent,"BUYAMMO") > -1)
					break;
			}
			return "Press '"$KeyName$"' to resupply";
			break;

		case 1:
			return "Resupply complete!";
			break;
		
		default:
			break;
	}
	return "";
}

function Message(PlayerReplicationInfo PRI, coerce string Msg, name MsgType)
{
	if (MsgType != 'War_Interface_BuildZoneMessage' || Msg ~= "")
	{
		Super.Message(PRI,Msg,MsgType);
		return;
	}
}

function DrawWaveMessage(Canvas C)
{
	local color MessageColor;

	if (FOPawn(PawnOwner) == None || PlayerOwner.IsDead() || PlayerOwner.IsInState('Building'))
		return;

	// fade out if leaving buildzone
	if (!FOPawn(PawnOwner).HasClientBuildZone() && !bFadeOut && (Level.TimeSeconds - WaveMessageTime > 0.1*WaveDelay))
	{
		bFadeOut = true;
		if (WaveMessage != "Resupply complete!")
			WaveMessageTime = Level.TimeSeconds - 0.9*WaveDelay;
	}

	MessageColor = Default.WhiteColor;
	if (Level.TimeSeconds-WaveMessageTime >= 0.9*WaveDelay)
		MessageColor.A = FClamp(255.0*((1.0-((Level.TimeSeconds-WaveMessageTime)/WaveDelay))/0.1), 0, 255.0);
	else
		MessageColor.A = FClamp(255.0*((Level.TimeSeconds-WaveMessageTime)/(0.08*WaveDelay)), 0, 255.0);

	C.DrawColor = MessageColor;
	C.Font = GetSmallerFontFor(C,4);
	C.SetPos(WaveMessageXPos*C.ClipX, WaveMessageYPos*C.ClipY);
	C.DrawText(WaveMessage,false);
}

function PrintWaveMessage(string WMsg)
{
	WaveMessageTime = Level.TimeSeconds;
	WaveMessage = WMsg;
	bFadeOut = false;
}

function DrawSpectatingHud(Canvas C)
{
	Super.DrawSpectatingHUD(C);
	
	if (PlayerOwner.bBehindView)
		DrawWaveInfo(C);
}

function DrawSpecTimes(Canvas C)
{
	if (!bShowPoints || PlayerOwner == None)
		return;

	// set Team
	if (PawnOwnerPRI != none && PawnOwnerPRI.Team != None)
		TeamIndex = Clamp(PawnOwnerPRI.Team.TeamIndex, 0, 1);
	
	// set GRI
	GRI = FragOps_GRI(PlayerController(Owner).GameReplicationInfo);

	// retrieve times
	if (GRI != None)
	{
		RoundMinutes.Value = GetMinutes(GRI.RoundTimeRemaining);
		RoundSeconds.Value = GetSeconds(GRI.RoundTimeRemaining);
		MapMinutes.Value = GetMinutes(GRI.RemainingTime);
		MapSeconds.Value = GetSeconds(GRI.RemainingTime);

		// wave
		if ((TeamIndex == 0 && War_GRI(GRI).MercWaveDelayTime <= War_GRI(GRI).WaveWindowTime) || (TeamIndex == 1 && War_GRI(GRI).SOWaveDelayTime <= War_GRI(GRI).WaveWindowTime))
		{
			WaveMinutes.Value = 0;
			WaveSeconds.Value = 0;
		}
		else if (TeamIndex == 0)
		{
			WaveMinutes.Value = GetMinutes(War_GRI(GRI).MercWaveTimeRemaining);
			WaveSeconds.Value = GetSeconds(War_GRI(GRI).MercWaveTimeRemaining);
		}
		else
		{
			WaveMinutes.Value = GetMinutes(War_GRI(GRI).SOWaveTimeRemaining);
			WaveSeconds.Value = GetSeconds(War_GRI(GRI).SOWaveTimeRemaining);
		}
	}

	// draw times
	DrawSpriteWidget (C, TimerBG);
	DrawNumericWidget (C, MapMinutes, DigitsBig);
	DrawNumericWidget (C, MapSeconds, DigitsBig);
	DrawSpriteWidget (C, TimeSeperator[0]);
	DrawNumericWidget (C, RoundMinutes, DigitsBig);
	DrawNumericWidget (C, RoundSeconds, DigitsBig);
	DrawSpriteWidget (C, TimeSeperator[1]);
	DrawNumericWidget(C, WaveMinutes, DigitsBig);
	DrawNumericWidget(C, WaveSeconds, DigitsBig);
	DrawSpriteWidget(C, WaveTimeSep);
	DrawSpriteWidget(C, MapTimeIcon);
	DrawSpriteWidget(C, RoundTimeIcon);
	DrawSpriteWidget(C, WaveTimeIcon);
}

function CalculateAmmo()
{
	if (PlayerOwner.Pawn == None)
	{
		Super.CalculateAmmo();
		return;
	}
	
	if (War_FOVehiclePawn(PlayerOwner.Pawn) != None && War_FOVehiclePawn(PlayerOwner.Pawn).Gun != None)
	{
		CurAmmoPrimary = War_FOVehicleWeapon(War_FOVehiclePawn(PlayerOwner.Pawn).Gun).AmmoAmount;
		ClipAmount = War_FOVehicleWeapon(War_FOVehiclePawn(PlayerOwner.Pawn).Gun).NumClips;
	}
	else if (ONSVehicle(PlayerOwner.Pawn) != None && ONSVehicle(PlayerOwner.Pawn).Weapons.Length > 0 && ONSVehicle(PlayerOwner.Pawn).Weapons[0] != None)
	{
		CurAmmoPrimary = War_FOVehicleWeapon(ONSVehicle(PlayerOwner.Pawn).Weapons[0]).AmmoAmount;
		ClipAmount = War_FOVehicleWeapon(ONSVehicle(PlayerOwner.Pawn).Weapons[0]).NumClips;
	}
	else if (War_FOVehicleAir(PlayerOwner.Pawn) != None && War_FOVehicleAir(PlayerOwner.Pawn).Default.AmmoCount > 0)
	{
		CurAmmoPrimary = War_FOVehicleAir(PlayerOwner.Pawn).AmmoCount;
		ClipAmount = 0;
	}
	else
		Super.CalculateAmmo();
}

function CalculateHealth()
{
	if (Vehicle(PawnOwner) != None)
		CurHealth = Vehicle(PawnOwner).Driver.Health;
	else
		Super.CalculateHealth();
}

function MoveHUDUp()
{
	GrenadeMaxIcon.PosY = Default.GrenadeMaxIcon.PosY-YBump;
	GrenadeCharge.PosY = Default.GrenadeCharge.PosY-YBump;
	PlayerBG.PosY = Default.PlayerBG.PosY-YBump;
	HealthCount.PosY = Default.HealthCount.PosY-YBump;
	HealthCross.PosY = Default.HealthCross.PosY-YBump;
	ArmorTypeBG.PosY = Default.ArmorTypeBG.PosY-YBump;
	ArmorGuyBG.PosY = Default.ArmorGuyBG.PosY-YBump;
	ArmorGuy.PosY = Default.ArmorGuy.PosY-YBump;
	ArmorGuyOutline.PosY += Default.ArmorGuyOutline.PosY-YBump;
	ArmorType.PosY = Default.ArmorType.PosY-YBump;
	HeadZone.PosY = Default.HeadZone.PosY-YBump;
	TorsoZone.PosY = Default.TorsoZone.PosY-YBump;
	LeftArmZone.PosY = Default.LeftArmZone.PosY-YBump;
	LeftLegZone.PosY = Default.LeftLegZone.PosY-YBump;
	RightArmZone.PosY = Default.RightArmZone.PosY-YBump;
	RightLegZone.PosY = Default.RightLegZone.PosY-YBump;
	PoisonLabel.PosY = Default.PoisonLabel.PosY-YBump;
}

function MoveHUDDown()
{
	GrenadeMaxIcon.PosY = Default.GrenadeMaxIcon.PosY;
	GrenadeCharge.PosY = Default.GrenadeCharge.PosY;
	PlayerBG.PosY = Default.PlayerBG.PosY;
	HealthCount.PosY = Default.HealthCount.PosY;
	HealthCross.PosY = Default.HealthCross.PosY;
	ArmorTypeBG.PosY = Default.ArmorTypeBG.PosY;
	ArmorGuyBG.PosY = Default.ArmorGuyBG.PosY;
	ArmorGuy.PosY = Default.ArmorGuy.PosY;
	ArmorGuyOutline.PosY += Default.ArmorGuyOutline.PosY;
	ArmorType.PosY = Default.ArmorType.PosY;
	HeadZone.PosY = Default.HeadZone.PosY;
	TorsoZone.PosY = Default.TorsoZone.PosY;
	LeftArmZone.PosY = Default.LeftArmZone.PosY;
	LeftLegZone.PosY = Default.LeftLegZone.PosY;
	RightArmZone.PosY = Default.RightArmZone.PosY;
	RightLegZone.PosY = Default.RightLegZone.PosY;
	PoisonLabel.PosY = Default.PoisonLabel.PosY;
}

// update HUD
function UpdateHud()
{
	local int i, Func;
	local War_BuildZone BuildZone;

	// ----- begin old
	// set GRI
	GRI = FragOps_GRI(PlayerController(Owner).GameReplicationInfo);

	// set Team
	if (PawnOwnerPRI != none && PawnOwnerPRI.Team != None)
		TeamIndex = Clamp(PawnOwnerPRI.Team.TeamIndex, 0, 1);

	// retrieve ammo counts
	CalculateAmmo();
	AmmoCount.Value = CurAmmoPrimary;
	ClipCount.Value = ClipAmount;

	// retrieve health
	CalculateHealth();
	HealthCount.Value = CurHealth;

	// set hitzones

	// some new
	if (PlayerOwner.Pawn != None && Vehicle(PlayerOwner.Pawn) != None)
	{
		MoveHUDUp();
		
		// get head damage
		HeadDamage = FOPawn(Vehicle(PlayerOwner.Pawn).Driver).HeadDamage;
		TorsoDamage = FOPawn(Vehicle(PlayerOwner.Pawn).Driver).TorsoDamage;
		LeftArmDamage = FOPawn(Vehicle(PlayerOwner.Pawn).Driver).LeftArmDamage;
		LeftLegDamage = FOPawn(Vehicle(PlayerOwner.Pawn).Driver).LeftLegDamage;
		RightArmDamage = FOPawn(Vehicle(PlayerOwner.Pawn).Driver).RightArmDamage;
		RightLegDamage = FOPawn(Vehicle(PlayerOwner.Pawn).Driver).RightLegDamage;

		// vehicle stuff
		if (ONSVehicle(PawnOwner) != None)
		{
			VehicleIcon.WidgetTexture = Vehicle(PawnOwner).VehicleIcon.Material;
			VehicleHealth.Value = PawnOwner.Health;
		}
		else if (ONSWeaponPawn(PawnOwner) != None && ONSWeaponPawn(PawnOwner).VehicleBase != None)
		{
			VehicleIcon.WidgetTexture = ONSWeaponPawn(PawnOwner).VehicleBase.VehicleIcon.Material;
			VehicleHealth.Value = ONSWeaponPawn(PawnOwner).VehicleBase.Health;
		}

		if (FOPawn(Vehicle(PawnOwner).Driver).MiscItem ~= "LIT. Armor")
			ArmorType.WidgetTexture = material'FOMisc.ArmorGuy.HUDArmorLit';
		else if (FOPawn(Vehicle(PawnOwner).Driver).MiscItem ~= "MED. Armor")
			ArmorType.WidgetTexture = material'FOMisc.ArmorGuy.HUDArmorMed';
		else
			ArmorType.WidgetTexture = material'FOMisc.ArmorGuy.HUDArmorHeavy';

		if (War_FOVehiclePawnFree(PawnOwner) != None)
			FinalBlend(WeaponIcon.WidgetTexture).Material = Weapon_Base(PawnOwner.Weapon).WeaponIcon;
		else
			FinalBlend(WeaponIcon.WidgetTexture).Material = None;
	} // end some new
	else if (PlayerOwner.Pawn != None && FOPawn(PlayerOwner.Pawn) != None)
	{
		MoveHUDDown();

		// retrieve AP counts
		APCount.Value = FOPlayer(PlayerOwner).ActionPoints;
		APTotal.Value = FOPlayer(PlayerOwner).GetMaxAP();

		// get head damage
		HeadDamage = FOPawn(PlayerOwner.Pawn).HeadDamage;
		TorsoDamage = FOPawn(PlayerOwner.Pawn).TorsoDamage;
		LeftArmDamage = FOPawn(PlayerOwner.Pawn).LeftArmDamage;
		LeftLegDamage = FOPawn(PlayerOwner.Pawn).LeftLegDamage;
		RightArmDamage = FOPawn(PlayerOwner.Pawn).RightArmDamage;
		RightLegDamage = FOPawn(PlayerOwner.Pawn).RightLegDamage;

		// retrieve money
		MoneyCount.Value = FOPlayer(PlayerOwner).CriticalReplicationInfo.Money;

		// swap weapon icons if akimbo
		if (PawnOwner != None && PawnOwner.Weapon != None)
		{
			if (!Weapon_Base(PawnOwner.Weapon).IsAkimbo())
				FinalBlend(WeaponIcon.WidgetTexture).Material = Weapon_Base(PawnOwner.Weapon).WeaponIcon;
			else
				FinalBlend(WeaponIcon.WidgetTexture).Material = Weapon_Base(PawnOwner.Weapon).AkimboIcon;
		}

		if (FOPawn(PawnOwner).MiscItem ~= "LIT. Armor")
			ArmorType.WidgetTexture = material'FOMisc.ArmorGuy.HUDArmorLit';
		else if (FOPawn(PawnOwner).MiscItem ~= "MED. Armor")
			ArmorType.WidgetTexture = material'FOMisc.ArmorGuy.HUDArmorMed';
		else
			ArmorType.WidgetTexture = material'FOMisc.ArmorGuy.HUDArmorHeavy';
	}
	else if (PlayerOwner.ViewTarget != None && FOPawn(PlayerOwner.ViewTarget) != None)
	{
		MoveHUDDown();
		// otherwise get the spectated viewtarget's information

		// AP counts
		APCount.Value = FOPlayer(PlayerOwner).TargetAP;
		APTotal.Value = FOPlayer(PlayerOwner).TargetMaxAP;
		
		// damage zones
		HeadDamage = FOPlayer(PlayerOwner).TargetHeadDamage;
		TorsoDamage = FOPlayer(PlayerOwner).TargetTorsoDamage;
		LeftArmDamage = FOPlayer(PlayerOwner).TargetLeftArmDamage;
		LeftLegDamage = FOPlayer(PlayerOwner).TargetLeftLegDamage;
		RightArmDamage = FOPlayer(PlayerOwner).TargetRightArmDamage;
		RightLegDamage = FOPlayer(PlayerOwner).TargetRightLegDamage;

		// money
		MoneyCount.Value = FOPlayer(PlayerOwner).TargetMoney;

		// weapon icons
		if (SpectatorWeapon != None)
		{
			if (SpectatorWeaponAkimbo == None)
				FinalBlend(WeaponIcon.WidgetTexture).Material = SpectatorWeapon.Master.Default.WeaponIcon;
			else
				FinalBlend(WeaponIcon.WidgetTexture).Material = SpectatorWeapon.Master.Default.AkimboIcon;
		}

		if (FOPlayer(PlayerOwner).TargetMiscItem ~= "LIT. Armor")
			ArmorType.WidgetTexture = material'FOMisc.ArmorGuy.HUDArmorLit';
		else if (FOPlayer(PlayerOwner).TargetMiscItem ~= "MED. Armor")
			ArmorType.WidgetTexture = material'FOMisc.ArmorGuy.HUDArmorMed';
		else
			ArmorType.WidgetTexture = material'FOMisc.ArmorGuy.HUDArmorHeavy';
	}
	else
		MoneyCount.Value = FOPlayer(PlayerOwner).CriticalReplicationInfo.Money;

	CalculateHitZones();

	if (FOPawn(PawnOwner) != None && FOPawn(PawnOwner).bCanBuy)
		MoneyIcon.WidgetTexture = FinalBlend'FOMisc.HUD2k4.HUDMoneyFlash';
	else
		MoneyIcon.WidgetTexture = material'FOMisc.HUD2k4.HUDMoneySign';

	// retrieve round/map times
	RoundMinutes.Value = GetMinutes(GRI.RoundTimeRemaining);
	RoundSeconds.Value = GetSeconds(GRI.RoundTimeRemaining);
	MapMinutes.Value = GetMinutes(GRI.RemainingTime);
	MapSeconds.Value = GetSeconds(GRI.RemainingTime);
	// ----- end old


	// ----- new
	if (PawnOwner == None)
		return;

	// time
	if ((TeamIndex == 0 && War_GRI(GRI).MercWaveDelayTime <= War_GRI(GRI).WaveWindowTime) || (TeamIndex == 1 && War_GRI(GRI).SOWaveDelayTime <= War_GRI(GRI).WaveWindowTime))
	{
		WaveMinutes.Value = 0;
		WaveSeconds.Value = 0;
	}
	else if (TeamIndex == 0)
	{
		WaveMinutes.Value = GetMinutes(War_GRI(GRI).MercWaveTimeRemaining);
		WaveSeconds.Value = GetSeconds(War_GRI(GRI).MercWaveTimeRemaining);
	}
	else
	{
		WaveMinutes.Value = GetMinutes(War_GRI(GRI).SOWaveTimeRemaining);
		WaveSeconds.Value = GetSeconds(War_GRI(GRI).SOWaveTimeRemaining);
	}

	if (FOPawn(PawnOwner) == None)
		return;

	BuildZone = FOPawn(PawnOwner).CurrentBuildZone;
	if (BuildZone == None)
		return;

	// empty
	Quickbar.BarFunction.Remove(0, Quickbar.BarFunction.Length);
	Quickbar.ResourceCount.Remove(0, Quickbar.ResourceCount.Length);
	Quickbar.ResourceMax.Remove(0, Quickbar.ResourceMax.Length);
	Quickbar.ClientUseTime.Remove(0, Quickbar.ClientUseTime.Length);

	// fill
	for (i=0;i<ArrayCount(BuildZone.BuildableItems);i++)
	{
		// ignore empty items
		if (BuildZone.BuildableItems[i].ResourceAmount == 0 || BuildZone.BuildableItems[i].ItemName == NO_ITEM)
			continue;
		
		Quickbar.BarFunction[Func] = BuildZone.BuildableItems[i].ItemType;
		Quickbar.ResourceMax[Func] = BuildZone.BuildableItems[i].ResourceAmount;
		Quickbar.ResourceCount[Func] = BuildZone.ResourceCount[i];
		Quickbar.ClientUseTime[Func] = BuildZone.ClientUseTime[i];
		Func++;
	}
}
// draw nifty quickbar
function DrawQuickbar(Canvas C)
{
	local int i;
	local float RealPosX, RealPosY, WiperHeight, XL, YL;
	
	if (PawnOwner == None || QuickbarBG.PosY == 1.1)
		return;

	C.Style = ERenderStyle.STY_Alpha;

	// draw icons
	for (i=0;i<10;i++)
	{
		// set position
		if (i < 5)
			RealPosX = C.ClipX*0.5 - C.ClipX*IconOffsetX - 5*(C.ClipX*IconSize) - 4*(C.ClipX*IconSpace) + i*(C.ClipX*IconSize) + i*(C.ClipX*IconSpace);
		else
			RealPosX = C.ClipX*0.5 + C.ClipX*IconOffsetX + (i-5)*(C.ClipX*IconSize) + (i-5)*(C.ClipX*IconSpace);
		RealPosY = QuickbarBG.PosY*C.ClipY - IconOffsetY*C.ClipY - C.ClipX*IconSize;

		// decide if active, set colors accordingly
		C.DrawColor = IconInActiveColor;

		// fallback, empty icon
		if (i >= Quickbar.BarFunction.Length)
		{
			// draw background
			C.DrawColor = IconEmptyColor;
			C.SetPos(RealPosX, RealPosY);
			C.DrawTileStretched(Material'FOMisc.Special.WhiteFlash', IconSize*C.ClipX, IconSize*C.ClipX);

			// draw item number
			C.DrawColor = Default.GrayColor;
			C.SetPos(RealPosX, RealPosY);
			C.Font = GetSmallerFontFor(C,4);
			C.StrLen(string(i+1), XL, YL);
			C.SetPos(RealPosX + ItemNumberOffsetX*C.ClipX - XL, RealPosY + ItemNumberOffsetY*C.ClipY - YL);
			C.DrawText(string(i+1), false);

			// draw border
			C.DrawColor = IconBorderColor;
			C.SetPos(RealPosX, RealPosY);
			C.DrawTileStretched(Material'FOMisc.HUD.BriefImageBorder', IconSize*C.ClipX, IconSize*C.ClipX);
			
			continue;
		}
		
		// brighten if active
		if (Quickbar.ResourceCount[i] > 0)
			C.DrawColor = IconActiveColor;

		// draw icon
		C.SetPos(RealPosX, RealPosY);
		if (War_FOPlayer(PlayerOwner).BuildFunction != None && War_FOPlayer(PlayerOwner).BuildFunction.static.GetLevel(FOPlayer(PlayerOwner)) == Quickbar.BarFunction[i].static.GetLevel(FOPlayer(PlayerOwner))) // flash if building
		{
			IconFlasher.Material1 = Quickbar.BarFunction[i].static.GetLevel(FOPlayer(PlayerOwner)).Default.BarIcon;
			C.DrawTile(IconFlasher, IconSize*C.ClipX, IconSize*C.ClipX, 0, 0, Quickbar.BarFunction[i].static.GetLevel(FOPlayer(PlayerOwner)).Default.BarIcon.MaterialUSize(), Quickbar.BarFunction[i].static.GetLevel(FOPlayer(PlayerOwner)).Default.BarIcon.MaterialVSize());
		}
		else
			C.DrawTile(Quickbar.BarFunction[i].static.GetLevel(FOPlayer(PlayerOwner)).Default.BarIcon, IconSize*C.ClipX, IconSize*C.ClipX, 0, 0, Quickbar.BarFunction[i].static.GetLevel(FOPlayer(PlayerOwner)).Default.BarIcon.MaterialUSize(), Quickbar.BarFunction[i].Default.BarIcon.MaterialVSize());

		// draw wiper if inactive
		if ((Level.TimeSeconds - Quickbar.ClientUseTime[i]) < Quickbar.Barfunction[i].static.GetLevel(FOPlayer(PlayerOwner)).Default.UseDelayTime && Quickbar.ClientUseTime[i] > 1)
		{
			// get wiper height
			WiperHeight = 1.0 - FClamp((Level.TimeSeconds-Quickbar.ClientUseTime[i]) / Quickbar.Barfunction[i].static.GetLevel(FOPlayer(PlayerOwner)).Default.UseDelayTime, 0.0, 1.0);
			// draw wiper
			C.SetDrawColor(26,26,26,175);
			C.SetPos(RealPosX, RealPosY);
			C.DrawTileStretched(Material'FOMisc.Special.WhiteFlash', IconSize*C.ClipX, WiperHeight*(IconSize*C.ClipX));
		}

		// draw item number
		if (((Level.TimeSeconds - Quickbar.ClientUseTime[i]) >= Quickbar.BarFunction[i].static.GetLevel(FOPlayer(PlayerOwner)).Default.UseDelayTime || Quickbar.ClientUseTime[i] == 1) && Quickbar.ResourceCount[i] > 0)
			C.DrawColor = Default.WhiteColor;
		else
			C.SetDrawColor(0,0,0,0);
		C.Font = GetSmallerFontFor(C,4);
		C.StrLen(string(i+1), XL, YL);
		C.SetPos(RealPosX + ItemNumberOffsetX*C.ClipX - XL, RealPosY + ItemNumberOffsetY*C.ClipY - YL);
		C.DrawText(string(i+1), false);

		// draw resource meter
		XL = FClamp(float(Quickbar.ResourceCount[i])/float(Quickbar.ResourceMax[i]), 0, 1.0);
		XL = XL*((IconSize*C.ClipX) - MeterOffsetX*2.0*C.ClipX);
		// drop shadow
		C.SetDrawColor(180,180,180,180);
		C.SetPos(RealPosX + MeterOffsetX*C.ClipX, RealPosY + MeterOffsetY*C.ClipY);
		C.DrawTileStretched(Material'FOMisc.Special.WhiteFlash', (IconSize*C.ClipX)-MeterOffsetX*2.0*C.ClipX, IconSize*0.1*C.ClipX);
		// meter
		C.DrawColor = MeterColor;
		C.SetPos(RealPosX + MeterOffsetX*C.ClipX, RealPosY + MeterOffsetY*C.ClipY);
		C.DrawTileStretched(Material'FOMisc.Special.WhiteFlash', XL, IconSize*0.1*C.ClipX);

		// draw border
		C.DrawColor = IconBorderColor;
		C.SetPos(RealPosX, RealPosY);
		C.DrawTileStretched(Material'FOMisc.HUD.BriefImageBorder', IconSize*C.ClipX, IconSize*C.ClipX);
	}
}

// slide/fade in nifty quickbar
function SlideBarIn(float dt)
{
	if (PawnOwner == None || FOPawn(PawnOwner) == None || FOPawn(PawnOwner).CurrentBuildZone == None)
		return;
	
	// slide in bg
	QuickbarBG.PosY = FClamp(QuickbarBG.PosY - dt*QuickbarBG.PosY*0.45, 1.0, 1.1);
	QuickbarBG.Tints[0].A = Clamp(QuickbarBG.Tints[0].A + dt*float(QuickbarBG.Tints[0].A)*10.0, 20, 255);
	QuickbarBG.Tints[1] = QuickbarBG.Tints[0];
}

// slide/fade out nifty quickbar
function SlideBarOut(float dt)
{
	if (PawnOwner == None || (FOPawn(PawnOwner) != None && FOPawn(PawnOwner).CurrentBuildZone != None))
		return;
	
	// slide out bg
	QuickbarBG.PosY = FClamp(QuickbarBG.PosY + dt*(1.1-QuickbarBG.PosY)*3.8, 1.01, 1.1);
	QuickbarBG.Tints[0].A = Clamp(QuickbarBG.Tints[0].A - dt*float(255-QuickbarBG.Tints[0].A)*20.0, 20, 254);
	QuickbarBG.Tints[1] = QuickbarBG.Tints[0];
}

// don't draw enemy names in War
function DrawEnemyName(Canvas C)
{
}

function Tick(float dt)
{
	Super.Tick(dt);

	// objective splash window
	if (bObjSplash)
		ObjSplashTime = FMax(ObjSplashTime-dt, 0);
	if (ObjSplashTime == 0 || PlayerOwner.IsDead() || PawnOwner == None)
	{
		ObjSplashTime = 0;
		bObjSplash = false;
	}

	// quickbar
	SlideBarIn(dt);
	SlideBarOut(dt);
	SlideObjIn(dt);
	SlideObjOut(dt);
}

// interact objective progress bar
function DrawProgressBar(Canvas Canvas)
{
	local int ProgressPercent;
	local float fProgress;

	if (bForceProgressStop)
		return;

	// setup the flash if needed
	if (Level.TimeSeconds - ObjectiveStart > ObjectiveTime && !bFlashBar)
	{
		bFlashBar = true;
		FlashTime = Level.TimeSeconds;
	}

	// flash the bar if the objective was completed
	if (bFlashBar && Level.TimeSeconds - FlashTime < 1.0 && Level.TimeSeconds - ObjectiveStart > ObjectiveTime)
	{
		DrawSpriteWidget(Canvas,ProgressFlashIcon);
		TexRotator(ProgressWiper.WidgetTexture).Rotation.Yaw = 0;
		DrawSpriteWidget(Canvas,ProgressWiperBorder);
		DrawProgressCount(Canvas,100);
		DrawSpriteWidget(Canvas,ProgressWiper);
		return;
	}
	else if (Level.TimeSeconds - FlashTime > 1.0 && Level.TimeSeconds - ObjectiveStart > ObjectiveTime)
	{
		bShowProgressBar = false;
		bFlashBar = false;
		return;
	}
	else if (Level.TimeSeconds - ObjectiveStart <= ObjectiveTime)
		bFlashbar = false;

	// determine the percentage
	fProgress = (((Level.TimeSeconds - ObjectiveStart) / ObjectiveTime) * 100.0);
	ProgressPercent = fProgress;

	// setup the color of the background circle
	if (ProgressPercent < 33)
	{
		ProgressIcon.Tints[0].R = 255;
		ProgressIcon.Tints[0].G = 30;
		ProgressIcon.Tints[0].B = 30;
		ProgressIcon.Tints[1] = ProgressIcon.Tints[0];
	}
	else if (ProgressPercent >= 33 && ProgressPercent < 66)
	{
		if (ProgressIcon.Tints[0].G == 30)
			WiperStart = Level.TimeSeconds;			
		if (!bSecondStage)
		{
			bSecondStage = true;
			if (War_FOPlayer(PlayerOwner).CurWarObjective != None)
				War_FOPlayer(PlayerOwner).CurWarObjective.PlayIntervalSound();
		}
		ProgressIcon.Tints[0].R = 255;
		ProgressIcon.Tints[0].G = 255;
		ProgressIcon.Tints[0].B = 30;
		ProgressIcon.Tints[1] = ProgressIcon.Tints[0];

	}
	else if (ProgressPercent >= 66)
	{
		if (ProgressIcon.Tints[0].R == 255)
			WiperStart = Level.TimeSeconds;
		if (!bThirdStage)
		{
			bThirdStage = true;
			if (War_FOPlayer(PlayerOwner).CurWarObjective != None)
				War_FOPlayer(PlayerOwner).CurWarObjective.PlayIntervalSound();
		}
		ProgressIcon.Tints[0].R = 30;
		ProgressIcon.Tints[0].G = 255;
		ProgressIcon.Tints[0].B = 30;
		ProgressIcon.Tints[1] = ProgressIcon.Tints[0];
	}

	// setup the rotating wiper
	fProgress = (fClamp(((Level.TimeSeconds - WiperStart) / (ObjectiveTime/3.0)), 0.0, 1.0)*65535.0);
	TexRotator(ProgressWiper.WidgetTexture).Rotation.Yaw = fProgress;

	// draw progress bar here
	DrawSpriteWidget(Canvas,ProgressIcon);
	DrawSpriteWidget(Canvas,ProgressWiperBorder);
	DrawProgressCount(Canvas,ProgressPercent);
	DrawSpriteWidget(Canvas,ProgressWiper);
}

function OpenTimedBriefing()
{
	if (BriefingScreen == None)
		return;

	bShowBriefing = true;
	BriefingScreen.bOpenTimed = true;
	BriefingScreen.SetTimer(13.0, false);
}

defaultproperties
{
	RoundStatsClass=class'War_Interface_RoundStats'
	BriefingClass=class'War_Interface_BriefingScreen'

	// wave stuff
	WaveDelay=4.0
	WaveMessageYPos=0.35
	WaveMessageXPos=0.01
	WaveFlashColor=(R=255,G=255,B=255,A=220)

	// radar
	RadarScale=0.13
	RadarPosX=0.92
	RadarPosY=0.005

	// kill message offset
	KillMessagePosY=0.18
	
	// objectives
	ObjPosX=1.03
	ObjPosY=0.65
	ObjAlpha=20
	ObjSplashTime=7.0

	// objective picture/text
	ObjPicBorder=Material'FOMisc.HUD.BriefImageBorder'
	ObjBG=Material'FOWarTextures.HUD.ObjectiveBorder'
	ObjBGScaleX=0.35
	ObjBGScaleY=0.16
	ObjPicScale=0.14
	ObjPicOffsetX=0.01
	ObjPicOffsetY=0.01
	ObjTextWidthX=0.18
	ObjTextOffsetX=0.008
	ObjTextOffsetY=0.01

	// bar bg
	QuickbarBG=(WidgetTexture=Material'FOWarTextures.HUD.QuickbarBG',TextureCoords=(X1=100,Y1=50,X2=924,Y2=128),TextureScale=0.65,DrawPivot=DP_LowerMiddle,PosX=0.5,PosY=1.1,OffsetX=0,OffsetY=0,ScaleMode=SM_Right,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=255,A=20),Tints[1]=(R=255,G=255,B=255,A=20)) // -- Bottom Right Clip Icon

	// vehicles
	VehicleBG=(WidgetTexture=Material'FOMisc.HUD2k4.HUDPlayerBG',TextureCoords=(X1=0,Y1=0,X2=256,Y2=64),TextureScale=0.65,DrawPivot=DP_LowerLeft,PosX=0.0,PosY=1.0,OffsetX=0,OffsetY=0,ScaleMode=SM_Right,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=255,A=255),Tints[1]=(R=255,G=255,B=255,A=255)) // -- Bottom Right Clip Icon
	VehicleIcon=(WidgetTexture=Material'FOMisc.HUD2k4.HUDHealthCross',TextureCoords=(X1=0,Y1=0,X2=32,Y2=32),TextureScale=0.65,DrawPivot=DP_LowerLeft,PosX=0.02,PosY=0.98,OffsetX=0,OffsetY=0,ScaleMode=SM_Right,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=220,G=220,B=220,A=191),Tints[1]=(R=220,G=220,B=220,A=191)) // -- Bottom Right Clip Icon
	VehicleHealth=(MinDigitCount=3,TextureScale=0.47,DrawPivot=DP_LowerLeft,PosX=0.059,PosY=0.99,OffsetX=0,OffsetY=0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=255,A=255),Tints[1]=(R=255,G=255,B=255,A=255),bPadWithZeroes=1)
	YBump=0.075

	// time
	TimerBG=(WidgetTexture=Material'FOWarTextures.HUD.TimerBG',TextureCoords=(X1=0,Y1=0,X2=114,Y2=100),TextureScale=0.65,DrawPivot=DP_UpperLeft,PosX=0,PosY=0,OffsetX=0,OffsetY=0,ScaleMode=SM_Right,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=255,A=255),Tints[1]=(R=255,G=255,B=255,A=255)) // -- Timers box 
	MapMinutes=(MinDigitCount=2,TextureScale=0.18,DrawPivot=DP_UpperLeft,PosX=0.035,PosY=0.013,OffsetX=0,OffsetY=0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=255,A=255),Tints[1]=(R=255,G=255,B=255,A=255),bPadWithZeroes=1)
	MapSeconds=(MinDigitCount=2,TextureScale=0.18,DrawPivot=DP_UpperLeft,PosX=0.035,PosY=0.013,OffsetX=110,OffsetY=0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=255,A=255),Tints[1]=(R=255,G=255,B=255,A=255),bPadWithZeroes=1)
	MapTimeIcon=(WidgetTexture=Material'FOWarTextures.HUD.MapTimeIcon',TextureCoords=(X1=0,Y1=0,X2=32,Y2=32),TextureScale=0.57,DrawPivot=DP_UpperLeft,PosX=0.008,PosY=0.006,OffsetX=0,OffsetY=0,ScaleMode=SM_None,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=255,A=220),Tints[1]=(R=255,G=255,B=255,A=220))
	TimeSeperator[0]=(WidgetTexture=Material'FOMisc.HUD2k4.HUDDigitsBig',TextureCoords=(X1=203,Y1=128,X2=245,Y2=192),TextureScale=0.18,DrawPivot=DP_UpperLeft,PosX=0.056,PosY=0.013,OffsetX=0,OffsetY=0,ScaleMode=SM_Right,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=255,A=255),Tints[1]=(R=255,G=255,B=255,A=255)) // -- Timers semicolon
	RoundMinutes=(MinDigitCount=2,TextureScale=0.18,DrawPivot=DP_UpperLeft,PosX=0.035,PosY=0.039,OffsetX=0,OffsetY=0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=255,A=255),Tints[1]=(R=255,G=255,B=255,A=255),bPadWithZeroes=1)
	RoundSeconds=(MinDigitCount=2,TextureScale=0.18,DrawPivot=DP_UpperLeft,PosX=0.035,PosY=0.039,OffsetX=110,OffsetY=0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=255,A=255),Tints[1]=(R=255,G=255,B=255,A=255),bPadWithZeroes=1)
	RoundTimeIcon=(WidgetTexture=Material'FOWarTextures.HUD.RoundTimeIcon',TextureCoords=(X1=0,Y1=0,X2=32,Y2=32),TextureScale=0.57,DrawPivot=DP_UpperLeft,PosX=0.008,PosY=0.032,OffsetX=0,OffsetY=0,ScaleMode=SM_None,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=255,A=220),Tints[1]=(R=255,G=255,B=255,A=220))
	TimeSeperator[1]=(WidgetTexture=Material'FOMisc.HUD2k4.HUDDigitsBig',TextureCoords=(X1=203,Y1=128,X2=245,Y2=192),TextureScale=0.18,DrawPivot=DP_UpperLeft,PosX=0.056,PosY=0.039,OffsetX=0,OffsetY=0,ScaleMode=SM_Right,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=255,A=255),Tints[1]=(R=255,G=255,B=255,A=255)) // -- Timers semicolon
	WaveMinutes=(MinDigitCount=1,TextureScale=0.18,DrawPivot=DP_UpperLeft,PosX=0.047,PosY=0.071,OffsetX=0,OffsetY=0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=255,A=255),Tints[1]=(R=255,G=255,B=255,A=255),bPadWithZeroes=1)
	WaveSeconds=(MinDigitCount=2,TextureScale=0.18,DrawPivot=DP_UpperLeft,PosX=0.035,PosY=0.071,OffsetX=110,OffsetY=0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=255,A=255),Tints[1]=(R=255,G=255,B=255,A=255),bPadWithZeroes=1)
	WaveTimeIcon=(WidgetTexture=Material'FOWarTextures.HUD.WaveTimeIcon',TextureCoords=(X1=0,Y1=0,X2=64,Y2=64),TextureScale=0.34,DrawPivot=DP_UpperLeft,PosX=0.009,PosY=0.0616,OffsetX=0,OffsetY=0,ScaleMode=SM_None,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=255,A=220),Tints[1]=(R=255,G=255,B=255,A=220))
	WaveTimeSep=(WidgetTexture=Material'FOMisc.HUD2k4.HUDDigitsBig',TextureCoords=(X1=203,Y1=128,X2=245,Y2=192),TextureScale=0.18,DrawPivot=DP_UpperLeft,PosX=0.056,PosY=0.071,OffsetX=0,OffsetY=0,ScaleMode=SM_Right,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=255,A=255),Tints[1]=(R=255,G=255,B=255,A=255)) // -- Timers semicolon

	// icon properties
	IconOffsetX=0.0033
	IconOffsetY=0.0168
	IconSize=0.051
	IconSpace=0.0066
	IconFlasher=Combiner'FOWarTextures.BuildIcons.IconFlash_cb'

	// meter properties
	MeterOffsetX=0.004
	MeterOffsetY=0.004

	// item number offset
	ItemNumberOffsetX=0.0473
	ItemNumberOffsetY=0.0703

	// colors
	IconActiveColor=(R=255,G=255,B=255,A=255)
	IconInActiveColor=(R=100,G=100,B=100,A=255)
	MeterColor=(R=255,G=255,B=90,A=255)
	WiperColor=(R=160,G=160,B=160,A=255)
	IconEmptyColor=(R=0,G=0,B=0,A=120)
	IconBorderColor=(R=255,G=255,B=255,A=255)

	// message settings
	ConsoleMessagePosX=0.011
	ConsoleMessagePosY=0.16
}