//=============================================================================
// Frag.Ops  2004 Matt 'SquirrelZero' Farber
//=============================================================================
class War_Interface_Scoreboard extends Interface_Scoreboard;

simulated function bool InOrder(PlayerReplicationInfo P1, PlayerReplicationInfo P2)
{
	if (P1.bOnlySpectator)
	{
		if (P2.bOnlySpectator)
			return true;
		else
			return false;
	}
	else if (P2.bOnlySpectator)
		return true;

	if (War_PRI(P1).CXP < War_PRI(P2).CXP)
		return false;

	if (War_PRI(P1).CXP == War_PRI(P2).CXP)
	{
		if (P1.Score > P2.Score)
			return false;
		if ((P1.Score == P2.Score) && (PlayerController(P2.Owner) != None) && (Viewport(PlayerController(P2.Owner).Player) != None))
			return false;
	}
	return true;
}


function DrawTeam(int TeamNum, int PlayerCount, int OwnerOffset, Canvas Canvas, int FontReduction, int BoxSpaceY, int PlayerBoxSizeY, int HeaderOffsetY)
{
	local int i, j, OrderXPos, ObjectiveXPos, OwnerPos, NetXPos, NameXPos, BoxTextOffsetY, NumMedals, ScoreXPos, ScoreYPos, BoxXPos, BoxWidth, DeathXPos, LineCount;
	local float XL,YL,NameHeight,MaxNamePos,MedalScale,TraitLength,TeamIconSizeX,TeamIconSizeY;
	local string PlayerName[MAXPLAYERS], AccText, DamageText, TraitLine;
	local int OtherTeam;
	local FragOps_CRI MedalCRI;

	BoxWidth = 0.491 * Canvas.ClipX;
	BoxXPos = 0.5 * (0.5 * Canvas.ClipX - BoxWidth);
	BoxWidth = 0.5 * Canvas.ClipX - 2*BoxXPos;
	BoxXPos = BoxXPos + TeamNum * 0.5 * Canvas.ClipX; 
	NameXPos = BoxXPos + 0.012 * BoxWidth;
	ScoreXPos = BoxXPos + 0.83 * BoxWidth;
	OrderXPos = BoxXPos + 0.627 * BoxWidth;
	DeathXPos = BoxXPos + 0.915 * BoxWidth;
	NetXPos = BoxXPos + 0.50 * BoxWidth;
	MedalScale = Canvas.ClipX/8192;
	ObjectiveXPos = BoxXPos + 0.578 * BoxWidth;

	// draw team header
	Canvas.Style = ERenderStyle.STY_Normal;
	Canvas.StrLen("0",XL,YL);

	// set size of header icons
	TeamIconSizeX = Canvas.ClipX*0.2;
	TeamIconSizeY = Canvas.ClipY*0.03;
	ScoreYPos = HeaderOffsetY - TeamIconSizeY - (Canvas.ClipY*0.02);

	// draw icon
	Canvas.DrawColor = HUDClass.default.WhiteColor * 0.8;
	Canvas.SetPos(Canvas.ClipX*0.25 + ((0.5*Canvas.ClipX)*TeamNum) - ((TeamIconSizeX+XL)*0.5), ScoreYPos);
	Canvas.DrawTile(ScoreText[TeamNum], TeamIconSizeX, TeamIconSizeY, 0, 0, ScoreText[TeamNum].MaterialUSize(), ScoreText[TeamNum].MaterialVSize());

	// draw score
	Canvas.Font = GetSmallFontFor(Canvas.ClipX, 0);
	Canvas.DrawColor = HUDClass.default.WhiteColor;
	Canvas.SetPos(Canvas.ClipX*0.25 + ((0.5*Canvas.ClipX)*TeamNum) - ((TeamIconSizeX+XL)*0.5) + TeamIconSizeX, ScoreYPos);
	Canvas.DrawText(int(GRI.Teams[TeamNum].Score));

	// draw BG box
	Canvas.Style = ERenderStyle.STY_Alpha;
	Canvas.SetPos(BoxXPos, HeaderOffsetY);
	Canvas.DrawColor = TeamColors[TeamNum];
	Canvas.DrawTileStretched(Material'FOMisc.Special.WhiteFlash', BoxWidth, PlayerCount*(PlayerBoxSizeY + BoxSpaceY));	
	Canvas.Style = ERenderStyle.STY_Normal;

	// set font
	Canvas.Font = GetSmallFontFor(Canvas.ClipX, FontReduction);

	// set maximum name length
	MaxNamePos = 0.80 * (OrderXPos - NameXPos);
	for (i=0;i<PlayerCount;i++)
	{
		PlayerName[i] = PRIArray[i].PlayerName;
		Canvas.StrLen(PlayerName[i], XL, YL);
		if (XL > MaxNamePos)
			PlayerName[i] = left(PlayerName[i], MaxNamePos/XL * len(PlayerName[i]));
	}
	if (OwnerOffset >= PlayerCount)
	{
		PlayerName[OwnerOffset] = PRIArray[OwnerOffset].PlayerName;
		Canvas.StrLen(PlayerName[OwnerOffset], XL, YL);
		if (XL > MaxNamePos)
			PlayerName[OwnerOffset] = left(PlayerName[OwnerOffset], MaxNamePos/XL * len(PlayerName[OwnerOffset]));
	}	

	// initialize some settings
	Canvas.Style = ERenderStyle.STY_Normal;
	BoxTextOffsetY = HeaderOffsetY;
	NameHeight = YL;

	for (i=0;i<PlayerCount;i++)
		if (i != OwnerOffset)
		{
			Canvas.SetPos(NameXPos, (PlayerBoxSizeY + BoxSpaceY)*i + BoxTextOffsetY);
			if (PRIArray[i].bAdmin)
			{
				if (PRIArray[i].bOutOfLives || FragOps_PRI(PRIArray[i]).bEscaped)
					Canvas.DrawColor = HUDClass.default.RedColor * 0.7;				
				else
					Canvas.DrawColor = HUDClass.default.RedColor;
			}
			else
			{
				if (PRIArray[i].bOutOfLives || FragOps_PRI(PRIArray[i]).bEscaped)
					Canvas.DrawColor = HUDClass.default.GrayColor * 0.7;				
				else
					Canvas.DrawColor = HUDClass.default.WhiteColor;
			}
			Canvas.DrawText(PlayerName[i],true);
		}

	// draw scores
	Canvas.DrawColor = HUDClass.default.WhiteColor;
	for (i=0;i<PlayerCount;i++)
		if (i != OwnerOffset)
		{
			if (PRIArray[i].bOutOfLives || FragOps_PRI(PRIArray[i]).bEscaped)
				Canvas.DrawColor = HUDClass.default.GrayColor  * 0.7;
			else
				Canvas.DrawColor = HUDClass.default.WhiteColor;
			Canvas.SetPos(ScoreXPos, (PlayerBoxSizeY + BoxSpaceY)*i + BoxTextOffsetY);
			Canvas.DrawText(War_PRI(PRIArray[i]).CXP,true);
		}

	// draw objective icon
	for (i=0;i<PlayerCount;i++)
		if (i != OwnerOffset && FragOps_PRI(PRIArray[i]).bHasObjective && PRIArray[i].Team.TeamIndex == PlayerController(Owner).PlayerReplicationInfo.Team.TeamIndex)
		{
			Canvas.SetDrawColor(255,255,255,200);
			Canvas.SetPos(ObjectiveXPos, (PlayerBoxSizeY + BoxSpaceY)*i + (HeaderOffsetY + 0.23*PlayerBoxSizeY));
			Canvas.DrawIcon(CarriedObjectiveIcon, Canvas.ClipX/1500);
		}

	// draw player traits and medals
	for (i=0;i<PlayerCount;i++)
		if (i != OwnerOffset)
		{
			// traits		
			Canvas.Font = GetSmallFontFor(Canvas.ClipX, FontReduction+1);
			if (FragOps_PRI(PRIArray[i]).bEscaped)
				TraitLine = "[ESCAPED]";
			else if (PRIArray[i].bOutOflives)
				TraitLine = "[DEAD]";
			else if (FragOps_PRI(PRIArray[i]).TraitType >= class'Interface_TabPlayerTeam'.Default.Traits.Length)
				TraitLine = "No Trait";
			else
				TraitLine = class'Interface_TabPlayerTeam'.Default.Traits[FragOps_PRI(PRIArray[i]).TraitType].Default.TraitName;
			Canvas.StrLen(TraitLine, XL, YL);
			if (PRIArray[i].bOutOfLives || FragOps_PRI(PRIArray[i]).bEscaped)
				Canvas.DrawColor = HUDClass.default.WhiteColor * 0.5;
			else
				Canvas.DrawColor = HUDClass.default.WhiteColor * 0.7;
			Canvas.SetPos(NameXPos, (PlayerBoxSizeY + BoxSpaceY)*i + BoxTextOffsetY + (NameHeight*1.1));
			Canvas.DrawText(TraitLine);
			TraitLength = XL;

			// medals
			for (j=0;j<FragOps_GRI(GRI).CRIArray.Length;j++)
			{
				if (CRIArray[j].PCName ~= FragOps_PRI(PRIArray[i]).PCName)
				{
					MedalCRI = CRIArray[j];
					break;
				}
			}
			if (PRIArray[i].bOutOfLives || FragOps_PRI(PRIArray[i]).bEscaped)
				Canvas.DrawColor = HUDClass.default.WhiteColor * 0.6;
			else
				Canvas.DrawColor = HUDClass.default.WhiteColor * 0.8;

			Canvas.SetPos((NameXPos + (Canvas.ClipX*0.005) + TraitLength), (PlayerBoxSizeY + BoxSpaceY)*i + BoxTextOffsetY + (NameHeight*1.04));
			NumMedals = 0;
			for(j=0;j<ArrayCount(MedalCRI.Medals);j++)
				//if (j == 0 || j == 1)
				if (MedalCRI.Medals[j] > 0)
				{
					Canvas.DrawTile(MedalMaterial[j], PlayerBoxSizeY*(0.40-(0.11*FontReduction)), PlayerBoxSizeY*(0.40-(0.11*FontReduction)), 0, 0, MedalMaterial[j].MaterialUSize(), MedalMaterial[j].MaterialVSize());
					NumMedals++;
					Canvas.SetPos((NameXPos + (Canvas.ClipX*0.005) + TraitLength) + PlayerBoxSizeY*(0.45-(0.11*FontReduction)), (PlayerBoxSizeY + BoxSpaceY)*i + BoxTextOffsetY + (NameHeight*1.04));
				}		
		}

	Canvas.Font = GetSmallFontFor(Canvas.ClipX, FontReduction);
	
	// draw owner line
	if (OwnerOffset >= 0)
	{
		for (j=0;j<FragOps_GRI(GRI).CRIArray.Length;j++)
		{
			if (CRIArray[j].PCName == FragOps_PRI(PRIArray[OwnerOffset]).PCName)
			{
				MedalCRI = CRIArray[j];
				break;
			}
		}
		if (OwnerOffset >= PlayerCount)
		{
			OwnerPos = (PlayerBoxSizeY + BoxSpaceY)*PlayerCount + BoxTextOffsetY;
			Canvas.Style = ERenderStyle.STY_Normal;	
		}
		else
			OwnerPos = (PlayerBoxSizeY + BoxSpaceY)*OwnerOffset + BoxTextOffsetY;


		if (PRIArray[OwnerOffset].bAdmin)
		{
			if (PRIArray[OwnerOffset].bOutOfLives || FragOps_PRI(PRIArray[OwnerOffset]).bEscaped)
				Canvas.DrawColor = HUDClass.default.RedColor * 0.7;				
			else
				Canvas.DrawColor = HUDClass.default.RedColor;
		}
		else
		{
			if (PRIArray[OwnerOffset].bOutOfLives || FragOps_PRI(PRIArray[OwnerOffset]).bEscaped)
				Canvas.DrawColor = HUDClass.default.GoldColor * 0.65;
			else
				Canvas.DrawColor = HUDClass.default.GoldColor;
		}

		Canvas.SetPos(NameXPos, OwnerPos);
		Canvas.DrawText(playername[OwnerOffset],true);
		Canvas.StrLen(playername[OwnerOffset],XL,YL);
		NameHeight = YL;

		if (PRIArray[OwnerOffset].bOutOfLives || FragOps_PRI(PRIArray[OwnerOffset]).bEscaped)
			Canvas.DrawColor = HUDClass.default.GoldColor * 0.65;
		else
			Canvas.DrawColor = HUDClass.default.GoldColor;
		
		Canvas.SetPos(ScoreXPos, OwnerPos);
		Canvas.DrawText(War_PRI(PRIArray[OwnerOffset]).CXP,true);

		// draw objectiveicon		
		if (FragOps_PRI(PRIArray[OwnerOffset]).bHasObjective)
		{
			Canvas.SetDrawColor(255,255,255,200);
			Canvas.SetPos(ObjectiveXPos, OwnerPos + 0.23*PlayerBoxSizeY);
			Canvas.DrawIcon(CarriedObjectiveIcon, Canvas.ClipX/1500);
		}

		// draw owner trait
		Canvas.Font = GetSmallFontFor(Canvas.ClipX, FontReduction+1);
		if (FragOps_PRI(PRIArray[OwnerOffset]).bEscaped)
			TraitLine = "[ESCAPED]";
		else if (PRIArray[OwnerOffset].bOutOflives)
			TraitLine = "[DEAD]";
		else if (FragOps_PRI(PRIArray[OwnerOffset]).TraitType >= class'Interface_TabPlayerTeam'.Default.Traits.Length)
			TraitLine = "No Trait";
		else
			TraitLine = class'Interface_TabPlayerTeam'.Default.Traits[FragOps_PRI(PRIArray[OwnerOffset]).TraitType].Default.TraitName;
		Canvas.StrLen(TraitLine, XL, YL);
		if (PRIArray[OwnerOffset].bOutOfLives || FragOps_PRI(PRIArray[OwnerOffset]).bEscaped)
			Canvas.DrawColor = HUDClass.default.WhiteColor * 0.5;
		else
			Canvas.DrawColor = HUDClass.default.WhiteColor * 0.7;
		Canvas.SetPos(NameXPos, OwnerPos + (NameHeight*1.1));
		Canvas.DrawText(TraitLine);
		TraitLength = XL;

		// draw medals
		NumMedals = 0;
		if (PRIArray[OwnerOffset].bOutOfLives || FragOps_PRI(PRIArray[OwnerOffset]).bEscaped)
			Canvas.DrawColor = HUDClass.default.WhiteColor * 0.6;
		else
			Canvas.DrawColor = HUDClass.default.WhiteColor * 0.8;

		Canvas.SetPos((NameXPos + (Canvas.ClipX*0.005) + TraitLength), OwnerPos + (NameHeight*1.04));
		for(j=0;j<ArrayCount(MedalCRI.Medals);j++)
		{
			//if (j == 0 || j == 1)
			if (MedalCRI.Medals[j] > 0)
			{
				Canvas.DrawTile(MedalMaterial[j], PlayerBoxSizeY*(0.40-(0.11*FontReduction)), PlayerBoxSizeY*(0.40-(0.11*FontReduction)), 0, 0, MedalMaterial[j].MaterialUSize(), MedalMaterial[j].MaterialVSize());
				NumMedals++;
				Canvas.SetPos((NameXPos + (Canvas.ClipX*0.005) + TraitLength) + PlayerBoxSizeY*(0.40-(0.11*FontReduction))*NumMedals, OwnerPos + (NameHeight*1.04));
			}
		}
	}

	if (TeamNum == 0)
		OtherTeam = 1;
		
	// draw accuracy/hitpoint score and/or orders
	if ((Canvas.ClipX >= 512))
	{
		//BoxTextOffsetY = HeaderOffsetY + 0.5*PlayerBoxSizeY;
		Canvas.DrawColor = HUDClass.default.GreenColor;
		Canvas.Font = GetSmallFontFor(Canvas.ClipX, FontReduction+1);
		Canvas.StrLen("TEST", XL, YL);
		for (i=0;i<PlayerCount;i++)
		{
			LineCount = 0;
			if (YL < 0.5 * PlayerBoxSizeY)
			{
				if (!PRIArray[i].bBot)
				{
					Canvas.SetPos(OrderXPos, ((PlayerBoxSizeY + BoxSpaceY)*i + BoxTextOffsetY)+Canvas.ClipY*(0.005-(float(FontReduction)*0.002)));
					Canvas.DrawColor = HUDClass.default.GreenColor;
					// fix for mac math bug
					AccText = "A: "$(string(max(0,int((float(FragOps_PRI(PRIArray[i]).NumShotsHit) / float(FragOps_PRI(PRIArray[i]).NumShots)) * 100))))$"/"$(string(max(0,int((float(FragOps_PRI(PRIArray[i]).NumHeadShots) / float(FragOps_PRI(PRIArray[i]).NumShotsHit)) * 100))))$"%";
					Canvas.DrawText(AccText);
				}
				Canvas.SetPos(ScoreXPos, (PlayerBoxSizeY + BoxSpaceY)*i + BoxTextOffsetY + (NameHeight*1.1));
				Canvas.DrawColor = HUDClass.default.WhiteColor;
				DamageText = "KD: "$int(PRIArray[i].Score)$"/"$int(PRIArray[i].Deaths);
				Canvas.DrawText(DamageText);
			}
			else
			{
				Canvas.SetPos(OrderXPos, (PlayerBoxSizeY + BoxSpaceY)*i + (HeaderOffsetY + 0.2*PlayerBoxSizeY));
				Canvas.DrawColor = HUDClass.default.WhiteColor;
				DamageText = "KD: "$int(PRIArray[i].Score)$"/"$int(PRIArray[i].Deaths);
				Canvas.DrawText(DamageText);
			}
		}
		if (OwnerOffset >= PlayerCount && PRIArray[OwnerOffset].Team.TeamIndex == TeamNum) 
		{	
			if (YL < 0.5 * PlayerBoxSizeY)
			{
				Canvas.SetPos(OrderXPos, OwnerPos+Canvas.ClipY*(0.005-(float(FontReduction)*0.002)));
				Canvas.DrawColor = HUDClass.default.GreenColor;
				// fix for mac math bug
				AccText = "A: "$(string(max(0,int((float(FragOps_PRI(PRIArray[OwnerOffset]).NumShotsHit) / float(FragOps_PRI(PRIArray[OwnerOffset]).NumShots)) * 100))))$"/"$(string(max(0,int((float(FragOps_PRI(PRIArray[OwnerOffset]).NumHeadShots) / float(FragOps_PRI(PRIArray[OwnerOffset]).NumShotsHit)) * 100))))$"%";
				Canvas.DrawText(AccText);
				Canvas.SetPos(ScoreXPos, OwnerPos + (NameHeight*1.1));
				Canvas.DrawColor = HUDClass.default.WhiteColor;
				DamageText = "KD: "$int(PRIArray[OwnerOffset].Score)$"/"$int(PRIArray[OwnerOffset].Deaths);
				Canvas.DrawText(DamageText);
			}
			else
			{
				Canvas.SetPos(OrderXPos, OwnerPos + (NameHeight*1.1));
				Canvas.DrawColor = HUDClass.default.WhiteColor;
				DamageText = "KD: "$int(PRIArray[OwnerOffset].Score)$"/"$int(PRIArray[OwnerOffset].Deaths);
				Canvas.DrawText(DamageText);
			}

		}
	}
				
	// no ping/time in botmatch
	if (Level.NetMode == NM_Standalone)
		return;

	DrawNetInfo(Canvas,FontReduction,HeaderOffsetY,PlayerBoxSizeY,BoxSpaceY,BoxTextOffsetY,OwnerOffset,PlayerCount,NetXPos);
}

defaultproperties
{
	MedalPrizes(0)="20 CXP Bonus"
	MedalPrizes(1)="20 CXP Bonus"
	MedalPrizes(2)="30 CXP Bonus"
	MedalPrizes(3)="Golden Eagle Unlock!"
	MedalPrizes(4)="35 CXP Bonus"
	MedalPrizes(5)="+1 AP Bonus"
}