//=============================================================================
// Frag.Ops  2004 Matt 'SquirrelZero' Farber
//=============================================================================
class War_Interface_TabPlayerTeam extends Interface_TabPlayerTeam
	config(User);

// build skill levels
var byte SelectedStructures, SelectedMechanic, SelectedWeapons;
var automated moNumericEdit StructuresEdit, MechanicEdit, WeaponsEdit;

// max points allowed
var int MaxPoints;

function InitComponent(GUIController MyController, GUIComponent MyOwner)
{
	local War_LevelInfo WLI;

	bJoinClicked = false;
	Super.InitComponent(MyController, MyOwner);
	foreach PlayerOwner().AllActors(class'War_LevelInfo',WLI)
	{
		break;
	}
	if (!WLI.bTeamDeathmatch)
	{
		TeamDescription.WinTop = 0.38;
		return;
	}
	TeamDescription.WinTop = 0.205;
	RemoveComponent(StructuresEdit);
	RemoveComponent(MechanicEdit);
	RemoveComponent(WeaponsEdit);
}

function bool JoinClick(GUIComponent Sender)
{
	bJoinClicked = true;
	if (!FOPlayer(PlayerOwner()).CriticalReplicationInfo.bChangedTeams)
	{
		if (PlayerOwner().PlayerReplicationInfo.Team == None || SelectedTeam != PlayerOwner().PlayerReplicationInfo.Team.TeamIndex)
			FOPlayer(PlayerOwner()).bOpenBriefing = true;
		FOPlayer(PlayerOwner()).GetNewTeam(SelectedTeam,SelectedRecord,BodySkinCount,HeadSkinCount,SelectedTrait,SelectedStructures,SelectedMechanic,SelectedWeapons);
	}
	if (Interface_LoginMenu(GUIController(PlayerOwner().Player.GUIController).ActivePage) != None)
		Controller.CloseMenu();
	return true;
}

function SkillChange(GUIComponent Sender)
{
	local int PointsLeft;
	local int PointUsage;
	
	PointsLeft = MaxPoints - Clamp((SelectedStructures + SelectedMechanic + SelectedWeapons), 0, MaxPoints);
	if (StructuresEdit == Sender)
	{
		if (StructuresEdit.GetValue() > SelectedStructures)
		{
			PointUsage = Clamp(StructuresEdit.GetValue()-SelectedStructures, 0, PointsLeft);
			StructuresEdit.SetValue(SelectedStructures+PointUsage);
		}
		else if (StructuresEdit.GetValue() < SelectedStructures)
		{
			PointUsage = Clamp(SelectedStructures-StructuresEdit.GetValue(), 0, MaxPoints-PointsLeft);
			StructuresEdit.SetValue(SelectedStructures-PointUsage);
		}
	}
	else if (MechanicEdit == Sender)
	{
		if (MechanicEdit.GetValue() > SelectedMechanic)
		{
			PointUsage = Clamp(MechanicEdit.GetValue()-SelectedMechanic, 0, PointsLeft);
			MechanicEdit.SetValue(SelectedMechanic+PointUsage);
		}
		else if (MechanicEdit.GetValue() < SelectedMechanic)
		{
			PointUsage = Clamp(SelectedMechanic-MechanicEdit.GetValue(), 0, MaxPoints-PointsLeft);
			MechanicEdit.SetValue(SelectedMechanic-PointUsage);
		}
	}
	else if (WeaponsEdit == Sender)
	{
		if (WeaponsEdit.GetValue() > SelectedWeapons)
		{
			PointUsage = Clamp(WeaponsEdit.GetValue()-SelectedWeapons, 0, PointsLeft);
			WeaponsEdit.SetValue(SelectedWeapons+PointUsage);
		}
		else if (WeaponsEdit.GetValue() < SelectedWeapons)
		{
			PointUsage = Clamp(SelectedWeapons-WeaponsEdit.GetValue(), 0, MaxPoints-PointsLeft);
			WeaponsEdit.SetValue(SelectedWeapons-PointUsage);
		}
	}
	SelectedStructures = StructuresEdit.GetValue();
	SelectedMechanic = MechanicEdit.GetValue();
	SelectedWeapons = WeaponsEdit.GetValue();
}

function bool JoinRandomClick(GUIComponent Sender)
{
	local int MercPlayers, SOPlayers, i;
	local float RandSeed;
	local GameReplicationInfo GRI;
	local FOPlayerRecord RandRecord;

	bJoinClicked = true;

	if (PlayerOwner().GameReplicationInfo != None)
		GRI = PlayerOwner().GameReplicationInfo;

	if (!FOPlayer(PlayerOwner()).CriticalReplicationInfo.bChangedTeams)
	{
		for (i=0;i<GRI.PRIArray.Length;i++)
		{
			if (GRI.PRIArray[i].Team != None && GRI.PRIArray[i].Team.TeamIndex == 0)
			{
				MercPlayers++;
			}
			else if (GRI.PRIArray[i].Team != None && GRI.PRIArray[i].Team.TeamIndex == 1)
			{
				SOPlayers++;
			}
		}
		RandSeed = FRand();
		if (SOPlayers > MercPlayers)
		{
			if (RandSeed > 0.5)
				SelectedRecord = 0;
			else
				SelectedRecord = 1;

			SelectedTeam = 0;
			RandRecord = MercRecord[SelectedRecord];
		}
		else
		{
			if (SOPlayers == MercPlayers)
			{
				if (FRand() < 0.5)
				{
					if (RandSeed > 0.5)
						SelectedRecord = 0;
					else
						SelectedRecord = 1;

					SelectedTeam = 0;
					RandRecord = MercRecord[SelectedRecord];
				}
				else
				{
					if (RandSeed > 0.5)
						SelectedRecord = 0;
					else
						SelectedRecord = 1;

					SelectedTeam = 1;
					RandRecord = SORecord[SelectedRecord];
				}
			}
			else
			{
					if (RandSeed > 0.5)
						SelectedRecord = 0;
					else
						SelectedRecord = 1;

					SelectedTeam = 1;
					RandRecord = SORecord[SelectedRecord];
			}
		}

		// pick a random head/bodyskin
		HeadSkinCount = Rand(RandRecord.HeadSkin.Length);
		BodySkinCount = Rand(RandRecord.BodySkin.Length);
		
		// now a random trait
		SelectedTrait = Rand(Traits.Length);

		if (PlayerOwner().PlayerReplicationInfo.Team == None || SelectedTeam != PlayerOwner().PlayerReplicationInfo.Team.TeamIndex)
			FOPlayer(PlayerOwner()).bOpenBriefing = true;

		FOPlayer(PlayerOwner()).GetNewTeam(SelectedTeam,SelectedRecord,BodySkinCount,HeadSkinCount,SelectedTrait,SelectedStructures,SelectedMechanic,SelectedWeapons);
	}

	// close the menu
	if (Interface_LoginMenu(GUIController(PlayerOwner().Player.GUIController).ActivePage) != None)
		Controller.CloseMenu();
	return true;
}

function bool LoadTeamLists(GUIComponent Sender)
{
	local int i, RandFavorite;

	CharacterList.List.Clear();

	if (Sender == MercButton)
	{
		SelectedTeam = 0;
		for (i=0;i<ArrayCount(MercChar);i++)
		{
			if (MercChar[i] != "")
				CharacterList.List.Add(MercChar[i],MercRecord[i]);
		}
	}
	else
	{
		SelectedTeam = 1;
		for (i=0;i<ArrayCount(SOChar);i++)
		{
			if (SOChar[i] != "")
				CharacterList.List.Add(SOChar[i],SORecord[i]);
		}
	}

	// reset
	SelectedTrait = 255;
	SelectedRecord = 0;
	HeadSkinCount = 0;
	BodySkinCount = 0;
	SelectedStructures = 8;
	SelectedMechanic = 8;
	SelectedWeapons = 8;

	// turn off
	StructuresEdit.OnChange = NullChange;
	MechanicEdit.OnChange = NullChange;
	WeaponsEdit.OnChange = NullChange;
	
	// rebuild the list
	RebuildList(SelectedTeam);

	// pick a random favorite
	switch(SelectedTeam)
	{
		case 1:
			CharacterName.Caption = "Special Ops";
			for(i=0;i<ArrayCount(SOFavorite);i++)
			{
				if (SOFavorite[i].SaveName ~= EmptyString)
					break;
			}
			if (i == 0)
			{
				CharacterList.List.OnChange = NullChange;
				CharacterList.List.SetIndex(0);
				CharacterList.List.OnChange = ListClick;
				PlayerLoad.MyComboBox.Edit.SetText(CustomString);
				break;
			}

			RandFavorite = Rand(i);
			SelectedRecord = Clamp(SOFavorite[RandFavorite].GroupType, 0, SORecord.Length-1);
			HeadSkinCount = Clamp(SOFavorite[RandFavorite].SoldierType, 0, SORecord[SelectedRecord].HeadSkin.Length-1);
			BodySkinCount = Clamp(SOFavorite[RandFavorite].BodyType, 0, SORecord[SelectedRecord].BodySkin.Length-1);
			SelectedStructures = Clamp(SOFavorite[RandFavorite].S_Structures, 0, MaxPoints);
			SelectedMechanic = Clamp(SOFavorite[RandFavorite].S_Mechanic, 0, MaxPoints-SelectedStructures);
			SelectedWeapons = (MaxPoints-(SelectedStructures+SelectedMechanic));
			if (SOFavorite[RandFavorite].Trait < Traits.Length)
				SelectedTrait = SOFavorite[RandFavorite].Trait;
			else
				SelectedTrait = 255;

			// set the combobox index to our chosen favorite
			PlayerLoad.OnChange = NullChange;
			PlayerLoad.SetIndex(RandFavorite);
			PlayerLoad.OnChange = InternalOnChange;			
			CharacterList.List.OnChange = NullChange;
			CharacterList.List.SetIndex(SelectedRecord);
			CharacterList.List.OnChange = ListClick;
			break;

		default:
			CharacterName.Caption = "Mercenaries";
			for(i=0;i<ArrayCount(MercFavorite);i++)
			{
				if (MercFavorite[i].SaveName ~= EmptyString)
					break;
			}
			if (i == 0)
			{
				CharacterList.List.OnChange = NullChange;
				CharacterList.List.SetIndex(SelectedRecord);
				CharacterList.List.OnChange = ListClick;
				PlayerLoad.MyComboBox.Edit.SetText(CustomString);
				break;
			}

			RandFavorite = Rand(i);
			SelectedRecord = Clamp(MercFavorite[RandFavorite].GroupType, 0, MercRecord.Length-1);
			HeadSkinCount = Clamp(MercFavorite[RandFavorite].SoldierType, 0, MercRecord[SelectedRecord].HeadSkin.Length-1);
			BodySkinCount = Clamp(MercFavorite[RandFavorite].BodyType, 0, MercRecord[SelectedRecord].BodySkin.Length-1);
			SelectedStructures = Clamp(MercFavorite[RandFavorite].S_Structures, 0, MaxPoints);
			SelectedMechanic = Clamp(MercFavorite[RandFavorite].S_Mechanic, 0, MaxPoints-SelectedStructures);
			SelectedWeapons = (MaxPoints-(SelectedStructures+SelectedMechanic));
			if (MercFavorite[RandFavorite].Trait < Traits.Length)
				SelectedTrait = MercFavorite[RandFavorite].Trait;
			else
				SelectedTrait = 255;

			// set the combobox index to our chosen favorite
			PlayerLoad.OnChange = NullChange;
			PlayerLoad.SetIndex(RandFavorite);
			PlayerLoad.OnChange = InternalOnChange;
			CharacterList.List.OnChange = NullChange;
			CharacterList.List.SetIndex(SelectedRecord);
			CharacterList.List.OnChange = ListClick;
			break;
	}

	// reset upgrade boxes
	StructuresEdit.SetValue(SelectedStructures);
	MechanicEdit.SetValue(SelectedMechanic);
	WeaponsEdit.SetValue(SelectedWeapons);
	StructuresEdit.OnChange = SkillChange;
	MechanicEdit.OnChange = SkillChange;
	WeaponsEdit.OnChange = SkillChange;

	// reset name box 
	PlayerSave.SetText(NameString);	

	// set caption on model box
	ModelBG.Caption = CharacterList.List.Get();

	// turn off model rendering
	bRenderDude = false;

	// set traitlist index
	if (SelectedTrait == 255)
		TraitList.List.SetIndex(0);
	else
		TraitList.List.SetIndex(SelectedTrait+1);

	if (SelectedTeam == 1 && SelectedRecord < ArrayCount(SODesc))
		TeamDescription.SetContent(SODesc[SelectedRecord]);
	else if (SelectedRecord < ArrayCount(MercDesc))
		TeamDescription.SetContent(MercDesc[SelectedRecord]);

	// set box content
	TraitScroll.SetContent(TraitList.List.GetExtra());
	
	// apply settings to model mockup
	SetupTeamDude();
	return false;
}

function bool PlayerClick(GUIComponent Sender)
{
	local int i, j;
	
	if (Sender == SaveButton)
	{
		if (PlayerSave.GetText() ~= NameString)
			return false;
		switch(SelectedTeam)
		{
			case 1:
				// check for dupes first
				for (i=0;i<ArrayCount(SOFavorite);i++)
				{
					if (SOFavorite[i].SaveName ~= PlayerSave.GetText())
					{
						SOFavorite[i].SaveName = PlayerSave.GetText();
						SOFavorite[i].GroupType = SelectedRecord;
						SOFavorite[i].SoldierType = HeadSkinCount;
						SOFavorite[i].BodyType = BodySkinCount;
						SOFavorite[i].Trait = SelectedTrait;
						SOFavorite[j].S_Structures = SelectedStructures;
						SOFavorite[j].S_Mechanic = SelectedMechanic;
						SOFavorite[j].S_Weapons = SelectedWeapons;
						PlayerSave.SetText(SavedString);
						bSaved = true;
						SaveConfig();
						NewIndex = i;
						RebuildList(1);
						SetTimer(2.0, false);
						return true;
					}
				}
				
				// otherwise find an empty slot
				for (i=0;i<ArrayCount(SOFavorite);i++)
				{
					if (SOFavorite[i].SaveName ~= EmptyString)
					{
						SOFavorite[i].SaveName = PlayerSave.GetText();
						SOFavorite[i].GroupType = SelectedRecord;
						SOFavorite[i].SoldierType = HeadSkinCount;
						SOFavorite[i].BodyType = BodySkinCount;
						SOFavorite[i].Trait = SelectedTrait;
						SOFavorite[j].S_Structures = SelectedStructures;
						SOFavorite[j].S_Mechanic = SelectedMechanic;
						SOFavorite[j].S_Weapons = SelectedWeapons;
						PlayerSave.SetText(SavedString);
						bSaved = true;
						SaveConfig();
						NewIndex = i;
						RebuildList(1);
						SetTimer(2.0, false);
						return true;
					}
				}

				// if we made it here, the list is full
				PlayerSave.SetText(ListFullString);
				bListFull = true;
				SetTimer(2.0, false);
				break;

			default:
				// check for dupes first
				for (i=0;i<ArrayCount(MercFavorite);i++)
				{
					if (MercFavorite[i].SaveName ~= PlayerSave.GetText())
					{
						MercFavorite[i].SaveName = PlayerSave.GetText();
						MercFavorite[i].GroupType = SelectedRecord;
						MercFavorite[i].SoldierType = HeadSkinCount;
						MercFavorite[i].BodyType = BodySkinCount;
						MercFavorite[i].Trait = SelectedTrait;
						MercFavorite[j].S_Structures = SelectedStructures;
						MercFavorite[j].S_Mechanic = SelectedMechanic;
						MercFavorite[j].S_Weapons = SelectedWeapons;
						PlayerSave.SetText(SavedString);
						bSaved = true;
						SaveConfig();
						NewIndex = i;
						RebuildList(0);
						SetTimer(2.0, false);
						return true;
					}
				}
				
				// otherwise find an empty slot
				for (i=0;i<ArrayCount(MercFavorite);i++)
				{
					if (MercFavorite[i].SaveName ~= EmptyString)
					{
						MercFavorite[i].SaveName = PlayerSave.GetText();
						MercFavorite[i].GroupType = SelectedRecord;
						MercFavorite[i].SoldierType = HeadSkinCount;
						MercFavorite[i].BodyType = BodySkinCount;
						MercFavorite[i].Trait = SelectedTrait;
						MercFavorite[j].S_Structures = SelectedStructures;
						MercFavorite[j].S_Mechanic = SelectedMechanic;
						MercFavorite[j].S_Weapons = SelectedWeapons;
						PlayerSave.SetText(SavedString);
						bSaved = true;
						SaveConfig();
						NewIndex = i;
						RebuildList(0);
						SetTimer(2.0, false);
						return true;
					}
				}

				// if we made it here, the list is full
				PlayerSave.SetText(ListFullString);
				bListFull = true;
				SetTimer(2.0, false);
				break;		
		}
	}
	else if (Sender == DeleteButton)
	{
		// so we don't get an endless loop
		PlayerLoad.OnChange = NullChange;
		
		// if player clicked an empty box, don't do anything
		if (PlayerLoad.GetItem(PlayerLoad.GetIndex()) ~= EmptyString)
			return true;

		// delete the favorite
		i = PlayerLoad.GetIndex();
		switch(SelectedTeam)
		{
			case 1:
				for(j=i;j<ArrayCount(SOFavorite);j++)
				{
					if (SOFavorite[j+1].SaveName ~= EmptyString)
						break;
					SOFavorite[j] = SOFavorite[j+1];
				}
				SOFavorite[j].SaveName = EmptyString;
				SOFavorite[j].GroupType = 255;
				SOFavorite[j].SoldierType = 255;
				SOFavorite[j].BodyType = 255;
				SOFavorite[j].Trait = 255;
				SOFavorite[j].S_Structures = 8;
				SOFavorite[j].S_Mechanic = 8;
				SOFavorite[j].S_Weapons = 8;
				break;

			default:
				for(j=i;j<ArrayCount(MercFavorite);j++)
				{
					if (MercFavorite[j+1].SaveName ~= EmptyString)
						break;
					MercFavorite[j] = MercFavorite[j+1];
				}
				MercFavorite[j].SaveName = EmptyString;
				MercFavorite[j].GroupType = 255;
				MercFavorite[j].SoldierType = 255;
				MercFavorite[j].BodyType = 255;
				MercFavorite[j].Trait = 255;
				MercFavorite[j].S_Structures = 8;
				MercFavorite[j].S_Mechanic = 8;
				MercFavorite[j].S_Weapons = 8;
				break;
		}
		
		PlayerLoad.RemoveItem(i);
		PlayerLoad.AddItem(EmptyString);
		PlayerLoad.SetIndex(9);
		PlayerSave.SetText(DeletedString);
		bDeleted = true;
		SaveConfig();
		PlayerLoad.OnChange = InternalOnChange;	
		SetTimer(2.0,false);
	}
	
	return true;
}

singular function InternalOnChange(GUIComponent Sender)
{
	local int i;

	if (PlayerLoad != Sender)
		return;

	// no endless loop
	PlayerLoad.OnChange = NullChange;

	// turn off
	StructuresEdit.OnChange = NullChange;
	MechanicEdit.OnChange = NullChange;
	WeaponsEdit.OnChange = NullChange;

	// if player clicked an empty string, do nothing
	if (PlayerLoad.GetText() ~= EmptyString)
	{
		PlayerLoad.SetIndex(0);

		// reset
		ModelBG.Caption = CharacterList.List.Get();
		PlayerLoad.OnChange = InternalOnChange;

		// reset upgrade boxes
		StructuresEdit.SetValue(8);
		MechanicEdit.SetValue(8);
		WeaponsEdit.SetValue(8);		

		SelectedStructures = 8;
		SelectedMechanic = 8;
		SelectedWeapons = 8;
		return;
	}
	else if (PlayerLoad.GetText() ~= CustomString)
	{
		// reset
		ModelBG.Caption = CharacterList.List.Get();
		PlayerLoad.OnChange = InternalOnChange;

		// reset upgrade boxes
		StructuresEdit.SetValue(8);
		MechanicEdit.SetValue(8);
		WeaponsEdit.SetValue(8);

		SelectedStructures = 8;
		SelectedMechanic = 8;
		SelectedWeapons = 8;		
		return;
	}

	// retrieve a matching favorite and apply it to model mockup
	switch(SelectedTeam)
	{
		case 1:
			for (i=0;i<ArrayCount(SOFavorite);i++)
			{
				if (PlayerLoad.GetText() ~= SOFavorite[i].SaveName)
				{
					SelectedRecord = Clamp(SOFavorite[i].GroupType, 0, SORecord.Length-1);
					if (SOFavorite[i].Trait < Traits.Length)
						SelectedTrait = SOFavorite[i].Trait;
					else
						SelectedTrait = 255;
					HeadSkinCount = Clamp(SOFavorite[i].SoldierType, 0, SORecord[SelectedRecord].HeadSkin.Length-1);
					BodySkinCount = Clamp(SOFavorite[i].BodyType, 0, SORecord[SelectedRecord].BodySkin.Length-1);
					SelectedStructures = Clamp(SOFavorite[i].S_Structures, 0, MaxPoints);
					SelectedMechanic = Clamp(SOFavorite[i].S_Mechanic, 0, MaxPoints-SelectedStructures);
					SelectedWeapons = (MaxPoints-(SelectedStructures+SelectedMechanic));
					CharacterList.List.OnChange = NullChange;
					CharacterList.List.SetIndex(SelectedRecord);
					CharacterList.List.OnChange = ListClick;
					ModelBG.Caption = CharacterList.List.Get();
					bRenderDude = false;
					SetupTeamDude();
					break;
				}
			}
			break;

		default:
			for (i=0;i<ArrayCount(MercFavorite);i++)
			{
				if (PlayerLoad.GetText() ~= MercFavorite[i].SaveName)
				{
					SelectedRecord = Clamp(MercFavorite[i].GroupType, 0, MercRecord.Length-1);
					if (MercFavorite[i].Trait < Traits.Length)
						SelectedTrait = MercFavorite[i].Trait;
					else
						SelectedTrait = 255;
					HeadSkinCount = Clamp(MercFavorite[i].SoldierType, 0, MercRecord[SelectedRecord].HeadSkin.Length-1);
					BodySkinCount = Clamp(MercFavorite[i].BodyType, 0, MercRecord[SelectedRecord].BodySkin.Length-1);
					SelectedStructures = Clamp(MercFavorite[i].S_Structures, 0, MaxPoints);
					SelectedMechanic = Clamp(MercFavorite[i].S_Mechanic, 0, MaxPoints-SelectedStructures);
					SelectedWeapons = (MaxPoints-(SelectedStructures+SelectedMechanic));
					CharacterList.List.OnChange = NullChange;
					CharacterList.List.SetIndex(SelectedRecord);
					CharacterList.List.OnChange = ListClick;
					ModelBG.Caption = CharacterList.List.Get();
					bRenderDude = false;
					SetupTeamDude();
					break;
				}
			}
			break;
	}

	// reset upgrade boxes
	StructuresEdit.SetValue(SelectedStructures);
	MechanicEdit.SetValue(SelectedMechanic);
	WeaponsEdit.SetValue(SelectedWeapons);
	StructuresEdit.OnChange = SkillChange;
	MechanicEdit.OnChange = SkillChange;
	WeaponsEdit.OnChange = SkillChange;

	// set traitlist index
	TraitList.List.OnChange = NullChange;
	if (SelectedTrait == 255)
		TraitList.List.SetIndex(0);
	else
		TraitList.List.SetIndex(SelectedTrait+1);
	TraitScroll.SetContent(TraitList.List.GetExtra());
	TraitList.List.OnChange = TraitClick;

	// reset
	PlayerLoad.OnChange = InternalOnChange;
}

defaultproperties
{
	Begin Object class=moNumericEdit Name=SEdit
		WinWidth=0.23
		WinLeft=0.69
		WinTop=0.2
		MinValue=0
		MaxValue=24
		Caption="Structures"
		Hint="Your skill at building structures. The higher it's set, the faster you learn upgrades."
		LabelJustification=TXTA_Left
		ComponentJustification=TXTA_Right
		CaptionWidth=0.63
		ComponentWidth=-1
		bAutoSizeCaption=True
		OnChange=SkillChange
		TabOrder=5
	End Object
	StructuresEdit=SEdit

	Begin Object class=moNumericEdit Name=MEdit
		WinWidth=0.23
		WinLeft=0.69
		WinTop=0.248
		MinValue=0
		MaxValue=24
		Caption="Mechanic"
		Hint="Your skill at deploying vehicles. The higher it's set, the faster you learn upgrades."
		LabelJustification=TXTA_Left
		ComponentJustification=TXTA_Right
		CaptionWidth=0.63
		ComponentWidth=-1
		bAutoSizeCaption=True
		OnChange=SkillChange
		TabOrder=5
	End Object
	MechanicEdit=MEdit

	Begin Object class=moNumericEdit Name=WEdit
		WinWidth=0.23
		WinLeft=0.69
		WinTop=0.296
		MinValue=0
		MaxValue=24
		Caption="Weapons"
		Hint="Your skill at deploying heavy weapons. The higher it's set, the faster you learn upgrades."
		LabelJustification=TXTA_Left
		ComponentJustification=TXTA_Right
		CaptionWidth=0.63
		ComponentWidth=-1
		bAutoSizeCaption=True
		OnChange=SkillChange
		TabOrder=5
	End Object
	WeaponsEdit=WEdit

	MaxPoints=24
}