//=============================================================================
// Frag.Ops  2004 Matt 'SquirrelZero' Farber
//=============================================================================
// Frag.Ops War LevelInfo
// --
// Master settings for the War gametype.  All maps MUST have one, and only one.
// --
// The ObjectiveInfo sections of the objective manager are used to show the
// attacking and defending teams exactly what's going on with the objectives
// around them.  Neither is needed, but i HIGHLY recommend filling in the info
// for the attacking team at the least.
//=============================================================================
class War_LevelInfo extends Info
	placeable;

// objective manager
struct ObjectiveManager
{
	var() editinlineuse War_Objective Objective;
	var() editinlineuse array<War_Objective> LinkedObjectives;
	var() bool bMustCompleteAllLinked;
};

// map briefings
struct sMapBriefing
{
	var() string SpecOps_MapEntryBriefing;
	var() string Merc_MapEntryBriefing;
};

// objective lists
var(War_Objectives) array<ObjectiveManager> SpecOps_Objectives, Merc_Objectives;
var array<ObjectiveManager> S_Obj, M_Obj, M_CaptureObj, S_CaptureObj;

// brief variable
var() sMapBriefing MapEntryBriefings;

// if set, objectives become irrelevant and team must eliminate each other
var(War_Objectives) bool bTeamDeathMatch;
var(War_Objectives) string TDMMercBriefing, TDMSOBriefing;
var(War_Objectives) Material TDMBriefIcon;

// teams can win by eliminating each other if set, if bTeamDeathMatch is set
// this setting becomes irrelevant.  If this is false, teams will be REQUIRED
// to complete objectives even if the other team is dead.
var(War_Objectives) bool bMercsCanWinByElimination;
var(War_Objectives) bool bSOCanWinByElimination;

// desired round time to use for this map, can be overridden
var() int RoundTimeLimit;

// delay between respawns, in seconds
var() int MercWaveDelayTime;
var() int SOWaveDelayTime;

// for fast access
var int LastSObjLink, LastSCaptureLink, LastMObjLink, LastMCaptureLink;

// initialize all objectives
simulated function PostBeginPlay()
{
	local int i, j;
	local int Order, CapOrder;
	
	Super.PostBeginPlay();
	
	for (i=0;i<Merc_Objectives.Length;i++)
	{
		Merc_Objectives[i].Objective.AttackingTeam = 0;

		if (Merc_Objectives[i].Objective.bCanCompleteInAnyOrder)
		{
			M_CaptureObj[CapOrder] = Merc_Objectives[i];
			M_CaptureObj[Order].Objective.LinkNumber = CapOrder;
			CapOrder++;
			continue;
		}

		M_Obj[Order] = Merc_Objectives[i];
		M_Obj[Order].Objective.LinkNumber = Order;
		if (M_Obj[Order].LinkedObjectives.Length > 0)
		{
			for (j=0;j<M_Obj[Order].LinkedObjectives.Length;j++)
			{
				M_Obj[Order].LinkedObjectives[j].LinkNumber = Order;
				M_Obj[Order].LinkedObjectives[j].AttackingTeam = 0;
			}
		}
		Order++;
	}

	Order = 0;
	CapOrder = 0;

	for (i=0;i<SpecOps_Objectives.Length;i++)
	{
		SpecOps_Objectives[i].Objective.AttackingTeam = 1;
		if (SpecOps_Objectives[i].Objective.bCanCompleteInAnyOrder)
		{
			S_CaptureObj[CapOrder] = SpecOps_Objectives[i];
			S_CaptureObj[Order].Objective.LinkNumber = CapOrder;
			CapOrder++;
			continue;
		}
		
		S_Obj[Order] = SpecOps_Objectives[i];
		S_Obj[Order].Objective.LinkNumber = Order;

		if (S_Obj[Order].LinkedObjectives.Length > 0)
		{
			for (j=0;j<S_Obj[Order].LinkedObjectives.Length;j++)
			{
				S_Obj[Order].LinkedObjectives[j].LinkNumber = Order;
				S_Obj[Order].LinkedObjectives[j].AttackingTeam = 1;
			}
		}
		Order++;
	}
}

// returns true if desired link is completed
simulated function bool LinkComplete(byte ObjTeam, byte LinkNumber)
{
	local bool bGotCompleteLink, bAllLinksComplete;
	local int i;

	bAllLinksComplete = true;

	Switch(ObjTeam)
	{
		case 0:
			if (M_Obj[LinkNumber].LinkedObjectives.Length > 0)
			{
				for (i=0;i<M_Obj[LinkNumber].LinkedObjectives.Length;i++)
				{
					if (M_Obj[LinkNumber].LinkedObjectives[i].bCompleted || M_Obj[LinkNumber].LinkedObjectives[i].bCompletionOptional)
						bGotCompleteLink = true;
					else
						bAllLinksComplete = false;
				}
			}
			if (((M_Obj[LinkNumber].Objective.bCompleted || M_Obj[LinkNumber].Objective.bCompletionOptional || bGotCompleteLink) && !M_Obj[LinkNumber].bMustCompleteAllLinked) || (M_Obj[LinkNumber].Objective.bCompleted && bAllLinksComplete))
				return true;
			break;

		default:
			if (S_Obj[LinkNumber].LinkedObjectives.Length > 0)
			{
				for (i=0;i<S_Obj[LinkNumber].LinkedObjectives.Length;i++)
				{
					if (S_Obj[LinkNumber].LinkedObjectives[i].bCompleted || M_Obj[LinkNumber].LinkedObjectives[i].bCompletionOptional)
						bGotCompleteLink = true;
					else
						bAllLinksComplete = false;
				}
			}
			if (((S_Obj[LinkNumber].Objective.bCompleted || S_Obj[LinkNumber].Objective.bCompletionOptional || bGotCompleteLink) && !S_Obj[LinkNumber].bMustCompleteAllLinked) || (S_Obj[LinkNumber].Objective.bCompleted && bAllLinksComplete))
				return true;
			break;
	}
	return false;
}

// check for a complete captured link
simulated function bool CaptureLinkComplete(byte ObjTeam, byte LinkNumber)
{
	local bool bGotCompleteLink, bAllLinksComplete;
	local int i;

	bAllLinksComplete = true;

	Switch(ObjTeam)
	{
		case 0:
			if (M_CaptureObj[LinkNumber].LinkedObjectives.Length > 0)
			{
				for (i=0;i<M_CaptureObj[LinkNumber].LinkedObjectives.Length;i++)
				{
					if (M_CaptureObj[LinkNumber].LinkedObjectives[i].bCompleted || M_CaptureObj[LinkNumber].LinkedObjectives[i].bCompletionOptional)
						bGotCompleteLink = true;
					else
						bAllLinksComplete = false;
				}
			}
			if (((M_CaptureObj[LinkNumber].Objective.bCompleted || M_CaptureObj[LinkNumber].Objective.bCompletionOptional || bGotCompleteLink) && !M_CaptureObj[LinkNumber].bMustCompleteAllLinked) || (M_CaptureObj[LinkNumber].Objective.bCompleted && bAllLinksComplete))
				return true;
			break;

		default:
			if (S_CaptureObj[LinkNumber].LinkedObjectives.Length > 0)
			{
				for (i=0;i<S_CaptureObj[LinkNumber].LinkedObjectives.Length;i++)
				{
					if (S_CaptureObj[LinkNumber].LinkedObjectives[i].bCompleted || S_CaptureObj[LinkNumber].LinkedObjectives[i].bCompletionOptional)
						bGotCompleteLink = true;
					else
						bAllLinksComplete = false;
				}
			}
			if (((S_CaptureObj[LinkNumber].Objective.bCompleted || S_CaptureObj[LinkNumber].Objective.bCompletionOptional || bGotCompleteLink) && !S_CaptureObj[LinkNumber].bMustCompleteAllLinked) || (S_CaptureObj[LinkNumber].Objective.bCompleted && bAllLinksComplete))
				return true;
			break;
	}
	return false;
}

function Reset()
{
	LastSObjLink = -1;
	LastSCaptureLink = -1;
	LastMObjLink = -1;
	LastMCaptureLink = -1;
}

defaultproperties
{
	Texture=S_ZoneInfo
	bStatic=false
	bNoDelete=true
	RemoteRole=ROLE_None
	TDMBriefIcon=Material'FOWarTextures.HUD.TDMBriefIcon'
	TDMSOBriefing="Eliminate as many Mercenaries as you can."
	TDMMercBriefing="Eliminate as many Special Ops as you can."
	bReplicateMovement=false
	bSkipActorPropertyReplication=true

	// defaults
	RoundTimeLimit=8
	MercWaveDelayTime=45
	SOWaveDelayTime=45
}