//=============================================================================
// Frag.Ops  2004 Matt 'SquirrelZero' Farber
//=============================================================================
class War_Objective_Interactive extends War_Objective
	placeable;

// see: Obj_InteractObjective.uc
var() float TriggerTime;

// see: Obj_InteractObjective.uc
var() sound TriggerSound;

// see: Obj_InteractObjective.uc
var() sound IntervalSound;

// see: Obj_InteractObjective.uc
var() sound FailedSound;

// works just like StartedBriefing in War_LevelInfo, used when objective is first
// triggered
var() string Attack_TriggeredMessage;
var() string Defend_TriggeredMessage;

// works just like StartedBriefing in War_LevelInfo, used when objective is
// interrupted
var() string Attack_InterruptedMessage;
var() string Defend_InterruptedMessage;

// event to trigger when starting
var() name TriggeredEvent;

// the initial state for this objective.  Basically the "idle" state
state IdleObjective
{
	function Cancel()
	{}

	function BeginState()
	{
		SetCollision(true,false,false);
	}
}

function bool OpenObjective(Pawn Other)
{
	if (!Super.OpenObjective(Other))
		return false;

	if (TriggerTime > 0.0)
	{
		if (TriggerSound != None)
			PlaySound(TriggerSound,,TransientSoundVolume*1.3);
		FOPawn(Other).StartWarObjective(self, TriggerTime);
		GotoState('PendingCompletion');
	}
	else
		Completed(Other);
}

state PendingCompletion
{
	function bool OpenObjective(Pawn Other)
	{
		return false;
	}

	function BeginState()
	{
		// disable for others
		SetCollision(false,false,false);
		PrintObjectiveSplash(Attack_TriggeredMessage, Defend_TriggeredMessage);
	}
}

function Cancel()
{
	if (FailedSound != None)
		PlaySound(FailedSound,,TransientSoundVolume*1.25);
	PrintObjectiveSplash(Attack_InterruptedMessage, Defend_InterruptedMessage);
	Reset();
}

event UsedBy(Pawn Other)
{
	OpenObjective(Other);
}

simulated function PlayIntervalSound()
{
	if (IntervalSound != None)
		PlaySound(IntervalSound,,TransientSoundVolume*1.3);
}

defaultproperties
{
	TriggerTime=10.0
	FailedSound=Sound'FOWeaponSounds.Objectives.PanelFailed'
	TriggerSound=Sound'FOWeaponSounds.Objectives.PanelStart'
	IntervalSound=Sound'FOWeaponSounds.Objectives.NiceBeep'
	CompletedSound=Sound'FOWeaponSounds.Objectives.PanelComplete'
	RemoteRole=ROLE_SimulatedProxy
	bCollideActors=true
}