//=============================================================================
// Frag.Ops  2004 Matt 'SquirrelZero' Farber
//=============================================================================
// Just like the Mission one.
//=============================================================================
class War_Objective_Retrieval extends War_Objective
	placeable;

// message to broadcast when a player picks up the objective
var() string Attack_PickupMessage;
var() string Defend_PickupMessage;

// message to broadcast when a player drops the objective
var() string Attack_DroppedMessage;
var() string Defend_DroppedMessage;

// picture used when picking up
var() Material Attack_PickupPicture;
var() Material Defend_PickupPicture;

// only announce pickup once
var() bool bAnnouncePickupOnce;

// sound to play upon pickup
var() sound PickupSound;

// so you can see the objective on the person
var() vector CarryOffset;
var() rotator CarryRotation;

// time before reset after dropping
var() int ResetSeconds;

var vector SpawnLocation;
var rotator SpawnRotation;
var bool bInitialBroadcast;
var Pawn Carrier;

function PostBeginPlay()
{
	Super.PostBeginPlay();
	bInitialBroadcast = false;
	SpawnLocation = Location;
	SpawnRotation = Rotation;
}

event Touch(Actor Other)
{
	if (Carrier != None || !Super.OpenObjective(Pawn(Other)))
		return;

	PickupObj(Pawn(Other));
}

// this objective was completed, so player should no longer be carrying it
function CarryCompleted(Pawn Other, vector CompleteLoc, rotator CompleteRot)
{
	SetOwner(None);
	bOnlyDrawIfAttached = false;
	Completed(Other);
	if (Carrier != None)
	{
		if (FOPawn(Carrier).War_CarriedObjective == self)
			FragOps_PRI(Carrier.PlayerReplicationInfo).bHasObjective = false;
		Carrier.DetachFromBone(self);
		UnTouch(Carrier);
		SetPhysics(PHYS_None);
		FOPawn(Carrier).War_CarriedObjective = None;
		Carrier = None;
	}
	SetLocation(CompleteLoc);
	SetRotation(CompleteRot);
}

// code for picking object up
function PickupObj(Pawn Other)
{
	Other.PlaySound(PickupSound,,TransientSoundVolume*1.3);
	bOnlyDrawIfAttached = true;
	Carrier = Other;
	SetTimer(0,false);
	SetPhysics(PHYS_None);
	SetCollision(true,false,false);
	bCollideWorld = false;
	Other.AttachToBone(self, 'Bip01 Spine');
	SetRelativeLocation(CarryOffset);
	SetRelativeRotation(CarryRotation);
	FOPawn(Other).War_CarriedObjective = self;
	FOPawn(Other).NetUpdateTime = Level.TimeSeconds - 1;
	NetUpdateTime = Level.TimeSeconds - 1;
	SetOwner(Other.Controller);
	if (Other.PlayerReplicationInfo != None)
		FragOps_PRI(Other.PlayerReplicationInfo).bHasObjective = true;

	if (!bInitialBroadcast || !bAnnouncePickupOnce)
	{
		PrintPickupSplash(Attack_PickupMessage, Defend_PickupMessage);
		bInitialBroadcast = true;
	}
}

// broadcast pickup message
function PrintPickupSplash(string AttackSplashString, string DefendSplashString)
{
	local Controller C;

	for (C=Level.ControllerList;C!=None;C=C.NextController)
	{
		if (FOBot(C) != None || C.Pawn == None)
			continue;

		if (C.PlayerReplicationInfo.Team.TeamIndex == AttackingTeam && AttackSplashString != "")
			War_FOPlayer(C).ClientSplashObjective(AttackSplashString, Attack_PickupPicture);
		else if (C.PlayerReplicationInfo.Team.TeamIndex != AttackingTeam && DefendSplashString != "")
			War_FOPlayer(C).ClientSplashObjective(DefendSplashString, Defend_PickupPicture);
	}
}

function Timer()
{
	if (Carrier != None)
		return;
	
	bCollideWorld = false;
	SetLocation(SpawnLocation);
	SetRotation(SpawnRotation);
	SetPhysics(PHYS_None);
	SetCollision(true,false,false);
	bInitialBroadcast = false;
	SetOwner(None);
}

// for falling objective
function HitWall(vector HitNormal, Actor Wall)
{
	local float Speed;
	
	SetPhysics(PHYS_Falling);
	Velocity = 0.4 * (Velocity - 2.0*HitNormal*(Velocity dot HitNormal));
	Speed = VSize(Velocity);

	if (Speed < 30)
		SetPhysics(PHYS_None);
}

function Reset()
{
	bCollideWorld = false;
	SetLocation(SpawnLocation);
	SetRotation(SpawnRotation);
	SetPhysics(PHYS_None);
	SetCollision(true,false,false);
	bInitialBroadcast = false;
	bOnlyDrawIfAttached = false;
	if (Carrier != None)
	{
		UnTouch(Carrier);
		Carrier.DetachFromBone(self);
	}
	else
		SetTimer(0,false);
	Carrier = None;
	SetOwner(None);
	Super.Reset();
}

defaultproperties
{
	ResetSeconds=75
	Physics=PHYS_None
	DrawType=DT_StaticMesh
	StaticMesh=StaticMesh'dessertstrike2.paper2'
	bHidden=false
	bWorldGeometry=false
	bCollideWorld=false
	bCollideActors=true
	bBlockPlayers=false
	bBlockActors=false
	bCollideWhenPlacing=false
	bAcceptsProjectors=true
	bAlwaysRelevant=true
	bHardAttach=true
	bBounce=true
	bReplicateMovement=true
	bUpdateSimulatedPosition=false
	NetPriority=2.0
	RemoteRole=ROLE_DumbProxy
	bUseCylinderCollision=false
	CollisionRadius=+0001.000000
	CollisionHeight=+0001.000000
	AttachmentBone="Bip 01 Spine"
	bOwnerNoSee=true
}