//=============================================================================
// Frag.Ops  2004 Matt 'SquirrelZero' Farber
//=============================================================================
class War_PlacedItem extends Projectile;

// quick access
var War_BuildZone BuildZone;
var War_FOPlayer FP;
var int BuildNum;

// placement location/rotation offset, adjusted by the mousewheel
var float PlaceOffset, RotationOffset;

// player cannot build if false
var bool bGotBZ;
var bool bCanBuild;

// number of "bad" touching actors
var int BadTouch;

// should share collision size or.. ?
var bool bShareCollisionSize;

// this is the most ugly, disgusting thing i ever had to write.
// Why oh why can i not get per-poly collision to work here?  Instead
// i have to spawn these "checker" actors and place them at strategic
// locations around the inside of the actor.  Totally insane.
var array<War_ColChecker> Checkers;
var array<vector> CheckerOffsets;
var float ZOffset;

function Tick(float dt)
{
	UpdateCheckers();
	UpdateLocation(dt);
}

// update collision checkers
function UpdateCheckers()
{
	local int i;
	local vector X, Y, Z;

	GetAxes(Rotation,X,Y,Z);

	for (i=0;i<CheckerOffsets.Length;i++)
	{
		if (i == Checkers.Length || Checkers[i] == None)
		{
			Checkers[i] = spawn(class'War_ColChecker',,,Location,Rotation);
			Checkers[i].SetLocation(Location + X*CheckerOffsets[i].X + Y*CheckerOffsets[i].Y + Z*CheckerOffsets[i].Z);
			Checkers[i].SetBase(self);
			if (bShareCollisionSize)
				Checkers[i].SetCollisionSize(CollisionRadius,CollisionHeight);
			Checkers[i].PlacingItem = self;
		}
	}
}

// update location and rotation of object
function UpdateLocation(optional float dt)
{
	local vector PlacedLocation, X, Y, Z;
	local rotator PlacedRotation;
	local bool bBadCheck;
	local int i;

	// destroy if anything bad occurred while placing
	if (BuildZone.ResourceCount[BuildNum] <= 0 || (BuildZone.LastUseTime[BuildNum] != 1 && Level.TimeSeconds - BuildZone.LastUseTime[BuildNum] < BuildZone.BuildableItems[BuildNum].ItemType.Default.UseDelayTime))
	{
		FP.ServerRequestItem();
		Disable('Tick');
		Destroy();
	}

	// set basic rotation relative to the player
	PlacedRotation = FP.Pawn.Rotation;
	PlacedRotation.Pitch = 0;
	PlacedRotation.Roll = 0;

	// update location
	GetAxes(PlacedRotation,X,Y,Z);
	PlacedLocation = FP.Pawn.Location;
	PlacedLocation += X*PlaceOffset + X*(CollisionRadius+FP.Pawn.CollisionRadius);
	PlacedLocation.Z = (BuildZone.Location.Z - BuildZone.CollisionHeight) + ZOffset;
	SetLocation(PlacedLocation);

	// reset to normal colors, enable build
	GoodBuild();

	// extra check
	for (i=0;i<Checkers.Length;i++)
	{
		if (Checkers[i] == None)
			continue;

		if (Checkers[i].BadTouch > 0 || !Checkers[i].bGotBZ)
		{
			bBadCheck = true;
			break;
		}
	}

	// bad build if no buyzone
	if (!bGotBZ || BadTouch > 0 || bBadCheck)
		BadBuild();

	// update rotation
	PlacedRotation.Yaw += RotationOffset;
	SetRotation(PlacedRotation);
}

function GoodBuild()
{
	local ConstantColor CC;
	
	CC = ConstantColor(Combiner(FinalBlend(Default.Skins[0]).Material).Material2);
	CC.Color.R = 0;
	CC.Color.G = 255;
	CC.Color.B = 0;
	bCanBuild = true;
}

function BadBuild()
{
	local ConstantColor CC;
	
	CC = ConstantColor(Combiner(FinalBlend(Default.Skins[0]).Material).Material2);
	CC.Color.R = 255;
	CC.Color.G = 0;
	CC.Color.B = 0;
	bCanBuild = false;
}

function Touch(Actor Other)
{
	if (War_BuildZone(Other) != None)
		bGotBZ = true;
	Super.Touch(Other);
}

function ProcessTouch(actor Other, vector HitLocation)
{
	if (War_BuildZone(Other) == None && War_ColChecker(Other) == None)
		BadTouch++;
}

event UnTouch(Actor Other)
{
	Super.UnTouch(Other);
	if (War_BuildZone(Other) != None)
		bGotBZ = false;
	else if (War_ColChecker(Other) == None)
		BadTouch = Max(BadTouch - 1, 0);
}

// kill the checkers
function Destroyed()
{
	local int i;
	
	Super.Destroyed();

	for (i=0;i<Checkers.Length;i++)
	{
		if (Checkers[i] != None)
			Checkers[i].Destroy();
	}
}

defaultproperties
{
	// collision
	bShareCollisionSize=true
	bCollideActors=true
	bCollideWorld=false
	bBlockNonZeroExtentTraces=true
	bBlockKarma=false 
	bUseCylinderCollision=true
	bUseCollisionStaticMesh=true

	// settings
	Physics=PHYS_None
	RemoteRole=ROLE_None
	bNoDelete=false
	bStatic=false
	bUseDynamicLights=false
	bAcceptsProjectors=false
	bUnlit=true
	LifeSpan=0
}