//=============================================================================
// Frag.Ops  2004 Matt 'SquirrelZero' Farber
//=============================================================================
class War_Stryker_ConcGrenade extends Grenade;

// water splashes
var sound SplashSound;
var float SplashThreshold;

// explosion sound
var sound ExplosionSound;

simulated function HurtRadius( float DamageAmount, float DamageRadius, class<DamageType> DamageType, float Momentum, vector HitLocation )
{
	local actor Victims;
	local float damageScale, dist;
	local vector dir;

	if (bHurtEntry)
		return;

	bHurtEntry = true;
	foreach VisibleCollidingActors( class 'Actor', Victims, DamageRadius, HitLocation )
	{
		// don't let blast damage affect fluid - VisibleCollisingActors doesn't really work for them - jag
		if( (Victims != self) && (Hurtwall != Victims) && (Victims.Role == ROLE_Authority) && !Victims.IsA('FluidSurfaceInfo') )
		{
			dir = Victims.Location - HitLocation;
			dist = FMax(1,VSize(dir));
			dir = dir/dist;
			damageScale = 1 - FMax(0,(dist - Victims.CollisionRadius)/DamageRadius);
			if (Instigator == None || Instigator.Controller == None)
				Victims.SetDelayedDamageInstigatorController( InstigatorController);
			if (Victims == LastTouched)
				LastTouched = None;
			Victims.TakeDamage
			(
				damageScale * DamageAmount,
				Instigator,
				Victims.Location - 0.5 * (Victims.CollisionHeight + Victims.CollisionRadius) * dir,
				(damageScale * Momentum * dir),
				DamageType
			);
			if (Vehicle(Victims) != None && Vehicle(Victims).Health > 0)
				Vehicle(Victims).DriverRadiusDamage(DamageAmount, DamageRadius, InstigatorController, DamageType, Momentum, HitLocation);

		}
	}
	if ( (LastTouched != None) && (LastTouched != self) && (LastTouched.Role == ROLE_Authority) && !LastTouched.IsA('FluidSurfaceInfo') )
	{
		Victims = LastTouched;
		LastTouched = None;
		dir = Victims.Location - HitLocation;
		dist = FMax(1,VSize(dir));
		dir = dir/dist;
		damageScale = FMax(Victims.CollisionRadius/(Victims.CollisionRadius + Victims.CollisionHeight),1 - FMax(0,(dist - Victims.CollisionRadius)/DamageRadius));
		if (Instigator == None || Instigator.Controller == None)
			Victims.SetDelayedDamageInstigatorController(InstigatorController);
		Victims.TakeDamage
		(
			damageScale * DamageAmount,
			Instigator,
			Victims.Location - 0.5 * (Victims.CollisionHeight + Victims.CollisionRadius) * dir,
			(damageScale * Momentum * dir),
			DamageType
		);
		if (Vehicle(Victims) != None && Vehicle(Victims).Health > 0)
			Vehicle(Victims).DriverRadiusDamage(DamageAmount, DamageRadius, InstigatorController, DamageType, Momentum, HitLocation);
	}

	bHurtEntry = false;
}

simulated singular function HitWall(vector HitNormal, actor Wall)
{
	local PlayerController PC;

	if (Role == ROLE_Authority)
	{
		if (!Wall.bStatic || Wall.bCanBeDamaged)
		{
			if (Instigator == None || Instigator.Controller == None)
				Wall.SetDelayedDamageInstigatorController( InstigatorController );
			Wall.TakeDamage( Damage, instigator, Location, MomentumTransfer * Normal(Velocity), MyDamageType);
			if (DamageRadius > 0 && Vehicle(Wall) != None && Vehicle(Wall).Health > 0)
				Vehicle(Wall).DriverRadiusDamage(Damage, DamageRadius, InstigatorController, MyDamageType, MomentumTransfer, Location);
			HurtWall = Wall;
		}
		MakeNoise(1.0);
	}
	Explode(Location + ExploWallOut * HitNormal, HitNormal);
	if ((ExplosionDecal != None) && (Level.NetMode != NM_DedicatedServer)  )
	{
		if (ExplosionDecal.Default.CullDistance != 0)
		{
			PC = Level.GetLocalPlayerController();
			if ( !PC.BeyondViewDistance(Location, ExplosionDecal.Default.CullDistance) )
				Spawn(ExplosionDecal,self,,Location, rotator(-HitNormal));
			else if ( (Instigator != None) && (PC == Instigator.Controller) && !PC.BeyondViewDistance(Location, 2*ExplosionDecal.Default.CullDistance) )
				Spawn(ExplosionDecal,self,,Location, rotator(-HitNormal));
		}
		else
			Spawn(ExplosionDecal,self,,Location, rotator(-HitNormal));
	}
	HurtWall = None;
}

simulated function PostBeginPlay()
{
	local PlayerController PC;

	Super(Projectile).PostBeginPlay();

	if (Level.NetMode != NM_DedicatedServer)
	{
		PC = Level.GetLocalPlayerController();
		if ((PC.ViewTarget != None) && VSize(PC.ViewTarget.Location - Location) < 5500)
			Trail = Spawn(class'Effect_GrenadeSmokeTrailLight', self,, Location, Rotation);
	}

	if (Role == ROLE_Authority)
	{
		Velocity = Speed * Vector(Rotation);
		Velocity.Z += 230;
		RandSpin(25000);
		bCanHitOwner = false;
		if (Instigator.HeadVolume.bWaterVolume)
		{
			bHitWater = true;
			Velocity = 0.6*Velocity;
		}
	}
}

simulated function Landed( vector HitNormal )
{
	Explode(Location, HitNormal);
}

simulated function Explode(vector HitLocation, vector HitNormal)
{
	BlowUp(HitLocation);
	PlaySound(ExplosionSound,,2.3*TransientSoundVolume,,,0.95);
	if (EffectIsRelevant(Location,false))
	{
		Spawn(class'War_Effect_ConcNadeExplosion',,, HitLocation, rotator(vect(0,0,1)));
			Spawn(ExplosionDecal,self,,HitLocation, rotator(-HitNormal));
	}
	if (Level.NetMode != NM_DedicatedServer)
		ShakeView();
	Destroy();
}

// slow grenade on impact, play splash sound effects
simulated event PhysicsVolumeChange(PhysicsVolume NewVolume)
{
	local float SplashVolume;

	Super.PhysicsVolumeChange(NewVolume);
	
	if (!NewVolume.bWaterVolume)
		return;

	if (!bHitWater && Role == ROLE_Authority)
	{
		Velocity *= 0.4;
		Velocity.Z += 60;
		bHitWater = true;
	}
	
	if (Level.NetMode == NM_DedicatedServer)
		return;

	Speed = VSize(Velocity);
	
	SplashVolume = FClamp(Speed/SplashThreshold, 0, 0.77);
	PlaySound(SplashSound, Slot_NONE, SplashVolume,, 500, 0.96+(FRand()*0.08));
}

simulated function ShakeView()
{
	local PlayerController PC;
	local float Dist, Scale;	

	PC = Level.GetLocalPlayerController();
	if (PC != None && PC.ViewTarget != None)
	{
		Dist = VSize(Location - PC.ViewTarget.Location);
		if (Dist < DamageRadius * 2.0)
		{
			if (Dist < DamageRadius)
				Scale = 1.0;
			else
				Scale = (DamageRadius*2.0 - Dist) / (DamageRadius);
			FOPlayer(PC).LocalShake(vect(350,350,350)*Scale,vect(9000,9000,9000),3.5, vect(10,10,10)*Scale,vect(2000,2000,2000),3.5);
		}
	}	
}

defaultproperties
{
	// splashes
	StaticMesh=StaticMesh'FOSMs.LAWSms.LAWRocket'
	bUseCollisionStaticMesh=true
	MyDamageType=class'DamType_HEGrenade'
	DrawScale=1.0
	SplashSound=sound'FOGameSounds.AmbientHit.Kerplunk'
	SplashThreshold=700
	Speed=950
	MaxSpeed=1450	

	ExplosionSound=sound'ONSVehicleSounds-S.Explosions.Explosion01'

	MomentumTransfer=1200.0
	DamageRadius=115.0
	Damage=300
}