//=============================================================================
// Frag.Ops  2004 Matt 'SquirrelZero' Farber
//=============================================================================
// Comanche gunner
//=============================================================================
class War_Vehicle_ComancheGunner extends War_FOVehiclePawn;

// enter
function KDriverEnter(Pawn P)
{
	Super.KDriverEnter(P);

	// save player replicationinfo
	P.PlayerReplicationInfo = Controller.PlayerReplicationInfo;
}

event bool KDriverLeave(bool bForceLeave)
{
	local Pawn OldDriver;

	if (VehicleBase != None && War_FOVehicleAir(VehicleBase).bEngineStarted && !bForceLeave)
		return false;

	OldDriver = Driver;
	if (Super.KDriverLeave(bForceLeave))
	{
		if (Gun != None)
			Gun.GotoState('InstantFireMode');
		return true;
	}
	return false;
}

function AltFire(optional float F)
{
	local PlayerController PC;

	PC = PlayerController(Controller);
	if (PC == None)
		return;

	bWeaponIsAltFiring = true;
	PC.ToggleZoom();
}

function ClientVehicleCeaseFire(bool bWasAltFire)
{
	local PlayerController PC;

	if (!bWasAltFire)
	{
		Super.ClientVehicleCeaseFire(bWasAltFire);
		return;
	}

	PC = PlayerController(Controller);
	if (PC == None)
		return;

	bWeaponIsAltFiring = false;
	PC.StopZoom();
}

simulated function ClientKDriverLeave(PlayerController PC)
{
	Super.ClientKDriverLeave(PC);
	if (Gun != None)
		Gun.GotoState('InstantFireMode');

	bWeaponIsAltFiring = false;
	PC.EndZoom();
}

simulated function Tick(float dt)
{
	Super(ONSWeaponPawn).Tick(dt);

	if (Driver == None || !FOPawn(Driver).bDriving || Driver.Health <= 0 || Driver.IsInState('Dying') || VehicleBase == None)
		return;

	Driver.bDoTorsoTwist = false;
	Driver.bRotateToDesired = false;

	Driver.SetPhysics(PHYS_None);
	Driver.SetLocation(VehicleBase.Location + (DrivePos >> VehicleBase.Rotation));
	Driver.SetRotation(VehicleBase.Rotation);
	Driver.NetUpdateTime = Level.TimeSeconds - 1;
}

simulated function SpecialCalcFirstPersonView(PlayerController PC, out actor ViewActor, out vector CameraLocation, out rotator CameraRotation )
{
	local vector x, y, z;
	local vector VehicleZ, CamViewOffsetWorld, HitLocation, HitNormal, Start;
	local float CamViewOffsetZAmount;

	GetAxes(CameraRotation, x, y, z);
	ViewActor = self;

	CamViewOffsetWorld = FPCamViewOffset >> CameraRotation;
	CameraLocation = GetCameraLocationStart() + (FPCamPos >> Rotation) + CamViewOffsetWorld;

	if (bFPNoZFromCameraPitch)
	{
		VehicleZ = vect(0,0,1) >> Rotation;
		CamViewOffsetZAmount = CamViewOffsetWorld Dot VehicleZ;
		CameraLocation -= CamViewOffsetZAmount * VehicleZ;
	}

	// adjust camera so not through ground
	Start = GetCameraLocationStart();
	Trace(HitLocation, HitNormal, CameraLocation, Start, false, vect(20,20,20));
	if (HitLocation != vect(0,0,0))
		CameraLocation = HitLocation;
	CameraRotation = Normalize(CameraRotation + PC.ShakeRot);
	CameraLocation = CameraLocation + PC.ShakeOffset.X * x + PC.ShakeOffset.Y * y + PC.ShakeOffset.Z * z;
}

simulated function SpecialCalcBehindView(PlayerController PC, out actor ViewActor, out vector CameraLocation, out rotator CameraRotation )
{
	local vector CamLookAt, HitLocation, HitNormal, OffsetVector;
	local vector x, y, z;

	if (DesiredTPCamDistance < TPCamDistance)
		TPCamDistance = FMax(DesiredTPCamDistance, TPCamDistance - CameraSpeed * (Level.TimeSeconds - LastCameraCalcTime));
	else if (DesiredTPCamDistance > TPCamDistance)
		TPCamDistance = FMin(DesiredTPCamDistance, TPCamDistance + CameraSpeed * (Level.TimeSeconds - LastCameraCalcTime));

	GetAxes(PC.Rotation, x, y, z);
	ViewActor = self;
	CamLookAt = GetCameraLocationStart() + (TPCamLookat >> Rotation) + TPCamWorldOffset;

	OffsetVector = vect(0, 0, 0);
	OffsetVector.X = -1.0 * TPCamDistance;

	CameraLocation = CamLookAt + (OffsetVector >> PC.Rotation);

	Trace(HitLocation, HitNormal, CameraLocation, GetCameraLocationStart(), false, vect(20,20,20));
	if (HitLocation != vect(0,0,0))
		CameraLocation = HitLocation;

	CameraRotation = Normalize(PC.Rotation + PC.ShakeRot);
	CameraLocation = CameraLocation + PC.ShakeOffset.X * x + PC.ShakeOffset.Y * y + PC.ShakeOffset.Z * z;
}

defaultproperties
{
	VehiclePositionString="manning an RAH-66 Minigun"
	VehicleNameString="RAH-66 Copilot"

	// anims
	DriveAnim=driveg

	// entry
	EntryPosition=(X=50,Y=0,Z=0)
	EntryRadius=210.0

	// exit
	ExitPositions(0)=(X=0,Y=-195,Z=10)
	ExitPositions(1)=(X=0,Y=195,Z=10)
	ExitPositions(2)=(X=-30,Y=-250,Z=25)
	ExitPositions(3)=(X=-30,Y=250,Z=25)
	ExitPositions(4)=(X=250,Y=0,Z=20)

	// driver
	DrivePos=(X=35.0,Y=0.0,Z=-30.5)

	// weaponry
	GunClass=class'FragOps.War_Vehicle_ComancheMinigun'

	// camera
	CameraBone=Body
	FPCamPos=(X=32,Y=0,Z=-38)
	FPCamViewOffset=(X=0,Y=0,Z=0)
	TPCamLookat=(X=10,Y=0,Z=-55)
	TPCamWorldOffset=(X=0,Y=0,Z=0)
	TPCamDistance=50
	TPCamDistRange=(Min=30,Max=300)
	
	// misc
	bFPNoZFromCameraPitch=false
	bZeroPCRotOnEntry=false
	bDrawDriverInTP=true
	bDrawMeshInFP=true
	DriverDamageMult=0.0
	bHasAltFire=talse
	bDriverCollideActors=false
	bDesiredBehindview=false
}