//=============================================================================
// Frag.Ops  2004 Matt 'SquirrelZero' Farber
//=============================================================================
// Jeep gunner, not much different than the tank gunner.
//=============================================================================
class War_Vehicle_JeepGunner extends War_FOVehiclePawn;

var rotator LastRotation;

// enter
function KDriverEnter(Pawn P)
{
	Super.KDriverEnter(P);

	// save player replicationinfo
	P.PlayerReplicationInfo = Controller.PlayerReplicationInfo;
}

event bool KDriverLeave(bool bForceLeave)
{
	local Pawn OldDriver;
	
	OldDriver = Driver;
	if (Super.KDriverLeave(bForceLeave))
	{
		if (OldDriver != None && OldDriver.Health > 0)
			OldDriver.SetBoneRotation('Bip01 Spine', rot(0,0,0));
		if (Gun != None)
			Gun.GotoState('InstantFireMode');
		return true;
	}
	return false;
}

function AltFire(optional float F)
{
	local PlayerController PC;

	PC = PlayerController(Controller);
	if (PC == None)
		return;

	bWeaponIsAltFiring = true;
	PC.ToggleZoom();
}

function ClientVehicleCeaseFire(bool bWasAltFire)
{
	local PlayerController PC;

	if (!bWasAltFire)
	{
		Super.ClientVehicleCeaseFire(bWasAltFire);
		return;
	}

	PC = PlayerController(Controller);
	if (PC == None)
		return;

	bWeaponIsAltFiring = false;
	PC.StopZoom();
}

simulated function ClientKDriverLeave(PlayerController PC)
{
	Super.ClientKDriverLeave(PC);
	if (Gun != None)
		Gun.GotoState('InstantFireMode');

	bWeaponIsAltFiring = false;
	PC.EndZoom();
}

simulated function Tick(float dt)
{
	local rotator NewRotation;	
	local rotator SpineRotation, TestRotation, ControllerRotation;	

	Super(ONSWeaponPawn).Tick(dt);

	if (VehicleBase == None || VehicleBase.Health <= 0 || VehicleBase.DrawType == DT_StaticMesh)
		return;

	if (Driver != None && Gun != None && Driver.Health > 0 && !Driver.IsInState('Dying'))
	{
		Driver.SetBase(None);
		Driver.bHardAttach = true;
		
		// update relative location
		Driver.SetBase(Gun);
		NewRotation = Gun.GetBoneRotation(Gun.YawBone);
		Driver.SetLocation(Gun.GetBoneCoords(Gun.YawBone).Origin-vector(NewRotation)*DrivePos.X + DrivePos.Z*vect(0,0,1));
		//NewRotation.Roll = rotator(vector(VehicleBase.Rotation) - vector(NewRotation)).Roll;
		Driver.SetRotation(rotator(-vector(NewRotation)));
		Driver.SetPhysics(PHYS_None);
		LastRotation = NewRotation;

		if (Role == ROLE_Authority)
			Driver.NetUpdateTime = Level.TimeSeconds - 1;

		if (Level.NetMode != NM_DedicatedServer)
		{
			//Driver.bDoTorsoTwist = false;
			//Driver.bRotateToDesired = false;

			// adjust driver rotation
			ControllerRotation.Pitch = Gun.GetBoneRotation(Gun.PitchBone).Pitch & 65535;
			ControllerRotation.Roll = 0;
			TestRotation.Pitch = Driver.Rotation.Pitch & 65535;
			SpineRotation.Yaw = fClamp(float((Normalize(ControllerRotation - TestRotation).Pitch)) * 1.2, -4000.0, 4000.0);		
			Driver.SetBoneRotation('Bip01 Spine',SpineRotation);
		}
	}
}

defaultproperties
{
	VehiclePositionString="manning an M240"
	VehicleNameString="M240 Machine Gun"

	// anims
	DriveAnim=Fists_Hold

	EntryPosition=(X=0,Y=0,Z=0)
	EntryRadius=80.0
	ExitPositions(0)=(X=-170,Y=0,Z=-40)
	ExitPositions(1)=(X=-170,Y=0,Z=40)
	ExitPositions(2)=(X=-100,Y=100,Z=-40)
	ExitPositions(3)=(X=-100,Y=-100,Z=40)

	GunClass=class'FragOps.War_Vehicle_JeepM240MG'
	CameraBone=Base
	FPCamPos=(X=-22,Y=0,Z=10)
	FPCamViewOffset=(X=10,Y=0,Z=30)
	bFPNoZFromCameraPitch=False
	bZeroPCRotOnEntry=false
	TPCamLookat=(X=-25,Y=0,Z=0)
	TPCamWorldOffset=(X=0,Y=0,Z=90)
	TPCamDistance=200
	DrivePos=(X=-10.0,Y=0.0,Z=-12.0)
	bDrawDriverInTP=True
	bDrawMeshInFP=True
	DriverDamageMult=0.0
	bHasAltFire=False
}