//=============================================================================
// Frag.Ops  2004 Matt 'SquirrelZero' Farber
//=============================================================================
class War_Vehicle_M240MG extends ONSVehicle;

var rotator LastRotation, TurretRotation;
var float YawAmount;

// placement sound
var sound PlacingSound;

// buyzone stuff
var War_BuildZone MyBZ;
var int BZSlot;

// restrict damage to this vehicle
var bool bRestrictDamage;

// weapon
var ONSWeapon MGWeapon;

replication
{
	reliable if (bNetDirty && Role == ROLE_Authority)
		MGWeapon, YawAmount;
}

simulated function PostBeginPlay()
{
	Super.PostBeginPlay();

	// nice placement sound
	if (Level.NetMode != NM_DedicatedServer)
		PlaySound(PlacingSound,,,,80,0.7,true);
}

simulated function PostNetBeginPlay()
{
	Super.PostNetBeginPlay();
	if (Role == ROLE_Authority && Weapons.Length > 0)
	{
		MGWeapon = Weapons[0];
	    	NetUpdateTime = Level.TimeSeconds - 1;
	}
}

event TakeDamage(int Damage, Pawn EventInstigator, vector HitLocation, vector Momentum, class<DamageType> DamageType)
{
	// reduce fist damage
	if (DamageType.Name == 'DamType_Machete' || DamageType.Name == 'DamType_Fists' || DamageType.Name == 'DamType_AKBayonet')
		Damage *= 0.1;

	Super.TakeDamage(Damage, EventInstigator, HitLocation, Momentum, DamageType);
}

function Pawn GetHitPawn(vector HitLocation, rotator HitRotation)
{
	local vector HitLoc;
	
	HitLoc = HitLocation;
	if (Driver != None)
	{
		HitLoc.Z = Driver.Location.Z;
		if (VSize(HitLoc-Driver.Location) <= Driver.CollisionRadius)
			return Driver;
	}
	return None;
}

// enter
function KDriverEnter(Pawn P)
{
	Super.KDriverEnter(P);

	// save player replicationinfo
	P.PlayerReplicationInfo = Controller.PlayerReplicationInfo;
}

simulated function ClientKDriverEnter(PlayerController PC)
{
	Super.ClientKDriverEnter(PC);
	PC.EndZoom();
}

function bool TryToDrive(Pawn P)
{
	if (P.Weapon != None && (P.Weapon.IsFiring() || (Weapon_Base(P.Weapon).ZoomLevel > 0 && Weapon_Base(P.Weapon).ScopeHUDTex != None)))
		return false;

	return Super.TryToDrive(P);
}

simulated function int LimitYaw(int InYaw)
{
	return InYaw;
}

simulated function AttachDriver(Pawn P)
{
	local vector AttachPos;
	local rotator NewRotator;

	if (MGWeapon == None)
		return;

	P.bHardAttach = true;
	AttachPos = MGWeapon.Location + (DrivePos >> MGWeapon.Rotation);
	P.SetLocation(AttachPos);
	NewRotator.Yaw = MGWeapon.Rotation.Yaw;
	Driver.SetRotation(NewRotator);
	P.SetPhysics(PHYS_None);
	P.SetBase(MGWeapon);
	P.SetRelativeRotation(DriveRot);
	P.SetBoneRotation('Bip01 Spine', rot(0,0,0));
}

simulated function DetachDriver(Pawn P)
{
	P.SetBase(None);
}

// update rotation
function UpdateRocketAcceleration(float deltaTime, float YawChange, float PitchChange)
{
	local rotator NewRotation;
	local float PrevRecoil;
	local War_FOVehicleWeapon MyGun;

	NewRotation = TurretRotation;
	NewRotation.Yaw += 32.0 * deltaTime * YawChange;
	NewRotation.Pitch += 32.0 * deltaTime * PitchChange;
	NewRotation.Yaw = LimitYaw(NewRotation.Yaw);
	if (MGWeapon != None)
	{
		MyGun = War_FOVehicleWeapon(MGWeapon);
		NewRotation.Pitch += MyGun.InitialRecoil;	
		PrevRecoil = MyGun.InitialRecoil;
		MyGun.InitialRecoil -= (PrevRecoil*deltaTime)*19.0;
		if (MyGun.InitialRecoil <= 0.0)
			MyGun.bHasRecoiled = false;
	}
	NewRotation.Pitch = LimitPitch(NewRotation.Pitch);
	TurretRotation = NewRotation;
}

event bool KDriverLeave(bool bForceLeave)
{
	local Pawn OldDriver;
	
	OldDriver = Driver;
	if (Super.KDriverLeave(bForceLeave))
	{
		if (OldDriver != None)
		{
			if (OldDriver.Health > 0)
				OldDriver.SetBoneRotation('Bip01 Spine', rot(0,0,0));
			if (OldDriver.Base == self)
				OldDriver.SetBase(None);
		}
		if (MGWeapon != None)
			MGWeapon.GotoState('InstantFireMode');
		YawAmount = 0;
		return true;
	}
	return false;
}

function AltFire(optional float F)
{
	local PlayerController PC;

	PC = PlayerController(Controller);
	if (PC == None)
		return;

	bWeaponIsAltFiring = true;
	PC.ToggleZoom();
}

function ClientVehicleCeaseFire(bool bWasAltFire)
{
	local PlayerController PC;

	if (!bWasAltFire)
	{
		Super.ClientVehicleCeaseFire(bWasAltFire);
		return;
	}

	PC = PlayerController(Controller);
	if (PC == None)
		return;

	bWeaponIsAltFiring = false;
	PC.StopZoom();
}

simulated function ClientKDriverLeave(PlayerController PC)
{
	Super.ClientKDriverLeave(PC);
	if (MGWeapon != None)
		MGWeapon.GotoState('InstantFireMode');

	bWeaponIsAltFiring = false;
	PC.EndZoom();
}

simulated function Tick(float dt)
{	
	local rotator NewRotation;
	local rotator SpineRotation, TestRotation, ControllerRotation;

	Super.Tick(dt);

	if (Driver != None && FOPawn(Driver).bDriving && MGWeapon != None && Driver.Health > 0 && !Driver.IsInState('Dying'))
	{
		Driver.SetBase(None);
		Driver.bHardAttach = true;

		// update relative location
		Driver.SetBase(self);
		NewRotation.Yaw = MGWeapon.GetBoneRotation(MGWeapon.YawBone).Yaw;
		NewRotation.Pitch = 0;
		NewRotation.Roll = 0;
		Driver.SetLocation(Location+vector(NewRotation)*DrivePos.X + DrivePos.Z*vect(0,0,1));
		Driver.SetRotation(NewRotation);
		Driver.SetPhysics(PHYS_None);
		LastRotation = NewRotation;

		if (Level.NetMode != NM_DedicatedServer)
		{
			// what is up with the twist look anyway?
			Driver.bDoTorsoTwist = false;
			Driver.bRotateToDesired = false;

			// adjust driver rotation
			ControllerRotation.Pitch = MGWeapon.GetBoneRotation(MGWeapon.PitchBone).Pitch & 65535;
			ControllerRotation.Roll = 0;
			TestRotation.Pitch = Driver.Rotation.Pitch & 65535;
			SpineRotation.Yaw = fClamp(float((Normalize(ControllerRotation - TestRotation).Pitch)) * 1.2, -3000.0, 3000.0);		
			Driver.SetBoneRotation('Bip01 Spine',SpineRotation);
		}
	}
}

// overridden -- unwanted
simulated event OnDaredevil()
{
}

simulated function PrecacheAnnouncer(AnnouncerVoice V, bool bRewardSounds)
{
}

function SpawnBuildEffect()
{
}

simulated function SetOverlayMaterial( Material mat, float time, bool bOverride)
{
}

simulated event DestroyAppearance()
{
	local int i;
	local KarmaParams KP;

	// For replication
	bDestroyAppearance = True;

	// Put brakes on
	Throttle	= 0;
	Steering	= 0;
	Rise		= 0;

	// Destroy the weapons
	if (Role == ROLE_Authority)
	{
		for(i=0;i<Weapons.Length;i++)
			Weapons[i].Destroy();
		for(i=0;i<WeaponPawns.Length;i++)
			WeaponPawns[i].Destroy();
	}
	Weapons.Length = 0;
	WeaponPawns.Length = 0;

	// Copy linear velocity from actor so it doesn't just stop.
	KP = KarmaParams(KParams);
	if (KP != None)
	{
		KP.KStartEnabled = true;
		KP.KStartLinVel = Velocity;
	}

	// Become the dead vehicle mesh
	SetPhysics(PHYS_None);
	KSetBlockKarma(False);
	SetStaticMesh(DestroyedVehicleMesh);
	SetDrawType(DT_StaticMesh);
	KSetBlockKarma(True);
	SetPhysics(PHYS_Karma);
	KWake();
	Skins.length = 0;
	NetPriority = 2.5;
	NetUpdateTime = Level.TimeSeconds - 1;
	LifeSpan = 20.0;
}

simulated function Destroyed()
{
	Super.Destroyed();
	
	// release resources
	if (MyBZ != None)
		MyBZ.ResourceUsage[BZSlot] = Max(MyBZ.ResourceUsage[BZSlot]-1, 0);
}

simulated function bool IndependentVehicle()
{
	return true;
}

function SetTeamNum(byte T)
{
	local byte Temp;

	Temp = Team;
	PrevTeam = T;
	Team = T;

	if (Temp != T)
		TeamChanged();

	if (Weapons.Length > 0)
		War_FOVehicleWeapon(Weapons[0]).TeamIndex = T;
}

function Died(Controller Killer, class<DamageType> damageType, vector HitLocation)
{
	local PlayerController PC;
	local Controller C;

	if (bDeleteMe || Level.bLevelChange || bVehicleDestroyed)
		return; // already destroyed, or level is being cleaned up

	bVehicleDestroyed = True;

	if (Level.Game.PreventDeath(self, Killer, damageType, HitLocation))
	{
		Health = max(Health, 1); //mutator should set this higher
		return;
	}
	Health = Min(0, Health);

	if (Controller != None)
	{
		C = Controller;
		C.WasKilledBy(Killer);
		Level.Game.Killed(Killer, C, self, damageType);
		if (C.bIsPlayer)
		{
			PC = PlayerController(C);
			if (PC != None)
				ClientKDriverLeave(PC); // Just to reset HUD etc.
			else
				ClientClearController();
			if ((bRemoteControlled || bEjectDriver) && (Driver != None) && (Driver.Health > 0))
			{
				C.Unpossess();
				C.Possess(Driver);
				if ( bEjectDriver )
					EjectDriver();

				Driver = None;
			}
			else
				C.PawnDied(self);
		}
		else
			C.Destroy();

    		if (Driver != None)
	    	{
			if (!bRemoteControlled && !bEjectDriver)
			{
				if (!bDrawDriverInTP && PlaceExitingDriver())
				{
					Driver.StopDriving(self);
					Driver.DrivenVehicle = self;
				}
				Driver.TearOffMomentum = Velocity * 0.25;
				Driver.Died(Controller, class'DamRanOver', Driver.Location);
			}
			else
			{
				if (bEjectDriver)
					EjectDriver();
				else
					KDriverLeave(false);
			}
	    	}

		bDriving = False;
	}
	else
		Level.Game.Killed(Killer, Controller(Owner), self, damageType);

	if (Killer != None)
	{
		TriggerEvent(Event, self, Killer.Pawn);
		Instigator = Killer.Pawn; //so if the dead vehicle crushes somebody the vehicle's killer gets the credit
	}
	else
		TriggerEvent(Event, self, None);

	RanOverDamageType = DestroyedRoadKillDamageType;
	CrushedDamageType = DestroyedRoadKillDamageType;

	if (IsHumanControlled())
		PlayerController(Controller).ForceDeathUpdate();

	GotoState('VehicleDestroyed');
}

event RanInto(Actor Other)
{
}

simulated function POVChanged(PlayerController PC, bool bBehindViewChanged)
{
	if (PC.bBehindView)
	{
		bOwnerNoSee = false;

		if (Driver != None)
		{
			if (bDrawDriverInTP)
				Driver.bOwnerNoSee = false;
			else
				Driver.bOwnerNoSee = true;
		}

		if (PC == Controller)   // No overlays for spectators
		    ActivateOverlay(False);
	}
	else
	{
		if (bDrawMeshInFP)
			bOwnerNoSee = false;
		else
			bOwnerNoSee = true;

		if (Driver != None)
			Driver.bOwnerNoSee = true;

		if (bDriving && PC == Controller)   // No overlays for spectators
		    ActivateOverlay(True);
	}
}

defaultproperties
{
	// general
	DrawType=DT_Mesh
	Mesh=Mesh'FOVehicleWeapons.M240_base_1'
	VehiclePositionString="manning an M240"
	VehicleNameString="M240 Machine Gun"
	VehicleIcon=(Material=Material'FOWarTextures.VehicleIcons.JeepIcon')
	DrawScale=1.0
	PlacingSound=sound'FOGameSounds.Menu.MedalStomp'
	StolenSound=None
	bZeroPCRotOnEntry=false

	// team beacon
	TeamBeaconTexture=Texture'FOWarTextures.HUD.HealthIcon'
	TeamBeaconBorderMaterial=Texture'FOWarTextures.HUD.HealthBorder'

	// health
	Health=220
	
	// anims
	DriveAnim=Fists_Hold

	// weaponry
	DriverWeapons(0)=(WeaponClass=class'FragOps.War_Vehicle_M240MGWeapon',WeaponBone=Dummy04)

	// camera
	FPCamPos=(X=-35,Y=0,Z=65)
	FPCamViewOffset=(X=0,Y=0,Z=0)
	bFPNoZFromCameraPitch=false
	TPCamLookat=(X=-10,Y=0,Z=0)
	TPCamDistRange=(Min=50,Max=160)
	TPCamWorldOffset=(X=0,Y=0,Z=85)
	TPCamDistance=160
	
	// driver
	bDriverCollideActors=true
	bEjectDriver=true
	EjectMomentum=100
	DrivePos=(X=-42.0,Y=0.0,Z=13.5)

	// entry, exit
	EntryPosition=(X=0,Y=0,Z=0)
	EntryRadius=70.0
	ExitPositions(0)=(X=-75,Y=0,Z=16)
	ExitPositions(1)=(X=0,Y=75,Z=16)
	ExitPositions(2)=(X=0,Y=-75,Z=16)
	ExitPositions(3)=(X=130,Y=0,Z=16)
	ExitPositions(4)=(X=-90,Y=0,Z=26)
	ExitPositions(5)=(X=0,Y=75,Z=26)
	ExitPositions(6)=(X=0,Y=-75,Z=26)
	ExitPositions(7)=(X=150,Y=0,Z=26)

	// destroyed shaking
	ViewShakeRadius=40.0
	ShakeRotMag=(Z=100)
	ShakeRotRate=(Z=1000)
	ShakeRotTime=3
	ShakeOffsetMag=(Z=8)
	ShakeOffsetRate=(Z=100)
	ShakeOffsetTime=8

	// bot desireability
	MaxDesireability=0.3	

	// destroyed mesh
	RanOverDamageType=class'DamType_Roadkill'
	CrushedDamageType=class'DamType_Roadkill'
	ExplosionDamageType=class'DamType_DestroyedVehicle'
	DestroyedRoadKillDamageType=class'DamType_DestroyedVehicle'
	DestroyedVehicleMesh=StaticMesh'FOWarSMs.DeadVehicles.m240_dead'
	DestructionLinearMomentum=(Min=50000,Max=75000)
	DestructionAngularMomentum=(Min=50,Max=75)
	DisintegrationEffectClass=class'FragOps.War_Effect_ConcNadeExplosion'
	DamagedEffectOffset=(X=0,Y=0,Z=25)
	DamagedEffectScale=0.5
	DamagedEffectHealthSmokeFactor=0.2
	DamagedEffectHealthFireFactor=0.01
	DisintegrationHealth=-150

	// MG state
	LandMovementState=PlayerStaticTurreting

	// physics
	Physics=PHYS_None

	// body karma
	Begin Object Class=KarmaParamsRBFull Name=KParams0
		KStartEnabled=false
		KFriction=0.5
		KLinearDamping=0.05
		KAngularDamping=0.05
		KImpactThreshold=500
		bKNonSphericalInertia=true
		bHighDetailOnly=false
		bClientOnly=false
		bKDoubleTickRate=true
		KInertiaTensor(0)=1.0
		KInertiaTensor(1)=0.0
		KInertiaTensor(2)=0.0
		KInertiaTensor(3)=1.0
		KInertiaTensor(4)=0.0
		KInertiaTensor(5)=1.0
		KCOMOffset=(X=0,Y=0.0,Z=0)
		bDestroyOnWorldPenetrate=true
		bDoSafetime=True
	Name="KParams0"
	End Object
	KParams=KarmaParams'KParams0'	

	// settings
	bFollowLookDir=true
	bStationary=true
	bIgnoreForces=true
	MaxLights=4
	bUseDynamicLights=false
	bVehicleShadows=false
	bDrawVehicleShadow=false
	DriverDamageMult=0.0
	bSeparateTurretFocus=true
	bPCRelativeFPRotation=true
	bDrawDriverInTP=true
	bDrawMeshInFP=true
	bHasAltFire=false
	bTeamLocked=false
	bDesiredBehindview=true

	// collision
	CollisionRadius=50.0
	CollisionHeight=60.0
}