//=============================================================================
// Frag.Ops  2004 Matt 'SquirrelZero' Farber
//=============================================================================
class War_Vehicle_Stryker extends War_FOVehicleWheeled;

// some damage modifiers
event TakeDamage(int Damage, Pawn EventInstigator, vector HitLocation, vector Momentum, class<DamageType> DamageType)
{
	// reduce fist damage
	if (DamageType.Name == 'DamType_LAW')
		Damage *= 0.8;
	else
		Damage = float(Damage)*0.6;

	Super.TakeDamage(Damage, EventInstigator, HitLocation, Momentum, DamageType);
}

function Pawn GetHitPawn(vector HitLocation, rotator HitRotation)
{
	return None;
}

defaultproperties
{
	// display
	Mesh=Mesh'FOVehicles.Stryker'
	VehiclePositionString="in a STRK4"
	VehicleNameString="STRK4"
	VehicleIcon=(Material=Material'FOWarTextures.VehicleIcons.JeepIcon')
	DrawScale=1.0
	StolenSound=None

	// health
	Health=850
	HealthMax=850

	// headlights - corona
	HeadlightCoronaOffset(0)=(X=182,Y=42,Z=4)
	HeadlightCoronaOffset(1)=(X=182,Y=-42,Z=4)
	HeadlightCoronaMaterial=Material'EpicParticles.flashflare1'
	HeadlightCoronaMaxSize=38

	// headlights - projector
	HeadlightProjectorOffset=(X=130,Y=0,Z=7)
	HeadlightProjectorRotation=(Yaw=0,Pitch=-1000,Roll=0)
	HeadlightProjectorMaterial=Texture'VMVehicles-TX.RVGroup.RVProjector'
	HeadlightProjectorScale=0.3
	
	// brakelights
	bMakeBrakeLights=true
	BrakeLightOffset(0)=(X=-145.3,Y=46.5,Z=-6.5)
	BrakeLightOffset(1)=(X=-145.3,Y=-46.1,Z=-6.9)
	BrakeLightMaterial=Material'EpicParticles.flashflare1'

	// sounds
	IdleSound=sound'FOVehicleSounds.Stryker.StrykerIdle'
	StartUpSound=sound'FOVehicleSounds.Stryker.StrykerStart'
	ShutDownSound=sound'FOVehicleSounds.Stryker.StrykerTurnOff'

	// damage
	ImpactDamageMult=0.0050
	DamagedEffectOffset=(X=100,Y=10,Z=10)
	DamagedEffectScale=1.0

	// destruction
	DestroyedVehicleMesh=StaticMesh'FOWarSMs.DeadVehicles.stryker_dead'

	// cameras
	FPCamPos=(X=0,Y=0,Z=90)
	TPCamLookat=(X=0,Y=0,Z=0)
	TPCamWorldOffset=(X=0,Y=0,Z=90)
	TPCamDistRange=(Min=220,Max=350)
	TPCamDistance=220

	// collision
	CollisionHeight=+90.0
	CollisionRadius=+110.0

	// driver-related
	MaxViewYaw=19000
	MaxViewPitch=19000
	DrivePos=(X=-10.8,Y=-25.0,Z=-2.0)
	DriveRot=(Pitch=-800)
	DriveAnim=driveg
	DriverDamageMult=0.90
	bDriverCollideActors=false
	bDrawDriverInTP=false
	bDrawMeshInFP=true

	// armament
	PassengerWeapons(0)=(WeaponPawnClass=class'FragOps.War_Vehicle_StrykerGunner',WeaponBone=Gunner)
	PassengerWeapons(1)=(WeaponPawnClass=class'FragOps.War_Vehicle_StrykerPassenger')
	PassengerWeapons(2)=(WeaponPawnClass=class'FragOps.War_Vehicle_StrykerPassengerB')
	PassengerWeapons(3)=(WeaponPawnClass=class'FragOps.War_Vehicle_StrykerPassengerC')
	PassengerWeapons(4)=(WeaponPawnClass=class'FragOps.War_Vehicle_StrykerPassengerD')

	// speed
	GroundSpeed=430

	// physics - wheels
	WheelPenScale=1.5
	WheelPenOffset=0.01
	WheelSoftness=0.04
	WheelRestitution=0.1
	WheelAdhesion=0.0
	WheelLongFrictionFunc=(Points=((InVal=0,OutVal=0.0),(InVal=100.0,OutVal=1.0),(InVal=200.0,OutVal=0.9),(InVal=10000000000.0,OutVal=0.9)))
	WheelLongFrictionScale=1.2
	WheelLatFrictionScale=1.5
	WheelLongSlip=0.0008
	WheelLatSlipFunc=(Points=((InVal=0.0,OutVal=0.0),(InVal=30.0,OutVal=0.009),(InVal=45.0,OutVal=0.00),(InVal=10000000000.0,OutVal=0.00)))
	WheelHandbrakeSlip=0.01
	WheelHandbrakeFriction=0.1
	WheelSuspensionTravel=25.0
	WheelSuspensionOffset=-10.0
	WheelSuspensionMaxRenderTravel=25.0
	
	// physics - control
	TurnDamping=35
	MinBrakeFriction=4.0
	MaxBrakeTorque=20.0
	MaxSteerAngleCurve=(Points=((InVal=0,OutVal=25.0),(InVal=1500.0,OutVal=8.0),(InVal=1000000000.0,OutVal=8.0)))
	SteerSpeed=110
	StopThreshold=100
	TorqueCurve=(Points=((InVal=0,OutVal=9.0),(InVal=195,OutVal=10.0),(InVal=1100,OutVal=12.0),(InVal=2070,OutVal=0.0)))
	EngineBrakeFactor=0.0001
	EngineBrakeRPMScale=0.1
	EngineInertia=0.1
	WheelInertia=0.1
	
	// physics - gears
	TransRatio=0.11
	GearRatios[0]=-0.43
	GearRatios[1]=0.32
	GearRatios[2]=0.57
	GearRatios[3]=0.77
	GearRatios[4]=1.1
	ChangeUpPoint=2000
	ChangeDownPoint=1000
	LSDFactor=1.0

	// physics - misc
	CenterSpringForce="SpringONSSRV"
	HandbrakeThresh=200
	FTScale=0.03
	ChassisTorqueScale=0.6
	VehicleMass=5.0

	// entry
	EntryPosition=(X=0,Y=0,Z=0)
	EntryRadius=160.0

	// exit
	ExitPositions(0)=(X=90,Y=-125,Z=25)
	ExitPositions(1)=(X=90,Y=125,Z=25)
	ExitPositions(3)=(X=90,Y=-160,Z=-25)
	ExitPositions(4)=(X=90,Y=160,Z=-25)
	ExitPositions(5)=(X=200,Y=0,Z=25)
	ExitPositions(6)=(X=-400,Y=0,Z=25)

	// body karma
	Begin Object Class=KarmaParamsRBFull Name=KParams0
		KStartEnabled=True
		KFriction=0.5
		KLinearDamping=0.05
		KAngularDamping=0.05
		KImpactThreshold=500
		bKNonSphericalInertia=True
		bHighDetailOnly=False
		bClientOnly=False
		bKDoubleTickRate=True
		KInertiaTensor(0)=3.8
		KInertiaTensor(1)=0.0
		KInertiaTensor(2)=0.0
		KInertiaTensor(3)=3.3
		KInertiaTensor(4)=0.0
		KInertiaTensor(5)=3.5
		KCOMOffset=(X=0.35,Y=0.0,Z=-0.66)
		bDestroyOnWorldPenetrate=true
		bDoSafetime=True
	Name="KParams0"
	End Object
	KParams=KarmaParams'KParams0'

	// body karma
	Begin Object Class=KarmaParamsRBFull Name=KParams1
		KStartEnabled=True
		KFriction=0.5
		KLinearDamping=0.05
		KAngularDamping=0.05
		KImpactThreshold=500
		bKNonSphericalInertia=True
		bHighDetailOnly=False
		bClientOnly=False
		bKDoubleTickRate=false
		KInertiaTensor(0)=3.8
		KInertiaTensor(1)=0.0
		KInertiaTensor(2)=0.0
		KInertiaTensor(3)=3.3
		KInertiaTensor(4)=0.0
		KInertiaTensor(5)=3.5
		KCOMOffset=(X=0.35,Y=0.0,Z=-0.66)
		bDestroyOnWorldPenetrate=true
		bDoSafetime=false
	Name="KParams1"
	End Object
	AltKParams=KarmaParams'KParams1'

	// rear wheels
	Begin Object Class=SVehicleWheel Name=RRWheel
		BoneName="Right_Tire04"
		BoneRollAxis=AXIS_Y
		BoneSteerAxis=AXIS_Z
		BoneOffset=(X=-12.0,Y=11.0,Z=0.0)
		WheelRadius=26
		bPoweredWheel=True
		bHandbrakeWheel=True
		SteerType=VST_Fixed
		SupportBoneAxis=AXIS_X
		SupportBoneName="Right_Strut04"
	End Object
	Wheels(0)=SVehicleWheel'RRWheel'

	Begin Object Class=SVehicleWheel Name=LRWheel
		BoneName="Left_Tire04"
		BoneRollAxis=AXIS_Y
		BoneSteerAxis=AXIS_Z
		BoneOffset=(X=-12.0,Y=-11.0,Z=0.0)
		WheelRadius=26
		bPoweredWheel=True
		bHandbrakeWheel=True
		SteerType=VST_Fixed
		SupportBoneAxis=AXIS_X
		SupportBoneName="Left_Strut04"
	End Object
	Wheels(1)=SVehicleWheel'LRWheel'

	Begin Object Class=SVehicleWheel Name=RRMWheel
		BoneName="Right_Tire03"
		BoneRollAxis=AXIS_Y
		BoneSteerAxis=AXIS_Z
		BoneOffset=(X=-12.0,Y=11.0,Z=0.0)
		WheelRadius=26
		bPoweredWheel=True
		bHandbrakeWheel=True
		SteerType=VST_Fixed
		SupportBoneAxis=AXIS_X
		SupportBoneName="Right_Strut03"
	End Object
	Wheels(2)=SVehicleWheel'RRMWheel'

	Begin Object Class=SVehicleWheel Name=LRMWheel
		BoneName="Left_Tire03"
		BoneRollAxis=AXIS_Y
		BoneSteerAxis=AXIS_Z
		BoneOffset=(X=-12.0,Y=-11.0,Z=0.0)
		WheelRadius=26
		bPoweredWheel=True
		bHandbrakeWheel=True
		SteerType=VST_Fixed
		SupportBoneAxis=AXIS_X
		SupportBoneName="Left_Strut03"
	End Object
	Wheels(3)=SVehicleWheel'LRMWheel'

	// front wheels
	Begin Object Class=SVehicleWheel Name=RFMWheel
		BoneName="Right_Tire01"
		BoneRollAxis=AXIS_Y
		BoneSteerAxis=AXIS_Z
		BoneOffset=(X=12.0,Y=11.0,Z=0.0)
		WheelRadius=26
		bPoweredWheel=True
		SteerType=VST_Steered
		SupportBoneAxis=AXIS_X
		SupportBoneName="Right_Strut01"
	End Object
	Wheels(4)=SVehicleWheel'RFMWheel'

	Begin Object Class=SVehicleWheel Name=LFMWheel
		BoneName="Left_Tire01"
		BoneRollAxis=AXIS_Y
		BoneSteerAxis=AXIS_Z
		BoneOffset=(X=12.0,Y=-11.0,Z=0.0)
		WheelRadius=26
		bPoweredWheel=True
		SteerType=VST_Steered
		SupportBoneAxis=AXIS_X
		SupportBoneName="Left_Strut01"
	End Object
	Wheels(5)=SVehicleWheel'LFMWheel'

	Begin Object Class=SVehicleWheel Name=RFWheel
		BoneName="Right_Tire02"
		BoneRollAxis=AXIS_Y
		BoneSteerAxis=AXIS_Z
		BoneOffset=(X=12.0,Y=11.0,Z=0.0)
		WheelRadius=26
		bPoweredWheel=True
		SteerType=VST_Steered
		SupportBoneAxis=AXIS_X
		SupportBoneName="Right_Strut02"
	End Object
	Wheels(6)=SVehicleWheel'RFWheel'

	Begin Object Class=SVehicleWheel Name=LFWheel
		BoneName="Left_Tire02"
		BoneRollAxis=AXIS_Y
		BoneSteerAxis=AXIS_Z
		BoneOffset=(X=12.0,Y=-11.0,Z=0.0)
		WheelRadius=26
		bPoweredWheel=True
		SteerType=VST_Steered
		SupportBoneAxis=AXIS_X
		SupportBoneName="Left_Strut02"
	End Object
	Wheels(7)=SVehicleWheel'LFWheel'
}