//=============================================================================
// Frag.Ops  2004 Matt 'SquirrelZero' Farber
//=============================================================================
// Like the machine gun from the onslaught tank.  Mostly a copy and paste job,
// main differences in how weapons handle and other effects.
//=============================================================================
class War_Vehicle_StrykerMG extends War_FOVehicleWeapon;

var rotator GrenadeRotationPivot;
var vector GrenadeFireOffset;
var bool bAlternating;

simulated function InitEffects()
{
	Super.InitEffects();
	if (ShellEjector != None)
		SetBoneRotation(ShellBone,rot(16384,0,0));
}

// limityaw.. necessary for us
simulated function int LimitYaw(int InYaw, rotator CurrentRotation)
{
	return InYaw;
}

static function StaticPrecache(LevelInfo L)
{
}

simulated function UpdatePrecacheMaterials()
{
}

function byte BestMode()
{
	return 0;
}

simulated event OwnerEffects()
{
	if ((AmmoAmount <= 0 && !bIsAltFire) || IsInState('Reloading'))
		return;

	if (!bIsRepeatingFF)
	{
		if (bIsAltFire)
			ClientPlayForceFeedback(AltFireForce);
		else
			ClientPlayForceFeedback(FireForce);
	}

	ShakeView();
	if (Role < ROLE_Authority)
	{
		if (bIsAltFire)
			FireCountdown = AltFireInterval;
		else
			FireCountdown = FireInterval;

		AimLockReleaseTime = Level.TimeSeconds + FireCountdown * FireIntervalAimLock;

		if (!bIsAltFire)
		        FlashMuzzleFlash();

		if (AmbientEffectEmitter != None)
			AmbientEffectEmitter.SetEmitterStatus(true);

		// Play firing noise
		if (!bAmbientFireSound)
		{
			if (bIsAltFire)
				PlaySound(AltFireSoundClass, SLOT_None, FireSoundVolume/255.0,, AltFireSoundRadius,, false);
			else
				PlaySound(FireSoundClass, SLOT_None, FireSoundVolume/255.0,, FireSoundRadius,, false);
		}
	}
}

state InstantFireMode
{
	// launch grenades here
	function AltFire(Controller C)
	{
		if (IsInState('Reloading'))
			return;
		
		SpawnProjectile(AltFireProjectileClass, true);
	}

	function Projectile SpawnProjectile(class<Projectile> ProjClass, bool bAltFire)
	{
		local Projectile P;
		local rotator GrenadeFireRotation;
		local vector StartLocation;

		// adjust fire offset
		GrenadeFireRotation = WeaponFireRotation;
		if (bAlternating)
		{
			bAlternating = false;
			GrenadeRotationPivot.Yaw = -Default.GrenadeRotationPivot.Yaw;
		}
		else
		{
			bAlternating = true;
			GrenadeRotationPivot = Default.GrenadeRotationPivot;
		}

		StartLocation = GetAltFireLocation();
		P = spawn(ProjClass, self,, StartLocation, GrenadeFireRotation);
		if (P != None)
		{
			if (bAmbientAltFireSound)
				AmbientSound = AltFireSoundClass;
			else
				PlayOwnedSound(AltFireSoundClass, SLOT_None, FireSoundVolume/255.0,, AltFireSoundRadius,, false);
		}
		return P;
	}
}

function vector GetAltFireLocation()
{
	local coords WeaponBoneCoords;
	local vector CurrentFireOffset;
	local rotator TempRotation;

	// Calculate fire offset in world space
	WeaponBoneCoords = GetBoneCoords('mg_base');
	CurrentFireOffset = GrenadeFireOffset;

	// Calculate rotation of the gun
	TempRotation = rotator(vector(CurrentAim) >> Rotation);

	// Calculate exact fire location
	return (WeaponBoneCoords.Origin + (CurrentFireOffset >> TempRotation));
}


defaultproperties
{
	// display
	Mesh=Mesh'FOVehicleWeapons.StrikerMG'
	DrawScale=2.0

	// grenades
	AltFireProjectileClass=class'War_Stryker_ConcGrenade'
	AltFireSoundClass=sound'FOVehicleSounds.Stryker.StrykerMortar'
	bInheritVelocity=true
	GrenadeRotationPivot=(Yaw=8000)
	GrenadeFireOffset=(X=30,Z=6)

	// HUD
	FireType="Auto"
	AmmoType=".50 AE"

	// rotation limits
	YawBone=YawBone
	YawStartConstraint=-4000
	YawEndConstraint=4000
	PitchBone=Pitch_Bone
	PitchUpLimit=4300
	PitchDownLimit=58500
	bInstantFire=true

	// sound
	bAmbientAltFireSound=false
	bAmbientFireSound=false
	FireSoundClass=sound'FOVehicleSounds.Stryker.StrykerFire'
	SoundVolume=255
	AmbientSoundScaling=1.3

	// muzzle flash
	NewFlashEmitterClass=class'War_Effect_M240MuzFlash'
	NewSmokeEmitterClass=class'War_Effect_M240MuzSmoke'
	WeaponFireAttachmentBone=bone_flash

	// recoil
	RecoilSize=55

	// ammo
	AmmoAmount=150
	MaxAmmo=150
	NumClips=1
	MaxClips=1

	// reloading
	ReloadTime=6.0
	ReloadBeginSound=sound'WeaponSounds.click_ammo_out'
	ReloadMidSound=sound'WeaponSounds.BReload6'
	ReloadEndSound=sound'WeaponSounds.PulseRifleReload'

	// conefire, maybe?
	Spread=0.005

	// fire rate
	FireInterval=0.13
	AltFireInterval=2.5
	
	// damage
	DamageType=class'DamType_M240mg'
	DamageMin=30
	DamageMax=35

	// ranges
	WeaponRange=2800	
	WeaponDamageShrink=170   // if maxrange exceeded, damage = damage - (DistanceRemainder / WeaponDamageShrink)
	TraceRange=7000

	// bullet shells
	ShellBone=Bone_Shell
	ShellEjectorClass=class'Effect_ShellEjector50aeFirst'

	// fire shake
	ShakeOffsetMag=(X=1.0,Y=1.0,Z=1.0)
	ShakeOffsetRate=(X=1000.0,Y=1000.0,Z=1000.0)
	ShakeOffsetTime=2
	ShakeRotMag=(X=50.0,Y=50.0,Z=50.0)
	ShakeRotRate=(X=10000.0,Y=10000.0,Z=10000.0)
	ShakeRotTime=2
	FireForce="minifireb"
	
	// settings
	RotationsPerSecond=2.0
	bInstantRotation=true
	bDoOffsetTrace=true
	bAimable=true
	bIsRepeatingFF=true

	// collision
	bUseCylinderCollision=false
	bBlockNonZeroExtentTraces=true
	bBlockActors=true
	bCollideActors=true

	// AI
	AIInfo(0)=(bInstantHit=true,AimError=500)

	// culling
	CullDistance=7000.0
}
