//=============================================================================
// Frag.Ops  2004 Matt 'SquirrelZero' Farber
//=============================================================================
class War_Vehicle_StrykerPassenger extends War_FOVehiclePawn;

function UpdateRocketAcceleration(float deltaTime, float YawChange, float PitchChange)
{
	local rotator NewRotation;

	NewRotation = Rotation;
	NewRotation.Yaw += 32.0 * deltaTime * YawChange;
	NewRotation.Pitch += 32.0 * deltaTime * PitchChange;
	NewRotation.Yaw = LimitYaw(NewRotation.Yaw);
	NewRotation.Pitch = LimitPitch(NewRotation.Pitch);
	SetRotation(NewRotation);
}

function AttachToVehicle(ONSVehicle VehiclePawn, name WeaponBone)
{
	if (Level.NetMode != NM_Client)
		VehicleBase = VehiclePawn;
}
simulated function int LimitYaw(int InYaw)
{
	return InYaw;
}

simulated function SpecialCalcFirstPersonView(PlayerController PC, out actor ViewActor, out vector CameraLocation, out rotator CameraRotation )
{
	local vector x, y, z;
	local vector VehicleZ, CamViewOffsetWorld;
	local float CamViewOffsetZAmount;
	local coords CamBoneCoords;

	GetAxes(CameraRotation, x, y, z);
	ViewActor = VehicleBase;

	CamViewOffsetWorld = FPCamViewOffset >> CameraRotation;

	if (CameraBone != '' && VehicleBase != None)
	{
		CamBoneCoords = VehicleBase.GetBoneCoords(CameraBone);
		CameraLocation = CamBoneCoords.Origin + (FPCamPos >> Rotation) + CamViewOffsetWorld;

		if (bFPNoZFromCameraPitch)
		{
			VehicleZ = vect(0,0,1) >> Rotation;
			CamViewOffsetZAmount = CamViewOffsetWorld Dot VehicleZ;
			CameraLocation -= CamViewOffsetZAmount * VehicleZ;
		}
	}
	else
	{
		CameraLocation = GetCameraLocationStart() + (FPCamPos >> Rotation) + CamViewOffsetWorld;

		if (bFPNoZFromCameraPitch)
		{
			VehicleZ = vect(0,0,1) >> Rotation;
			CamViewOffsetZAmount = CamViewOffsetWorld Dot VehicleZ;
			CameraLocation -= CamViewOffsetZAmount * VehicleZ;
		}
	}

	CameraRotation = Normalize(CameraRotation + PC.ShakeRot);
	CameraLocation = CameraLocation + PC.ShakeOffset.X * x + PC.ShakeOffset.Y * y + PC.ShakeOffset.Z * z;
}

function KDriverEnter(Pawn P)
{
	local rotator NewRotation;

	Super(Vehicle).KDriverEnter(P);

	if (VehicleBase != None && VehicleBase.bTeamLocked && VehicleBase.bEnterringUnlocks)
		VehicleBase.bTeamLocked = false;

	if (!bHasOwnHealth && VehicleBase == None)
	{
		Health = Driver.Health;
		HealthMax = Driver.HealthMax;
	}

	NewRotation = Controller.Rotation;
	NewRotation.Pitch = LimitPitch(NewRotation.Pitch);
	SetRotation(NewRotation);
	Driver.bSetPCRotOnPossess = false; //so when driver gets out he'll be facing the same direction as he was inside the vehicle
}

function AltFire(optional float F)
{
}

function Fire(optional float F)
{
}

function ClientVehicleCeaseFire(bool bWasAltFire)
{
	if (!bWasAltFire)
	{
		Super.ClientVehicleCeaseFire(bWasAltFire);
		return;
	}

	bWeaponIsAltFiring = false;
}


simulated function vector GetCameraLocationStart()
{
	if (VehicleBase != None)
		return VehicleBase.GetBoneCoords(CameraBone).Origin;
	else
		return Super.GetCameraLocationStart();
}

function Pawn GetHitPawn(vector HitLocation, rotator HitRotation)
{
	return None;
}

defaultproperties
{
	PitchUpLimit=9000
	PitchDownLimit=-9000
	
	VehiclePositionString="riding in a STRK4"
	VehicleNameString="STRK4 passenger"

	TPCamDistRange=(Min=70,Max=300)

	// driver
	DrivePos=(X=-100.0,Y=35.0,Z=10.0)

	// entry/exit
	EntryPosition=(X=0,Y=0,Z=0)
	EntryRadius=160.0
	ExitPositions(0)=(X=-205,Y=0,Z=25)
	ExitPositions(1)=(X=-205,Y=0,Z=25)
	ExitPositions(2)=(X=-205,Y=50,Z=25)
	ExitPositions(3)=(X=-205,Y=-50,Z=25)
	ExitPositions(4)=(X=-220,Y=50,Z=-25)
	ExitPositions(5)=(X=-220,Y=-50,Z=-25)
	ExitPositions(6)=(X=-100,Y=120,Z=25)
	ExitPositions(7)=(X=-100,Y=-120,Z=25)
	ExitPositions(8)=(X=-100,Y=120,Z=-25)
	ExitPositions(9)=(X=-100,Y=-120,Z=-25)

	// camera
	CameraBone=Passenger01
	FPCamPos=(X=0,Y=0,Z=10)
	FPCamViewOffset=(X=0,Y=0,Z=80)
	TPCamLookat=(X=-25,Y=0,Z=0)
	TPCamWorldOffset=(X=0,Y=0,Z=80)
	TPCamDistance=200
	bFPNoZFromCameraPitch=False

	// settings
	bDriverCollideActors=false
	bDrawDriverInTP=false
	bDrawMeshInFP=true
	bHasAltFire=False
	DriverDamageMult=0.0

	GunClass=None
	LandMovementState=PlayerTurreting
}