//=============================================================================
// Frag.Ops  2003 Matt 'SquirrelZero' Farber
//=============================================================================
class Weapon_AK103Stab extends FireMode_Base;

var float PreStabTime;
var bool bIsStabbing;

simulated function PostBeginPlay()
{
	bIsStabbing = false;
}

simulated function bool AllowFire()
{
	return (!Instigator.Controller.IsInState('PreRound') && !Instigator.Controller.IsInState('Building') && !Weapon_Base(Weapon).bIsReloading && Instigator.Controller != None);
}

function DoFireEffect()
{
	if (!bIsStabbing)
	{
		Weapon.GotoState('Stabbing','Begin');
		bIsStabbing = true;
	}
}

function DoStab()
{
	if (bIsStabbing)
	{
		Super.DoFireEffect();
		bIsStabbing = false;
	}
}

event ModeDoFire()
{
	if ((Level.TimeSeconds - LastFireTime) > 0.2)
	{
		Spread = Default.Spread;
		NumFireTimes = 0;
	}
	else
	{
		NumFireTimes++;
		Spread += NumFireTimes * ConeMultiPlier;
		Spread = FMin(Spread, ConeMax);
	}

	if (!AllowFire())
		return;

	if (MaxHoldTime > 0.0)
		HoldTime = FMin(HoldTime, MaxHoldTime);

	// server
	if (Weapon.Role == ROLE_Authority)
	{
		DoFireEffect();
		if ( (Instigator == None) || (Instigator.Controller == None) )
			return;
		if ( AIController(Instigator.Controller) != None )
			AIController(Instigator.Controller).WeaponFireAgain(BotRefireRate, true);
		Instigator.SpawnTime = -100000;
	}
	// client
	if (Instigator.IsLocallyControlled())
	{
		ShakeView();
		PlayFiring();
		FlashMuzzleFlash();
		StartMuzzleSmoke();
	}

	Weapon.IncrementFlashCount(1);

	// set the next firing time. must be careful here so client and server do not get out of sync
	if (bFireOnRelease)
	{
	        if (bIsFiring)
			NextFireTime += MaxHoldTime + FireRate;
		else
			NextFireTime = Level.TimeSeconds + FireRate;
	}
	else
	{
		NextFireTime += FireRate;
		NextFireTime = FMax(NextFireTime, Level.TimeSeconds);
	}

	Load = AmmoPerFire;
	HoldTime = 0;

	if (Instigator.PendingWeapon != Weapon && Instigator.PendingWeapon != None)
	{
		bIsFiring = false;
		Weapon.PutDown();
	}
	LastFireTime = Level.TimeSeconds;
}

defaultproperties
{
	AmmoPerFire=1
	WeaponDamage=100
	DamageType=class'FragOps.DamType_AKBayonet'
	bPawnRapidFireAnim=false

	FireAnim=Stab
	FireAnimRate=1.0
	Momentum=2.0
	FireSound=Sound'FOWeaponSounds.AK103.AK103Stab'

	FlashEmitterClass=None
	SmokeEmitterClass=None
	SmokeEmitter2ndClass=None
	ShellEjectorClass=None

	BulletHitEffectClass=class'Effect_KnifeHit'

	RecoilSize=0
	Spread=0.0  // best to leave this at 0 for now
	SpreadStyle=SS_None
	AimError=0  // leave at 0 i guess, till i find a use for it
	WeaponRange=110   // let the gun stab just slightly in front of the player
	DeadBulletRange=110    // we don't want to do damage reduction here
	
	FireRate=0.6
	PreStabTime=0.21

	bWaitForRelease=true
	bModeExclusive=true
}