//=============================================================================
// Frag.Ops  2003 Matt 'SquirrelZero' Farber
//=============================================================================

#exec OBJ LOAD FILE=FOWeaponSounds.uax
#exec OBJ LOAD FILE=pickups.ukx
#exec OBJ LOAD FILE=magnum2.utx
#exec OBJ LOAD FILE=FOHighDetailTextures.utx

class Weapon_Base extends Weapon
	HideDropDown
	CacheExempt;

// not sure if i'll use it but this is for attaching tracers to the gun
var() name TracerBone, ShellBone, SmokeBone;

var() Name IdleEmptyAnim;
var() Name ScopeDownAnim;
var() float ScopeDownAnimRate;
var() Sound WeaponEmptySound, FireSwitchSound;
var() localized string AttachName;
var() localized string GunDesc;
var const int GunCost;
var int NumFireTypes, CurrentFireType;
var float ReloadTime;
var() float BurstMultiple; // for 3 and 2 round burst firetypes
var() float ZoomMultiple; // how far to zoom on scoped weapons
var float LastFireTime;
var Material ScopeHUDTex;
var() Material WeaponIcon;
var() Material AkimboIcon;

var int Unlocked; // for unlockable weapons

var int ZoomLevel; // for sniper rifles, to balance recoil
var float SniperZoomLevels[2];
var float LaserDistance;  // for drawing the canvas laser on laser based weapons
var bool bUp, bIsAkimbo, bIsSpecial, bFiredLast, bCanBeBought, bIsReloading, bPlayIdleEmpty, bWeaponOwnsCross, bScopeZoomed, bHasRecoiled, bDontSwitchNow, bJustBought, bHasSpecialReload, bAkimbo;

var class<Weapon_Base> GunAttachMent[3], BaseWeaponClass; // more attachment variables
var localized string StaticName; // specifies the base weapon type of the gun, never changes in attachments
var string FireTypeNames[3]; // Readable names for the switchable firemodes

// recoil stuff
var float InitialRecoil, InitialYawRecoil;

// recoil prediction
var rotator FireRotation;
var float FireTimeStamp;

// spectating
var float NewPutdownTime, NewBringUpTime;

// smoothed weapon rotation
var() float WeaponTurnLimit, WeaponPitchLimit;
var float YAWTurnOffset, PITCHTurnOffset, LastTurn, LastLookUp;
var rotator RotationBase;

// offset for drawing staticmesh in buymenu
var vector BuyMenuOffset;

// hacked in offset for ejecting shells when akimbo
var float ShellHackOffset;

// akimbo slave
var Weapon_SlaveBase SlaveWeapon;
var class<actor> AkimboAttachmentClass;

// CXP requirement for use
var rotator RealPivot;
var int CXPReq;

// sniper breathing
var int ForwardBreathMax, BackwardBreathMax;
var float ForwardBreathSpeed, BackwardBreathSpeed, BreathFOV, BreathRate, MasterBreathScale;
var rotator BreathRot, ActualRot;
var vector BreathLoc;
var bool bForwardBreath, bHoldingBreath;

// Specifies the type of weapon.  Used across several classes.
var enum EWeaponType
{
	Melee,
	Sidearm,
	SMG,
	Rifle,
	Sniper,
	Shotgun,
	Misc,
	Grenade,
	Heavy,
	Fists
}WeaponType;

// war
var bool bBuilding;
var float PitchAdjustment, ZAdjustment;

replication
{
	reliable if (Role < Role_Authority)
		ServerReload, ServerPutDown, ServerSetFireType, ServerSetZoom;

	unreliable if (Role < Role_Authority && WeaponType != Grenade && WeaponType != Heavy && WeaponType != Fists && WeaponType != Melee)
		ServerSetFireData;

	reliable if (ROLE == ROLE_Authority)
		ClientSetSwitch;

	reliable if ((bAkimbo) && (bNetOwner) && (Role == Role_Authority))
		bIsAkimbo;
}

simulated function PreBeginPlay()
{
	Super.PreBeginPlay();
	if (Level.NetMode == NM_DedicatedServer || Level.NetMode == NM_ListenServer)
		return;
		
	if (AttachName != "")
	{
		ExchangeFireModes = BaseWeaponClass.Default.ExchangeFireModes;
		Priority = BaseWeaponClass.Default.Priority;
		CustomCrossHair = BaseWeaponClass.Default.CustomCrossHair;
		CustomCrossHairScale = BaseWeaponClass.Default.CustomCrossHairScale;
		CustomCrossHairColor = BaseWeaponClass.Default.CustomCrossHairColor;
		CustomCrossHairTextureName = BaseWeaponClass.Default.CustomCrossHairTextureName;
	}
}


function AttachToPawn(Pawn P)
{
	local name BoneName;

	Instigator = P;
	if (ThirdPersonActor == None)
	{
		ThirdPersonActor = Spawn(AttachmentClass,Owner);
		InventoryAttachment(ThirdPersonActor).InitFor(self);
	}
	else
		ThirdPersonActor.NetUpdateTime = Level.TimeSeconds - 1;
	BoneName = P.GetWeaponBoneFor(self);
	if (BoneName == '')
	{
		ThirdPersonActor.SetLocation(P.Location);
		ThirdPersonActor.SetBase(P);
	}
	else
		P.AttachToBone(ThirdPersonActor,BoneName);
	if (Level.NetMode == NM_DedicatedServer)
		FOPawn(P).SetWeaponAttachment(xWeaponAttachment(ThirdPersonActor));
}

final function ServerSetFireData(int FireYaw, int FirePitch, byte FireModeNum)
{
	if (Level.NetMode == NM_StandAlone || Level.NetMode == NM_ListenServer)
		return;
	
	FireTimeStamp = Level.TimeSeconds;
	FireRotation.Yaw = FireYaw;
	FireRotation.Pitch = FirePitch;
	FireRotation.Roll = 0;
	FireMode_Base(FireMode[FireModeNum]).bClientApproved = true;
	FireMode_Base(FireMode[FireModeNum]).ModeDoFire();
}

// our own ammo functions.  Protected variables suck (tm).
simulated function Ammo_Base GetAmmo(int mode)
{
	if (Ammo[mode] == None)
		return None;

	return Ammo_Base(Ammo[mode]);
}

// firemode retrieval
simulated function FireMode_Base GetFM(int mode)
{
	if (mode >= NUM_FIRE_MODES)
		return None;

	return FireMode_Base(FireMode[mode]);
}

// slave weapon control functions, for akimbo
function SpawnSlave(int AddedAmmo, int AddedClips, optional bool bWasBought, optional bool bAutoSwitch);
function KillSlave();

// update accuracy arrays, for medals
function AddMedalPoints(int HitZone, int HitTeam)
{
	if (HitTeam == Instigator.PlayerReplicationInfo.Team.TeamIndex || HitTeam > 1)
		return;

	switch (Default.WeaponType)
	{
		case Sidearm:
			if (FOPawn(Instigator).CriticalReplicationInfo.mPistolPoints[0] < 2)
				FOPawn(Instigator).CriticalReplicationInfo.mPistolPoints[HitZone]++;
			break;

		default:
			break;
	}
}

// set pivot adjustment
simulated function SetTurnPivot(float NewTurn, float NewLookUp)
{
	LastTurn = NewTurn;
	LastLookUp = NewLookUp;
}

// new pivot calculation system
simulated event RenderOverlays( Canvas Canvas )
{
	local int m;
	local vector NewScale3D, CamPos;
	local rotator CamRot;

	if (Instigator == None)
		return;

	if (Instigator.Controller != None)
		Hand = Instigator.Controller.Handedness;

	if ((Hand < -1.0) || (Hand > 1.0))
	{
		SetLocation(Instigator.Location);
		return;
	}

	// draw muzzleflashes/smoke for all fire modes so idle state won't
	// cause emitters to just disappear
	for (m=0;m<NUM_FIRE_MODES;m++)
	{
		if (FireMode[m] != None)
		{
			FireMode[m].DrawMuzzleFlash(Canvas);
		}
	}

	if (Hand != RenderedHand)
	{
		newScale3D = Default.DrawScale3D;
		newScale3D.Y *= Hand;
		SetDrawScale3D(newScale3D);
		PlayerViewPivot.Roll = Default.PlayerViewPivot.Roll * Hand;
		PlayerViewPivot.Yaw = Default.PlayerViewPivot.Yaw * Hand;
		RenderedHand = Hand;
	}
	PlayerViewPivot.Roll = Default.PlayerViewPivot.Roll*Hand + RealPivot.Roll*Hand;
	PlayerViewOffset = Default.PlayerViewOffset;
	PlayerViewOffset.Y *= Hand;

	Canvas.GetCameraLocation(CamPos,CamRot);	

	// location
	SetLocation(CamPos + ((0.9/DisplayFOV*100*PlayerViewOffset) >> RotationBase) + (Instigator.WeaponBob(BobDamping)*0.14));

	// set to new rotation
	SetRotation(RotationBase);

	PreDrawFPWeapon();	// Laurent -- Hook to override things before render (like rotation if using a staticmesh)

	bDrawingFirstPerson = true;
	Canvas.DrawActor(self, false, false, DisplayFOV);
	bDrawingFirstPerson = false;
}

simulated function bool IsAkimbo()
{
	return false;
}

simulated function IncrementFlashCount(int Mode)
{
	local int i;
	
	LastFireTime = Level.TimeSeconds;
	NetUpdateTime = Level.TimeSeconds - 1;

	for (i=0;i<FOPawn(Instigator).SpectatorControllers.Length;i++)
	{
		if (FOPawn(Instigator).SpectatorControllers[i] != None)
			FOPawn(Instigator).SpectatorControllers[i].ClientFire(Mode);
	}

	if (WeaponAttachment(ThirdPersonActor) != None)
	{
		if (Mode == 0)
			WeaponAttachment(ThirdPersonActor).FiringMode = 0;
		else
			WeaponAttachment(ThirdPersonActor).FiringMode = 1;
		ThirdPersonActor.NetUpdateTime = Level.TimeSeconds - 2;
		WeaponAttachment(ThirdPersonActor).FlashCount++;
		WeaponAttachment(ThirdPersonActor).ThirdPersonEffects();
	}
}

simulated event StopFire(int Mode)
{
	if (FireMode[Mode].bIsFiring)
		FireMode[Mode].bInstantStop = true;
	if (Instigator.IsLocallyControlled() && !FireMode[Mode].bFireOnRelease)
		FireMode[Mode].PlayFireEnd();

	FireMode[Mode].bIsFiring = false;
	FireMode[Mode].StopFiring();

	// fix.. was causing 3rd person effects to misfire
	if (!FireMode[Mode].bFireOnRelease && (bIsReloading || Instigator.Health <= 0 || (Instigator.Controller != None && Instigator.Controller.IsInState('PreRound'))))
		ZeroFlashCount(Mode);
}

final simulated function DoRecoil(float NewRecoilSize, byte YawLevel)
{
	local rotator RecoilRot;
	local float AddedRecoil, YawRecoil, TempHand;

	if (Instigator.Controller != None && NewRecoilSize > 0)
	{
		if (Instigator.Physics == PHYS_None)
			AddedRecoil = NewRecoilSize * 0.33;
		else if (Instigator.Physics == PHYS_Falling) // falling / jumping
			AddedRecoil = NewRecoilSize * 0.57;
		else if (Instigator.bIsCrouched) // crouched
			AddedRecoil = NewRecoilSize * 0.22;
		else if (Instigator.bIsWalking) // walking
			AddedRecoil = NewRecoilSize * 0.33;
		else if (Instigator.IsInState('PlayerAction')) // action (sprinting/rolling)
			AddedRecoil = NewRecoilSize * 0.46;
		else if (Instigator.Velocity.X != 0 || Instigator.Velocity.Y != 0 || Instigator.Velocity.Z != 0) // running
			AddedRecoil = NewRecoilSize * 0.39;
		else // otherwise, standing still
			AddedRecoil = NewRecoilSize * 0.29;

		if (ZoomLevel > 0)
		{
			if (WeaponType == Sniper)
				AddedRecoil = AddedRecoil * (1.12-(ZoomLevel*0.17));
			else
				AddedRecoil = AddedRecoil * 0.94;
		}
		else if (WeaponType == Sniper)
			AddedRecoil = AddedRecoil * 1.1;

		// add yaw recoil
		if (Hand != -1 && Hand != 1)
			TempHand = 1;
		else
			TempHand = Hand;
		if (FRand() < FClamp(float(YawLevel)*0.1, 0, 0.9))
			YawRecoil = (AddedRecoil * FMin((float(YawLevel)*0.022), 0.7))*TempHand;
		if (FRand() > 0.6)
			YawRecoil = -YawRecoil;

		RecoilRot = Instigator.Controller.Rotation;
		RecoilRot.Pitch += AddedRecoil*1.05;
		if (RecoilRot.Pitch > 17999 && RecoilRot.Pitch < 49153)
			RecoilRot.Pitch -= (AddedRecoil*1.7);

		RecoilRot.Yaw += YawRecoil;
		Instigator.Controller.SetRotation(RecoilRot);

		// smoothing
		bHasRecoiled = true;
		InitialRecoil = 0.6 * AddedRecoil;
		InitialYawRecoil = 0.73 * YawRecoil;
	}
	else
		bHasRecoiled = false;
}

final simulated function AddRecoil(float NewRecoilSize)
{
	local rotator RecoilRot;

	if (Instigator.Controller != None && NewRecoilSize > 0)
	{
		RecoilRot = Instigator.Controller.Rotation;
		RecoilRot.Pitch += NewRecoilSize;
		if (RecoilRot.Pitch > 17999 && RecoilRot.Pitch < 49153)
			RecoilRot.Pitch -= (NewRecoilSize*2);

		Instigator.Controller.SetRotation(RecoilRot);
	}
}

final simulated function AddYawRecoil(float NewRecoilSize)
{
	local rotator RecoilRot;

	if (Instigator.Controller != None && NewRecoilSize > 0)
	{
		RecoilRot = Instigator.Controller.Rotation;
		RecoilRot.Yaw += NewRecoilSize;
		Instigator.Controller.SetRotation(RecoilRot);
	}
}

simulated function WeaponTick(float dt)
{
	local float PrevRecoil;
	local bool bIsHuman;

	Super.WeaponTick(dt);

	// reset flashcount to 0
	if (LastFireTime > 0 && Level.TimeSeconds - LastFireTime > 2.0)
	{
		LastFireTime = 0;
		ZeroFlashCount(0);
	}

	// update location for AI
	bIsHuman = Instigator.IsHumanControlled();
	if (!bIsHuman)
		SetLocation(Instigator.Location);

	if (Level.NetMode == NM_DedicatedServer)
		return;

	// scoped breathing
	if (bScopeZoomed)
		Breathe(dt);

	// get rotation base
	RotationBase = Instigator.GetViewRotation();

	// reset for vehicles
	if (Instigator.DrivenVehicle != None && War_FOVehiclePawnFree(Instigator.DrivenVehicle) == None)
	{
		PlayerViewPivot = Default.PlayerViewPivot;
		PlayerViewOffset = Default.PlayerViewOffset;
		return;
	}

	// now add aTurn and aLook pivot adjustments
	if (bIsHuman)
	{
		if (LastTurn == 0)
		{
			YAWTurnOffset = YAWTurnOffset - (YAWTurnOffset*dt*7.0);
			if ((YAWTurnOffset < 2.0 && YAWTurnOffset > 0) || (YAWTurnOffset > -2.0 && YAWTurnOffset < 0))
				YAWTurnOffset = 0;
		}
		else
			YAWTurnOffset = fClamp(YAWTurnOffset - (LastTurn*dt*1.2), -WeaponTurnLimit, WeaponTurnLimit);

		if (LastLookUp == 0)
		{
			PITCHTurnOffset = PITCHTurnOffset - (PITCHTurnOffset*dt*5.0);
			if ((PITCHTurnOffset < 2.0 && PITCHTurnOffset > 0) || (PITCHTurnOffset > -2.0 && PITCHTurnOffset < 0))
				PITCHTurnOffset = 0;
		}
		else
			PITCHTurnOffset = fClamp(PITCHTurnOffset - (LastLookUp*dt*1.2), -WeaponPitchLimit, WeaponPitchLimit);

		RotationBase.Yaw += YAWTurnOffset;
		RotationBase.Pitch += PITCHTurnOffset;
	}

	// recoil smoothing
	if (bHasRecoiled && Instigator != None && Instigator.Controller != None)
	{
		AddRecoil(InitialRecoil);
		PrevRecoil = InitialRecoil;
		InitialRecoil -= (PrevRecoil*dt)*19.0;

		AddYawRecoil(InitialYawRecoil);
		InitialYawRecoil = FMax((InitialYawRecoil*dt)*19.0, 0);

		if (InitialRecoil <= 0.0)
			bHasRecoiled = false;
	}

	// adjust while building, no firing
	if (bBuilding && (PitchAdjustment < 5000 || ZAdjustment < 3.0))
	{
		PitchAdjustment = FClamp(PitchAdjustment + dt*7000.0, 0, 5000);
		ZAdjustment = FClamp(ZAdjustment + dt*10.0, 0, 3.0);
		RealPivot.Roll = -PitchAdjustment;
		PlayerViewOffset.Z = Default.PlayerViewOffset.Z - ZAdjustment;
	}
	else if (!bBuilding && (PitchAdjustment > 0 || ZAdjustment > 0))
	{
		PitchAdjustment = FClamp(PitchAdjustment - dt*7000.0, 0, 5000);
		ZAdjustment = FClamp(ZAdjustment - dt*10.0, 0, 3.0);
		RealPivot.Roll = -PitchAdjustment;
		PlayerViewOffset.Z = Default.PlayerViewOffset.Z - ZAdjustment;
	}
}

// one really nice breathing function
final simulated function Breathe(float dt)
{
	local PlayerController PC;
	local float BreathScale, AddedSpeed;
	
	PC = PlayerController(Instigator.Controller);
	if (PC == None)
		return;

	// speed up when holding breath
	if (FOPawn(Instigator).bHoldingBreath)
		AddedSpeed = 500;

	// breathe faster if running faster
	BreathScale = FClamp((VSize(Instigator.Velocity)+AddedSpeed+BreathRate)/BreathRate, 1.0, 1.95);

	// adjust constraints for shortness of breath
	ForwardBreathMax = float(Default.ForwardBreathMax)*(2.0-FMax(BreathScale*0.82, 1.0));
	BackwardBreathMax = float(Default.BackwardBreathMax)*(2.0-FMax(BreathScale*0.82, 1.0));

	// reset and breathe out when hold time ends
	if (!FOPawn(Instigator).bHoldingBreath && bHoldingBreath)
	{
		bForwardBreath = true;
		bHoldingBreath = false;
		BackwardBreathMax = 0;
	}

	// breathe in, breathe out
	if (BreathRot.Pitch >= BackwardBreathMax && !bForwardBreath && !bHoldingBreath)
	{
		Instigator.PlaySound(Sound'FOGameSounds.PlayerSounds.BreatheIn',SLOT_Pain,3.5*TransientSoundVolume,,,FMin(0.76+(BreathScale*0.2), 1.16));
		bForwardBreath = true;
	}
	else if (BreathRot.Pitch <= -ForwardBreathMax && bForwardBreath)
	{
		if (FOPawn(Instigator).bHoldingBreath)
		{
			bHoldingBreath = true;
			Instigator.PlaySound(Sound'FOGameSounds.PlayerSounds.BreatheOut',SLOT_Pain,3.1*TransientSoundVolume,,,1.3);
		}
		else
			Instigator.PlaySound(Sound'FOGameSounds.PlayerSounds.BreatheOut',SLOT_Pain,3.1*TransientSoundVolume,,,FMin(0.76+(BreathScale*0.2), 1.16));
		bForwardBreath = false;
	}

	if (bHoldingBreath)
	{
		BackwardBreathMax = 0;
		BackwardBreathSpeed = 3.0*Default.BackwardBreathSpeed;
	}
	else
		BackwardBreathSpeed = Default.BackwardBreathSpeed;

	// camera rotation adjustment
	if (bForwardBreath)
		BreathRot.Pitch = Max(BreathRot.Pitch+Min((dt*-((ForwardBreathSpeed*BreathScale)*FMax(1.0-(float(BreathRot.Pitch)/float(-ForwardBreathMax)), 0.43))), -2), -ForwardBreathMax);
	else
		BreathRot.Pitch = Min(BreathRot.Pitch+(dt*(BackwardBreathSpeed*BreathScale)), BackwardBreathMax);

	// camera location adjustment
	BreathLoc.X = (-(float(BreathRot.Pitch)*0.02))*MasterBreathScale;
	BreathLoc.Z = (float(BreathRot.Pitch)*0.01)*MasterBreathScale;
	BreathLoc.Y = (float(BreathRot.Pitch)*0.01)*MasterBreathScale;

	// FOV adjustment
	//BreathFOV = float(BreathRot.Pitch)*0.006;

	// now set actual rotation
	ActualRot.Pitch = BreathRot.Pitch*(0.3*MasterBreathScale);
}

simulated function SpecialFire();

function GiveTo(Pawn Other, optional Pickup Pickup)
{
	local int m;
	local weapon w;
	local bool bPossiblySwitch, bJustSpawned;

	// set the instigator
	Instigator = Other;

	// maybe some leftover weapon from buymenu?  kill it.
	W = Weapon(Instigator.FindInventoryType(class));
	if (W != None && W != self && W.Default.InventoryGroup != 4)
		W.Destroy();

	// add to inventory chain
	if (Other.AddInventory(Self))
		GotoState('');
	else
		Destroy();
	W = self;

	// if it's a pickup, we might want it to switch
	if (Pickup == None)
		bPossiblySwitch = true;
	else
		bPossiblySwitch = false;

	// tell weapons on clientside whether they can switch or not, set grenade inventory groups
	ClientSetSwitch(bDontSwitchNow,bJustBought,InventoryGroup);

	// set firemode/ammo instigators
	for (m = 0; m < NUM_FIRE_MODES; m++)
	{
		if (FireMode[m] != None)
		{
			FireMode[m].Instigator = Instigator;
			GiveAmmo(m,WeaponPickup(Pickup),bJustSpawned);
		}
	}

	// if firing don't switch at all
	if ((Instigator.Weapon != None) && Instigator.Weapon.IsFiring())
		bPossiblySwitch = false;

	// now set the weapon on the client
	ClientWeaponSet(bPossiblySwitch);
}

function GiveAmmo(int m, WeaponPickup WP, bool bJustSpawned)
{
	local bool bJustSpawnedAmmo;
	local int addAmount, ClipAmount;
	local Ammo_Null NullAmmo;

	if (FireMode[m] != None && FireMode[m].AmmoClass != None)
	{
		if (FireMode[m].AmmoClass.Name == 'Ammo_Null')
		{
			foreach DynamicActors(class'Ammo_Null', NullAmmo)
			{
				if (NullAmmo.Instigator != Instigator)
					continue;

				Ammo[m] = NullAmmo;
				Ammo[m].NetUpdateTime = Level.TimeSeconds - 1;
				break;
			}
		}
		
		if ((Ammo[m] == None) && (FireMode[m].AmmoClass != None))
		{
			Ammo[m] = Spawn(FireMode[m].AmmoClass, Instigator);
			Instigator.AddInventory(Ammo[m]);
			bJustSpawnedAmmo = true;
		}
		else if ((m == 0) || (FireMode[m].AmmoClass != FireMode[0].AmmoClass))
			bJustSpawnedAmmo = (bJustSpawned || WP == None);

		if (WP != None)
		{
			addAmount = WP.AmmoAmount[m];
			ClipAmount = FOWeaponPickup(WP).NumClips[m];
		}
		else if (bJustSpawnedAmmo)
		{
			ClipAmount = 0;
			AddAmount = Ammo[m].InitialAmount;
		}

		Ammo[m].AddAmmo(addAmount);
		Ammo_Base(Ammo[m]).AddClips(ClipAmount);
		Ammo[m].GotoState('');
	}
}

simulated function ClientSetSwitch(bool bTempSwitchNow, bool bTempJustBought, byte TempInventoryGroup)
{
	bDontSwitchNow = bTempSwitchNow;
	bJustBought = bTempJustBought;
	if (WeaponType == Grenade)
		InventoryGroup = TempInventoryGroup;
}

simulated function ClientWeaponSet(bool bPossiblySwitch)
{
	local int Mode;

	bPendingSwitch = bPossiblySwitch;

	if (Instigator == None)
	{
		GotoState('PendingClientWeaponSet');
		return;
	}

	// set skins
	if (FOPawn(Instigator).Default.TeamIndex == 0)
	{
		Skins[0] = FOPawn(Instigator).Default.MercSleaveSkins[FOPawn(Instigator).BodySkin];
		Skins[1] = FOPawn(Instigator).Default.MercGloveSkin;
	}
	else
	{
		Skins[0] = FOPawn(Instigator).Default.SOSleaveSkins[FOPawn(Instigator).BodySkin];
		Skins[1] = FOPawn(Instigator).Default.SOGloveSkin;
	}

	for (Mode=0;Mode<NUM_FIRE_MODES;Mode++)
	{
		if (FireModeClass[Mode] != None)
		{	
			if ((FireMode[Mode] == None ) || (FireMode[Mode].AmmoClass != None) && (Ammo[Mode] == None))
			{
				GotoState('PendingClientWeaponSet');
				return;
			}
		}

		FireMode[Mode].Instigator = Instigator;
		FireMode[Mode].Level = Level;
	}

	if (Level.NetMode == NM_DedicatedServer && !Instigator.IsLocallyControlled())
			return;

	ClientState = WS_Hidden;
	GotoState('Hidden');

	if (Level.NetMode == NM_DedicatedServer && !Instigator.IsHumanControlled())
		return;

	if (Instigator.Weapon == self || Instigator.PendingWeapon == self) // this weapon was switched to while waiting for replication, switch to it now
	{
		if (Instigator.PendingWeapon != None)
		{
			if (Instigator.Weapon != self)
				Instigator.Weapon.PutDown();
			else
				Instigator.ChangedWeapon();
		}
		else
			BringUp();
		return;
	}

	if (Instigator.PendingWeapon != None && Instigator.PendingWeapon.bForceSwitch)
		return;

	if (Instigator.Weapon == None && !bDontSwitchNow)
	{
		Instigator.PendingWeapon = self;
		Instigator.ChangedWeapon();
	}
	else if (bPossiblySwitch && (!bDontSwitchNow || bJustBought))
	{
		if (Instigator.PendingWeapon != None)
		{
			if (RateSelf() > Instigator.PendingWeapon.RateSelf())
			{
				Instigator.PendingWeapon = self;
				Instigator.Weapon.PutDown();
			}
		}
		else if (RateSelf() > Instigator.Weapon.RateSelf() && (Instigator.Weapon.ClientState == WS_ReadyToFire || Instigator.Weapon.ClientState == WS_Hidden))
		{
			Instigator.PendingWeapon = self;
			Instigator.Weapon.PutDown();
		}
		else if (bJustBought && Instigator.Weapon == None || (Instigator.Weapon != None && (Instigator.Weapon.ClientState == WS_ReadyToFire || Instigator.Weapon.ClientState == WS_Hidden)))
		{
			Instigator.PendingWeapon = self;
			if (Instigator.Weapon != None)
				Instigator.Weapon.PutDown();
			else
				Instigator.ChangedWeapon();
		}
	}
}

state PendingClientWeaponSet
{
	simulated function Timer()
	{
		ClientWeaponSet(bPendingSwitch);
		if (IsInState('PendingClientWeaponSet'))
			SetTimer(0.05, false);
	}

	simulated function BeginState()
	{
		SetTimer(0.05, false);
	}

	simulated function EndState()
	{
	}
}

simulated function Timer()
{
	local int Mode;

	if (ClientState == WS_BringUp)
	{
		for (Mode = 0; Mode < NUM_FIRE_MODES; Mode++)
			FireMode[Mode].InitEffects();
		PlayIdle();
		ClientState = WS_ReadyToFire;
	}
	else if (ClientState == WS_PutDown)
	{
		if ( Instigator.PendingWeapon == None )
		{
			PlayIdle();
			ClientState = WS_ReadyToFire;
		}
		else
		{
			ClientState = WS_Hidden;
			Instigator.ChangedWeapon();
			for (Mode = 0;Mode<NUM_FIRE_MODES;Mode++)
				FireMode[Mode].DestroyEffects();
		}
	}
}

simulated function SwitchFireType(optional int i)
{
	local int TempFireType;
	
	if (NumFireTypes == 0)
		return;

	PlayOwnedSound(FireSwitchSound,,TransientSoundVolume*0.95,,30,,false);

	if (i > 0)
	{
		TempFireType = CurrentFireType - 1;
		if (TempFireType >= 1)
		{
			SetFireType(TempFireType);
			if (ROLE < ROLE_Authority)
				ServerSetFireType(TempFireType);
		}
		else
		{
			SetFireType(NumFireTypes);
			if (ROLE < ROLE_Authority)
				ServerSetFireType(NumFireTypes);
		}
	}
	else
	{
		TempFireType = CurrentFireType + 1;
		if (TempFireType <= NumFireTypes)
		{
			SetFireType(TempFireType);
			if (ROLE < ROLE_Authority)
				ServerSetFireType(TempFireType);
		}
		else
		{
			SetFireType(1);
			if (ROLE < ROLE_Authority)
				ServerSetFireType(1);
		}
	}
}

simulated function SetFireType(int i)
{
	CurrentFireType = i;

	if (!FireMode[0].Default.bWaitForRelease && CurrentFireType == 1 && NumFireTypes >= 1)
	{
		FireMode[0].bWaitForRelease = false;
		FireMode[0].bNowWaiting = false;
		FireMode[0].FireRate = FireMode[0].Default.FireRate;
	}
	else if (CurrentFireType == 2 && NumFireTypes >= 2)
	{
		FireMode[0].bWaitForRelease = true;
		FireMode[0].bNowWaiting = true;
		FireMode[0].FireRate = FireMode[0].Default.FireRate * BurstMultiple;
	}
	else if (CurrentFireType == 3 && NumFireTypes >= 3)
	{
		FireMode[0].bWaitForRelease = true;
		FireMode[0].bNowWaiting = true;
		FireMode[0].FireRate = FireMode[0].Default.FireRate * 1.1;
	}
}

function ServerSetFireType(int i)
{
	SetFireType(i);
}

function ServerSetZoom(int i)
{
	if (ScopeHUDTex != None && Instigator != None && ((ZoomLevel == 0 && i > 0) || (ZoomLevel > 0 && i == 0)))
		FOPawn(Instigator).StopAction();
	ZoomLevel = i;
}

function float RefireRate()
{
	return FireMode[0].BotRefireRate;
}

simulated function bool CanThrow()
{
	local int Mode;

	for (Mode=0;Mode<NUM_FIRE_MODES;Mode++)
	{
		if (FireMode[Mode].bFireOnRelease && FireMode[Mode].bIsFiring )
			return false;
		if (FireMode[Mode].NextFireTime > Level.TimeSeconds)
			return false;
	}
	return (bCanThrow && !IsReloading() && (ClientState == WS_ReadyToFire || (Level.NetMode == NM_DedicatedServer) || (Level.NetMode == NM_ListenServer)));
}

function DropFrom(vector StartLocation)
{
	local int m;
	local Pickup Pickup;

	if ((!Default.bCanThrow || FragOps_GRI(Level.Game.GameReplicationInfo).bPreRound) && (Instigator.Health <= 0 || Instigator == None))
	{

		// if not preround and this is a non-droppable item, find a better item to drop
		if (!FragOps_GRI(Level.Game.GameReplicationInfo).bPreRound)
			FindBetterDrop(StartLocation);

		if (Instigator != None)
		{
			FOPawn(Instigator).EquipList[InventoryGroup-1] = None;
			FOPawn(Instigator).AttachList[InventoryGroup-1] = None;
			DetachFromPawn(Instigator);
		}
		Destroy();
		return;
	}

	if (ZoomLevel > 0)
		ZoomLevel = 0;

	FOPawn(Instigator).EquipList[InventoryGroup-1] = None;
	FOPawn(Instigator).AttachList[InventoryGroup-1] = None;

	ClientWeaponThrown();

	for (m = 0; m < NUM_FIRE_MODES; m++)
	{
		if (FireMode[m].bIsFiring)
			StopFire(m);
	}

	if (Instigator != None)
		DetachFromPawn(Instigator);

	Pickup = Spawn(PickupClass,,, StartLocation);
	if (Pickup != None)
	{
		Pickup.InitDroppedPickupFor(self);
		Pickup.Velocity = Velocity;
		if (Instigator.Health > 0)
			WeaponPickup(Pickup).bThrown = true;
	}

	Destroy();
}

function FindBetterDrop(vector StartLoc)
{
	local Weapon BetterWeapon;

	if (Instigator == None || FOPawn(Instigator) == None || (FOPawn(Instigator).EquipList[1] == None && FOPawn(Instigator).EquipList[2] == None && FOPawn(Instigator).EquipList[3] == None && FOPawn(Instigator).EquipList[4] == None))
		return;

	BetterWeapon = FOPawn(Instigator).GetBestWeapon();
	
	if (BetterWeapon == None || BetterWeapon == self)
		return;

	BetterWeapon.Velocity = Velocity;
	BetterWeapon.DropFrom(StartLoc);
}

function DestroyWeapon()
{
	local int m;

	ClientWeaponThrown();

	for (m = 0; m < NUM_FIRE_MODES; m++)
	{
		if (FireMode[m].bIsFiring)
			StopFire(m);
	}
	if (Instigator != None)
	{
		DetachFromPawn(Instigator);
	}
	Destroy();
}

simulated function bool HasAmmo()
{
	return(Ammo[0] == None || AmmoAmount(0) > 0);
}

// states

// reload state
simulated state ReloadWeapon
{
	simulated function BeginState()
	{
		local FireMode_Base FOFireClass;

		// setup
		bIsReloading = true;
		FOPawn(Instigator).bIsReloading = true;
		bCanThrow = false;

		// 3rd person animation, both server and client
		Instigator.SetAnimAction('Weapon_Switch');

		if (Instigator.IsLocallyControlled())
		{
			if (FireMode[0].bIsFiring)
				ClientStopFire(0);
			if (FireMode[1].bIsFiring)
				ClientStopFire(1);
			Instigator.Controller.bFire = 0;
			Instigator.Controller.bAltFire = 0;

			// anim/sound
			PlaySound(FireMode[0].ReloadSound,,FireMode_Base(FireMode[0]).ReloadVolume*TransientSoundVolume*0.82,,100,,false);
			FOFireClass = FireMode_Base(FireMode[0]);
			bPlayIdleEmpty = false;
		}
		// animation fix for demos
		if (Instigator.IsLocallyControlled() || bDemoOwner)
			PlayAnim(FOFireClass.ReloadAnim, FOFireClass.ReloadAnimRate, 0.0);
		// server -- sound
		else if (Level.NetMode == NM_DedicatedServer)
			Instigator.PlayOwnedSound(Sound'WeaponSounds.AssaultRifle.SwitchToAssaultRifle',,1.1*TransientSoundVolume,,100,,false);

		// server -- readd ammo
		if (ROLE == ROLE_Authority)
		{
			Ammo_Base(Ammo[0]).ReloadClip();
		}
	}
	
	simulated function EndState()
	{
		// reset
		bIsReloading = false;
		FOPawn(Instigator).bIsReloading = false;
		if (Default.bCanThrow)
			bCanThrow = true;
		if (FOBot(Instigator.Controller) != None)
			FOBot(Instigator.Controller).DoCharge();
		if (Instigator.PendingWeapon != None)
			Instigator.PendingWeapon = None;
	}

	simulated function bool IsReloading()
	{
		return true;
	}

Begin:
	sleep(ReloadTime);
	if (Instigator.IsLocallyControlled())
		PlayIdle();
	GotoState('');
}

// bursting
simulated state Burst
{
	simulated function BeginState();
	simulated function EndState();

Begin:
	if (FireMode[0].AllowFire())
	{
		Sleep(FireMode[0].Default.FireRate);
		FireMode[0].ModeDoFire();
	}		
	if (FireMode[0].AllowFire())
	{
		Sleep(FireMode[0].Default.FireRate);
		FireMode[0].ModeDoFire();
	}
	GotoState('');
}

simulated function bool IsReloading()
{
	return false;
}

simulated function bool AkimboIsReloading()
{
	return false;
}

function ServerReload()
{
	GotoState('ReloadWeapon','Begin');
}

simulated function PlayReload()
{
	GotoState('ReloadWeapon','Begin');
	// send the server to this state
	if (ROLE < ROLE_Authority)
		ServerReload();
}

simulated function PlayIdle()
{
	if (bPlayIdleEmpty && HasAnim(IdleEmptyAnim))
		PlayIdleEmpty();
	else
		LoopAnim(IdleAnim, IdleAnimRate, 0.15);
}

simulated function PlayIdleEmpty()
{
	LoopAnim(IdleEmptyAnim, IdleAnimRate, 0.15);
}

simulated function PlayEmptySound()
{
	if (Instigator.Controller != None)
		PlayOwnedSound(WeaponEmptySound,,TransientSoundVolume*1.8,,,,false);
	else
		Instigator.PlayOwnedSound(WeaponEmptySound,,TransientSoundVolume*1.8,,,,false);
}

simulated function GetClipCount(out int ClipAmount)
{
	local Ammo_Base FOAmmoClass;
	FOAmmoClass = Ammo_Base(Ammo[0]);
	if (FOAmmoClass == None)
		return;
	ClipAmount = FOAmmoClass.NumClips;
}

simulated function Weapon WeaponChange( byte F, bool bSilent )
{
	if (InventoryGroup == F)
	{
		return self;
	}
	else if (Inventory == None)
		return None;
	else
		return Inventory.WeaponChange(F,bSilent);
}

simulated function Weapon PrevWeapon(Weapon CurrentChoice, Weapon CurrentWeapon)
{
	if ((CurrentChoice == None))
	{
		if (CurrentWeapon != self)
			CurrentChoice = self;
	}
	else if (InventoryGroup == CurrentWeapon.InventoryGroup)
	{
		if ((GroupOffset < CurrentWeapon.GroupOffset)
		&& ((CurrentChoice.InventoryGroup != InventoryGroup) || (GroupOffset > CurrentChoice.GroupOffset)))
			CurrentChoice = self;
	}
	else if (InventoryGroup == CurrentChoice.InventoryGroup)
	{
		if (GroupOffset > CurrentChoice.GroupOffset)
			CurrentChoice = self;
	}
	else if (InventoryGroup > CurrentChoice.InventoryGroup)
	{
		if ((InventoryGroup < CurrentWeapon.InventoryGroup)
		|| (CurrentChoice.InventoryGroup > CurrentWeapon.InventoryGroup))
			CurrentChoice = self;
	}
	else if ( (CurrentChoice.InventoryGroup > CurrentWeapon.InventoryGroup)
	&& (InventoryGroup < CurrentWeapon.InventoryGroup))
		CurrentChoice = self;

	if (Inventory == None)
		return CurrentChoice;
	else
		return Inventory.PrevWeapon(CurrentChoice,CurrentWeapon);
}

simulated function Weapon NextWeapon(Weapon CurrentChoice, Weapon CurrentWeapon)
{
	if ((CurrentChoice == None))
	{
		if ( CurrentWeapon != self )
			CurrentChoice = self;
	}
	else if (InventoryGroup == CurrentWeapon.InventoryGroup)
	{
		if ((GroupOffset > CurrentWeapon.GroupOffset)
		&& ((CurrentChoice.InventoryGroup != InventoryGroup) || (GroupOffset < CurrentChoice.GroupOffset)))
			CurrentChoice = self;
	}
	else if (InventoryGroup == CurrentChoice.InventoryGroup)
	{
		if (GroupOffset < CurrentChoice.GroupOffset)
			CurrentChoice = self;
	}
	else if (InventoryGroup < CurrentChoice.InventoryGroup)
	{
		if ((InventoryGroup > CurrentWeapon.InventoryGroup)
		|| (CurrentChoice.InventoryGroup < CurrentWeapon.InventoryGroup))
			CurrentChoice = self;
	}
	else if ((CurrentChoice.InventoryGroup < CurrentWeapon.InventoryGroup)
	&& (InventoryGroup > CurrentWeapon.InventoryGroup))
		CurrentChoice = self;

	if (Inventory == None)
		return CurrentChoice;
	else
		return Inventory.NextWeapon(CurrentChoice,CurrentWeapon);
}

simulated function BringUp(optional Weapon PrevWeapon)
{
	local int Mode;

	if (ClientState == WS_Hidden)
	{
		PlayOwnedSound(SelectSound, SLOT_Interact,,,,, false);
		ClientPlayForceFeedback(SelectForce);  // jdf

		if (Instigator.IsLocallyControlled() || bDemoOwner)
		{
			RotationBase = Instigator.GetViewRotation();
			
			// better way to tell the HUD we've switched guns
			if (FOBot(Instigator.Controller) == None)
			{
				Interface_MainHUD(FOPlayer(Instigator.Controller).MyHUD).CurWeaponName = GetHumanReadableName();
				Interface_MainHUD(FOPlayer(Instigator.Controller).MyHUD).CurInventoryGroup = InventoryGroup;
			}
			if (HasAnim(SelectAnim) && FOBot(Instigator.Controller) == None)
				PlayAnim(SelectAnim, SelectAnimRate, 0.0);
	        }

		ClientState = WS_BringUp;
		SetTimer(BringUpTime, false);
	}
	NewBringUpTime = Level.TimeSeconds;
	bUp = true;
	if (Role == ROLE_Authority)
		Ammo[0].NetUpdateFrequency = 50;
	for (Mode = 0; Mode < NUM_FIRE_MODES; Mode++)
	{
		FireMode[Mode].bIsFiring = false;
		FireMode[Mode].HoldTime = 0.0;
		FireMode[Mode].bServerDelayStartFire = false;
		FireMode[Mode].bServerDelayStopFire = false;
		FireMode[Mode].bInstantStop = false;
	}
	if ((PrevWeapon != None) && !PrevWeapon.bNoVoluntarySwitch)
		OldWeapon = PrevWeapon;
	else
		OldWeapon = None;
}

simulated function ServerPutDown()
{
	NewPutdownTime = Level.TimeSeconds;
	FireRotation = rot(0,0,0);
}

simulated function bool PutDown()
{
	local int Mode;

	if (ClientState == WS_BringUp || ClientState == WS_ReadyToFire)
	{
		if (!Instigator.PendingWeapon.bForceSwitch)
		{
			for (Mode = 0; Mode < NUM_FIRE_MODES; Mode++)
			{
				if (FireMode[Mode].bFireOnRelease && FireMode[Mode].bIsFiring)
					return false;
			}
		}

		ServerPutDown();
		if (Instigator.IsLocallyControlled())
		{
			for (Mode = 0; Mode < NUM_FIRE_MODES; Mode++)
			{
				if (FireMode[Mode].bIsFiring)
					ClientStopFire(Mode);
			}

			if (ClientState != WS_BringUp && HasAnim(PutDownAnim))
				PlayAnim(PutDownAnim, PutDownAnimRate, 0.0);
		}
		ClientState = WS_PutDown;
		SetTimer(0.3, false);
	}
	for (Mode = 0; Mode < NUM_FIRE_MODES; Mode++)
	{
		FireMode[Mode].bServerDelayStartFire = false;
		FireMode[Mode].bServerDelayStopFire = false;
	}
	Instigator.AmbientSound = None;
	OldWeapon = None;
	return true; // return false if preventing weapon switch
}

simulated event ClientStartFire(int Mode)
{
	if (Pawn(Owner).Controller.IsInState('GameEnded') || Pawn(Owner).Controller.IsInState('RoundEnded'))
		return;

	if (Role < ROLE_Authority)
	{
		if (bDebugging)
			log(self$" client start fire ");
		if (StartFire(Mode))
		{
			if (bDebugging)
				log(self$" call server start fire with role "$Role);
			ServerStartFire(Mode);
			NetUpdateTime = Level.TimeSeconds - 1;
		}
	}
	else
	{
		StartFire(Mode);
	}
}

//// server only ////
event ServerStartFire(byte Mode)
{
	if ((Instigator != None) && (Instigator.Weapon != self))
	{
		if (bDebugging)
			ClientWriteFire("Skipped");
		if (Instigator.Weapon == None)
			Instigator.ServerChangedWeapon(None,self);
		else
			Instigator.Weapon.SynchronizeWeapon(self);
		return;
	}

	if ((FireMode[Mode].NextFireTime <= Level.TimeSeconds + FireMode[Mode].PreFireTime) && StartFire(Mode))
	{
		if (bDebugging)
			ClientWriteFire("Success");
		FireMode[Mode].ServerStartFireTime = Level.TimeSeconds;
		FireMode[Mode].bServerDelayStartFire = false;

		// stop players from resting
		if (Instigator.IsInState('PlayerResting') && Instigator.Controller != None)
			FOPlayer(Instigator.Controller).ServerUnAction();
	}
	else
	{
		FireMode[Mode].bServerDelayStartFire = true;
		if (bDebugging) 
			ClientWriteFire("Delayed");
	}
}

simulated function bool ReadyToFire(int Mode)
{
	local int alt;

	if (Mode == 0)
		alt = 1;
	else
		alt = 0;

	if (((FireMode[alt] != FireMode[Mode]) && FireMode[alt].bModeExclusive && FireMode[alt].bIsFiring) || (FireMode[Mode].NextFireTime > Level.TimeSeconds + FireMode[Mode].PreFireTime))
		return false;

	return true;
}

simulated function SetScope()
{
	if (bScopeZoomed)
	{
		FOPlayer(Instigator.Controller).StartZoom();
		FOPlayer(Instigator.Controller).ZoomLevel = ZoomMultiple;
		bWeaponOwnsCross = true;
	}
	else
	{
		FOPlayer(Instigator.Controller).EndZoom();
		if (GetAnimSeq() != FireMode[0].Default.ReloadAnim && !bIsReloading && HasAnim(ScopeDownAnim))
			PlayAnim(ScopeDownAnim, ScopeDownAnimRate, 0.0);
		if (WeaponType != Sniper && WeaponType != Heavy)
			bWeaponOwnsCross = false;
	}
}

// lazily copied from another class..
simulated function name GetAnimSeq()
{
	local name anim;
	local float frame, rate;
    
	GetAnimParams(0, anim, frame, rate);
	return anim;
}

simulated function DoAutoSwitch()
{
}

simulated function OutOfAmmo()
{
}

simulated function StartBerserk()
{
}

simulated function StopBerserk()
{
}

simulated function bool WeaponCentered()
{
	return false;
}

// need to figure out modified rating based on enemy/tactical situation
simulated function float RateSelf()
{
	if (Instigator.Controller == None)
		return 0;
	else if (!Instigator.IsHumanControlled())
	{
		if (Ammo_Base(Ammo[0]).NumClips == 0 || Ammo[0].AmmoAmount == 0)
			CurrentRating = -2;
		else
			CurrentRating = Instigator.Controller.RateWeapon(self);
	}
	else
		CurrentRating = Instigator.Controller.RateWeapon(self);
	return CurrentRating;
}

function byte BestMode()
{
	return 0;
}

// destroyed, edited to fit old firemode code
simulated function Destroyed()
{
	local int m;

	AmbientSound = None;

	for (m=0;m<NUM_FIRE_MODES;m++)
	{
		if (FireMode[m] != None)
		{
			FragOps_FireMode(FireMode[m]).Destroyed();
			FragOps_FireMode(FireMode[m]).DestroyEffects();
		}
		if (Ammo[m] != None)
		{
			Ammo[m].Destroy();
			Ammo[m] = None;
		}
	}
	Super(Inventory).Destroyed();
}

function DetachFromPawn(Pawn Dropped)
{
	bUp = false;
	Super.DetachFromPawn(Dropped);
}

// War
simulated function StartBuild()
{
	bBuilding = true;
}

simulated function EndBuild()
{
	bBuilding = false;
}

defaultproperties
{
	DrawType=DT_Mesh
	Style=STY_Normal
	PlayerViewOffset=(X=0,Y=0,Z=0)
	EffectOffset=(X=100.0,Y=14.5,Z=-20.0)
	WeaponPitchLimit=400
	WeaponTurnLimit=1000
	WeaponType=Rifle
	InventoryGroup=1
	Unlocked=-1
	FireModeClass(0)=None
	FireModeClass(1)=None

	// breathing
	ForwardBreathSpeed=690.0
	BackwardBreathSpeed=565.0
	ForwardBreathMax=445.0
	BackwardBreathMax=325.0
	MasterBreathScale=0.6
	BreathRate=300.0

	// switchable firing modes
	NumFireTypes=0
	CurrentFireType=1
	FireTypeNames(0)="Auto"

	UV2Texture=Material'XGameShaders.WeaponEnvShader'
	BaseWeaponClass=None

	// animation
	IdleAnim=idle
	IdleEmptyAnim=idleempty
	RestAnim=rest
	AimAnim=aim
	RunAnim=run
	SelectAnim=Select
	PutDownAnim=Down
	ScopeDownAnim=None

	IdleAnimRate=1.0
	RestAnimRate=0.0
	AimAnimRate=0.0
	RunAnimRate=0.0
	SelectAnimRate=1.5
	PutDownAnimRate=1.5

	BringUpTime=0.33

	// other useful stuff
	ReloadTime=1.0
	MessageNoAmmo=" has no ammo"
	WeaponEmptySound=Sound'FOWeaponSounds.Misc.EmptySound1'
	FireSwitchSound=Sound'FOWeaponSounds.Misc.FireTypeClick'

	BurstMultiple=1.2
	DisplayFOV=90.0

	bNoAmmoInstances=false
	bReplicateInstigator=true
	bCanThrow=true
	bIsReloading=false
	bPlayIdleEmpty=false
	bWeaponOwnsCross=false
	bScopeZoomed=false
	bDontSwitchNow=false
	bCanBeBought=true

	AIRating=0.5
	CurrentRating=0.5

	AttachmentClass=class'WeaponAttachment'
	WeaponIcon=Material'FOMisc.WeaponHUD.WeapHUD_uspbare'

	bDynamicLight=false
	ScaleGlow=1.0
	AmbientGlow=11
	MaxLights=5
	SoundVolume=255
	HudColor=(r=255,g=255,b=0,a=255)
	
	CustomCrosshair=-1
	CustomCrossHairColor=(R=255,G=255,B=255,A=255)
	CustomCrossHairScale=+1.0
	bNetNotify=false
	NetUpdateFrequency=40
}