//=============================================================================
// Frag.Ops  2003 Matt 'SquirrelZero' Farber
//=============================================================================
#exec OBJ LOAD FILE=dsrtextures.utx

class Weapon_DSRBase extends Weapon_Base;

var vector ScopeLocation;
var float MovementThreshold;
var float VelocityThreshold;

replication
{
	reliable if (ROLE == ROLE_Authority)
		ClientForceUnzoom;
}

simulated function ClientForceUnzoom(optional bool bFireForced)
{
	if (!bScopeZoomed)
		return;

	bScopeZoomed = false;
	CurrentFireType = 1;

	if (!bFireForced)
	{
		SetScope();
			PlayOwnedSound(Sound'FOWeaponSounds.Misc.ZoomClick',SLOT_Pain,TransientSoundVolume,,,,false);
	}
	else
		FOPlayer(Instigator.Controller).EndZoom();
}

simulated function WeaponTick(float deltatime)
{
	Super.WeaponTick(deltatime);
	
	if (ZoomLevel == 0 || ROLE < ROLE_Authority)
		return;
	CheckUnzoom();
}

function ServerSetZoom(int i)
{
	if (i > 0)
		ScopeLocation = Instigator.Location;
	if (Instigator != None && ((ZoomLevel == 0 && i > 0) || (ZoomLevel > 0 && i == 0)))
		FOPawn(Instigator).StopAction();
	ZoomLevel = i;
}

function CheckUnZoom()
{
	if ((Instigator.Location.X < ScopeLocation.X - MovementThreshold || Instigator.Location.X > ScopeLocation.X + MovementThreshold) || (Instigator.Location.Y < ScopeLocation.Y - MovementThreshold || Instigator.Location.Y > ScopeLocation.Y + MovementThreshold) || (Instigator.Location.Z < ScopeLocation.Z - MovementThreshold || Instigator.Location.Z > ScopeLocation.Z + MovementThreshold))
	{
		if (Instigator.IsInState('PlayerSprinting') || Abs(Instigator.Velocity.Z) > 40 || (Instigator.Velocity.X > VelocityThreshold || Instigator.Velocity.X < -VelocityThreshold) || (Instigator.Velocity.Y > VelocityThreshold || Instigator.Velocity.Y < -VelocityThreshold))
		{
			ServerSetZoom(0);
			ClientForceUnzoom();
		}
		else
			ScopeLocation = Instigator.Location;
	}
}

simulated event RenderOverlays( Canvas Canvas )
{
	if (bScopeZoomed && !IsReloading() && !FragOps_GRI(PlayerController(Instigator.Controller).GameReplicationInfo).bPreRound)
	{
		Canvas.SetPos(0, 0);
		Canvas.DrawTile(ScopeHUDTex, Canvas.ClipX, Canvas.ClipY, 0, 0, 1024, 1024);
		SetLocation(Instigator.Location);
	}
	else
	{
		if (bScopeZoomed)
		{
			ServerSetZoom(0);
			bScopeZoomed = false;
			SetScope();
			CurrentFireType = 1;
		}
		Super.RenderOverlays(Canvas);
	}
}

simulated function SwitchFireType(optional int i)
{
	local int TempFireType;
	
	if (NumFireTypes == 0 || !bScopeZoomed)
		return;

	if (i > 0)
	{
		TempFireType = CurrentFireType - 1;
		if (TempFireType >= 1)
		{
			SetFireType(TempFireType);
			PlayOwnedSound(FireSwitchSound,,TransientSoundVolume*0.95,,30,,false);
		}
	}
	else
	{
		TempFireType = CurrentFireType + 1;
		if (TempFireType <= NumFireTypes)
		{
			SetFireType(TempFireType);
			PlayOwnedSound(FireSwitchSound,,TransientSoundVolume*0.95,,30,,false);
		}
	}
}

simulated function SetFireType(int i)
{
	CurrentFireType = i;

	ServerSetZoom(i);

	if (CurrentFireType == 1)
	{
		FOPlayer(Instigator.Controller).ZoomLevel = Default.ZoomMultiple;
	}
	else if (CurrentFireType == 2)
	{
		FOPlayer(Instigator.Controller).ZoomLevel = Default.ZoomMultiple * SniperZoomLevels[0];
	}
	else if (CurrentFireType == 3)
	{
		FOPlayer(Instigator.Controller).ZoomLevel = Default.ZoomMultiple * SniperZoomLevels[1];
	}
}

simulated function bool PutDown()
{
	if (Instigator.Controller.IsA('PlayerController') && Instigator.IsLocallyControlled())
	{
		ServerSetZoom(0);
		if (bScopeZoomed)
		{
			bScopeZoomed = false;		
			SetScope();
		}
		CurrentFireType = 1;
	}
	if (Super.PutDown())
		return true;

	return false;
}

simulated function ClientWeaponThrown()
{
	if ((Instigator != None) && (Instigator.Controller.IsA('PlayerController')))
	{
		bScopeZoomed = false;
		SetScope();
	}
	Super.ClientWeaponThrown();
}

simulated function ClientStartFire(int mode)
{
	if (Instigator.Controller.IsInState('PreRound') || IsReloading() || Instigator.Controller.IsInState('GameEnded'))
		return;
	if (mode == 1)
	{
		if (Instigator.HeadVolume != None && Instigator.HeadVolume.bWaterVolume)
			return;
		if ((VSize(Instigator.Velocity) > 150) || (Instigator.Velocity.Z > VelocityThreshold || Instigator.Velocity.Z < -VelocityThreshold) || (Instigator.Velocity.X > VelocityThreshold || Instigator.Velocity.X < -VelocityThreshold) || (Instigator.Velocity.Y > VelocityThreshold || Instigator.Velocity.Y < -VelocityThreshold))
			return;
		if (Instigator.Controller.IsA('PlayerController'))
		{
			FireMode[mode].bIsFiring = true;
			Instigator.Controller.bFire = 0;
			Instigator.Controller.bAltFire = 0;
			bScopeZoomed = !bScopeZoomed;
			CurrentFireType = 1;
			SetScope();
			if (bScopeZoomed)
				ServerSetZoom(1);
			else
				ServerSetZoom(0);
			PlayOwnedSound(Sound'FOWeaponSounds.Misc.ZoomClick',SLOT_Pain,TransientSoundVolume,,,,false);
		}
	}
	else
		Super.ClientStartFire(mode);
}

simulated function ClientStopFire(int mode)
{
	if (mode == 1)
		FireMode[mode].bIsFiring = false;
	else
		Super.ClientStopFire(mode);
}

defaultproperties
{
	// weapon data, readable names, attachments
	ItemName="STF-U30"
	StaticName="STF-U30"
	InventoryGroup=3
	WeaponType=Sniper
	WeaponTurnLimit=650
	GunDesc="COST: $300|AMMO: 30 x 4|DMG: High|RNG: High"
	GunCost=1600

	// CXP reqs
	CXPReq=120

	// firemodes, types
	NumFireTypes=3
	FireTypeNames(0)="2X Zoom"
	FireTypeNames(1)="5X Zoom"
	FireTypeNames(2)="7X Zoom"
	FireModeClass(0)=Weapon_DSRFire
	FireModeClass(1)=Weapon_DSRScopeFire

	// AI
	AIRating=+0.3
	CurrentRating=0.3
	Priority=3

	// reload, bob, burst timings
	ReloadTime=3.06
	MasterBreathScale=0.7
	BobDamping=1.94
	ZoomMultiple=0.7
	MovementThreshold=40.0
	VelocityThreshold=160.0

	// sniper zooms
	SniperZoomLevels(0)=1.17
	SniperZoomLevels(1)=1.3

	// sound
	SelectSound=Sound'FOWeaponSounds.SelectSounds.SelectPrimary'

	// 3rd person
	PickupClass=class'FragOps.Weapon_DSRPickup'
	AttachmentClass=class'FragOps.Weapon_DSRAttachment'
	TracerBone=Bone_Flash
	ShellBone=Bone_Shell
	SmokeBone=Bone_Smoke

	// 1st person
	Mesh=mesh'FORifles.DSR'

	DisplayFOV=65
	DrawScale=0.78
	PlayerViewOffset=(X=3.3,Y=-7.5,Z=3.2)
	PlayerViewPivot=(Pitch=100,Roll=-1000,Yaw=17750)

	IdleAnim=Idle
	RestAnim=Idle
	AimAnim=Idle
	RunAnim=Idle
	SelectAnim=Select
	PutDownAnim=Deselect
	SelectAnimRate=1.0
	PutDownAnimRate=1.3
	ScopeDownAnim=ScopeOut
	ScopeDownAnimRate=1.0
	SelectForce="SwitchToAssaultRifle"

	ScopeHUDTex=FinalBlend'FOMisc.HUD.dsrscope_fb'
	WeaponIcon=Material'FOMisc.WeaponHUD.WeapHUD_dsr'
	bWeaponOwnsCross=true

	LightType=LT_Pulse
	LightEffect=LE_NonIncidence
	LightPeriod=2
	LightBrightness=170
	LightHue=30
	LightSaturation=150
	LightRadius=6.0

	// buymenu offset
	BuyMenuOffset=(X=0,Y=2,Z=0)

	BringUpTime=0.6
}