//=============================================================================
// Frag.Ops  2004 Matt 'SquirrelZero' Farber
//=============================================================================
class Weapon_FistsSwing extends Weapon_MacheteSwing;

event ModeDoFire()
{
	if ((Level.TimeSeconds - LastFireTime) > 0.2)
	{
		Spread = Default.Spread;
		NumFireTimes = 0;
	}
	else
	{
		NumFireTimes++;
		Spread += NumFireTimes * ConeMultiPlier;
		Spread = FMin(Spread, ConeMax);
	}

	if (!AllowFire())
		return;

	if (MaxHoldTime > 0.0)
		HoldTime = FMin(HoldTime, MaxHoldTime);

	// if jabbing, fire rate is faster, damager is lower
	if (FOPlayer(Instigator.Controller).SlashDirection == 0)
	{
		WeaponDamage = 0.55 * Default.WeaponDamage;
		FireRate = 0.55 * Default.FireRate;
		FireLoopAnimRate = 1.4 * Default.FireLoopAnimRate;
		FireAnimRate = 1.4 * Default.FireLoopAnimRate;
	}
	else
	{
		WeaponDamage = Default.WeaponDamage;
		FireRate = Default.FireRate;
		FireLoopAnimRate = Default.FireLoopAnimRate;
		FireAnimRate = Default.FireLoopAnimRate;
	}
	// server
	if (Weapon.Role == ROLE_Authority)
	{
		DoFireEffect();
		if ((Instigator == None) || (Instigator.Controller == None))
			return;
		if (AIController(Instigator.Controller) != None)
			AIController(Instigator.Controller).WeaponFireAgain(BotRefireRate, true);
		Instigator.SpawnTime = -100000;
	}
	// client
	if (Instigator.IsLocallyControlled())
	{
		ShakeView();
		PlayFiring();
		FlashMuzzleFlash();
	}

	Weapon.IncrementFlashCount(ThisModeNum);

	// set the next firing time. must be careful here so client and server do not get out of sync
	if (bFireOnRelease)
	{
	        if (bIsFiring)
			NextFireTime += MaxHoldTime + FireRate;
		else
			NextFireTime = Level.TimeSeconds + FireRate;
	}
	else
	{
		NextFireTime += FireRate;
		NextFireTime = FMax(NextFireTime, Level.TimeSeconds);
	}

	HoldTime = 0;

	if (Instigator.PendingWeapon != Weapon && Instigator.PendingWeapon != None)
	{
		bIsFiring = false;
		Weapon.PutDown();
	}
	LastFireTime = Level.TimeSeconds;
}

function PlayFiring()
{
	local float RandJab;

	if (Instigator != None && Instigator.Controller != None && Instigator.IsHumanControlled())
	{
		switch(FOPlayer(Instigator.Controller).SlashDirection)
		{
			case 0:
				RandJab = FRand();
				if (FRand() < 0.5)
				{
					if (RandJab < 0.33)
					{
						FireAnim = 'LeftJab1';
						FireLoopAnim = 'LeftJab1';
					}
					else if (RandJab < 0.66)
					{
						FireAnim = 'LeftJab2';
						FireLoopAnim = 'LeftJab2';
					}
					else
					{
						FireAnim = 'LeftJab3';
						FireLoopAnim = 'LeftJab3';
					}
					FireSound = sound'FOWeaponSounds.Machete.OverHead';
				}
				else
				{
					if (RandJab < 0.33)
					{
						FireAnim = 'RightJab1';
						FireLoopAnim = 'RightJab1';
					}
					else if (RandJab < 0.66)
					{
						FireAnim = 'RightJab2';
						FireLoopAnim = 'RightJab2';
					}
					else
					{
						FireAnim = 'RightJab3';
						FireLoopAnim = 'RightJab3';
					}
					FireSound = sound'FOWeaponSounds.Machete.OverHead';
				}
				break;

			// left slash
			case 1:
				FireAnim = 'RightPunch';
				FireLoopAnim = 'RightPunch';
				FireSound = sound'FOWeaponSounds.Machete.LeftSlash';
				break;

			// right slash
			case 2:
				FireAnim = 'LeftPunch';
				FireLoopAnim = 'LeftPunch';
				FireSound = sound'FOWeaponSounds.Machete.RightSlash';
				break;
		}
	}
	Super(FireMode_Base).PlayFiring();
}

function PlayFireSound()
{
	Weapon.PlayOwnedSound(FireSound,SLOT_Interact,TransientSoundVolume*0.25,,,2.0,false);
}

// nice shuddering vibration when hitting
function HitObject()
{
	if (FOPlayer(Instigator.Controller) != None)
		FOPlayer(Instigator.Controller).ClientShake(vect(150,120,120),vect(3000,2000,2000),1.0, vect(8,6,6),vect(3000,2000,2000),1.0);
}

function ClashHitEffect(Vector Start, Rotator Dir, Vector HitLocation, Vector HitNormal)
{
	local Effect_FistClashHit ClashHit;

	ClashHit = Spawn(class'Effect_FistClashHit',,, Start, Dir);
	ClashHit.ImpactAt(HitLocation, HitNormal, None, Instigator, DamageType);
}

defaultproperties
{
	AmmoClass=class'Ammo_Fists'
	DamageType=class'FragOps.DamType_Fists'
	FlashEmitterClass=None
	WeaponDamage=8
	WeaponRange=45   // let the knife stab just slightly in front of the player
	DeadBulletRange=45    // we don't want to do damage reduction here
	FireLoopAnimRate=1.0
	FireAnimRate=1.0
	FireRate=0.5

	MaxIntervals=3

	// blade offsets -- forward slash
	StabIntervals(0)=0.06
	StabIntervals(1)=0.03
	BladeTraceOffset(0)=(Y=0.0,X=0.0,Z=0.0)
	BladeTraceOffset(1)=(Y=0.0,X=0.0,Z=1.0)

	// blade offsets -- left slash
	StabIntervals(4)=0.02
	StabIntervals(5)=0.02
	BladeTraceOffset(4)=(Y=-2.0,X=0.0,Z=1.5)
	BladeTraceOffset(5)=(Y=0.0,X=0.0,Z=2.5)

	// blade offsets -- right slash
	StabIntervals(8)=0.05
	StabIntervals(9)=0.02
	BladeTraceOffset(8)=(Y=2.0,X=0.0,Z=1.5)
	BladeTraceOffset(9)=(Y=0.0,X=0.0,Z=2.5)

	// puff of smoke, thats it
	BulletHitEffectClass=class'Effect_FistsHit'

	bWaitForRelease=true
}