//=============================================================================
// Frag.Ops  2003 Matt 'SquirrelZero' Farber
//=============================================================================
// Base grenade class.
//=============================================================================
class Weapon_GrenadeBase extends Weapon_Base
	HideDropDown
	CacheExempt;

var bool bShowCharge, bForceStay, bWasPrimed, bPrimed, bThrown;
var float LastHeldFireMode, mWaitTime, PreThrowTime, PrimeTime, TimeHeld, HoldStartTime;
var vector StartVector;
var rotator StartDir;

// current mode primed
var int PrimedMode;

// anims
var() name PrimeAnim;
var() float PrimeAnimRate;

function ThrowGrenade();

replication
{
	reliable if (Role == ROLE_Authority)
		KillGrenade;
}

simulated event RenderOverlays( Canvas Canvas )
{
	local Interface_MainHUD myTempHUD;
	local float HeldFireMode;

	Super.RenderOverlays(Canvas);

	HeldFireMode = FireMode[0].HoldTime;
	if (HeldFireMode <= 0.0)
		HeldFireMode = FireMode[1].HoldTime;

	if (bShowCharge && FOPlayer(Instigator.Controller) != None)
	{
		if (HeldFireMode <= 0.0)
			HeldFireMode = LastHeldFireMode;
		myTempHUD = Interface_MainHUD(PlayerController(Instigator.Controller).myHUD);
		myTempHUD.GrenadeChargeTemp = myTempHUD.GrenadeCharge;
		myTempHUD.GrenadeChargeTemp.TextureCoords.Y1 = 128 - (128*(FMin(1.5, HeldFireMode)/1.5));
		myTempHUD.DrawSpriteWidget(Canvas, myTempHUD.GrenadeChargeTemp);
		if (myTempHUD.GrenadeChargeTemp.TextureCoords.Y1 <= 1)
			myTempHUD.DrawSpriteWidget(Canvas, myTempHUD.GrenadeMaxIcon);
	}
	LastHeldFireMode = HeldFireMode;
}

simulated function bool ReadyToFire(int Mode)
{
	return Super(Weapon).ReadyToFire(Mode);
}

event ServerStartFire(byte Mode)
{
	Super(Weapon).ServerStartFire(Mode);
}

simulated function PostBeginPlay()
{
	local int m;
	
	Super.PostBeginPlay();

	for (m=0;m<NUM_FIRE_MODES;m++)
		Weapon_GrenadeThrow(FireMode[m]).mGrenade = self;
}

simulated function StopThrowing()
{
	local int m;

	if (!bPrimed)
		return;

	for (m = 0; m < NUM_FIRE_MODES; m++)
	{
		FireMode[m].bIsFiring = false;
		bPrimed = false;
	}
	if (ROLE == ROLE_Authority)
	{
		if (Ammo[0].AmmoAmount < 1)
			Ammo[0].AmmoAmount += 1;
		FOPawn(Instigator).EquipList[InventoryGroup-1] = Class;		
		GotoState('');
		GotoState('');
	}
	if (Instigator.IsLocallyControlled())
	{
		bPlayIdleEmpty = false;
		bForceStay = false;
		bShowCharge = false;
		FOPlayer(Instigator.Controller).NewSwitchToBestWeapon();
	}
}
		
simulated function KillGrenade()
{
	AmbientSound = None;
	Instigator.AmbientSound = None;
	Instigator.DeleteInventory(self);
	FOPlayer(Instigator.Controller).ForceBestWeapon();
}

function ServerDestroyWeapon()
{
	local int m;

	FOPawn(Instigator).EquipList[InventoryGroup-1] = None;
	for (m = 0; m < NUM_FIRE_MODES; m++)
	{
		if (FireMode[m].bIsFiring)
			StopFire(m);
	}
	KillGrenade();
	if ( Instigator != None )
	{
		DetachFromPawn(Instigator);
	}
	Destroy();
}

function HolderDied()
{
	local int m;
	local Weapon_GrenadeProjectile g;

	for (m = 0; m < NUM_FIRE_MODES; m++)
	{
		if (bPrimed)
		{
			g = Spawn(class<Weapon_GrenadeProjectile>(ProjectileFire_Base(FireMode[m]).ProjectileClass),None,,Instigator.Location+Instigator.EyePosition(), Instigator.GetViewRotation());
			if (g != None)
			{
				g.Instigator = Instigator;
				g.Speed = 25;
				g.Velocity = g.Speed * vector(Instigator.GetViewRotation());
				bWasPrimed = true;
			}
			break;
		}
	}
}

function DropFrom(vector StartLocation)
{
	local Pickup Pickup;

	if (!bCanThrow)
		return;

	if (Ammo[0].AmmoAmount > 1)
		Ammo[0].AmmoAmount = 1;

	FOPawn(Instigator).EquipList[InventoryGroup-1] = None;
	ClientWeaponThrown();

	if (Instigator != None)
	{
		DetachFromPawn(Instigator);
	}

	if (bWasPrimed)
	{
		Destroy();
		return;
	}

	Pickup = Spawn(PickupClass,,, StartLocation);
	if ( Pickup != None )
	{
		Pickup.InitDroppedPickupFor(self);
		Pickup.Velocity = Velocity;
		if (Instigator.Health > 0)
			WeaponPickup(Pickup).bThrown = true;
	}

	Destroy();
}

// hand out specialized ammo
function GiveAmmo(int m, WeaponPickup WP, bool bJustSpawned)
{
	local bool bJustSpawnedAmmo;
	local int addAmount;

	if (FireMode[m] != None && FireMode[m].AmmoClass != None)
	{
		Ammo[m] = Spawn(FireMode[m].AmmoClass, Instigator);
		bJustSpawnedAmmo = true;

		if ((WP != None) && (WP.AmmoAmount[m] > 0))
			addAmount = WP.AmmoAmount[m];
		else
			addAmount = Ammo[m].InitialAmount;

		Ammo[m].AddAmmo(addAmount);
		Ammo_Base(Ammo[m]).NumClips = 0;
		Ammo[m].GotoState('');
	}
}

// grenade states

// prethrow
state DoPreThrow
{
	function BeginState()
	{
	}

Begin:
	TimeHeld = Level.TimeSeconds - HoldStartTime;
	if (TimeHeld < Default.PrimeTime)
	{
		Sleep((PrimeTime-TimeHeld));
		GotoState('DoThrow','Begin');
	}
	else
		GotoState('DoThrow','Begin');
}

// throw
state DoThrow
{
	function ThrowGrenade()
	{
		local Weapon_GrenadeProjectile g;
		local vector X, Y, Z, StartProj, StartTrace, HitLocation, HitNormal;
		local float AddMoreSpeed;
		local rotator Aim; 
		local float PlayerSpeed;
		local Actor Other;

		GetViewAxes(X,Y,Z);

		StartTrace = Instigator.Location + Instigator.EyePosition();
		StartProj = StartTrace + X*Weapon_GrenadeThrow(FireMode[PrimedMode]).ProjSpawnOffset.X;
		StartProj = StartProj + Hand*Y * Weapon_GrenadeThrow(FireMode[PrimedMode]).ProjSpawnOffset.Y + Z*Weapon_GrenadeThrow(FireMode[PrimedMode]).ProjSpawnOffset.Z;

		// check if projectile would spawn through a wall and adjust start location accordingly
		Other = Trace(HitLocation, HitNormal, StartProj, StartTrace, false);

		if (Other != None)
			StartProj = HitLocation + (4*HitNormal);
	    
		if (FOPlayer(Instigator.Controller) != None)
			Aim = rotator(vector(Instigator.Controller.Rotation)+VRand()*FRand()*FOPawn(Instigator).GrenadeConeMod);
		else
			Aim = ProjectileFire_Base(FireMode[PrimedMode]).AdjustAim(StartProj, FireMode[PrimedMode].AimError);

		g = Spawn(class<Weapon_GrenadeProjectile>(ProjectileFire_Base(FireMode[PrimedMode]).ProjectileClass),None,,StartProj,Aim);
		g.Instigator = Instigator;
		if (g != None)
		{
			g.Instigator = Instigator;
			PlayerSpeed = X dot Instigator.Velocity;
			AddMoreSpeed = FMin(100.0, PlayerSpeed);
			    
			if (TimeHeld < 1.6 && TimeHeld > 0.0)
				AddMoreSpeed += TimeHeld * 39.0;
			else if (TimeHeld >= 1.5)
				AddMoreSpeed += 1.5 * 42.0;

			g.Speed = FOPawn(Instigator).ModifyThrow(g.Default.Speed) + AddMoreSpeed;
			g.Velocity = g.Speed * Vector(Aim);
		}
		FOPawn(Instigator).EquipList[InventoryGroup-1] = None;
		GotoState('FinalStage','Begin');
	}

	function BeginState()
	{
	}

Begin:
	Sleep(PreThrowTime);
	bPrimed = false;
	ThrowGrenade();
}

// finishing up
state FinalStage
{
	function BeginState()
	{}

	function EndState()
	{}

Begin:
	Sleep(0.03);
	ServerDestroyWeapon();
}

// make sure grenade is thrown at right time 
simulated function Timer()
{
	if (bPrimed)
		PlayThrow();
	else if (bThrown)
	{
		bShowCharge = false;
		bForceStay = false;
	}
	else
		Super.Timer();
}


// play the throw, local only
simulated function PlayThrow()
{
	bPrimed = false;
	bThrown = true;
	PlayAnim(FireMode[PrimedMode].FireAnim, FireMode[PrimedMode].FireAnimRate, 0.0);

	//do we need a fire sound?  Maybe, maybe not.  Force will be implemented later.
	//if (Level.NetMode != NM_Client)
		//Weapon.PlaySound(FireSound,SLOT_Interact,TransientSoundVolume,,512.0,,false);
            
	//ClientPlayForceFeedback(FireForce);  // jdf
	SetTimer(0.31, false);
	bPlayIdleEmpty = false;
}

defaultproperties
{
	InventoryGroup=4
	FireTypeNames(0)="Throw"
	WeaponType=Grenade

	mWaitTime=0.3
	PreThrowTime=0.16
	PrimeTime=0.5

	bCanThrow=true
	bPlayIdleEmpty=false

	SelectSound=Sound'FOWeaponSounds.Misc.SelectSound2'
	SelectForce="SwitchToAssaultRifle"

	BobDamping=1.94

	DisplayFOV=65
	DrawScale=0.78
	PlayerViewOffset=(X=-7.0,Y=-8.2,Z=3.7)
	PlayerViewPivot=(Pitch=250,Roll=-1200,Yaw=18100)

	IdleAnim=Idle
	IdleEmptyAnim=IdlePrimed
	RestAnim=Idle
	AimAnim=Idle
	RunAnim=Idle
	SelectAnim=Select
	PutDownAnim=Deselect
	SelectAnimRate=1.0
	PutDownAnimRate=1.2

	PrimeAnim=PullPin
	PrimeAnimRate=1.2

	AIRating=+0.05
	CurrentRating+0.05
	Priority=1

	LightType=LT_None
}