//=============================================================================
// Frag.Ops  2004 Matt 'SquirrelZero' Farber
//=============================================================================
class Weapon_GrenadeHEPickup extends FOWeaponPickup;

var class<Weapon_GrenadeProjectile> ProjClass;
var Controller InstigatorController;
var vector BoostVelocity, LastBoost;

replication
{
	reliable if (bNetDirty && (Role == ROLE_Authority))
		BoostVelocity;
}

function TakeDamage(int Damage, Pawn instigatedBy, Vector HitLocation, Vector Momentum, class<DamageType> damageType)
{
	local vector start, end, hl, HitNormal, VelVector;
	local actor A;

	if (DamageType.Name != 'DamType_AKBayonet' && DamageType.Name != 'DamType_Machete' && damageType.Name != 'DamType_MachetePoison')
		DamageTaken += Damage;

	// we want them to blow up if they take too much damage
	if (DamageTaken > 60)
	{
		// the person who destroyed it is now the explosion instigator
		if (InstigatedBy != None)
		{
			Instigator = InstigatedBy;
			if (InstigatedBy.Controller != None)
				InstigatorController = InstigatedBy.Controller;
		}
		else
			Instigator = None;

		GotoState('Exploding','Begin');
		return;
	}
	else
	{
		// find out if we're on the ground, otherwise just apply normal momentum
		start = Location - vect(0,0,1)*CollisionHeight;
		end = start - vect(0,0,6);
		A = Trace(hl,HitNormal,end,start,false);
		if (hl != Vect(0,0,0))
		{
			SetPhysics(PHYS_Falling);
			// we want it to bounce away from the bullet hit, but we need to add lift first
			VelVector = 55*(-(HitNormal - (Momentum*17)));

			// reset bounce
			bBounce = true;

			if (Level.NetMode == NM_DedicatedServer)
				BoostVelocity = VelVector;
			Velocity += VelVector;
		}
		else
		{
			if (Level.NetMode == NM_DedicatedServer)
				BoostVelocity = 200*Momentum;
			Velocity += 200*Momentum;
		}
		RandSpin(50000);
	}
}

function Explode(vector HitLocation, vector HitNormal)
{
	local Effect_ExplosionSpawner ExplosionSpawner;

	if (Role == ROLE_Authority)
	{
		NetUpdateTime = Level.TimeSeconds - 1;
		ShakeView();
		MakeNoise(1.0);
	}
	ExplosionSpawner = spawn(ProjClass.Default.ExplosionSpawnerClass, none,, Location, Rotation);
	ExplosionSpawner.DoExplosion(HitLocation, HitNormal);
	DelayedHurtRadius(ProjClass.Default.Damage, ProjClass.Default.DamageRadius, ProjClass.Default.MyDamageType, ProjClass.Default.MomentumTransfer, HitLocation);
	Destroy();
}

// hurtradius and delayedhurtradius, raided from the projectile class
simulated function HurtRadius(float DamageAmount, float DamageRadius, class<DamageType> DamageType, float Momentum, vector HitLocation)
{
	local actor Victims;
	local float damageScale, dist;
	local vector dir;

	if (bHurtEntry)
		return;

	bHurtEntry = true;
	foreach VisibleCollidingActors(class 'Actor', Victims, DamageRadius, HitLocation)
	{
		// don't let blast damage affect fluid - VisibleCollisingActors doesn't really work for them - jag
		if( (Victims != self) && (Victims.Role == ROLE_Authority) && !Victims.IsA('FluidSurfaceInfo') )
		{
			dir = Victims.Location - HitLocation;
			dist = FMax(1,VSize(dir));
			dir = dir/dist;
			damageScale = 1 - FMax(0,(dist - Victims.CollisionRadius)/DamageRadius);
			if (Instigator == None || Instigator.Controller == None)
				Victims.SetDelayedDamageInstigatorController(InstigatorController);
			Victims.TakeDamage
			(
				damageScale * DamageAmount,
				Instigator,
				Victims.Location - 0.5 * (Victims.CollisionHeight + Victims.CollisionRadius) * dir,
				(damageScale * Momentum * dir),
				DamageType
			);
			if (Vehicle(Victims) != None && Vehicle(Victims).Health > 0)
				Vehicle(Victims).DriverRadiusDamage(DamageAmount, DamageRadius, InstigatorController, DamageType, Momentum, HitLocation);

		}
	}

	bHurtEntry = false;
}

function ShakeView()
{
	local Controller C;
	local PlayerController PC;
	local float Dist, Scale;

	if (!ProjClass.Default.bShakeView)
		return;

	for (C=Level.ControllerList;C!=None;C=C.NextController)
	{
		PC = PlayerController(C);
		if (PC != None && PC.ViewTarget != None)
		{
			Dist = VSize(Location - PC.ViewTarget.Location);
			if (Dist < ProjClass.Default.DamageRadius * 2.0)
			{
				if (Dist < ProjClass.Default.DamageRadius)
					Scale = 1.0;
				else
					Scale = (ProjClass.Default.DamageRadius*2.0 - Dist) / (ProjClass.Default.DamageRadius);
				FOPlayer(PC).ClientShake(vect(350,350,350)*Scale,vect(9000,9000,9000),3.5, vect(10,10,10)*Scale,vect(2000,2000,2000),3.5);
			}
		}
	}
}


simulated function DelayedHurtRadius(float DamageAmount, float DamageRadius, class<DamageType> DamageType, float Momentum, vector HitLocation)
{
	HurtRadius(DamageAmount, DamageRadius, DamageType, Momentum, HitLocation);
}

simulated function PostNetReceive()
{
	if (BoostVelocity != LastBoost && BoostVelocity != vect(0,0,0))
	{
		// reset bounce
		bBounce = true;
		LastBoost = BoostVelocity;
		Velocity += BoostVelocity;
	}
}

state Exploding
{
	function BeginState();
	function EndState();

Begin:
	Sleep(0.05+(FRand()*0.5));
	Explode(Location,vect(0,0,1));
}

defaultproperties
{
	// explodable pickups
	ProjClass=class'Weapon_GrenadeHEProjectile'

	DrawType=DT_StaticMesh
	StaticMesh=StaticMesh'FOPickupSMs.sms.flnade_sm'
	
	InventoryType=class'FragOps.Weapon_GrenadeHE'
	PickupMessage="Picked up an HE Grenade"
	PickupSound=Sound'PickupSounds.AssaultRiflePickup'
	PickupForce="AssaultRiflePickup"  // jdf

	MaxDesireability=+0.4
	DrawScale=0.35

	CollisionHeight=+6.3
	CollisionRadius=+6.3
	bNetNotify=true
}