//=============================================================================
// Frag.Ops  2003 Matt 'SquirrelZero' Farber
//=============================================================================
class Weapon_GrenadeIncinProj extends Weapon_GrenadeProjectile;

// spew out a few napalm balls
function Explode(vector HitLocation, vector HitNormal)
{
	local vector NapalmStart;
	local rotator NapalmRotation;
	local int i;
	local Weapon_IncineratorNapalm NapalmBall;

	NapalmStart = Location + (10*vect(0,0,1));
	if (Role == ROLE_Authority)
	{
		for (i=0;i<5;i++)
		{
			NapalmRotation = Rotation;
			NapalmRotation.Yaw += FRand()*32000-16000;
			NapalmRotation.Pitch += FRand()*32000-16000;
			NapalmRotation.Roll += FRand()*32000-16000;
			NapalmBall = Spawn(class'Weapon_IncineratorNapalm',,, NapalmStart, NapalmRotation);
		}
	}
	Super.Explode(HitLocation, HitNormal);
}

simulated function HurtRadius( float DamageAmount, float DamageRadius, class<DamageType> DamageType, float Momentum, vector HitLocation )
{
	local actor Victims;
	local float damageScale, dist;
	local vector dir;

	if (bHurtEntry)
		return;

	bHurtEntry = true;
	foreach VisibleCollidingActors( class 'Actor', Victims, DamageRadius, HitLocation )
	{
		// don't let blast damage affect fluid - VisibleCollisingActors doesn't really work for them - jag
		if( (Victims != self) && (Hurtwall != Victims) && (Victims.Role == ROLE_Authority) && !Victims.IsA('FluidSurfaceInfo') )
		{
			dir = Victims.Location - HitLocation;
			dist = FMax(1,VSize(dir));
			dir = dir/dist;
			damageScale = 1 - FMax(0,(dist - Victims.CollisionRadius)/DamageRadius);
			if (Instigator == None || Instigator.Controller == None)
				Victims.SetDelayedDamageInstigatorController(InstigatorController);
			if (Victims == LastTouched )
				LastTouched = None;
			if (FOPawn(Instigator) != None)
				damageScale = FOPawn(Instigator).ModifyNapalmDamageDone(damagescale);
			if (FOPawn(Victims) != None)
				FOPawn(Victims).LastBone = 'spine';
			Victims.TakeDamage
			(
				damageScale * DamageAmount,
				Instigator,
				Victims.Location - 0.5 * (Victims.CollisionHeight + Victims.CollisionRadius) * dir,
				(damageScale * Momentum * dir),
				DamageType
			);
			if (Vehicle(Victims) != None && Vehicle(Victims).Health > 0)
				Vehicle(Victims).DriverRadiusDamage(DamageAmount, DamageRadius, InstigatorController, DamageType, Momentum, HitLocation);

		}
	}
	if ( (LastTouched != None) && (LastTouched != self) && (LastTouched.Role == ROLE_Authority) && !LastTouched.IsA('FluidSurfaceInfo') )
	{
		Victims = LastTouched;
		LastTouched = None;
		dir = Victims.Location - HitLocation;
		dist = FMax(1,VSize(dir));
		dir = dir/dist;
		damageScale = FMax(Victims.CollisionRadius/(Victims.CollisionRadius + Victims.CollisionHeight),1 - FMax(0,(dist - Victims.CollisionRadius)/DamageRadius));
		if (Instigator == None || Instigator.Controller == None)
			Victims.SetDelayedDamageInstigatorController(InstigatorController);
		if (FOPawn(Instigator) != None)
			damageScale = FOPawn(Instigator).ModifyNapalmDamageDone(damagescale);
		if (FOPawn(Victims) != None)
			FOPawn(Victims).LastBone = 'spine';
		Victims.TakeDamage
		(
			damageScale * DamageAmount,
			Instigator,
			Victims.Location - 0.5 * (Victims.CollisionHeight + Victims.CollisionRadius) * dir,
			(damageScale * Momentum * dir),
			DamageType
		);
		if (Vehicle(Victims) != None && Vehicle(Victims).Health > 0)
			Vehicle(Victims).DriverRadiusDamage(DamageAmount, DamageRadius, InstigatorController, DamageType, Momentum, HitLocation);
	}

	bHurtEntry = false;
}

simulated function HitWall( vector HitNormal, actor Wall )
{
	SetPhysics(PHYS_Falling);
	Velocity = 0.4 * (Velocity - 2.0*HitNormal*(Velocity dot HitNormal));
	RandSpin(50000);
	Speed = VSize(Velocity);

	if (Speed < 50)
		bBounce = false;
	else
	{
		if (Level.NetMode != NM_DedicatedServer)
		{
			if (Speed > 50)
				PlaySound(ImpactSound, SLOT_Misc,0.75,,450,1.15);
			if (!Level.bDropDetail && Level.DetailMode != DM_Low && Level.TimeSeconds - LastSparkTime > 0.5 && EffectIsRelevant(Location,false))
			{
				Spawn(HitEffectClass,,, Location, Rotator(HitNormal));
				LastSparkTime = Level.TimeSeconds;
			}
		}
	}
}

defaultproperties
{
	StaticMesh=StaticMesh'FOPickupSMs.sms.smokenade_sm'
	Skins(0)=Texture'FOPickupSkins.incinpickupskin'
	DamageRadius=50.0
	Damage=300
	MomentumTransfer=100.0
	ExplosionSpawnerClass=class'Effect_NapalmSpawner'
	MyDamageType=class'DamType_IncinGrenade'
	ImpactSound=Sound'FOWeaponSounds.Impact.NapalmImpact'
	GravityDampen=11.5
	DrawScale=1.4
}