//=============================================================================
// Frag.Ops  2003 Matt 'SquirrelZero' Farber
//=============================================================================
class Weapon_GrenadeSmokeProj extends Weapon_GrenadeProjectile;

var xEmitter Trail;
var Emitter SmokeGenerator;
var bool bTimerSet;

simulated function PostBeginPlay()
{
	local vector TossVel;

	Super(Projectile).PostBeginPlay();

	SetOwner(None);
	SetPhysics(PHYS_Falling);
	if (Level.NetMode != NM_DedicatedServer)
		Trail = Spawn(class'GrenadeSmokeTrail', self,, Location, Rotation);
	if (Role == ROLE_Authority)
	{
		TossVel = (Vector(Rotation) * (Speed));
		TossVel.Z += TossZ;
		Velocity = Speed * Vector(Rotation);
		RandSpin(50000);    
		if (Instigator.HeadVolume.bWaterVolume)
			Velocity = 0.6*Velocity;            
	} 
	if (Instigator != None)
		InstigatorController = Instigator.Controller;
	bTimerSet = false;
}

simulated function HitWall( vector HitNormal, actor Wall )
{
	// Reflect off Wall w/damping
	SetPhysics(PHYS_Falling);
	Velocity = 0.4 * (Velocity - 2.0*HitNormal*(Velocity dot HitNormal));
	Speed = VSize(Velocity);
	RandSpin(50000);

	if (Speed < 50 && !bTimerSet)
	{
		bBounce = false;
		if (Trail != None)
			Trail.mRegen = false; // stop the emitter from regenerating

		if (SmokeGenerator == None)
			SmokeGenerator = spawn(class'Effect_GrenadeSmokeGen',self,, Location, rotator(vect(0,0,1)));
		SetTimer(GrenExplodeTimer, false); 
		bTimerSet = true;
	}
	else
	{
		if (Level.NetMode != NM_DedicatedServer)
		{
			if (Speed > 50)
				PlaySound(ImpactSound,SLOT_Misc,0.7,,450,0.9);
			if (!Level.bDropDetail && Level.DetailMode != DM_Low && Level.TimeSeconds - LastSparkTime > 0.5 && EffectIsRelevant(Location,false) )
			{
				Spawn(HitEffectClass,,, Location, Rotator(HitNormal));
				LastSparkTime = Level.TimeSeconds;
			}
		}
	}
}

simulated function Explode(vector HitLocation, vector HitNormal)
{
	Destroy();
}

simulated function Destroyed()
{
	if (Trail != None)
		Trail.mRegen = false;
	if (SmokeGenerator != None)
	{
		SmokeGenerator.Emitters[0].ParticlesPerSecond = 0;
		SmokeGenerator.Emitters[0].RespawnDeadParticles = false;
		SmokeGenerator.Emitters[1].ParticlesPerSecond = 0;
		SmokeGenerator.Emitters[1].RespawnDeadParticles = false;
		SmokeGenerator.Emitters[2].ParticlesPerSecond = 0;
		SmokeGenerator.Emitters[2].RespawnDeadParticles = false;
		SmokeGenerator.SetTimer(0.0,false);
	}
	Super.Destroyed();
}

simulated function Reset()
{
	if (Trail != None)
		Trail.Destroy();
	if (SmokeGenerator != None)
		SmokeGenerator.Destroy();
	Destroy();
}

defaultproperties
{
	StaticMesh=StaticMesh'FOPickupSMs.sms.smokenade_sm'
	Skins(0)=Texture'FOPickupSkins.Weapons.RedSmokepickupskin'
	ImpactSound=Sound'WeaponSounds.P1GrenFloor1'
	GrenExplodeTimer=20.0
	RemoteRole=ROLE_SimulatedProxy
	LifeSpan=30.0
	bShakeView=false
	GravityDampen=10.2
	DrawScale=1.4
	MaxDamage=200
}