//=============================================================================
// Frag.Ops  2003 Matt 'SquirrelZero' Farber
//=============================================================================
class Weapon_IncineratorFlames extends Projectile;

var Effect_NapalmFlames NapalmFlames;

simulated function PostBeginPlay()
{
	Velocity = Vector(Rotation) * (Speed);
	if (PhysicsVolume.bWaterVolume)
		Velocity *= 0.65;

	SetRotation(RotRand());
	Super.PostBeginPlay();

	if (Role == ROLE_Authority)
		SetTimer(0.05,false);
}

simulated function PostNetBeginPlay()
{
	if (Level.NetMode != NM_DedicatedServer)
		NapalmFlames = spawn(class'Effect_NapalmFlames',,, Location, rotator(vect(0,0,1)));
}

function Timer()
{
	BurnVictims();
	SetTimer(0.1,false);
}

function ProcessTouch(Actor Other, vector HitLocation)
{
	//BurnVictim(Other);
}

simulated function Landed(Vector HitNormal)
{
	bBounce = false;
	SetPhysics(PHYS_None);
}

simulated function HitWall(vector HitNormal, Actor Wall)
{
	bBounce = false;
	SetPhysics(PHYS_None);
}

function BurnVictim(Actor Victim)
{
	local float dist;
	local int NewDamage;

	if (FOPawn(Victim) != None)
		FOPawn(Victim).LastBone = Victim.GetClosestBone(Location, Victim.Location - Location, dist);
	NewDamage = Default.Damage;
	if (FOPawn(Instigator) != None)
		NewDamage = int(FOPawn(Instigator).ModifyNapalmDamageDone(Default.Damage));
	Victim.TakeDamage(NewDamage, Instigator, Location, vect(0,0,0), Default.MyDamageType);
}

function BurnVictims()
{
	local Actor Victims;
	local float dist;
	local int NewDamage;

	if( bHurtEntry )
		return;

	bHurtEntry = true;
	foreach VisibleCollidingActors(class'Actor', Victims, Default.DamageRadius, Location)
	{
		// don't let blast damage affect fluid - VisibleCollisingActors doesn't really work for them - jag
		if( (Victims != self) && (Victims.Role == ROLE_Authority) && (!Victims.IsA('FluidSurfaceInfo')))
		{
			if (MyDamageType.Default.bDelayedDamage && (Instigator == None) && (Pawn(Victims) != None) )
				Pawn(Victims).DelayedDamageInstigatorController = InstigatorController;
			if (FOPawn(Victims) != None)
				FOPawn(Victims).LastBone = Victims.GetClosestBone(Location, Victims.Location - Location, dist);
			NewDamage = Default.Damage;
			if (FOPawn(Instigator) != None)
				NewDamage = int(FOPawn(Instigator).ModifyNapalmDamageDone(Default.Damage));
			Victims.TakeDamage(Default.Damage, Instigator, Location, vect(0,0,0), Default.MyDamageType);
		} 
	}
	bHurtEntry = false;
}		

simulated function Destroyed()
{
	if (NapalmFlames != None)
		NapalmFlames.StopFlames();
}

function Reset()
{
	Destroy();
}

defaultproperties
{
	// display
	StaticMesh=StaticMesh'ParticleMeshes.Simple.ParticleSphere2'
	Texture=None
	Style=STY_Normal
	bHidden=true
	DrawType=DT_StaticMesh
	DrawScale=0.2
	ScaleGlow=1.0
	AmbientGlow=254

	CollisionRadius=10.0
	CollisionHeight=5.0

	// projectile properties
	Physics=PHYS_Falling
	Damage=7
	DamageRadius=72.0
	MyDamageType=class'DamType_Napalm'
	Speed=175.000000
	MaxSpeed=1000.000000
	MomentumTransfer=0.0
	bBounce=true

	LifeSpan=10.0
	bAlwaysRelevant=true
}