//=============================================================================
// Frag.Ops  2003 Matt 'SquirrelZero' Farber
//=============================================================================
class Weapon_IncineratorNapalm extends Projectile;

simulated function PostBeginPlay()
{
	Velocity = Vector(Rotation) * (Speed);
	Velocity.Z += 55;
	
	if (PhysicsVolume.bWaterVolume)
		Velocity *= 0.65;

	SetRotation(RotRand());
	Super.PostBeginPlay();
}

simulated function ProcessTouch(Actor Other, vector HitLocation)
{
	// dont allow other napalm balls to collide with this one
	if (Weapon_IncineratorNapalm(Other) != None)
		return;

	HitWall(HitLocation, Other);
}

simulated function Landed(Vector HitNormal)
{
	HitWall(HitNormal, None);
}

simulated function HitWall(vector HitNormal, Actor Wall)
{
	Explode(vect(0,0,1), HitNormal);
}

simulated function Explode(vector HitLocation, vector HitNormal)
{
	if (PhysicsVolume.bWaterVolume)
	{
		Destroy();
		return;
	}

	if (Level.NetMode != NM_DedicatedServer)
	{
		if (FRand() < 0.25)
			spawn(class'Effect_FireGlow',,, Location, rotator(vect(0,0,1)));
		spawn(class'Effect_NapalmCore',,, Location, rotator(vect(0,0,1)));
	}
	if (Role == ROLE_Authority)
	{
		DelayedHurtRadius(Damage, DamageRadius, MyDamageType, MomentumTransfer, HitLocation);
		SpawnFlames();
		MakeNoise(0.3);
	}
	if (Level.NetMode != NM_DedicatedServer)
		PlaySound(sound'FOWeaponSounds.Napalm.NapalmSpread',,1.05*TransientSoundVolume,,300);
	Destroy();
}

function SpawnFlames()
{
	local vector FlameStart;
	local rotator FlameRotation;
	local int i;
	local Weapon_IncineratorFlames NapalmFlames;

	FlameStart = Location + (10*vect(0,0,1));
	if (Role == ROLE_Authority)
	{
		for (i=0;i<3;i++)
		{
			FlameRotation = Rotation;
			FlameRotation.Yaw += FRand()*32000-16000;
			FlameRotation.Pitch += FRand()*32000-16000;
			FlameRotation.Roll += FRand()*32000-16000;
			NapalmFlames = Spawn(class'Weapon_IncineratorFlames',,, FlameStart, FlameRotation);
		}
	}
}

simulated function HurtRadius( float DamageAmount, float DamageRadius, class<DamageType> DamageType, float Momentum, vector HitLocation )
{
	local actor Victims;
	local float damageScale, dist;
	local vector dir;

	if ( bHurtEntry )
		return;

	bHurtEntry = true;
	foreach VisibleCollidingActors( class 'Actor', Victims, DamageRadius, HitLocation )
	{
		// don't let blast damage affect fluid - VisibleCollisingActors doesn't really work for them - jag
		if( (Victims != self) && (Hurtwall != Victims) && (Victims.Role == ROLE_Authority) && !Victims.IsA('FluidSurfaceInfo') )
		{
			dir = Victims.Location - HitLocation;
			dist = FMax(1,VSize(dir));
			dir = dir/dist;
			damageScale = 1 - FMax(0,(dist - Victims.CollisionRadius)/DamageRadius);
			if ( Instigator == None || Instigator.Controller == None )
				Victims.SetDelayedDamageInstigatorController( InstigatorController );
			if ( Victims == LastTouched )
				LastTouched = None;
			if (FOPawn(Instigator) != None)
				damageScale = FOPawn(Instigator).ModifyNapalmDamageDone(damagescale);
			if (FOPawn(Victims) != None)
				FOPawn(Victims).LastBone = 'spine';
			Victims.TakeDamage
			(
				damageScale * DamageAmount,
				Instigator,
				Victims.Location - 0.5 * (Victims.CollisionHeight + Victims.CollisionRadius) * dir,
				(damageScale * Momentum * dir),
				DamageType
			);
			if (Vehicle(Victims) != None && Vehicle(Victims).Health > 0)
				Vehicle(Victims).DriverRadiusDamage(DamageAmount, DamageRadius, InstigatorController, DamageType, Momentum, HitLocation);

		}
	}
	if ( (LastTouched != None) && (LastTouched != self) && (LastTouched.Role == ROLE_Authority) && !LastTouched.IsA('FluidSurfaceInfo') )
	{
		Victims = LastTouched;
		LastTouched = None;
		dir = Victims.Location - HitLocation;
		dist = FMax(1,VSize(dir));
		dir = dir/dist;
		damageScale = FMax(Victims.CollisionRadius/(Victims.CollisionRadius + Victims.CollisionHeight),1 - FMax(0,(dist - Victims.CollisionRadius)/DamageRadius));
		if ( Instigator == None || Instigator.Controller == None )
			Victims.SetDelayedDamageInstigatorController(InstigatorController);
		if (FOPawn(Instigator) != None)
			damageScale = FOPawn(Instigator).ModifyNapalmDamageDone(damagescale);
		if (FOPawn(Victims) != None)
			FOPawn(Victims).LastBone = 'spine';
		Victims.TakeDamage
		(
			damageScale * DamageAmount,
			Instigator,
			Victims.Location - 0.5 * (Victims.CollisionHeight + Victims.CollisionRadius) * dir,
			(damageScale * Momentum * dir),
			DamageType
		);
		if (Vehicle(Victims) != None && Vehicle(Victims).Health > 0)
			Vehicle(Victims).DriverRadiusDamage(DamageAmount, DamageRadius, InstigatorController, DamageType, Momentum, HitLocation);
	}

	bHurtEntry = false;
}

function Reset()
{
	Destroy();
}

defaultproperties
{
	// display
	StaticMesh=StaticMesh'ParticleMeshes.Simple.ParticleSphere2'
	Texture=TexEnvMap'AWGlobal.Cubes.BriteEnv1'
	skins(0)=TexEnvMap'AWGlobal.Cubes.BriteEnv1'
	Style=STY_Normal
	DrawType=DT_StaticMesh
	DrawScale=0.02
	ScaleGlow=1.0
	AmbientGlow=254

	// projectile properties
	Physics=PHYS_Falling
	Damage=300
	DamageRadius=50
	MyDamageType=class'DamType_IncinGrenade'
	Speed=300.000000
	MaxSpeed=1000.000000
	MomentumTransfer=10.0
	bBounce=true

	CollisionRadius=3.0
	CollisionHeight=3.0

	LifeSpan=10.0
}