class Weapon_KnifeAkimbo extends Weapon_KnifeBase
	HideDropDown
	CacheExempt;

var bool bNoRender, bNoRenderSlave;
var actor AkimboThirdPersonActor;

simulated function IncrementFlashCount(int Mode)
{
	local int i;
	
	if (AkimboThirdPersonActor != None)
	{
		for (i=0;i<FOPawn(Instigator).SpectatorControllers.Length;i++)
		{
			if (FOPawn(Instigator).SpectatorControllers[i] != None)
			{
				if (bFiredLast)
					FOPawn(Instigator).SpectatorControllers[i].ClientAkimboFire(Mode);
				else
					FOPawn(Instigator).SpectatorControllers[i].ClientFire(Mode);
			}
		}
		xWeaponAttachment(AkimboThirdPersonActor).FlashCount++;
		NetUpdateTime = Level.TimeSeconds - 1;
		AkimboThirdPersonActor.NetUpdateTime = Level.TimeSeconds - 1;
		if (WeaponAttachment(ThirdPersonActor) != None)
		{
			if (Mode == 0)
				WeaponAttachment(ThirdPersonActor).FiringMode = 0;
			else
				WeaponAttachment(ThirdPersonActor).FiringMode = 1;
			ThirdPersonActor.NetUpdateTime = Level.TimeSeconds - 1;
			WeaponAttachment(ThirdPersonActor).FlashCount++;
			WeaponAttachment(ThirdPersonActor).ThirdPersonEffects();
		}
	}
	else
		Super.IncrementFlashCount(Mode);
}

function SpawnSlave(int AddedAmmo, int AddedClips, optional bool bWasBought, optional bool bAutoSwitch)
{
	if (Instigator.Weapon == self)
		AttachSlaveToPawn(Instigator);
	bIsAkimbo = true;
}

function KillSlave()
{
	bIsAkimbo = false;
	DetachSlaveFromPawn(Instigator);
}

function AttachSlaveToPawn(Pawn P)
{
	local name BoneName;
	local rotator TempRotator;

	if (AkimboThirdPersonActor != None || ThirdPersonActor == None)
		return;

	AkimboThirdPersonActor = Spawn(AkimboAttachmentClass,Owner);
	InventoryAttachment(AkimboThirdPersonActor).InitFor(self);

	BoneName = 'lefthand';
	P.AttachToBone(AkimboThirdPersonActor,BoneName);
	if (FOPawnFemale(P) != None)
		return;
	AkimboThirdPersonActor.SetRelativeLocation(vect(11,-1.8,-5.5));
	TempRotator.Yaw = -6000;
	TempRotator.Pitch = -1500;
	AkimboThirdPersonActor.SetRelativeRotation(TempRotator);

	// for dual muzzle flashes
	Attachment_Base(ThirdPersonActor).MasterAttachment = Attachment_Base(AkimboThirdPersonActor);
	Attachment_Base(AkimboThirdPersonActor).MasterAttachment = Attachment_Base(ThirdPersonActor);
}

function DetachSlaveFromPawn(Pawn P)
{
	if (AkimboThirdPersonActor != None)
	{
		AkimboThirdPersonActor.Destroy();
		AkimboThirdPersonActor = None;
	}
	FOPawn(Instigator).WeaponAttachment = xWeaponAttachment(ThirdPersonActor);
}

function DropFrom(vector StartLocation)
{
	local bool bJustDroppedSlave;
	
	if (bIsAkimbo)
	{
		DropSlave(StartLocation);
		bJustDroppedSlave = true;
	}
	if ((!bIsAkimbo && !bJustDroppedSlave) || Instigator == None || Instigator.Health <= 0)
		Super.DropFrom(StartLocation);
}

function DropSlave(vector StartLocation)
{
	local Pickup Pickup;

	if ((!bCanThrow && Instigator != None && Instigator.Health >= 0) || !Default.bCanThrow || ((Instigator == None || Instigator.Health <= 0) && FragOpsGame(Level.Game).PreRoundTime > 0))
		return;

	if (ZoomLevel > 0)
		ZoomLevel = 0;

	FOPawn(Instigator).AttachList[InventoryGroup-1] = None;

	if (Instigator != None)
		DetachSlaveFromPawn(Instigator);

	Pickup = Spawn(PickupClass,,, StartLocation);
	if (Pickup != None)
	{
		Pickup.InitDroppedPickupFor(self);
		Pickup.Velocity = Velocity;
		if (Instigator.Health > 0)
			WeaponPickup(Pickup).bThrown = true;
	}

	bIsAkimbo = false;
}

function AttachToPawn(Pawn P)
{
	Super.AttachToPawn(P);
	if (bIsAkimbo)
	{
		AttachSlaveToPawn(P);
	}
}

function DetachFromPawn(Pawn P)
{
	if (bIsAkimbo)
		DetachSlaveFromPawn(P);
	Super.DetachFromPawn(P);
}

function GiveTo(Pawn Other, optional Pickup Pickup)
{
	Super.GiveTo(Other,Pickup);
	if (FOPawn(Other).AttachList[InventoryGroup-1] != None)
		bIsAkimbo = true;
	if (bIsAkimbo && !bDontSwitchNow)
		AttachSlaveToPawn(Other);
}

simulated event RenderOverlays(Canvas Canvas)
{
	if (!bNoRender)
		Super.RenderOverlays(Canvas);
	if (SlaveWeapon != None)
	{
		if ((!SlaveWeapon.bNoRender || SlaveWeapon.GetAnimSeq() == PutDownAnim) && !bNoRenderSlave)
			SlaveWeapon.RenderOverlays(Canvas);
		SlaveWeapon.DrawSlaveInfo(Canvas);
	}
}

simulated function Destroyed()
{
	Super.Destroyed();
	if (SlaveWeapon != None)
		SlaveWeapon.Destroy();
	if (AkimboThirdPersonActor != None)
		AkimboThirdPersonActor.Destroy();
}

simulated function BringUp(optional Weapon PrevWeapon)
{
	local int Mode;

	if (ClientState == WS_Hidden)
	{
		PlayOwnedSound(SelectSound, SLOT_Interact,,,,, false);
		ClientPlayForceFeedback(SelectForce);  // jdf

		if (Instigator.IsLocallyControlled() || bDemoOwner)
		{
			// better way to tell the HUD we've switched guns
			if (FOBot(Instigator.Controller) == None)
			{
				Interface_MainHUD(FOPlayer(Instigator.Controller).MyHUD).CurWeaponName = GetHumanReadableName();
				Interface_MainHUD(FOPlayer(Instigator.Controller).MyHUD).CurInventoryGroup = InventoryGroup;
			}
			if (HasAnim(SelectAnim) && FOBot(Instigator.Controller) == None)
				PlayAnim(SelectAnim, SelectAnimRate, 0.0);
	        }

		ClientState = WS_BringUp;
		SetTimer(0.45, false);
	}
	NewBringUpTime = Level.TimeSeconds;
	bUp = true;
	for (Mode=0;Mode < NUM_FIRE_MODES;Mode++)
	{
		FireMode[Mode].bIsFiring = false;
		FireMode[Mode].HoldTime = 0.0;
		FireMode[Mode].bServerDelayStartFire = false;
		FireMode[Mode].bServerDelayStopFire = false;
		FireMode[Mode].bInstantStop = false;
	}
	if ( (PrevWeapon != None) && !PrevWeapon.bNoVoluntarySwitch )
		OldWeapon = PrevWeapon;
	else
		OldWeapon = None;

	// slave handling
	if (SlaveWeapon != None && !SlaveWeapon.bLatentBringUp && SlaveWeapon.GetAnimSeq() != SelectAnim)
	{
		SlaveWeapon.bNoRender = true;
		SlaveWeapon.BringUp();
	}
}

simulated function bool PutDown()
{
	if (!Super.PutDown())
		return false;

	if (SlaveWeapon != None)
	{
		SlaveWeapon.PutDown();
		SlaveWeapon.bNoRender = true;
	}

	return true;
}

simulated function bool HasAmmo()
{
	return true;
}

simulated function WeaponTick(float dt)
{
	Super.WeaponTick(dt);

	if (Level.NetMode == NM_DedicatedServer)
		return;
	
	if (bIsAkimbo && SlaveWeapon == None)
		SlaveWeapon = spawn(class'Weapon_KnifeSlave',self);
	else if (!bIsAkimbo && SlaveWeapon != None)
		SlaveWeapon.Destroy();
}	

simulated function bool ConsumeAmmo(int Mode, float load, optional bool bAmountNeededIsMax)
{
	return true;
}

simulated function bool CanThrow()
{
	return ((bIsAkimbo || bCanThrow) && (ClientState == WS_ReadyToFire || (Level.NetMode == NM_DedicatedServer) || (Level.NetMode == NM_ListenServer)));
}

simulated function bool IsAkimbo()
{
	return bIsAkimbo;
}

defaultproperties
{
	GunDesc="COST: $25|+RNG"
	GunCost=100
	ItemName="Combat Knife"
	AttachName="akimbo"
	BaseWeaponClass=class'Weapon_KnifeBase'
	PickupClass=class'FragOps.Weapon_KnifePickup'
	AttachmentClass=class'FragOps.Weapon_KnifeAttachment'
	AkimboAttachmentClass=class'Weapon_KnifeAkimboAttach'
	FireModeClass(0)=Weapon_KnifeAkimboSwing
	FireModeClass(1)=Weapon_KnifeStab
	AkimboIcon=Material'FOMisc.WeaponHUD.WeapHUD_knifeakimbo'
	bAkimbo=true
}