//=============================================================================
// Frag.Ops  2004 Matt 'SquirrelZero' Farber
//=============================================================================
class Weapon_KnifeBase extends Weapon_MacheteBase
	abstract;

var float LungeForce;
var bool bReceivedHitNotify, bAttemptedIdle;

replication
{
	reliable if (Role == ROLE_Authority)
		KillKnife;
}

state Lunging
{

	function BeginState();
	function EndState();

Begin:
	Sleep(0.17);
	DoJumpingEffect();
	Sleep(0.03);
	Weapon_KnifeStab(FireMode[1]).DoDelayedFireEffect();
}

simulated function AnimEnd(int channel)
{
	local name anim;
	local float frame, rate;

	GetAnimParams(0, anim, frame, rate);

	if (ClientState == WS_ReadyToFire)
	{
		if (anim == FireMode[1].FireAnim)
		{
			if (!bReceivedHitNotify)
				bAttemptedIdle = true;
			else
			{
				PlayIdle();
				bReceivedHitNotify = false;
			}
			return;
		}
		else if (anim == FireMode[0].FireAnim)
			PlayIdle();
		else if ((FireMode[0] == None || !FireMode[0].bIsFiring) && (FireMode[1] == None || !FireMode[1].bIsFiring))
		{
			PlayIdle();
		}
	}
}

function DoJumpingEffect()
{
	local vector LungeVelocity, X,Y,Z;
	local rotator LungeRot;

	if (Instigator.Physics != PHYS_Walking)
		return;

	GetViewAxes(X,Y,Z);
	LungeRot.Yaw = Instigator.GetViewRotation().Yaw;
	LungeForce = FClamp((Default.LungeForce-(Abs(Y dot Instigator.Velocity)*0.66))+(X dot Instigator.Velocity), 70, 340);
	LungeVelocity = LungeForce*vector(LungeRot);
	LungeVelocity.Z = 135;
	Instigator.AddVelocity(LungeVelocity);
	Instigator.PlaySound(FOPawn(Instigator).GetSound(EST_Jump), SLOT_Pain, FOPawn(Instigator).GruntVolume*0.7,,50);
}

function DestroyKnife(bool bWasHit)
{
	local int m;

	NetUpdateTime = Level.TimeSeconds - 1;
	KillKnife(bWasHit);

	if (!bWasHit)
		return;

	for (m = 0; m < NUM_FIRE_MODES; m++)
	{
		if (FireMode[m].bIsFiring)
			StopFire(m);
	}
	if (Instigator != None)
	{
		DetachFromPawn(Instigator);
		FOPawn(Instigator).GiveFists();
	}
	Destroy();
}


simulated function KillKnife(bool bWasHit)
{
	if (!bWasHit)
	{
		if (bAttemptedIdle)
			PlayIdle();
		bAttemptedIdle = false;
		bReceivedHitNotify = true;
		return;
	}
	AmbientSound = None;
	Instigator.AmbientSound = None;
	Instigator.DeleteInventory(self);
	FOPlayer(Instigator.Controller).ForceBestWeapon();
}

defaultproperties
{
	// weapon data, readable names, attachments
	ItemName="Combat Knife"
	StaticName="Combat Knife"
	GunAttachment[0]=None
	GunCost=0

	LungeForce=310

	// firemodes, types
	NumFireTypes=0
	FireTypeNames(0)="Melee"
	FireModeClass(0)=Weapon_KnifeSwing
	FireModeClass(1)=Weapon_KnifeStab

	// 3rd person
	PickupClass=class'Weapon_KnifePickup'
	AttachmentClass=class'Weapon_KnifeAttachment'

	// 1st person
	Mesh=mesh'FOMeleeWeapons.KaBar'
	SelectSound=Sound'FOWeaponSounds.SelectSounds.SelectMelee'

	WeaponIcon=Material'FOMisc.WeaponHUD.WeapHUD_knife'

	DisplayFOV=65
	DrawScale=0.78
	PlayerViewOffset=(X=-5.0,Y=1.3,Z=3.0)
	PlayerViewPivot=(Pitch=250,Roll=-1400,Yaw=16500)

	// buymenu offset
	BuyMenuOffset=(X=0,Y=-1,Z=1)
}