//=============================================================================
// Frag.Ops  2004 Matt 'SquirrelZero' Farber
//=============================================================================
class Weapon_LAWBase extends Weapon_DSRBase;

var bool bAutoLock, bPlayAimIdle, bStartedLock, bForceAimPos;
var Actor MissileTarget;
var float LAWReleaseTime;
var material LAWLockTex;
var color LAWLockColor;
var int TargetRange;

// animation
var() name CradleDownAnim;
var() float CradleDownAnimRate;

var() name CradleUpAnim;
var() float CradleUpAnimRate;

// the seeking rocket launcher made an excellent base for this.
// thanks a lot epic/DE.

replication
{
	reliable if (Role == ROLE_Authority)
		ClientKillLAW;

	reliable if (Role < ROLE_Authority)
		ServerSetTarget;
}

simulated function ClientKillLAW()
{
	local int m;

	AmbientSound = None;
	Instigator.AmbientSound = None;
	Instigator.DeleteInventory(self);

	for (m = 0; m < NUM_FIRE_MODES; m++)
	{
		if (Ammo[m] != None)
			Instigator.DeleteInventory(Ammo[m]);
	}
	FOPlayer(Instigator.Controller).ForceBestWeapon();
}

function DestroyLAW()
{
	local int m;

	for (m = 0; m < NUM_FIRE_MODES; m++)
	{
		if (FireMode[m].bIsFiring)
			StopFire(m);
	}
	ClientKillLAW();
	if ( Instigator != None )
	{
		DetachFromPawn(Instigator);
	}
	Destroy();
}

simulated function WeaponTick(float dt)
{
	local Actor Other;
	local vector StartTrace, End, XVec, HitNormal, HitLocation;
	local rotator Aim;
	local int TempRange;

	Super.WeaponTick(dt);

	if (bScopeZoomed)
	{
		StartTrace = Instigator.Location + Instigator.EyePosition();
		Aim = Instigator.GetViewRotation();
		XVec = vector(Aim);
		End = StartTrace + 7000 * XVec;
		Other = Trace(HitLocation,HitNormal,End,StartTrace,true);
		TempRange = int(VSize(Instigator.Location - HitLocation));
		if (TempRange <= 0)
			TempRange = 7000;
		TargetRange = TempRange;
		if (!CanLockOnTo(Other))
		{
			MissileTarget = None;
			return;
		}
		MissileTarget = Other;
	}
}

simulated function ClientForceUnzoom(optional bool bFireForced)
{
	if (!bScopeZoomed)
		return;
	MissileTarget = None;
	ClientStopFire(1);
}

function CheckUnZoom()
{
	if ((Instigator.Location.X < ScopeLocation.X - MovementThreshold || Instigator.Location.X > ScopeLocation.X + MovementThreshold) || (Instigator.Location.Y < ScopeLocation.Y - MovementThreshold || Instigator.Location.Y > ScopeLocation.Y + MovementThreshold) || (Instigator.Location.Z < ScopeLocation.Z - MovementThreshold || Instigator.Location.Z > ScopeLocation.Z + MovementThreshold))
	{
		if ((Instigator.Velocity.Z > 30 || Instigator.Velocity.Z < -30) || (Instigator.Velocity.X > VelocityThreshold || Instigator.Velocity.X < -VelocityThreshold) || (Instigator.Velocity.Y > VelocityThreshold || Instigator.Velocity.Y < -VelocityThreshold))
		{
			MissileTarget = None;
			ServerSetZoom(0);
			ClientForceUnzoom();
		}
		else
			ScopeLocation = Instigator.Location;
	}
}

function Projectile FireRocket(Vector Start, Rotator Dir)
{
	local Weapon_LAWProj LAWProj;
	local Weapon_LAWGuidedProj LAWGuidedProj;

	if (MissileTarget != None)
	{
		LAWGuidedProj = Spawn(class'Weapon_LAWGuidedProj',,, Start, Dir);
		LAWGuidedProj.Seeking = MissileTarget;
		return LAWGuidedProj;
	}
	else
	{
		LAWProj = Spawn(class'Weapon_LAWProj',,, Start, Dir);
		return LAWProj;
	}
	MissileTarget = None;
}

simulated function bool CanLockOnTo(Actor Other)
{
	if (Other == None)
		return false;

	if (Obj_LawObjective(Other) != None && Obj_LawObjective(Other).ObjectiveTeam == Instigator.PlayerReplicationInfo.Team.TeamIndex)
		return true;

	return false;
}

simulated event RenderOverlays( Canvas Canvas )
{
	local Interface_MainHUD myTempHUD;
	local bool bCradleUp;
	local int cuChannel;
	local float cuFrame, cuRate, XL, YL;
	local name cuAnim;
	
	if (PlayerController(Instigator.Controller) == None)
		return;

	myTempHUD = Interface_MainHUD(PlayerController(Instigator.Controller).myHUD);

	GetAnimParams(cuChannel,cuAnim,cuFrame,cuRate);
	bCradleUp = (((cuAnim == CradleUpAnim) && (cuFrame >= 0.66)) || (cuAnim != CradleUpAnim));

	if (Instigator.Controller.IsInState('PreRound'))
	{
		if (bScopeZoomed)
		{
			bScopeZoomed = false;
			CurrentFireType = 1;
			SetScope();
			ServerSetZoom(0);
		}
	}
	if (bStartedLock && bCradleUp)
	{
		SetLock();
		bStartedLock = false;
		PlaySound(Sound'FOWeaponSounds.Objectives.LAWZoomIn',SLOT_Interface,TransientSoundVolume*1.8,,30,,false);
	}
	if (bScopeZoomed && !bStartedLock)
	{
		if (bAutoLock && MissileTarget != None)
		{
			if (Level.TimeSeconds - LAWReleaseTime > 0.55)
			{
				ResetLockedScope();
				Super.RenderOverlays(Canvas);
				return;
			}
			myTempHUD.LAWZoomTex.TextureScale = fMax(0.34, myTempHUD.LAWZoomTex.TextureScale - 0.55);
			myTempHUD.DrawSpriteWidget(Canvas, myTempHUD.LAWZoomTex);
			myTempHUD.DrawSpriteWidget(Canvas, myTempHUD.LAWCenterIdleTex);
		}
		else if (MissileTarget != None)
		{
			myTempHUD.LAWCenterTex.Tints[0].R = 245;
			myTempHUD.LAWCenterTex.Tints[0].G = 20;
			myTempHUD.LAWCenterTex.Tints[0].B = 20;
			myTempHUD.LawCenterTex.Tints[1] = myTempHUD.LAWCenterTex.Tints[0];
			myTempHUD.DrawSpriteWidget(Canvas, myTempHUD.LAWCenterTex);
		}
		else
		{
			myTempHUD.LAWCenterTex.Tints[0].R = 245;
			myTempHUD.LAWCenterTex.Tints[0].G = 245;
			myTempHUD.LAWCenterTex.Tints[0].B = 245;
			myTempHUD.LawCenterTex.Tints[1] = myTempHUD.LAWCenterTex.Tints[0];
			myTempHUD.DrawSpriteWidget(Canvas, myTempHUD.LAWCenterTex);
		}
		myTempHUD.DrawSpriteWidget(Canvas, myTempHUD.LAWPointerTex);
		myTempHUD.DrawSpriteWidget(Canvas, myTempHUD.LAWAimerTex);
		Canvas.Font = myTempHUD.GetConsoleFont(Canvas);
		Canvas.StrLen(string(TargetRange),XL,YL);
		Canvas.SetPos((0.5 * Canvas.ClipX) - (0.5*XL),0.59 * Canvas.ClipY);
		Canvas.DrawColor = myTempHUD.Default.WhiteColor;
		Canvas.DrawText(TargetRange);
	}
	if (!bScopeZoomed && MissileTarget != None)
		DrawLockIndicator(Canvas);
	Super.RenderOverlays(Canvas);
}

// borrowed from hudmutant for now
simulated function DrawLockIndicator(Canvas C)
{
	local vector vecPawnView;
	local float fX, fY;
	local vector X, Y, Z, L, V;

	if (Instigator == None || Instigator.Weapon != self || FOPlayer(Instigator.Controller) == None)
		return;
	
	GetViewAxes(X,Y,Z);

	// Do we have a target?
	if (MissileTarget != None)
	{
		// Get a vector from the player to the pawn
		vecPawnView = (MissileTarget.Location+(MissileTarget.CollisionHeight*0.5)*vect(0,0,1)) - Instigator.Location - (Instigator.EyeHeight * vect(0,0,1));
		
		// Are they visible to the player
		if (TargetIsVisible(vecPawnView,X))
		{
			L = MissileTarget.Location + (vect(0,0,1) * MissileTarget.CollisionHeight); 						
			V = FOPlayer(Instigator.Controller).Player.Console.WorldToScreen(L, Instigator.Location + (Instigator.EyeHeight * Vect(0,0,1)), Instigator.Rotation);				
			fx = V.X;
			fy = V.Y;

			// Draw stuff!
			C.DrawColor = LAWLockColor;
			C.SetPos(fx - 0.125 * 256.0, fy - 0.125 * 256.0);
			C.DrawTile(LAWLockTex, 
				0.25 * 256.0, 
				0.25 * 256.0, 
				0.0,
				0.0,
				256.0,
				256.0);


		}
		else
		{
			TargetLost();
			ServerSetTarget(None);
		}
	}
}

simulated function TargetLost()
{
	MissileTarget = None;
	PlayAnim(CradleDownAnim, CradleDownAnimRate, 0.0);
}

function ServerSetTarget(Actor Target)
{
	MissileTarget = Target;
}

simulated function bool TargetIsVisible(vector vecPawnView, vector X)
{
	local vector StartTrace, EndTrace;

	if ((vecPawnView Dot X) <= 0.70)
		return false;
		
	StartTrace = Instigator.Location + Instigator.EyePosition();

	EndTrace = MissileTarget.Location + 250*Normal(StartTrace-MissileTarget.Location);
	EndTrace.Z += (MissileTarget.CollisionHeight*0.5);

	if (!FastTrace(EndTrace, StartTrace) || VSize(Instigator.Location - EndTrace) > 8000)
		return false;

	return true;		
}

simulated function PlayIdle()
{
	if (MissileTarget != None || bForceAimPos)
		LoopAnim(AimAnim, AimAnimRate, 0.2);
	else
		LoopAnim(IdleAnim, IdleAnimRate, 0.2);
}

simulated function BringUp(optional Weapon PrevWeapon)
{
	MissileTarget = None;
	bForceAimPos = false;
	bAutoLock = false;
	bStartedLock = false;
	Super.BringUp(PrevWeapon);
}

simulated function SwitchFireType(optional int i)
{
	local float TempFireType;
	
	if (!bScopeZoomed)
		return;

	if (i > 0)
	{
		TempFireType = FOPlayer(Instigator.Controller).ZoomLevel - 0.02;
		if (TempFireType >= ZoomMultiple)
		{
			PlaySound(Sound'FOWeaponSounds.Objectives.LAWZoomOut',SLOT_Interface,TransientSoundVolume*1.8,,30,,false);
			FOPlayer(Instigator.Controller).ZoomLevel -= 0.02;
		}
	}
	else
	{
		TempFireType = FOPlayer(Instigator.Controller).ZoomLevel + 0.02;
		if (TempFireType <= 1.1)
		{
			PlaySound(Sound'FOWeaponSounds.Objectives.LAWZoomIn',SLOT_Interface,TransientSoundVolume*1.8,,30,,false);
			FOPlayer(Instigator.Controller).ZoomLevel += 0.02;
		}
	}
}

simulated function ClientStartFire(int mode)
{
	if (Instigator.Controller.IsInState('PreRound') || IsReloading() || Instigator.Controller.IsInState('GameEnded'))
		return;
	if (mode == 1)
	{
		if ((Instigator.Velocity.Z > VelocityThreshold || Instigator.Velocity.Z < -VelocityThreshold) || (Instigator.Velocity.X > VelocityThreshold || Instigator.Velocity.X < -VelocityThreshold) || (Instigator.Velocity.Y > VelocityThreshold || Instigator.Velocity.Y < -VelocityThreshold) || bAutoLock)
			return;
		if (Instigator.Controller.IsA('PlayerController'))
		{
			FireMode[mode].bIsFiring = true;
			if (MissileTarget == None)
			{
				bStartedLock = true;
				PlayAnim(CradleUpAnim, CradleUpAnimRate, 0.0);
				return;
			}
			bStartedLock = false;
			SetLock();
		}
	}
	else
		Super.ClientStartFire(mode);
}

function ServerSetZoom(int i)
{
	if (i == 1)
		MissileTarget = None;
	Super.ServerSetZoom(i);
}

simulated function ClientStopFire(int mode)
{
	if (mode == 1 && FOPlayer(Instigator.Controller) != None)
	{
		FireMode[mode].bIsFiring = false;
		if (bStartedLock && MissileTarget == None)
		{
			bStartedLock = false;
			PlayAnim(CradleDownAnim, CradleDownAnimRate, 0.0);
		}
		if (MissileTarget != None)
		{
			ServerSetTarget(MissileTarget);
			bAutoLock = true;
			LAWReleaseTime = Level.TimeSeconds;
			Interface_MainHUD(PlayerController(Instigator.Controller).myHUD).LawZoomTex.TextureScale = 2.0;
			PlaySound(Sound'FOWeaponSounds.Objectives.LAWLock',SLOT_Interface,TransientSoundVolume*2.0,,,,false);
			return;
		}
		else
			PlaySound(Sound'FOWeaponSounds.Misc.ZoomClick',,TransientSoundVolume,,30,,false);
		Instigator.Controller.bFire = 0;
		bScopeZoomed = false;
		CurrentFireType = 1;
		SetScope();
		ServerSetZoom(0);
	}
	else
		Super.ClientStopFire(mode);
}

function ServerStopFire(byte Mode)
{
	if (Mode != 1)
	{
		Super.ServerStopFire(Mode);
		return;
	}
}

simulated function ResetLockedScope()
{
	bAutoLock = false;
	Instigator.Controller.bFire = 0;
	bScopeZoomed = false;
	CurrentFireType = 1;
	SetScope();
	ServerSetZoom(0);
	PlaySound(Sound'FOWeaponSounds.Misc.ZoomClick',,TransientSoundVolume,,,,false);
}

simulated function SetLock()
{
	Instigator.Controller.bFire = 0;
	bScopeZoomed = true;
	CurrentFireType = 1;
	SetScope();
	ServerSetZoom(1);
	PlaySound(Sound'FOWeaponSounds.Misc.ZoomClick',,TransientSoundVolume,,,,false);
}

simulated function PlayReload()
{
}

defaultproperties
{
	// weapon data, readable names, attachments
	ItemName="MBT LAW"
	StaticName="LAW"
	InventoryGroup=6
	WeaponType=Heavy
	GunCost=1
    
	// firemodes, types
	NumFireTypes=0
	FireTypeNames(0)="Target"
	FireModeClass(0)=Weapon_LAWFire
	FireModeClass(1)=Weapon_LAWScopeFire

	// AI
	AIRating=+0.1
	CurrentRating=0.1
	Priority=1
	
	// reload, bob, burst timings
	ReloadTime=2.4
	BobDamping=1.94

	// sound
	SelectSound=Sound'FOWeaponSounds.Misc.SelectPrimary'
	
	// 3rd person
	PickupClass=class'Weapon_LAWPickup'
	AttachmentClass=class'Weapon_LAWAttachment'
	TracerBone=Bone_Flash
	
	// 1st person
	Mesh=mesh'FOSpecialWeapons.LAW'

	DisplayFOV=65
	DrawScale=0.78
	PlayerViewOffset=(X=-31.0,Y=-1.1,Z=7.7)
	PlayerViewPivot=(Pitch=100,Roll=-1200,Yaw=17500)

	IdleAnim=CradleIdle
	RestAnim=Idle
	AimAnim=IdleAim
	RunAnim=Idle
	SelectAnim=Select
	PutDownAnim=DeselectCradle
	CradleDownAnim=CradleDown
	CradleUpAnim=CradleUp
	SelectAnimRate=1.0
	PutDownAnimRate=1.0
	CradleDownAnimRate=1.0
	CradleUpAnimRate=1.0
	SelectForce="SwitchToAssaultRifle"

	ScopeHUDTex=FinalBlend'FOMisc.HUD.LAWScope_fb'
	LAWLockTex=FinalBlend'FOMisc.HUD.LAWZoom_fb'
	bWeaponOwnsCross=true
	bNetNotify=true
	bCanBeBought=false
	LAWLockColor=(R=255,G=40,B=40,A=255)

	WeaponIcon=Material'FOMisc.WeaponHUD.WeapHUD_law'

	LightType=LT_Pulse
	LightEffect=LE_NonIncidence
	LightPeriod=3
	LightBrightness=170
	LightHue=30
	LightSaturation=170
	LightRadius=5.0
}