//=============================================================================
// Frag.Ops  2004 Matt 'SquirrelZero' Farber
//=============================================================================
class Weapon_LAWFire extends ProjectileFire_Base;

simulated function bool NewAllowFire()
{
	return (Weapon_LAWBase(Weapon).MissileTarget != None && Weapon.AmmoAmount(0) >= 1 && !Instigator.Controller.IsInState('PreRound') && !Instigator.Controller.IsInState('Building') && Instigator.Controller != None && ((Instigator.IsLocallyControlled() && !Weapon_Base(Weapon).bScopeZoomed) || (Weapon.Role == ROLE_Authority && Weapon_Base(Weapon).ZoomLevel <= 0)));
}

simulated function bool AllowFire()
{
	return (Weapon_LAWBase(Weapon).MissileTarget != None && Weapon.AmmoAmount(0) >= 1 && !Instigator.Controller.IsInState('PreRound') && !Instigator.Controller.IsInState('Building') && Instigator.Controller != None && ((Instigator.IsLocallyControlled() && !Weapon_Base(Weapon).bScopeZoomed) || (Weapon.Role == ROLE_Authority && Weapon_Base(Weapon).ZoomLevel <= 0)));
}

event ModeDoFire()
{
	if (AmmoPerFire <= 0 || !NewAllowFire())
		return;

	if (MaxHoldTime > 0.0)
		HoldTime = FMin(HoldTime, MaxHoldTime);
	// set the next firing time. must be careful here so client and server do not get out of sync
	if (bFireOnRelease)
	{
	        if (bIsFiring)
			NextFireTime += MaxHoldTime + FireRate;
		else
			NextFireTime = Level.TimeSeconds + FireRate;
	}
	else
	{
		NextFireTime += FireRate;
		NextFireTime = FMax(NextFireTime, Level.TimeSeconds);
	}
	Load = AmmoPerFire;
	Weapon.ConsumeAmmo(0, Load);
	if (Instigator.IsLocallyControlled() && Weapon_LAWBase(Weapon) != None)
	{
		Weapon_LAWBase(Weapon).bForceAimPos = true;
	}
	SetTimer(0.3,false);
}

simulated function Timer()
{
	if (Instigator.IsLocallyControlled() && Weapon_LAWBase(Weapon) != None)
	{
		Weapon_LAWBase(Weapon).bForceAimPos = false;
	}
	DelayedModeDoFire();
}

simulated function DelayedModeDoFire()
{
	// client
	if (Instigator.IsLocallyControlled())
	{
		ShakeView();
		PlayFiring();
		FlashMuzzleFlash();
		StartMuzzleSmoke();
	}

	Weapon.IncrementFlashCount(ThisModeNum);

	// server
	if (Weapon.Role == ROLE_Authority)
	{
		DoFireEffect();
		if ( (Instigator == None) || (Instigator.Controller == None) )
			return;
		if ( AIController(Instigator.Controller) != None )
			AIController(Instigator.Controller).WeaponFireAgain(BotRefireRate, true);
		Instigator.SpawnTime = -100000;
	}

	HoldTime = 0;
	if (Instigator.PendingWeapon != Weapon && Instigator.PendingWeapon != None)
	{
		bIsFiring = false;
		Weapon.PutDown();
	}
}

simulated function InitEffects()
{
	Super.InitEffects();
	if (NewMuzFlash != None )
		Weapon.AttachToBone(NewMuzFlash, 'Bone_Flash');
}

simulated function DestroyEffects()
{
	if (NewMuzFlash != None)
		NewMuzFlash.Destroy();
}

function Projectile SpawnProjectile(Vector Start, Rotator Dir)
{
	local Projectile p;
    
	p = Weapon_LAWBase(Weapon).FireRocket(Start, Dir);
	if (p != None)
		p.Damage *= DamageAtten;
	Weapon_LAWBase(Weapon).bAutoLock = false;
	return p;
}

function PlayPreFire()
{
	if (Weapon_LAWBase(Weapon).MissileTarget == None)
		return;
	
	if (Weapon_LAWBase(Weapon) != None && Weapon_LAWBase(Weapon).MissileTarget == None)
		Super.PlayPreFire();
	Weapon.PlaySound(Sound'FOWeaponSounds.Objectives.LAWRumble',SLOT_Interact,TransientSoundVolume*2.3,,,,false);
}

function PlayFireSound()
{
	Weapon.PlayOwnedSound(FireSound,SLOT_Interact,TransientSoundVolume*2.4,,,Default.FireAnimRate/FireAnimRate,false);
}

function PlayFiring()
{
	Weapon_LAWBase(Weapon).bAutoLock = false;
	Weapon_LAWBase(Weapon).bForceAimPos = false;
	Weapon_LAWBase(Weapon).bStartedLock = false;
	if (Weapon.Role < Role_Authority)
		Weapon_LAWBase(Weapon).MissileTarget = None;

	if (FireCount > 0)
	{
		if (Weapon.HasAnim(FireLoopAnim))
		{
			Weapon.PlayAnim(FireLoopAnim, FireLoopAnimRate, 0.0);
		}
		else
		{
			Weapon.PlayAnim(FireAnim, FireAnimRate, TweenTime);
		}
	}
	else
	{
		Weapon.PlayAnim(FireAnim, FireAnimRate, TweenTime);
	}
    
	PlayFireSound();
       
	ClientPlayForceFeedback(FireForce);  // jdf
    
	FireCount++;
}

defaultproperties
{
	AmmoClass=class'Ammo_LAW_Missile'
	AmmoPerFire=1

	PreFireAnim=CradleUp
	FireAnim=Fire
	FireAnimRate=1.0

	ProjectileClass=class'FragOps.Weapon_LAWProj'

	NewMuzFlashClass=class'Effect_LAWMuzFlash'

	ProjSpawnOffset=(X=25,Y=14,Z=-7.2)

	FireSound=Sound'FOWeaponSounds.Objectives.LAWFire'
	FireForce="RocketLauncherFire"  // jdf

	bWaitForRelease=true
	bModeExclusive=true
	PreFireTime=0.00001
	PreFireAnimRate=1.6
	FireRate=3.0
	TweenTime=0.0

	bSplashDamage=true
	bRecommendSplashDamage=true
	bSplashJump=true
	BotRefireRate=0.5
	WarnTargetPct=+0.9

	ShakeOffsetMag=(X=-25.0,Y=0.00,Z=0.00)
	ShakeOffsetRate=(X=-1000.0,Y=0.0,Z=0.0)
	ShakeOffsetTime=2
	ShakeRotMag=(X=-20.0,Y=20.0,Z=20.0)
	ShakeRotRate=(X=-300.0,Y=300.0,Z=300.0)
	ShakeRotTime=2
}