//=============================================================================
// Frag.Ops  2003 Matt 'SquirrelZero' Farber
//=============================================================================
class Weapon_LAWPickup extends FOWeaponPickup;

// overwrite validtouch, inherit the rest from the previous class
auto state Pickup
{
	function bool ValidTouch( actor Other )
	{
		// make sure its a live player
		if ((Pawn(Other) == None) || !Pawn(Other).bCanPickupInventory || (Pawn(Other).Health <= 0) || (Pawn(Other).PlayerReplicationInfo == None))
			return false;

		// make sure not touching through wall, and ensure player on this team can use the LAW
		if (!FastTrace(Other.Location, Location) || FragOpsMission(Level.Game).LAWCount[Pawn(Other).PlayerReplicationInfo.Team.TeamIndex] <= 0 || bWeaponBroken)
			return false;

		if (Pawn(Other).Inventory == None)
			return true;

		// make sure game will let player pick me up
		TriggerEvent(Event, self, Pawn(Other));
		return true;
	}
}

function TakeDamage(int Damage, Pawn instigatedBy, Vector HitLocation, Vector Momentum, class<DamageType> damageType)
{
	local vector start, end, hl, HitNormal, VelVector;
	local actor A;

	if (bWeaponBroken)
		return;

	Damage *= 0.2;

	if (DamageType.Name == 'DamType_Machete' || DamageType.Name == 'DamType_AKBayonet' || DamageType.Name == 'DamType_MachetePoison')
		DamageTaken += Damage/3;
	else
		DamageTaken += Damage;

	// we want them to break apart if they take too much damage
	if (DamageTaken >= 50)
	{
		bHidden = true;
		LifeSpan = 2.0;
		bWeaponBroken = true;
		if (Level.NetMode != NM_DedicatedServer)
			PostNetReceive();
		FragOpsMission(Level.Game).LAWDestroyed();
	}
	else
	{
		if (Level.NetMode != NM_DedicatedServer)
			Spawn(class'FragOps.Effect_KnifeSparkImpact',,, HitLocation, Rotator(-Momentum));

		if (Velocity.Y > 50 || Velocity.X > 50 || Velocity.Z > 25)
			return;

		// find out if we're on the ground, otherwise just apply normal momentum
		start = Location - vect(0,0,1)*CollisionHeight;
		end = start - vect(0,0,10);
		A = Trace(hl,HitNormal,end,start,false);
		if (hl != Vect(0,0,0))
		{
			SetPhysics(PHYS_Falling);
			// pickups shouldnt get much lift, but skid instead
			VelVector = 20*(-(HitNormal - (Momentum*17)));

			// reset bounce
			bBounce = true;
			GBounces = 0;
			
			if (Velocity.Z < 100)
				VelVector.Z = max(20,Rand(50));
			else
				VelVector.Z = 0;

			Velocity += VelVector;
		}
		else
		{
			Velocity += 200*Momentum;
		}
		RandSpin(100000);
	}
}

defaultproperties
{
	InventoryType=class'FragOps.Weapon_LAWBase'

	PickupMessage="Picked up an MBT LAW"
	PickupSound=Sound'PickupSounds.AssaultRiflePickup'
	PickupForce="AssaultRiflePickup"  // jdf

	MaxDesireability=+0.45

	StaticMesh=StaticMesh'FOPickupSMs.sms.law_sm'
	DrawType=DT_StaticMesh
	DrawScale=0.66
	CollisionRadius=+24.0
	CollisionHeight=+16.6
}