//=============================================================================
// Frag.Ops  2003 Matt 'SquirrelZero' Farber
//=============================================================================
class Weapon_LAWProj extends RocketProj;

var bool bSoftLaunch;
var bool bShakeView;

simulated function PostBeginPlay()
{
	if (Level.NetMode != NM_DedicatedServer)
	{
		SmokeTrail = Spawn(class'Effect_LAWTrailSmoke',self);
		Corona = Spawn(class'Effect_LAWCorona',self);
	}

	Dir = vector(Rotation);
	Velocity = 200 * Dir;
	if (PhysicsVolume.bWaterVolume)
	{
		bHitWater = True;
		Velocity=0.6*Velocity;
	}
	if (Level.bDropDetail || Level.NetMode == NM_DedicatedServer)
	{
		bDynamicLight = false;
		LightType = LT_None;
	}
	bSoftLaunch = true;
	SetTimer(0.16, false);
}

simulated function Timer()
{
	Dir = vector(Rotation);
	Velocity = Speed * Dir;
}

simulated singular function ProcessTouch (Actor Other, Vector HitLocation)
{
	if ((Other != instigator) && (!Other.IsA('Projectile') || Other.bProjTarget)) 
		Explode(HitLocation,Vect(0,0,1));

	// fallback, instant explosion
	if (Obj_LAWObjective(Other) != None || War_Objective_Destroyable(Other) != None || Vehicle(Other) != None)
		Other.TakeDamage(400, Instigator, HitLocation, vect(1,1,1), MyDamageType); 
}

simulated singular function HitWall(vector HitNormal, actor Wall)
{
	local PlayerController PC;

	if (Role == ROLE_Authority)
	{
		if (!Wall.bStatic || Wall.bCanBeDamaged)
		{
			if (Instigator == None || Instigator.Controller == None)
				Wall.SetDelayedDamageInstigatorController(InstigatorController);
			Wall.TakeDamage( Damage, instigator, Location, MomentumTransfer * Normal(Velocity), MyDamageType);
			if (DamageRadius > 0 && Vehicle(Wall) != None && Vehicle(Wall).Health > 0)
				Vehicle(Wall).DriverRadiusDamage(Damage, DamageRadius, InstigatorController, MyDamageType, MomentumTransfer, Location);
			HurtWall = Wall;
		}
		MakeNoise(1.0);
	}
	Explode(Location + ExploWallOut * HitNormal, HitNormal);
	if ((ExplosionDecal != None) && (Level.NetMode != NM_DedicatedServer))
	{
		if (ExplosionDecal.Default.CullDistance != 0)
		{
			PC = Level.GetLocalPlayerController();
			if (!PC.BeyondViewDistance(Location, ExplosionDecal.Default.CullDistance))
				Spawn(ExplosionDecal,self,,Location, rotator(-HitNormal));
			else if ( (Instigator != None) && (PC == Instigator.Controller) && !PC.BeyondViewDistance(Location, 2*ExplosionDecal.Default.CullDistance) )
				Spawn(ExplosionDecal,self,,Location, rotator(-HitNormal));
		}
		else
			Spawn(ExplosionDecal,self,,Location, rotator(-HitNormal));
	}
	HurtWall = None;
}

simulated function Explode(vector HitLocation, vector HitNormal) 
{
	if (Level.NetMode != NM_DedicatedServer)
		ShakeView();
	PlaySound(sound'FOWeaponSounds.Objectives.LAWExplosion',,4.3*TransientSoundVolume,,1000,,false);
	if (EffectIsRelevant(Location,false) || Level.NetMode == NM_DedicatedServer)
	{
		Spawn(class'Effect_LAWExplosion',,,HitLocation + HitNormal*16,rotator(HitNormal));	
		if ((ExplosionDecal != None) && (Level.NetMode != NM_DedicatedServer))
			Spawn(ExplosionDecal,self,,Location, rotator(-HitNormal));
	}
	BlowUp(HitLocation);
	Destroy();
}

function BlowUp(vector HitLocation)
{
	DelayedHurtRadius(Damage, DamageRadius, MyDamageType, MomentumTransfer, HitLocation );
	MakeNoise(1.0);
}

function DelayedHurtRadius( float DamageAmount, float DamageRadius, class<DamageType> DamageType, float Momentum, vector HitLocation )
{
	local actor Victims;
	local FOPawn FOVictim;
	local float damageScale, dist, TempDamage;
	local vector Direction;
	
	if (bHurtEntry)
		return;

	bHurtEntry = true;
	foreach CollidingActors(class 'Actor', Victims, DamageRadius, HitLocation)
	{
		// don't let blast damage affect fluid - VisibleCollidingActors doesn't really work for them - jag
		if ((Victims != self) && (Victims.Role == ROLE_Authority) && (!Victims.IsA('FluidSurfaceInfo')))
		{
			Direction = Victims.Location - HitLocation;
			dist = FMax(1,VSize(Direction));
			Direction = Direction/dist; 
			damageScale = 1 - FMax(0,(dist - Victims.CollisionRadius)/DamageRadius);
			if (DamageType.Default.bDelayedDamage && (Instigator == None) && (Pawn(Victims) != None))
				Pawn(Victims).DelayedDamageInstigatorController = InstigatorController;
			// for hitpoint score calculation
			FOVictim = FOPawn(Victims);
			if (Level.Game != None && FOVictim != None && InstigatorController != None && FOVictim.PlayerReplicationInfo != None)
			{
				if (FOVictim.Health <= 0)
					TempDamage = 0;
				else if (FOVictim.Health - (damageScale*DamageAmount) > 0)
					TempDamage = (damageScale*DamageAmount);
				else
					TempDamage = (damageScale*DamageAmount) + (FOVictim.Health - (damageScale*DamageAmount));
				
				// reverse if friendly fire
				if (FragOpsGame(Level.Game).FriendlyFireScale > 0.0 && FOVictim.PlayerReplicationInfo != None && FOVictim.PlayerReplicationInfo.Team == InstigatorController.PlayerReplicationInfo.Team)
					TempDamage *= -1;
			}
			Victims.TakeDamage
			(
				damageScale * DamageAmount,
				Instigator, 
				Victims.Location - 0.5 * (Victims.CollisionHeight + Victims.CollisionRadius) * Direction,
				(damageScale * Momentum * Direction),
				DamageType
			);
		} 
	}
	bHurtEntry = false;
}

simulated function ShakeView()
{
	local PlayerController PC;
	local float Dist, Scale;	

	if (bShakeView)
		return;
	
	bShakeView = true;

	PC = Level.GetLocalPlayerController();
	if (PC != None && PC.ViewTarget != None && (PC.Pawn == None || Vehicle(PC.Pawn) == None))
	{
		Dist = VSize(Location - PC.ViewTarget.Location);
		if (Dist < DamageRadius * 2.0)
		{
			if (Dist < DamageRadius)
				Scale = 1.0;
			else
				Scale = FClamp((DamageRadius*2.0 - Dist) / (DamageRadius), 0, 1.0);
			FOPlayer(PC).LocalShake(vect(350,350,350)*Scale,vect(9000,9000,9000),3.5, vect(10,10,10)*Scale,vect(2000,2000,2000),3.5);
		}
	}	
}

function Reset()
{
	Destroy();
}

defaultproperties
{
	Damage=400
	DamageRadius=500.0 
	MomentumTransfer=1200.0
	MyDamageType=class'DamType_LAW'
	Speed=1600.0
	StaticMesh=StaticMesh'FOSMs.LAWSms.LAWRocket'
	DrawScale=1.5
	FluidSurfaceShootStrengthMod=3.f
	bUseCollisionStaticMesh=true
	bCollideWorld=true
}