//=============================================================================
// Frag.Ops  2003 Matt 'SquirrelZero' Farber
//=============================================================================
class Weapon_MP5Stalker extends Weapon_MP5TacLight
	HideDropDown
	CacheExempt;

var Material InfraHUDTex;

replication
{
	reliable if (Role < ROLE_Authority)
		ServerHideTacLight;
}

function ServerHideTacLight(optional bool bForceHide)
{
	if (!bForceHide)
	{
		if (TacLight != None)
			TacLight.bLightOn = false;
	}
	else
	{
		if (TacLight != None)
			TacLight.bLightOn = true;
		else
		{
			TacLight = spawn(class'Effect_TacLightSpawner', None,, Instigator.Location + (vect(16, 0, 16) >> Instigator.Rotation), Instigator.Rotation);
			TacLight.LightPawn = Instigator;
			TacLight.bLightOn = true;
			TacLight.SetLight();
		}
	}
}

simulated event RenderOverlays( Canvas Canvas )
{
	local FOPawn DynamicPawn;

	if (bScopeZoomed && !IsReloading() && !FragOps_GRI(PlayerController(Instigator.Controller).GameReplicationInfo).bPreRound )
	{
		// infrared =]
		if (TacLightCorona != None)
		{
			if (!bInfrared)
			{
				ServerHideTacLight();
				bInfrared = true;
				SpawnGlow();
				PlayOwnedSound(InfraredSound,,TransientSoundVolume,,30,,false);
				ZoomNoise = 220;
			}
			TacLightCorona.Destroy();
		}
		if (bInfrared)
		{
			if (ZoomNoise > 29)
				ZoomNoise = ZoomNoise - 15;
			else if (ZoomNoise != 29)
				ZoomNoise = 29;
			ColorModifier(FinalBlend(InfraHUDTex).Material).Color.R = ZoomNoise;
			Canvas.SetPos(0,0);
			Canvas.DrawTile(InfraHUDTex, Canvas.ClipX, Canvas.ClipY, 0.0, 0.0, 1600, 1600);
		}
		SetLocation(Instigator.Location);
		Canvas.SetPos(0,0);
		Canvas.DrawTile(ScopeHUDTex, Canvas.ClipX, Canvas.ClipY, 0, 0, 1024, 1024);
	}
	else
	{
		if (bInfrared)
		{
			ServerHideTacLight(true);
			if (InfraGlow != None)
				InfraGlow.Destroy();
			foreach DynamicActors(class'FOPawn',DynamicPawn)
			{
				DynamicPawn.AmbientGlow = DynamicPawn.Default.AmbientGlow;
				DynamicPawn.ScaleGlow = DynamicPawn.Default.ScaleGlow;
			}
			if (TacLightCorona == None)
			{
				TacLightCorona = spawn(class'Effect_TacLightCorona');
				AttachToBone(TacLightCorona,'BoneLight');
				TacLightCorona.SetRelativeLocation(vect(0, 0, 0));
			}
		}
		if (bScopeZoomed)
		{
			ServerSetZoom(0);
			bScopeZoomed = false;
			SetScope();
		}
		Super.RenderOverlays(Canvas);
		if (TacLightCorona != None && bInfrared)
		{
			bInfrared = false;
			AttachToBone(TacLightCorona,'BoneLight');
		}
	}
}

simulated function bool PutDown()
{
	local FOPawn DynamicPawn;
	
	if (Instigator.Controller.IsA('PlayerController') && Instigator.IsLocallyControlled())
	{
		ServerSetZoom(0);
		bScopeZoomed = false;
		SetScope();
		bInfrared = false;
		if (InfraGlow != None)
			InfraGlow.Destroy();
		ServerSetTaclight(true);
		if (TacLightCorona != None)
			TacLightCorona.Destroy();

		foreach DynamicActors(class'FOPawn',DynamicPawn)
		{
			DynamicPawn.AmbientGlow = DynamicPawn.Default.AmbientGlow;
			DynamicPawn.ScaleGlow = DynamicPawn.Default.ScaleGlow;
		}		
	}
	if (Super(Weapon_MP5Base).PutDown())
	{
		return true;
	}
	return false;
}

simulated function ClientWeaponThrown()
{
	if( (Instigator != None) && (PlayerController(Instigator.Controller) != None) )
	{
		bScopeZoomed = false;
		SetScope();
		bInfrared = false;
		if (InfraGlow != None)
			InfraGlow.Destroy();
		ServerSetTaclight(true);
		if (TacLightCorona != None)
			TacLightCorona.Destroy();
	}
	Super(Weapon_MP5Base).ClientWeaponThrown();
}

simulated function ClientStartFire(int mode)
{
	if (Instigator.Controller.IsInState('PreRound') || IsReloading() || Instigator.Controller.IsInState('GameEnded'))
		return;
	if (mode == 1)
	{
		if (Instigator.HeadVolume != None && Instigator.HeadVolume.bWaterVolume)
			return;

		FireMode[mode].bIsFiring = true;
		if (Instigator.Controller.IsA('PlayerController'))
		{
			bScopeZoomed = !bScopeZoomed;
			SetScope();
			if (bScopeZoomed)
				ServerSetZoom(1);
			else
				ServerSetZoom(0);
			PlayOwnedSound(Sound'FOWeaponSounds.Misc.ZoomClick',SLOT_Pain,TransientSoundVolume,,,,false);
		}
	}
	else
		Super.ClientStartFire(mode);
}

simulated function ClientStopFire(int mode)
{
	if (mode == 1)
		FireMode[mode].bIsFiring = false;
	else
		Super.ClientStopFire(mode);
}

simulated function Destroyed()
{
	if (InfraGlow != None)
		InfraGlow.Destroy();
	Super.Destroyed();
}

defaultproperties
{
	// CXP reqs
	CXPReq=100
	
	ScopeHUDTex=FinalBlend'FOMisc.HUD.mp5scope_fb'
	InfraHUDTex=FinalBlend'FOMisc.Special.MP5InfraFinal'
	InfraredSound=Sound'FOWeaponSounds.MP5SR.Infrared'
	InfrastopSound=Sound'FOWeaponSounds.MP5SR.Infrastop'
	ScopeDownAnim=ScopeOut
	ScopeDownAnimRate=1.1
	ZoomMultiple=0.6
	GunDesc="COST: $100|+ZOOM|+INFRA"
	GunCost=200
	ItemName="MG5-SR (Stalker)"
	AttachName="stalker"
	Mesh=mesh'FOSMGs.MP5_Scope_Light'
	FireModeClass(1)=Weapon_MP5ScopeFire
	BaseWeaponClass=class'Weapon_MP5Base'
	PickupClass=class'FragOps.Weapon_MP5StalkerPickup'
	AttachmentClass=class'FragOps.Weapon_MP5StalkerAttach'
	WeaponIcon=Material'FOMisc.WeaponHUD.WeapHUD_mp5stalker'
}