//=============================================================================
// Frag.Ops  2003 Matt 'SquirrelZero' Farber
//=============================================================================
class Weapon_MP5TacLight extends Weapon_MP5Base
	HideDropDown
	CacheExempt;

// taclight projector
var Effect_TacLightSpawner TacLight;

// taclight corona
var Effect_TacLightCorona TacLightCorona;

// infrared for the stalker model
var bool bInfrared;

// when turning on the infrared
var int ZoomNoise;
var() Sound InfraredSound;
var() Sound InfrastopSound;

// the locally spawned light
var Effect_InfraGlow InfraGlow;

replication
{
	reliable if (Role < ROLE_Authority)
		ServerSetTacLight;
}

simulated function SpecialFire()
{
	local FOPawn DynamicPawn;

	if (bScopeZoomed)
	{
		bInfrared = !bInfrared;
		ZoomNoise = 220;
		if (InfraGlow != None)
			InfraGlow.bDynamicLight = bInfrared;
		else if (bInfrared)
			SpawnGlow();
		if (bInfrared)
		{
			foreach DynamicActors(class'FOPawn',DynamicPawn)
			{
				DynamicPawn.AmbientGlow = 254;
				DynamicPawn.ScaleGlow = 10.0;
			}
			PlayOwnedSound(InfraredSound,,TransientSoundVolume,,30,,false);
		}
		else
		{
			foreach DynamicActors(class'FOPawn',DynamicPawn)
			{
				DynamicPawn.AmbientGlow = DynamicPawn.Default.AmbientGlow;
				DynamicPawn.ScaleGlow = DynamicPawn.Default.ScaleGlow;
			}
			PlayOwnedSound(InfrastopSound,,TransientSoundVolume,,30,,false);
		}
		return;
	}
	ServerSetTacLight();
	if (TacLightCorona == None)
	{
		TacLightCorona = spawn(class'Effect_TacLightCorona');
		AttachToBone(TacLightCorona,'BoneLight');
		TacLightCorona.SetRelativeLocation(vect(0, 0, 0));
	}
	else
		TacLightCorona.Destroy();
	PlayOwnedSound(FireSwitchSound,,TransientSoundVolume*0.95,,30,,false);
}

function ServerSetTacLight(optional bool bForceDestroy)
{
	if (bForceDestroy)
	{
		if (TacLight != None)
		{
			TacLight.bLightOn = false;
			TacLight.Destroy();
		}
		return;
	}
	if (TacLight == None)
	{
		TacLight = Instigator.spawn(class'Effect_TacLightSpawner', None,, Instigator.Location + (vect(16, 0, 16) >> Instigator.Rotation), Instigator.Rotation);
		TacLight.LightPawn = Instigator;
		TacLight.bLightOn = true;
		TacLight.SetLight();
	}
	else
		TacLight.bLightOn = !TacLight.bLightOn;
}

simulated function SpawnGlow()
{
	local FOPawn DynamicPawn;

	if (InfraGlow == None)
		InfraGlow = spawn(class'Effect_InfraGlow',Instigator,, Instigator.Location + (vect(16, 0, -5) >> Instigator.Rotation), Instigator.Rotation);

	foreach DynamicActors(class'FOPawn',DynamicPawn)
	{
		DynamicPawn.AmbientGlow = 254;
		DynamicPawn.ScaleGlow = 10.0;
	}
}

simulated function Destroyed()
{
	local FOPawn DynamicPawn;

	if (TacLight != None)
		TacLight.Destroy();
	if (TacLightCorona != None)
		TacLightCorona.Destroy();
	if (InfraGlow != None)
		InfraGlow.Destroy();
	foreach DynamicActors(class'FOPawn',DynamicPawn)
	{
		if (Level.NetMode != NM_DedicatedServer)
		{
			DynamicPawn.AmbientGlow = DynamicPawn.Default.AmbientGlow;
			DynamicPawn.ScaleGlow = DynamicPawn.Default.ScaleGlow;
		}
	}
	Super.Destroyed();
}

simulated function bool PutDown()
{
	if (Instigator.Controller.IsA('PlayerController') && Instigator.IsLocallyControlled())
	{
		ServerSetTaclight(true);
		if (TacLightCorona != None)
			TacLightCorona.Destroy();
	}
	if (Super.PutDown())
	{
		return true;
	}
	return false;
}

defaultproperties
{
	// CXP reqs
	CXPReq=25
	
	// weapon data, readable names, attachments
	ItemName="MG5-SR (TacLight)"
	AttachName="taclight"
	GunDesc="COST: $50|TYPE: Light"
	GunCost=75
	BaseWeaponClass=class'Weapon_MP5Base'
	PickupClass=class'FragOps.Weapon_MP5TacLightPickup'
	AttachmentClass=class'FragOps.Weapon_MP5TaclightAttach'
	Mesh=mesh'FOSMGs.MP5_Light'
	WeaponIcon=Material'FOMisc.WeaponHUD.WeapHUD_mp5light'
}