//=============================================================================
// Frag.Ops  2003 Matt 'SquirrelZero' Farber
//=============================================================================
class Weapon_MacheteSwing extends FireMode_Base;

var int FireLevel, SwingType, ClashPoint;
var vector BladeTraceOffset[12];
var float StabIntervals[12];
var bool bIsStabbing;
var int MaxIntervals;

simulated function bool AllowFire()
{
	return (!Instigator.Controller.IsInState('PreRound') && !Instigator.Controller.IsInState('Building') && Instigator.Controller != None);
}

event ModeDoFire()
{
	if ((Level.TimeSeconds - LastFireTime) > 0.2)
	{
		Spread = Default.Spread;
		NumFireTimes = 0;
	}
	else
	{
		NumFireTimes++;
		Spread += NumFireTimes * ConeMultiPlier;
		Spread = FMin(Spread, ConeMax);
	}

	if (!AllowFire())
		return;

	if (MaxHoldTime > 0.0)
		HoldTime = FMin(HoldTime, MaxHoldTime);

	// server
	if (Weapon.Role == ROLE_Authority)
	{
		DoFireEffect();
		if ((Instigator == None) || (Instigator.Controller == None))
			return;
		if (AIController(Instigator.Controller) != None)
			AIController(Instigator.Controller).WeaponFireAgain(BotRefireRate, true);
		Instigator.SpawnTime = -100000;
	}
	// client
	if (Instigator.IsLocallyControlled())
	{
		ShakeView();
		PlayFiring();
		FlashMuzzleFlash();
	}

	Weapon.IncrementFlashCount(ThisModeNum);

	// set the next firing time. must be careful here so client and server do not get out of sync
	if (bFireOnRelease)
	{
	        if (bIsFiring)
			NextFireTime += MaxHoldTime + FireRate;
		else
			NextFireTime = Level.TimeSeconds + FireRate;
	}
	else
	{
		NextFireTime += FireRate;
		NextFireTime = FMax(NextFireTime, Level.TimeSeconds);
	}

	HoldTime = 0;

	if (Instigator.PendingWeapon != Weapon && Instigator.PendingWeapon != None)
	{
		bIsFiring = false;
		Weapon.PutDown();
	}
	LastFireTime = Level.TimeSeconds;
}

function PlayFiring()
{
	if (Instigator != None && Instigator.Controller != None && Instigator.IsHumanControlled())
	{
		switch(FOPlayer(Instigator.Controller).SlashDirection)
		{
			case 0:
				FireAnim = 'slasha';
				FireLoopAnim = 'slasha';
				FireLoopAnimRate = 1.1;
				FireAnimRate = FireLoopAnimRate;
				FireSound = sound'FOWeaponSounds.Machete.OverHead';
				break;

			// left slash
			case 1:
				FireAnim = 'slasha';
				FireLoopAnim = 'slasha';
				FireLoopAnimRate = 1.1;
				FireAnimRate = FireLoopAnimRate;
				FireSound = sound'FOWeaponSounds.Machete.RightSlash';
				break;

			// right slash
			case 2:
				FireAnim = 'slashb';
				FireLoopAnim = 'slashb';
				FireLoopAnimRate = 1.0;
				FireAnimRate = FireLoopAnimRate;
				FireSound = sound'FOWeaponSounds.Machete.LeftSlash';
				break;
		}
	}
	Super.PlayFiring();
}

function PlayFireSound()
{
	Weapon.PlayOwnedSound(FireSound,SLOT_Interact,TransientSoundVolume*1.5,,,1.15,false);
}

function DoFireEffect()
{
	if (Instigator != None && Instigator.Controller != None && Instigator.IsHumanControlled())
	{
		ClashPoint = Rand(MaxIntervals);
		SwingType = FOPlayer(Instigator.Controller).SlashDirection;
	}
	FireLevel = 0;
	bIsStabbing = true;
	Weapon_MacheteBase(Weapon).LastFireMode = ThisModeNum;
	Weapon_MacheteBase(Weapon).LastInterval = StabIntervals[FireLevel+(SwingType*MaxIntervals)];
	Weapon.GotoState('Stabbing','Begin');
}

function Timer()
{
	if (!bIsStabbing)
		return;

	DoStabEffect();

	if (Weapon == None)
		return;
	
	FireLevel++;

	if (FireLevel < MaxIntervals)
	{
		if (FireLevel == ClashPoint)
			FOPawn(Instigator).bCanClash = true;
		else
			FOPawn(Instigator).bCanClash = false;

		Weapon_MacheteBase(Weapon).LastFireMode = ThisModeNum;
		Weapon_MacheteBase(Weapon).LastInterval = StabIntervals[FireLevel+(SwingType*MaxIntervals)];
		Weapon.GotoState('Stabbing','Begin');
	}
	else
	{
		FOPawn(Instigator).bCanClash = false;
		FireLevel = 0;
		bIsStabbing = false;
	}
}

function DoStabEffect()
{
	local Vector StartTrace, End, XVec, X,Y,Z;
	local Rotator R, Aim;
	local Vector HitLocation, HitNormal, HitRay;
	local Actor Other;
	local float BulletDistance;
	local float FODamage;
	local int Damage, i;
	local vector BoneTestLocation;
	local name HitBone;
	local Material SurfaceMat;
	local bool bLastShot;
	local FOPawn FP;

	bLastShot = false;

	Instigator.MakeNoise(0.2);

	if (FOPlayer(Instigator.Controller) != None)
		Aim = Instigator.Controller.GetViewRotation();
	else
		Aim = AdjustAim(StartTrace, AimError);

	while (true)
	{
		R = Aim;
		Weapon.GetViewAxes(X,Y,Z);
		XVec = vector(R);
		StartTrace = Instigator.Location + Instigator.EyePosition();
		StartTrace = StartTrace + X*BladeTraceOffset[FireLevel+(SwingType*MaxIntervals)].X;
		StartTrace = StartTrace + Weapon.Hand * Y*BladeTraceOffset[FireLevel+(SwingType*MaxIntervals)].Y + Z*BladeTraceOffset[FireLevel+(SwingType*MaxIntervals)].Z;
		End = StartTrace + DeadBulletRange * XVec;
		Weapon.bUseCollisionStaticMesh = true;
		Other = Weapon.Trace(HitLocation,HitNormal,End,StartTrace,true,,SurfaceMat);
		Weapon.bUseCollisionStaticMesh = true;
		BulletDistance = VSize(Instigator.Location - HitLocation);

		// vehicle tweak
		if (Other == Instigator)
			break;

		if (Other != None)
		{
			FP = FOPawn(Other);
			if (FP != None)
			{
				if (FP.bCanClash)
				{
					ClashHitEffect(StartTrace, R, HitLocation, HitNormal);
					break;
				}
				
				Damage = FOPawn(Instigator).ModifyDamageDone(class<Weapon_Base>(Weapon.Class),WeaponDamage);
				Damage = Damage * DamageAtten;
				AdjustDamage(Damage);
				if (BulletDistance > WeaponRange)
				{
					FODamage = BulletDistance - WeaponRange;
					FODamage = FODamage / WeaponDamageShrink;
					Damage = Damage - int(FODamage);
				}

				// this new method works much better.
				BoneTestLocation = HitLocation + (Other.CollisionRadius*0.12)*XVec;
				HitRay = Normal(XVec);
								
				// find the closest bone.
				HitBone = 'none';
				// scale the head bone location to the center of the face
				if (Pawn(Other).IsHeadShot(HitLocation, XVec, 1.0))
					HitBone = 'headshot';
				else
				{
					for (i=0;i<BoneData.Length;i++)
					{
						if (FP.IsBoneHit(HitLocation, XVec, 1.0, BoneData[i].BoneName, BoneData[i].BoneRadius, BoneData[i].BoneHeight))
						{
							HitBone = BoneData[i].BoneName;
							break;
						}
					}
				}

				// if we hit no bone, move the shot beyond the player's collision radius and continue down fire path
				if (HitBone == 'none' || HitBone == '')
					break;
				else
				{
					//log("hitbone: "@hitbone);
					//log("dist: "@dist);
					//log("old damage: "@damage);
					CalcDamage(Damage, HitBone, Other);
					//log("new damage: "@damage);

					if (HitBone == 'headshot' && FP.Health > 0)
					{
						if (Level.TimeSeconds - LastHeadShotTime >= 0.02)
						{
							LastHeadShotTime = Weapon.Level.TimeSeconds;					
							if (AllowHitSound(Pawn(Other)))
							{
								if (Weapon_Base(Weapon).WeaponType != Fists)
									FP.PlayHeadShotSound();
								else
									FP.PlayBodyHitSound();
							}
						}
					}
					else if (HitBone != 'headshot' && AllowHitSound(Pawn(Other)))
						FP.PlayBodyHitSound();
					if (Damage < MinDamage)
						Damage = MinDamage;

					FP.LastBone = HitBone;
					NotifyHit(FP, HitBone, HitRay);
					Other.TakeDamage(Damage, Instigator, HitLocation, HitRay, DamageType);
					break;
				}
			}
			else if (Other.Default.bCanBeDamaged || Other.Physics == PHYS_Karma || !Other.bWorldGeometry)
			{
				if (Vehicle(Other) != None)
					SurfaceMat = MetalSkin;
				if (Projectile(Other) == None)
					BulletHitEffect(StartTrace, R, HitLocation, HitNormal, SurfaceMat);
				Other.TakeDamage(Default.WeaponDamage, Instigator, HitLocation, HitRay, DamageType);
				break;
			}
			else
			{
				BulletHitEffect(StartTrace, R, HitLocation, HitNormal, SurfaceMat);
				break;
			}
		}
		break;
	}
}

function NotifyHit(FOPawn HitPawn, name HitBone, vector HitNormal);

function HitObject();

function ClashHitEffect(Vector Start, Rotator Dir, Vector HitLocation, Vector HitNormal)
{
	local Effect_ClashHit ClashHit;

	ClashHit = Weapon.Spawn(class'Effect_ClashHit',,, Start, Dir);
	ClashHit.ImpactAt(HitLocation, HitNormal, None, Weapon.Instigator, DamageType);
}

simulated function InitEffects()
{
	if ((Level.NetMode == NM_DedicatedServer) || (AIController(Instigator.Controller) != None))
		return;

	if ((FlashEmitterClass != None) && ((FlashEmitter == None) || FlashEmitter.bDeleteMe))
	{
		if (FlashEmitter != None)
			FlashEmitter.Destroy();
		FlashEmitter = Weapon.Spawn(FlashEmitterClass);
	}
	if (FlashEmitter != None)
		Weapon.AttachToBone(FlashEmitter, 'bone_streak');
}

simulated function ShakeView()
{
	FOPlayer(Instigator.Controller).LockSlash(FireRate);
}

function FlashMuzzleFlash()
{
	if (FlashEmitter != None)
		FlashEmitter.Trigger(Weapon, Instigator);
}

defaultproperties
{
	AmmoClass=class'Ammo_Null'
	DamageType=class'FragOps.DamType_Machete'
	FireAnimRate=4.0
	FireAnim=slasha
	FireRate=0.45
	FireLoopAnim=slasha
	FireLoopAnimRate=3.0
	FireEmptyAnim=None
	FireEndAnim=None
	bPawnRapidFireAnim=true

	FlashEmitterClass=class'Effect_BladeTrail'
	BulletHitEffectClass=class'Effect_KnifeHitSmall'

	RecoilSize=0
	Spread=0.0  // best to leave this at 0 for now
	SpreadStyle=SS_None
	AimError=0  // leave at 0 i guess, till i find a use for it
	WeaponDamage=27
	WeaponRange=70   // let the knife stab just slightly in front of the player
	DeadBulletRange=100    // we don't want to do damage reduction here
	Momentum=3.0

	bWaitForRelease=true
	bFireOnRelease=false
	bModeExclusive=true

	MaxIntervals=4

	// blade offsets -- forward slash
	StabIntervals(0)=0.14
	StabIntervals(1)=0.02
	StabIntervals(2)=0.03
	StabIntervals(3)=0.02
	BladeTraceOffset(0)=(Y=0.0,X=0.0,Z=10.0)
	BladeTraceOffset(1)=(Y=0.0,X=0.0,Z=2.0)
	BladeTraceOffset(2)=(Y=0.0,X=0.0,Z=-2.0)
	BladeTraceOffset(3)=(Y=0.0,X=0.0,Z=-5.0)

	// blade offsets -- left slash
	StabIntervals(4)=0.06
	StabIntervals(5)=0.03
	StabIntervals(6)=0.03
	StabIntervals(7)=0.02
	BladeTraceOffset(4)=(Y=7.0,X=0.0,Z=3.0)
	BladeTraceOffset(5)=(Y=3.0,X=0.0,Z=-4.0)
	BladeTraceOffset(6)=(Y=-2.0,X=0.0,Z=-8.0)
	BladeTraceOffset(7)=(Y=-7.0,X=0.0,Z=-12.0)

	// blade offsets -- right slash
	StabIntervals(8)=0.08
	StabIntervals(9)=0.02
	StabIntervals(10)=0.03
	StabIntervals(11)=0.02
	BladeTraceOffset(8)=(Y=-5.0,X=0.0,Z=8.0)
	BladeTraceOffset(9)=(Y=0.0,X=0.0,Z=0.0)
	BladeTraceOffset(10)=(Y=4.0,X=0.0,Z=-5.0)
	BladeTraceOffset(11)=(Y=8.0,X=0.0,Z=-10.0)
}