//=============================================================================
// Frag.Ops  2003 Matt 'SquirrelZero' Farber
//=============================================================================
class Weapon_RedLabelBase extends Weapon_Base
	config(User);

var float ReloadTimeDouble;
var float ReloadSleepTime;

simulated state ReloadWeapon
{
	simulated function BeginState()
	{
		local FireMode_Base FOFireClass;

		// setup
		bIsReloading = true;
		FOPawn(Instigator).bIsReloading = true;
		bCanThrow = false;

		// 3rd person
		Instigator.SetAnimAction('Weapon_Switch');

		if (Instigator.IsLocallyControlled())
		{
			if (FireMode[0].bIsFiring)
				ClientStopFire(0);
			if (FireMode[1].bIsFiring)
				ClientStopFire(1);
			Instigator.Controller.bFire = 0;
			Instigator.Controller.bAltFire = 0;

			FOFireClass = FireMode_Base(FireMode[0]);
			if (Ammo[0].AmmoAmount == 1 || Ammo_Base(Ammo[0]).NumClips == 1)
			{
				PlayAnim(FOFireClass.ReloadAnim, FOFireClass.ReloadAnimRate, 0.0);
				Instigator.PlaySound(FOFireClass.ReloadSound,,FOFireClass.ReloadVolume*TransientSoundVolume*0.82,,100,,false);
			}
			else
			{
				PlayAnim(Weapon_RedLabelFire(FOFireClass).ReloadAnimDouble, Weapon_RedLabelFire(FOFireClass).ReloadAnimRateDouble, 0.0);
				PlaySound(Weapon_RedLabelFire(FOFireClass).ReloadSoundDouble,,FOFireClass.ReloadVolume*TransientSoundVolume*0.82,,100,,false);
			}
			bPlayIdleEmpty = false;
		}
		else if (Level.NetMode == NM_DedicatedServer)
			Instigator.PlayOwnedSound(Sound'WeaponSounds.AssaultRifle.SwitchToAssaultRifle',,1.1*TransientSoundVolume,,100,,false);

		// readd ammo
		if (ROLE == ROLE_Authority)
			Ammo_Base(Ammo[0]).ReloadClip();
	}

	simulated function bool IsReloading()
	{
		return true;
	}

	simulated function EndState()
	{
		// reset
		bIsReloading = false;
		FOPawn(Instigator).bIsReloading = false;
		if (Default.bCanThrow)
			bCanThrow = true;
		if (Instigator.IsLocallyControlled())
			PlayIdle();
		if (FOBot(Instigator.Controller) != None)
			FOBot(Instigator.Controller).DoCharge();
		if (Instigator.PendingWeapon != None)
			Instigator.PendingWeapon = None;
	}

Begin:
	Sleep(ReloadSleepTime);
	GotoState('');
}

function ServerReload()
{
	if (Ammo[0].AmmoAmount == 1 || Ammo_Base(Ammo[0]).NumClips == 1)
		ReloadSleepTime = ReloadTime;
	else
		ReloadSleepTime = ReloadTimeDouble;
	GotoState('ReloadWeapon','Begin');
}

simulated function PlayReload()
{
	if (Ammo[0].AmmoAmount == 1 || Ammo_Base(Ammo[0]).NumClips == 1)
		ReloadSleepTime = ReloadTime;
	else
		ReloadSleepTime = ReloadTimeDouble;
	GotoState('ReloadWeapon','Begin');
	// send the server to this state
	if (ROLE < ROLE_Authority)
		ServerReload();
}

simulated function bool ConsumeAmmo(int Mode, float load, optional bool bAmmoNeededIsMax)
{
	if (Ammo[0] != None)
	{
		Ammo[0].UseAmmo(int(load));
		Ammo[0].NetUpdateTime = Level.TimeSeconds - 1;
		return true;
	}
	return false;
}

defaultproperties
{
	// CXP reqs
	CXPReq=40
	
	ItemName="Red Dragon"
	StaticName="Red Dragon"
	WeaponType=Shotgun
	GunDesc="COST: $300|AMMO: 30 x 4|DMG: High|RNG: High"
	GunCost=900
    
	FireModeClass(0)=Weapon_RedLabelFire
	FireModeClass(1)=Weapon_RedLabelAltFire
	FireTypeNames(0)="Spread"

	InventoryGroup=3
	Mesh=mesh'FOShotguns.RedLabel'
	ReloadTime=1.8
	ReloadTimeDouble=2.6

	SelectSound=Sound'FOWeaponSounds.SelectSounds.SelectPrimary'
	SelectForce="SwitchToAssaultRifle"

	BobDamping=1.94
	PickupClass=class'FragOps.Weapon_RedLabelPickup'

	AttachmentClass=class'FragOps.Weapon_RedLabelAttachment'
	TracerBone=Bone_Flash
	SmokeBone=Bone_Smoke

	DisplayFOV=65
	DrawScale=0.78
	PlayerViewOffset=(X=-14.4,Y=-1.15,Z=1.9)
	PlayerViewPivot=(Pitch=0,Roll=-850,Yaw=16600)

	IdleAnim=Idle
	RestAnim=Idle
	AimAnim=Idle
	RunAnim=Idle
	SelectAnim=Select
	PutDownAnim=Deselect
	SelectAnimRate=1.0
	PutDownAnimRate=1.4

	WeaponIcon=Material'FOMisc.WeaponHUD.WeapHUD_redlabel'

	//DrawScale=1.0
	//DisplayFOV=55
	//PlayerViewOffset=(X=1.3,Y=-2.65,Z=-11.7)
	//PlayerViewPivot=(Pitch=0,Roll=-725,Yaw=-15600)

	AIRating=+0.3
	CurrentRating=0.3

	LightType=LT_Pulse
	LightEffect=LE_NonIncidence
	LightPeriod=2
	LightBrightness=170
	LightHue=30
	LightSaturation=150
	LightRadius=6.0

	DefaultPriority=3
	Priority=3
}