//=============================================================================
// Frag.Ops  2003 Matt 'SquirrelZero' Farber
//=============================================================================
// Not really a weapon, but a slimmed down copy of one.  All settings are
// initialized by the real weapon on spawn, allowing us to only make this
// actor exist on the client.  Compared to my old akimbo weapon system, 
// this is a huge boost in performance.  Modular so it is easy to plug in
// to any weapon without any modification.
//=============================================================================
class Weapon_SlaveBase extends Inventory;

// the master weapon, controls everything
var Weapon_Base Master;

// we'll use some settings here for easy access
var name IdleAnim, IdleEmptyAnim, ReloadAnim, SelectAnim, PutdownAnim, FireAnim, FireLoopAnim, FireEmptyAnim;
var float IdleAnimRate, ReloadAnimRate, SelectAnimRate, PutDownAnimRate, FireAnimRate, FireLoopAnimRate;
var sound SelectSound, ReloadSound, FireSound;
var rotator RotationBase;
var float DisplayFOV, Hand, RenderedHand, ReloadVolume;
var float YAWTurnOffset, PITCHTurnOffset;
var bool bNoRender, bLatentBringUp, bJustBought, bPlayLastFireAnim, bPlayIdleEmpty;
var Actor FlashEmitter;
var xEmitter SmokeEmitter, SmokeEmitter2nd;
var Effect_ShellEjectorFirst ShellEjector;
var int AmmoAmountSec;

function PostBeginPlay()
{
	local int i;
	
	// initialize master
	Master = Weapon_Base(Owner);

	// set mesh
	LinkMesh(Master.Default.Mesh);

	// load all skins first
	for (i=0;i<Master.Default.Skins.Length;i++)
		Skins[i] = Master.Default.Skins[i];

	// set team skins
	if (Instigator.PlayerReplicationInfo.Team.TeamIndex == 0)
	{
		Skins[0] = FOPawn(Instigator).Default.MercSleaveSkins[FOPawn(Instigator).BodySkin];
		Skins[1] = FOPawn(Instigator).Default.MercGloveSkin;
	}
	else
	{
		Skins[0] = FOPawn(Instigator).Default.SOSleaveSkins[FOPawn(Instigator).BodySkin];
		Skins[1] = FOPawn(Instigator).Default.SOGloveSkin;
	}

	// set base attributes
	SetDrawScale(Master.DrawScale);
	DisplayFOV = Master.DisplayFOV;
	PlayerViewOffset = Master.PlayerViewOffset;
	PlayerViewPivot = Master.PlayerViewPivot;
	SelectSound = Master.SelectSound;
	ReloadSound = Master.GetFM(0).ReloadSound;
	FireSound = Master.GetFM(0).FireSound;
	ReloadVolume = Master.GetFM(0).ReloadVolume;

	// initalize anims
	IdleAnim = Master.IdleAnim;
	IdleAnimRate = Master.IdleAnimRate;
	IdleEmptyAnim = Master.IdleEmptyAnim;
	ReloadAnim = Master.GetFM(0).ReloadAnim;
	ReloadAnimRate = Master.GetFM(0).ReloadAnimRate;
	SelectAnim = Master.SelectAnim;
	SelectAnimRate = Master.SelectAnimRate;
	PutdownAnim = Master.PutdownAnim;
	PutdownAnimRate = Master.PutdownAnimRate;
	FireLoopAnim = Master.GetFM(0).FireLoopAnim;
	FireLoopAnimRate = Master.GetFM(0).FireLoopAnimRate*0.75;
	FireAnim = Master.GetFM(0).FireAnim;
	FireAnimRate = Master.GetFM(0).FireAnimRate*0.75;
	FireEmptyAnim = Master.GetFM(0).FireEmptyAnim;

	// bring into view
	if (Master.ClientState == WS_Hidden || Master.ClientState == WS_BringUp)
		BringUp();
	else
		LatentBringUp();
}

// draw extra ammo count
function DrawSlaveInfo(Canvas Canvas)
{
	local Interface_MainHUD TempMainHUD;
	
	// draw 
	if (Instigator != None && FOPlayer(Instigator.Controller) != None)
		TempMainHUD = Interface_MainHUD(FOPlayer(Instigator.Controller).MyHUD);

	if (TempMainHUD == None || Instigator.bHideRegularHUD || !TempMainHUD.bShowWeaponInfo || TempMainHUD.bHideHUD)
		return;

	if (!Master.bIsReloading)
		AmmoAmountSec = Ammo_AkimboBase(Master.GetAmmo(0)).AmmoSec;

	TempMainHUD.AmmoSecCount.Value = AmmoAmountSec;

	TempMainHUD.DrawSpriteWidget(Canvas, TempMainHUD.AmmoSecIcon);
	TempMainHUD.DrawNumericWidget(Canvas, TempMainHUD.AmmoSecCount, TempMainHUD.DigitsBig);
}

// functions needed to fool player into thinking this is a normal weapon
function LatentBringUp()
{
	if (bNoRender)
	{
		PlayOwnedSound(SelectSound, SLOT_Interact,,,,, false);
		PlayAnim(SelectAnim,SelectAnimRate,0.0);
		InitEffects();
		bNoRender = false;
	}
}

function BringUp()
{
	bLatentBringUp = true;
	SetTimer(0.15, false);
	bJustBought = false;
}

function Timer()
{
	if (bLatentBringUp)
	{
		bLatentBringUp = false;
		LatentBringUp();
	}
}

function PutDown()
{
	PlayAnim(PutdownAnim,PutdownAnimRate,0.0);
	DestroyEffects();
}

event RenderOverlays(Canvas Canvas)
{
	local vector NewScale3D, CamPos;
	local rotator CamRot;

	if (Instigator == None)
		return;

	Hand = -Instigator.Controller.Handedness;

	if ((Hand < -1.0) || (Hand > 1.0))
	{
		SetLocation(Instigator.Location);
		return;
	}

	if (Hand != RenderedHand)
	{
		newScale3D = Master.Default.DrawScale3D;
		newScale3D.Y *= Hand;
		PlayerViewPivot.Yaw = Master.Default.PlayerViewPivot.Yaw * Hand;
		SetDrawScale3D(newScale3D);
		RenderedHand = Hand;
	}
	PlayerViewPivot.Roll = Master.Default.PlayerViewPivot.Roll*Hand + Master.RealPivot.Roll*Hand;
	PlayerViewOffset = Master.Default.PlayerViewOffset;
	PlayerViewOffset.Y *= Hand;
	PlayerViewOffset.Z = Master.PlayerViewOffset.Z;

	Canvas.GetCameraLocation(CamPos,CamRot);
	SetLocation(CamPos + ((0.9/DisplayFOV*100*PlayerViewOffset) >> RotationBase) + (Instigator.WeaponBob(Master.BobDamping)*0.14));
	SetRotation(RotationBase);

	bDrawingFirstPerson = true;
	Canvas.DrawActor(self, false, false, DisplayFOV);
	bDrawingFirstPerson = false;
}

function Tick(float dt)
{
	// treat it like weapontick
	if (Master != Instigator.Weapon)
		return;
		
	Super.Tick(dt);

	// get rotation base
	RotationBase = Instigator.GetViewRotation();

	// now add aTurn and aLook pivot adjustments
	if (Instigator.IsHumanControlled())
	{
		if (Master.LastTurn == 0)
		{
			YAWTurnOffset = YAWTurnOffset - (YAWTurnOffset*dt*7.0);
			if ((YAWTurnOffset < 2.0 && YAWTurnOffset > 0) || (YAWTurnOffset > -2.0 && YAWTurnOffset < 0))
				YAWTurnOffset = 0;
		}
		else
			YAWTurnOffset = fClamp(YAWTurnOffset - (Master.LastTurn*dt*1.2), -Master.WeaponTurnLimit, Master.WeaponTurnLimit);

		if (Master.LastLookUp == 0)
		{
			PITCHTurnOffset = PITCHTurnOffset - (PITCHTurnOffset*dt*5.0);
			if ((PITCHTurnOffset < 2.0 && PITCHTurnOffset > 0) || (PITCHTurnOffset > -2.0 && PITCHTurnOffset < 0))
				PITCHTurnOffset = 0;
		}
		else
			PITCHTurnOffset = fClamp(PITCHTurnOffset - (Master.LastLookUp*dt*1.2), -Master.WeaponPitchLimit, Master.WeaponPitchLimit);
		
		RotationBase.Yaw += YAWTurnOffset;
		RotationBase.Pitch += PITCHTurnOffset;
	}
}

function PlayReload()
{
	GotoState('ReloadWeapon','Begin');
}

state ReloadWeapon
{
	function BeginState()
	{
		PlayAnim(ReloadAnim,ReloadAnimRate);
		PlaySound(ReloadSound,,ReloadVolume*Master.TransientSoundVolume,,100,,false);
		bPlayIdleEmpty = false;
	}

	function EndState()
	{
		PlayIdle();
	}

Begin:
	sleep(Master.ReloadTime-0.05);
	GotoState('');
}

// some functions maliciously raided from the weapon class! :)
function name GetAnimSeq()
{
	local name anim;
	local float frame, rate;
    
	GetAnimParams(0, anim, frame, rate);
	return anim;
}

function AnimEnd(int channel)
{
	PlayIdle();
}

function PlayIdle()
{
	if (bPlayIdleEmpty && HasAnim(IdleEmptyAnim))
		PlayIdleEmpty();
	else
		LoopAnim(IdleAnim, IdleAnimRate, 0.2);
}

function PlayIdleEmpty()
{
	LoopAnim(IdleEmptyAnim, IdleAnimRate, 0.2);
}

function FireSlave(sound FireSound, float FireVolume)
{
	bPlayLastFireAnim = (Ammo_AkimboBase(Master.GetAmmo(0)).AmmoSec <= 1);
	PlayFiring();
	FlashMuzzleFlash();
	StartMuzzleSmoke();
	PlaySound(FireSound,SLOT_Interact,TransientSoundVolume*FireVolume,,,(0.97)+(FRand()*0.06),false);
}

function PlayFiring()
{
	if (HasAnim(FireEmptyAnim) && bPlayLastFireAnim)
	{
		if (HasAnim(FireLoopAnim))
			PlayAnim(FireEmptyAnim, FireAnimRate, 0.0);
		else if (Master.CurrentFireType != 1)
			PlayAnim(FireEmptyAnim, FireAnimRate, 0.0);
		else
			PlayAnim(FireEmptyAnim, FireAnimRate, Master.GetFM(0).TweenTime);
		bPlayIdleEmpty = true;
	}
	else if (HasAnim(FireLoopAnim))
		PlayAnim(FireLoopAnim, FireLoopAnimRate, 0.0);
	else
		PlayAnim(FireAnim, FireAnimRate, Master.GetFM(0).TweenTime);

	Master.GetFM(0).ClientPlayForceFeedback(Master.GetFM(0).FireForce);
}

// firemode raid!  some functions had to be modified
function DestroyEffects()
{
	if (SmokeEmitter != None)
		SmokeEmitter.Destroy();
	if (SmokeEmitter2nd != None)
		SmokeEmitter2nd.Destroy();
	if (ShellEjector != None)
	        ShellEjector.Destroy();
	if (FlashEmitter != None)
		FlashEmitter.Destroy();
}

function InitEffects()
{
	// bots don't want muzzle flashes
	if (AIController(Instigator.Controller) != None)
		return;

	if ((Master.GetFM(0).FlashEmitterClass != None) && ((FlashEmitter == None) || FlashEmitter.bDeleteMe))
	{
		if (FlashEmitter != None)
			FlashEmitter.Destroy();
		FlashEmitter = Spawn(Master.GetFM(0).FlashEmitterClass,self);
	}

	if ((Master.GetFM(0).SmokeEmitterClass != None) && ((SmokeEmitter == None) || SmokeEmitter.bDeleteMe) )
	{
		if (SmokeEmitter != None)
			SmokeEmitter.Destroy();
		SmokeEmitter = Spawn(Master.GetFM(0).SmokeEmitterClass,self);
	}

	if ((Master.GetFM(0).SmokeEmitter2ndClass != None) && ((SmokeEmitter2nd == None) || SmokeEmitter2nd.bDeleteMe) )
	{
		if (SmokeEmitter2nd != None)
			SmokeEmitter2nd.Destroy();
		SmokeEmitter2nd = Spawn(Master.GetFM(0).SmokeEmitter2ndClass,self);
	}

	if ((Master.GetFM(0).ShellEjectorClass != None) && ((ShellEjector == None) || ShellEjector.bDeleteMe))
	{
		if (ShellEjector != None)
			ShellEjector.Destroy();
		ShellEjector = Spawn(Master.GetFM(0).ShellEjectorClass,self);
		ShellEjector.InitEjector();
	}
	AttachEffects();
}

function AttachEffects()
{
	if (FlashEmitter != None)
		AttachToBone(FlashEmitter, Master.Default.TracerBone);
	if (SmokeEmitter != None)
		AttachToBone(SmokeEmitter, Master.Default.SmokeBone);
	if (SmokeEmitter2nd != None)
		AttachToBone(SmokeEmitter2nd, Master.Default.SmokeBone);
}

function FlashMuzzleFlash()
{
	local rotator r;
	r.Roll = Rand(65536);
	r.Yaw = 25;
	r.Pitch = -50;
	AttachToBone(FlashEmitter, Master.Default.TracerBone);
	SetBoneRotation(Master.Default.TracerBone, r, 0, 1.f);
	if (FlashEmitter != None)
		FlashEmitter.Trigger(self, Instigator);
}

function EjectShells()
{
	if (ShellEjector == None)
		return;

	if (FOBot(Instigator.Controller) != None && ShellEjector != None)
	{
		ShellEjector.Destroy();
		return;
	}
	else if (ShellEjector != None)
		ShellEjector.Trigger(self, Instigator);
}

function StartMuzzleSmoke()
{
	local rotator r;
	r.Roll = 30000 + Rand(8000);
	r.Yaw = 4500;
	r.Pitch = -2550;
	SetBoneRotation(Master.Default.SmokeBone, r, 0, 1.f);
	if ((SmokeEmitter2nd != None))
		SmokeEmitter2nd.Trigger(self, Instigator);
	if ((SmokeEmitter != None) && !Level.bDropDetail)
		SmokeEmitter.Trigger(self, Instigator);
	EjectShells();
}

function Destroyed()
{
	DestroyEffects();
}

defaultproperties
{
	DrawType=DT_Mesh
	Style=STY_Normal
	bAcceptsProjectors=true
	bClientAnim=true
	bHidden=true
	bNoRender=true

	ScaleGlow=1.0
	AmbientGlow=11
	MaxLights=5

	RemoteRole=ROLE_None
}