//=============================================================================
// Frag.Ops  2003 Matt 'SquirrelZero' Farber
//=============================================================================
class Weapon_Super90Base extends Weapon_Base
	config(User);

var bool bWasCancelled;

replication
{
	reliable if (Role < Role_Authority)
		ServerCancelReload;
}

function ServerReload()
{
	GotoState('BeginReload','Begin');
}

simulated function PlayReload()
{
	GotoState('BeginReload','Begin');
	// send the server to this state
	if (ROLE < ROLE_Authority)
		ServerReload();
}

simulated function CancelReload()
{
	if (!IsReloading())
		return;

	bWasCancelled = true;
	ServerCancelReload();
}

function ServerCancelReload()
{
	bWasCancelled = true;
}

simulated state ReloadWeapon
{
	simulated function BeginState()
	{
		local Weapon_Super90Fire FOFireClass;

		// setup
		bIsReloading = true;
		FOPawn(Instigator).bIsReloading = true;
		bCanThrow = false;

		// 3rd person animation, both server and client
		Instigator.SetAnimAction('Weapon_Switch');

		if (Instigator.IsLocallyControlled())
		{
			// anim/sound
			PlayOwnedSound(FireMode[0].ReloadSound,,FireMode_Base(FireMode[0]).ReloadVolume*TransientSoundVolume*0.82,,100,,false);
			FOFireClass = Weapon_Super90Fire(FireMode[0]);
			PlayAnim(FOFireClass.ReloadAnim, FOFireClass.ReloadAnimRate, 0.0);
		}

		// server -- readd ammo
		if (ROLE == ROLE_Authority && Ammo[0].AmmoAmount < Ammo[0].MaxAmmo)
		{
			Ammo_Base(Ammo[0]).NumClips--;
			Ammo[0].AmmoAmount++;
		}
		else if (Ammo[0].AmmoAmount == Ammo[0].MaxAmmo)
			GotoState('EndReload','Begin');
		
		SetTimer(ReloadTime,false);
	}
	
	simulated function Timer()
	{
		if (AmmoAmount(0) < Ammo[0].MaxAmmo && !bWasCancelled && Ammo_Base(Ammo[0]).NumClips > 0)
		{
			BeginState();
		}
		else 
			GotoState('EndReload','Begin');
	}

	simulated function bool IsReloading()
	{
		return true;
	}

	simulated function EndState()
	{
	}

Begin:
}

simulated state BeginReload
{
	simulated function BeginState()
	{
		local Weapon_Super90Fire FOFireClass;

		// setup
		bIsReloading = true;
		bCanThrow = false;
		bWasCancelled = false;

		// 3rd person animation, both server and client
		Instigator.SetAnimAction('Weapon_Switch');

		if (Instigator.IsLocallyControlled())
		{
			if (FireMode[0].bIsFiring)
				ClientStopFire(0);
			if (FireMode[1].bIsFiring)
				ClientStopFire(1);

			Instigator.Controller.bFire = 0;
			Instigator.Controller.bAltFire = 0;

			FOFireClass = Weapon_Super90Fire(FireMode[0]);
			PlayAnim(FOFireClass.ReloadBeginAnim, FOFireClass.ReloadBeginAnimRate, 0.0);
			PlayOwnedSound(FOFireClass.ReloadBeginSound,,FireMode_Base(FireMode[0]).ReloadVolume*TransientSoundVolume*0.82,,100,,false);
			bPlayIdleEmpty = false;
		}
		else if (Level.NetMode == NM_DedicatedServer || !Instigator.IsHumanControlled())
			Instigator.PlayOwnedSound(Sound'WeaponSounds.AssaultRifle.SwitchToAssaultRifle',,1.1*TransientSoundVolume,,100,,false);
	}

	simulated function bool IsReloading()
	{
		return true;
	}
	
	simulated function EndState()
	{
	}

Begin:
	Sleep(Weapon_Super90Fire(FireMode[0]).ReloadBeginTime);
	GotoState('ReloadWeapon','Begin');
}

simulated state EndReload
{
	simulated function BeginState()
	{
		local Weapon_Super90Fire FOFireClass;

		FOFireClass = Weapon_Super90Fire(FireMode[0]);
		PlayAnim(FOFireClass.ReloadEndAnim, FOFireClass.ReloadEndAnimRate, 0.0);
		if (FOFireClass.ReloadEndSound != None && Level.NetMode != NM_DedicatedServer)
			PlayOwnedSound(FOFireClass.ReloadEndSound,,FireMode_Base(FireMode[0]).ReloadVolume*TransientSoundVolume*0.62,,100,1.1,false);
	}

	simulated function EndState()
	{
		// reset
		bIsReloading = false;
		FOPawn(Instigator).bIsReloading = false;
		bWasCancelled = false;
		if (Default.bCanThrow)
			bCanThrow = true;
		if (FOBot(Instigator.Controller) != None)
			FOBot(Instigator.Controller).DoCharge();
		if (Instigator.PendingWeapon != None)
			Instigator.PendingWeapon = None;
	}

	simulated function bool IsReloading()
	{
		return true;
	}

Begin:
	Sleep(Weapon_Super90Fire(FireMode[0]).ReloadEndTime);
	if (Instigator.IsLocallyControlled())
		PlayIdle();
	GotoState('');
}

defaultproperties
{
	// weapon data, readable names, attachments
	ItemName="Ultra90"
	StaticName="Ultra90"
	WeaponType=Shotgun
	InventoryGroup=3
	GunDesc="COST: $100|AMMO: 12 x 4|DMG: Low|RNG: Low"
	GunCost=625
    
	// firemodes, types
	NumFireTypes=0
	FireTypeNames(0)="Spread"
	FireModeClass(0)=Weapon_Super90Fire
	FireModeClass(1)=Weapon_Super90FireNull

	// AI
	AIRating=+0.3
	CurrentRating=0.3
	Priority=3
	
	// reload, bob, burst timings
	bHasSpecialReload=true
	ReloadTime=0.6
	BobDamping=1.94

	// sound
	SelectSound=Sound'FOWeaponSounds.SelectSounds.SelectPrimary'
	
	// 3rd person
	PickupClass=class'Weapon_Super90Pickup'
	AttachmentClass=class'Weapon_Super90Attachment'
	TracerBone=Bone_Flash
	SmokeBone=Bone_Smoke
	ShellBone=Bone_Shell

	// 1st person
	Mesh=mesh'FOShotguns.Benelli'

	WeaponIcon=Material'FOMisc.WeaponHUD.WeapHUD_benelli'

	DisplayFOV=65
	DrawScale=0.78
	PlayerViewOffset=(X=-15.4,Y=-3.37,Z=0.8)
	PlayerViewPivot=(Pitch=250,Roll=-450,Yaw=16820)

	IdleAnim=Idle
	IdleEmptyAnim=Idle
	RestAnim=Idle
	AimAnim=Idle
	RunAnim=Idle
	SelectAnim=Select
	PutDownAnim=Deselect
	SelectAnimRate=1.0
	PutDownAnimRate=1.4
	SelectForce="SwitchToAssaultRifle"

	LightType=LT_Pulse
	LightEffect=LE_NonIncidence
	LightPeriod=2
	LightBrightness=170
	LightHue=30
	LightSaturation=150
	LightRadius=6.0
}