//=============================================================================
// Frag.Ops  2003 Matt 'SquirrelZero' Farber
//=============================================================================
class Weapon_XM8Base extends Weapon_Base
	config(User);

simulated function SetFireType(int i)
{
	CurrentFireType = i;

	if (!FireMode[0].Default.bWaitForRelease && CurrentFireType == 1 && NumFireTypes >= 1)
	{
		FireMode[0].bWaitForRelease = false;
		FireMode[0].bNowWaiting = false;
		FireMode[0].FireRate = FireMode[0].Default.FireRate;
	}
	else if (CurrentFireType == 2 && NumFireTypes >= 2)
	{
		FireMode[0].bWaitForRelease = true;
		FireMode[0].bNowWaiting = true;
		FireMode[0].FireRate = FireMode[0].Default.FireRate * 1.1;
	}
}

simulated event RenderOverlays( Canvas Canvas )
{
	if (bScopeZoomed && !IsReloading() && !FragOps_GRI(PlayerController(Instigator.Controller).GameReplicationInfo).bPreRound )
	{
		Canvas.SetPos(0, 0);
		Canvas.DrawTile(ScopeHUDTex, Canvas.ClipX, Canvas.ClipY, 0, 0, 1024, 1024);
		SetLocation(Instigator.Location);
	}
	else
	{
		if (bScopeZoomed)
		{
			ServerSetZoom(0);
			bScopeZoomed = false;
			SetScope();
		}
		Super.RenderOverlays(Canvas);
	}
}

simulated function bool PutDown()
{
	if (Instigator.Controller.IsA('PlayerController') && Instigator.IsLocallyControlled())
	{
		ServerSetZoom(0);
		bScopeZoomed = false;
		SetScope();
	}

	if (Super.PutDown()) 
	{
		return true;
	}
	return false;
}

simulated function ClientWeaponThrown()
{
	if( (Instigator != None) && (PlayerController(Instigator.Controller) != None) )
	{
		bScopeZoomed = false;
		SetScope();
	}
	Super.ClientWeaponThrown();
}

simulated function ClientStartFire(int mode)
{
	if (Instigator.Controller.IsInState('PreRound') || IsReloading() || Instigator.Controller.IsInState('GameEnded'))
		return;
	if (mode == 1)
	{
		if (Instigator.HeadVolume != None && Instigator.HeadVolume.bWaterVolume)
			return;

		FireMode[mode].bIsFiring = true;
		if (Instigator.Controller.IsA('PlayerController'))
		{
			bScopeZoomed = !bScopeZoomed;
			SetScope();
			if (bScopeZoomed)
				ServerSetZoom(1);
			else
				ServerSetZoom(0);
			PlayOwnedSound(Sound'FOWeaponSounds.Misc.ZoomClick',SLOT_Pain,TransientSoundVolume,,,,false);
		}
	}
	else
		Super.ClientStartFire(mode);
}

simulated function ClientStopFire(int mode)
{
	if (mode == 1)
		FireMode[mode].bIsFiring = false;
	else
		Super.ClientStopFire(mode);
}

defaultproperties
{
	// weapon data, readable names, attachments
	ItemName="ZM8"
	StaticName="ZM8"
	InventoryGroup=3
	WeaponType=Rifle
	GunAttachment[0]=class'Weapon_XM8NVision'
	GunAttachment[1]=class'Weapon_XM8Sharp'
	GunAttachment[2]=class'Weapon_XM8Loaded'
	GunDesc="COST: $350|AMMO: 30 x 4|DMG: High|RNG: Med"
	GunCost=1400
    
	// CXP reqs
	CXPReq=80

	// firemodes, types
	NumFireTypes=2
	FireTypeNames(0)="Auto"
	FireTypeNames(1)="Semi"
	FireModeClass(0)=Weapon_XM8Fire
	FireModeClass(1)=Weapon_XM8FireNull

	// AI
	AIRating=+0.3
	CurrentRating=0.3
	Priority=3
	
	// reload, bob, burst timings
	ReloadTime=2.92
	BobDamping=1.94

	// sound
	SelectSound=Sound'FOWeaponSounds.SelectSounds.SelectPrimary'
	
	// 3rd person
	PickupClass=class'Weapon_XM8Pickup'
	AttachmentClass=class'Weapon_XM8Attachment'
	TracerBone=Bone_Flash
	ShellBone=Bone_Shell
	SmokeBone=Bone_Smoke

	// 1st person
	Mesh=mesh'FORifles.XM8'

	// scope
	ScopeHUDTex=FinalBlend'FOMisc.HUD.xm8base_fb'
	ScopeDownAnim=ScopeOut
	ScopeDownAnimRate=1.2
	ZoomMultiple=0.65

	WeaponIcon=Material'FOMisc.WeaponHUD.WeapHUD_xm8bare'

	DisplayFOV=65
	DrawScale=0.78
	PlayerViewOffset=(X=-13.2,Y=-8.4,Z=3.6)
	PlayerViewPivot=(Pitch=490,Roll=-900,Yaw=17725)

	IdleAnim=Idle
	RestAnim=Idle
	AimAnim=Idle
	RunAnim=Idle
	SelectAnim=Select
	PutDownAnim=Deselect
	SelectAnimRate=1.0
	PutDownAnimRate=1.4
	SelectForce="SwitchToAssaultRifle"

	LightType=LT_Pulse
	LightEffect=LE_NonIncidence
	LightPeriod=2
	LightBrightness=170
	LightHue=30
	LightSaturation=150
	LightRadius=6.0

	// buymenu offset
	BuyMenuOffset=(X=0,Y=3.5,Z=0)
}